@@ -3,7 +3,7 @@
 **
 **	(c) COPYRIGHT MIT 1995.
 **	Please first read the full copyright statement in the file COPYRIGH.
-**	@(#) $Id: HTAABrow.c,v 2.60 2001/03/14 16:25:50 kahan Exp $
+**	@(#) $Id: HTAABrow.c,v 2.62 2003/04/14 09:00:16 kahan Exp $
 **
 **	Contains code for parsing challenges and creating credentials for 
 **	basic authentication schemes. See also the HTAAUtil module
@@ -771,6 +771,13 @@ PRIVATE BOOL digest_credentials (HTRequest * request, HTDigest * digest)
 	    add_param (&cleartext, "qop", "auth", NO);
 	    add_param (&cleartext, "nc", nc, NO);
 	    add_param (&cleartext, "cnonce", digest->cnonce, YES);
+	    if (strcasecomp (digest->qop, "auth")) {
+	      /* we got some options, as we only know how to handle auth, we'll do so.
+	       We should split the options and see if it was a valid Qop here, though,
+	       to things correctly */
+	      HT_FREE (digest->qop);
+	      StrAllocCopy (digest->qop, "auth");
+	    }
 	}
 	/* compute the response digest */
 	/* @@@ md5 hard coded, change it to something from the answer, 
