@@ -3,7 +3,7 @@
 **
 **	(c) COPYRIGHT MIT 1995.
 **	Please first read the full copyright statement in the file COPYRIGH.
-**	@(#) $Id: HTAnchor.c,v 1.87 1999/03/08 16:52:17 frystyk Exp $
+**	@(#) $Id: HTAnchor.c,v 1.89 2001/03/06 16:25:09 kahan Exp $
 **
 **	An anchor represents a region of a hypertext document which is
 **	linked to another anchor in the same or a different document.
@@ -509,6 +509,38 @@ PUBLIC BOOL HTAnchor_delete (HTParentAnchor * me)
 	}
     }
 
+    /* 2001/03/06: Bug fix by Serge Adda <sAdda@infovista.com>
+       HTAnchor_delete wasn't removing the reference to the deleted
+       anchor. This caused a bug whenever requesting another anchor
+       for the same URL.
+    */
+    if (adult_table) {
+      int hash;
+      const char *p;
+      HTList * adults;
+      HTList * grownups;
+      HTList * last;
+      HTParentAnchor * foundAnchor;
+
+      /* Select list from hash table */
+      for(p=me->address, hash=0; *p; p++)
+	hash = (int) ((hash * 3 + (*(unsigned char*)p)) %
+		      PARENT_HASH_SIZE);
+      adults = adult_table[hash];
+
+      /* Search list for anchor */
+      grownups = adults;
+      last = grownups;
+      while ((foundAnchor = (HTParentAnchor *)
+	      HTList_nextObject(grownups))){
+	if (!strcmp(foundAnchor->address, me->address)) {
+	  HTList_quickRemoveElement (grownups, last);
+	  break;
+	}
+	last = grownups;
+      }
+    }
+
     /* Now kill myself */
     delete_parent(me);
     return YES;  /* Parent deleted */
@@ -1309,7 +1341,9 @@ PUBLIC char * HTAnchor_etag (HTParentAnchor * me)
 
 PUBLIC void HTAnchor_setEtag (HTParentAnchor * me, const char * etag)
 {
-    if (me && etag && me->etag != etag) StrAllocCopy(me->etag, etag);
+  /* JK: add a new etag if it doesn't exist or if the value has changed */
+    if (me && etag && ((me->etag == NULL) || strcmp (me->etag, etag)))
+	StrAllocCopy(me->etag, etag);
 }
 
 PUBLIC BOOL HTAnchor_isEtagWeak (HTParentAnchor * me)
