@@ -3,7 +3,7 @@
 **
 **	(c) COPYRIGHT MIT 1995.
 **	Please first read the full copyright statement in the file COPYRIGH.
-**	@(#) $Id: HTDNS.c,v 2.31 2000/01/06 10:48:50 kahan Exp $
+**	@(#) $Id: HTDNS.c,v 2.32 2002/03/21 13:42:01 kahan Exp $
 **
 **	This object manages a cache of hosts we have looked up vis DNS.
 **	The object contains the necessary parts from hostent. For Internet host
@@ -283,19 +283,19 @@ PUBLIC int HTGetHostByName (HTHost * host, char *hostname, HTRequest* request)
 	char buffer[HOSTENT_MAX];
 	struct hostent result;			      /* For gethostbyname_r */
 #endif
-#ifdef GETHOSTBYNAME_R_3
+#ifdef HAVE_GETHOSTBYNAME_R_3
         struct hostent_data hdata;
 #endif
 
 	if (cbf) (*cbf)(request, HT_PROG_DNS, HT_MSG_NULL,NULL,hostname,NULL);
-#ifdef GETHOSTBYNAME_R_5
+#ifdef HAVE_GETHOSTBYNAME_R_5
 	hostelement = gethostbyname_r(hostname, &result, buffer,
 				      HOSTENT_MAX, &thd_errno);
-#elif defined(GETHOSTBYNAME_R_6)
+#elif defined(HAVE_GETHOSTBYNAME_R_6)
 	gethostbyname_r(hostname, &result, buffer,
 		        HOSTENT_MAX, &hostelement, &thd_errno);
 
-#elif defined(GETHOSTBYNAME_R_3)
+#elif defined(HAVE_GETHOSTBYNAME_R_3)
         if (gethostbyname_r(hostname, &result, &hdata) == 0) {
 	    hostelement = &result;
 	}
