@@ -21,7 +21,7 @@ of the  W3C Sample Code Library.
 We use James Clark's expat
 XML parser which is very neat indeed. As the code doesn't come as a separate
 library, I included it in the libwww CVS code base where I compile is as
-two libraries: libxmltok.a and libxmlparse.a. See
+one library: libexpat.a. See
 the external modules that libwww works with for
 details. Thanks so much to John Punin for writing this code!
 */
@@ -36,7 +36,7 @@ details. Thanks so much to John Punin for writing this code!
 #include "SGML.h"
 #endif /* HT_STRUCT_XML_STREAM */
 
-#include <xmlparse.h>
+#include <expat.h>
 
 /*
 .
@@ -101,7 +101,7 @@ extern BOOL HTXMLStructured_setHandlers(
 extern BOOL HTXMLStructured_setUserData (HTStream * me, void * user_data);
 extern HTStream * HTXMLStructured_new (const SGML_dtd * dtd, HTStructured *
 starget);
-#endif /* HT_STRUCT_XML_STREAM */
+#endif
 
 /*
 */
@@ -112,6 +112,6 @@ starget);
 
   
 
-  @(#) $Id: HTXML.html,v 2.4 2000/08/29 16:16:37 kahan Exp $
+  @(#) $Id: HTXML.html,v 2.7 2003/03/21 10:15:44 kahan Exp $
 
 */
