@@ -1,10 +1,10 @@
 /*
  *
- *  (c) COPYRIGHT MIT and INRIA, 1996.
+ *  (c) COPYRIGHT INRIA and W3C, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
- 
+  
 /*
  * AHTBridge.c : This module implements the callback setup and
  * handlers between the Xt, libwww, and Amaya procedures. It's responsible
@@ -13,13 +13,18 @@
  *
  * Author: J Kahan
  *         J. K./R. Guetari. Windows NT/95 routines
+ *         J. K./S. Gully GTK routines
  *
  */
-#ifdef _GTK
-#include <glib.h>
+
+#if defined(_GTK)
+  #include <glib.h>
 #endif /* _GTK */
 
-#ifndef AMAYA_JAVA
+#ifdef _WX
+  #include "wxAmayaTimer.h"
+  #include "wxAmayaSocketEvent.h"
+#endif /* _WX */
 
 #define THOT_EXPORT extern
 #include "amaya.h"
@@ -34,86 +39,20 @@
 #define THD_TRACE 1
 #endif
 
-#ifndef _WINDOWS
-/* Amaya's X appcontext */
-extern ThotAppContext app_cont;
-#ifndef _GTK
+#ifdef _MOTIF
+  /* Amaya's X appcontext */
+  extern ThotAppContext app_cont;
+#endif /* #ifdef _MOTIF */
 
-/* Private functions */
-#ifdef __STDC__
-static void         RequestRegisterReadXtevent (SOCKET);
-static void         RequestKillReadXtevent (SOCKET);
-static void         RequestRegisterWriteXtevent ( SOCKET);
-static void         RequestKillWriteXtevent (SOCKET);
-static void         RequestRegisterExceptXtevent ( SOCKET);
-static void         RequestKillExceptXtevent (SOCKET);
-#else /* __STDC__ */
-static void         RequesAddReadXtevent ();
-static void         RequestKillReadXtevent ();
-static void         RequesAddWriteXtevent ();
-static void         RequestKillWriteXtevent ();
-static void         RequestRegisterExceptXtevent ();
-static void         RequestKillExceptXtevent ();
-#endif /* __STDC__ */
-
-#endif /* _GTK */
-#endif /* !_WINDOWS */
-
-#ifdef _GTK
-#define WWW_HIGH_PRIORITY (G_PRIORITY_HIGH_IDLE + 50)
-#define WWW_LOW_PRIORITY G_PRIORITY_LOW
-#define WWW_SCALE_PRIORITY(p) ((WWW_HIGH_PRIORITY - WWW_LOW_PRIORITY) * p \
-                          / HT_PRIORITY_MAX + WWW_LOW_PRIORITY)
-     
-#define READ_CONDITION (G_IO_IN | G_IO_HUP | G_IO_ERR)
-#define WRITE_CONDITION (G_IO_OUT | G_IO_ERR)
-#define EXCEPTION_CONDITION (G_IO_PRI)
-     
- typedef struct _SockEventInfo SockEventInfo;
- struct _SockEventInfo {
-   SOCKET s;
-   HTEventType type;
-   HTEvent *event;
-   guint io_tag;
-   guint timer_tag;
- };
- 
- typedef struct _SockInfo SockInfo;
- struct _SockInfo {
-   SOCKET s;
-   GIOChannel *io;
-   SockEventInfo ev[HTEvent_TYPES];
- };
- 
- static GHashTable *sockhash = NULL;
- 
- 
- static SockInfo *
- get_sock_info(SOCKET s, gboolean create)
- {
-   SockInfo *info;
- 
-   if (!sockhash)
-     sockhash = g_hash_table_new(g_direct_hash, g_direct_equal);
- 
-   info = g_hash_table_lookup(sockhash, GINT_TO_POINTER(s));
-   if (!info && create) {
-     info = g_new0(SockInfo, 1);
-     info->s = s;
-     info->io = g_io_channel_unix_new(s);
-     info->ev[0].s = info->ev[1].s = info->ev[2].s = s;
-     info->ev[0].type = HTEvent_READ;
-     info->ev[1].type = HTEvent_WRITE;
-     info->ev[2].type = HTEvent_OOB;
-     g_hash_table_insert(sockhash, GINT_TO_POINTER(s), info);
-   }
-   return info;
- }
- 
- static gboolean glibwww_timeout_func (gpointer data);
- static gboolean glibwww_io_func(GIOChannel *source, GIOCondition condition,
-                                 gpointer data);
-#endif /* _GTK */
+#if defined(_MOTIF) || defined(_GTK) || defined(_WX) || defined(_NOGUI)
+  /* Private functions */
+  static void         RequestRegisterReadXtevent (SOCKET);
+  static void         RequestKillReadXtevent (SOCKET);
+  static void         RequestRegisterWriteXtevent ( SOCKET);
+  static void         RequestKillWriteXtevent (SOCKET);
+  static void         RequestRegisterExceptXtevent ( SOCKET);
+  static void         RequestKillExceptXtevent (SOCKET);
+#endif /* #if defined(_MOTIF) || defined(_GTK) */
 
 /* Private variables */
 
@@ -126,22 +65,48 @@ static void         RequestKillExceptXtevent ();
  * BSD Unix semantics 
  */
 
-#ifndef _GTK
-static const HTEventType ReadBits = HTEvent_READ | HTEvent_ACCEPT | HTEvent_CLOSE;
-static const HTEventType WriteBits = HTEvent_WRITE | HTEvent_CONNECT;
-static const HTEventType ExceptBits = HTEvent_OOB;
+static const HTEventType ReadBits = (HTEventType)(HTEvent_READ | HTEvent_ACCEPT | HTEvent_CLOSE);
+static const HTEventType WriteBits = (HTEventType)(HTEvent_WRITE | HTEvent_CONNECT);
+static const HTEventType ExceptBits = (HTEventType)HTEvent_OOB;
 
-typedef struct sStatus {
-  XtInputId read;             /* the different XtId's */
-  XtInputId write;
-  XtInputId except;
-} SocketStatus;
+#ifdef _MOTIF
+  typedef struct sStatus {
+    XtInputId read;             /* the different Xt Id's */
+    XtInputId write;
+    XtInputId except;
+  } SocketStatus;
+#endif /*! #ifdef _MOTIF */
 
-#define SOCK_TABLE_SIZE 67
-#define HASH(s) ((s) % SOCK_TABLE_SIZE)
-static SocketStatus persSockets[SOCK_TABLE_SIZE];
+#if defined(_GTK)
+  typedef struct sStatus {
+    gint read;             /* the different GTK Id's */
+    gint write;
+    gint except;
+  } SocketStatus;
 #endif /* _GTK */
 
+#if defined(_WX)
+typedef struct sStatus {
+  int read;
+  int write;
+  int except;
+} SocketStatus;
+#endif /* _WX */
+
+#if defined(_NOGUI)
+  typedef struct sStatus {
+    int read;
+    int write;
+    int except;
+  } SocketStatus;
+#endif /* #ifdef _NOGUI */
+
+#if defined(_MOTIF) || defined(_GTK) || defined(_WX) || defined(_NOGUI)
+  #define SOCK_TABLE_SIZE 67
+  #define HASH(s) ((s) % SOCK_TABLE_SIZE)
+  static SocketStatus persSockets[SOCK_TABLE_SIZE];
+#endif /* #if defined(_MOTIF) || defined(_GTK) || defined(_WX) || defined(_NOGUI) */
+
 /*--------------------------------------------------------------------
   AHTCallback_bridge
   this function acts as a bridge between Xt and libwww. From the Xt
@@ -158,17 +123,9 @@ static SocketStatus persSockets[SOCK_TABLE_SIZE];
   the state of the request and, if it's an asynchronous request, deletes
   the memory allocated to it.
   -------------------------------------------------------------------*/
-#ifndef _WINDOWS
-#ifndef _GTK
-#ifdef __STDC__
 void *AHTCallback_bridge (caddr_t cd, int *s, XtInputId * id)
-#else  /* __STDC__ */
-void *AHTCallback_bridge (cd, s, id)
-caddr_t             cd;
-int                *s;
-XtInputId          *id;
-#endif /* __STDC__ */
 {
+#ifdef _MOTIF
    int                 status;  /* the status result of the libwwww call */
    HTEventType         type  = HTEvent_ALL;	
    int                 v;
@@ -208,11 +165,99 @@ XtInputId          *id;
      }
 
    /***   CanDoStop_set (TRUE); **/
-
+#endif /* _MOTIF */
+   
    return (0);
 }
-#endif /* _GTK */
-#endif /* !_WINDOWS */
+
+#if defined(_GTK)
+static void AHTCallback_bridgeGTK (gpointer data,  gint source, GdkInputCondition condition)
+{
+   int                 status;  /* the status result of the libwwww call */
+   HTEventType         type  = HTEvent_ALL;	
+   int                 v;
+   int 		       socket;
+   ms_t                now = HTGetTimeInMillis();
+   
+   socket = (SOCKET) data;
+   v = HASH (socket);
+
+   /* convert the FD into an HTEventType which will allow us to find the
+      request associated with the socket */
+
+   /* I could send some other data here, like the event itself, right */
+   switch (condition) 
+     {
+     case GDK_INPUT_READ:
+       type = HTEvent_READ;
+       break;
+     case GDK_INPUT_WRITE:
+       type = HTEvent_WRITE;
+       break;
+     case GDK_INPUT_EXCEPTION:
+       type = HTEvent_OOB;
+       break;
+     default:
+       type = HTEvent_ALL; 
+       break;
+     } /* switch */
+   
+   /* Invokes the callback associated to the requests */
+   
+   /**   CanDoStop_set (FALSE); **/
+   if ((status = HTEventList_dispatch (socket, type, now)) != HT_OK)
+     {
+#ifdef DEBUG_LIBWWW
+     HTTrace ("Callback.... returned a value != HT_OK");
+#endif
+     }
+   /***   CanDoStop_set (TRUE); **/
+}
+#endif /* _GTK  */
+
+
+#if defined(_WX)
+static void AHTCallback_bridgeWX ( int register_id,  int socket, wxAmayaSocketCondition condition)
+{
+  int                 status;  /* the status result of the libwwww call */
+  HTEventType         type  = HTEvent_ALL;	
+  int                 v;
+  ms_t                now = HTGetTimeInMillis();
+   
+  v = HASH (socket);
+
+  /* convert the FD into an HTEventType which will allow us to find the
+     request associated with the socket */
+
+  /* I could send some other data here, like the event itself, right */
+  switch (condition) 
+    {
+    case WXAMAYASOCKET_READ:
+      type = HTEvent_READ;
+      break;
+    case WXAMAYASOCKET_WRITE:
+      type = HTEvent_WRITE;
+      break;
+    case WXAMAYASOCKET_EXCEPTION:
+      type = HTEvent_OOB;
+      break;
+    default:
+      type = HTEvent_ALL; 
+      break;
+    } /* switch */
+   
+   /* Invokes the callback associated to the requests */
+   
+   /**   CanDoStop_set (FALSE); **/
+   if ((status = HTEventList_dispatch (socket, type, now)) != HT_OK)
+     {
+#ifdef DEBUG_LIBWWW
+     HTTrace ("Callback.... returned a value != HT_OK");
+#endif
+     }
+   /***   CanDoStop_set (TRUE); **/
+}
+#endif /* _WX  */
 
 /*--------------------------------------------------------------------
   ProcessTerminateRequest
@@ -220,17 +265,10 @@ XtInputId          *id;
   ended normally, the function will call any callback associated to the
   request. Otherwise, it will just mark the request as over.
   -------------------------------------------------------------------*/
-#ifdef __STDC__
-void  ProcessTerminateRequest (HTRequest * request, HTResponse * response, void *param, int status)
-#else
-void ProcessTerminateRequest (request, response, param, status)
-HTRequest *request;
-HTResponse *response;
-void *param;
-int status;
-#endif
+void  ProcessTerminateRequest (HTRequest *request, HTResponse *response,
+			       void *param, int status)
 {   
-  AHTReqContext *me = HTRequest_context (request);
+  AHTReqContext *me = (AHTReqContext *)HTRequest_context (request);
 
   /* choose a correct treatment in function of the request's
      being associated with an error, with an interruption, or with a
@@ -238,13 +276,14 @@ int status;
 
 #ifdef DEBUG_LIBWWW  
   if (THD_TRACE)
-    fprintf (stderr,"ProcessTerminateRequest: processing req %p, url %s, status %d\n", me, me->urlName, me->reqStatus);  
+    fprintf (stderr,"ProcessTerminateRequest: processing req %p, url %s,\
+ status %d\n", me, me->urlName, me->reqStatus);  
 #endif /* DEBUG_LIBWWW */
   if (me->reqStatus == HT_END)
     {
       if (AmayaIsAlive ()  && me->terminate_cbf)
-	(*me->terminate_cbf) (me->docid, 0, ISO2WideChar(me->urlName), ISO2WideChar(me->outputfile),
-			      ISO2WideChar(me->content_type), me->context_tcbf);
+	(*me->terminate_cbf) (me->docid, 0, me->urlName, me->outputfile,
+			      &(me->http_headers), me->context_tcbf);
 
     }
   else if (me->reqStatus == HT_ABORT)
@@ -252,24 +291,24 @@ int status;
        button. We erase the incoming file, if it exists */
     {
       if (AmayaIsAlive () && me->terminate_cbf)
-	(*me->terminate_cbf) (me->docid, -1, ISO2WideChar(me->urlName), ISO2WideChar(me->outputfile),
-			      ISO2WideChar(me->content_type), me->context_tcbf);
+	(*me->terminate_cbf) (me->docid, -1, me->urlName, me->outputfile,
+			      &(me->http_headers), me->context_tcbf);
       if (me->outputfile && me->outputfile[0] != EOS)
 	{
-	  TtaFileUnlink (ISO2WideChar (me->outputfile));
-	  me->outputfile[0] = EOS;
-	}
+	  TtaFileUnlink (me->outputfile);
+	  me->outputfile[0] = EOS; 
+	} 
     }
   else if (me->reqStatus == HT_ERR)
     {
       /* there was an error */
-      if (AmayaIsAlive && me->terminate_cbf)
-	(*me->terminate_cbf) (me->docid, -1, ISO2WideChar(me->urlName), ISO2WideChar(me->outputfile),
-			      ISO2WideChar(me->content_type), me->context_tcbf);
+      if (AmayaIsAlive () && me->terminate_cbf)
+	(*me->terminate_cbf) (me->docid, -2, me->urlName, me->outputfile,
+			      &(me->http_headers), me->context_tcbf);
       
       if (me->outputfile && me->outputfile[0] != EOS)
 	{
-	  TtaFileUnlink (ISO2WideChar (me->outputfile));
+	  TtaFileUnlink (me->outputfile);
 	  me->outputfile[0] = EOS;
 	}
     }
@@ -287,7 +326,7 @@ int status;
     }
 }
 
-#ifdef _WINDOWS
+#ifdef _WINGUI
 /*----------------------------------------------------------------
   WIN_Activate_Request
   when there are more open requests than available sockets, the 
@@ -300,18 +339,7 @@ int status;
   Consult the libwww manual for more details on the signature
   of this function.
   ----------------------------------------------------------------*/
-#ifdef __STDC__
-int                 WIN_Activate_Request (HTRequest * request, HTAlertOpcode op, int msgnum, const char *dfault, void *input, HTAlertPar * reply)
-#else
-int                 WIN_Activate_Request (request, op, msgnum, dfault, input, reply)
-HTRequest          *request;
-HTAlertOpcode       op;
-int                 msgnum;
-const char         *dfault;
-void               *input;
-HTAlertPar         *reply;
-
-#endif /* __STDC__ */
+int WIN_Activate_Request (HTRequest * request, HTAlertOpcode op, int msgnum, const char *dfault, void *input, HTAlertPar * reply)
 {
    AHTReqContext      *me = HTRequest_context (request);
 
@@ -320,12 +348,10 @@ HTAlertPar         *reply;
        if (!(me->output)
 	   && (me->output != stdout) 
 	   && me->outputfile
-	   &&  (me->output = ufopen (ISO2WideChar (me->outputfile), _WBinaryMODE_)) == NULL) {
+	   &&  (me->output = fopen (me->outputfile, "wb")) == NULL) {
 	 /* the request is associated with a file */
 	 me->outputfile[0] = EOS;	/* file could not be opened */
-	 TtaSetStatus (me->docid, 1, 
-		       TtaGetMessage (AMAYA, AM_CANNOT_CREATE_FILE),
-		       ISO2WideChar(me->outputfile));
+	 TtaSetStatus (me->docid, 1, TtaGetMessage (AMAYA, AM_CANNOT_CREATE_FILE), me->outputfile);
 	 me->reqStatus = HT_ERR;
 
 	 if (me->error_html)
@@ -357,11 +383,7 @@ HTAlertPar         *reply;
    return ((me->reqStatus != HT_ERR) ? HT_OK : HT_ERROR);
 }
 
-#endif /* WINDOWS but should be an else */
-/* #else  */ /* _WINDOWS */
-
-#ifndef _WINDOWS
-#ifndef _GTK
+#endif /* _WINGUI */
 
 /*----------------------------------------------------------------------
   AHTEvent_register
@@ -370,16 +392,9 @@ HTAlertPar         *reply;
   loops gets an interruption whenever there's action of the socket. 
   In addition, it registers the request with libwww.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 int AHTEvent_register (SOCKET sock, HTEventType type, HTEvent *event)
-#else
-int AHTEvent_register (sock, type, event)
-SOCKET sock;
-HTEventType type;
-HTEvent *event;
-#endif /* __STDC__ */
 {
-  int  status;
+  int  status = 0;
 
   if (sock == INVSOC)
     {
@@ -389,8 +404,8 @@ HTEvent *event;
       return (0);
     }
 
-#ifndef _WINDOWS
-	/* need something special for HTEvent_CLOSE */
+#if defined(_MOTIF) || defined(_GTK) || defined(_WX)
+  /* need something special for HTEvent_CLOSE */
   if (type & ReadBits)
     RequestRegisterReadXtevent (sock);
   
@@ -399,66 +414,21 @@ HTEvent *event;
   
   if (type & ExceptBits)
     RequestRegisterExceptXtevent (sock);
-#endif	 /* !_WINDOWS */
+#endif	 /* #if defined(_MOTIF) || defined(_GTK) || defined(_WX) */
   
-#ifdef _WINDOWS   
+#ifdef _WINDOWS
   /* under windows, libwww requires an explicit FD_CLOSE registration 
      to detect HTTP responses not having a Content-Length header */
-  status = HTEventList_register (sock, type | HTEvent_CLOSE , event);
-#else
-  status = HTEventList_register (sock, type, event);
+  status = HTEventList_register (sock, (HTEventType)(type | HTEvent_CLOSE) , event);
 #endif /* _WINDOWS */
+
+#if defined(_UNIX)
+  status = HTEventList_register (sock, type, event);
+#endif /* #if defined(_UNIX) */
   
   return (status);
 }
-#endif /* _GTK */
-#endif /* _WINDOWS */
 
-#ifdef _GTK
-/*----------------------------------------------------------------------
-  AHTEvent_register FOR GTK
-  callback called by libwww whenever a socket is open and associated
-  to a request. It sets the pertinent Xt events so that the Xt Event
-  loops gets an interruption whenever there's action of the socket. 
-  In addition, it registers the request with libwww.
-  ----------------------------------------------------------------------*/
-int AHTEvent_register (SOCKET s, HTEventType type, HTEvent *event)
- {
-   SockInfo *info;
-   gint priority = G_PRIORITY_DEFAULT;
-   GIOCondition condition;
- 
-   if (s == INVSOC || HTEvent_INDEX(type) >= HTEvent_TYPES)
-     return 0;
- 
-   info = get_sock_info(s, TRUE);
-   info->ev[HTEvent_INDEX(type)].event = event;
- 
-   switch (HTEvent_INDEX(type)) {
-   case HTEvent_INDEX(HTEvent_READ):
-     condition = READ_CONDITION;      break;
-   case HTEvent_INDEX(HTEvent_WRITE):
-     condition = WRITE_CONDITION;     break;
-   case HTEvent_INDEX(HTEvent_OOB):
-     condition = EXCEPTION_CONDITION; break;
-   }
-   if (event->priority != HT_PRIORITY_OFF)
-     priority = WWW_SCALE_PRIORITY(event->priority);
- 
-   info->ev[HTEvent_INDEX(type)].io_tag =
-     g_io_add_watch_full(info->io, priority, condition, glibwww_io_func,
-                          &info->ev[HTEvent_INDEX(type)], NULL);
- 
-   if (event->millis >= 0)
-     info->ev[HTEvent_INDEX(type)].timer_tag =
-       g_timeout_add_full(priority, event->millis, glibwww_timeout_func,
-                          &info->ev[HTEvent_INDEX(type)], NULL);
- 
-   return HT_OK;
- }
-#endif /* _GTK */
-
-#ifndef _GTK
 /*----------------------------------------------------------------------
   AHTEvent_unregister
   callback called by libwww each time a request is unregistered. This
@@ -466,18 +436,14 @@ int AHTEvent_register (SOCKET s, HTEventType type, HTEvent *event)
   associated with the request's socket. In addition, it unregisters
   the request from libwww.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 int AHTEvent_unregister (SOCKET sock, HTEventType type)
-#else
-int AHTEvent_unregister (sock, type)
-SOCKET              sock;
-HTEventType         type;
-
-#endif /* __STDC__ */
 {
   int    status;
+  
+  if (sock == INVSOC)
+    return HT_OK;
 
-#ifndef _WINDOWS   
+#if defined(_MOTIF) || defined(_GTK) || defined(_WX)
    /* remove the Xt event hooks */
    if (type & ReadBits) 
      RequestKillReadXtevent (sock);
@@ -487,7 +453,7 @@ HTEventType         type;
    
    if (type & ExceptBits) 
      RequestKillExceptXtevent (sock);
-#endif /* !_WINDOWS */
+#endif /* #if defined(_MOTIF) || defined(_GTK) || defined(_WX) */
 
    /* @@@ if this is the default for windows, no need to have AHTEvent_..
       in windows!! */
@@ -497,78 +463,6 @@ HTEventType         type;
    
    return (status);
 }
-#endif /* _GTK */
-
-#ifdef _GTK
-/*----------------------------------------------------------------------
-  AHTEvent_unregister FOR GTK
-  callback called by libwww each time a request is unregistered. This
-  function takes care of unregistering the pertinent Xt events
-  associated with the request's socket. In addition, it unregisters
-  the request from libwww.
-  ----------------------------------------------------------------------*/
-int AHTEvent_unregister (SOCKET s, HTEventType type)
- {
-   SockInfo *info = get_sock_info(s, FALSE);
- 
-   if (info) {
-     if (info->ev[HTEvent_INDEX(type)].io_tag)
-       g_source_remove(info->ev[HTEvent_INDEX(type)].io_tag);
-     if (info->ev[HTEvent_INDEX(type)].timer_tag)
-       g_source_remove(info->ev[HTEvent_INDEX(type)].timer_tag);
- 
-     info->ev[HTEvent_INDEX(type)].event = NULL;
-     info->ev[HTEvent_INDEX(type)].io_tag = 0;
-     info->ev[HTEvent_INDEX(type)].timer_tag = 0;
- 
-     /* clean up sock hash if needed */
-     if (info->ev[0].event == NULL &&
-         info->ev[1].event == NULL &&
-         info->ev[2].event == NULL) {
-       g_hash_table_remove(sockhash, GINT_TO_POINTER(s));
-       g_io_channel_unref(info->io);
-       g_free(info);
-     }
-     
-     return HT_OK;
-   }
-   return HT_ERROR;
- }
-static gboolean
-glibwww_timeout_func (gpointer data)
- {
-   SockEventInfo *info = (SockEventInfo *)data;
-   HTEvent *event = info->event;
- 
-   (* event->cbf) (info->s, event->param, HTEvent_TIMEOUT);
-   return TRUE;
- }
- 
-static gboolean
-glibwww_io_func(GIOChannel *source, GIOCondition condition, gpointer data)
- {
-   SockEventInfo *info = (SockEventInfo *)data;
-   HTEvent *event = info->event;
- 
-   if (info->timer_tag)
-       g_source_remove(info->timer_tag);
-   if (info->event->millis >= 0) {
-     gint priority = G_PRIORITY_DEFAULT;
- 
-     if (event->priority != HT_PRIORITY_OFF)
-       priority = WWW_SCALE_PRIORITY(event->priority);
-     info->timer_tag =
-       g_timeout_add_full(priority, info->event->millis, glibwww_timeout_func,
-                          info, NULL);
-   }
- 
-   (* event->cbf) (info->s, event->param, info->type);
-   return TRUE;
- }
-
-#endif /* _GTK */
-
-#ifndef _WINDOWS
 
 /* Private functions */
 
@@ -577,14 +471,9 @@ glibwww_io_func(GIOChannel *source, GIOCondition condition, gpointer data)
   front-end for kill all Xt events associated with the request pointed
   to by "me".
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                RequestKillAllXtevents (AHTReqContext * me)
-#else
-void                RequestKillAllXtevents (me)
-AHTReqContext      *me;
-#endif /* __STDC__ */
+void RequestKillAllXtevents (AHTReqContext * me)
 {
-#ifndef _GTK
+#if defined(_MOTIF) || defined(_GTK) || defined(_WX) || defined(_NOGUI)
   int sock = INVSOC;
 
   return;
@@ -608,54 +497,63 @@ AHTReqContext      *me;
    RequestKillReadXtevent (sock);
    RequestKillWriteXtevent (sock);
    RequestKillExceptXtevent (sock);
-#endif /* _GTK */
+#endif /* #if defined(_MOTIF) || defined(_GTK) || defined(_WX) || defined(_NOGUI) */   
 }
 
-#ifndef _GTK
 /*----------------------------------------------------------------------
   RequestRegisterReadXtevent
   Registers with Xt the read events associated with socket sock
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void         RequestRegisterReadXtevent (SOCKET sock)
-#else
-static void         RequestRegisterReadXtevent (sock)
-SOCKET sock;
-#endif /* __STDC__ */
+static void RequestRegisterReadXtevent (SOCKET sock)
 {
+#if defined(_MOTIF) || defined(_GTK) || defined(_WX) || defined(_NOGUI)
   int v;
 
   v = HASH (sock);
 
   if (!persSockets[v].read)
     {
+      
+#ifdef _MOTIF
       persSockets[v].read  =
 	XtAppAddInput (app_cont,
 		       sock,
 		       (XtPointer) XtInputReadMask,
 		       (XtInputCallbackProc) AHTCallback_bridge,
 		       (XtPointer) XtInputReadMask);
+#endif /* _MOTIF */
       
+#if defined(_GTK)
+     persSockets[v].read  =
+       gdk_input_add ((gint) sock,
+		      GDK_INPUT_READ,
+		      AHTCallback_bridgeGTK,
+		      (gpointer) sock);
+#endif /* _GTK */
+
+#if defined(_WX)
+     persSockets[v].read  =
+       wxAmayaSocketEvent::RegisterSocket( sock,
+					   WXAMAYASOCKET_READ,
+					   AHTCallback_bridgeWX );
+#endif /* _WX */
+
 #ifdef DEBUG_LIBWWW
       if (THD_TRACE)
 	fprintf (stderr, "RegisterReadXtEvent: adding XtInput %lu Socket %d\n",
 		 persSockets[v].read, sock);
 #endif /* DEBUG_LIBWWW */
     }
-
+#endif /* #if defined(_MOTIF) || defined(_GTK) || defined(_WX) || defined(_NOGUI) */
 }
 
 /*----------------------------------------------------------------------
   RequestKillReadXtevent
   kills any read Xt event associated with the request pointed to by "me".
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void         RequestKillReadXtevent (SOCKET sock)
-#else
-static void         RequestKillReadXtevent (sock)
-SOCKET              sock;
-#endif /* __STDC__ */
+static void RequestKillReadXtevent (SOCKET sock)
 {
+#if defined(_MOTIF) || defined(_GTK) || defined(_WX) || defined(_NOGUI)
   int v;
 
   v = HASH (sock);
@@ -667,34 +565,62 @@ SOCKET              sock;
 	fprintf (stderr, "UnregisterReadXtEvent: Clearing XtInput %lu\n",
 		 persSockets[v].read);
 #endif /* DEBUG_LIBWWW */
+
+#ifdef _MOTIF
       XtRemoveInput (persSockets[v].read);
       persSockets[v].read = (XtInputId) NULL;
+#endif /* _MOTIF */
+      
+#if defined(_GTK)
+      gdk_input_remove (persSockets[v].read);
+      persSockets[v].read = (gint) 0;
+#endif /* !_GTK */
+
+#if defined(_WX)
+      wxAmayaSocketEvent::UnregisterSocket( persSockets[v].read );
+      persSockets[v].read = 0;
+#endif /* _WX */
+
     }
+#endif /* #if defined(_MOTIF) || defined(_GTK) || defined(_WX) || defined(_NOGUI) */  
 }
 
 /*----------------------------------------------------------------------
   RequestRegisterWriteXtevent
   Registers with Xt the write events associated with socket sock
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void         RequestRegisterWriteXtevent (SOCKET sock)
-#else
-static void         RequestRegisterWriteXtevent (sock)
-SOCKET              sock;
-
-#endif /* __STDC__ */
+static void RequestRegisterWriteXtevent (SOCKET sock)
 {
+#if defined(_MOTIF) || defined(_GTK) || defined(_WX) || defined(_NOGUI)
   int v;
   v = HASH (sock);
 
   if (!persSockets[v].write)
     {   
+#ifdef _MOTIF
       persSockets[v].write =
 	XtAppAddInput (app_cont,
 		       sock,
 		   (XtPointer) XtInputWriteMask,
 		   (XtInputCallbackProc) AHTCallback_bridge,
 		   (XtPointer) XtInputWriteMask);
+#endif /* _MOTIF */
+      
+#if defined(_GTK) 
+     persSockets[v].write  =
+       gdk_input_add ((gint) sock,
+		      GDK_INPUT_WRITE,
+		      AHTCallback_bridgeGTK,
+		      (gpointer) sock);
+#endif /* _GTK  */
+
+#if defined(_WX)
+     persSockets[v].write  =
+       wxAmayaSocketEvent::RegisterSocket( sock,
+					   WXAMAYASOCKET_WRITE,
+					   AHTCallback_bridgeWX );
+#endif /* _WX */
+
 #ifdef DEBUG_LIBWWW   
   if (THD_TRACE)
     fprintf (stderr, "RegisterWriteXtEvent: Adding XtInput %lu Socket %d\n",
@@ -702,6 +628,7 @@ SOCKET              sock;
 #endif /* DEBUG_LIBWWW */
   
     }
+#endif /* #if defined(_MOTIF) || defined(_GTK) || defined(_WX) || defined(_NOGUI) */  
 }
 
 /*----------------------------------------------------------------------
@@ -709,13 +636,9 @@ SOCKET              sock;
   kills any write Xt event associated with the request pointed to
   by "me".
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void         RequestKillWriteXtevent (SOCKET sock)
-#else
-static void         RequestKillWriteXtevent (sock)
-SOCKET sock;
-#endif /* __STDC__ */
+static void RequestKillWriteXtevent (SOCKET sock)
 {
+#if defined(_MOTIF) || defined(_GTK) || defined(_WX) || defined(_NOGUI)
   int v;
 
   v = HASH (sock);
@@ -728,42 +651,70 @@ SOCKET sock;
 		 "%lu\n",
 		 persSockets[v].write);
 #endif /* DEBUG_LIBWWW */
+      
+#ifdef _MOTIF
       XtRemoveInput (persSockets[v].write);
       persSockets[v].write =  (XtInputId) NULL;
+#endif /* _MOTIF */
+      
+#if defined(_GTK)
+      gdk_input_remove (persSockets[v].write);
+      persSockets[v].write = (gint) 0;
+#endif /* _GTK */
+
+#if defined(_WX)
+      wxAmayaSocketEvent::UnregisterSocket( persSockets[v].write );
+      persSockets[v].write = 0;
+#endif /* _WX */
+
     }
+#endif /* #if defined(_MOTIF) || defined(_GTK) || defined(_WX) || defined(_NOGUI) */  
 }
 
 /*----------------------------------------------------------------------
   RequestRegisterExceptXtevent
   Registers with Xt the except events associated with socket sock
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void         RequestRegisterExceptXtevent (SOCKET sock)
-#else
-static void         RequestRegisterExceptXtevent (sock)
-SOCKET              sock;
-
-#endif /* __STDC__ */
+static void RequestRegisterExceptXtevent (SOCKET sock)
 {
+#if defined(_MOTIF) || defined(_GTK) || defined(_WX) || defined(_NOGUI)
   int v;
 
   v = HASH (sock);
 
    if (!persSockets[v].except)
      {
-   
+#ifdef _MOTIF
        persSockets[v].except =
 	 XtAppAddInput (app_cont,
 			sock,
 			(XtPointer) XtInputExceptMask,
 			(XtInputCallbackProc) AHTCallback_bridge,
 			(XtPointer) XtInputExceptMask);
-#ifdef DEBUG_LIBWWW      
+#endif /* _MOTIF */
+       
+#if defined(_GTK)
+     persSockets[v].except  =
+       gdk_input_add ((gint) sock,
+		      GDK_INPUT_EXCEPTION,
+		      AHTCallback_bridgeGTK,
+		      (gpointer) sock);
+#endif /* _GTK */
+
+#if defined(_WX)
+     persSockets[v].except =
+       wxAmayaSocketEvent::RegisterSocket( sock,
+					   WXAMAYASOCKET_EXCEPTION,
+					   AHTCallback_bridgeWX );
+#endif /* _WX */
+
+#ifdef DEBUG_LIBWWW
    if (THD_TRACE)
      fprintf (stderr, "RegisterExceptXtEvent: adding XtInput %lu Socket %d\n",
 	      persSockets[v].except, sock);
 #endif /* DEBUG_LIBWWW */
      }
+#endif /* #if defined(_MOTIF) || defined(_GTK) || defined(_WX) || defined(_NOGUI) */   
 }
 
 /*----------------------------------------------------------------------
@@ -771,13 +722,9 @@ SOCKET              sock;
   kills any exception Xt event associated with the request pointed to
   by "me".
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void         RequestKillExceptXtevent (SOCKET sock)
-#else
-static void         RequestKillExceptXtevent (sock)
-SOCKET sock;
-#endif /* __STDC__ */
+static void RequestKillExceptXtevent (SOCKET sock)
 {
+#if defined(_MOTIF) || defined(_GTK) || defined(_WX) || defined(_NOGUI)
   int v;
 
   v = HASH (sock);
@@ -788,9 +735,24 @@ SOCKET sock;
 	fprintf (stderr, "UnregisterExceptXtEvent: Clearing Except XtInputs "
 		 "%lu\n", persSockets[v].except);
 #endif /* DEBUG_LIBWWW */
+
+#ifdef _MOTIF
       XtRemoveInput (persSockets[v].except);
       persSockets[v].except = (XtInputId) NULL;
+#endif /* _MOTIF */
+      
+#if defined(_GTK)
+      gdk_input_remove (persSockets[v].except);
+      persSockets[v].except = (gint) 0;
+#endif /* _GTK */
+
+#if defined(_WX)
+      wxAmayaSocketEvent::UnregisterSocket( persSockets[v].except );
+      persSockets[v].except = 0;
+#endif /* _WX */
+      
     }
+#endif /* #if defined(_MOTIF) || defined(_GTK) || defined(_WX) || defined(_NOGUI) */
 }
 
 /*----------------------------------------------------------------------
@@ -808,7 +770,23 @@ struct _HTTimer {
 
 struct _AmayaTimer {
   HTTimer *libwww_timer;
+
+#ifdef _MOTIF
   XtIntervalId xt_timer;
+#endif /* _MOTIF */
+  
+#ifdef _GTK
+  guint  xt_timer;
+#endif /* _GTK */
+
+#ifdef _WX
+  wxAmayaTimer * xt_timer;
+#endif /* _WX */
+
+#ifdef _NOGUI
+  unsigned int xt_timer;
+#endif /* #ifdef _NOGUI */  
+  
 };
 
 typedef struct _AmayaTimer AmayaTimer;
@@ -816,9 +794,13 @@ typedef struct _AmayaTimer AmayaTimer;
 static HTList *Timers = NULL;
 
 /*----------------------------------------------------------------------
+  TimerCallback
+  called by the system event loop. Timers shouldn't be restarted
+  on exiting.
   ----------------------------------------------------------------------*/
 void *TimerCallback (XtPointer cdata, XtIntervalId *id)
 {
+#ifdef _MOTIF
   HTList *cur, *last;
   AmayaTimer *me;
   HTTimer *libwww_timer;
@@ -844,39 +826,116 @@ void *TimerCallback (XtPointer cdata, XtIntervalId *id)
       TtaFreeMemory (me);
       HTTimer_dispatch (libwww_timer);
     }
-
+#endif /* _MOTIF */
+  
   return (0);
+  
+}
+
+#ifdef _GTK
+/*----------------------------------------------------------------------
+  TimerCallbackGTK
+  The callback returns FALSE to destroy the timer that called it.
+  ----------------------------------------------------------------------*/
+gboolean TimerCallbackGTK (gpointer id)
+{
+  HTList *cur, *last;
+  AmayaTimer *me;
+  HTTimer *libwww_timer;
+  AmayaTimer *data;
+
+  data = (AmayaTimer *) id;
+
+  if (!AmayaIsAlive () 
+      || Timers == NULL)
+    return (FALSE);
+
+  /* find the timer from the uid */
+  last = cur = Timers;
+  while ((me = (AmayaTimer * ) HTList_nextObject (cur)))
+    {
+      if (me == data)
+	break;
+      last = cur;
+    }
+
+  if (me)
+    {
+      libwww_timer = me->libwww_timer;
+      /* remove the element from the list @@@ can be optimized later */
+      HTList_quickRemoveElement(cur, last);
+      TtaFreeMemory (me);
+      HTTimer_dispatch (libwww_timer);
+    }
+  return (FALSE);
+}
+#endif /* _GTK */
+
+/*----------------------------------------------------------------------
+  TimerCallbackWX
+  called when a timer is throw
+  ----------------------------------------------------------------------*/
+void TimerCallbackWX( void * p_context )
+{
+#ifdef _WX
+  HTList *cur, *last;
+  AmayaTimer *me;
+  HTTimer *libwww_timer;
+  AmayaTimer *data;
+
+  data = (AmayaTimer *)p_context;
+
+  if (!AmayaIsAlive () 
+      || Timers == NULL)
+    return;
+
+  /* find the timer from the uid */
+  last = cur = Timers;
+  while ((me = (AmayaTimer * ) HTList_nextObject (cur)))
+    {
+      if (me == data)
+	break;
+      last = cur;
+    }
+
+  if (me)
+    {
+      libwww_timer = me->libwww_timer;
+      /* remove the element from the list @@@ can be optimized later */
+      HTList_quickRemoveElement(cur, last);
+
+      /* delete explicitely the AmayaTimer */
+      delete wxDynamicCast(me->xt_timer, wxAmayaTimer);
+
+      TtaFreeMemory (me);
+      HTTimer_dispatch (libwww_timer);
+    }
+#endif /* _WX */
 }
 
 /*----------------------------------------------------------------------
   KillAllTimers
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 void KillAllTimers (void)
-#else
-void KillAllTimers ()
-#endif /* __STDC__ */
 {
+#if defined(_MOTIF) || defined(_GTK) || defined(_WX) || defined(_NOGUI)
   /* @@@ maybe add something else to kill the Xt things */
   if (Timers)
     HTList_delete (Timers);
   Timers = NULL;
+#endif /* #if defined(_MOTIF) || defined(_GTK) || defined(_WX) || defined(_NOGUI) */
 }
 
 /*----------------------------------------------------------------------
  AMAYA_SetTimer
  ----------------------------------------------------------------------*/
-#ifdef __STDC__
 void AMAYA_SetTimer (HTTimer *libwww_timer)
-#else
-void AMAYA_SetTimer (libwww_timer)
-HTTimer *libwww_timer;
-#endif /* __STDC__ */
 {
+#if defined(_MOTIF) || defined(_GTK) || defined(_WX) || defined(_NOGUI)
   HTList *cur, *last;
   AmayaTimer *me;
 
-  if (!AmayaIsAlive 
+  if (!AmayaIsAlive () 
       || libwww_timer == NULL
       || libwww_timer->expires == 0)
     return;
@@ -898,37 +957,60 @@ HTTimer *libwww_timer;
     /* remove the old timer */
       if (me->xt_timer) 
 	{
+#ifdef _MOTIF
 	  XtRemoveTimeOut (me->xt_timer);
 	  me->xt_timer = (XtIntervalId) NULL;
+#endif /* _MOTIF */
+
+#ifdef _GTK
+	  gtk_timeout_remove (me->xt_timer);
+	  me->xt_timer = (guint) 0;
+#endif /* !_GTK */
+
+#ifdef _WX
+	  delete wxDynamicCast(me->xt_timer, wxTimer);
+	  me->xt_timer = NULL;
+#endif /* _WX */
 	}
     }
   else
     {
       /* create a new element */
-      me = TtaGetMemory (sizeof (AmayaTimer));
+      me = (AmayaTimer*)TtaGetMemory (sizeof (AmayaTimer));
       /* and add it to the list */
       HTList_addObject(last, (void *) me);
       me->libwww_timer = libwww_timer;
     }
 
   /* add a new time out */
+#ifdef _MOTIF
   me->xt_timer = XtAppAddTimeOut (app_cont,
 				 me->libwww_timer->millis,
 				 (XtTimerCallbackProc) TimerCallback,
 				 (XtPointer *) (void *) me);
+#endif /* _MOTIF */
+  
+#ifdef _GTK
+  me->xt_timer = gtk_timeout_add ((guint32) me->libwww_timer->millis,
+				  (GtkFunction) TimerCallbackGTK,
+				  (gpointer) me);
+#endif /* _GTK */
 
+#ifdef _WX
+  me->xt_timer = new wxAmayaTimer( TimerCallbackWX, me);
+  /* start a one shot timer */
+  me->xt_timer->Start( me->libwww_timer->millis, TRUE );
+#endif /* _WX */
+
+#endif /* #if defined(_MOTIF) || defined(_GTK) || defined(_WX) || defined(_NOGUI) */  
 }
 
 /*----------------------------------------------------------------------
   AMAYA_DeleteTimer
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 void AMAYA_DeleteTimer (HTTimer *libwww_timer)
-#else
-void AMAYA_DeleteTimer (libwww_timer)
-HTTimer *libwww_timer;
-#endif /* __STDC__ */
 {
+#if defined(_MOTIF) || defined(_GTK) || defined(_WX) || defined(_NOGUI)
   HTList *cur, *last;
   AmayaTimer *me;
 
@@ -947,20 +1029,22 @@ HTTimer *libwww_timer;
   if (me)
     {
       /* remove the Xt timer */
+#ifdef _MOTIF
       XtRemoveTimeOut (me->xt_timer);
+#endif /* _MOTIF */
+      
+#ifdef _GTK
+      gtk_timeout_remove (me->xt_timer);
+#endif /* _GTK */
+
+#ifdef _WX
+      delete wxDynamicCast(me->xt_timer, wxTimer);
+#endif /* _WX */
+
       /* and the element from the list */
       HTList_removeObject (Timers, me);
       TtaFreeMemory (me);
     }
+#endif /* #if defined(_MOTIF) || defined(_GTK) || defined(_WX) || defined(_NOGUI) */
 }
-#endif /* _GTK */
-
-#endif /* !_WINDOWS */
-
-#endif /* !AMAYA_JAVA */
-
-
-
-
-
 
