@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT MIT and INRIA, 1996.
+ *  (c) COPYRIGHT INRIA and W3C, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -88,7 +88,7 @@ static int         AHTMemConv_write (HTStream * me, const char *s, int l)
 
    reqcont = (AHTReqContext *) HTRequest_context (me->request);
 
-   tmp = TtaGetMemory (l + 1);
+   tmp = (char *)TtaGetMemory (l + 1);
 
    if (!tmp)
      {
@@ -173,7 +173,7 @@ HTStream           *AHTMemConv_new (HTRequest * request)
 
    reqcont = (AHTReqContext *) HTRequest_context (request);
 
-   if (reqcont->error_stream)
+   if (reqcont && reqcont->error_stream)
      {
 	HT_FREE (reqcont->error_stream);
 	reqcont->error_stream = NULL;
