@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT MIT and INRIA, 1996.
+ *  (c) COPYRIGHT INRIA and W3C, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -12,39 +12,36 @@
  * Author: I. Vatton
  *
  */
-
+ 
 /* Included headerfiles */
 #define THOT_EXPORT extern
 #include "amaya.h"
 #include "css.h"
-#ifdef MATHML
 #include "MathML.h"
-#endif
-#ifdef GRAPHML
-#include "GraphML.h"
+#ifdef _SVG
+#include "SVG.h"
 #endif
 #include "presentation.h"
 
 #include "EDITstyle_f.h"
 #include "HTMLimage_f.h"
 #include "html2thot_f.h"
+#include "HTMLpresentation_f.h"
 #include "styleparser_f.h"
 #include "undo.h"
+#include "XHTMLbuilder_f.h"
+
 
 /*----------------------------------------------------------------------
- MakeASpan
- if element elem is a text string that is not the single child of a
- Span element, create a span element that contains that text string
- and return TRUE; span contains then the created Span element.
+  MakeASpan
+  if element elem is a text string that is not the single child of a
+  Span element, create a span element that contains that text string
+  and return TRUE; span contains then the created Span element.
+  if the parameter presRule is not NULL and a span is generated, the
+  the presRule should be moved to the new span.
  -----------------------------------------------------------------------*/
-#ifdef __STDC__
-ThotBool MakeASpan (Element elem, Element *span, Document doc)
-#else /* __STDC__*/
-ThotBool MakeASpan (elem, span, doc)
-     Element elem;
-     Element *span;
-     Document doc;
-#endif /* __STDC__*/
+ThotBool MakeASpan (Element elem, Element *span, Document doc,
+		    PRule presRule)
 {
   ElementType	elType;
   Element	parent, sibling;
@@ -53,16 +50,23 @@ ThotBool MakeASpan (elem, span, doc)
   ret = FALSE;
   *span = NULL;
   elType = TtaGetElementType (elem);
-  if (ustrcmp(TtaGetSSchemaName (elType.ElSSchema), TEXT("HTML")) == 0)
-    /* it's an HTML element */
-    if (elType.ElTypeNum == HTML_EL_TEXT_UNIT)
-     {
-     parent = TtaGetParent (elem);
-     if (parent != NULL)
+  if (!strcmp(TtaGetSSchemaName (elType.ElSSchema), "HTML") ||
+      !strcmp(TtaGetSSchemaName (elType.ElSSchema), "SVG"))
+    /* it's an HTML or SVG element */
+    if (elType.ElTypeNum == HTML_EL_TEXT_UNIT ||
+	elType.ElTypeNum == HTML_EL_Basic_Elem)
+      /* it's a text leaf */
+      {
+      parent = TtaGetParent (elem);
+      if (parent != NULL)
 	{
 	doit = TRUE;
 	elType = TtaGetElementType (parent);
-	if (elType.ElTypeNum == HTML_EL_Span)
+	if ((elType.ElTypeNum == HTML_EL_Span &&
+	     !strcmp(TtaGetSSchemaName (elType.ElSSchema), "HTML")) ||
+	    (elType.ElTypeNum == SVG_EL_tspan &&
+	     !strcmp(TtaGetSSchemaName (elType.ElSSchema), "SVG")))
+	   /* element parent is a span element */
 	   {
 	   sibling = elem;
 	   TtaNextSibling (&sibling);
@@ -71,26 +75,39 @@ ThotBool MakeASpan (elem, span, doc)
 	      sibling = elem;
 	      TtaPreviousSibling (&sibling);
 	      if (sibling == NULL)
+		/* the text leaf is the only child of its span parent.
+		   No need to create a new span element */
 		{
 	         doit = FALSE;
 		 *span = parent;
+		 if (presRule)
+		   MovePRule (presRule, elem, *span, doc, TRUE);
 		 ret = TRUE;
 		}
 	      }
 	   }
 	if (doit)
+	   /* enclose the text leaf within a span element */
 	   {
-	   elType.ElTypeNum = HTML_EL_Span;
+	   if (!strcmp(TtaGetSSchemaName (elType.ElSSchema), "HTML"))
+	      elType.ElTypeNum = HTML_EL_Span;
+	   else
+	      elType.ElTypeNum = SVG_EL_tspan;
 	   *span = TtaNewElement (doc, elType);
 	   TtaInsertSibling (*span, elem, FALSE, doc);
+	   TtaRegisterElementCreate (*span, doc);
+	   if (presRule)
+	     MovePRule (presRule, elem, *span, doc, TRUE);
+	   TtaRegisterElementDelete (elem, doc);
 	   TtaRemoveTree (elem, doc);
 	   TtaInsertFirstChild (&elem, *span, doc);
+	   TtaRegisterElementCreate (elem, doc);
 	   /* mark the document as modified */
 	   TtaSetDocumentModified (doc);
 	   ret = TRUE;
 	   }
 	}
-     }
+      }
   return ret;
 }
 
@@ -102,15 +119,8 @@ ThotBool MakeASpan (elem, span, doc)
  last child of the former SPAN element.
  Otherwise, firstChild and lastChild are NULL.
  -----------------------------------------------------------------------*/
-#ifdef __STDC__
-void DeleteSpanIfNoAttr (Element el, Document doc, Element *firstChild, Element *lastChild)
-#else /* __STDC__*/
-void DeleteSpanIfNoAttr (el, doc, firstChild, lastChild)
-     Element el;
-     Document doc;
-     Element *firstChild;
-     Element *lastChild;
-#endif /* __STDC__*/
+void DeleteSpanIfNoAttr (Element el, Document doc, Element *firstChild,
+			 Element *lastChild)
 {
   ElementType	elType;
   Element	span, child, next;
@@ -122,13 +132,14 @@ void DeleteSpanIfNoAttr (el, doc, firstChild, lastChild)
   *lastChild = NULL;
   elType = TtaGetElementType (el);
   if (elType.ElTypeNum == HTML_EL_Span &&
-      ustrcmp(TtaGetSSchemaName (elType.ElSSchema), TEXT("HTML")) == 0)
+      strcmp(TtaGetSSchemaName (elType.ElSSchema), "HTML") == 0)
      {
      span = el;
      attr = NULL;
      TtaNextAttribute (span, &attr);
      if (attr == NULL)
         {
+	TtaRegisterElementDelete (span, doc);
 	child = TtaGetFirstChild (span);
 	while (child != NULL)
 	   {
@@ -136,18 +147,14 @@ void DeleteSpanIfNoAttr (el, doc, firstChild, lastChild)
 	   TtaNextSibling (&next);
 	   TtaRemoveTree (child, doc);
 	   TtaInsertSibling (child, span, TRUE, doc);
+	   TtaRegisterElementCreate (child, doc);
 	   if (*firstChild == NULL)
 	      *firstChild = child;
 	   *lastChild = child;
 	   child = next;
 	   }
-	/* The standard Thot command has registered the operation that
- 	   removed the attribute from the SPAN element.  Cancel this operation
-	   from the editing history and register the removal for the TEXT
-	   element instead */
-	if (*firstChild)
-	   TtaChangeLastRegisteredAttr (span, *firstChild, NULL, NULL, doc);
 	TtaDeleteTree (span, doc);
+	TtaSelectElement (doc, *firstChild);
 	}
      }
 }
@@ -157,56 +164,57 @@ void DeleteSpanIfNoAttr (el, doc, firstChild, lastChild)
   If attribute attr is on a text string (elem), create a SPAN element
   enclosing this text string and move the attribute to that SPAN element.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 void  AttrToSpan (Element elem, Attribute attr, Document doc)
-#else /* __STDC__*/
-void  AttrToSpan (elem, attr, doc)
-     Element elem;
-     Attribute attr;
-     Document doc;
-#endif /* __STDC__*/
 {
   Element	span, parent;
   Attribute	newAttr;
   AttributeType	attrType;
   ElementType   elType;
-  int		kind, len;
-#define ATTRLEN 64
-  STRING	oldValue; /* [ATTRLEN]; */
+  int		kind, len, val;
+  char	       *oldValue; /* [ATTRLEN]; */
 
   elType = TtaGetElementType (elem);
-  if (elType.ElTypeNum == HTML_EL_TEXT_UNIT)
+  if (elType.ElTypeNum == HTML_EL_TEXT_UNIT ||
+      elType.ElTypeNum == HTML_EL_Basic_Elem)
     /* it's a character string */
     {
       parent = TtaGetParent (elem);
       elType = TtaGetElementType (parent);
-      if (ustrcmp(TtaGetSSchemaName (elType.ElSSchema), TEXT("HTML")) == 0)
+      if (strcmp(TtaGetSSchemaName (elType.ElSSchema), "HTML") == 0)
         /* the parent element is an HTML element */
-	/* Create a Span element and move to attribute to this Span element */
-        MakeASpan (elem, &span, doc);
+	/* Create a Span element and move the attribute to this Span element */
+        MakeASpan (elem, &span, doc, NULL);
       else
         /* move the attribute to the parent element */
 	span = parent;
       if (span != NULL)
         {
-          oldValue = TtaAllocString (ATTRLEN);
           TtaGiveAttributeType (attr, &attrType, &kind);
 	  newAttr = TtaGetAttribute (span, attrType);
 	  if (newAttr == NULL)
 	     {
-	     newAttr = TtaNewAttribute (attrType);
-	     TtaAttachAttribute (span, newAttr, doc);
+	       newAttr = TtaNewAttribute (attrType);
+	       TtaAttachAttribute (span, newAttr, doc);
 	     }
-          len = ATTRLEN - 1;
-          TtaGiveTextAttributeValue (attr, oldValue, &len);
-	  TtaSetAttributeText (newAttr, oldValue, span, doc);
-	  /* The standard Thot command has registered the operation that
- 	     added the attribute to the TEXT element.  Cancel this operation
-	     from the editing history and register the new attribute on the
-	     span element instead */
-	  TtaChangeLastRegisteredAttr (elem, span, attr, newAttr, doc);
+	  if (kind == 2)
+	    /* it's a text attribute */
+	    {
+	      len = TtaGetTextAttributeLength (attr);
+	      oldValue = (char *)TtaGetMemory (len + 1);
+	      TtaGiveTextAttributeValue (attr, oldValue, &len);
+	      TtaSetAttributeText (newAttr, oldValue, span, doc);
+	      TtaFreeMemory (oldValue);
+	    }
+	  else if (kind == 0 || kind == 1)
+	    /* enumerate or integer attribute */
+	    {
+	      val = TtaGetAttributeValue (attr);
+	      TtaSetAttributeValue (newAttr, val, span, doc);
+	    }
+	  TtaRegisterAttributeCreate (newAttr, span, doc);
+	  TtaRegisterAttributeDelete (attr, elem, doc);
           TtaRemoveAttribute (elem, attr, doc);
-          TtaFreeMemory (oldValue);
+	  TtaSelectElement (doc, elem);
         }
     }
 }
@@ -214,14 +222,11 @@ void  AttrToSpan (elem, attr, doc)
 /*----------------------------------------------------------------------
   GlobalAttrCreated: the user has created a global attribute
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                GlobalAttrCreated (NotifyAttribute * event)
-#else
-void                GlobalAttrCreated (event)
-NotifyAttribute    *event;
- 
-#endif
+void GlobalAttrCreated (NotifyAttribute * event)
 {
+   if (event->info == 1)
+     /* undo will do the job */
+     return;
    /* if the attribute is on a text string, create a SPAN element that encloses
       this text string and move the attribute to that SPAN element */
    AttrToSpan (event->element, event->attribute, event->document);
@@ -230,16 +235,13 @@ NotifyAttribute    *event;
 /*----------------------------------------------------------------------
   GlobalAttrDeleted: the user has removed a global attribute
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                GlobalAttrDeleted (NotifyAttribute * event)
-#else
-void                GlobalAttrDeleted (event)
-NotifyAttribute    *event;
- 
-#endif
+void GlobalAttrDeleted (NotifyAttribute *event)
 {
    Element	firstChild, lastChild;
 
+   if (event->info == 1)
+     /* undo will do the job */
+     return;
    /* if the element is a SPAN without any other attribute, remove the SPAN
       element */
    DeleteSpanIfNoAttr (event->element, event->document, &firstChild, &lastChild);
@@ -249,20 +251,18 @@ NotifyAttribute    *event;
   AttrClassChanged: the user has created removed or modified a Class
   attribute
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                AttrClassChanged (NotifyAttribute * event)
-#else
-void                AttrClassChanged (event)
-NotifyAttribute    *event;
- 
-#endif
+void AttrClassChanged (NotifyAttribute * event)
 {
    Element	firstChild, lastChild;
 
+   if (event->info == 1)
+     /* undo will do the job */
+     return;
    if (event->event == TteAttrDelete)
       /* if the element is a SPAN without any other attribute, remove the SPAN
          element */
-      DeleteSpanIfNoAttr (event->element, event->document, &firstChild, &lastChild);
+      DeleteSpanIfNoAttr (event->element, event->document, &firstChild,
+			  &lastChild);
    else if (event->event == TteAttrCreate)
       /* if the Class attribute is on a text string, create a SPAN element that
          encloses this text string and move the Class attribute to that SPAN
@@ -275,18 +275,10 @@ NotifyAttribute    *event;
   MovePRule
   remove presentation rule presRule from element fromEl and associate a copy
   of that rule with element toEl, for the main view.
-  showBoxAllowed is TRUE if the ShowBox rule could be generated
+  showBoxAllowed is TRUE if the ShowBox rule could be generated.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void MovePRule (PRule presRule, Element fromEl, Element toEl, Document doc, ThotBool showBoxAllowed)
-#else  /* __STDC__ */
-static void MovePRule (presRule, fromEl, toEl, doc, showBoxAllowed)
-PRule       presRule;
-Element     fromEl;
-Element     toEl;
-Document    doc;
-ThotBool    showBoxAllowed;
-#endif /* __STDC__ */
+void MovePRule (PRule presRule, Element fromEl, Element toEl, Document doc,
+		ThotBool showBoxAllowed)
 {
    int         presRuleType;
    PRule       newPRule, oldPRule;
@@ -307,7 +299,8 @@ ThotBool    showBoxAllowed;
 	}
    /* this PRule applies to view 1 (main view) */
    TtaSetPRuleView (newPRule, 1);
-   TtaRemovePRule (fromEl, presRule, doc);
+   if (fromEl)
+     TtaRemovePRule (fromEl, presRule, doc);
    TtaAttachPRule (toEl, newPRule, doc);
    if (addShow)
      {
@@ -325,52 +318,43 @@ ThotBool    showBoxAllowed;
 /*----------------------------------------------------------------------
   SetStyleAttribute.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                SetStyleAttribute (Document doc, Element elem)
-#else  /* __STDC__ */
-void                SetStyleAttribute (doc, elem)
-Document            doc;
-Element             elem;
-#endif /* __STDC__ */
+void SetStyleAttribute (Document doc, Element elem)
 {
    AttributeType       attrType;
    Attribute           styleAttr;
-#ifdef MATHML
    ElementType         elType;
-#endif
    Element	       firstChild, lastChild;
-#define STYLELEN 1000
-   STRING              style;
+   char               *schName;
+   char               *style;
    int                 len;
+#define STYLELEN 1000
 
    /* does the element have a Style_ attribute ? */
-#ifdef MATHML
    elType = TtaGetElementType (elem);
-   if (ustrcmp (TtaGetSSchemaName (elType.ElSSchema), TEXT("MathML")) == 0)
-      {
-	attrType.AttrSSchema = elType.ElSSchema;
-	attrType.AttrTypeNum = MathML_ATTR_style_;
-      }
+   schName = TtaGetSSchemaName (elType.ElSSchema);
+   if (strcmp (schName, "MathML") == 0)
+     {
+       attrType.AttrSSchema = elType.ElSSchema;
+       attrType.AttrTypeNum = MathML_ATTR_style_;
+     }
    else
-#endif
-#ifdef GRAPHML
-   elType = TtaGetElementType (elem);
-   if (ustrcmp (TtaGetSSchemaName (elType.ElSSchema), "GraphML") == 0)
-      {
-	attrType.AttrSSchema = elType.ElSSchema;
-	attrType.AttrTypeNum = GraphML_ATTR_style_;
-      }
+#ifdef _SVG
+   if (strcmp (schName, "SVG") == 0)
+     {
+       attrType.AttrSSchema = elType.ElSSchema;
+       attrType.AttrTypeNum = SVG_ATTR_style_;
+     }
    else
-#endif
+#endif /* _SVG */
       {
-	attrType.AttrSSchema = TtaGetSSchema (TEXT("HTML"), doc);
+	attrType.AttrSSchema = TtaGetSSchema ("HTML", doc);
 	attrType.AttrTypeNum = HTML_ATTR_Style_;
       }
    styleAttr = TtaGetAttribute (elem, attrType);
 
    /* keep the new style string */
    len = STYLELEN;
-   style = TtaAllocString (STYLELEN);
+   style = (char *)TtaGetMemory (STYLELEN);
    GetHTMLStyleString (elem, doc, style, &len);
    if (len == 0)
      {
@@ -402,29 +386,62 @@ Element             elem;
    TtaFreeMemory (style);
 }
 
+/*----------------------------------------------------------------------
+  GetParentDiv get the enclosing div if it's only include this element
+  or generates an enclosing div.
+  ----------------------------------------------------------------------*/
+static Element GetParentDiv (Element el, SSchema HTMLschema, Document doc)
+{
+  Element	 div, prev, next;
+  ElementType    elType;
+
+  /* generate a division around this pseudo paragraph */
+  div = TtaGetParent (el);
+  if (div)
+    {
+      elType = TtaGetElementType (div);
+      prev = next = el;
+      if (elType.ElSSchema == HTMLschema &&
+	  elType.ElTypeNum == HTML_EL_Division)
+	{
+	  TtaPreviousSibling (&prev);
+	  TtaNextSibling (&next);
+	}
+      if (prev || next)
+	/* this div includes more than the pseudo paragraph */
+	div = NULL;
+    }
+  if (div == NULL)
+    {
+      elType.ElSSchema = HTMLschema;
+      elType.ElTypeNum = HTML_EL_Division;
+      div = TtaNewElement (doc, elType);
+      TtaInsertSibling (div, el, TRUE, doc);
+      TtaRegisterElementCreate (div, doc);
+      TtaRegisterElementDelete (el, doc);
+      TtaRemoveTree (el, doc);
+      TtaInsertFirstChild  (&el, div, doc);
+      TtaRegisterElementCreate (el, doc);
+    }
+  return div;
+}
 
 /*----------------------------------------------------------------------
   ChangePRule
   A specific PRule will be created or modified by the user for
   a given element. (pre-event)
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-ThotBool            ChangePRule (NotifyPresentation * event)
-#else  /* __STDC__ */
-ThotBool            ChangePRule (event)
-NotifyPresentation *event;
-
-#endif /* __STDC__ */
+ThotBool ChangePRule (NotifyPresentation *event)
 {
   ElementType	     elType, parentType;
-  Element	     el, span, body, root, parent;
+  Element	     el, span, body, parent;
   PRule	             presRule;
   Document	     doc;
   SSchema	     HTMLschema;
   AttributeType      attrType;
   Attribute          attr;
 #define STYLELEN 1000
-  CHAR_T               buffer[15];
+  char               buffer[15];
   int                presType;
   int                w, h, unit, value, i;
   ThotBool           ret;
@@ -435,12 +452,20 @@ NotifyPresentation *event;
   presRule = event->pRule;
   elType = TtaGetElementType (el);
   ret = FALSE;
-  HTMLschema = TtaGetSSchema (TEXT("HTML"), doc);
+  HTMLschema = TtaGetSSchema ("HTML", doc);
 
-  /* if it's a background rule on element BODY, move it to element HTML */
-  /* if it's a rule on element HTML and it's not a background rule, move
-     it to element BODY */
   if (event->event != TtePRuleDelete)
+    {
+     /* if a style property is being changed, we have its new value in the
+	PRule to set the corresponding style attribute, but if its a
+	change in the geometry, we have the old value to see the
+	difference */
+      if (presType == PRHeight || presType == PRWidth ||
+	  presType == PRVertPos || presType == PRHorizPos)
+	/* Store the new value into the presentation rule before it is
+	   potentially copied by MovePRule */
+	TtaSetPRuleValue (el, presRule, event->value, doc);
+
     if (elType.ElSSchema != HTMLschema)
       /* it's not an HTML element */
       {
@@ -454,27 +479,21 @@ NotifyPresentation *event;
       }
     else
       /* it's an HTML element */
+      /* if it's a rule on the root or HTML element, move it to BODY element:
+	 the HTML DTD does not allow a style attribute on the HTML element */
       {
-	if (elType.ElTypeNum == HTML_EL_BODY
-	    && (presType == PRFillPattern || presType == PRBackground ||
-		presType == PRShowBox))
-	  {
-	    root = TtaGetParent (el);
-	    if (presType == PRBackground)
-	      MovePRule (presRule, el, root, doc, TRUE);
-	    ret = TRUE; /* don't let Thot perform normal operation */      
-	  }
-	else if (elType.ElTypeNum == HTML_EL_HTML)
+	if (elType.ElTypeNum == HTML_EL_HTML ||
+	    elType.ElTypeNum == HTML_EL_Document)
 	  {
 	    elType.ElTypeNum = HTML_EL_BODY;
 	    body = TtaSearchTypedElement (elType, SearchInTree, el);
-	    if (presType != PRFillPattern && presType != PRBackground
-		&& presType != PRShowBox)
+	    if (body)
 	      {
 		MovePRule (presRule, el, body, doc, TRUE);
 		ret = TRUE; /* don't let Thot perform normal operation */
+		/* the style attribute must be created on the BODY element */
+		el = body;
 	      }      
-	    el = body;
 	  }
 	else
 	  {
@@ -490,9 +509,9 @@ NotifyPresentation *event;
 
 	    if (parent != NULL)
 	      {
-		TtaSetStatus (doc, 1, TtaGetMessage (AMAYA, AM_INVALID_TARGET), NULL);
+		TtaSetStatus (doc, 1, TtaGetMessage (AMAYA, AM_INVALID_TARGET),
+			      NULL);
 		return (TRUE); /* don't let Thot perform normal operation */
-;
 	      }
 	    else
 	      {
@@ -534,9 +553,6 @@ NotifyPresentation *event;
 		 elType.ElTypeNum == HTML_EL_Data_cell ||
 		 elType.ElTypeNum == HTML_EL_Heading_cell ||
 		 elType.ElTypeNum == HTML_EL_Object ||
-#ifdef GRAPHML
-		 elType.ElTypeNum == HTML_EL_XMLGraphics ||
-#endif /* GRAPHML */
 		 elType.ElTypeNum == HTML_EL_Applet))
 	      {
 		/* store information in Width or Height attribute */
@@ -547,19 +563,18 @@ NotifyPresentation *event;
 		    w = -1;
 		    h = -1;
 		  }
-		TtaSwitchSelection (doc, 1, FALSE);
 		attrType.AttrSSchema = HTMLschema;
 		unit = TtaGetPRuleUnit (presRule);
 
 		if (presType == PRWidth)
 		  {
 		    /* the new value is the old one plus the delta */
-		    TtaGiveBoxSize (el, doc, 1, unit, &value, &i);
-		    value += TtaGetPRuleValue (presRule);
+		    TtaGiveBoxSize (el, doc, 1, (TypeUnit)unit, &value, &i);
+		    value = event->value;
 		    if (unit == UnPercent)
-		      usprintf (buffer, TEXT("%d%%"), value);
+		      sprintf (buffer, "%d%%", value);
 		    else
-		      usprintf (buffer, TEXT("%d"), value);
+		      sprintf (buffer, "%d", value);
 
 		    attrType.AttrTypeNum = HTML_ATTR_Width__;
 		    attr = TtaGetAttribute (el, attrType);
@@ -575,21 +590,24 @@ NotifyPresentation *event;
 		        TtaRegisterAttributeReplace (attr, el, doc);
 		        TtaSetAttributeText (attr, buffer, el, doc);
 		      }
-		    TtaSetAttributeText (attr, buffer, el, doc);
 		    CreateAttrWidthPercentPxl (buffer, el, doc, w);
 		  }
 		else
 		  {
 		    /* the new value is the old one plus the delta */
-		    TtaGiveBoxSize (el, doc, 1, unit, &i, &value);
-		    value += TtaGetPRuleValue (presRule);
+		    TtaGiveBoxSize (el, doc, 1, (TypeUnit)unit, &i, &value);
+		    value = event->value;
+		    if (unit == UnPercent)
+		      sprintf (buffer, "%d%%", value);
+		    else
+		      sprintf (buffer, "%d", value);
 		    attrType.AttrTypeNum = HTML_ATTR_Height_;
 		    attr = TtaGetAttribute (el, attrType);
 		    if (attr == NULL)
 		      {
 			attr = TtaNewAttribute (attrType);
 			TtaAttachAttribute (el, attr, doc);
-		        TtaSetAttributeValue (attr, value, el, doc);
+			TtaSetAttributeText (attr, buffer, el, doc);
 			TtaRegisterAttributeCreate (attr, el, doc);
 		      }
 		    else
@@ -597,231 +615,146 @@ NotifyPresentation *event;
 			TtaRegisterAttributeReplace (attr, el, doc);
 		        TtaSetAttributeValue (attr, value, el, doc);
 		      }
-		    /* update the associated map */
-		    if (elType.ElTypeNum == HTML_EL_PICTURE_UNIT)
-		      UpdateImageMap (el, doc, -1, h);
+		    CreateAttrHeightPercentPxl (buffer, el, doc, h);
 		  }
-		TtaSwitchSelection (doc, 1, TRUE);
-		return (TRUE);
+		TtaSetDocumentModified (doc);
+		return TRUE; /* don't let Thot perform normal operation */
 	      }
-	    else if (IsCharacterLevelElement (el)
-		&& (presType == PRIndent || presType == PRLineSpacing
-		    || presType == PRAdjust || presType == PRJustify
-		    || presType == PRHyphenate))
-	      /* if the rule is a Format rule applied to a character-level element,
-		 move it to the first enclosing non character-level element */
+	    else if (IsCharacterLevelElement (el) &&
+		     (presType == PRIndent || presType == PRLineSpacing
+		      || presType == PRAdjust || presType == PRHyphenate))
+	      /* if the rule is a Format rule applied to a character-level
+		 element, move it to the first enclosing non character-level
+		 element */
 	      {
 		do
 		  el = TtaGetParent (el);
-		while (el != NULL && IsCharacterLevelElement (el));
-		/* if the PRule is on a Pseudo-Paragraph, move it to the enclosing
-		   element */
-		if (elType.ElTypeNum == HTML_EL_Pseudo_paragraph)
-		  el = TtaGetParent (el);
+		while (el && IsCharacterLevelElement (el));
+		/* if the PRule is on a Pseudo-Paragraph, move it to the
+		   enclosing element */
+		elType = TtaGetElementType (el);
+		if (elType.ElSSchema == HTMLschema &&
+		    elType.ElTypeNum == HTML_EL_Pseudo_paragraph)
+		  el = GetParentDiv (el, HTMLschema, doc);
 		MovePRule (presRule, event->element, el, doc, TRUE);
 		ret = TRUE; /* don't let Thot perform normal operation */
 	      }	  
 	    else if (elType.ElTypeNum == HTML_EL_Pseudo_paragraph)
-	      /* if the PRule is on a Pseudo-Paragraph, move it to the enclosing
-		 element */
+	      /* if the PRule is on a Pseudo-Paragraph, move it to the
+		 enclosing element */
 	      {
-		el = TtaGetParent (el);
+		el = GetParentDiv (el, HTMLschema, doc);
 		MovePRule (presRule, event->element, el, doc, TRUE);
 		ret = TRUE; /* don't let Thot perform normal operation */
 	      }
-	    else if (MakeASpan (el, &span, doc))
-	      /* if it is a new PRule on a text string, create a SPAN element that
-		 encloses this text string and move the PRule to that SPAN element */
+	    else if (MakeASpan (el, &span, doc, presRule))
+	      /* if it is a new PRule on a text string, create a SPAN element
+		 that encloses this text string and move the PRule to that
+		 SPAN element */
 	      {
-		MovePRule (presRule, el, span, doc, TRUE);
 		el = span;
 		ret = TRUE; /* don't let Thot perform normal operation */
 	      }
 	  }
       }
+    }
   /* set the Style_ attribute ? */
   SetStyleAttribute (doc, el);
   TtaSetDocumentModified (doc);
   return (ret);
 }
 
-
 /*----------------------------------------------------------------------
   PRuleDeleted
   A specific PRule has been deleted by the user for a given element
   (post-event)
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                PRuleDeleted (NotifyPresentation * event)
-#else  /* __STDC__ */
-void                PRuleDeleted (event)
-NotifyPresentation *event;
-#endif /* __STDC__ */
+void PRuleDeleted (NotifyPresentation * event)
 {
   /* set the Style_ attribute ? */
   SetStyleAttribute (event->document, event->element);
 }
 
-
 /*----------------------------------------------------------------------
  AttrLangDeleted
  A Lang attribute has been deleted
  -----------------------------------------------------------------------*/
-#ifdef __STDC__
 void AttrLangDeleted (NotifyAttribute *event)
-#else /* __STDC__*/
-void AttrLangDeleted(event)
-     NotifyAttribute *event;
-#endif /* __STDC__*/
 {
-   Element	 firstChild, lastChild;
-   AttributeType attrType;
-   Attribute	 attr;
+  Element	 elem, firstChild, lastChild;
+  ElementType    elType;
+  AttributeType  attrType;
+  Attribute	 attr;
 
+  elem = event->element;
+  elType = TtaGetElementType (elem);
   /* if the element is a SPAN without any other attribute, remove the SPAN
      element */
-  DeleteSpanIfNoAttr (event->element, event->document, &firstChild, &lastChild);
-  /* if it's the root (HTML) element, delete the RealLang attribute */
-  if (!TtaGetParent (event->element))
-     /* it's the root element */
-     {
-     attrType.AttrSSchema = event->attributeType.AttrSSchema;
-     attrType.AttrTypeNum = HTML_ATTR_RealLang;
-     attr = TtaGetAttribute (event->element, attrType);
-     if (attr)
-	TtaRemoveAttribute (event->element, attr, event->document);
+  if (event->info == 0 &&
+      strcmp(TtaGetSSchemaName (elType.ElSSchema), "HTML") == 0)
+    DeleteSpanIfNoAttr (elem, event->document, &firstChild,&lastChild);
+  /* if it's the root (HTML, SVG, MathML) element, delete the RealLang
+     attribute */
+  if (elem == TtaGetRootElement (event->document))
+    /* it's the root element */
+    {
+      elType = TtaGetElementType (elem);
+      if (strcmp(TtaGetSSchemaName (elType.ElSSchema), "HTML") == 0)
+	attrType.AttrTypeNum = HTML_ATTR_RealLang;
+      else if (strcmp(TtaGetSSchemaName (elType.ElSSchema), "SVG") == 0)
+	attrType.AttrTypeNum = SVG_ATTR_RealLang;
+      else if (strcmp(TtaGetSSchemaName (elType.ElSSchema), "MathML") == 0)
+	attrType.AttrTypeNum = MathML_ATTR_RealLang;
+      attrType.AttrSSchema = event->attributeType.AttrSSchema;
+      attr = TtaGetAttribute (elem, attrType);
+      if (attr)
+	TtaRemoveAttribute (elem, attr, event->document);
      }
 }
 
-
-/*----------------------------------------------------------------------
- MoveAttrLang
- 
- -----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void MoveAttrLang (Attribute oldAttr, Element *el, Document doc)
-#else /* __STDC__*/
-static void MoveAttrLang (oldAttr, el, doc)
-     Attribute oldAttr;
-     Element *el;
-     Document doc;
-#endif /* __STDC__*/
-{
-  Element	first, parent, sibling, next, firstChild, lastChild;
-  Attribute	newAttr, attr;
-  AttributeType	attrType;
-  int		kind, len;
-  STRING	value    = TtaAllocString (ATTRLEN); 
-  STRING oldValue = TtaAllocString (ATTRLEN);
-  ThotBool	sameLang;
-
-  /* if all siblings have the same LANG attribute, move that attibute to
-     the parent element, unless the parent element has exception Hidden */
-  parent = TtaGetParent (*el);
-  if (parent)
-     if (HiddenType (parent))
-        parent = NULL;
-  if (parent != NULL)
-     {
-     TtaGiveAttributeType (oldAttr, &attrType, &kind);
-     len = ATTRLEN - 1;
-     TtaGiveTextAttributeValue (oldAttr, oldValue, &len);
-     first = TtaGetFirstChild (parent);
-     sameLang = TRUE;
-     sibling = first;
-     while (sibling != NULL && sameLang)
-	{
-	if (sibling != *el)
-	   {
-	   attr = TtaGetAttribute (sibling, attrType);
-	   if (attr == NULL)
-	      sameLang = FALSE;
-	   else
-	      {
-	      len = ATTRLEN - 1;
-	      TtaGiveTextAttributeValue (attr, value, &len);
-	      if (ustrcasecmp(oldValue, value) != 0)
-		 sameLang = FALSE;
-	      }
-	   }
-	if (sameLang)
-	   TtaNextSibling (&sibling);
-	}
-     if (sameLang)
-        /* all sibling have the same LANG attribute */
-	{
-	/* delete the LANG attribute for all siblings */
-	sibling = first;
-	while (sibling != NULL)
-	   {
-	   attr = TtaGetAttribute (sibling, attrType);
-	   TtaRemoveAttribute (sibling, attr, doc);
-	   next = sibling;
-	   TtaNextSibling (&next);
-	   DeleteSpanIfNoAttr (sibling, doc, &firstChild, &lastChild);
-	   sibling = next;
-	   }
-	/* associate a LANG attribute with the parent element */
-	newAttr = TtaGetAttribute (parent, attrType);
-	if (newAttr == NULL)
-	  {
-	    newAttr = TtaNewAttribute (attrType);
-	    TtaAttachAttribute (parent, newAttr, doc);
-	  }
-	TtaSetAttributeText (newAttr, oldValue, parent, doc);
-	*el = parent;
-	/* do it again for the parent element */
-        MoveAttrLang (newAttr, el, doc);
-	}
-     }
-  TtaFreeMemory (value);
-  TtaFreeMemory (oldValue);
-}
-
-
 /*----------------------------------------------------------------------
  AttrLangCreated
  A Lang attribute has been created
  -----------------------------------------------------------------------*/
-#ifdef __STDC__
 void AttrLangCreated (NotifyAttribute *event)
-#else /* __STDC__*/
-void AttrLangCreated(event)
-     NotifyAttribute *event;
-#endif /* __STDC__*/
 {
   Element	elem;
+  ElementType   elType;
   int		len;
   AttributeType attrType;
   Attribute	attr;
-  STRING	value    = TtaAllocString (ATTRLEN); 
+#define ATTRLEN 64
+  char	       *value = (char *)TtaGetMemory (ATTRLEN); 
 
-  /* move the LANG attribute to the parent element if all sibling have the
-     same attribute with the same value */
   elem = event->element;
-  MoveAttrLang (event->attribute, &elem, event->document);
-
+  elType = TtaGetElementType (elem);
   len = ATTRLEN - 1;
   TtaGiveTextAttributeValue (event->attribute, value, &len);
-  if (ustrcasecmp(value, TEXT("Symbol")) == 0)
-     /* it's a character string in the Symbol character set, it's not really
-	a language */
-    {
+  if (strcasecmp(value, "Symbol") == 0)
+    /* it's a character string in the Symbol character set, it's not really
+       a language */
     TtaRemoveAttribute (elem, event->attribute, event->document);      
-    }
   else
+    /* if the LANG attribute is on a text string in a HTML document, create
+       a SPAN element that encloses this text string and move the LANG
+       attribute to that SPAN element */
     {
-      /* if the LANG attribute is on a text string, create a SPAN element that
-	 encloses this text string and move the LANG attribute to that SPAN
-	 element */
-      AttrToSpan (elem, event->attribute, event->document);
+      if (event->info == 0 &&
+	  strcmp (TtaGetSSchemaName (elType.ElSSchema), "HTML") == 0)
+	AttrToSpan (elem, event->attribute, event->document);
     }
-  /* if it's the root (HTML) element, create a RealLang attribute too */
-  if (!TtaGetParent (elem))
+  /* if it's the root (HTML, SVG, MathML) element, create a RealLang
+     attribute too */
+  if (elem == TtaGetRootElement (event->document))
      /* it's the root element */
      {
+      if (strcmp(TtaGetSSchemaName (elType.ElSSchema), "HTML") == 0)
+	attrType.AttrTypeNum = HTML_ATTR_RealLang;
+      else if (strcmp(TtaGetSSchemaName (elType.ElSSchema), "SVG") == 0)
+	attrType.AttrTypeNum = SVG_ATTR_RealLang;
+      else if (strcmp(TtaGetSSchemaName (elType.ElSSchema), "MathML") == 0)
+	attrType.AttrTypeNum = MathML_ATTR_RealLang;
      attrType.AttrSSchema = event->attributeType.AttrSSchema;
-     attrType.AttrTypeNum = HTML_ATTR_RealLang;
      attr = TtaNewAttribute (attrType);
      TtaAttachAttribute (elem, attr, event->document);
      TtaSetAttributeValue (attr, HTML_ATTR_RealLang_VAL_Yes_, elem,
@@ -834,31 +767,34 @@ void AttrLangCreated(event)
  AttrLangModified
  A Lang attribute has been modified
  -----------------------------------------------------------------------*/
-#ifdef __STDC__
 void AttrLangModified (NotifyAttribute *event)
-#else /* __STDC__*/
-void AttrLangModified(event)
-     NotifyAttribute *event;
-#endif /* __STDC__*/
 {
   Element	elem;
+  ElementType   elType;
   AttributeType attrType;
   Attribute	attr;
 
   elem = event->element;
-  /* if it's the root (HTML) element, create a RealLang attribute */
-  if (!TtaGetParent (elem))
-     /* it's the root element */
-     {
-     attrType.AttrSSchema = event->attributeType.AttrSSchema;
-     attrType.AttrTypeNum = HTML_ATTR_RealLang;
-     attr = TtaGetAttribute (elem, attrType);
-     if (!attr)
+  /* if it's the root (HTML, SVG, MathML) element, create a RealLang
+     attribute */
+  if (elem == TtaGetRootElement (event->document))
+    /* it's the root element */
+    {
+      elType = TtaGetElementType (elem);
+      if (strcmp(TtaGetSSchemaName (elType.ElSSchema), "HTML") == 0)
+	attrType.AttrTypeNum = HTML_ATTR_RealLang;
+      else if (strcmp(TtaGetSSchemaName (elType.ElSSchema), "SVG") == 0)
+	attrType.AttrTypeNum = SVG_ATTR_RealLang;
+      else if (strcmp(TtaGetSSchemaName (elType.ElSSchema), "MathML") == 0)
+	attrType.AttrTypeNum = MathML_ATTR_RealLang;
+      attrType.AttrSSchema = event->attributeType.AttrSSchema;
+      attr = TtaGetAttribute (elem, attrType);
+      if (!attr)
 	{
-        attr = TtaNewAttribute (attrType);
-        TtaAttachAttribute (elem, attr, event->document);
-        TtaSetAttributeValue (attr, HTML_ATTR_RealLang_VAL_Yes_, elem,
-			      event->document);
+	  attr = TtaNewAttribute (attrType);
+	  TtaAttachAttribute (elem, attr, event->document);
+	  TtaSetAttributeValue (attr, HTML_ATTR_RealLang_VAL_Yes_, elem,
+				event->document);
 	}
      }  
 }
@@ -867,27 +803,29 @@ void AttrLangModified(event)
  AttrLangShouldBeDeleted
  The user wants to remove a Lang attribute
  -----------------------------------------------------------------------*/
-#ifdef __STDC__
 ThotBool AttrLangShouldBeDeleted (NotifyAttribute *event)
-#else /* __STDC__*/
-ThotBool AttrLangShouldBeDeleted(event)
-     NotifyAttribute *event;
-#endif /* __STDC__*/
 {
   Element	elem;
+  ElementType   elType;
   AttributeType attrType;
   Attribute	attr;
 
   elem = event->element;
   /* if it's the root (HTML) element, delete the RealLang attribute */
-  if (!TtaGetParent (elem))
-     /* it's the root element */
-     {
-     attrType.AttrSSchema = event->attributeType.AttrSSchema;
-     attrType.AttrTypeNum = HTML_ATTR_RealLang;
-     attr = TtaGetAttribute (elem, attrType);
-     if (attr)
+  if (elem == TtaGetRootElement (event->document))
+    /* it's the root element */
+    {
+      elType = TtaGetElementType (elem);
+      if (strcmp(TtaGetSSchemaName (elType.ElSSchema), "HTML") == 0)
+	attrType.AttrTypeNum = HTML_ATTR_RealLang;
+      else if (strcmp(TtaGetSSchemaName (elType.ElSSchema), "SVG") == 0)
+	attrType.AttrTypeNum = SVG_ATTR_RealLang;
+      else if (strcmp(TtaGetSSchemaName (elType.ElSSchema), "MathML") == 0)
+	attrType.AttrTypeNum = MathML_ATTR_RealLang;
+      attrType.AttrSSchema = event->attributeType.AttrSSchema;
+      attr = TtaGetAttribute (elem, attrType);
+      if (attr)
 	TtaRemoveAttribute (elem, attr, event->document);
-     }  
+    }  
   return FALSE; /* let Thot perform normal operation */
 }
