@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT MIT and INRIA, 2000
+ *  (c) COPYRIGHT INRIA and W3C, 2000-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -22,15 +22,17 @@
 
 #undef DEBUG_XPOINTER
 
-#ifdef _WINDOWS
+#ifdef _WINGUI
 #define snprintf _snprintf
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
 
 #define THOT_EXPORT extern
 #include "amaya.h"
 #include "MathML.h"
-#include "GraphML.h"
+#include "SVG.h"
 #include "XLink.h"
+#include "XML.h"
+#include "init_f.h"
 #ifdef ANNOTATIONS
 #include "Annot.h"
 #endif /* ANNOTATIONS */
@@ -77,7 +79,6 @@ static Element RootElement; /* the root of the tree we're parsing (so that
 
 /*----------------------------------------------------------------------
   StrACat
-
   A not very efficient function that makes a dynamic memory allocation
   strcat
   ----------------------------------------------------------------------*/
@@ -93,7 +94,7 @@ static void StrACat (char ** dest, const char * src)
 	status = TtaRealloc (*dest, length + strlen(src) + 1);
         if (status != NULL)
 	  {
-            *dest = status;
+            *dest = (char*)status;
 	    strcpy (*dest + length, src);
           }
       } 
@@ -148,7 +149,7 @@ static int CountInlineChars (Element *mark, int firstCh, selMode *mode)
 	 text sibling elements, adjust the selection mode flag */
       if (firstCh == 0)
 	{
-	  *mode |=  SEL_STRING_RANGE;
+	  *mode = (selMode)((int)*mode | SEL_STRING_RANGE);
 	  /* START or END point? */
 	  el = *mark;
 	  TtaNextSibling (&el);
@@ -156,12 +157,12 @@ static int CountInlineChars (Element *mark, int firstCh, selMode *mode)
 	    {
 	      elType = TtaGetElementType (el);
 	      if (elType.ElTypeNum == THOT_TEXT_UNIT)
-		*mode |= SEL_START_POINT;
+		*mode = (selMode)((int)*mode | SEL_START_POINT);
 	      else
-		*mode |= SEL_END_POINT;
+		*mode = (selMode)((int)*mode | SEL_END_POINT);
 	    }
 	  else
-	    *mode |= SEL_END_POINT;
+	    *mode = (selMode)((int)*mode | SEL_END_POINT);
 	}
 
       /* now point to the parent */
@@ -181,11 +182,22 @@ static int CountInlineChars (Element *mark, int firstCh, selMode *mode)
 static ThotBool ElIsHidden (Element el)
 {
   ElementType elType;
+  ThotBool result;
 
   if (!el)
     return FALSE;
+
   elType = TtaGetElementType (el);
-  return (TtaHasHiddenException (elType));
+
+  result = TtaHasHiddenException (elType);
+
+  /* ignore all elements that were added by Amaya (they don't have
+     a line number */
+  if (!result)
+    result = (TtaGetElementLineNumber (el) == 0);
+
+
+  return result;
 }
 
 /*----------------------------------------------------------------------
@@ -283,17 +295,25 @@ static void AdjustSelMode (Element *el, int *start, int index, selMode *mode, se
       *el = tmp_el;
     }
 	  
-  len = TtaGetTextLength (*el);
   if (*start > 0)
     {
-      *mode |= SEL_STRING_RANGE;
+      len = TtaGetTextLength (*el);
+      *mode = (selMode)((int)*mode | SEL_STRING_RANGE);
       if (*start > len)
 	{
-	  *mode |= SEL_END_POINT;
+	  *mode = (selMode)((int)*mode | SEL_END_POINT);
 	  *start = len;
 	}
       else if (*start > index)
-	*mode |= SEL_START_POINT;
+	{
+	  if (type == SEL_FIRST_EL)
+	    *mode = (selMode)((int)*mode | SEL_START_POINT);
+	  else
+	    {
+	      *mode = (selMode)((int)*mode | SEL_END_POINT);
+	      *start = *start - 1;
+	    }
+	}
     }
 }
 
@@ -322,17 +342,19 @@ static char * GetIdValue (Element el)
     return NULL;
   else if (!strcmp (schema_name, "MathML"))
     attrType.AttrTypeNum = MathML_ATTR_id;
-  else if (!strcmp (schema_name, "GraphML"))
-    attrType.AttrTypeNum = GraphML_ATTR_id;
-  else
+  else if (!strcmp (schema_name, "SVG"))
+    attrType.AttrTypeNum = SVG_ATTR_id;
+  else if (!strcmp (schema_name, "HTML") || !strcmp (schema_name, "XHTML"))
     attrType.AttrTypeNum = HTML_ATTR_ID;
+  else /* assume it is generic XML */
+    attrType.AttrTypeNum = XML_ATTR_id;
 
   attr = TtaGetAttribute (el, attrType);
   if (attr != NULL)
     {
       /* there's an ID attribute */
       len = TtaGetTextAttributeLength (attr) + 1;
-      value = TtaGetMemory (len);
+      value = (char *)TtaGetMemory (len);
       TtaGiveTextAttributeValue (attr, value, &len);
       
     }
@@ -354,7 +376,7 @@ static ThotBool TestIdValue (Element el, char *val)
 
   id_value = GetIdValue (el);
 
-  if (id_value)
+  if (id_value && val)
     {
       if (!strcmp (id_value, val))
 	result = TRUE;
@@ -445,6 +467,31 @@ static Element AGetParent (Element el)
   return parent;
 }
 
+/*----------------------------------------------------------------------
+  AGetLastChild
+
+  returns the last child in a tree
+  ----------------------------------------------------------------------*/
+static Element AGetLastChild (Element el)
+{
+  Element child;
+
+  if (!el)
+    return NULL;
+  
+  child = el;
+
+  while (child)
+    {
+      child = TtaGetLastChild (child);
+      if (child && !ElIsHidden (child))
+	break;
+    }
+
+  return child;
+}
+
+
 /*----------------------------------------------------------------------
   PreviousSibling
 
@@ -475,7 +522,7 @@ static void PreviousSibling (Element *el)
       /* if the element is hidden, return the latest child */
       if (ElIsHidden (sibling))
 	{
-	  tmp_el = TtaGetLastChild (sibling);
+	  tmp_el = AGetLastChild (sibling);
 	  if (!tmp_el)
 	    {
 	      /* there was no child, let's try the next brother */
@@ -638,11 +685,13 @@ ThotBool SearchTextPosition (Element *mark, int *firstCh)
   the caller to free the returned string.
   Returns NULL in case of error.
   ----------------------------------------------------------------------*/
-static char * XPathList2Str (XPathList *xpath_list, int firstCh, int len, int mode, ThotBool firstF)
+static char * XPathList2Str (XPathList *xpath_list, int firstCh, int len,
+			     int mode, ThotBool firstF)
 {
   XPathItem *xpath_item, *xpath_tmp;
-  char buffer[500];
-  char *xpath_expr = NULL;
+  char      *buffer = NULL, *name;
+  char      *xpath_expr = NULL;
+  int        l;
 
   xpath_item = *xpath_list;
   if  (mode & SEL_STRING_RANGE)
@@ -666,14 +715,23 @@ static char * XPathList2Str (XPathList *xpath_list, int firstCh, int len, int mo
       if (xpath_item->elType.ElTypeNum != THOT_TEXT_UNIT)
 	{
 	  if (xpath_item->id_value)
-	    snprintf (buffer, sizeof (buffer),
-		      "id(\"%s\")", xpath_item->id_value);
+	    {
+	      l = strlen (xpath_item->id_value) + 8;
+	      buffer = (char *)TtaGetMemory (l);
+	      strcpy (buffer, "id(\""); 
+	      strcat (buffer, xpath_item->id_value);
+	      strcat (buffer, "\")");
+	    }
 	  else
-	    snprintf (buffer, sizeof (buffer),
-		      "/%s[%d]",  
-		      TtaGetElementTypeName (xpath_item->elType),
-		      xpath_item->index);
+	    {
+	      name = TtaGetElementTypeName (xpath_item->elType);
+	      l = strlen (name) + 14;
+	      buffer = (char *)TtaGetMemory (l);
+	      sprintf (buffer, "/%s[%d]", name, xpath_item->index);
+	    }
 	  StrACat (&xpath_expr, buffer);
+	  TtaFreeMemory (buffer);
+	  buffer = NULL;
 	}
       if (xpath_item->id_value)
 	TtaFreeMemory (xpath_item->id_value);
@@ -684,8 +742,9 @@ static char * XPathList2Str (XPathList *xpath_list, int firstCh, int len, int mo
 
   if (mode & SEL_STRING_RANGE)
     {
-      snprintf (buffer, sizeof (buffer),
-		",\"\",%d,%d)", firstCh, len);
+      l = 30;
+      buffer = (char *)TtaGetMemory (l);
+      sprintf (buffer, ",\"\",%d,%d)", firstCh, len);
       StrACat (&xpath_expr, buffer);
       if (mode & SEL_START_POINT || mode & SEL_END_POINT)
 	StrACat (&xpath_expr, ")");
@@ -705,15 +764,16 @@ static char * XPathList2Str (XPathList *xpath_list, int firstCh, int len, int mo
   It's up to the caller to free the returned string.
   Returns NULL in case of failure.
   ----------------------------------------------------------------------*/
-static char *XPointer_ThotEl2XPath (Element start, int firstCh, int len, selMode mode, ThotBool firstF)
+static char *XPointer_ThotEl2XPath (Element start, int firstCh, int len,
+				    selMode mode, ThotBool firstF)
 {
-  Element el, prev;
+  Element     el, prev;
   ElementType elType, prevElType;
-  int child_count;
-  XPathItem *xpath_item;
-  XPathList xpath_list = (XPathItem *) NULL;
-  char *xpath_expr;
-  char *id_value = NULL;
+  int         child_count;
+  XPathItem  *xpath_item;
+  XPathList   xpath_list = (XPathItem *) NULL;
+  char       *xpath_expr;
+  char       *id_value = NULL;
 
   /* if the user selected a text, adjust the start/end indexes according
      to its siblings inlined text */
@@ -747,7 +807,7 @@ static char *XPointer_ThotEl2XPath (Element start, int firstCh, int len, selMode
 	    }
 	}
       /* add the info we found to the xpath list*/
-      xpath_item = TtaGetMemory (sizeof (XPathItem));
+      xpath_item = (XPathItem*)TtaGetMemory (sizeof (XPathItem));
       xpath_item->elType = elType;
       xpath_item->index = child_count;
       xpath_item->id_value = NULL;
@@ -759,7 +819,7 @@ static char *XPointer_ThotEl2XPath (Element start, int firstCh, int len, selMode
 
   if (id_value)
     {
-      xpath_item = TtaGetMemory (sizeof (XPathItem));
+      xpath_item = (XPathItem*)TtaGetMemory (sizeof (XPathItem));
       xpath_item->elType = TtaGetElementType (el);	
       xpath_item->id_value = id_value;
       xpath_item->next = xpath_list;
@@ -786,17 +846,15 @@ static char *XPointer_ThotEl2XPath (Element start, int firstCh, int len, selMode
 char * XPointer_build (Document doc, View view, ThotBool useDocRoot)
 {
   Element     firstEl, lastEl;
-
+  ElementType elType;
   int         firstCh, lastCh, i;
   int         firstLen;
-
   char       *firstXpath;
   char       *lastXpath = NULL;
-  ElementType elType;
+  char       *schemaName;
 
-  char     *schemaName;
-
-  selMode    mode = 0;
+  selMode    firstMode = (selMode)0;
+  selMode    lastMode = (selMode)0;
 
   /* @@ debug */
   char       *xptr_expr = NULL;
@@ -804,15 +862,17 @@ char * XPointer_build (Document doc, View view, ThotBool useDocRoot)
   elType.ElSSchema = TtaGetDocumentSSchema (doc);
   /* only do this operation on XML and HTML documents */
   /* @@ JK: should be a function in AHTURLTools */
+
   schemaName = TtaGetSSchemaName (elType.ElSSchema);
   if (strcmp(schemaName, "HTML")
       && strcmp(schemaName, "XHTML")
       && strcmp(schemaName, "XML")
       && strcmp(schemaName, "MathML")
-      && strcmp(schemaName, "GraphML")
-      && strcmp(schemaName, "Annot"))
+      && strcmp(schemaName, "SVG")
+      && strcmp(schemaName, "Annot")
+      && DocumentTypes[doc] != docXml)
     return NULL;
-
+  
   /* is the document selected? */
   if (useDocRoot)
     {
@@ -822,7 +882,7 @@ char * XPointer_build (Document doc, View view, ThotBool useDocRoot)
       firstLen = 0;
       firstCh = 0;
       lastEl = NULL;
-      mode  = mode & ~(SEL_START_POINT | SEL_END_POINT);
+      firstMode  = (selMode)((int)firstMode & ~(SEL_START_POINT | SEL_END_POINT));
     }
   else
     {
@@ -835,11 +895,19 @@ char * XPointer_build (Document doc, View view, ThotBool useDocRoot)
 #ifdef DEBUG_XPOINTER
       printf ("first Ch is %d, i is %d\n", firstCh, i);
 #endif 
-      AdjustSelMode (&firstEl, &firstCh, i, &mode, SEL_FIRST_EL);
+      AdjustSelMode (&firstEl, &firstCh, i, &firstMode, SEL_FIRST_EL);
       
       if (firstEl == NULL)
 	return NULL; /* ERROR, there is no selection */
 
+      if  (TtaGetElementLineNumber (firstEl) == 0)
+	{
+	  /* ERROR, don't annotate end elements that Amaya added
+	     internally, but that don't exist in the document */
+	  InitInfo ("XPointer", TtaGetMessage (AMAYA, AM_NO_SXPOINTER));
+	  return NULL;
+	}
+
       /* is it a caret or an extension selection? */
       if (TtaIsSelectionEmpty ())
 	lastEl = NULL;
@@ -849,24 +917,43 @@ char * XPointer_build (Document doc, View view, ThotBool useDocRoot)
 #ifdef DEBUG_XPOINTER
 	  printf ("last Ch is %d, i is %d\n", lastCh, i);
 #endif 
-	  AdjustSelMode (&lastEl, &lastCh, i, &mode, SEL_LAST_EL);
+	  AdjustSelMode (&lastEl, &lastCh, i, &lastMode, SEL_LAST_EL);
+
+	  if  (TtaGetElementLineNumber (lastEl) == 0)
+	    {
+	      /* ERROR, don't annotate end elements that Amaya added
+		 internally, but that don't exist in the document */
+	      InitInfo ("XPointer", TtaGetMessage (AMAYA, AM_NO_EXPOINTER));
+	      return NULL;
+	    }
+
 	}
+
       /* if the selection is in the same element, adjust the first element's
 	 length */
       if (firstEl == lastEl)
 	{
-	  firstLen = lastCh - firstCh + 1;
+	  /* if we have an empty selection, ignore the last element and its data */
+	  if (lastCh != firstCh)
+	    {
+	      firstLen = lastCh - firstCh + 1;
+	      firstMode  = (selMode)((int)firstMode & ~(SEL_START_POINT | SEL_END_POINT));
+	    }
+	  else  /* the selection is only a caret */
+	    {
+	      firstLen = 1;
+	    }
 	  lastEl = NULL;
-	  mode  = mode & ~(SEL_START_POINT | SEL_END_POINT);
 	}
       else
 	{
 	  firstLen = 1;
-	  mode |= SEL_RANGE_TO;
+	  firstMode = (selMode)((int)firstMode | SEL_RANGE_TO);
+	  lastMode = (selMode)((int)lastMode | SEL_RANGE_TO);
 	}
     }
 
-  /* @@ JK: W e don't know yet how to handle annotations on symbols, so we just
+  /* @@ JK: We don't know yet how to handle annotations on symbols, so we just
      forbid making XPointers on them, for the moment @@ */
   elType = TtaGetElementType (firstEl);
   if (elType.ElTypeNum == THOT_SYMBOL_UNIT)
@@ -882,13 +969,13 @@ char * XPointer_build (Document doc, View view, ThotBool useDocRoot)
      walking the tree when we reach it */
   RootElement = AGetRootElement (doc);
 
-  firstXpath = XPointer_ThotEl2XPath (firstEl, firstCh, firstLen, mode, TRUE);
+  firstXpath = XPointer_ThotEl2XPath (firstEl, firstCh, firstLen, firstMode, TRUE);
 #ifdef DEBUG_XPOINTER
   fprintf (stderr, "\nfirst xpointer is %s", firstXpath);
 #endif  
   if (lastEl)
     {
-      lastXpath = XPointer_ThotEl2XPath (lastEl, lastCh, 1, mode, FALSE);
+      lastXpath = XPointer_ThotEl2XPath (lastEl, lastCh, 1, lastMode, FALSE);
 #ifdef DEBUG_XPOINTER
       fprintf (stderr, "\nlast xpointer is %s\n", lastXpath);
 #endif  
@@ -901,15 +988,18 @@ char * XPointer_build (Document doc, View view, ThotBool useDocRoot)
     }
 
   /* calculate the length of the xptr buffer */
-  i = sizeof ("xpointer()/range-to()") + strlen (firstXpath) 
-	      + ((lastEl) ? strlen (lastXpath) : 0) + 1;
-  xptr_expr = TtaGetMemory (i);
-
-  if (lastEl)
-      sprintf (xptr_expr, "xpointer(%s/range-to(%s))", firstXpath, lastXpath);
-  else
-    sprintf (xptr_expr, "xpointer(%s)", firstXpath);
-
+  if (firstXpath)
+    {
+      i = sizeof ("xpointer()/range-to()") + strlen (firstXpath) 
+	+ ((lastEl) ? strlen (lastXpath) : 0) + 1;
+      xptr_expr = (char *)TtaGetMemory (i);
+      
+      if (lastEl)
+	sprintf (xptr_expr, "xpointer(%s/range-to(%s))", firstXpath, lastXpath);
+      else
+	sprintf (xptr_expr, "xpointer(%s)", firstXpath);
+    }
+  
   TtaFreeMemory (firstXpath);
   if (lastEl)
     TtaFreeMemory (lastXpath);
@@ -950,7 +1040,7 @@ void XPointer_bufferStore (Document doc, View view)
   /* make a full URI */
   if (xptr)
     {
-      xptr_buffer = TtaGetMemory (strlen (DocumentURLs[doc]) + strlen (xptr) + 2);
+      xptr_buffer = (char *)TtaGetMemory (strlen (DocumentURLs[doc]) + strlen (xptr) + 2);
       sprintf (xptr_buffer, "%s#%s", DocumentURLs[doc], xptr);
     }
 }
