@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT MIT and INRIA, 1996-2001.
+ *  (c) COPYRIGHT INRIA and W3C, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -32,20 +32,18 @@
 #include "EDITOR.h"
 #include "HTML.h"
 #include "MathML.h"
-#include "GraphML.h"
+#include "SVG.h"
 #include "TextFile.h"
+#include "XML.h"
 #include "amayamsg.h"
 #include "parser.h"
 
 #define NAME_LENGTH     32
 
-#define HTAppName     "amaya"
-#define HTAppVersion  "5.1"
-#define HTAppDate     " 11 July 2001"
-
 #define URL_STR       "/"
 #define URL_SEP       '/'
 
+
 /* Number of views used in Amaya */
 #define AMAYA_MAX_VIEW_DOC  7
 
@@ -65,7 +63,9 @@ typedef enum _AHTHeaderName
     AM_HTTP_CONTENT_TYPE = 0,
     AM_HTTP_CHARSET = 1,
     AM_HTTP_CONTENT_LENGTH = 2,
-    AM_HTTP_REASON = 3
+    AM_HTTP_REASON = 3,
+    AM_HTTP_CONTENT_LOCATION = 4,
+    AM_HTTP_FULL_CONTENT_LOCATION = 5
   } 
 AHTHeaderName;
 
@@ -77,6 +77,9 @@ typedef struct _AHTHeaders
     char *charset;
     char *content_length;
     char *reason;
+    char *content_location;
+    char *full_content_location;
+    int   status;
   }
 AHTHeaders;
 
@@ -189,13 +192,17 @@ typedef enum _ClickEvent {
 #define DocInfoCharset        467
 #define DocInfoContentTitle   468
 #define DocInfoContent        469
-#define DocInfoLocationTitle  470
-#define DocInfoLocation       471
+#define DocInfoURLTitle       470
+#define DocInfoURL            471
 #define DocInfoTitle1         472
 #define DocInfoTitle2         473
 #define DocInfoContent1       474
 #define DocInfoContent2       475
-#define MAX_REF         476
+#define DocInfoLocationTitle  476
+#define DocInfoLocation       477
+#define DocInfoDocTypeTitle   478
+#define DocInfoDocType        479
+#define MAX_REF         480
 
 #define FormPrint        1
 #define PrintOptions     2
@@ -206,6 +213,28 @@ typedef enum _ClickEvent {
 #define PPagesPerSheet   7
 #define PRINT_MAX_REF	 8
 
+#define FormLibrary              1
+#define CopySVGLibSelection      2
+#define ReferToSVGLibSelection   3
+#define AddSVGModel              4
+#define NewSVGFileURL            5
+#define SVGLibCatalogueTitle     6
+#define SVGLibraryLabel          7
+#define SVGLibraryLabel1         8
+#define SVGLibraryLabel2         9
+#define SVGLibraryLabel3        10
+#define SVGLibraryLabel4        11
+#define NewSVGLibraryTitle      12
+#define NewSVGLibrary           13
+#define SVGLibraryURL           14
+#define SVGLibraryDir           15
+#define SVGLibrarySel           16
+#define LibraryFilter           17
+#define SVGLibFileBrowser       18
+#define SVGLibFileBrowserText   19
+#define LibraryCopyOption       20
+#define LIBRARY_MAX_REF         21
+
 #define ImageURL	1
 #define ImageLabel	2
 #define ImageLabel2	3
@@ -227,13 +256,13 @@ typedef enum _ClickEvent {
 
 #define FormMaths 0
 #define MenuMaths 1
-#define MAX_MATHS  2
+#define MenuMaths1 2
+#define MAX_MATHS  3
 
-#ifdef GRAPHML
 #define FormGraph 0
 #define MenuGraph 1
-#define MAX_GRAPH 2
-#endif /* GRAPHML */
+#define MenuGraph1 2
+#define MAX_GRAPH 3
 
 /* The possible GET/POST/PUT request modes */
 /*synchronous request*/
@@ -249,20 +278,23 @@ typedef enum _ClickEvent {
 /* send the form using the GET HTTP method */
 #define AMAYA_FORM_GET  32	/*0x100000 */
 /* bypass caching */
-#define AMAYA_NOCACHE	64
+#define AMAYA_NOCACHE	64 /* 2^6 */
 /* don't follow redirections */
-#define AMAYA_NOREDIR	128
+#define AMAYA_NOREDIR	128 /* 2^7 */
 /* Prevents a stop race condition in ASYNC transfers */
-#define AMAYA_ASYNC_SAFE_STOP  256
-#define AMAYA_LOAD_CSS   512
-#define AMAYA_FLUSH_REQUEST 1024
-#define AMAYA_USE_PRECONDITIONS 2048
-#define AMAYA_LOAD_IMAGE 4096
+#define AMAYA_ASYNC_SAFE_STOP  256 /* 2^8 */
+#define AMAYA_LOAD_CSS   512 /* 2^9 */
+#define AMAYA_FLUSH_REQUEST 1024 /* 2^10 */
+#define AMAYA_USE_PRECONDITIONS 2048 /* 2^11 */
+#define AMAYA_LOAD_IMAGE 4096 /* 2^12 */
 /* post an annonation */
-#define AMAYA_FILE_POST 8192
+#define AMAYA_FILE_POST 8192 /* 2^13 */
 /* delete an annotation */
-#define AMAYA_DELETE    16384
-#define AMAYA_MBOOK_IMAGE 32768
+#define AMAYA_DELETE    16384 /* 2^14 */
+#define AMAYA_MBOOK_IMAGE 32768 /* 2^15 */
+/* post an annotation using PUT */
+#define AMAYA_SIMPLE_PUT 65536 /* 2^16 */
+
 /*
  * Flags to indicate the action to take when the network options
  * are modified
@@ -292,8 +324,8 @@ typedef enum _ClickEvent {
 #define AMAYA_PARSE_ALL         31  /* All the parts */
 
 THOT_EXPORT int        AMAYA;     /* Index of amaya message table */
-THOT_EXPORT int        appArgc;
-THOT_EXPORT char     **appArgv;
+extern      int        appArgc;	  /* defined in EDITORAPP.c */
+extern      char     **appArgv;   /* defined in EDITORAPP.c */
 THOT_EXPORT char       TempFileDirectory[MAX_LENGTH];
 THOT_EXPORT char       Answer_text[MAX_LENGTH];
 THOT_EXPORT char       Answer_name[NAME_LENGTH];
@@ -304,6 +336,10 @@ THOT_EXPORT char       MathMLEntityName[MAX_LENGTH]; /* entity name typed by the
 THOT_EXPORT char       IdElemName[MAX_LENGTH]; /* element name typed by the user from the MakeID menu */
 THOT_EXPORT char       IdStatus[50]; /* element name typed by the user from the MakeID menu */
 THOT_EXPORT char       ImgFilter[NAME_LENGTH];
+THOT_EXPORT char       DirectoryImage[MAX_LENGTH];
+THOT_EXPORT char       LastURLImage[MAX_LENGTH];
+THOT_EXPORT char       ImageName[MAX_LENGTH];
+THOT_EXPORT char       ImgAlt[MAX_LENGTH];
 THOT_EXPORT char      *LastURLName;	/* last URL requested               */
 THOT_EXPORT char      *DirectoryName;	/* local path of the document       */
 THOT_EXPORT char      *DocumentName;	/* document name                    */
@@ -333,9 +369,7 @@ THOT_EXPORT int        BaseDialog;
 THOT_EXPORT int        BasePrint;
 THOT_EXPORT int        BaseImage;
 THOT_EXPORT int	       MathsDialogue;
-#ifdef GRAPHML
 THOT_EXPORT int	       GraphDialogue;
-#endif /* GRAPHML */
 THOT_EXPORT int	       BaseCSS;
 THOT_EXPORT int        ReturnOption;
 THOT_EXPORT int        NumberRows;
@@ -343,24 +377,31 @@ THOT_EXPORT int        NumberCols;
 THOT_EXPORT int        TBorder;
 THOT_EXPORT int        ReturnOptionMenu;
 THOT_EXPORT int        IdDoc;
+THOT_EXPORT int        BaseLibrary;
 THOT_EXPORT Document   CurrentDocument;
 THOT_EXPORT Document   SavingDocument;
 THOT_EXPORT Document   SavingObject;
 THOT_EXPORT Document   AttrHREFdocument;
 THOT_EXPORT Document   DocBook;
 THOT_EXPORT Document   IncludedDocument;
+THOT_EXPORT Document   ParsedDoc; /* The document to which CSS are to be applied */
 THOT_EXPORT Element    AttrHREFelement;
 THOT_EXPORT char      *AttrHREFvalue;
 THOT_EXPORT Document   SelectionDoc;
-THOT_EXPORT ThotBool  IsNewAnchor;
-THOT_EXPORT ThotBool  UseLastTarget;
-THOT_EXPORT ThotBool  LinkAsCSS;
+THOT_EXPORT ThotBool   AttrHREFundoable;
+THOT_EXPORT ThotBool   IsNewAnchor;
+THOT_EXPORT ThotBool   UseLastTarget;
+THOT_EXPORT ThotBool   LinkAsCSS;
+THOT_EXPORT ThotBool   LinkAsXmlCSS;
 THOT_EXPORT ThotBool   SaveAsHTML;
 THOT_EXPORT ThotBool   SaveAsXML;
 THOT_EXPORT ThotBool   SaveAsText;
 THOT_EXPORT ThotBool   CopyImages;	/* should we copy images in Save As */
 THOT_EXPORT ThotBool   UpdateURLs;	/* should we update URLs in Save As */
 THOT_EXPORT ThotBool   UserAnswer;
+THOT_EXPORT ThotBool   ExtraChoice;
+THOT_EXPORT ThotBool   TableMenuActive;
+THOT_EXPORT ThotBool   MTableMenuActive;
 THOT_EXPORT ThotBool   InNewWindow;
 THOT_EXPORT ThotBool   SelectionInPRE;
 THOT_EXPORT ThotBool   SelectionInComment;
@@ -392,7 +433,10 @@ THOT_EXPORT ThotBool   XMLErrorsFoundInProfile;
 THOT_EXPORT ThotBool   XMLNotWellFormed;
 THOT_EXPORT ThotBool   XMLCharacterNotSupported;
 THOT_EXPORT ThotBool   XMLUnknownEncoding;
-
+/*  enabling/disabling DAVLib */
+THOT_EXPORT ThotBool   DAVLibEnable;
+THOT_EXPORT ThotBool   Synchronizing;
+THOT_EXPORT ThotBool   AmayaUniqueInstance;
 THOT_EXPORT ThotBool   IdApplyToSelection; /* used in the Make ID menu */
 THOT_EXPORT FILE      *ErrFile;
 typedef enum
@@ -406,24 +450,26 @@ typedef enum
   docLog,
   docSVG,
   docMath,
-  docXml
+  docXml,
+  docLibrary,
+  docBookmark
 } DocumentType;
 
 
 /* a record for data associated with a request */
 typedef struct _DocumentMetaDataElement
 {
-  char    *initial_url;      /* if the server returns a different URL name
-				after a redirection, we store here the one
-				that the browser first asked */
-  char    *form_data;        /* form data associated with a URL */
-  char    *content_type;     /* content type returned by the server */
-  char    *charset;          /* charset returned by the server */
-  char    *content_length;   /* content length returned by the server */
+  char      *initial_url;      /* if the server returns a different URL name
+				  after a redirection, we store here the one
+				  that the browser first asked */
+  char      *form_data;        /* form data associated with a URL */
+  char      *content_type;     /* content type returned by the server */
+  char      *charset;          /* charset returned by the server */
+  char      *content_length;   /* content length returned by the server */
+  char      *content_location; /* simplified content location returned by the server */
+  char      *full_content_location; /* full content location returned by the server */
   ClickEvent method;           /* method used to send this data */
-  ThotBool   put_default_name; /* URL name was concatenated with DEFAULT_NAME*/
   ThotBool   xmlformat;        /* the document should be exported in xml format */
-  
 #ifdef ANNOTATIONS
   Document   source_doc;       /* if the document is an annotation,
 				  this variable giveso the annoted document
@@ -441,17 +487,37 @@ THOT_EXPORT DocumentMetaDataElement *DocumentMeta[DocumentTableLength];
 THOT_EXPORT DocumentType             DocumentTypes[DocumentTableLength];
 /* Document is in read only mode */
 THOT_EXPORT ThotBool                 ReadOnlyDocument[DocumentTableLength];
+/* Document with section numbering */
+THOT_EXPORT ThotBool                 SNumbering[DocumentTableLength];
+/* Document that shows buttons */
+THOT_EXPORT ThotBool                 SButtons[DocumentTableLength];
+/* Document that shows address */
+THOT_EXPORT ThotBool                 SAddress[DocumentTableLength];
+/* Document that shows map areas */
+THOT_EXPORT ThotBool                 MapAreas[DocumentTableLength];
 /* identifier of the document displaying the source code */
 THOT_EXPORT Document                 DocumentSource[DocumentTableLength];
 /* The whole document is loaded when the corresponding entry in FilesLoading is 0 */
 THOT_EXPORT int                      FilesLoading[DocumentTableLength];
 /* Status (error, success) of the download of the objects of a document */
 THOT_EXPORT int                      DocNetworkStatus[DocumentTableLength];
-/* Status of the XHTML document (basic, strict, transitional) */
-THOT_EXPORT int                      ParsingLevel[DocumentTableLength];
 
 THOT_EXPORT Document                 W3Loading; /* the document being loaded */
 THOT_EXPORT Document                 BackupDocument; /* the current backup */
+/* list of previous open URLs */
+#define MAX_URL_list 20
+THOT_EXPORT char                    *URL_list;
+THOT_EXPORT int                      URL_list_len;
+THOT_EXPORT ThotBool                 URL_list_keep;
+#ifdef _SVG
+THOT_EXPORT char                    *SVGlib_list;
+#endif /* _SVG */
+
+/* list of auto-saved files */
+#define MAX_AutoSave_list 20
+THOT_EXPORT char                    *AutoSave_list;
+THOT_EXPORT int                      AutoSave_list_len;
+THOT_EXPORT int                      AutoSave_Interval;
 
 /* button indexes */
 THOT_EXPORT int iStop;
@@ -482,7 +548,8 @@ THOT_EXPORT int iTable;
 #define IMAGE_MODIFIED		3
 
 
-typedef void (*LoadedImageCallback)(Document doc, Element el, char *file, void *extra);
+typedef void (*LoadedImageCallback)(Document doc, Element el, char *file,
+				    void *extra, ThotBool isnew);
 typedef struct _ElemImage
   {
      Element             currentElement;/* first element using this image */
@@ -495,7 +562,8 @@ ElemImage;
 typedef struct _LoadedImageDesc
   {
      char               *originalName;  /* complete URL of the image                */
-     char               *localName;     /* local name (without path) of the image   */
+     char               *localName;     /* relative name (without path) of the image*/
+     char               *tempfile;      /* name of the file that stores the image   */ 
      char               *content_type;  /* the MIME type as sent by the server      */
      struct _LoadedImageDesc *prevImage;/* double linked list                       */
      struct _LoadedImageDesc *nextImage;/* easier to unchain                        */
@@ -525,4 +593,42 @@ THOT_EXPORT LoadedImageDesc *ImageLocal;
 #define MAX_TXT_LEN 1024	/* Max. length of strings */
 #endif  /* MAX_TXT_LEN */
 
+/* a collection of the XML MIME types that keep changing and that we use in Amaya */
+#define AM_XHTML_MIME_TYPE        "application/xhtml+xml"
+#define AM_SVG_MIME_TYPE          "image/svg+xml"
+#define AM_MATHML_MIME_TYPE       "application/mathml+xml"
+#define AM_GENERIC_XML_MIME_TYPE  "text/xml"
+
+
+/* Doctype declarations used in Amaya */
+/* first line of doctype declarations */
+#define DOCTYPE1_HTML_STRICT          "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01//EN\""
+#define DOCTYPE1_HTML_TRANSITIONAL    "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\""
+#define DOCTYPE1_XHTML10_BASIC        "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML Basic 1.0//EN\""
+#define DOCTYPE1_XHTML10_STRICT       "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\""
+#define DOCTYPE1_XHTML11              "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.1//EN\""
+#define DOCTYPE1_XHTML11_PLUS_MATHML  "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.1 plus MathML 2.0//EN\""
+#define DOCTYPE1_XHTML11_PLUS_MATHML_PLUS_SVG  "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.1 plus MathML 2.0 plus SVG 1.1//EN\""
+#define DOCTYPE1_XHTML10_TRANSITIONAL "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\""
+#define DOCTYPE1_MATHML20             "<!DOCTYPE math PUBLIC \"-//W3C//DTD MathML 2.0//EN\""
+#define DOCTYPE1_SVG10                "<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.0//EN\""
+
+/* second line of doctype declarations */
+#define DOCTYPE2_HTML_STRICT          "      \"http://www.w3.org/TR/html4/strict.dtd\">"
+#define DOCTYPE2_HTML_TRANSITIONAL    "      \"http://www.w3.org/TR/html4/loose.dtd\">"
+#define DOCTYPE2_XHTML10_BASIC        "      \"http://www.w3.org/TR/xhtml-basic/xhtml-basic10.dtd\">"
+#define DOCTYPE2_XHTML10_STRICT       "      \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">"
+#define DOCTYPE2_XHTML11              "      \"http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd\">"
+#define DOCTYPE2_XHTML11_PLUS_MATHML  "      \"http://www.w3.org/Math/DTD/mathml2/xhtml-math11-f.dtd\">"
+#define DOCTYPE2_XHTML11_PLUS_MATHML_PLUS_SVG  "      \"http://www.w3.org/2002/04/xhtml-math-svg/xhtml-math-svg.dtd\">"
+#define DOCTYPE2_XHTML10_TRANSITIONAL "      \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">"
+#define DOCTYPE2_MATHML20             "      \"http://www.w3.org/TR/MathML2/dtd/mathml2.dtd\">"
+#define DOCTYPE2_SVG10                "      \"http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd\">"
+
+#define MATHML_XSLT_NAME     "pmathml.xsl"
+#define MATHML_XSLT_URI      "<!-- Processing Instruction generated by Amaya; the stylesheet is\n     available on the W3C site at http://www.w3.org/Math/XSL/pmathml.xsl -->\n<?xml-stylesheet type=\"text/xsl\" href=\""
+
+#define SVG_VERSION "1.0" /* version of the SVG specificatrion implemented
+			     by the Amaya SVG editor */
+
 #endif /* AMAYA_H */
