@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT MIT and INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA and W3C, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -19,7 +19,7 @@
 #include "parser.h"
 #include "HTMLnames.h"
 #include "MathMLnames.h"
-#include "GraphMLnames.h"
+#include "SVGnames.h"
 #include "XLinknames.h"
 
 /* define some pointers to let other parser functions access the local table */
@@ -39,7 +39,7 @@ static int        MathSup = 0;
 /*----------------------------------------------------------------------
    GetXHTMLSSchema returns the XHTML Thot schema for document doc.
   ----------------------------------------------------------------------*/
-SSchema         GetXHTMLSSchema (Document doc)
+SSchema GetXHTMLSSchema (Document doc)
 {
   SSchema	XHTMLSSchema;
 
@@ -50,10 +50,10 @@ SSchema         GetXHTMLSSchema (Document doc)
    return (XHTMLSSchema);
 }
 
-/*----------------------------------------------------------------------charName
+/*----------------------------------------------------------------------
    GetMathMLSSchema returns the MathML Thot schema for document doc.
   ----------------------------------------------------------------------*/
-SSchema         GetMathMLSSchema (Document doc)
+SSchema GetMathMLSSchema (Document doc)
 {
   SSchema	MathMLSSchema;
 
@@ -66,25 +66,25 @@ SSchema         GetMathMLSSchema (Document doc)
 }
 
 /*----------------------------------------------------------------------
-   GetGraphMLSSchema returns the GraphML Thot schema for document doc.
+   GetSVGSSchema returns the SVG Thot schema for document doc.
   ----------------------------------------------------------------------*/
-SSchema         GetGraphMLSSchema (Document doc)
+SSchema GetSVGSSchema (Document doc)
 
 {
-  SSchema	GraphMLSSchema;
+  SSchema	SVGSSchema;
 
-  GraphMLSSchema = TtaGetSSchema ("GraphML", doc);
-  if (GraphMLSSchema == NULL)
-    GraphMLSSchema = TtaNewNature(doc,
+  SVGSSchema = TtaGetSSchema ("SVG", doc);
+  if (SVGSSchema == NULL)
+    SVGSSchema = TtaNewNature(doc,
 				  TtaGetDocumentSSchema(doc),
-				  "GraphML", "GraphMLP");
-  return (GraphMLSSchema);
+				  "SVG", "SVGP");
+  return (SVGSSchema);
 }
 
 /*----------------------------------------------------------------------
    GetXLinkSSchema returns the XLink Thot schema for document doc.
   ----------------------------------------------------------------------*/
-SSchema         GetXLinkSSchema (Document doc)
+SSchema GetXLinkSSchema (Document doc)
 
 {
   SSchema	XLinkSSchema;
@@ -97,14 +97,31 @@ SSchema         GetXLinkSSchema (Document doc)
 }
 
 /*----------------------------------------------------------------------
-   GetGenericXMLSSchema returns the XML Thot schema for the document doc.
+   GetTextSSchema returns the TextFile Thot schema for document doc.
+   (this is not XML, but its useful to have this function here).
   ----------------------------------------------------------------------*/
-SSchema         GetGenericXMLSSchema (Document doc)
+SSchema GetTextSSchema (Document doc)
+
+{
+  SSchema	XLinkSSchema;
+
+  XLinkSSchema = TtaGetSSchema ("TextFile", doc);
+  if (XLinkSSchema == NULL)
+    XLinkSSchema = TtaNewNature(doc, TtaGetDocumentSSchema(doc),
+				"TextFile", "TextFileP");
+  return (XLinkSSchema);
+}
+
+/*----------------------------------------------------------------------
+   GetGenericXMLSSchema
+   Returns the XML Thot schema of name schemaName for the document doc.
+  ----------------------------------------------------------------------*/
+SSchema GetGenericXMLSSchema (char *schemaName, Document doc)
 
 {
   SSchema	XMLSSchema;
 
-  XMLSSchema = TtaGetSSchema ("XML", doc);
+  XMLSSchema = TtaGetSSchema (schemaName, doc);
   if (XMLSSchema == NULL)
       XMLSSchema = TtaNewNature(doc, TtaGetDocumentSSchema(doc),
 				"XML", "XMLP");
@@ -112,9 +129,10 @@ SSchema         GetGenericXMLSSchema (Document doc)
 }
 
 /*----------------------------------------------------------------------
-   GetGenericXMLSSchemaByUri returns the XML Thot schema for the document doc.
+   GetGenericXMLSSchemaByUri
+   Returns the XML Thot schema for the document doc.
   ----------------------------------------------------------------------*/
-SSchema         GetGenericXMLSSchemaByUri (char *uriName, Document doc, ThotBool *isnew)
+SSchema GetGenericXMLSSchemaByUri (char *uriName, Document doc, ThotBool *isnew)
 
 {
   SSchema	XMLSSchema;
@@ -139,110 +157,114 @@ SSchema GetXMLSSchema (int XMLtype, Document doc)
     return GetXHTMLSSchema (doc);
   else if (XMLtype == MATH_TYPE)
     return GetMathMLSSchema (doc);
-  else if (XMLtype == GRAPH_TYPE)
-    return GetGraphMLSSchema (doc);
+  else if (XMLtype == SVG_TYPE)
+    return GetSVGSSchema (doc);
   else if (XMLtype == XLINK_TYPE)
     return GetXLinkSSchema (doc);
   else
     return NULL;
 }
 
-
 /*----------------------------------------------------------------------
   MapXMLElementType
   Generic function which searchs in the Element Mapping table, selected
   by the parameter XMLtype, the entry XMLname and returns the corresponding
   Thot element type.
-   Returns:
+  Returns:
     - ElTypeNum and ElSSchema into elType  ElTypeNum = 0 if not found.
-    - content 
+    - content information about this entry
+    - checkProfile TRUE if the entry is valid for the current Doc profile.
   ----------------------------------------------------------------------*/
-void MapXMLElementType (int XMLtype,
-			char *XMLname,
-			ElementType *elType,
-			char **mappedName,
-			char *content,
-			ThotBool *highEnoughLevel,
-			Document doc)
+void MapXMLElementType (int XMLtype, char *XMLname, ElementType *elType,
+			char **mappedName, char *content,
+			ThotBool *checkProfile, Document doc)
 {
-   int                 i;
-   ElemMapping        *ptr;
+  ElemMapping        *ptr;
+  char                c;
+  int                 i, profile;
 
-   /* Initialize variables */
-   *mappedName = NULL;
-   *highEnoughLevel = TRUE;
-   elType->ElTypeNum = 0;
-
-   /* Select the right table */
-   if (XMLtype == XHTML_TYPE)
-     ptr = XHTMLElemMappingTable;
-   else if (XMLtype == MATH_TYPE)
-     {
-       if (ParsingLevel[doc] == L_Basic && DocumentTypes[doc] == docHTML)
-	 {
-	   /* Maths are not allowed in this document */
-	   ptr = NULL;
-	   *highEnoughLevel = FALSE;
-	 }
-       else
-	 ptr = MathMLElemMappingTable;
-     }
-   else if (XMLtype == GRAPH_TYPE)
-     {
-       if (ParsingLevel[doc] == L_Basic && DocumentTypes[doc] == docHTML)
-	 {
-	   /* Graphics are not allowed in this document */
-	   ptr = NULL;
-	   *highEnoughLevel = FALSE;
-	 }
-       else
-	 ptr = GraphMLElemMappingTable;
-     }
-   else
-     ptr = NULL;
+  /* Initialize variables */
+  *mappedName = NULL;
+  *checkProfile = TRUE;
+  elType->ElTypeNum = 0;
+  profile = TtaGetDocumentProfile (doc);
+  /* Select the right table */
+  if (XMLtype == XHTML_TYPE)
+    ptr = XHTMLElemMappingTable;
+  else if (XMLtype == MATH_TYPE)
+    {
+      if (profile == L_Basic && DocumentTypes[doc] == docHTML)
+	{
+	  /* Maths are not allowed in this document */
+	  ptr = NULL;
+	  *checkProfile = FALSE;
+	}
+      else
+	ptr = MathMLElemMappingTable;
+    }
+  else if (XMLtype == SVG_TYPE)
+    {
+      if (profile == L_Basic && DocumentTypes[doc] == docHTML)
+	{
+	  /* Graphics are not allowed in this document */
+	  ptr = NULL;
+	  *checkProfile = FALSE;
+	}
+      else
+	ptr = SVGElemMappingTable;
+    }
+  else
+    ptr = NULL;
    
-   if (ptr != NULL)
-     {
-       /* search in the ElemMappingTable */
-       i = 0;
-       /* look for the first concerned entry in the table */
-       while (ptr[i].XMLname[0] < XMLname[0] && ptr[i].XMLname[0] != EOS)
-	 i++;     
-       /* look at all entries starting with the right character */
-       do
-	 if (strcmp (ptr[i].XMLname, XMLname))
-	   /* it's not the tag */
-	   i++;
-	 else if (ParsingLevel[doc] != L_Other &&
-		  !(ptr[i].Level & ParsingLevel[doc]))
-	   {
-	     /* this tag is not valid for the current parsing level */
-	     *highEnoughLevel = FALSE;
-	     i++;
-	   }
-	 else
-	   {
-	     elType->ElTypeNum = ptr[i].ThotType;
-	     if (elType->ElSSchema == NULL)
-	       elType->ElSSchema = GetXMLSSchema (XMLtype, doc);
-	     *mappedName = ptr[i].XMLname;
-	     *content = ptr[i].XMLcontents;
-	   }
-       while (elType->ElTypeNum <= 0 && ptr[i].XMLname[0] == XMLname[0]);
-     }
+  if (ptr)
+    {
+      /* search in the ElemMappingTable */
+      i = 0;
+      /* case insensitive for HTML */
+      if (ptr == XHTMLElemMappingTable)
+	c = tolower (XMLname[0]);
+      else
+	c = XMLname[0];
+      /* look for the first concerned entry in the table */
+      while (ptr[i].XMLname[0] < c && ptr[i].XMLname[0] != EOS)
+	i++;     
+      /* look at all entries starting with the right character */
+      do
+	if (ptr == XHTMLElemMappingTable && strcasecmp (ptr[i].XMLname, XMLname))
+	  /* it's not the tag */
+	  i++;
+	else if (ptr != XHTMLElemMappingTable && strcmp (ptr[i].XMLname, XMLname))
+	  /* it's not the tag */
+	  i++;
+	else if (profile != L_Other && !(ptr[i].Level & profile))
+	  {
+	    /* this tag is not valid in the document profile */
+	    *checkProfile = FALSE;
+	    i++;
+	  }
+	else
+	  {
+	    elType->ElTypeNum = ptr[i].ThotType;
+	    if (elType->ElSSchema == NULL)
+	      elType->ElSSchema = GetXMLSSchema (XMLtype, doc);
+	    *mappedName = ptr[i].XMLname;
+	    *content = ptr[i].XMLcontents;
+	  }
+      while (elType->ElTypeNum <= 0 && ptr[i].XMLname[0] == c);
+    }
 }
 
 
 /*----------------------------------------------------------------------
    GetXMLElementName
-   Generic function which searchs in the mapping table the XML name for
+   Generic function which searchs in the mapping tables the XML name for
    a given Thot type.
   ----------------------------------------------------------------------*/
-char*           GetXMLElementName (ElementType elType, Document doc)
+char *GetXMLElementName (ElementType elType, Document doc)
 {
   ElemMapping  *ptr;
   char         *name;
-  int           i;
+  int           i, profile;
   ThotBool      invalid = FALSE;
 
   if (elType.ElTypeNum > 0)
@@ -252,19 +274,21 @@ char*           GetXMLElementName (ElementType elType, Document doc)
       name = TtaGetSSchemaName (elType.ElSSchema);
       if (strcmp ("MathML", name) == 0)
 	ptr = MathMLElemMappingTable;
-      else if (strcmp ("GraphML", name) == 0)
-	ptr = GraphMLElemMappingTable;
-      else
+      else if (strcmp ("SVG", name) == 0)
+	ptr = SVGElemMappingTable;
+      else if (strcmp ("HTML", name) == 0)
 	ptr = XHTMLElemMappingTable;
-      
+      else
+	ptr = NULL;
+
+      profile = TtaGetDocumentProfile (doc);
       if (ptr)
 	do
 	  {
 	    if (ptr[i].ThotType == elType.ElTypeNum)
 	      {
 		if (doc == 0 || 
-		    ParsingLevel[doc] == L_Other ||
-		    (ptr[i].Level & ParsingLevel[doc]))
+		    profile == L_Other || (ptr[i].Level & profile))
 		  return ptr[i].XMLname;
 		else
 		  invalid = TRUE;
@@ -283,10 +307,10 @@ char*           GetXMLElementName (ElementType elType, Document doc)
 
 /*----------------------------------------------------------------------
    IsXMLElementInline
-   Generic function which searchs in the mapping table if a given
+   Generic function which searchs in the mapping tables if a given
    Thot type is an inline character or not
   ----------------------------------------------------------------------*/
-ThotBool         IsXMLElementInline (ElementType elType)
+ThotBool IsXMLElementInline (ElementType elType, Document doc)
 {
   int            i;
   ThotBool       ret = FALSE;
@@ -300,10 +324,12 @@ ThotBool         IsXMLElementInline (ElementType elType)
       name = TtaGetSSchemaName (elType.ElSSchema);
       if (strcmp ("MathML", name) == 0)
 	ptr = MathMLElemMappingTable;
-      else if (strcmp ("GraphML", name) == 0)
-	ptr = GraphMLElemMappingTable;
-      else
+      else if (strcmp ("SVG", name) == 0)
+	ptr = SVGElemMappingTable;
+      else if (strcmp ("HTML", name) == 0)
 	ptr = XHTMLElemMappingTable;
+      else
+	ptr = NULL;
       
       if (ptr)
 	{
@@ -311,65 +337,125 @@ ThotBool         IsXMLElementInline (ElementType elType)
 		 ptr[i].ThotType != elType.ElTypeNum)
 	    i++;
 	  if (ptr[i].ThotType == elType.ElTypeNum)
-	    ret = ptr[i].Inline;
+	    ret = ptr[i].InlineElem;
 	}
     }
   return ret;
 }
 
+/*----------------------------------------------------------------------
+   MapMathMLAttributeValue
+   Search in the Attribute Value Mapping Table the entry for the attribute
+   ThotAtt and its value attVal. Returns the corresponding Thot value.
+  ----------------------------------------------------------------------*/
+void MapXMLAttributeValue (int XMLtype, char *attVal, const AttributeType *attrType,
+			      int *value)
+{
+  AttrValueMapping   *ptr;
+  int                 i;
+
+  /* Select the right table */
+  if (XMLtype == XHTML_TYPE)
+    ptr = XhtmlAttrValueMappingTable;
+  else if (XMLtype == MATH_TYPE)
+    ptr = MathMLAttrValueMappingTable;
+  else if (XMLtype == SVG_TYPE)
+    ptr = SVGAttrValueMappingTable;
+  else
+    ptr = NULL;
+  
+  *value = 0;
+  i = 0;
+  if (ptr == NULL)
+    return;
+  while (ptr[i].ThotAttr != attrType->AttrTypeNum && ptr[i].ThotAttr != 0)
+    i++;
+  if (ptr[i].ThotAttr == attrType->AttrTypeNum)
+    do
+      if (!strcmp (ptr[i].XMLattrValue, attVal))
+	*value = ptr[i].ThotAttrValue;
+      else
+	i++;
+    while (*value == 0 && ptr[i].ThotAttr == attrType->AttrTypeNum);
+}
+
+
 /*----------------------------------------------------------------------
    MapXMLAttribute
    Generic function which searchs in the Attribute Mapping Table (table)
    the entry attrName associated to the element elementName.
    Returns the corresponding entry or -1.
   ----------------------------------------------------------------------*/
-int       MapXMLAttribute (int XMLtype, char *attrName,
-			   char *elementName, ThotBool *highEnoughLevel,
-			   Document doc, int *thotType)
+int MapXMLAttribute (int XMLtype, char *attrName, char *elementName,
+		     ThotBool *checkProfile, Document doc, int *thotType)
 {
-  int               i;
-  AttributeMapping *ptr;
+  AttributeMapping   *ptr;
+  char                c;
+  int                 i, profile;
 
   /* Initialization */
-  *highEnoughLevel = TRUE;
+  *checkProfile = TRUE;
   i = 1;
   *thotType = 0;
-
-   /* Select the right table */
-   if (XMLtype == XHTML_TYPE)
-     ptr = XHTMLAttributeMappingTable;
-   else if (XMLtype == MATH_TYPE)
-     ptr = MathMLAttributeMappingTable;
-   else if (XMLtype == GRAPH_TYPE)
-     ptr = GraphMLAttributeMappingTable;
-   else if (XMLtype == XLINK_TYPE)
-     ptr = XLinkAttributeMappingTable;
-   else
-     ptr = NULL;
-
+  
+  /* Select the right table */
+  if (XMLtype == XHTML_TYPE)
+    ptr = XHTMLAttributeMappingTable;
+  else if (XMLtype == MATH_TYPE)
+    ptr = MathMLAttributeMappingTable;
+  else if (XMLtype == SVG_TYPE)
+    ptr = SVGAttributeMappingTable;
+  else if (XMLtype == XLINK_TYPE)
+    ptr = XLinkAttributeMappingTable;
+  else
+    ptr = NULL;
+  
   if (ptr == NULL)
     return -1;
-
-  /* look for the first concerned entry in the table */
-  while (ptr[i].XMLattribute[0] < attrName[0] && 
-	 ptr[i].XMLattribute[0] != EOS)
-    i++;
-
-  while (ptr[i].XMLattribute[0] == attrName[0])
+  
+  if (strcmp (attrName, "unknown_attr") == 0)
     {
-      if (strcmp (ptr[i].XMLattribute, attrName) ||
-	  (ptr[i].XMLelement[0] != EOS &&
-	   strcmp (ptr[i].XMLelement, elementName)))
+      *thotType = ptr[0].ThotAttribute;
+      return 0;
+    }
+
+  /* case insensitive for HTML */
+  if (ptr == XHTMLAttributeMappingTable)
+    c = tolower (attrName[0]);
+  else
+    c = attrName[0];
+
+  profile = TtaGetDocumentProfile (doc);
+  /* look for the first concerned entry in the table */
+  while (ptr[i].XMLattribute[0] < c &&  ptr[i].XMLattribute[0] != EOS)
+    i++;
+  while (ptr[i].XMLattribute[0] == c)
+    {
+      if (ptr == XHTMLAttributeMappingTable &&
+	  (strcasecmp (ptr[i].XMLattribute, attrName) ||
+	   (ptr[i].XMLelement[0] != EOS &&
+	    strcasecmp (ptr[i].XMLelement, elementName))))
 	i++;
-      else if (ParsingLevel[doc] != L_Other &&
-	       !(ptr[i].Level & ParsingLevel[doc]))
+      else if (ptr != XHTMLAttributeMappingTable &&
+	  (strcmp (ptr[i].XMLattribute, attrName) ||
+	   (ptr[i].XMLelement[0] != EOS &&
+	    strcmp (ptr[i].XMLelement, elementName))))
+	i++;
+      else if (profile != L_Other && !(ptr[i].Level & profile))
 	{
-	  *highEnoughLevel = FALSE;
+	  *checkProfile = FALSE;
 	  i++;
 	}
       else
 	{
-	  *thotType = ptr[i].ThotAttribute;
+	  /* Special case for the 'name' attribute for 
+	     elements 'a' and 'map' in xhtml1.1 profile */
+	  if ((profile == L_Xhtml11) &&
+	      !strcmp (attrName, "name") &&
+	      (!strcmp (elementName, "a") || !strcmp (elementName, "map")))
+	    *checkProfile = FALSE;
+	  else
+	    *thotType = ptr[i].ThotAttribute;
 	  return (i);
 	}
     }
@@ -379,16 +465,15 @@ int       MapXMLAttribute (int XMLtype, char *attrName,
 
 /*----------------------------------------------------------------------
    GetXMLAttributeName
-   Generic function which searchs in the mapping table the XML name for
+   Generic function which searchs in the mapping tables the XML name for
    a given Thot type.
   ----------------------------------------------------------------------*/
-char*           GetXMLAttributeName (AttributeType attrType,
-				     ElementType elType,
-				     Document doc)
+char *GetXMLAttributeName (AttributeType attrType, ElementType elType,
+			   Document doc)
 {
   AttributeMapping   *ptr;
   char               *name, *tag;
-  int                 i;
+  int                 i, profile;
   ThotBool            invalid = FALSE;
 
   if (attrType.AttrTypeNum > 0)
@@ -404,13 +489,14 @@ char*           GetXMLAttributeName (AttributeType attrType,
       name = TtaGetSSchemaName (attrType.AttrSSchema);
       if (strcmp ("MathML", name) == 0)
 	ptr = MathMLAttributeMappingTable;
-      else if (strcmp ("GraphML", name) == 0)
-	ptr = GraphMLAttributeMappingTable;
+      else if (strcmp ("SVG", name) == 0)
+	ptr = SVGAttributeMappingTable;
       else if (strcmp ("XLink", name) == 0)
 	ptr = XLinkAttributeMappingTable;
       else
 	ptr = XHTMLAttributeMappingTable;
       
+      profile = TtaGetDocumentProfile (doc);
       if (ptr)
 	do
 	  {
@@ -419,8 +505,8 @@ char*           GetXMLAttributeName (AttributeType attrType,
 		 !strcmp (ptr[i].XMLelement, tag)))
 	      {
 		if (doc != 0 &&
-		    ParsingLevel[doc] != L_Other &&
-		    !(ptr[i].Level & ParsingLevel[doc]))
+		    profile != L_Other &&
+		    !(ptr[i].Level & profile))
 		  invalid = TRUE;
 		else
 		  return ptr[i].XMLattribute;
