@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT MIT and INRIA, 1999.
+ *  (c) COPYRIGHT INRIA and W3C, 1999-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -16,21 +16,24 @@
  then associate a macro to its index.
 ***************************************/
 
-static STRING       Manual[] = {
+static char *Manual[] = {
   "Index.html",
 "Browsing.html",
-"Selecting.html",
-"Searching.html",
 "Views.html",
+"Selecting.html",
+"Document.html",
 "Creating.html",
-"Links.html",
+"Attributes.html",
+"EditChar.html",
 "Changing.html",
+"Links.html",
 "Tables.html",
+"ImageMaps.html",
 "Math.html",
 "SVG.html",
-"ImageMaps.html",
+"Xml.html",
 "StyleSheets.html",
-"Attributes.html",
+"Searching.html",
 "SpellChecking.html",
 "Publishing.html",
 "Printing.html",
@@ -39,33 +42,40 @@ static STRING       Manual[] = {
 "Annotations.html",
 "Configure.html",
 "ShortCuts.html",
-"Access.html"
+"Access.html",
+"bookmarks.html",
+"WebDAV.html"
 };
 
 /* index name... which relates to the precedent table */
 #define INDEX          0
 #define BROWSING       1
-#define SELECTING      2
-#define SEARCHING      3
-#define VIEWS          4
+#define VIEWS          2
+#define SELECTING      3
+#define DOCUMENT       4
 #define CREATING       5
-#define LINKS          6
-#define CHANGING       7
-#define TABLES         8
-#define MATH           9
-#define SVG           10
+#define ATTRIBUTES     6
+#define EDITCHAR       7
+#define CHANGING       8
+#define LINKS          9
+#define TABLES        10
 #define IMAGEMAPS     11
-#define CSS           12
-#define ATTRIBUTES    13
-#define SPELLCHECKING 14
-#define PUBLISHING    15
-#define PRINTING      16
-#define NUMBERING     17
-#define MAKEBOOK      18
-#define ANNOTATE      19
-#define CONFIGURE     20
-#define SHORTCUTS     21
-#define ACCESS        22
+#define MATH          12
+#define SVG           13
+#define XML           14
+#define CSS           15
+#define SEARCHING     16
+#define SPELLCHECKING 17
+#define PUBLISHING    18
+#define PRINTING      19
+#define NUMBERING     20
+#define MAKEBOOK      21
+#define ANNOTATE      22
+#define CONFIGURE     23
+#define SHORTCUTS     24
+#define ACCESS        25
+#define BOOK_MARKS    26
+#define WEBDAV        27
 
 #endif /* AMAYA_HELP_H */
 
