@@ -3,21 +3,13 @@
 // Used by amaya.rc
 //
 #define LINKDIALOG                      101
-#define EN_LINKDIALOG                   101
 #define PRINTDIALOG                     103
-#define EN_PRINTDIALOG                  103
 #define SEARCHDIALOG                    104
-#define EN_SEARCHDIALOG                 104
 #define CLOSEDOCDIALOG                  109
-#define EN_CLOSEDOCDIALOG               109
 #define OPENDOCDIALOG                   110
-#define EN_OPENDOCDIALOG                110
 #define CHARACTERSDIALOG                112
-#define EN_CHARACTERSDIALOG             112
 #define INITCONFIRMDIALOG               144
-#define EN_INITCONFIRMDIALOG            144
 #define FORMATDIALOG                    146
-#define EN_FORMATDIALOG                 146
 #define IDB_BINDENT1                    150
 #define IDB_BINDENT2                    151
 #define IDB_BSSMALL                     152
@@ -25,26 +17,14 @@
 #define IDB_BSLARGE                     154
 #define IDR_TOOLBAR1                    165
 #define SPELLCHECKDIALOG                169
-#define EN_SPELLCHECKDIALOG             169
 #define CREATERULEDIALOG                170
-#define EN_CREATERULEDIALOG             170
 #define APPLYCLASSDIALOG                171
-#define EN_APPLYCLASSDIALOG             171
 #define LANGUAGEDIALOG                  172
-#define EN_LANGUAGEDIALOG               172
-#define ALIGNDIALOG                     173
-#define ALIGN2DIALOG                    173
 #define ATTR5ITEMSDIALOG                173
-#define EN_ATTR5ITEMSDIALOG             173
 #define SAVEASDIALOG                    174
-#define EN_SAVEASDIALOG                 174
 #define AUTHENTIFICATIONDIALOG          175
-#define EN_AUTHENTIFICATIONDIALOG       175
-#define ALIGN1DIALOG                    176
 #define ATTR3ITEMSDIALOG                176
-#define EN_ATTR3ITEMSDIALOG             176
 #define HELPDIALOG                      181
-#define EN_HELPDIALOG                   181
 #define IDB_AMAYALOGO                   182
 #define IDB_MATH                        183
 #define IDB_MATHD                       184
@@ -61,7 +41,6 @@
 #define IDB_UDLR                        195
 #define IDB_SYM                         196
 #define MATHDIALOG                      199
-#define EN_MATHDIALOG                   199
 #define IDB_GKEY001                     200
 #define IDB_GKEY002                     201
 #define IDB_GKEY003                     202
@@ -223,23 +202,13 @@
 #define IDB_GKEY159                     361
 #define IDB_GKEY160                     362
 #define GALPHABETDIALOG                 363
-#define EN_GALPHABETDIALOG              363
 #define GETSAVEDIALOG                   364
-#define EN_GETSAVEDIALOG                364
 #define SAVELISTDIALOG                  365
-#define EN_SAVELISTDIALOG               365
 #define IDC_WINCURSOR                   376
 #define BGIMAGEDIALOG                   380
-#define EN_BGIMAGEDIALOG                380
-#define PRINTPROGRESSDLG                389
-#define IDI_ICON1                       392
-#define IDD_DIALOG1                     393
 #define CSSDIALOG                       395
-#define EN_CSSDIALOG                    395
 #define IDB_MATRIX                      397
-#define ALIGN3DIALOG                    398
 #define ATTR4ITEMSDIALOG                398
-#define EN_ATTR4ITEMSDIALOG             398
 #define IDB_GLINE                       401
 #define IDB_GRECT                       402
 #define IDB_GRRECT                      403
@@ -253,119 +222,59 @@
 #define IDB_GALPHA2                     411
 #define IDB_GGROUP                      412
 #define GRAPHICSDIALOG                  413
-#define EN_GRAPHICSDIALOG               413
 #define ATTR2ITEMSDIALOG                415
-#define EN_ATTR2ITEMSDIALOG             415
 #define TABLEDIALOG                     416
-#define EN_TABLEDIALOG                  416
 #define MATRIXDIALOG                    417
-#define EN_MATRIXDIALOG                 417
 #define OPENIMAGEDIALOG                 418
-#define EN_OPENIMAGEDIALOG              418
 #define GENERALMENU                     421
-#define EN_GENERALMENU                  421
-#define FR_TABLEDIALOG                  422
-#define FR_SPELLCHECKDIALOG             423
-#define FR_SEARCHDIALOG                 424
-#define FR_SAVELISTDIALOG               425
-#define FR_SAVEASDIALOG                 426
-#define FR_PRINTDIALOG                  427
-#define FR_OPENIMAGEDIALOG              428
-#define FR_OPENDOCDIALOG                429
-#define FR_MATRIXDIALOG                 430
-#define FR_MATHDIALOG                   431
-#define FR_LINKDIALOG                   432
-#define FR_LANGUAGEDIALOG               433
-#define FR_INITCONFIRMDIALOG            434
-#define FR_HELPDIALOG                   435
-#define FR_GRAPHICSDIALOG               436
-#define FR_GETSAVEDIALOG                437
-#define FR_GENERALMENU                  438
-#define FR_GALPHABETDIALOG              439
-#define FR_FORMATDIALOG                 440
-#define FR_CSSDIALOG                    441
-#define FR_CREATERULEDIALOG             442
-#define FR_CLOSEDOCDIALOG               443
-#define FR_CHARACTERSDIALOG             444
-#define FR_BGIMAGEDIALOG                445
-#define FR_AUTHENTIFICATIONDIALOG       446
-#define FR_ATTR5ITEMSDIALOG             447
-#define FR_ATTR4ITEMSDIALOG             448
-#define FR_ATTR3ITEMSDIALOG             449
-#define FR_ATTR2ITEMSDIALOG             450
-#define FR_APPLYCLASSDIALOG             451
-#define DE_APPLYCLASSDIALOG             452
-#define DE_ATTR2ITEMSDIALOG             453
-#define DE_ATTR3ITEMSDIALOG             454
-#define DE_ATTR4ITEMSDIALOG             455
-#define DE_ATTR5ITEMSDIALOG             456
-#define DE_AUTHENTIFICATIONDIALOG       457
-#define DE_BGIMAGEDIALOG                458
-#define DE_CHARACTERSDIALOG             459
-#define DE_CLOSEDOCDIALOG               460
-#define DE_CREATERULEDIALOG             461
-#define DE_CSSDIALOG                    462
-#define DE_FORMATDIALOG                 463
-#define DE_GALPHABETDIALOG              464
-#define DE_GENERALMENU                  465
-#define DE_GETSAVEDIALOG                466
-#define DE_GRAPHICSDIALOG               467
-#define DE_HELPDIALOG                   468
-#define DE_INITCONFIRMDIALOG            469
-#define DE_LANGUAGEDIALOG               470
-#define DE_LINKDIALOG                   471
-#define DE_MATHDIALOG                   472
-#define DE_MATRIXDIALOG                 473
-#define DE_OPENDOCDIALOG                474
-#define DE_OPENIMAGEDIALOG              475
-#define DE_PRINTDIALOG                  476
-#define DE_SAVEASDIALOG                 477
-#define DE_SAVELISTDIALOG               478
-#define DE_SEARCHDIALOG                 479
-#define DE_SPELLCHECKDIALOG             480
-#define DE_TABLEDIALOG                  481
-#define EN_PUBLISHMENU                  482
-#define EN_CACHEMENU                    485
-#define EN_PROXYMENU                    486
-#define DE_PUBLISHMENU                  493
-#define FR_PUBLISHMENU                  494
-#define FR_PROXYMENU                    495
-#define DE_PROXYMENU                    496
-#define DE_CACHEMENU                    497
-#define FR_CACHEMENU                    498
-#define EN_GEOMETRYMENU                 499
-#define FR_GEOMETRYMENU                 500
-#define DE_GEOMETRYMENU                 501
-#define EN_COLORMENU                    502
-#define FR_COLORMENU                    503
-#define DE_COLORMENU                    504
-#define EN_GETALTERNATEDIALOG           505
-#define FR_GETALTERNATEDIALOG           506
-#define DE_GETALTERNATEDIALOG           507
-#define EN_LANNEGMENU                   509
-#define FR_LANNEGMENU                   510
-#define DE_LANNEGMENU                   511
-#define EN_TEMPLATESMENU                513
-#define FR_TEMPLATESMENU                514
-#define DE_TEMPLATESMENU                515
-#define FR_PROFILEMENU                  518
-#define EN_PROFILEMENU                  519
-#define DE_PROFILEMENU                  520
+#define PUBLISHMENU                     482
+#define CACHEMENU                       485
+#define PROXYMENU                       486
+#define GEOMETRYMENU                    499
+#define COLORMENU                       502
+#define GETALTERNATEDIALOG              505
+#define LANNEGMENU                      509
+#define TEMPLATESMENU                   513
+#define PROFILEMENU                     519
+#define MATH_ENTITY_DLG                 534
+#define INITCONFIRM3LDIALOG             536
+#define IDD_DIALOG3                     537
+#define ATTR6ITEMSDIALOG                538
+#define ATTR7ITEMSDIALOG                539
+#define ATTR8ITEMSDIALOG                540
+#define ATTR9ITEMSDIALOG                541
+#define ANNOTMENU                       542
+#define INITCONFIRM3LDIALOG1            543
+#define ANNOTFILTERMENU                 544
+#define MAKEIDMENU                      545
+#define SAVEASDIALOG1                   546
+#define DOCINFOMENU                     547
+#define IDD_DIALOG1                     548
+#define IDD_DIALOG2                     549
+#define IDD_DIALOG4                     550
+#define IDC_CURSOR1                     551
+#define MIMETYPEDIALOG                  552
+#define CHARSETDIALOG                   553
+#define SAVEASDIALOG2                   554
+#define INITCONFIRMDIALOG1              555
+#define BROWSEMENU                      556
+#define PASTELIBRARYDIALOG              557
+#define ADDNEWMODELINTOLIBRARYDIALOG    558
+#define CREATELIBRARYDIALOG             559
+#define IDD_DIALOG5                     560
+#define CREATEHREF                      561
+#define NUMATTRDIALOG                   566
+#define TEXTATTRDIALOG                  567
+#define REQATTRDIALOG                   570
 #define ID_CONFIRM                      1000
 #define ID_DONE                         1001
 #define IDC_URLEDIT                     1002
+#define ID_CHANGEMIMETYPE               1002
+#define ID_CHANGECHARSET                1003
 #define ID_PRINT                        1011
 #define IDC_OPTIONS                     1012
-#define IDC_MANUALFEED                  1013
 #define IDC_TABOFCONTENTS               1014
 #define IDC_LINKS                       1015
-#define IDC_OUTPUT                      1016
-#define IDC_PRINTER                     1017
-#define IDC_POSTSCRIPT                  1018
-#define IDC_PRINTEREDIT                 1019
-#define IDC_PAPERFORMAT                 1020
-#define IDC_A4                          1021
-#define IDC_US                          1022
 #define ID_NOREPLACE                    1024
 #define IDC_SEARCHFOR                   1025
 #define IDC_SEARCHEDIT                  1026
@@ -376,7 +285,6 @@
 #define IDC_NOREPLACE                   1031
 #define IDC_ONREQUEST                   1032
 #define IDC_AUTOMATIC                   1033
-#define IDC_SEARCHWHERE                 1034
 #define IDC_BEFORE                      1035
 #define IDC_WITHIN                      1036
 #define IDC_AFTER                       1037
@@ -388,25 +296,22 @@
 #define IDC_URLMESSAGE                  1057
 #define ID_APPLY                        1058
 #define IDC_GETALT                      1058
-#define IDC_FONTFAMILY                  1059
+#define IDC_CLEAR                       1058
+#define IDC_CLEAR2                      1059
+#define IDC_CLICK                       1059
 #define IDC_TIMES                       1060
 #define IDC_HELVETICA                   1061
 #define IDC_COURIER                     1062
 #define IDC_DEFAULTFAMILY               1063
-#define IDC_STYLE                       1064
 #define IDC_ROMAN                       1065
 #define IDC_BOLD                        1066
 #define IDC_ITALIC                      1067
 #define IDC_OBLIQUE                     1068
-#define IDC_BOLDITALIC                  1069
-#define IDC_BOLDOBLIQUE                 1070
 #define IDC_DEFAULTSTYLE                1071
-#define IDC_UNDER_LINE                  1072
 #define IDC_NORMAL                      1073
 #define IDC_UNDERLINE                   1074
 #define IDC_CROSSOUT                    1075
-#define IDC_DEFAULTUNDERLINE            1076
-#define IDC_BODYSIZE                    1077
+#define IDC_OVERLINE                    1076
 #define IDC_06PT                        1079
 #define IDC_10PT                        1090
 #define IDC_08PT                        1091
@@ -418,32 +323,27 @@
 #define IDC_30PT                        1097
 #define IDC_40PT                        1098
 #define IDC_60PT                        1099
-#define IDC_ALIGN                       1099
 #define IDC_DEFAULTSIZE                 1100
 #define IDC_LEFT                        1100
 #define IDC_RIGHT                       1101
 #define IDC_CENTER                      1102
-#define IDC_DEFAULTALIGN                1103
+#define IDC_JUSTIFY                     1103
 #define IDC_BLEFT                       1104
 #define IDC_BCENTER                     1106
 #define IDC_BRIGHT                      1107
 #define IDC_INDENTPT                    1108
 #define IDC_INDENTPTVALUE               1109
-#define IDC_INDENTPTEDIT                1110
-#define IDC_INDENT                      1112
+#define IDC_DEFAULTALIGN                1110
+#define IDC_INDENTPTEDIT                1111
 #define IDC_INDENT1                     1113
 #define IDC_INDENT2                     1114
 #define IDC_INDENTDEFAULT               1115
 #define IDC_BINDENT1                    1116
 #define IDC_BINDENT2                    1118
-#define IDC_JUSTIFICATION               1119
-#define IDC_JUSTIFYES                   1120
-#define IDC_JUSTIFNO                    1121
-#define IDC_JUSTIFDEFAULT               1122
+#define IDC_BJUSTIFY                    1120
 #define IDC_LINESPACINGPT               1123
 #define IDC_LINESPACINGPTVALUE          1124
 #define IDC_LINESPACINGEDIT             1125
-#define IDC_LINESPACING                 1126
 #define IDC_SSMALL                      1127
 #define IDC_SMEDIUM                     1129
 #define IDC_SLARGE                      1130
@@ -455,24 +355,16 @@
 #define ID_REPLACEDIC                   1136
 #define ID_SKIPNEXT                     1138
 #define ID_SKIPDIC                      1139
-#define IDC_LISTWORDS                   1141
-#define IDC_EDITWORD                    1142
-#define IDC_CHECKING                    1143
 #define IDC_NBPROPOSALS                 1150
 #define IDC_PROPOSALS                   1151
 #define IDC_EDITPROPOSALS               1152
-#define IDC_IGNORE                      1153
 #define IDC_IGNORE1                     1154
 #define IDC_IGNORE2                     1155
 #define IDC_IGNORE3                     1156
 #define IDC_IGNORE4                     1157
 #define IDC_EDITIGNORE                  1158
 #define ID_DELETE                       1162
-#define IDC_TOP                         1163
-#define IDC_MIDDLE                      1165
-#define IDC_BOTTOM                      1166
 #define ID_CLEAR                        1170
-#define IDC_OUTPUTFORMAT                1174
 #define IDC_HTML                        1175
 #define IDC_TEXT                        1176
 #define IDC_COPYIMG                     1177
@@ -481,16 +373,12 @@
 #define IDC_EDITDOCSAVE                 1180
 #define IDC_IMGLOCATION                 1181
 #define IDC_EDITIMGSAVE                 1182
-#define IDC_NAME                        1183
 #define IDC_NAMEEDIT                    1184
-#define IDC_PASSWD                      1185
 #define IDC_PASSWDEDIT                  1186
-#define IDC_ALIGN2                      1188
 #define IDC_AUTHSERVER                  1189
 #define IDC_AUTHREALM                   1190
 #define IDC_LOGO                        1192
 #define IDC_MATH                        1193
-#define IDC_MATHD                       1194
 #define IDC_ROOT                        1195
 #define IDC_SROOT                       1196
 #define IDC_DIV                         1197
@@ -672,15 +560,8 @@
 #define IDC_REPEATX                     1382
 #define IDC_REPEATY                     1383
 #define IDC_NOREPEAT                    1384
-#define IDC_ENCODING                    1386
-#define IDC_ISO_LATIN_1                 1387
-#define IDC_ISO_LATIN_2                 1388
-#define IDC_ISO_LATIN_9                 1389
-#define IDC_SYMBOL_ENCOD                1390
-#define IDC_ABORTPRNT                   1399
 #define IDC_XML                         1403
 #define IDC_GRRECT                      1404
-#define IDC_JUSTIFY                     1405
 #define IDC_GELLIPSE                    1405
 #define IDC_GLINE                       1406
 #define IDC_GCPOLYLINE                  1407
@@ -698,20 +579,24 @@
 #define IDC_IGNORE_CSS                  1412
 #define IDC_GPOLYLINE                   1413
 #define IDC_NUMROWS                     1413
-#define IDC_OVERLINE                    1413
+#define IDC_DEFAULTUNDERLINE            1413
 #define IDC_GALPHA2                     1414
 #define IDC_BORDER                      1414
-#define IDC_RICHEDIT1                   1414
 #define IDC_GGROUP                      1415
 #define ID_DEFAULTS                     1417
 #define IDC_HOMEPAGE                    1419
-#define IDC_FONTMENUSIZE                1420
 #define IDC_ZOOM                        1421
 #define IDC_DIALOGUELANG                1422
 #define IDC_MULTIKEY                    1423
+#define IDC_LINES                       1423
 #define IDC_BGIMAGES                    1424
+#define IDC_AUTOSAVE                    1424
 #define IDC_DOUBLECLICK                 1425
-#define IDC_TAB1                        1427
+#define IDC_SAVE_GEOMETRY_EXIT          1425
+#define IDC_ENABLEFTP                   1426
+#define IDC_SHOWADDRESS                 1427
+#define IDC_SHOWTARGET                  1428
+#define IDC_CRLF                        1429
 #define IDC_LOSTUPDATECHECK             1430
 #define IDC_NOPROXY                     1430
 #define IDC_VERIFYPUBLISH               1431
@@ -719,6 +604,7 @@
 #define IDC_ENABLECACHE                 1433
 #define IDC_SAFEPUTREDIRECT             1433
 #define IDC_CACHEPROTECTEDDOCS          1434
+#define IDC_USEXHTMLMIMETYPE            1434
 #define IDC_CACHEDISCONNECTEDMODE       1435
 #define IDC_CACHEEXPIREIGNORE           1436
 #define IDC_CACHEDIRECTORY              1437
@@ -731,9 +617,12 @@
 #define IDC_FGCOLOR                     1450
 #define IDC_BGCOLOR                     1451
 #define IDC_ALTTEXT                     1452
-#define IDC_FGCOLOR2                    1452
-#define IDC_BGCOLOR2                    1453
+#define IDC_SECOLOR                     1452
+#define IDC_FGSELCOLOR                  1452
+#define IDC_INCOLOR                     1453
+#define IDC_BGSELCOLOR                  1453
 #define IDC_CHANGCOLOR                  1454
+#define IDC_CHANGCOLOR2                 1455
 #define IDC_LANNEG                      1456
 #define IDC_NORMALBOLD                  1457
 #define IDC_DEFAULTBOLD                 1459
@@ -742,20 +631,146 @@
 #define IDC_PROFILENAME                 1463
 #define IDC_PROXYDOMAIN                 1464
 #define IDC_ONLYPROXY                   1465
+#define IDC_FILTERBYSERVER              1466
 #define IDC_ABOUT2                      1469
+#define IDC_EDIT_NAME                   1470
 #define IDC_VERSION                     1471
+#define IDC_ENTITY_NAME                 1471
 #define IDC_ABOUT1                      1472
+#define IDC_IDC_TCACHEDIRECTORY         1472
+#define IDC_TCACHESIZE                  1473
+#define IDC_TMAXCACHEFILE               1474
+#define IDC_THTTPPROXY                  1475
+#define IDC_TPROXYDOMAIN                1476
+#define IDC_TSEPENTRIESSPACE            1477
+#define IDC_TAPPHOME                    1478
+#define IDC_TTMPDIR                     1479
+#define IDC_THOMEPAGE                   1480
+#define IDC_TZOOM                       1481
+#define IDC_TDIALOGUELANG               1482
+#define IDC_TDEFAULTNAME                1483
+#define IDC_TSAFEPUTREDIRECT            1484
+#define IDC_TFGCOLOR                    1485
+#define IDC_TBGCOLOR                    1486
+#define IDC_GEOMCHANGE                  1487
+#define IDC_TSECOLOR                    1487
+#define IDC_TFGSELCOLOR                 1487
+#define IDC_TPROFILESLOCA               1488
+#define IDC_TINCOLOR                    1488
+#define IDC_TBGSELCOLOR                 1488
+#define IDC_TPROFILESELECT              1489
+#define IDC_TPROFILECHANGE              1490
+#define IDC_CSSFILES                    1491
+#define IDC_MESSAGE1                    1492
+#define IDC_MESSAGE2                    1493
+#define IDC_MESSAGE3                    1494
+#define IDC_CLOSEMSG                    1495
+#define IDC_LANGELEM                    1496
+#define IDC_INHERITEDLANG               1497
+#define IDC_LNGEDIT                     1498
+#define IDC_UNDERLINEGROUP              1499
+#define IDC_FONTFAMILYGROUP             1500
+#define IDC_CHARSTYLEGROUP              1501
+#define IDC_BOLDGROUP                   1502
+#define IDC_BODYSIZEGROUP               1503
+#define IDC_LABEL                       1505
+#define IDC_CURWORD                     1506
+#define IDC_LANG                        1507
+#define IDC_URL_TEXT                    1510
+#define IDC_ANNOTUSER                   1511
+#define IDC_TANNOTUSER                  1512
+#define IDC_ANNOTSERVERS                1513
+#define IDC_TANNOTSERVERS               1514
+#define IDC_TANNOTPOSTSERVER            1515
+#define IDC_ANNOTPOSTSERVER             1516
+#define IDC_TANNOTAUTOLOAD              1518
+#define IDC_ANNOTAUTOLOAD               1519
+#define IDC_ANNOTLAUTOLOAD              1519
+#define IDC_ANNOTRAUTOLOAD              1520
+#define ID_ANNOTSHOW                    1521
+#define IDC_ANNOTRAUTOLOADRST           1521
+#define ID_ANNOTHIDE                    1522
+#define ID_ANNOTSHOWALL                 1523
+#define ID_ANNOTHIDEALL                 1524
+#define IDC_FILTERBYAUTHOR              1525
+#define IDC_FILTERBYTYPE                1526
+#define IDC_TFILTEROPTIONS              1528
+#define IDC_TFILTERHIDDEN               1529
+#define IDC_TFILTERHELP                 1529
+#define IDC_FILTERSEL                   1531
+#define ID_CREATEID                     1533
+#define ID_REMOVEID                     1534
+#define IDC_IDC_TIDELEMNAME             1535
+#define IDC_IDELEMNAME                  1536
+#define IDC_IDAPPLYTODOC                1538
+#define IDC_IDAPPLYTOSEL                1539
+#define IDC_IDSTATUS                    1540
+#define IDC_TAUTHREALM                  1542
+#define IDC_TAUTHSERVER                 1543
+#define IDC_TAUTHINFO                   1544
+#define IDC_TPASSWDEDIT                 1545
+#define IDC_TNAMEEDIT                   1546
+#define IDC_AALT                        1547
+#define IDC_ACTRL                       1548
+#define IDC_ANONE                       1549
+#define IDC_ACCESSKEY                   1550
+#define IDC_DIURL                       1551
+#define IDC_DICONTENTTYPE               1552
+#define IDC_DICHARSET                   1553
+#define IDC_DICONTENTLEN                1554
+#define IDC_DIDTYPE                     1555
+#define IDC_DICONTENTTYPE_VAL           1556
+#define IDC_DICHARSET_VAL               1557
+#define IDC_DICONTENTLEN_VAL            1558
+#define IDC_DIURL_VAL                   1559
+#define IDC_PAGENO                      1560
+#define IDC_DICONTENTLOC                1560
+#define IDC_CHARSET                     1561
+#define IDC_DICONTENTLOC_VAL            1561
+#define IDC_MIMETYPE                    1562
+#define IDC_DIDTYPE_VAL                 1562
+#define IDC_MTEDIT                      1564
+#define IDC_STATUS                      1566
+#define IDC_USASCII                     1567
+#define IDC_UTF8                        1568
+#define IDC_ISOL1                       1569
+#define ID_SHOW                         1570
+#define ID_MIDDLE                       1570
+#define IDC_LOADIMG                     1572
+#define IDC_SHOWBUTTONS                 1573
+#define IDC_LOADCSS                     1573
+#define IDC_NUMBER                      1574
+#define IDC_LOADOBJ                     1574
+#define ID_COPYSVGLIB                   1575
+#define ID_REFERSVGLIB                  1576
+#define ID_CREATELIBRARY                1577
+#define IDC_LIBRARYTITLE                1579
+#define ID_BROWSELIBRARY                1580
+#define ID_GETLIBRARYURL                1582
+#define ID_GETLIBRARYTITLE              1583
+#define ID_LIBRARYURL                   1584
+#define ID_LIBRARYTITLE                 1585
+#define IDC_LIBRARYLIST                 1586
+#define IDC_SCREEN                      1587
+#define IDC_SCREENLIST                  1588
+#define IDC_PROFILELIST                 1589
+#define IDC_CSSLIST                     1590
+#define IDC_LANGLIST                    1591
+#define IDC_ATTRNAME                    1595
+#define ID_EDITVALUE                    1596
+#define IDC_CHARSET_TITLE               1599
+#define IDC_CHARSET_LIST                1600
+#define IDC_WORDLIST                    1601
+#define IDC_WORDEDIT                    1602
 #define ID_BUTTON40017                  40017
 #define ID_BUTTON40018                  40018
 #define ID_BUTTON40019                  40019
 #define ID_BUTTON40020                  40020
-#define ID_BUTTON40021                  40021
 #define ID_BUTTON40022                  40022
 #define ID_BUTTON40023                  40023
 #define ID_BUTTON40024                  40024
 #define ID_BUTTON40025                  40025
 #define ID_BUTTON40026                  40026
-#define ID_BUTTON40027                  40027
 #define ID_BUTTON40028                  40028
 #define ID_BUTTON40029                  40029
 #define ID_BUTTON40030                  40030
@@ -765,7 +780,6 @@
 #define ID_BUTTON40034                  40034
 #define ID_BUTTON40035                  40035
 #define ID_BUTTON40036                  40036
-#define ID_BUTTON40037                  40037
 #define ID_BUTTON40038                  40038
 #define ID_BUTTON40039                  40039
 #define ID_BUTTON40040                  40040
@@ -777,15 +791,24 @@
 #define ID_BUTTON40049                  40049
 #define ID_BUTTON40050                  40050
 #define ID_BUTTON40051                  40051
-#define IDC_URL                         -1
+#define IDC_CHECKGROUP                  65500
+#define IDC_IGNOREGROUP                 65501
+#define IDC_INDENTGROUP                 65530
+#define IDCALIGNGROUP                   65531
+#define IDC_JUSTIFGROUP                 65532
+#define IDC_LINESPACEGROUP              65533
+#define IDC_TLANNEG                     65535
+#define IDC_TTEMPLATESURL               65535
+#define IDC_WHEREGROUP                  65535
+#define IDC_OUTPUTGROUP                 65535
 
 // Next default values for new objects
 // 
 #ifdef APSTUDIO_INVOKED
 #ifndef APSTUDIO_READONLY_SYMBOLS
-#define _APS_NEXT_RESOURCE_VALUE        534
+#define _APS_NEXT_RESOURCE_VALUE        571
 #define _APS_NEXT_COMMAND_VALUE         40052
-#define _APS_NEXT_CONTROL_VALUE         1470
+#define _APS_NEXT_CONTROL_VALUE         1603
 #define _APS_NEXT_SYMED_VALUE           101
 #endif
 #endif
