@@ -1,6 +1,6 @@
 /*
  *
- *  COPYRIGHT MIT and INRIA, 1996-2001.
+ *  COPYRIGHT INRIA and W3C, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -25,7 +25,6 @@
 #define META_TEMPLATE_NAME "AMAYA_TEMPLATE"
 /* URL of the script providing templates (for reload) */
 static char   *script_URL;
-
 #include "init_f.h"
 
 
@@ -34,12 +33,19 @@ static char   *script_URL;
   ----------------------------------------------------------------------*/
 void NewTemplate (Document doc, View view)
 {
-  char      *url;
+  char       *url;
 
   url = TtaGetEnvString ("TEMPLATE_URL");
-  if (url)
-    GetHTMLDocument (url, NULL, 0, 0, CE_ABSOLUTE, FALSE, NULL, NULL);
+  if (url && TtaCheckDirectory (url))
+    {
+      /*TtaNewScrollPopup (BaseDialog + xxx, TtaGetViewFrame (doc, 1),
+			 NULL, nbitems, FormBuf, NULL, multipleOptions, 'L');
+      InitOpenDocForm (doc, view, "New.html",
+      "xxx", docHTML);*/
+      /*GetAmayaDoc (url, NULL, 0, 0, CE_ABSOLUTE, FALSE, NULL, NULL);*/
+    }
 }
+
 /*----------------------------------------------------------------------
    OpenTemplateDocument: Process the meta of a template document,
      changes the URL, try to save it and warn the user if it cannot be
@@ -52,11 +58,9 @@ void OpenTemplateDocument (Document doc)
   Attribute           metaAttr;
   AttributeType       metaAttrType;
   ThotBool            found = FALSE;
-  ThotBool            ok = FALSE;
   int                 length;
-  char              buffer[MAX_LENGTH];
-  char             *URLdir;
-  char             *URLname;
+  char               *utf8path, *path;
+  char               *URLdir, *URLname;
 
   metaElType.ElSSchema = TtaGetDocumentSSchema (doc);
   metaElType.ElTypeNum = HTML_EL_META;
@@ -64,6 +68,7 @@ void OpenTemplateDocument (Document doc)
   metaAttrType.AttrTypeNum = HTML_ATTR_meta_name;
   /* search the meta element */
   metaEl = TtaGetMainRoot (doc);
+  path = NULL;
   do
     {
       metaEl = TtaSearchTypedElement (metaElType, SearchForward, metaEl);
@@ -72,13 +77,18 @@ void OpenTemplateDocument (Document doc)
           metaAttr = TtaGetAttribute (metaEl, metaAttrType); 
           if (metaAttr != NULL)
             {
-              length = MAX_LENGTH - 1;
- 	      TtaGiveTextAttributeValue (metaAttr, buffer, &length);
-              found = !strcmp (buffer, META_TEMPLATE_NAME);
+	      length = TtaGetTextAttributeLength (metaAttr) + 1;
+	      utf8path = (char *) TtaGetMemory (length);
+ 	      TtaGiveTextAttributeValue (metaAttr, utf8path, &length);
+	      path = (char *)TtaConvertMbsToByte ((unsigned char *)utf8path,
+						  TtaGetDefaultCharset ());
+	      TtaFreeMemory (utf8path);
+              found = !strcmp (path, META_TEMPLATE_NAME);
             }
         }
     }
-  while (metaEl != NULL && !found);
+  while (metaEl && !found);
+
   if (found)
     {
       /* get the url of the document */
@@ -86,42 +96,32 @@ void OpenTemplateDocument (Document doc)
       metaAttr = TtaGetAttribute (metaEl, metaAttrType);
       if (metaAttr != NULL)
 	{
-	  length = MAX_LENGTH - 1;
-	  TtaGiveTextAttributeValue (metaAttr, buffer, &length);
+	  length = TtaGetTextAttributeLength (metaAttr) + 1;
+	  utf8path = (char *) TtaGetMemory (length);
+	  TtaGiveTextAttributeValue (metaAttr, utf8path, &length);
+	  path = (char *)TtaConvertMbsToByte ((unsigned char *)utf8path,
+					      TtaGetDefaultCharset ());
+	  TtaFreeMemory (utf8path);
 	}
+
       /* Delete the meta element */
       TtaDeleteTree (metaEl, doc);  
-      /* try to save to the new url */
-#ifdef WITH_PRELABLE_SAVE_POUR_VOIR      
-      ok = SaveDocumentThroughNet (doc, 1, buffer, FALSE, TRUE, TRUE);
-#else
-      ok = TRUE;
-#endif
-      if (ok)
-        {
-          if (script_URL == NULL)
-            script_URL = TtaStrdup (DocumentURLs[doc]);
-	  TtaFreeMemory (DocumentURLs[doc]);
-	  DocumentURLs[doc] = TtaStrdup (buffer);
-          DocumentMeta[doc]->method = CE_TEMPLATE;
-	  TtaSetTextZone (doc, 1, 1, DocumentURLs[doc]);
-	  TtaSetDocumentUnmodified (doc);
-          
-	}
-      else 
-	{
-	  /* save failed : print warning message */
-         
-	}
-	
-      /* set the document name and dir */
+      if (script_URL == NULL)
+	script_URL = TtaStrdup (DocumentURLs[doc]);
+      TtaFreeMemory (DocumentURLs[doc]);
+      DocumentURLs[doc] = TtaStrdup (path);
+      DocumentMeta[doc]->method = CE_TEMPLATE;
+      AddURLInCombobox (DocumentURLs[doc], NULL, TRUE);
+      TtaSetTextZone (doc, 1, URL_list);
+      TtaSetDocumentUnmodified (doc);
       
-      URLname = strrchr (buffer, URL_SEP);
+      /* set the document name and dir */
+      URLname = strrchr (path, URL_SEP);
       if (URLname)
 	{
 	  URLname[0] = EOS;
 	  URLname++;
-	  URLdir = buffer;
+	  URLdir = path;
 	  TtaSetDocumentDirectory (doc, URLdir);
 	  TtaSetDocumentName (doc, URLname);
 	  /* SetBrowserEditor(doc); */ 
@@ -129,11 +129,11 @@ void OpenTemplateDocument (Document doc)
       else
 	{
 	  TtaSetDocumentDirectory (doc, "");
-	  TtaSetDocumentName (doc, buffer);
+	  TtaSetDocumentName (doc, path);
 	}
       SetWindowTitle (doc, doc, 0);
     }
-    
+  TtaFreeMemory (path);
 }
   
 /*----------------------------------------------------------------------
