@@ -29,7 +29,7 @@
 #include "HTMLhistory_f.h"
 
 /* windows includes */
-#ifdef _WINDOWS
+#ifdef _WINGUI
 #include "resource.h"
 #include "wininclude.h"
 
@@ -37,7 +37,7 @@
 #include "appdialogue.h"
 
 extern HINSTANCE hInstance;
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
 
 /* schema includes */
 #include "XLink.h"
@@ -67,7 +67,7 @@ static SelType  AnnotSelType;
 
 static List     *typesList;
 
-#ifdef _WINDOWS
+#ifdef _WINGUI
 static HWND       FilterHwnd = NULL;
 #endif /* WINDOWS */
 
@@ -77,7 +77,12 @@ typedef struct _typeSelector
   RDFResourceP type;
 } TypeSelector;
 
-#ifndef _WINDOWS
+#if defined(_WX) && !defined(__STDC__)
+/* SG compilation fix under wxWindows */
+#define __STDC__
+#endif /* #if defined(_WX) && !defined(__STDC__) */
+
+#ifndef _WINGUI
 /*----------------------------------------------------------------------
   CustomQueryCallbackDialog
   callback of the annot custom query menu
@@ -166,7 +171,7 @@ char              *data;
 	}
     }
 }
-#endif /* !_WINDOWS */
+#endif /* !_WINGUI */
 
 /*----------------------------------------------------------------------
   CustomQueryMenuInit
@@ -180,13 +185,13 @@ Document     document;
 View         view;
 #endif /* __STDC__ */
 {
-#ifndef _WINDOWS
+#ifndef _WINGUI
    int              i;
    char          *ptr;
 
    /* initialize the base if it hasn't yet been done */
    if (!CustomQueryBase)
-     CustomQueryBase = TtaSetCallback (CustomQueryCallbackDialog, 
+     CustomQueryBase = TtaSetCallback ((Proc)CustomQueryCallbackDialog, 
 				       MAX_QUERYCONFMENU_DLG);
 
    /* Create the dialogue form */
@@ -283,7 +288,7 @@ View         view;
    TtaSetDialoguePosition ();
    TtaShowDialogue (CustomQueryBase + CustomQueryMenu, TRUE);
 
-#endif /* !_WINDOWS */
+#endif /* !_WINGUI */
 }
 
 /**************************************************
@@ -297,7 +302,7 @@ View         view;
   Does the equivalent of the Thotlib selector for the Annotation
   Filter.
   ------------------------------------------------------------------*/
-#ifdef _WINDOWS
+#ifdef _WINGUI
 static void WIN_AnnotFilterNewSelector (Document doc, char *entries, int nb_entries)
 {
   int index = 0;
@@ -315,7 +320,7 @@ static void WIN_AnnotFilterNewSelector (Document doc, char *entries, int nb_entr
       i++;
     }
 }
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
 
 /*---------------------------------------------------------------
   BuildAnnotFilterSelector builds the list allowing to select a profile
@@ -394,9 +399,9 @@ SelType  selector;
        list_item = list_item->next;
      }
 
-#ifdef _WINDOWS
+#ifdef _WINGUI
   WIN_AnnotFilterNewSelector (doc, s, nb_entries);
-#else /* _WINDOWS */
+#else /* _WINGUI */
   /* Fill in the form  */
   TtaNewSelector (AnnotFilterBase + mFilterSelector, 
 		  AnnotFilterBase + AnnotFilterMenu,
@@ -407,7 +412,7 @@ SelType  selector;
 		  NULL,
 		  TRUE,
 		  TRUE);
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
 }
 
 /*---------------------------------------------------------------
@@ -517,7 +522,7 @@ ThotBool show;
 	/* this looks like an error! */
 	continue;
       length = TtaGetTextAttributeLength (attr) + 1;
-      annot_url = TtaGetMemory (length);
+      annot_url = (char *)TtaGetMemory (length);
       TtaGiveTextAttributeValue (attr, annot_url, &length);
       
       /* now look in the filters to see if we need to hide it or not */
@@ -529,7 +534,7 @@ ThotBool show;
 
 #ifdef ANNOT_ON_ANNOT
       /* skip the reply annotations */
-      if (annot->isReplyTo)
+      if (annot->inReplyTo)
 	continue;
 #endif /* ANNOT_ON_ANNOT */
       
@@ -590,9 +595,9 @@ ThotBool show;
     AnnotSelItem[0] = ' ';
   else
     AnnotSelItem[0] = ' ';
-#ifndef _WINDOWS
+#ifndef _WINGUI
   TtaSetSelector (AnnotFilterBase + mFilterSelector, -1, AnnotSelItem);
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
 }
 
 /*----------------------------------------------------------------------
@@ -636,7 +641,7 @@ ThotBool show;
   list_item = AnnotMetaData[document].annotations;
   for (; list_item; list_item = list_item->next)
     {
-      AnnotMeta *annot = list_item->object;
+      AnnotMeta *annot = (AnnotMeta *)list_item->object;
       if (annot && annot->is_visible && !(annot->is_orphan))
 	annot->show = show;
     }
@@ -694,12 +699,12 @@ ThotBool show;
   BuildAnnotFilterSelector (document, AnnotSelType);
   /* and clear the selector text */
   AnnotSelItem[0] = EOS;
-#ifndef _WINDOWS
+#ifndef _WINGUI
   TtaSetSelector (AnnotFilterBase + mFilterSelector, -1, "");
-#endif /* !_WINDOWS */
+#endif /* !_WINGUI */
 }
 
-#ifdef _WINDOWS
+#ifdef _WINGUI
 /*-----------------------------------------------------------------------
  AnnotFilterDlgProc
  ------------------------------------------------------------------------*/
@@ -849,7 +854,7 @@ LPARAM lParam;
     }
   return TRUE;
 }
-#else /* _WINDOWS */
+#else /* _WINGUI */
 
 /*----------------------------------------------------------------------
    callback of the AnnotFilter menu
@@ -922,7 +927,7 @@ char             *data;
 	    {
 	      AnnotSelType = (SelType) val;
 	      AnnotSelItem[0] = EOS;
-	      BuildAnnotFilterSelector (AnnotFilterDoc, val);
+	      BuildAnnotFilterSelector (AnnotFilterDoc, (SelType)val);
 	      TtaSetSelector (AnnotFilterBase + mFilterSelector, -1, "");
 	    }
 	  break;
@@ -932,7 +937,7 @@ char             *data;
 	}
     }
 }
-#endif /* !_WINDOWS */
+#endif /* !_WINGUI */
 
 /*----------------------------------------------------------------------
   AnnotFilter
@@ -947,20 +952,20 @@ View                view;
 #endif /* __STDC__*/
 {
 	/* local variables */
-#ifndef _WINDOWS
+#ifndef _WINGUI
   int              i;
-#endif /* !_WINDOWS */
+#endif /* !_WINGUI */
 
   /* build the filter structures from the downloaded 
      annotation info */
   AnnotFilter_build (document);
 
-#ifndef _WINDOWS
+#ifndef _WINGUI
   /* initialize the base if it hasn't yet been done */
   if (AnnotFilterBase == 0)
-    AnnotFilterBase =  TtaSetCallback (AnnotFilterCallbackDialog,
+    AnnotFilterBase =  TtaSetCallback ((Proc)AnnotFilterCallbackDialog,
 					 MAX_ANNOTFILTER_DLG);
-#endif /* !_WINDOWS */
+#endif /* !_WINGUI */
 
   /* make a copy of the current document and view, so that we can
      find this info in the callback */
@@ -968,10 +973,10 @@ View                view;
   AnnotFilterView = view;
   AnnotSelItem[0] = EOS;
   AnnotSelIndex = -1;
-  AnnotSelType = 0;
+  AnnotSelType = (SelType)0;
 
   /* Create the dialogue form */
-#ifndef _WINDOWS
+#ifndef _WINGUI
   i = 0;
   strcpy (&s[i], TtaGetMessage (AMAYA, AM_AFILTER_SHOW));
   i += strlen (&s[i]) + 1;
@@ -1023,19 +1028,19 @@ View                view;
 
   /* choose the BY_AUTHOR radio button */
   TtaSetMenuForm (AnnotFilterBase + mSelectFilter, 0);
-#endif /* !_WINDOWS */
+#endif /* !_WINGUI */
 
   /* display the menu */
-#ifndef _WINDOWS
+#ifndef _WINGUI
   TtaSetDialoguePosition ();
   TtaShowDialogue (AnnotFilterBase + AnnotFilterMenu, TRUE);
-#else /* !_WINDOWS */
+#else /* !_WINGUI */
   if (!FilterHwnd)
     /* only activate the menu if it isn't active already */
      DialogBox (hInstance, MAKEINTRESOURCE (ANNOTFILTERMENU), NULL, (DLGPROC) WIN_AnnotFilterDlgProc);
   else
      SetFocus (FilterHwnd);
-#endif /* !_WINDOWS */
+#endif /* !_WINGUI */
 }
 
 /***************************************************
@@ -1064,13 +1069,18 @@ Document doc;
   strcpy (s, "");
   i = 0;
 
-  annotClass = ANNOTATION_CLASS;
+#ifdef ANNOT_ON_ANNOT
+  if (Annot_IsReplyTo (doc))
+    annotClass = THREAD_REPLY_CLASS;
+  else
+#endif /* ANNOT_ON_ANNOT */
+    annotClass = ANNOTATION_CLASS;
 
-  if (annotClass && annotClass->class)
+  if (annotClass && annotClass->class_)
     {
       List *item;
 
-      for (item=annotClass->class->subClasses; item; item=item->next)
+      for (item=annotClass->class_->subClasses; item; item=item->next)
 	{
 	  RDFClassP subType = (RDFClassP) item->object;
 	  TypeSelector *t = (TypeSelector *) TtaGetMemory (sizeof(TypeSelector));
@@ -1130,12 +1140,13 @@ View                view;
   /* activate the menu that has just been created */
   ReturnOption = -1;
   ReturnOptionMenu = -1;
-#ifndef _WINDOWS
+#ifndef _WINGUI
   TtaSetDialoguePosition ();
-#endif /* !_WINDOWS */
+#endif /* !_WINGUI */
   TtaShowDialogue (BaseDialog + OptionMenu, FALSE);
   /* wait for an answer from the user */
   TtaWaitShowDialogue ();
+  TtaDestroyDialogue (BaseDialog + OptionMenu);
   if (ReturnOption >= 0 && ReturnOptionMenu >= 0) 
     {
       /* make the returned option selected */
