@@ -14,11 +14,16 @@
  * the application and must be freed as an entire unit.  This
  * permits the use of simple (memory address) handles rather
  * than strings (URIs) for Resources.
+
+ * NOTE: the code will use the libwww RDF parser by default; if
+ *    RAPTOR_RDF_PARSER is defined the Raptor RDF parser:
  *
- * NOTE: the code assumes libwww's RDF parser.
+ *     http://www.redland.opensource.ac.uk/raptor/
+ *
+ *    will be used.
  *
  * Author: Ralph R. Swick (W3C/MIT)
- * $Revision: 1.22 $
+ * $Revision: 1.33 $
  *
  */
 
@@ -33,9 +38,17 @@
 #include "ANNOTtools_f.h"
 #include "ANNOTevent_f.h"
 
+/* RDF parser */
+#ifdef AM_REDLAND
+#include "librdf.h"
+#endif /* AM_REDLAND */
+#ifdef RAPTOR_RDF_PARSER
+#include "raptor.h"
+#else
 /* libwww includes */
 #include "xmlparse.h"
 #include "HTRDF.h"
+#endif
 
 /* amaya includes */
 #include "AHTURLTools_f.h"
@@ -60,14 +73,29 @@ char *ANNOTATION_CLASSNAME = NULL;
 RDFPropertyP PROP_name = NULL;
 RDFPropertyP PROP_firstName = NULL;
 RDFPropertyP PROP_Email = NULL;
+RDFPropertyP PROP_usesIcon = NULL;
 RDFClassP ANNOTATION_CLASS;
 RDFClassP DEFAULT_ANNOTATION_TYPE;
 
+#ifdef ANNOT_ON_ANNOT
+RDFClassP THREAD_REPLY_CLASS;
+RDFClassP DEFAULT_REPLY_TYPE;
+#endif /* ANNOT_ON_ANNOT */
+
+
 typedef struct _ReadCallbackContext
 {
   char filename[MAX_LENGTH];
 } ReadCallbackContext;
 
+typedef struct _ParseContext
+{
+  List **annot_schema_list;
+#ifdef RAPTOR_RDF_PARSER
+  char *base_uri;   /* base URI for anonymous RDF names */
+#endif
+} ParseContext, *ParseContextP;
+
 /*------------------------------------------------------------
    _ListSearchResource
   ------------------------------------------------------------*/
@@ -95,14 +123,7 @@ static RDFResourceP _ListSearchResource( List* list, char* name)
 /*------------------------------------------------------------
    SCHEMA_AddStatement
   ------------------------------------------------------------*/
-#ifdef  __STDC__
 void SCHEMA_AddStatement (RDFResourceP s, RDFPropertyP p, RDFResourceP o)
-#else  /* __STDC__ */
-void SCHEMA_AddStatement (s, p, o)
-RDFResourceP s;
-RDFPropertyP p;
-RDFResourceP o;
-#endif
 {
   RDFStatement *statement;
 
@@ -119,30 +140,30 @@ RDFResourceP o;
 /*------------------------------------------------------------
    _AddInstance
   ------------------------------------------------------------*/
-static void _AddInstance( RDFClassP class, RDFResourceP instance )
+static void _AddInstance( RDFClassP c, RDFResourceP instance )
 {
-  if (!class->class)
+  if (!c->class_)
     {
-      class->class = (RDFClassExtP) TtaGetMemory (sizeof(RDFClassExt));
-      memset (class->class, 0, sizeof (RDFClassExt));
+      c->class_ = (RDFClassExtP) TtaGetMemory (sizeof(RDFClassExt));
+      memset (c->class_, 0, sizeof (RDFClassExt));
     }
 
-  List_add (&class->class->instances, (void*)instance);
+  List_add (&c->class_->instances, (void*)instance);
 }
 
 /*------------------------------------------------------------
    _AddSubClass
   ------------------------------------------------------------*/
-static void _AddSubClass( RDFClassP class, RDFClassP sub )
+static void _AddSubClass( RDFClassP c, RDFClassP sub )
 {
-  if (!class->class)
+  if (!c->class_)
     {
-      class->class = (RDFClassExtP)TtaGetMemory (sizeof(RDFClassExt));
-      class->class->instances = NULL;
-      class->class->subClasses = NULL;
+      c->class_ = (RDFClassExtP)TtaGetMemory (sizeof(RDFClassExt));
+      c->class_->instances = NULL;
+      c->class_->subClasses = NULL;
     }
 
-  List_add (&class->class->subClasses, (void*)sub);
+  List_add (&c->class_->subClasses, (void*)sub);
 }
 
 /* ------------------------------------------------------------
@@ -158,20 +179,79 @@ static void _AddSubClass( RDFClassP class, RDFClassP sub )
      triple - an RDF triple
      context - pointer to an RDFResource list
  ------------------------------------------------------------*/
+#ifdef RAPTOR_RDF_PARSER
+static void triple_handler (void * context, const raptor_statement *triple)
+#else
 static void triple_handler (HTRDF * rdfp, HTTriple * triple, void * context)
+#endif
 {
-  List **listP = (List**)context;
 
+#ifdef RAPTOR_RDF_PARSER
+  ParseContextP parse_ctx = (ParseContextP) context;
+  List **listP = parse_ctx->annot_schema_list;
+  char test;
+#else
+  List **listP = (List**) context;
+#endif
+
+#ifdef RAPTOR_RDF_PARSER
+  if (triple) 
+    {
+      char * predicate = AM_RAPTOR_URI_AS_STRING(triple->predicate);
+      char * subject = NULL;
+      char * object = NULL;
+#else
   if (rdfp && triple) 
     {
       char * predicate = HTTriple_predicate(triple);
       char * subject = HTTriple_subject(triple);
       char * object = HTTriple_object(triple);
+#endif
 
-      RDFResourceP subjectP = ANNOT_FindRDFResource (listP, subject, TRUE);
+      RDFResourceP subjectP;
       RDFPropertyP predicateP = ANNOT_FindRDFResource (listP, predicate, TRUE);
-      /* ugly, ugly; libwww discards info -- is the object a Literal? */
-      RDFResourceP objectP = ANNOT_FindRDFResource (listP, object, TRUE);
+
+      RDFResourceP objectP;
+
+#ifdef RAPTOR_RDF_PARSER
+      /* if it's an anoynmous subject, add the base_uri */
+      if (triple->subject_type == RAPTOR_IDENTIFIER_TYPE_ANONYMOUS)
+	{
+	  char *base_uri = parse_ctx->base_uri;
+	  char *ptr = (char *) triple->subject;
+	  subject = (char *)TtaGetMemory (strlen (base_uri) + strlen (ptr) + 2);
+	  sprintf (subject, "%s#%s", base_uri, ptr);
+	}
+      else
+	subject = AM_RAPTOR_URI_AS_STRING(triple->subject);
+      test = subject[2];
+
+#endif
+      subjectP = ANNOT_FindRDFResource (listP, subject, TRUE);
+
+#ifdef RAPTOR_RDF_PARSER
+          /* @@ Do anything different for Raptor RDF Parser ?? */
+#ifdef AM_REDLAND
+      if (triple->object_type ==  RAPTOR_IDENTIFIER_TYPE_LITERAL)
+	object = (char *) triple->object;
+      else if (triple->object_type ==  RAPTOR_IDENTIFIER_TYPE_ANONYMOUS)
+	{
+	  ParseContext *parseCtx = (ParseContext *) context;
+	  char *base_uri = parseCtx->base_uri;
+	  char *ptr =  (char *) triple->object;
+	  object = (char *) TtaGetMemory (strlen (base_uri) + strlen (ptr) + 2);
+	  sprintf (object, "%s#%s", base_uri, ptr);
+	}
+      else
+#endif /* AM_REDLAND */
+	object = AM_RAPTOR_URI_AS_STRING(triple->object);
+      test = object[2];
+
+      objectP = ANNOT_FindRDFResource (listP, object, TRUE);
+#else
+	  /* ugly, ugly; libwww discards info -- is the object a Literal? */
+      objectP = ANNOT_FindRDFResource (listP, object, TRUE);
+#endif
 
       SCHEMA_AddStatement (subjectP, predicateP, objectP);
 
@@ -186,10 +266,14 @@ static void triple_handler (HTRDF * rdfp, HTTriple * triple, void * context)
       else if (predicateP == subclassOfP)
 	{
 	  _AddSubClass( objectP, subjectP );
-	  subjectP->class = (RDFClassExtP)TtaGetMemory (sizeof(RDFClassExt));
-	  subjectP->class->instances = NULL;
-	  subjectP->class->subClasses = NULL;
+	  subjectP->class_ = (RDFClassExtP)TtaGetMemory (sizeof(RDFClassExt));
+	  subjectP->class_->instances = NULL;
+	  subjectP->class_->subClasses = NULL;
 	}
+#ifdef RAPTOR_RDF_PARSER
+      if (triple->subject_type == RAPTOR_IDENTIFIER_TYPE_ANONYMOUS)
+	TtaFreeMemory (subject);
+#endif
     }
 }
 
@@ -197,63 +281,98 @@ static void triple_handler (HTRDF * rdfp, HTTriple * triple, void * context)
 /*-----------------------------------------------------------------------
   ReadSchema_callback
   -----------------------------------------------------------------------*/
-#ifdef __STDC__
 static void ReadSchema_callback (Document doc, int status, 
 				 char *urlName,
 				 char *outputfile, 
 				 AHTHeaders *http_headers,
 				 void * context)
-#else  /* __STDC__ */
-static void ReadSchema_callback (doc, status, urlName,
-				 outputfile, http_headers,
-				 context)
-Document doc;
-int status;
-char *urlName;
-char *outputfile;
-AHTHeaders *http_headers;
-void *context;
-
-#endif
 {
-   ReadCallbackContext *ctx = (ReadCallbackContext*) context;
-   BOOL parse = NO;
+#ifdef RAPTOR_RDF_PARSER
+  raptor_parser* rdfxml_parser=NULL;
+#ifdef AM_REDLAND
+  raptor_uri *uri = NULL;
+#else
+  char *uri = NULL;
+#endif /* AM_REDLAND */
+  char *full_file_name = NULL;
+  ParseContextP parse_ctx = NULL;
+#endif
 
+  ReadCallbackContext *ctx = (ReadCallbackContext*) context;
+  BOOL parse = NO;
+  
    if (!ctx)
      return;
 
    ResetStop (doc);
 
-   if (status == HT_OK)
-     parse = HTRDF_parseFile (ctx->filename,
-			      triple_handler,
-			      &annot_schema_list);
+  if (status == HT_OK)
+    {
+#ifdef RAPTOR_RDF_PARSER
+#ifdef AM_REDLAND
+      rdfxml_parser=raptor_new_parser ("rdfxml");
+#else
+      rdfxml_parser=raptor_new ();
+#endif /* AM_REDLAND */ 
+      if (!rdfxml_parser)
+	return;
 
-   if (parse)
-     TtaSetStatus (doc, 1, "Schema read", NULL); /* @@ */
-   else
-     TtaSetStatus (doc, 1, "Error during schema read", NULL); /* @@ */
+      parse_ctx = (ParseContextP)TtaGetMemory (sizeof (ParseContext));
 
-   TtaFreeMemory (ctx);
+      /* @@ this is what we should do eventually */
+      /* if (!IsW3Path (file_name))
+	 full_file_name = LocalToWWW (file_name);
+	 else
+      */
+      /* raptor doesn't grok file URIs under windows. The following is a patch so
+	 that we can use it */
+      full_file_name = (char *)TtaGetMemory (strlen (ctx->filename) + sizeof ("file:"));
+      sprintf (full_file_name, "file:%s", ctx->filename);
+      parse_ctx->base_uri = full_file_name;
+      parse_ctx->annot_schema_list = &annot_schema_list;
+      raptor_set_statement_handler(rdfxml_parser, (void *) parse_ctx, triple_handler);
+      
+      /* remember the base name for anoynmous subjects */
+#ifdef AM_REDLAND
+      uri = raptor_new_uri ((const char *) full_file_name);
+#else
+      uri = full_file_name;
+#endif /* AM_REDLAND */
+      parse = raptor_parse_file(rdfxml_parser, uri, uri);
+#else
+      parse = HTRDF_parseFile (ctx->filename,
+			       triple_handler,
+			       &annot_schema_list);
+#endif
+      if (parse)
+	TtaSetStatus (doc, 1, "Schema read", NULL); /* @@ */
+      else
+	TtaSetStatus (doc, 1, "Error during schema read", NULL); /* @@ */
+    }  
+
+#ifdef RAPTOR_RDF_PARSER
+#ifdef AM_REDLAND
+  raptor_free_uri (uri);
+  raptor_free_parser (rdfxml_parser);
+#else
+  raptor_free (rdfxml_parser);
+#endif /* AM_REDLAND */
+  TtaFreeMemory (full_file_name);
+  TtaFreeMemory (parse_ctx);
+#endif    
+  TtaFreeMemory (ctx);
 }
-
+ 
 /********************** Public API entry point ********************/
 
 /*------------------------------------------------------------
    ANNOT_FindRDFResource
   ------------------------------------------------------------*/
-#ifdef __STDC__
 RDFResourceP ANNOT_FindRDFResource( List** listP, char* name, ThotBool create )
-#else /* __STDC__ */
-RDFResourceP ANNOT_FindRDFResource( listP, name, create )
-     List** listP;
-     char* name;
-     ThotBool create;
-#endif /* __STDC__ */
 {
   RDFResourceP resource;
 
-  if (!name)
+  if (!name || !name[0])
     return NULL;
 
   /* search for resource in list */
@@ -261,11 +380,11 @@ RDFResourceP ANNOT_FindRDFResource( listP, name, create )
   if ((!resource || !resource->name) && create)
     {
       if (!resource)
-        resource = TtaGetMemory (sizeof(RDFResource));
+        resource = (RDFResourceP) TtaGetMemory (sizeof(RDFResource));
 
       resource->name = TtaStrdup (name);
       resource->statements = NULL;
-      resource->class = NULL;
+      resource->class_ = NULL;
       List_add (listP, (void*) resource);
     }
 
@@ -276,13 +395,7 @@ RDFResourceP ANNOT_FindRDFResource( listP, name, create )
 /*------------------------------------------------------------
    ANNOT_FindRDFStatement
   ------------------------------------------------------------*/
-#ifdef __STDC__
 RDFStatementP ANNOT_FindRDFStatement( List* list, RDFPropertyP p )
-#else /* __STDC__ */
-RDFStatementP ANNOT_FindRDFStatement( listP, p )
-     List** listP;
-     RDFPredicateP p;
-#endif /* __STDC__ */
 {
   if (!p)
     return NULL;
@@ -307,13 +420,7 @@ RDFStatementP ANNOT_FindRDFStatement( listP, p )
    the resource is a class, then) for any of its superclasses,
    then return the name of the resource.
   ------------------------------------------------------------*/
-#ifdef __STDC__
 char *ANNOT_GetLabel (List **listP, RDFResourceP r)
-#else /*  __STDC__ */
-char *ANNOT_GetLabel (listP, r)
-     ListP **listP;
-     RDFResourceP r;
-#endif /*  __STDC__ */
 {
   RDFStatementP labelS = NULL;
 
@@ -333,7 +440,7 @@ char *ANNOT_GetLabel (listP, r)
 	  return labelS->object->name;
 	}
 
-      if (r->class)
+      if (r->class_)
 	{
 	  /* search superclass(es) for a label */
 	  /* RRS @@ only does one superclass for now */
@@ -365,13 +472,7 @@ char *ANNOT_GetLabel (listP, r)
    Parameters:
      namespace_URI - the name of the Schema to parse
   ------------------------------------------------------------*/
-#ifdef __STDC__
 void SCHEMA_ReadSchema (Document doc, char *namespace_URI)
-#else /* __STDC__ */
-void SCHEMA_ReadSchema (doc, namespace_URI)
-     Document doc;
-     char *namespace_URI;
-#endif /* __STDC__ */
 {
   ReadCallbackContext *ctx;
   int res;
@@ -391,6 +492,7 @@ void SCHEMA_ReadSchema (doc, namespace_URI)
     {
       /* launch the request */
       res = GetObjectWWW (doc,
+			  0,
 			  namespace_URI,
 			  NULL,
 			  ctx->filename,
@@ -398,7 +500,7 @@ void SCHEMA_ReadSchema (doc, namespace_URI)
 			  AMAYA_SYNC | AMAYA_FLUSH_REQUEST,
 			  NULL,
 			  NULL, 
-			  (void *) ReadSchema_callback,
+			  (void (*)(int, int, char*, char*, const AHTHeaders*, void*)) ReadSchema_callback,
 			  (void *) ctx,
 			  NO,
 			  "application/xml");
@@ -415,11 +517,7 @@ void SCHEMA_ReadSchema (doc, namespace_URI)
   ------------------------------------------------------------
    Frees resources associated with the base annotation namespace name
   ------------------------------------------------------------*/
-#ifdef __STDC__
 static void FreeAnnotNS (void)
-#else /* __STDC__ */
-static void FreeAnnotNS ( /* void */ )
-#endif /* __STDC__ */
 {
   if (ANNOT_NS)
     {
@@ -436,15 +534,10 @@ static void FreeAnnotNS ( /* void */ )
   ------------------------------------------------------------
    Sets the base annotation namespace name
   ------------------------------------------------------------*/
-#ifdef __STDC__
 static void SetAnnotNS (char *ns_name)
-#else /* __STDC__ */
-static void SetAnnotNS ()
-     char *ns_name;
-#endif /* __STDC__ */
 {
   ANNOT_NS = TtaStrdup (ns_name);
-  ANNOTATION_CLASSNAME = TtaGetMemory (strlen(ns_name)
+  ANNOTATION_CLASSNAME = (char *)TtaGetMemory (strlen(ns_name)
 				       + strlen(ANNOT_LOCAL_NAME)
 				       + 1);
   strcpy (ANNOTATION_CLASSNAME, ns_name);
@@ -469,12 +562,7 @@ static void SetAnnotNS ()
   ------------------------------------------------------------
    Initializes the annotation schemas (annot_schema_list) from a config file.
   ------------------------------------------------------------*/
-#ifdef __STDC__
 void SCHEMA_InitSchemas (Document doc)
-#else /* __STDC__ */
-void SCHEMA_InitSchemas (doc)
-     Document doc;
-#endif /* __STDC__ */
 {
   char* thotdir;
   char *app_home;
@@ -501,7 +589,7 @@ void SCHEMA_InitSchemas (doc)
 
   len = strlen(thotdir) + strlen(app_home) + MAX_LENGTH + 32;
 
-  buffer = TtaGetMemory(len);
+  buffer = (char *)TtaGetMemory(len);
   sprintf (buffer, "%s%cannot.schemas", app_home, DIR_SEP);
 
   if (!TtaFileExist (buffer))
@@ -576,7 +664,7 @@ void SCHEMA_InitSchemas (doc)
 	    /* no room at beginning of buffer to expand THOTDIR,
 	       so copy the filename elsewhere before concatenating */
 	    {
-	      char *temp = TtaGetMemory(len);
+	      char *temp = (char *)TtaGetMemory(len);
 
 	      strcpy (temp, fname); /* copy */
 	      strcpy (buffer, thotdir);
@@ -597,7 +685,7 @@ void SCHEMA_InitSchemas (doc)
 	      /* no room at beginning of buffer to expand APP_HOME,
 		 so copy the filename elsewhere before concatenating */
 	      {
-		char *temp = TtaGetMemory(len);
+		char *temp = (char *)TtaGetMemory(len);
 
 		strcpy (temp, fname); /* copy */
 		strcpy (buffer, app_home);
@@ -614,7 +702,18 @@ void SCHEMA_InitSchemas (doc)
 
       if (TtaFileExist (fname))
 	{
+#ifdef RAPTOR_RDF_PARSER
+	  char *ptr;
+	  if (!IsW3Path (fname) && !IsFilePath (fname))
+	    ptr = ANNOT_MakeFileURL (fname);
+	  else
+	    ptr = fname;
+	  SCHEMA_ReadSchema (doc, ptr);
+	  if (ptr != fname)
+	    TtaFreeMemory (ptr);
+#else
 	  HTRDF_parseFile (fname, triple_handler, &annot_schema_list);
+#endif /* RAPTOR_RDF_PARSER */
 	  if (!ANNOT_NS)	/* is this the first schema listed? */
 	    SetAnnotNS (nsname);
 	}
@@ -641,10 +740,10 @@ void SCHEMA_InitSchemas (doc)
   else /* localname only */
     { /* Search the subtypes of ANNOTATION_CLASS for one whose name matches */
       RDFClassP annotType = ANNOTATION_CLASS;
-      if (annotType && annotType->class)
+      if (annotType && annotType->class_)
 	{
 	  int len = strlen (buffer);
-	  List *item = annotType->class->subClasses;
+	  List *item = annotType->class_->subClasses;
 	  for (; item; item=item->next)
 	    {
 	      RDFClassP annotType = (RDFClassP)item->object;
@@ -660,6 +759,44 @@ void SCHEMA_InitSchemas (doc)
 	    }
 	}
     }
+#ifdef ANNOT_ON_ANNOT
+  THREAD_REPLY_CLASS = ANNOT_FindRDFResource (&annot_schema_list,
+					      THREAD_NS THREAD_REPLY_LOCAL_NAME,
+					      TRUE);
+  DEFAULT_REPLY_TYPE = THREAD_REPLY_CLASS;
+  /* @@ RRS: REPLY_DEFAULT_TYPE should be accessible from the config menu */
+  buffer = TtaGetEnvString ("REPLY_DEFAULT_TYPE");
+  if (!buffer)
+    buffer = "Comment";	/* fallback default type */
+
+  /* two options; user can specify a full property URI or just the localname */
+  if (IsW3Path (buffer)) /* full URI */
+    DEFAULT_REPLY_TYPE = ANNOT_FindRDFResource (&annot_schema_list,
+						buffer,
+						TRUE);
+  else /* localname only */
+    { /* Search the subtypes of THREAD_REPLY_CLASS for one whose name matches */
+      RDFClassP annotType = THREAD_REPLY_CLASS;
+      if (annotType && annotType->class_)
+	{
+	  int len = strlen (buffer);
+	  List *item = annotType->class_->subClasses;
+	  for (; item; item=item->next)
+	    {
+	      RDFClassP annotType = (RDFClassP)item->object;
+	      int p = strlen (annotType->name) - len;
+
+	      /* @@ RRS: should check the entire localname, not just the tail.
+		 Use rdfs:isDefinedBy to split out the namespace name. */
+	      if (!strncmp (buffer, &annotType->name[p], len))
+		{
+		  DEFAULT_REPLY_TYPE = annotType;
+		  break;
+		}
+	    }
+	}
+    }
+#endif /* ANNOT_ON_ANNOT */
 }
 
 
@@ -702,12 +839,13 @@ static ThotBool SCHEMA_FreeRDFResource( void *item )
 
   TtaFreeMemory (r->name);
   List_delAll (&r->statements, List_delCharObj);
-  if (r->class)
+  if (r->class_)
     {
-      List_delAll (&r->class->instances, NULL);
-      List_delAll (&r->class->subClasses, NULL);
-      TtaFreeMemory ((char *)r->class);
+      List_delAll (&r->class_->instances, NULL);
+      List_delAll (&r->class_->subClasses, NULL);
+      TtaFreeMemory ((char *)r->class_);
     }
+  TtaFreeMemory (r);
   return TRUE;
 }
 
@@ -718,11 +856,7 @@ static ThotBool SCHEMA_FreeRDFResource( void *item )
    Frees the dynamic heap for all resources in the RDF Model
    for the loaded schema(s)
   ------------------------------------------------------------*/
-#ifdef __STDC__
 void SCHEMA_FreeAnnotSchema( void )
-#else /* __STDC__ */
-void SCHEMA_FreeAnnotSchema()
-#endif /* __STDC__ */
 {
   SCHEMA_FreeRDFModel (&annot_schema_list);
   subclassOfP = NULL;
@@ -731,8 +865,13 @@ void SCHEMA_FreeAnnotSchema()
   PROP_name = NULL;
   PROP_firstName = NULL;
   PROP_Email = NULL;
+  PROP_usesIcon = NULL;
   ANNOTATION_CLASS = NULL;
   DEFAULT_ANNOTATION_TYPE = NULL;
+#ifdef ANNOT_ON_ANNOT
+  THREAD_REPLY_CLASS = NULL;
+  DEFAULT_REPLY_TYPE = NULL;
+#endif /* ANNOT_ON_ANNOT */
   FreeAnnotNS();
 }
 
@@ -742,12 +881,7 @@ void SCHEMA_FreeAnnotSchema()
   ------------------------------------------------------------
    Frees the dynamic heap for all resources in an RDF Model
   ------------------------------------------------------------*/
-#ifdef __STDC__
 void SCHEMA_FreeRDFModel( List **model )
-#else /* __STDC__ */
-void SCHEMA_FreeRDFModel()
-     List **model;
-#endif /* __STDC__ */
 {
   List_delAll (model, SCHEMA_FreeRDFResource);
 }
