@@ -30,12 +30,11 @@
 #include "typegrm.h"
 #include "typeint.h"
 #include "fileaccess.h"
-#include "thotdir.h"
 
 typedef char        fname[30];	/* nom de fichier */
 
 #undef THOT_EXPORT
-#define THOT_EXPORT
+#define THOT_EXPORT extern
 #include "compil_tv.h"
 #undef THOT_EXPORT
 #define THOT_EXPORT extern
@@ -82,12 +81,12 @@ static FILE         *listFile;	/* fichier des listes */
 #include "message_f.h"
 #include "registry_f.h"
 
-#ifdef _WINDOWS
+#ifdef _WINGUI
 #define DLLEXPORT __declspec (dllexport)
 #define FATAL_EXIT_CODE 33
 #define COMP_SUCCESS     0
 #include "compilers_f.h"
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
 
 /*----------------------------------------------------------------------
    InitRefTables initialise les tables des references.		
@@ -709,11 +708,11 @@ static ThotBool      CheckDefAndRef ()
 /*----------------------------------------------------------------------
    main program                                                    
   ----------------------------------------------------------------------*/
-#ifdef _WINDOWS
+#ifdef _WINGUI
 int GRMmain (HWND hwnd, int argc, char **argv, int *Y)
-#else  /* !_WINDOWS */
+#else  /* !_WINGUI */
 int main (int argc, char **argv)
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
 {
   FILE               *infile;
   ThotBool             fileOK;
@@ -727,12 +726,12 @@ int main (int argc, char **argv)
   SyntRuleNum         pr;	/* numero de la regle precedente */
   SyntacticCode       code;	/* code grammatical du mot trouve */
   int                 rank;	/* indice dans Identifier du mot trouve */
-#ifdef _WINDOWS 
+#ifdef _WINGUI 
   char                msg [800];
   int                 ndx;
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
 
-#ifdef _WINDOWS
+#ifdef _WINGUI
   hWnd = hwnd;
   compilersDC = GetDC (hwnd);
   _CY_ = *Y;
@@ -743,7 +742,7 @@ int main (int argc, char **argv)
       strcat (msg, " ");
     }
   TtaDisplayMessage (INFO, msg);
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
 
   TtaInitializeAppRegistry (argv[0]);
   GRM = TtaGetMessageTable ("grmdialogue", GRM_MSG_MAX);
@@ -807,7 +806,7 @@ int main (int argc, char **argv)
 	      /* marque la fin reelle de la ligne */
 	      inputLine[i - 1] = '\0';
 	      /* garde une copie de la ligne avant traduction */
-	      strncpy (sourceLine, inputLine, LINE_LENGTH);
+	      strncpy ((char *)sourceLine, (char *)inputLine, LINE_LENGTH);
 	      LineNum++;
 	      /* traduit les caracteres de la ligne */
 	      OctalToChar ();
@@ -854,13 +853,13 @@ int main (int argc, char **argv)
 	  TtaSaveAppRegistry ();
 	}
     }
-#ifdef _WINDOWS
+#ifdef _WINGUI
   *Y = _CY_;
   ReleaseDC (hwnd, compilersDC);
   if (error)
     return FATAL_EXIT_CODE;
   return COMP_SUCCESS;
-#else  /* !_WINDOWS */
+#else  /* !_WINGUI */
   return (0);
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
 }
