@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -10,6 +10,8 @@
    du schema de structure associe' et liste le contenu du schema de
    presentation.        
   ----------------------------------------------------------------------*/
+#undef THOT_EXPORT
+#define THOT_EXPORT extern
 
 #include "thot_sys.h"
 #include "constmedia.h"
@@ -20,12 +22,12 @@
 #include "fileaccess.h"
 #include "message.h"
 
-#define THOT_EXPORT
 #include "platform_tv.h"
 
-#include "readprs_f.h"
-#include "memory_f.h"
 #include "fileaccess_f.h"
+#include "memory_f.h"
+#include "readprs_f.h"
+#include "readstr_f.h"
 
 static PtrSSchema   pSchemaStr;
 static PtrPSchema   pSchemaPrs;
@@ -38,11 +40,11 @@ CntrItem           *pCp1;
 PresConstant       *pPr1;
 PresVariable       *pPres1;
 PresVarItem        *pVa1;
-PresentationBox    *pBo1;
+PtrPresentationBox  pBo1;
 TtAttribute        *pAt1;
 AttributePres      *pRP1;
 NumAttrCase        *pCa1;
-ThotBool             Transm;
+ThotBool            Transm;
 
 /*----------------------------------------------------------------------
    wrnb ecrit au terminal l'entier nb.                             
@@ -176,24 +178,27 @@ static void         wrrepere (BoxEdge r)
 /*----------------------------------------------------------------------
    wrnom ecrit au terminal le nom n.                               
   ----------------------------------------------------------------------*/
-static void         wrnom (Name n)
+static void wrnom (char *n)
 {
    int                 i;
 
-   i = 1;
-   while (n[i - 1] != '\0')
+   if (n)
      {
-	if (n[i - 1] < ' ' || n[i - 1] > '~')
-	   /* caractere non imprimable, ecrit son code */
-	  {
-	     printf ("\\%o", (unsigned char) n[i - 1]);
-	  }
-	else
-	  {
-	     /* caractere imprimable, ecrit le  caractere */
-	     putchar (n[i - 1]);
-	  }
-	i++;
+       i = 1;
+       while (n[i - 1] != '\0')
+	 {
+	   if (n[i - 1] < ' ' || n[i - 1] > '~')
+	     /* caractere non imprimable, ecrit son code */
+	     {
+	       printf ("\\%o", (unsigned char) n[i - 1]);
+	     }
+	   else
+	     {
+	       /* caractere imprimable, ecrit le  caractere */
+	       putchar (n[i - 1]);
+	     }
+	   i++;
+	 }
      }
 }
 
@@ -204,7 +209,7 @@ static void         wrnom (Name n)
 static void         wrnomregle (int r)
 {
    if (r > 0)
-      wrnom (pSchemaStr->SsRule[r - 1].SrName);
+      wrnom (pSchemaStr->SsRule->SrElem[r - 1]->SrName);
 }
 
 /*----------------------------------------------------------------------
@@ -213,7 +218,7 @@ static void         wrnomregle (int r)
 static void         wrnomattr (int a)
 {
    if (a != 0)
-      wrnom (pSchemaStr->SsAttribute[abs (a) - 1].AttrName);
+      wrnom (pSchemaStr->SsAttribute->TtAttr[abs (a) - 1]->AttrName);
 }
 
 
@@ -224,7 +229,7 @@ static void         wrnomattr (int a)
 static void         wrnomboite (int b)
 {
    if (b > 0)
-      wrnom (pSchemaPrs->PsPresentBox[b - 1].PbName);
+      wrnom (pSchemaPrs->PsPresentBox->PresBox[b - 1]->PbName);
 }
 
 /*----------------------------------------------------------------------
@@ -296,11 +301,16 @@ static void         wrsize (PtrPRule pR)
      }
    else if (pR->PrPresMode == PresImmediate)
      {
-	if (pR->PrMinAttr)
-	   wrnomattr (pR->PrMinValue);
-	else
-	   wrnb (pR->PrMinValue);
-	wrdistunit (pR->PrMinUnit);
+       if (pR->PrMinUnit == UnAuto)
+	 printf ("auto");
+       else
+	 {
+	   if (pR->PrMinAttr)
+	     wrnomattr (pR->PrMinValue);
+	   else
+	     wrnb (pR->PrMinValue);
+	   wrdistunit (pR->PrMinUnit);
+	 }
      }
    else
       printf ("??????");
@@ -409,6 +419,35 @@ static void         wrfontstyle (PtrPRule pR)
 		     putchar (pR->PrChrValue);
 		     break;
 	       }
+      else if (pR->PrType == PtDirection)
+	 switch (pR->PrChrValue)
+	       {
+		  case 'L':
+		     printf ("ltr");
+		     break;
+		  case 'R':
+		     printf ("rtl");
+		     break;
+		  default:
+		     putchar (pR->PrChrValue);
+		     break;
+	       }
+      else if (pR->PrType == PtUnicodeBidi)
+	 switch (pR->PrChrValue)
+	       {
+		  case 'N':
+		     printf ("Normal");
+		     break;
+		  case 'E':
+		     printf ("Embed");
+		     break;
+		  case 'O':
+		     printf ("Override");
+		     break;
+		  default:
+		     putchar (pR->PrChrValue);
+		     break;
+	       }
       else if (pR->PrType == PtLineStyle)
 	 switch (pR->PrChrValue)
 	       {
@@ -425,6 +464,28 @@ static void         wrfontstyle (PtrPRule pR)
 		     putchar (pR->PrChrValue);
 		     break;
 	       }
+      else if (pR->PrType == PtDisplay)
+	 switch (pR->PrChrValue)
+	       {
+		  case 'I':
+		     printf ("inline");
+		     break;
+		  case 'B':
+		     printf ("block");
+		     break;
+		  case 'L':
+		     printf ("listItem");
+		     break;
+		  case 'R':
+		     printf ("runIn");
+		     break;
+		  case 'C':
+		     printf ("compact");
+		     break;
+		  case 'M':
+		     printf ("marker");
+		     break;
+	       }
       else if (pR->PrType == PtBorderTopStyle ||
 	       pR->PrType == PtBorderRightStyle ||
 	       pR->PrType == PtBorderBottomStyle ||
@@ -524,27 +585,27 @@ static void         wrnbherit (PtrPRule pR)
   ----------------------------------------------------------------------*/
 static void         wrminind (PtrPRule pR)
 {
-   if (pR->PrPresMode == PresInherit)
-      wrnbherit (pR);
-   else
-     {
-	if (pR->PrPresMode == PresImmediate)
-	  {
-	     if (pR->PrMinUnit == UnAuto)
-	        printf ("auto");
-	     else
-	        {
-	        if (pR->PrMinAttr)
-		   wrnomattr (pR->PrMinValue);
-	        else
-		   wrnb (pR->PrMinValue);
-	        wrdistunit (pR->PrMinUnit);
-	        }
-	  }
-	else
-	   printf ("??????");
-	printf (";");
-     }
+  if (pR->PrPresMode == PresInherit)
+    wrnbherit (pR);
+  else
+    {
+      if (pR->PrPresMode == PresImmediate)
+	{
+	  if (pR->PrMinUnit == UnAuto)
+	    printf ("auto");
+	  else
+	    {
+	      if (pR->PrMinAttr)
+		wrnomattr (pR->PrMinValue);
+	      else
+		wrnb (pR->PrMinValue);
+	      wrdistunit (pR->PrMinUnit);
+	    }
+	}
+      else
+	printf ("??????");
+      printf (";");
+    }
 }
 
 /*----------------------------------------------------------------------
@@ -553,16 +614,18 @@ static void         wrminind (PtrPRule pR)
 static void         wrpos (PosRule pos, ThotBool Def)
 {
    if (Def)
-      if (pos.PoPosDef == NoEdge)
+     {
+       if (pos.PoPosDef == NoEdge)
 	 printf (" NULL");
-      else
-	{
+       else
+	 {
 	   wrrepere (pos.PoPosDef);
 	   printf (" = ");
-	}
+	 }
+     }
    if (!Def || pos.PoPosDef != NoEdge)
      {
-	wrlevel (pos.PoRelation);
+       wrlevel (pos.PoRelation);
 	if (pos.PoNotRel)
 	   printf (" NOT");
 	printf (" ");
@@ -572,6 +635,10 @@ static void         wrpos (PosRule pos, ThotBool Def)
 	   wrnomboite (pos.PoRefIdent);
 	else if (pos.PoRefKind == RkAttr)
 	   wrnomattr (pos.PoRefIdent);
+	else if (pos.PoRefKind == RkAnyElem)
+	   printf ("AnyElem");
+	else if (pos.PoRefKind == RkAnyBox)
+	   printf ("AnyBox");
 	printf (". ");
 	wrrepere (pos.PoPosRef);
 	if (pos.PoDistance != 0)
@@ -626,6 +693,10 @@ static void         wrdimens (DimensionRule Dim, ThotBool Hauteur)
 		wrnomboite (Dim.DrRefIdent);
 	     else if (Dim.DrRefKind == RkAttr)
 		wrnomattr (Dim.DrRefIdent);
+	     else if (Dim.DrRefKind == RkAnyElem)
+	       printf ("AnyElem");
+	     else if (Dim.DrRefKind == RkAnyBox)
+	       printf ("AnyBox");
 	     printf (". ");
 	     if ((Dim.DrSameDimens && Hauteur) || (!Dim.DrSameDimens && !Hauteur))
 		printf ("Height");
@@ -770,11 +841,23 @@ static void         wrCondition (PtrCondition pCond)
 		 }
 	       break;
 	    case PcElemType:
-	       wrnomregle (pCond->CoTypeElAttr);
+	       wrnomregle (pCond->CoTypeElem);
 	       printf (" ");
 	       break;
+	    case PcInheritAttribute:
+	       printf ("Inherited ");
 	    case PcAttribute:
-	       wrnomattr (pCond->CoTypeElAttr);
+	       wrnomattr (pCond->CoTypeAttr);
+	       if (pCond->CoTestAttrValue)
+		 {
+		   printf (" = ");
+		   if (pSchemaStr->SsAttribute->TtAttr[pCond->CoTypeAttr - 1]->AttrType == AtTextAttr)
+		     wrnom (pCond->CoAttrTextValue);
+		   else if (pSchemaStr->SsAttribute->TtAttr[pCond->CoTypeAttr - 1]->AttrType == AtEnumAttr)
+		     wrnom (pSchemaStr->SsAttribute->TtAttr[pCond->CoTypeAttr - 1]->AttrEnumValue[pCond->CoAttrValue - 1]);
+		   else
+		     wrnb (pCond->CoAttrValue);
+		 }
 	       printf (" ");
 	       break;
 	    default:
@@ -943,10 +1026,12 @@ static void         wrboolean (PtrPRule pR)
    if (pR->PrPresMode == PresInherit)
       wrnbherit (pR);
    if (pR->PrPresMode == PresImmediate)
+     {
       if (pR->PrBoolValue)
 	 printf ("Yes;");
       else
 	 printf ("No;");
+     }
 }
 
 /*----------------------------------------------------------------------
@@ -1012,136 +1097,44 @@ static void         wrsuiteregles (PtrPRule RP)
 	  }
 	switch (RP->PrType)
 	      {
-		 case PtFunction:
-		    wrFonctPres (RP);
-		    break;
 		 case PtVisibility:
 		    printf ("Visibility: ");
 		    wrnbherit (RP);
 		    break;
-		 case PtDepth:
-		    printf ("Depth: ");
-		    wrnbherit (RP);
+		 case PtFunction:
+		    wrFonctPres (RP);
 		    break;
-		 case PtFillPattern:
-		    printf ("FillPattern: ");
-		    wrnbherit (RP);
+		 case PtVertOverflow:
+		    printf ("VertOverflow: ");
+		    wrboolean (RP);
 		    break;
-		 case PtBackground:
-		    printf ("Background: ");
-		    wrnbherit (RP);
+		 case PtHorizOverflow:
+		    printf ("HorizOverflow: ");
+		    wrboolean (RP);
 		    break;
-		 case PtForeground:
-		    printf ("Foreground: ");
-		    wrnbherit (RP);
+		 case PtVertRef:
+		    printf ("VertRef: ");
+		    wrpos (RP->PrPosRule, False);
 		    break;
-		 case PtBorderTopColor:
-		    printf ("BorderTopColor: ");
-	            if (RP->PrPresMode == PresImmediate &&
-			!RP->PrAttrValue && RP->PrIntValue == -2)
-		       printf ("transparent;");
-		    else if (RP->PrPresMode == PresImmediate &&
-			     !RP->PrAttrValue && RP->PrIntValue == -1)
-		       printf ("foreground;");
-		    else
-		       wrnbherit (RP);
+		 case PtHorizRef:
+		    printf ("HorizRef: ");
+		    wrpos (RP->PrPosRule, False);
 		    break;
-		 case PtBorderRightColor:
-		    printf ("BorderRightColor: ");
-	            if (RP->PrPresMode == PresImmediate &&
-			!RP->PrAttrValue && RP->PrIntValue == -2)
-		       printf ("transparent;");
-		    else if (RP->PrPresMode == PresImmediate &&
-			     !RP->PrAttrValue && RP->PrIntValue == -1)
-		       printf ("foreground;");
-		    else
-		       wrnbherit (RP);
+		 case PtHeight:
+		    printf ("Height: ");
+		    wrdimens (RP->PrDimRule, True);
 		    break;
-		 case PtBorderBottomColor:
-		    printf ("BorderBottomColor: ");
-	            if (RP->PrPresMode == PresImmediate &&
-			!RP->PrAttrValue && RP->PrIntValue == -2)
-		       printf ("transparent;");
-		    else if (RP->PrPresMode == PresImmediate &&
-			     !RP->PrAttrValue && RP->PrIntValue == -1)
-		       printf ("foreground;");
-		    else
-		       wrnbherit (RP);
+		 case PtWidth:
+		    printf ("Width: ");
+		    wrdimens (RP->PrDimRule, False);
 		    break;
-		 case PtBorderLeftColor:
-		    printf ("BorderLeftColor: ");
-	            if (RP->PrPresMode == PresImmediate &&
-			!RP->PrAttrValue && RP->PrIntValue == -2)
-		       printf ("transparent;");
-		    else if (RP->PrPresMode == PresImmediate &&
-			     !RP->PrAttrValue && RP->PrIntValue == -1)
-		       printf ("foreground;");
-		    else
-		       wrnbherit (RP);
+		 case PtVertPos:
+		    printf ("VertPos: ");
+		    wrpos (RP->PrPosRule, True);
 		    break;
-		 case PtFont:
-		    printf ("Font: ");
-		    wrfontstyle (RP);
-		    break;
-		 case PtStyle:
-		    printf ("Style: ");
-		    wrfontstyle (RP);
-		    break;
-		 case PtWeight:
-		    printf ("Weight: ");
-		    wrfontstyle (RP);
-		    break;
-		 case PtUnderline:
-		    printf ("Underline: ");
-		    wrfontstyle (RP);
-		    break;
-		 case PtThickness:
-		    printf ("Thickness: ");
-		    wrfontstyle (RP);
-		    break;
-		 case PtLineStyle:
-		    printf ("LineStyle: ");
-		    wrfontstyle (RP);
-		    break;
-		 case PtBorderTopStyle:
-		    printf ("BorderTopStyle: ");
-		    wrfontstyle (RP);
-		    break;
-		 case PtBorderRightStyle:
-		    printf ("BorderRightStyle: ");
-		    wrfontstyle (RP);
-		    break;
-		 case PtBorderBottomStyle:
-		    printf ("BorderBottomStyle: ");
-		    wrfontstyle (RP);
-		    break;
-		 case PtBorderLeftStyle:
-		    printf ("BorderLeftStyle: ");
-		    wrfontstyle (RP);
-		    break;
-		 case PtBreak1:
-		    printf ("NoBreak1: ");
-		    wrminind (RP);
-		    break;
-		 case PtBreak2:
-		    printf ("NoBreak2: ");
-		    wrminind (RP);
-		    break;
-		 case PtIndent:
-		    printf ("Indent: ");
-		    wrminind (RP);
-		    break;
-		 case PtSize:
-		    printf ("Size: ");
-		    wrsize (RP);
-		    break;
-		 case PtLineSpacing:
-		    printf ("LineSpacing: ");
-		    wrminind (RP);
-		    break;
-		 case PtLineWeight:
-		    printf ("LineWeight: ");
-		    wrminind (RP);
+		 case PtHorizPos:
+		    printf ("HorizPos: ");
+		    wrpos (RP->PrPosRule, True);
 		    break;
 		 case PtMarginTop:
 		    printf ("MarginTop: ");
@@ -1191,46 +1184,150 @@ static void         wrsuiteregles (PtrPRule RP)
 		    printf ("BorderLeftWidth: ");
 		    wrminind (RP);
 		    break;
-		 case PtVertRef:
-		    printf ("VertRef: ");
-		    wrpos (RP->PrPosRule, False);
+		 case PtBorderTopColor:
+		    printf ("BorderTopColor: ");
+	            if (RP->PrPresMode == PresImmediate &&
+			!RP->PrAttrValue && RP->PrIntValue == -2)
+		       printf ("transparent;");
+		    else if (RP->PrPresMode == PresImmediate &&
+			     !RP->PrAttrValue && RP->PrIntValue == -1)
+		       printf ("foreground;");
+		    else
+		       wrnbherit (RP);
 		    break;
-		 case PtHorizRef:
-		    printf ("HorizRef: ");
-		    wrpos (RP->PrPosRule, False);
+		 case PtBorderRightColor:
+		    printf ("BorderRightColor: ");
+	            if (RP->PrPresMode == PresImmediate &&
+			!RP->PrAttrValue && RP->PrIntValue == -2)
+		       printf ("transparent;");
+		    else if (RP->PrPresMode == PresImmediate &&
+			     !RP->PrAttrValue && RP->PrIntValue == -1)
+		       printf ("foreground;");
+		    else
+		       wrnbherit (RP);
 		    break;
-		 case PtVertPos:
-		    printf ("VertPos: ");
-		    wrpos (RP->PrPosRule, True);
+		 case PtBorderBottomColor:
+		    printf ("BorderBottomColor: ");
+	            if (RP->PrPresMode == PresImmediate &&
+			!RP->PrAttrValue && RP->PrIntValue == -2)
+		       printf ("transparent;");
+		    else if (RP->PrPresMode == PresImmediate &&
+			     !RP->PrAttrValue && RP->PrIntValue == -1)
+		       printf ("foreground;");
+		    else
+		       wrnbherit (RP);
 		    break;
-		 case PtHorizPos:
-		    printf ("HorizPos: ");
-		    wrpos (RP->PrPosRule, True);
+		 case PtBorderLeftColor:
+		    printf ("BorderLeftColor: ");
+	            if (RP->PrPresMode == PresImmediate &&
+			!RP->PrAttrValue && RP->PrIntValue == -2)
+		       printf ("transparent;");
+		    else if (RP->PrPresMode == PresImmediate &&
+			     !RP->PrAttrValue && RP->PrIntValue == -1)
+		       printf ("foreground;");
+		    else
+		       wrnbherit (RP);
 		    break;
-		 case PtHeight:
-		    printf ("Height: ");
-		    wrdimens (RP->PrDimRule, True);
+		 case PtBorderTopStyle:
+		    printf ("BorderTopStyle: ");
+		    wrfontstyle (RP);
 		    break;
-		 case PtWidth:
-		    printf ("Width: ");
-		    wrdimens (RP->PrDimRule, False);
+		 case PtBorderRightStyle:
+		    printf ("BorderRightStyle: ");
+		    wrfontstyle (RP);
+		    break;
+		 case PtBorderBottomStyle:
+		    printf ("BorderBottomStyle: ");
+		    wrfontstyle (RP);
+		    break;
+		 case PtBorderLeftStyle:
+		    printf ("BorderLeftStyle: ");
+		    wrfontstyle (RP);
+		    break;
+		 case PtSize:
+		    printf ("Size: ");
+		    wrsize (RP);
+		    break;
+		 case PtStyle:
+		    printf ("Style: ");
+		    wrfontstyle (RP);
+		    break;
+		 case PtWeight:
+		    printf ("Weight: ");
+		    wrfontstyle (RP);
+		    break;
+		 case PtFont:
+		    printf ("Font: ");
+		    wrfontstyle (RP);
+		    break;
+		 case PtUnderline:
+		    printf ("Underline: ");
+		    wrfontstyle (RP);
+		    break;
+		 case PtThickness:
+		    printf ("Thickness: ");
+		    wrfontstyle (RP);
+		    break;
+		 case PtIndent:
+		    printf ("Indent: ");
+		    wrminind (RP);
+		    break;
+		 case PtLineSpacing:
+		    printf ("LineSpacing: ");
+		    wrminind (RP);
+		    break;
+		 case PtDepth:
+		    printf ("Depth: ");
+		    wrnbherit (RP);
+		    break;
+		 case PtAdjust:
+		    printf ("Adjust: ");
+		    wrajust (RP);
+		    break;
+		 case PtDirection:
+		    printf ("Direction: ");
+		    wrfontstyle (RP);
+		    break;
+		 case PtUnicodeBidi:
+		    printf ("UnicodeBidi: ");
+		    wrfontstyle (RP);
+		    break;
+		 case PtLineStyle:
+		    printf ("LineStyle: ");
+		    wrfontstyle (RP);
+		    break;
+		 case PtLineWeight:
+		    printf ("LineWeight: ");
+		    wrminind (RP);
+		    break;
+		 case PtFillPattern:
+		    printf ("FillPattern: ");
+		    wrnbherit (RP);
+		    break;
+		 case PtOpacity;
+		    printf ("Opacity: ");
+		    wrnbherit (RP);
+		    break;
+		 case PtFillOpacity;
+		    printf ("FillOpacity: ");
+		    wrnbherit (RP);
+		    break;
+		  case PtStrokeOpacity;
+		    printf ("StrokeOpacity: ");
+		    wrnbherit (RP);
+		    break; 
+		 case PtBackground:
+		    printf ("Background: ");
+		    wrnbherit (RP);
+		    break;
+		 case PtForeground:
+		    printf ("Foreground: ");
+		    wrnbherit (RP);
 		    break;
 		 case PtHyphenate:
 		    printf ("Hyphenate: ");
 		    wrboolean (RP);
 		    break;
-		 case PtVertOverflow:
-		    printf ("VertOverflow: ");
-		    wrboolean (RP);
-		    break;
-		 case PtHorizOverflow:
-		    printf ("HorizOverflow: ");
-		    wrboolean (RP);
-		    break;
-	         case PtGather:
-		    printf ("Gather: ");
-		    wrboolean (RP);
-		    break;
 	         case PtPageBreak:
 		    printf ("PageBreak: ");
 		    wrboolean (RP);
@@ -1239,13 +1336,35 @@ static void         wrsuiteregles (PtrPRule RP)
 		    printf ("LineBreak: ");
 		    wrboolean (RP);
 		    break;
-		 case PtAdjust:
-		    printf ("Adjust: ");
-		    wrajust (RP);
+	         case PtGather:
+		    printf ("Gather: ");
+		    wrboolean (RP);
+		    break;
+	         case PtXRadius:
+		    printf ("XRadius: ");
+		    wrboolean (RP);
+		    break;
+	         case PtYRadius:
+		    printf ("YRadius: ");
+		    wrboolean (RP);
+		    break;
+	         case PtDisplay:
+		    printf ("Display: ");
+		    wrfontstyle (RP);
+		    break;
+		 case PtBreak1:
+		    printf ("NoBreak1: ");
+		    wrminind (RP);
+		    break;
+		 case PtBreak2:
+		    printf ("NoBreak2: ");
+		    wrminind (RP);
 		    break;
 		 case PtPictInfo:
 		    break;
 	      }
+	if (RP->PrDuplicate)
+	  printf ("  {duplicate}");
 	printf ("\n");		/* passe a la regle suivante */
 	RP = RP->PrNextPRule;
      }
@@ -1451,7 +1570,7 @@ int                 main (int argc, char **argv)
 			{
 			   case CharString:
 			      printf ("Text ");
-			      putchar (pPr1->PdAlphabet);
+			      putchar (pPr1->PdScript);
 			      break;
 			   case GraphicElem:
 			      printf ("Graphics ");
@@ -1564,7 +1683,7 @@ int                 main (int argc, char **argv)
 	     printf ("\n");
 	     for (i = 1; i <= pSc1->PsNPresentBoxes; i++)
 	       {
-		  pBo1 = &pSc1->PsPresentBox[i - 1];
+		  pBo1 = pSc1->PsPresentBox->PresBox[i - 1];
 		  wrnom (pBo1->PbName);
 		  printf (":	{%3d }\n", i);
 		  if (pBo1->PbFirstPRule != NULL)
@@ -1615,17 +1734,19 @@ int                 main (int argc, char **argv)
 	printf ("\nRULES\n\n");
 	for (El = 1; El <= pSchemaStr->SsNRules; El++)
 	  {
-	     if (pSchemaStr->SsRule[El - 1].SrConstruct == CsPairedElement)
-		if (pSchemaStr->SsRule[El - 1].SrFirstOfPair)
+	    if (pSchemaStr->SsRule->SrElem[El - 1]->SrConstruct == CsPairedElement)
+	      {
+		if (pSchemaStr->SsRule->SrElem[El - 1]->SrFirstOfPair)
 		   printf ("First ");
 		else
 		   printf ("Second ");
+	      }
 	     wrnomregle (El);
 	     printf (":\n");
-	     if (pSc1->PsElemPRule[El - 1] != NULL)
+	     if (pSc1->PsElemPRule->ElemPres[El - 1])
 		printf ("   BEGIN\n");
-	     wrsuiteregles (pSc1->PsElemPRule[El - 1]);
-	     if (pSc1->PsElemPRule[El - 1] != NULL)
+	     wrsuiteregles (pSc1->PsElemPRule->ElemPres[El - 1]);
+	     if (pSc1->PsElemPRule->ElemPres[El - 1])
 		printf ("   END;\n");
 	     printf ("\n");
 	  }
@@ -1637,9 +1758,9 @@ int                 main (int argc, char **argv)
 	     printf ("\n");
 	     for (Attr = 1; Attr <= pSchemaStr->SsNAttributes; Attr++)
 	       {
-	       pAt1 = &pSchemaStr->SsAttribute[Attr - 1];
-	       pRP1 = pSc1->PsAttrPRule[Attr - 1];
-	       for (k = pSc1->PsNAttrPRule[Attr - 1]; k-- > 0;
+	       pAt1 = pSchemaStr->SsAttribute->TtAttr[Attr - 1];
+	       pRP1 = pSc1->PsAttrPRule->AttrPres[Attr - 1];
+	       for (k = pSc1->PsNAttrPRule->Num[Attr - 1]; k-- > 0;
 		    pRP1 = pRP1->ApNextAttrPres)
 		  if (pRP1 != NULL)
 		    {
@@ -1673,6 +1794,7 @@ int                 main (int argc, char **argv)
 					  }
 					else if (pCa1->CaLowerBound != -MAX_INT_ATTR_VAL - 1
 						 || pCa1->CaUpperBound != MAX_INT_ATTR_VAL + 1)
+					  {
 					   if (pCa1->CaLowerBound != -MAX_INT_ATTR_VAL - 1)
 					     {
 						printf (">");
@@ -1683,6 +1805,7 @@ int                 main (int argc, char **argv)
 						printf ("<");
 						wrnb (pCa1->CaUpperBound + 1);
 					     }
+					  }
 					printf (":\n");
 					if (pCa1->CaFirstPRule == NULL)
 					   printf ("   BEGIN END;\n");
@@ -1801,7 +1924,7 @@ int                 main (int argc, char **argv)
 	/* ecrit les regles de transmission de valeurs d'elements */
 	if (pSc1->PsNTransmElems > 0)
 	   for (i = 1; i <= pSchemaStr->SsNRules; i++)
-	      if (pSc1->PsElemTransmit[i - 1] > 0)
+	      if (pSc1->PsElemTransmit->Num[i - 1] > 0)
 		{
 		   if (!Transm)
 		     {
@@ -1811,9 +1934,9 @@ int                 main (int argc, char **argv)
 		   printf ("   ");
 		   wrnomregle (i);
 		   printf (" To ");
-		   wrnom (pSc1->PsTransmElem[pSc1->PsElemTransmit[i - 1] - 1].TeTargetAttr);
+		   wrnom (pSc1->PsTransmElem[pSc1->PsElemTransmit->Num[i - 1] - 1].TeTargetAttr);
 		   printf ("(");
-		   wrnomregle (pSc1->PsTransmElem[pSc1->PsElemTransmit[i - 1] - 1].TeTargetDoc);
+		   wrnomregle (pSc1->PsTransmElem[pSc1->PsElemTransmit->Num[i - 1] - 1].TeTargetDoc);
 		   printf (");\n");
 		}
 
