@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -20,7 +20,7 @@
 #include "strmsg.h"
 #include "message.h"
 
-#define THOT_EXPORT
+#define THOT_EXPORT extern
 #include "platform_tv.h"
 
 static PtrSSchema   pSchStr;
@@ -36,8 +36,7 @@ static ThotBool     optionl = False;
 static int          CommentNum;
 static int          STR;	/* Identification des messages Str */
 PtrSSchema          pSc1;
-SRule              *pRe1;
-TtAttribute        *pAt1;
+PtrTtAttribute      pAt1;
 
 #include "readstr_f.h"
 
@@ -176,14 +175,14 @@ static void         wrnom (Name n)
   ----------------------------------------------------------------------*/
 static void         wrnomregle (int r)
 {
-   if (pSchStr->SsRule[r - 1].SrName[0] == '\0')
+   if (pSchStr->SsRule->SrElem[r - 1]->SrName[0] == '\0')
       /* Name vide, on ecrit son numero */
      {
 	printf ("Ident");
 	writenb (r);
      }
    else
-      wrnom (pSchStr->SsRule[r - 1].SrName);
+      wrnom (pSchStr->SsRule->SrElem[r - 1]->SrName);
 }
 
 /*----------------------------------------------------------------------
@@ -226,8 +225,8 @@ static void         wrnomreglext (int r, Name N)
 {
    if (N[0] == '\0')
      {
-	if (pSchStr->SsRule[r - 1].SrConstruct == CsPairedElement)
-	   if (!pSchStr->SsRule[r - 1].SrFirstOfPair)
+	if (pSchStr->SsRule->SrElem[r - 1]->SrConstruct == CsPairedElement)
+	   if (!pSchStr->SsRule->SrElem[r - 1]->SrFirstOfPair)
 	      printf ("Second ");
 	wrnomregle (r);
      }
@@ -237,10 +236,10 @@ static void         wrnomreglext (int r, Name N)
 	   printf ("******");
 	else
 	  {
-	     if (pSchStrExt->SsRule[r - 1].SrConstruct == CsPairedElement)
-		if (!pSchStrExt->SsRule[r - 1].SrFirstOfPair)
+	     if (pSchStrExt->SsRule->SrElem[r - 1]->SrConstruct == CsPairedElement)
+		if (!pSchStrExt->SsRule->SrElem[r - 1]->SrFirstOfPair)
 		   printf ("Second ");
-	     wrnom (pSchStrExt->SsRule[r - 1].SrName);
+	     wrnom (pSchStrExt->SsRule->SrElem[r - 1]->SrName);
 	  }
 	printf ("(");
 	wrnom (N);
@@ -363,6 +362,9 @@ static void         wrNumExcept (int Num)
 	    case ExcParagraphBreak:
 	       printf ("ParagraphBreak");
 	       break;
+	    case ExcIsBreak:
+	       printf ("IsBreak");
+	       break;
 	    case ExcHighlightChildren:
 	       printf ("HighlightChildren");
 	       break;
@@ -426,9 +428,9 @@ static void         Wdefine ()
 static void         wrattr (int a)
 {
    int                 j;
-   TtAttribute        *pAt1;
+   PtrTtAttribute      pAt1;
 
-   pAt1 = &pSchStr->SsAttribute[a - 1];
+   pAt1 = pSchStr->SsAttribute->TtAttr[a - 1];
    if (optionh)
      {
 	if (pAt1->AttrGlobal)
@@ -503,9 +505,9 @@ static void         wrattr (int a)
   ----------------------------------------------------------------------*/
 static void         wrext (int r)
 {
-   SRule              *pRe1;
+   PtrSRule         pRe1;
 
-   pRe1 = &pSchStr->SsRule[r - 1];
+   pRe1 = pSchStr->SsRule->SrElem[r - 1];
    if (pRe1->SrRefImportedDoc)
       if (pRe1->SrConstruct == CsReference)
 	 printf (" extern");
@@ -550,7 +552,7 @@ static void         welembase ()
 
    Wdefine ();
    wrnom (pSc1->SsName);
-   r = Picture + 1;
+   r = tt_Picture + 1;
    printf ("_EL_");
    wrnomregle (r);
    printf (" ");
@@ -565,6 +567,15 @@ static void         welembase ()
    printf (" ");
    writenb (r);
    printf ("\n");
+
+   Wdefine ();
+   wrnom (pSc1->SsName);
+   r = AnyType + 1;
+   printf ("_EL_");
+   wrnomregle (r);
+   printf (" ");
+   writenb (r);
+   printf ("\n");
 }
 
 /*----------------------------------------------------------------------
@@ -575,14 +586,14 @@ static void         wrrule (int r, SRule * pRegleExtens)
 {
    int                 i;
    PtrSSchema          pSc1;
-   SRule              *pRe1;
-   TtAttribute        *pAt1;
+   PtrSRule            pRe1;
+   PtrTtAttribute      pAt1;
 
    pSc1 = pSchStr;
    if (pRegleExtens != NULL)
       pRe1 = pRegleExtens;
    else
-      pRe1 = &pSc1->SsRule[r - 1];
+      pRe1 = pSc1->SsRule->SrElem[r - 1];
    if (pRe1->SrConstruct != CsNatureSchema &&
        !(pRe1->SrConstruct == CsPairedElement && !pRe1->SrFirstOfPair))
      {
@@ -607,10 +618,10 @@ static void         wrrule (int r, SRule * pRegleExtens)
 	     writenb (r);
 	     printf ("\n");
 
-	     if (pRe1->SrNLocalAttrs > 0)
+	     if (pRe1->SrNLocalAttrs > 0 && pRe1->SrLocalAttr)
 	       {
-		 for (i = 1; i <= pRe1->SrNLocalAttrs; i++)
-		   wrattr (pRe1->SrLocalAttr[i - 1]);
+		 for (i = 0; i < pRe1->SrNLocalAttrs; i++)
+		   wrattr (pRe1->SrLocalAttr->Num[i]);
 	       }
 	     return;
 	  }
@@ -635,14 +646,15 @@ static void         wrrule (int r, SRule * pRegleExtens)
 	      OpenComment ();
 	    wrnomregle (r);
 	  }
-	if (pRe1->SrNLocalAttrs > 0)
+	if (pRe1->SrNLocalAttrs > 0 &&
+	    pRe1->SrRequiredAttr && pRe1->SrLocalAttr)
 	  {
 	     printf (" (ATTR ");
-	     for (i = 1; i <= pRe1->SrNLocalAttrs; i++)
+	     for (i = 0; i < pRe1->SrNLocalAttrs; i++)
 	       {
-		  if (pRe1->SrRequiredAttr[i - 1])
+		  if (pRe1->SrRequiredAttr->Bln[i])
 		     printf ("!");
-		  wrattr (pRe1->SrLocalAttr[i - 1]);
+		  wrattr (pRe1->SrLocalAttr->Num[i]);
 		  if (i == pRe1->SrNLocalAttrs)
 		     printf (")");
 		  else
@@ -756,9 +768,11 @@ static void         wrrule (int r, SRule * pRegleExtens)
 		 case CsPairedElement:
 		    printf ("PAIR");
 		    break;
+	         case CsAny:
+		    printf ("ANY");
+		    break;
 		 case CsExtensionRule:
 	         case CsDocument:
-	         case CsAny:
 	         case CsEmpty:
 		    break;
 	      }
@@ -797,7 +811,7 @@ static void         wrrule (int r, SRule * pRegleExtens)
 	     printf ("%c%cWITH ", '\t', '\t');
 	     for (i = 1; i <= pRe1->SrNDefAttrs; i++)
 	       {
-		  pAt1 = &pSc1->SsAttribute[pRe1->SrDefAttr[i - 1] - 1];
+		  pAt1 = pSc1->SsAttribute->TtAttr[pRe1->SrDefAttr[i - 1] - 1];
 		  wrnom (pAt1->AttrName);
 		  if (pAt1->AttrType != AtReferenceAttr)
 		    {
@@ -858,6 +872,7 @@ void                printName (char *buffer)
   ----------------------------------------------------------------------*/
 int                 main (int argc, char **argv)
 {
+   PtrSRule            pRe1;
    char               *exec, c;
    int                 inputfile = False;
    int                 k, l;
@@ -918,7 +933,8 @@ int                 main (int argc, char **argv)
 	     printf (";\n");
 	  }
 	/* ecrit les attributs globaux */
-	if (pSc1->SsNAttributes > 0 && pSc1->SsAttribute[0].AttrGlobal)
+	if (pSc1->SsNAttributes > 0 &&
+	    pSc1->SsAttribute->TtAttr[0]->AttrGlobal)
 	  {
 	     if (!optionh)
 		printf ("ATTR\n");
@@ -927,15 +943,15 @@ int                 main (int argc, char **argv)
 	  }
 
 	for (i = 1; i <= pSc1->SsNAttributes; i++)
-	   if (pSc1->SsAttribute[i - 1].AttrGlobal)
+	   if (pSc1->SsAttribute->TtAttr[i - 1]->AttrGlobal)
 	     {
 		if (!optionh)
 		   printf ("   ");
-		pSc1->SsAttribute[i - 1].AttrGlobal = False;
+		pSc1->SsAttribute->TtAttr[i - 1]->AttrGlobal = False;
 		/* indique a wrattr qu'il faut */
 		/* ecrire les valeurs de l'attribut */
 		wrattr (i);
-		pSc1->SsAttribute[i - 1].AttrGlobal = True;
+		pSc1->SsAttribute->TtAttr[i - 1]->AttrGlobal = True;
 		if (!optionh)
 		   printf (";\n");
 	     }
@@ -944,7 +960,7 @@ int                 main (int argc, char **argv)
 	  {
 	     premattrlocal = True;
 	     for (i = 1; i <= pSc1->SsNAttributes; i++)
-		if (!pSc1->SsAttribute[i - 1].AttrGlobal)
+		if (!pSc1->SsAttribute->TtAttr[i - 1]->AttrGlobal)
 		  {
 		     if (premattrlocal)
 		       {
@@ -964,14 +980,14 @@ int                 main (int argc, char **argv)
 
 	/* ecrit les constantes */
 	r = MAX_BASIC_TYPE + 1;
-	if (pSc1->SsRule[r - 1].SrConstruct == CsConstant)
+	if (pSc1->SsRule->SrElem[r - 1]->SrConstruct == CsConstant)
 	  {
 	   if (!optionh)
 	     {
 		printf ("CONST\n");
-		while (pSc1->SsRule[r - 1].SrConstruct == CsConstant)
+		while (pSc1->SsRule->SrElem[r - 1]->SrConstruct == CsConstant)
 		  {
-		     pRe1 = &pSc1->SsRule[r - 1];
+		     pRe1 = pSc1->SsRule->SrElem[r - 1];
 		     printf (" ");
 		     OpenComment ();
 		     writenb (r);
@@ -992,7 +1008,7 @@ int                 main (int argc, char **argv)
 	   else
 	     {
 		printf ("\n/* Constants */\n");
-		while (pSc1->SsRule[r - 1].SrConstruct == CsConstant)
+		while (pSc1->SsRule->SrElem[r - 1]->SrConstruct == CsConstant)
 		  {
 		     wrrule (r, NULL);
 		     r++;
@@ -1010,7 +1026,7 @@ int                 main (int argc, char **argv)
 	  }
 	for (r = FirstRule; r <= pSc1->SsNRules; r++)
 	  {
-	     pRe1 = &pSc1->SsRule[r - 1];
+	     pRe1 = pSc1->SsRule->SrElem[r - 1];
 	     /* saute les elements Extern et Included et les unites */
 	     if (!pRe1->SrRefImportedDoc && !pRe1->SrUnitElem)
 	       wrrule (r, NULL);
@@ -1031,7 +1047,7 @@ int                 main (int argc, char **argv)
 	/* ecrit les unites exportees */
 	First = True;
 	for (r = FirstRule; r <= pSc1->SsNRules; r++)
-	   if (pSc1->SsRule[r - 1].SrUnitElem)
+	   if (pSc1->SsRule->SrElem[r - 1]->SrUnitElem)
 	     {
 		if (First)
 		  {
@@ -1051,7 +1067,7 @@ int                 main (int argc, char **argv)
 		printf ("THOT_EXPORT\n");
 		for (r = FirstRule; r <= pSc1->SsNRules; r++)
 		  {
-		     pRe1 = &pSc1->SsRule[r - 1];
+		     pRe1 = pSc1->SsRule->SrElem[r - 1];
 		     if (pRe1->SrExportedElem)
 		       {
 			  if (!First)
@@ -1078,7 +1094,7 @@ int                 main (int argc, char **argv)
 		/* ecrit d'abord les exceptions de tous les attributs */
 		for (a = 1; a <= pSc1->SsNAttributes; a++)
 		  {
-		     pAt1 = &pSc1->SsAttribute[a - 1];
+		     pAt1 = pSc1->SsAttribute->TtAttr[a - 1];
 		     if (pAt1->AttrFirstExcept > 0)
 			/* cet attribut a des exceptions */
 		       {
@@ -1098,7 +1114,7 @@ int                 main (int argc, char **argv)
 		/* ecrit les exceptions des types d'elements */
 		for (r = 1; r <= pSc1->SsNRules; r++)
 		  {
-		     pRe1 = &pSc1->SsRule[r - 1];
+		     pRe1 = pSc1->SsRule->SrElem[r - 1];
 		     if (pRe1->SrFirstExcept > 0)
 			/* ce type a des exceptions */
 		       {
@@ -1145,7 +1161,7 @@ int                 main (int argc, char **argv)
 	  {
 	     First = True;
 	     for (r = FirstRule; r <= pSc1->SsNRules; r++)
-		if (pSc1->SsRule[r - 1].SrConstruct == CsNatureSchema)
+		if (pSc1->SsRule->SrElem[r - 1]->SrConstruct == CsNatureSchema)
 		  {
 		     if (First)
 		       {
@@ -1165,7 +1181,7 @@ int                 main (int argc, char **argv)
 	  {
 	     First = True;
 	     for (r = FirstRule; r <= pSc1->SsNRules; r++)
-		if (pSc1->SsRule[r - 1].SrConstruct == CsNatureSchema)
+		if (pSc1->SsRule->SrElem[r - 1]->SrConstruct == CsNatureSchema)
 		  {
 		     if (First)
 		       {
@@ -1211,19 +1227,19 @@ int                 main (int argc, char **argv)
      {
 	printf ("document\n\ntranslation\n\n");
 	for (k = 0; k < pSchStr->SsNRules; k++)
-	   printName (&(pSchStr->SsRule[k].SrName[0]));
+	   printName (&(pSchStr->SsRule->SrElem[k]->SrName[0]));
 
 	for (k = 0; k < pSchStr->SsNExtensRules; k++)
 	   if (pSchStr->SsExtensBlock->EbExtensRule[k].SrName[0] != '\0')
 	      printName (&(pSchStr->SsExtensBlock->EbExtensRule[k].SrName[0]));
 	for (k = 0; k < pSchStr->SsNAttributes; k++)
-	   printName (&(pSchStr->SsAttribute[k].AttrName[0]));
+	   printName (&(pSchStr->SsAttribute->TtAttr[k]->AttrName[0]));
 
 	for (k = 0; k < pSchStr->SsNAttributes; k++)
 	  {
-	     if (pSchStr->SsAttribute[k].AttrType == AtEnumAttr)
-		for (l = 0; l < pSchStr->SsAttribute[k].AttrNEnumValues; l++)
-		   printName (&(pSchStr->SsAttribute[k].AttrEnumValue[l][0]));
+	     if (pSchStr->SsAttribute->TtAttr[k]->AttrType == AtEnumAttr)
+		for (l = 0; l < pSchStr->SsAttribute->TtAttr[k]->AttrNEnumValues; l++)
+		   printName (&(pSchStr->SsAttribute->TtAttr[k]->AttrEnumValue[l][0]));
 	  }
      }
    TtaSaveAppRegistry ();
