@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA  1996-2001
+ *  (c) COPYRIGHT INRIA  1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -10,7 +10,6 @@
  * en memoire.
  *
  * Author: V. Quint (INRIA)
- *         R. Guetari (W3C/INRIA): Unicode.
  *
  */
 
@@ -67,7 +66,7 @@ void                WriteName (Name name)
 
    i = 0;
    do
-      TtaWriteWideChar (outfile, name[i++], ISO_8859_1 /* UTF_8 */);
+      TtaWriteByte (outfile, name[i++]);
    while (name[i - 1] != '\0');
 }
 
@@ -173,7 +172,7 @@ void                WriteBasicType (BasicType typ)
 	    case Symbol:
 	       TtaWriteByte (outfile, C_SYMBOL);
 	       break;
-	    case Picture:
+	    case tt_Picture:
 	       TtaWriteByte (outfile, C_PICTURE);
 	       break;
 	    case Refer:
@@ -182,8 +181,8 @@ void                WriteBasicType (BasicType typ)
 	    case PageBreak:
 	       TtaWriteByte (outfile, C_PAGE_BREAK);
 	       break;
-	    case UnusedBasicType:
-	       TtaWriteByte (outfile, C_UNUSED);
+	    case AnyType:
+	       TtaWriteByte (outfile, C_ANY_TYPE);
 	       break;
 	 }
 }
@@ -205,9 +204,9 @@ static void         WriteRule (SRule * pSRule)
       WriteBoolean (pSRule->SrDefAttrModif[j]);
    WriteShort (pSRule->SrNLocalAttrs);
    for (j = 0; j < pSRule->SrNLocalAttrs; j++)
-      WriteShort (pSRule->SrLocalAttr[j]);
+      WriteShort (pSRule->SrLocalAttr->Num[j]);
    for (j = 0; j < pSRule->SrNLocalAttrs; j++)
-      WriteBoolean (pSRule->SrRequiredAttr[j]);
+      WriteBoolean (pSRule->SrRequiredAttr->Bln[j]);
    WriteBoolean (pSRule->SrUnitElem);
    WriteBoolean (pSRule->SrRecursive);
    WriteBoolean (pSRule->SrExportedElem);
@@ -285,7 +284,7 @@ static void         WriteRule (SRule * pSRule)
 ThotBool             WriteStructureSchema (Name fileName, PtrSSchema pSS,
 					   int code)
 {
-   TtAttribute        *pAttr;
+   PtrTtAttribute      pAttr;
    int                 i, j;
 
    /* ouvre le fichier */
@@ -316,7 +315,7 @@ ThotBool             WriteStructureSchema (Name fileName, PtrSSchema pSS,
    /* ecrit le texte des constantes */
    i = 0;
    do
-      TtaWriteWideChar (outfile, pSS->SsConstBuffer[i++], ISO_8859_1 /* UTF_8 */);
+      TtaWriteByte (outfile, pSS->SsConstBuffer[i++]);
    while (pSS->SsConstBuffer[i - 1] != '\0' || pSS->SsConstBuffer[i] != '\0');
 
    /* SsFirstDynNature */
@@ -325,7 +324,7 @@ ThotBool             WriteStructureSchema (Name fileName, PtrSSchema pSS,
    /* ecrit les attributs */
    for (i = 0; i < pSS->SsNAttributes; i++)
      {
-	pAttr = &pSS->SsAttribute[i];
+	pAttr = pSS->SsAttribute->TtAttr[i];
 	WriteName (pAttr->AttrName);
 	WriteBoolean (pAttr->AttrGlobal);
 	WriteShort (pAttr->AttrFirstExcept);
@@ -349,7 +348,7 @@ ThotBool             WriteStructureSchema (Name fileName, PtrSSchema pSS,
      }
    /* ecrit les regles de structure */
    for (i = 0; i < pSS->SsNRules; i++)
-      WriteRule (&pSS->SsRule[i]);
+      WriteRule (pSS->SsRule->SrElem[i]);
 
    /* ecrit les regles d'extension */
    if (pSS->SsExtension)
