@@ -1,24 +1,16 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, Grif, 1996.
+ *  (c) COPYRIGHT INRIA  1996-2001
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
 
-/*
- * Warning:
- * This module is part of the Thot library, which was originally
- * developed in French. That's why some comments are still in
- * French, but their translation is in progress and the full module
- * will be available in English in the next release.
- * 
- */
-
 /*
  * Ce module sauve dans un fichier un schema de traduction qui a ete
  * compile' en memoire.
  *
  * Author: V. Quint (INRIA)
+ *         R. Guetari (W3C/INRIA) : Unicode.
  *
  */
 
@@ -38,30 +30,16 @@ static BinFile      outfile;
 /*----------------------------------------------------------------------
    WriteShort      ecrit un entier sur 2 octets			
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 void                WriteShort (int n)
-#else  /* __STDC__ */
-void                WriteShort (n)
-int                 n;
-
-#endif /* __STDC__ */
 {
-   TtaWriteByte (outfile, (CHAR_T) (n / 256));
-   TtaWriteByte (outfile, (CHAR_T) (n % 256));
+   TtaWriteByte (outfile, (char) (n / 256));
+   TtaWriteByte (outfile, (char) (n % 256));
 }
 
 /*----------------------------------------------------------------------
    WriteSignedShort  ecrit un entier signe' sur 2 octets		
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 void                WriteSignedShort (int n)
-
-#else  /* __STDC__ */
-void                WriteSignedShort (n)
-int                 n;
-
-#endif /* __STDC__ */
-
 {
    if (n >= 0)
       WriteShort (n);
@@ -72,16 +50,7 @@ int                 n;
 /*----------------------------------------------------------------------
    WriteName                ecrit un nom				
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 void                WriteName (Name n)
-
-#else  /* __STDC__ */
-void                WriteName (n)
-Name                n;
-
-#endif /* __STDC__ */
-
 {
    int                 i;
 
@@ -95,16 +64,7 @@ Name                n;
 /*----------------------------------------------------------------------
    WriteBoolean       ecrit un booleen				
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 void                WriteBoolean (ThotBool b)
-
-#else  /* __STDC__ */
-void                WriteBoolean (b)
-ThotBool             b;
-
-#endif /* __STDC__ */
-
 {
 
    if (b)
@@ -118,18 +78,8 @@ ThotBool             b;
    WriteTransCondition  ecrit un type de condition d'application	
    de regle de traduction						
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 void                WriteTransCondition (TransCondition cond)
-
-#else  /* __STDC__ */
-void                WriteTransCondition (cond)
-TransCondition      cond;
-
-#endif /* __STDC__ */
-
 {
-
    switch (cond)
 	 {
 	    case TcondFirst:
@@ -138,9 +88,6 @@ TransCondition      cond;
 	    case TcondLast:
 	       TtaWriteByte (outfile, C_TR_LAST);
 	       break;
-	    case TcondDefined:
-	       TtaWriteByte (outfile, C_TR_DEFINED);
-	       break;
 	    case TcondReferred:
 	       TtaWriteByte (outfile, C_TR_REFERRED);
 	       break;
@@ -165,10 +112,7 @@ TransCondition      cond;
 	    case TcondPRule:
 	       TtaWriteByte (outfile, C_TR_PRULE);
 	       break;
-	    case TcondComment:
-	       TtaWriteByte (outfile, C_TR_COMMENT);
-	       break;
-	    case TcondAlphabet:
+	    case TcondScript:
 	       TtaWriteByte (outfile, C_TR_ALPHABET);
 	       break;
 	    case TcondAttributes:
@@ -209,16 +153,7 @@ TransCondition      cond;
 /*----------------------------------------------------------------------
    WriteRelatNAscend						
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 void                WriteRelatNAscend (RelatNAscend rel)
-
-#else  /* __STDC__ */
-void                WriteRelatNAscend (rel)
-RelatNAscend        rel;
-
-#endif /* __STDC__ */
-
 {
    switch (rel)
 	 {
@@ -238,18 +173,8 @@ RelatNAscend        rel;
 /*----------------------------------------------------------------------
    WriteTRuleType      ecrit un type de regle de traduction	
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 void                WriteTRuleType (TRuleType typ)
-
-#else  /* __STDC__ */
-void                WriteTRuleType (typ)
-TRuleType           typ;
-
-#endif /* __STDC__ */
-
 {
-
    switch (typ)
 	 {
 	    case TCreate:
@@ -264,6 +189,9 @@ TRuleType           typ;
 	    case TRemove:
 	       TtaWriteByte (outfile, C_TR_REMOVE);
 	       break;
+	    case TIgnore:
+	       TtaWriteByte (outfile, C_TR_IGNORE);
+	       break;
 	    case TWrite:
 	       TtaWriteByte (outfile, C_TR_WRITE);
 	       break;
@@ -306,18 +234,8 @@ TRuleType           typ;
    WriteTOrder ecrit la position ou il faut creer les chaines	
    produites par le traducteur					
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 void                WriteTOrder (TOrder order)
-
-#else  /* __STDC__ */
-void                WriteTOrder (order)
-TOrder              order;
-
-#endif /* __STDC__ */
-
 {
-
    switch (order)
 	 {
 	    case TAfter:
@@ -334,18 +252,8 @@ TOrder              order;
 /*----------------------------------------------------------------------
    WriteCreatedObject   ecrit le type de chose a creer		
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 void                WriteCreatedObject (CreatedObject obj)
-
-#else  /* __STDC__ */
-void                WriteCreatedObject (obj)
-CreatedObject       obj;
-
-#endif /* __STDC__ */
-
 {
-
    switch (obj)
 	 {
 	    case ToConst:
@@ -363,9 +271,6 @@ CreatedObject       obj;
 	    case ToContent:
 	       TtaWriteByte (outfile, C_OB_CONTENT);
 	       break;
-	    case ToComment:
-	       TtaWriteByte (outfile, C_OB_COMMENT);
-	       break;
 	    case ToAllAttr:
 	       TtaWriteByte (outfile, C_OB_ATTRIBUTES);
 	       break;
@@ -419,18 +324,8 @@ CreatedObject       obj;
 /*----------------------------------------------------------------------
    WriteRelativePosition   ecrit une position relative		
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 void                WriteRelativePosition (TRelatPosition pos)
-
-#else  /* __STDC__ */
-void                WriteRelativePosition (pos)
-TRelatPosition      pos;
-
-#endif /* __STDC__ */
-
 {
-
    switch (pos)
 	 {
 	    case RpSibling:
@@ -442,29 +337,15 @@ TRelatPosition      pos;
 	    case RpReferred:
 	       TtaWriteByte (outfile, C_TR_REFERRED_EL);
 	       break;
-	    case RpAssoc:
-	       TtaWriteByte (outfile, C_TR_ASSOC);
-	       break;
 	 }
-
 }
 
 
 /*----------------------------------------------------------------------
    WriteIndentType   ecrit un type de regle d'indentation		
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static void         WriteIndentType (TIndentType typ)
-
-#else  /* __STDC__ */
-static void         WriteIndentType (typ)
-TIndentType         typ;
-
-#endif /* __STDC__ */
-
 {
-
    switch (typ)
 	 {
 	    case ItAbsolute:
@@ -486,18 +367,8 @@ TIndentType         typ;
 /*----------------------------------------------------------------------
    WriteTCounterOp   ecrit le type d'une operation sur un compteur	
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 void                WriteTCounterOp (TCounterOp op)
-
-#else  /* __STDC__ */
-void                WriteTCounterOp (op)
-TCounterOp          op;
-
-#endif /* __STDC__ */
-
 {
-
    switch (op)
 	 {
 	    case TCntrRank:
@@ -513,23 +384,13 @@ TCounterOp          op;
 	       TtaWriteByte (outfile, C_TR_NOOP);
 	       break;
 	 }
-
 }
 
 
 /*----------------------------------------------------------------------
    WriteTVarType ecrit le type des elements de variables de traduction 
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 void                WriteTVarType (TranslVarType typ)
-
-#else  /* __STDC__ */
-void                WriteTVarType (typ)
-TranslVarType       typ;
-
-#endif /* __STDC__ */
-
 {
    switch (typ)
 	 {
@@ -566,18 +427,8 @@ TranslVarType       typ;
 /*----------------------------------------------------------------------
    WriteCounterStyle ecrit un style de compteur dans le fichier	
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 void                WriteCounterStyle (CounterStyle style)
-
-#else  /* __STDC__ */
-void                WriteCounterStyle (style)
-CounterStyle        style;
-
-#endif /* __STDC__ */
-
 {
-
    switch (style)
 	 {
 	    case CntArabic:
@@ -596,21 +447,12 @@ CounterStyle        style;
 	       TtaWriteByte (outfile, C_NUM_LOWERCASE);
 	       break;
 	 }
-
 }
 
 /*----------------------------------------------------------------------
    WriteTRulePtr	ecrit un pointeur de regle de traduction	
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 void                WriteTRulePtr (PtrTRule ptr)
-
-#else  /* __STDC__ */
-void                WriteTRulePtr (ptr)
-PtrTRule            ptr;
-
-#endif /* __STDC__ */
-
 {
    if (ptr == NULL)
       TtaWriteByte (outfile, '\0');
@@ -622,16 +464,7 @@ PtrTRule            ptr;
 /*----------------------------------------------------------------------
    WriteTRules  ecrit une suite de regles de traduction		
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 void                WriteTRules (PtrTRule pTRule)
-
-#else  /* __STDC__ */
-void                WriteTRules (pTRule)
-PtrTRule            pTRule;
-
-#endif /* __STDC__ */
-
 {
    PtrTRule            currentRule, nextRule;
 
@@ -662,6 +495,7 @@ PtrTRule            pTRule;
 		    WriteName (currentRule->TrTranslSchemaName);
 		    break;
 		 case TRemove:
+	         case TIgnore:
 		 case TNoTranslation:
 		 case TNoLineBreak:
 
@@ -699,17 +533,8 @@ PtrTRule            pTRule;
 /*----------------------------------------------------------------------
    WriteBlockPtr							
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 void                WriteBlockPtr (PtrTRuleBlock pBlock)
-
-#else  /* __STDC__ */
-void                WriteBlockPtr (pBlock)
-PtrTRuleBlock       pBlock;
-
-#endif /* __STDC__ */
-
 {
-
    if (pBlock == NULL)
       TtaWriteByte (outfile, '\0');
    else
@@ -720,17 +545,7 @@ PtrTRuleBlock       pBlock;
 /*----------------------------------------------------------------------
    WriteBlocks   ecrit une suite de blocs de regles		
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
-void                WriteBlocks (PtrTRuleBlock pBlock, PtrSSchema pSS)
-
-#else  /* __STDC__ */
-void                WriteBlocks (pBlock, pSS)
-PtrTRuleBlock       pBlock;
-PtrSSchema          pSS;
-
-#endif /* __STDC__ */
-
+void WriteBlocks (PtrTRuleBlock pBlock, PtrSSchema pSS)
 {
    PtrTRuleBlock       curBlock, nextBlock;
    TranslCondition    *pCond;
@@ -753,8 +568,8 @@ PtrSSchema          pSS;
 	     WriteSignedShort (pCond->TcAscendRelLevel);
 	     switch (pCond->TcCondition)
 		   {
-		      case TcondAlphabet:
-			 TtaWriteByte (outfile, pCond->TcAlphabet);
+		      case TcondScript:
+			 TtaWriteByte (outfile, pCond->TcScript);
 			 break;
 		      case TcondWithin:
 		      case TcondFirstWithin:
@@ -766,7 +581,7 @@ PtrSSchema          pSS;
 			 break;
 		      case TcondAttr:
 			 WriteShort (pCond->TcAttr);
-			 switch (pSS->SsAttribute[pCond->TcAttr - 1].AttrType)
+			 switch (pSS->SsAttribute->TtAttr[pCond->TcAttr - 1]->AttrType)
 			       {
 				  case AtNumAttr:
 				     WriteSignedShort (pCond->TcLowerBound);
@@ -821,25 +636,15 @@ PtrSSchema          pSS;
    WriteTRulesAttr  ecrit les regles de traduction de l'attribut	
    de numero att appartenant au schema de structure pSS		
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
-void                WriteTRulesAttr (int att, PtrSSchema pSS, PtrTSchema pTSch)
-
-#else  /* __STDC__ */
-void                WriteTRulesAttr (att, pSS, pTSch)
-int                 att;
-PtrSSchema          pSS;
-PtrTSchema          pTSch;
-
-#endif /* __STDC__ */
+void WriteTRulesAttr (int att, PtrSSchema pSS, PtrTSchema pTSch)
 {
-   AttributeTransl    *pAttrT;
+   PtrAttributeTransl  pAttrT;
    TranslNumAttrCase  *pCase;
    int                 i;
 
-   pAttrT = &pTSch->TsAttrTRule[att - 1];
+   pAttrT = pTSch->TsAttrTRule->TsAttrTransl[att - 1];
    WriteShort (pAttrT->AtrElemType);
-   switch (pSS->SsAttribute[att - 1].AttrType)
+   switch (pSS->SsAttribute->TtAttr[att - 1]->AttrType)
 	 {
 	    case AtNumAttr:
 	       WriteShort (pAttrT->AtrNCases);
@@ -859,7 +664,7 @@ PtrTSchema          pTSch;
 	       WriteBlocks (pAttrT->AtrRefTRuleBlock, pSS);
 	       break;
 	    case AtEnumAttr:
-	       for (i = 0; i <= pSS->SsAttribute[att - 1].AttrNEnumValues; i++)
+	       for (i = 0; i <= pSS->SsAttribute->TtAttr[att - 1]->AttrNEnumValues; i++)
 		  WriteBlocks (pAttrT->AtrEnuTRuleBlock[i], pSS);
 	       break;
 	    default:
@@ -871,18 +676,7 @@ PtrTSchema          pTSch;
    WritePRuleTrans ecrit les regles de traduction de la presentation 
    de numero pres							
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
-void                WritePRuleTrans (int pres, PtrSSchema pSS, PtrTSchema pTSch)
-
-#else  /* __STDC__ */
-void                WritePRuleTrans (pres, pSS, pTSch)
-int                 pres;
-PtrSSchema          pSS;
-PtrTSchema          pTSch;
-
-#endif /* __STDC__ */
-
+void WritePRuleTrans (int pres, PtrSSchema pSS, PtrTSchema pTSch)
 {
    PRuleTransl        *pPRuleTr;
    TranslNumAttrCase  *pCase;
@@ -890,6 +684,7 @@ PtrTSchema          pTSch;
 
    pPRuleTr = &pTSch->TsPresTRule[pres - 1];
    if (pPRuleTr->RtExist)
+     {
       if (pres == PtSize + 1 || pres == PtIndent + 1 ||
 	  pres == PtLineSpacing + 1 || pres == PtLineWeight + 1 ||
 	  pres == PtFillPattern + 1 || pres == PtBackground + 1 ||
@@ -911,6 +706,7 @@ PtrTSchema          pTSch;
 	      TtaWriteByte (outfile, pPRuleTr->RtPRuleValue[i]);
 	      WriteBlocks (pPRuleTr->RtPRuleValueBlock[i], pSS);
 	   }
+     }
 }
 
 
@@ -918,22 +714,13 @@ PtrTSchema          pTSch;
    WriteTranslationSchema cree le fichier de sortie et y ecrit le	
    schema de traduction						
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-ThotBool             WriteTranslationSchema (Name fileName, PtrTSchema pTSch, PtrSSchema pSS)
-
-#else  /* __STDC__ */
-ThotBool             WriteTranslationSchema (fileName, pTSch, pSS)
-Name                fileName;
-PtrTSchema          pTSch;
-PtrSSchema          pSS;
-
-#endif /* __STDC__ */
+ThotBool WriteTranslationSchema (Name fileName, PtrTSchema pTSch, PtrSSchema pSS)
 {
    TCounter           *pCntr;
    TranslVariable     *pVar;
    TranslVarItem      *pVarItem;
-   AttributeTransl    *pAttrTr;
-   AlphabetTransl     *pAlphTr;
+   PtrAttributeTransl  pAttrTr;
+   ScriptTransl     *pAlphTr;
    StringTransl       *pStrnTr;
    PRuleTransl        *pPruleTr;
    int                 i, j;
@@ -989,40 +776,46 @@ PtrSSchema          pSS;
 	  }
      }
    WriteShort (pTSch->TsPictureBuffer);
+   WriteShort (pTSch->TsNVarBuffers);
+   for (i = 0; i < pTSch->TsNVarBuffers; i++)
+     {
+       WriteName (pTSch->TsVarBuffer[i].VbIdent);
+       WriteShort (pTSch->TsVarBuffer[i].VbNum);
+     }
    for (i = 0; i < pSS->SsNRules; i++)
-      WriteBlockPtr (pTSch->TsElemTRule[i]);
+      WriteBlockPtr (pTSch->TsElemTRule->TsElemTransl[i]);
    for (i = 0; i < pSS->SsNRules; i++)
-      WriteBoolean (pTSch->TsInheritAttr[i]);
+      WriteBoolean (pTSch->TsInheritAttr->Bln[i]);
    for (i = 0; i < pSS->SsNAttributes; i++)
      {
-	pAttrTr = &pTSch->TsAttrTRule[i];
-	switch (pSS->SsAttribute[i].AttrType)
-	      {
-		 case AtNumAttr:
-		    WriteShort (pAttrTr->AtrNCases);
-		    for (j = 0; j < pAttrTr->AtrNCases; j++)
-		       WriteBlockPtr (pAttrTr->AtrCase[j].TaTRuleBlock);
-		    break;
-		 case AtTextAttr:
-		    WriteBlockPtr (pAttrTr->AtrTxtTRuleBlock);
-		    break;
-		 case AtReferenceAttr:
-		    WriteBlockPtr (pAttrTr->AtrRefTRuleBlock);
-		    break;
-		 case AtEnumAttr:
-		    for (j = 0; j <= pSS->SsAttribute[i].AttrNEnumValues; j++)
-		       WriteBlockPtr (pAttrTr->AtrEnuTRuleBlock[j]);
-		    break;
-		 default:
-		    break;
-	      }
-
+       pAttrTr = pTSch->TsAttrTRule->TsAttrTransl[i];
+       switch (pSS->SsAttribute->TtAttr[i]->AttrType)
+	 {
+	 case AtNumAttr:
+	   WriteShort (pAttrTr->AtrNCases);
+	   for (j = 0; j < pAttrTr->AtrNCases; j++)
+	     WriteBlockPtr (pAttrTr->AtrCase[j].TaTRuleBlock);
+	   break;
+	 case AtTextAttr:
+	   WriteBlockPtr (pAttrTr->AtrTxtTRuleBlock);
+	   break;
+	 case AtReferenceAttr:
+	   WriteBlockPtr (pAttrTr->AtrRefTRuleBlock);
+	   break;
+	 case AtEnumAttr:
+	   for (j = 0; j <= pSS->SsAttribute->TtAttr[i]->AttrNEnumValues; j++)
+	     WriteBlockPtr (pAttrTr->AtrEnuTRuleBlock[j]);
+	   break;
+	 default:
+	   break;
+	 }
      }
    for (i = 0; i < MAX_TRANSL_PRULE; i++)
      {
 	pPruleTr = &pTSch->TsPresTRule[i];
 	WriteBoolean (pPruleTr->RtExist);
 	if (pPruleTr->RtExist)
+	  {
 	   if (i == PtSize || i == PtIndent || i == PtLineSpacing ||
 	     i == PtLineWeight || i == PtFillPattern || i == PtBackground ||
 	       i == PtForeground)
@@ -1035,12 +828,13 @@ PtrSSchema          pSS;
 	   else
 	      for (j = 0; j <= MAX_TRANSL_PRES_VAL; j++)
 		 WriteBlockPtr (pPruleTr->RtPRuleValueBlock[j]);
+	  }
      }
-   WriteShort (pTSch->TsNTranslAlphabets);
-   for (i = 0; i < pTSch->TsNTranslAlphabets; i++)
+   WriteShort (pTSch->TsNTranslScripts);
+   for (i = 0; i < pTSch->TsNTranslScripts; i++)
      {
-	pAlphTr = &pTSch->TsTranslAlphabet[i];
-	TtaWriteByte (outfile, pAlphTr->AlAlphabet);
+	pAlphTr = &pTSch->TsTranslScript[i];
+	TtaWriteByte (outfile, pAlphTr->AlScript);
 	WriteShort (pAlphTr->AlBegin);
 	WriteShort (pAlphTr->AlEnd);
      }
@@ -1069,7 +863,7 @@ PtrSSchema          pSS;
 	while (pTSch->TsConstant[j - 1] != '\0');
      }
    for (i = 0; i < pSS->SsNRules; i++)
-      WriteBlocks (pTSch->TsElemTRule[i], pSS);
+      WriteBlocks (pTSch->TsElemTRule->TsElemTransl[i], pSS);
    for (i = 0; i < pSS->SsNAttributes; i++)
       WriteTRulesAttr (i + 1, pSS, pTSch);
    for (i = 0; i < MAX_TRANSL_PRULE; i++)
