@@ -1,5 +1,5 @@
 /*
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  */
 
@@ -10,12 +10,13 @@
  * Authors: I. Vatton (INRIA)
  * Changed: S. Bonhomme (INRIA) 
  */
-
 #include "thot_gui.h"
 #include "thot_sys.h"
 #include "appstruct.h"
 #include "constmedia.h"
 #include "appdialogue.h"
+
+
 #undef THOT_EXPORT
 #define THOT_EXPORT extern
 
@@ -37,6 +38,14 @@ void TtaSetBackup (Proc procedure)
   TteConnectAction (T_backuponfatal, procedure);
 }
  
+/*----------------------------------------------------------------------
+   TtaSetAutoSave connects the AutoSave function: procedure().
+  ----------------------------------------------------------------------*/
+void TtaSetAutoSave (Proc procedure)
+{
+  TteConnectAction (T_autosave, procedure);
+}
+ 
 /*----------------------------------------------------------------------
    TtaSetDocStatusUpdate connects the function: procedure().
    That procedure will receive two paramters:
