@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -8,13 +8,12 @@
 /*
  * gestion des fichiers de configuration et de langue.
  *
- * Authors: V. Quint (INRIA)
+ * Authors: V. Quint (IRNIA)
  *
  */
 
 #include "thot_gui.h"
 #include "thot_sys.h"
-#include "thotdir.h"
 #include "constmedia.h"
 #include "constmenu.h"
 #include "typemedia.h"
@@ -24,9 +23,12 @@
 #include "appdialogue.h"
 #include "fileaccess.h"
 #include "document.h"
-#ifdef _WINDOWS
-#include "wininclude.h"
-#endif /* _WINDOWS */
+
+#ifdef _WINGUI
+  #include "wininclude.h"
+  int app_lang;
+#endif /* _WINGUI */
+
 #undef THOT_EXPORT
 #define THOT_EXPORT extern
 #include "platform_tv.h"
@@ -78,7 +80,7 @@ void ConfigInit ()
       export_items[i] = NULL;
       export_items_menu[i] = NULL;
     }
-  TtaConfigReadConfigFiles (SchemaPath);
+  /* TtaConfigReadConfigFiles (SchemaPath);*/
 }
 
 /*----------------------------------------------------------------------
@@ -230,8 +232,8 @@ static ThotBool readUntil (FILE *file, char *word1, char *word2)
 	 stop = TRUE;
       else
 	{
-	   getFirstWord (line, word);
-	   if (singleWord (line))
+	   getFirstWord ((unsigned char*)line, word);
+	   if (singleWord ((unsigned char*)line))
 	     {
 		if (*word1 != EOS)
 		   if (strcmp (word, word1) == 0)
@@ -245,6 +247,7 @@ static ThotBool readUntil (FILE *file, char *word1, char *word2)
    return ret;
 }
 
+#if 0
 /*----------------------------------------------------------------------
    namesOfDocType                                                  
   ----------------------------------------------------------------------*/
@@ -389,6 +392,7 @@ static void namesOfDocType (char *fname, char **doctypeOrig,
 	strcpy (*doctypeTrans, *doctypeOrig);
      }
 }
+#endif
 
 /*----------------------------------------------------------------------
    ConfigFree
@@ -434,7 +438,7 @@ void                ConfigFree (void)
 	  }
      }
    }
-
+#ifdef IV
 /*----------------------------------------------------------------------
    TtaConfigReadConfigFiles (re)initialise les tables des schemas de
    structure (documents, natures et extensions) qui ont    
@@ -448,7 +452,6 @@ void TtaConfigReadConfigFiles (char *aSchemaPath)
   ThotBool            import;
   char               *Dir;
   PathBuffer          DirBuffer;
-  ThotDirBrowse       thotDir;
 #define NAME_LENGTH     100
 #define MAX_NAME         80
 #define SELECTOR_NB_ITEMS 5
@@ -460,9 +463,9 @@ void TtaConfigReadConfigFiles (char *aSchemaPath)
 
   /* force the english language in schemas */
   suffix = "en";
-#ifdef _WINDOWS
+#ifdef _WINGUI
   app_lang = EN_LANG;
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
   /* libere les anciennes entrees des tables de types de documents */
   /* de natures et d'extensions */
   ConfigFree ();
@@ -492,47 +495,6 @@ void TtaConfigReadConfigFiles (char *aSchemaPath)
 	    /* c'est bien un directory */
 	    {
 	      /* commande "ls" sur le directory */
-	      thotDir.buf = fname;
-	      thotDir.bufLen = sizeof (fname) / sizeof (char);
-	      thotDir.PicMask = ThotDirBrowse_FILES;
-	      if (ThotDirBrowse_first (&thotDir, Dir, "*.", suffix) == 1)
-		do
-		  {
-		    namesOfDocType (fname, &nameOrig, &nameTrans, &typ, &import);
-		    if (nameOrig != NULL)
-		      {
-			if (typ == CONFIG_DOCUMENT_STRUCT || typ == CONFIG_EXCLUSION)
-			  {
-			    doc_items[nbitemdoc] = nameOrig;
-			    doc_items_menu[nbitemdoc] = nameTrans;
-			    if (import)
-			      doc_import[nbitemdoc] = TRUE;
-			    nbitemdoc++;
-			  }
-			if (typ == CONFIG_NATURE_STRUCT)
-			  {
-			    nat_items[nbitemnat] = nameOrig;
-			    nat_items_menu[nbitemnat] = nameTrans;
-			    nbitemnat++;
-			  }
-			if (typ == CONFIG_EXCLUSION)
-			  {
-			    nat_items[nbitemnat] = TtaGetMemory (strlen (nameOrig) + 1);
-			    nat_items_menu[nbitemnat] = TtaGetMemory (strlen (nameTrans) + 1);
-			    strcpy (nat_items[nbitemnat], nameOrig);
-			    strcpy (nat_items_menu[nbitemnat], nameTrans);
-			    nbitemnat++;
-			  }
-			if (typ == CONFIG_EXTENSION_STRUCT)
-			  {
-			    ext_items[nbitemext] = nameOrig;
-			    ext_items_menu[nbitemext] = nameTrans;
-			    nbitemext++;
-			  }
-		      }
-		  }
-		while (ThotDirBrowse_next (&thotDir) == 1);
-	      ThotDirBrowse_close (&thotDir);
 	    }
 	  /* continue a chercher les directories dans le path des schemas */
 	  i++;
@@ -541,7 +503,6 @@ void TtaConfigReadConfigFiles (char *aSchemaPath)
     }
 }
 
-
 /*----------------------------------------------------------------------
    ConfigMakeDocTypeMenu cree dans BufMenu la liste des schemas de 
    structure qui ont des fichiers de langue dans les       
@@ -704,6 +665,7 @@ void ConfigSSchemaInternalName (char *nameUser, char *nameSchema,
 	      i++;
 	}
 }
+#endif /* IV */
 
 
 /*----------------------------------------------------------------------
@@ -732,6 +694,15 @@ static FILE *openConfigFile (char *name, ThotBool lang)
        /* des schemas et le suffixe */
        strncpy (DirBuffer, SchemaPath, MAX_PATH);
        MakeCompleteName (name, suffix, DirBuffer, filename, &i);
+       if (!TtaFileExist (filename))
+	 {
+	   if (strcmp (name, "Annot"))
+	     {
+	       /* it's probably a generic name */
+	       strncpy (DirBuffer, SchemaPath, MAX_PATH);
+	       MakeCompleteName ("XML", suffix, DirBuffer, filename, &i);
+	     }
+	 }
      }
    /* ouvre le fichier */
    file = TtaReadOpen (filename);
@@ -768,13 +739,13 @@ int ConfigMakeMenuPres (char *schema, char *BufMenu)
 	   stop = TRUE;
 	 else
 	   {
-	     getFirstWord (line, word);
+	     getFirstWord ((unsigned char*)line, word);
 	     if (word[0] != EOS)
 	       /* la ligne n'est pas vide */
 	       {
 		 /* si la ligne contient un mot cle marquant le debut */
 		 /* d'une autre section, on a fini */
-		 if (singleWord (line))
+		 if (singleWord ((unsigned char*)line))
 		   {
 		     if (strcmp (word, "export") == 0)
 		       stop = TRUE;
@@ -785,22 +756,22 @@ int ConfigMakeMenuPres (char *schema, char *BufMenu)
 		   }
 		 if (!stop)
 		   {
-		     getStringAfterColon (line, text);
+		     getStringAfterColon ((unsigned char*)line, text);
 		     if (text[0] == EOS)
 		       fprintf (stderr, "invalid line in file %s\n   %s\n",
 				schema, line);
 		     else
 		       {
-			 strcpy (textISO, AsciiTranslate (text));
+			 strcpy (textISO, (char *)AsciiTranslate (text));
 			 if (pres_items[nbitem] != NULL)
 			   TtaFreeMemory (pres_items[nbitem]);
-			 pres_items[nbitem] = TtaGetMemory (strlen(word)+1);
-			 strcpy (pres_items[nbitem], word);
+			 pres_items[nbitem] = (char *)TtaGetMemory (strlen(word)+1);
+			 strcpy (pres_items[nbitem], (char *)word);
 			 if (pres_items_menu[nbitem] != NULL)
 			   TtaFreeMemory (pres_items_menu[nbitem]);
 			 len = strlen (textISO) + 1;
-			 pres_items_menu[nbitem] = TtaGetMemory (len);
-			 strcpy (pres_items_menu[nbitem], textISO);
+			 pres_items_menu[nbitem] = (char *)TtaGetMemory (len);
+			 strcpy (pres_items_menu[nbitem], (char *)textISO);
 			 if (BufMenu != NULL)
 			   {
 			     strcpy (&BufMenu[indmenu], textISO);
@@ -898,13 +869,13 @@ int ConfigMakeMenuExport (char *schema, char *BufMenu)
 	      stop = TRUE;
 	   else
 	     {
-		getFirstWord (line, word);
+		getFirstWord ((unsigned char*)line, word);
 		if (word[0] != EOS)
 		   /* la ligne n'est pas vide */
 		  {
 		    /* si la ligne contient un mot cle marquant le debut */
 		    /* d'une autre section, on a fini */
-		     if (singleWord (line))
+		     if (singleWord ((unsigned char*)line))
 		       {
 			if (strcmp (word, "presentation") == 0)
 			   stop = TRUE;
@@ -915,20 +886,20 @@ int ConfigMakeMenuExport (char *schema, char *BufMenu)
 		       }
 		     if (!stop)
 		       {
-			  getStringAfterColon (line, text);
+			  getStringAfterColon ((unsigned char*)line, text);
 			  if (text[0] == EOS)
 			     fprintf (stderr, "invalid line in file %s\n   %s\n", schema, line);
 			  else
 			    {
-			       strcpy (textISO, AsciiTranslate (text));
+			       strcpy (textISO, (char *)AsciiTranslate (text));
 			       if (export_items[nbitem] != NULL)
 				  TtaFreeMemory (export_items[nbitem]);
-			       export_items[nbitem] = TtaGetMemory (strlen (word) + 10);
+			       export_items[nbitem] = (char *)TtaGetMemory (strlen (word) + 10);
 			       strcpy (export_items[nbitem], word);
 			       if (export_items_menu[nbitem] != NULL)
 				  TtaFreeMemory (export_items_menu[nbitem]);
 			       len = strlen (textISO) + 1;
-			       export_items_menu[nbitem] = TtaGetMemory (len);
+			       export_items_menu[nbitem] = (char *)TtaGetMemory (len);
 			       strcpy (export_items_menu[nbitem], textISO);
 			       if (BufMenu != NULL)
 				 {
@@ -964,43 +935,51 @@ static ThotBool Translate (PtrSSchema pSS, char *word, char *trans)
 {
    ThotBool            found;
    int                 i, j;
-   TtAttribute        *pAttr;
-   char                terme[MAX_NAME_LENGTH];
+   PtrTtAttribute      pAttr;
 
    found = FALSE;
-   strncpy (terme, AsciiTranslate (word), MAX_NAME_LENGTH - 1);
    /* cherche le mot a traduire d'abord parmi les noms d'elements */
    for (i = 0; i < pSS->SsNRules; i++)
-      if (strcmp (terme, pSS->SsRule[i].SrName) == 0)
-	{
-	   strncpy (pSS->SsRule[i].SrName, AsciiTranslate (trans), MAX_NAME_LENGTH - 1);
-	   found = TRUE;
-	}
+     if (pSS->SsRule->SrElem[i]->SrName != NULL &&
+	 strcmp ((char *)AsciiTranslate (word), pSS->SsRule->SrElem[i]->SrName) == 0)
+       {
+	 TtaFreeMemory (pSS->SsRule->SrElem[i]->SrName);
+	 pSS->SsRule->SrElem[i]->SrName = TtaStrdup ((char *)AsciiTranslate (trans));
+	 found = TRUE;
+       }
+
    /* cherche ensuite parmi les noms d'attributs et de valeurs d'attributs */
    for (i = 0; i < pSS->SsNAttributes; i++)
 	{
-	   pAttr = &pSS->SsAttribute[i];
-	   if (strcmp (terme, pAttr->AttrName) == 0)
+	   pAttr = pSS->SsAttribute->TtAttr[i];
+	   if (pAttr->AttrName != NULL &&
+	       strcmp ((char *)AsciiTranslate (word), pAttr->AttrName) == 0)
 	     {
-		strncpy (pAttr->AttrName, AsciiTranslate (trans), MAX_NAME_LENGTH - 1);
-		found = TRUE;
+	       TtaFreeMemory (pAttr->AttrName);
+	       pAttr->AttrName = TtaStrdup ((char *)AsciiTranslate (trans));
+	       found = TRUE;
 	     }
 	   else if (pAttr->AttrType == AtEnumAttr)
 	      for (j = 0; j < pAttr->AttrNEnumValues; j++)
-		 if (strcmp (terme, pAttr->AttrEnumValue[j]) == 0)
+		 if (strcmp ((char *)AsciiTranslate (word), pAttr->AttrEnumValue[j]) == 0)
 		   {
-		      strncpy (pAttr->AttrEnumValue[j], AsciiTranslate (trans), MAX_NAME_LENGTH - 1);
+		      strncpy (pAttr->AttrEnumValue[j],
+			       (char *)AsciiTranslate (trans), MAX_NAME_LENGTH - 1);
 		      found = TRUE;
 		   }
 	}
+
    /* cherche enfin parmi les regles d'extension, si c'est un schema d'extension */
    if (pSS->SsExtension)
       if (pSS->SsNExtensRules > 0 && pSS->SsExtensBlock != NULL)
 	 for (i = 0; i < pSS->SsNExtensRules; i++)
-	    if (strcmp (terme, pSS->SsExtensBlock->EbExtensRule[i].SrName) == 0)
+	    if (pSS->SsExtensBlock->EbExtensRule[i].SrName != NULL &&
+		strcmp ((char *)AsciiTranslate (word), pSS->SsExtensBlock->EbExtensRule[i].SrName) == 0)
 	      {
-		 strncpy (pSS->SsExtensBlock->EbExtensRule[i].SrName, AsciiTranslate (trans), MAX_NAME_LENGTH - 1);
-		 found = TRUE;
+		TtaFreeMemory (pSS->SsExtensBlock->EbExtensRule[i].SrName);
+		pSS->SsExtensBlock->EbExtensRule[i].SrName =
+		  TtaStrdup ((char *)AsciiTranslate (trans));
+		found = TRUE;
 	      }
    return found;
 }
@@ -1026,9 +1005,9 @@ void ConfigTranslateSSchema (PtrSSchema pSS)
    stop = FALSE;
    /* avance dans le fichier jusqu'a la ligne qui contient le seul */
    /* mot "translation" */
-   line = TtaGetMemory (MAX_TXT_LEN);
-   text = TtaGetMemory (MAX_TXT_LEN);
-   word = TtaGetMemory (MAX_TXT_LEN);
+   line = (char *)TtaGetMemory (MAX_TXT_LEN);
+   text = (char *)TtaGetMemory (MAX_TXT_LEN);
+   word = (char *)TtaGetMemory (MAX_TXT_LEN);
    if (readUntil (file, "translation", ""))
       /* lit le fichier ligne a ligne */
       do
@@ -1042,13 +1021,13 @@ void ConfigTranslateSSchema (PtrSSchema pSS)
 	   else
 	     {
 		/* prend le premier mot de la ligne */
-		getFirstWord (line, word);
+		getFirstWord ((unsigned char*)line, word);
 		if (word[0] != EOS)
 		   /* la ligne n'est pas vide */
 		  {
 		    /* si la ligne contient un mot cle marquant le debut */
 		    /* d'une autre section, on a fini */
-		    if (singleWord (line))
+		    if (singleWord ((unsigned char*)line))
 		      {
 			if (strcmp (word, "presentation") == 0)
 			  stop = TRUE;
@@ -1066,7 +1045,7 @@ void ConfigTranslateSSchema (PtrSSchema pSS)
 		    if (!stop && !error)
 		      {
 			/* cherche la chaine de caracteres qui suit ':' */
-			getStringAfterColon (line, text);
+			getStringAfterColon ((unsigned char*)line, text);
 			if (text[0] == EOS)
 			  fprintf (stderr, "invalid line in file %s\n   %s\n",
 				   pSS->SsName, line);
@@ -1113,12 +1092,12 @@ ThotBool ConfigDefaultPSchema (char *schstr, char *schpres)
 	   else
 	     {
 		/* prend le premier mot de la ligne */
-		getFirstWord (line, word);
+		getFirstWord ((unsigned char*)line, word);
 		if (strcmp (word, "style") == 0)
 		  {
 		     /* le 1er mot est "style". Cherche le mot qui suit : c'est le */
 		     /* nom du schema de presentation cherche' */
-		     getSecondWord (line, word);
+		     getSecondWord ((unsigned char *)line, word);
 		     if (word[0] != EOS)
 			/* il y a bien un 2eme mot : succes */
 		       {
@@ -1156,12 +1135,13 @@ static ThotBool readUntilStyle (FILE *file, char *namePSchema)
         stop = TRUE;
      else
         {
-           getFirstWord (line, word);
+           getFirstWord ((unsigned char*)line, word);
            if (strcmp (word, "style") == 0)
               {
-                 getSecondWord (line, word);
+                 getSecondWord ((unsigned char *)line, word);
                  strcpy (name, word);
-                 if (strcmp (name, namePSchema) == 0)
+                 if (strcmp (name, namePSchema) == 0 ||
+		     strcmp (name, "XMLP") == 0)
                     ok = TRUE;
               }
         }
@@ -1230,13 +1210,13 @@ static ThotBool getNextLineInSection (FILE * file, char *line)
       else
 	{
 	   /* prend le permier mot de la ligne lue */
-	   getFirstWord (line, word1);
+	   getFirstWord ((unsigned char*)line, word1);
 	   if (word1[0] != EOS)
 	      /* la ligne n'est pas vide */
 	     {
 		/* si la ligne contient un mot cle marquant le debut d'une autre */
 		/* section, on a fini */
-		if (singleWord (line))
+		if (singleWord ((unsigned char*)line))
 		   /* la ligne contient un seul mot */
 		  {
 		     if (strcmp (word1, "open") == 0)
@@ -1255,7 +1235,7 @@ static ThotBool getNextLineInSection (FILE * file, char *line)
 		   /* la ligne contient plus d'un mot */
 		if (strcmp (word1, "style") == 0)
 		  {
-		     getSecondWord (line, word2);
+		     getSecondWord ((unsigned char *)line, word2);
 		     if (word2[0] != ':')
 			/* la ligne est du type "style xxxx". C'est une fin de section */
 			stop = TRUE;
@@ -1290,7 +1270,7 @@ void             ConfigKeyboard (int *x, int *y)
    getNextLineInSection (file, line);
 
    /* extrait la partie de la ligne qui suit les deux-points */
-   getStringAfterColon (line, seqLine);
+   getStringAfterColon ((unsigned char*)line, seqLine);
    if (seqLine[0] != EOS)
      /* extrait les 4 entiers */
      nbIntegers = sscanf (seqLine, "%d %d", x, y);
@@ -1312,7 +1292,7 @@ static ThotBool getXYWidthHeight (char *line, PtrDocument pDoc, int *x,
 
   result = FALSE;
   /* extrait la partie de la ligne qui suit les deux-points */
-  getStringAfterColon (line, seqLine);
+  getStringAfterColon ((unsigned char*)line, seqLine);
   if (seqLine[0] != EOS)
     {
       /* extrait les 4 entiers */
@@ -1347,7 +1327,7 @@ void ConfigOpenFirstViews (PtrDocument pDoc)
          while (getNextLineInSection (file, line))
                {
                   /* le 1er mot de la ligne est le nom d'une vue a ouvrir */
-                  getFirstWord (line, nameview);
+                  getFirstWord ((unsigned char*)line, nameview);
                   /* lit les coordonnees (x, y) et dimensions (width, height) de la */
                   /* frame ou doit s'afficher la vue */
                   if (getXYWidthHeight (line, pDoc, &x, &y, &width, &height))
@@ -1388,7 +1368,7 @@ void  ConfigGetViewGeometry (PtrDocument pDoc, char *view, int *x,
        while (!found && getNextLineInSection (file, line))
 	 {
 	   /* le 1er mot de la ligne est le nom d'une vue */
-	   getFirstWord (line, nameview);
+	   getFirstWord ((unsigned char*)line, nameview);
 	   /* est-ce le nom de la vue cherchee ? */
 	   found = (strcmp (nameview, view) == 0);
 	 }
@@ -1402,7 +1382,7 @@ void  ConfigGetViewGeometry (PtrDocument pDoc, char *view, int *x,
 	       while (!found && getNextLineInSection (file, line))
 		 {
 		   /* le 1er mot de la ligne est le nom d'une vue */
-		   getFirstWord (line, nameview);
+		   getFirstWord ((unsigned char*)line, nameview);
 		   /* est-ce le nom de la vue cherchee ? */
 		   found = (strcmp (nameview, view) == 0);
 		 }
@@ -1420,26 +1400,51 @@ void  ConfigGetViewGeometry (PtrDocument pDoc, char *view, int *x,
 void TtaGetViewXYWH (Document doc, int view, int *xmm, int *ymm, int *width,
 		     int *height)
 {
-#ifndef _WINDOWS
+#ifdef _WX
+
+  /* TODO
+   * 
+  int 		frame;
+  ThotFrame	pFrame = NULL;
+    
+  frame = GetWindowNumber (doc, view);
+  pFrame = FrameTable[frame].WdFrame;
+
+  if (pFrame)
+    {
+      pFrame->GetPosition( xmm, ymm );
+      pFrame->GetSize( width,height );
+    }*/
+#endif /* _WX */
+#if defined(_MOTIF) || defined(_GTK)
   int                 frame;
-  int                 n;
   ThotWidget          widget;
 #ifdef _GTK
-  GtkArg              args[20];
-#else /* ! _GTK */
+  ThotWidget          tmpw;
+#endif /* _GTK */
+#ifdef _MOTIF  
+  int                 n;
   Arg                 args[20];
-#endif /* ! _GTK */
+#endif /* _MOTIF */
   Position            x, y;
   Dimension           w, h;
 
   frame =  GetWindowNumber (doc, view);
   widget = (ThotWidget) FrameTable[frame].WdFrame;
 #ifdef _GTK
-  widget = widget->parent->parent;
-  args[0].name = "width";
-  args[1].name = "height";
-  gtk_object_getv(GTK_OBJECT(widget), 2, args);
-#else /* !_GTK */
+  /*tmpw = GTK_WIDGET (widget);*/
+  tmpw = gtk_widget_get_toplevel (GTK_WIDGET (widget));
+  /* values of w h are not realy exact, there are too much pixel (2 or 3) */
+  w = tmpw->allocation.width - 4;
+  h = tmpw->allocation.height - 4;
+  x = tmpw->allocation.x;
+  y = tmpw->allocation.y;
+  *xmm = x;
+  *ymm = y;
+  *width = w;
+  *height = h;
+#endif /* _GTK */
+#ifdef _MOTIF  
   widget = XtParent (XtParent (XtParent (widget)));
 
   /* Ask X what's the geometry of the frame */
@@ -1453,14 +1458,15 @@ void TtaGetViewXYWH (Document doc, int view, int *xmm, int *ymm, int *width,
   XtSetArg (args[n], XmNheight, &h);
   n++;
   XtGetValues (widget, args, n);
-#endif /* !_GTK */
   /* convert the result into mm */
   *xmm = x;
   /* take into account the window manager headband */
   *ymm = y - 18;
   *width = w;
   *height = h;
-#else /* !_WINDOWS */
+#endif /* _MOTIF */
+#endif /* #if defined(_MOTIF) || defined(_GTK) */
+#ifdef _WINGUI
   int  frame;
   HWND hWnd;
   RECT rect;
@@ -1478,7 +1484,7 @@ void TtaGetViewXYWH (Document doc, int view, int *xmm, int *ymm, int *width,
       *width = (int) (rect.right - rect.left);
       *height = (int) (rect.bottom - rect.top);
     }
-#endif /* !_WINDOWS */
+#endif /* _WINGUI */
 }
 
 /*----------------------------------------------------------------------
@@ -1558,7 +1564,7 @@ ThotBool ConfigGetPSchemaNature (PtrSSchema pSS, char *nameNature,
 	while (!found && getNextLineInSection (file, line))
 	  {
 	     /* le 1er mot de la ligne est le nom d'une nature */
-	     getFirstWord (line, name);
+	     getFirstWord ((unsigned char*)line, name);
 	     /* est-ce le nom de la nature cherchee ? */
 	     found = (strcmp (name, nameNature) == 0);
 	  }
@@ -1568,7 +1574,7 @@ ThotBool ConfigGetPSchemaNature (PtrSSchema pSS, char *nameNature,
 	  {
 	     /* le nom de nature est suivi, apres ":", du nom du schema de */
 	     /* presentation a appliquer */
-	     getStringAfterColon (line, seqLine);
+	     getStringAfterColon ((unsigned char*)line, seqLine);
 	     if (seqLine[0] == EOS)
 		fprintf (stderr, "invalid line in file %s.conf\n   %s\n", pSS->SsName, line);
 	     else
@@ -1610,7 +1616,7 @@ void ConfigGetPresentationOption (PtrSSchema pSS, char *optionName,
 	while (!found && getNextLineInSection (file, line))
 	  {
 	     /* le 1er mot de la ligne est le nom d'une option */
-	     getFirstWord (line, name);
+	     getFirstWord ((unsigned char*)line, name);
 	     /* est-ce le nom de l'option cherchee ? */
 	     found = (strcmp (name, optionName) == 0);
 	  }
@@ -1619,7 +1625,7 @@ void ConfigGetPresentationOption (PtrSSchema pSS, char *optionName,
 	   /* le nom de l'option voulue */
 	  {
 	     /* le nom de l'option est suivi, apres ":", de la valeur de l'option */
-	     getStringAfterColon (line, seqLine);
+	     getStringAfterColon ((unsigned char*)line, seqLine);
 	     if (seqLine[0] == EOS)
 		fprintf (stderr, "invalid line in file %s.conf\n   %s\n", pSS->SsName, line);
 	     else
@@ -1673,12 +1679,12 @@ void ConfigGetPSchemaForPageSize (PtrSSchema pSS, char *pageSize,
 	   else
 	     {
 		/* prend le 1er mot de la ligne lue */
-		getFirstWord (line, word);
+		getFirstWord ((unsigned char*)line, word);
 		if (strcmp (word, "style") == 0)
 		   /* c'est une ligne "style". On conserve le nom du schema de */
 		   /* presentation qui suit le mot-cle "style" */
 		  {
-		    getSecondWord (line, lastStyle);
+		    getSecondWord ((unsigned char *)line, lastStyle);
 		    lastPrefixLen=0;
 		    while(lastStyle[lastPrefixLen]!=EOS &&
 			  (pSS->SsDefaultPSchema)[lastPrefixLen]!=EOS &&
@@ -1699,7 +1705,7 @@ void ConfigGetPSchemaForPageSize (PtrSSchema pSS, char *pageSize,
 		else if (strcmp (word, "pagesize") == 0)
 		   /* c'est une ligne "pagesize", on la traite */
 		  {
-		     getStringAfterColon (line, seqLine);
+		     getStringAfterColon ((unsigned char*)line, seqLine);
 		     if (seqLine[0] == EOS)
                 fprintf (stderr, "invalid line in file %s.conf\n   %s\n", pSS->SsName, line);
              else if (strcmp (seqLine, pageSize) == 0)
@@ -1844,7 +1850,7 @@ ThotBool ConfigDefaultTypoSchema (PtrSSchema pSS, char *nameNature,
 	while (!found && getNextLineInSection (file, line))
 	  {
 	     /* le 1er mot de la ligne est le nom d'une nature */
-	     getFirstWord (line, name);
+	     getFirstWord ((unsigned char*)line, name);
 	     /* est-ce le nom de la nature cherchee ? */
 	     found = (strcmp (name, nameNature) == 0);
 	  }
@@ -1854,7 +1860,7 @@ ThotBool ConfigDefaultTypoSchema (PtrSSchema pSS, char *nameNature,
 	  {
 	     /* le nom de nature est suivi, apres ":", du nom du schema de */
 	     /* typographie a appliquer */
-	     getStringAfterColon (line, seqLine);
+	     getStringAfterColon ((unsigned char*)line, seqLine);
 	     if (seqLine[0] == EOS)
 		fprintf (stderr, "invalid line in file %s.conf\n   %s\n", pSS->SsName, line);
 	     else
