@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001.
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -22,17 +22,23 @@
  * Extensions: J. KAHAN (INRIA/W3C)
  *
  */
+#ifdef _WX
+  #include "wx/wx.h"
+#endif /* _WX */
 
+#include "thot_gui.h"
 #include "thotkey.h"
 #include "thot_sys.h"
 #include "constmedia.h"
 #include "typemedia.h"
 #include "registry.h"
 #include "application.h"
-#include "thotdir.h"
-#define THOT_EXPORT
+#undef THOT_EXPORT
+#define THOT_EXPORT extern
 #include "platform_tv.h"
+#include "edit_tv.h"
 
+#include "applicationapi_f.h"
 #include "fileaccess_f.h"
 #include "memory_f.h"
 #include "platform_f.h"
@@ -99,9 +105,18 @@ PathBuffer path;
 
 #ifdef _WINDOWS
 /* @@why do we need this here? */
-int errno;
+/*#undef errno
+int errno = 0;
+int * __cdecl _errno(void)
+{
+	return & errno;
+}*/
 #endif
 
+#ifdef _WX
+  #include "wx/utils.h"
+#endif /* _WX */
+
 /*----------------------------------------------------------------------
   ----------------------------------------------------------------------*/
 static char *SkipToEqual (char *ptr)
@@ -387,7 +402,7 @@ static int AddRegisterEntry (char *appli, char *name, char *value,
 /*----------------------------------------------------------------------
  PrintEnv : print the Registry to an open File.
   ----------------------------------------------------------------------*/
-static void         PrintEnv (FILE *output)
+static void PrintEnv (FILE *output)
 {
   RegistryEntry       cour, next;
   
@@ -408,6 +423,11 @@ static void         PrintEnv (FILE *output)
       fprintf (output, "[%s]\n", cour->appli);
       /* add all the entries under the same appli name */
       if (cour->level == REGISTRY_USER 
+#ifdef _WINDOWS
+		  && strcasecmp (cour->name, "TMP")
+		  && strcasecmp (cour->name, "TEMP")
+		  && strcasecmp (cour->name, "TEMPDIR")
+#endif /* _WINDOWS */
 		  && strcasecmp (cour->name, "TMPDIR")
 		  && strcasecmp (cour->name, "APP_TMPDIR")
 		  && strcasecmp (cour->name, "APP_HOME"))
@@ -417,7 +437,11 @@ static void         PrintEnv (FILE *output)
 	{
 	  if (next->level == REGISTRY_USER
 	      && strcasecmp (cour->name, "TMPDIR")
-#ifndef _WINDOWS
+#ifdef _WINDOWS
+		  && strcasecmp (cour->name, "TMP")
+		  && strcasecmp (cour->name, "TEMP")
+		  && strcasecmp (cour->name, "TEMPDIR")
+#else
 	      && strcasecmp (next->name, "APP_TMPDIR")
 #endif /* _WINDOWS */
 	      && strcasecmp (next->name, "APP_HOME"))
@@ -432,7 +456,7 @@ static void         PrintEnv (FILE *output)
 /*----------------------------------------------------------------------
  SortEnv : sort the Registry by application and name entries.
   ----------------------------------------------------------------------*/
-static void         SortEnv (void)
+static void SortEnv (void)
 {
   RegistryEntry       prev, cour, tmp;
   RegistryEntry      *start;
@@ -491,7 +515,6 @@ ThotBool TtaGetEnvInt (char *name, int *value)
      *value = 0;
      return FALSE;
    }
-
   strptr = TtaGetEnvString (name);
   /* the name entry doesn't exist */
   if (!strptr || *strptr == EOS)
@@ -500,8 +523,7 @@ ThotBool TtaGetEnvInt (char *name, int *value)
      return FALSE;
    }
  /* make the convertion */
- *value = uatoi (strptr);
-
+ *value = atoi (strptr);
  return TRUE;
 }
 
@@ -553,7 +575,7 @@ char *TtaGetEnvString (char *name)
 
       /* appname allows to get the application name */
    if (!strcasecmp ("appname", name))
-      return(AppRegistryEntryAppli);
+      return (AppRegistryEntryAppli);
 
    if (!strcasecmp (name, "cwd") || !strcasecmp (name, "pwd"))
       return (getcwd (&CurrentDir[0], sizeof(CurrentDir)));
@@ -561,59 +583,63 @@ char *TtaGetEnvString (char *name)
    /* first lookup in the System defaults */
    cour = AppRegistryEntry;
    while (cour != NULL)
- {
+     {
          if (!strcasecmp (cour->appli, "System") &&
 	     !strcmp (cour->name, name) && cour->value[0] != EOS)
- {
+	   {
 #ifdef DEBUG_REGISTRY
             fprintf (stderr, "TtaGetEnvString(\"%s\") = %s\n", name, cour->value);
 #endif
             return (cour->value);
-		 }
+	   }
          cour = cour->next;
-   }
+     }
 
    /* then lookup in the application user preferences */
    cour = AppRegistryEntry;
    while (cour != NULL)
- {
-         if (!strcasecmp (cour->appli, AppRegistryEntryAppli)      && 
-             !strcmp (cour->name, name) && cour->value[0] != EOS && 
-             cour->level == REGISTRY_USER)
- {
+     {
+       if (!strcasecmp (cour->appli, AppRegistryEntryAppli)      && 
+	   !strcmp (cour->name, name) && cour->value[0] != EOS && 
+	   cour->level == REGISTRY_USER)
+	 {
 #ifdef DEBUG_REGISTRY
-            fprintf (stderr, "TtaGetEnvString(\"%s\") = %s\n", name, cour->value);
+	   fprintf (stderr, "TtaGetEnvString(\"%s\") = %s\n", name, cour->value);
 #endif
-            return (cour->value);
-		 }
-         cour = cour->next;
-   } 
+	   return (cour->value);
+	 }
+       cour = cour->next;
+     }
 
    /* then lookup in the application defaults */
    cour = AppRegistryEntry;
-   while (cour != NULL) {
+   while (cour != NULL)
+     {
          if (!strcasecmp (cour->appli, AppRegistryEntryAppli)      && 
              !strcmp (cour->name, name) && cour->value[0] != EOS && 
-             cour->level == REGISTRY_SYSTEM) {
+             cour->level == REGISTRY_SYSTEM)
+	   {
 #ifdef DEBUG_REGISTRY
             fprintf (stderr, "TtaGetEnvString(\"%s\") = %s\n", name, cour->value);
 #endif
             return (cour->value);
-		 }
+	   }
          cour = cour->next;
-   } 
+     }
 
    /* then lookup in the Thot library defaults */
    cour = AppRegistryEntry;
-   while (cour != NULL) {
-         if (!strcasecmp (cour->appli, THOT_LIB_DEFAULTNAME) && !strcmp (cour->name, name) && cour->value[0] != EOS) {
+   while (cour != NULL)
+     {
+         if (!strcasecmp (cour->appli, THOT_LIB_DEFAULTNAME) && !strcmp (cour->name, name) && cour->value[0] != EOS)
+	   {
 #ifdef DEBUG_REGISTRY
             fprintf (stderr, "TtaGetEnvString(\"%s\") = %s\n", name, cour->value);
 #endif
             return (cour->value);
-		 }
+	   }
          cour = cour->next;
-   }
+     }
 
    /*
     * If still not found, look in the environment variables.
@@ -667,9 +693,9 @@ void TtaSetEnvBoolean (char *name, ThotBool value, int overwrite)
    char *ptr;
 
    if (value)
-      ptr = "yes";
+     ptr = "yes";
    else
-       ptr = "no";
+     ptr = "no";
    AddRegisterEntry (AppRegistryEntryAppli, name, ptr, REGISTRY_USER, overwrite);
 }
 
@@ -681,9 +707,11 @@ void TtaSetEnvString (char *name, char *value, int overwrite)
 {
    char *tmp = value;
   
+   /* make sure that value isn't NULL */
    if (!tmp)
-      tmp = "";
-   AddRegisterEntry (AppRegistryEntryAppli, name, value, REGISTRY_USER, overwrite);
+     tmp = "";
+   
+   AddRegisterEntry (AppRegistryEntryAppli, name, tmp, REGISTRY_USER, overwrite);
 }
 
 /*----------------------------------------------------------------------
@@ -692,12 +720,13 @@ void TtaSetEnvString (char *name, char *value, int overwrite)
   ----------------------------------------------------------------------*/
 void TtaSetDefEnvString (char *name, char *value, int overwrite)
 {
-  char *tmp = value; /* ??? */
-                         /* ??? */
-  if (!tmp)              /* ??? */
-    tmp = "";   /* ??? */
+  char *tmp = value;
+             
+  /* make sure that value isn't NULL */
+  if (!tmp)
+    tmp = "";
 
-  AddRegisterEntry (AppRegistryEntryAppli, name, value, REGISTRY_SYSTEM, overwrite);
+  AddRegisterEntry (AppRegistryEntryAppli, name, tmp, REGISTRY_SYSTEM, overwrite);
 }
 
 /*----------------------------------------------------------------------
@@ -714,19 +743,15 @@ ThotBool TtaGetDefEnvInt (char *name, int *value)
      *value = 0;
      return FALSE;
    }
-
  strptr = TtaGetDefEnvString (name);
-
  /* the name entry doesn't exist */
  if (!strptr || *strptr == EOS)
    {
      *value = 0;
      return FALSE;
    }
-
  /* make the convertion */
- *value = uatoi (strptr);
-
+ *value = atoi (strptr);
  return TRUE;
 }
 
@@ -877,7 +902,7 @@ static int          IsThotDir (CONST char *path)
     }
 }
 
-#ifdef _WINDOWS
+#ifdef _WINGUI
 #ifndef __GNUC__
 /*----------------------------------------------------------------------
   WINReg_get - simulates getenv in the WIN32 registry
@@ -946,7 +971,7 @@ static char *WINIni_get (CONST char *env)
   return res ? EnVar : NULL;
 }
 #endif
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
 
 
 /*----------------------------------------------------------------------
@@ -1058,7 +1083,7 @@ static void ImportRegistryFile (char *filename, RegistryLevel level)
   InitEnviron : initialize the standard environment (i.e global	
   variables) with values stored in the registry.			
   ----------------------------------------------------------------------*/
-static void         InitEnviron ()
+static void InitEnviron ()
 {
    char *pT;
    char *Thot_Sys_Sch;
@@ -1066,19 +1091,21 @@ static void         InitEnviron ()
 
    /* default values for various global variables */
    FirstCreation = FALSE;
-   CurSaveInterval = 0;
-   pT = TtaGetEnvString ("AUTOSAVE");
+   DocBackUpInterval = 0;
+   pT = TtaGetEnvString ("AUTO_SAVE");
    if (pT != NULL)
-     CurSaveInterval = uatoi (pT);
-   if (CurSaveInterval <= 0)
-     CurSaveInterval = DEF_SAVE_INTVL;
+     {
+       DocBackUpInterval = atoi (pT);
+       if (DocBackUpInterval <= 0)
+	 DocBackUpInterval = 0;
+     }
    HighlightBoxErrors = FALSE;
    InsertionLevels = 4;
 #ifndef _WINDOWS
    TtaSetDefEnvString ("DOUBLECLICKDELAY", "500", TRUE);
    pT = TtaGetEnvString ("DOUBLECLICKDELAY");
    if (pT != NULL)
-     DoubleClickDelay = uatoi (pT);
+     DoubleClickDelay = atoi (pT);
    else 
        DoubleClickDelay = 500;
 #endif /* _WINDOWS */
@@ -1119,9 +1146,9 @@ static void         InitEnviron ()
    TtaSetDefEnvString ("TOOLTIPDELAY", "500", FALSE);
    TtaSetDefEnvString ("FontMenuSize", "12", FALSE);
    TtaSetDefEnvString ("ForegroundColor", "Black", FALSE);
-   TtaSetDefEnvString ("BackgroundColor", "#c0c0c0", FALSE);
-   TtaSetDefEnvString ("InserPointColor", "Red", FALSE);
-   TtaSetDefEnvString ("DocSelectColor", "LightGold3", FALSE);
+   TtaSetDefEnvString ("BackgroundColor", "White", FALSE);
+   TtaSetDefEnvString ("FgSelectColor", "White", FALSE);
+   TtaSetDefEnvString ("BgSelectColor", "#008BB2", FALSE);
    TtaSetDefEnvString ("MenuFgColor", "Black", FALSE);
    TtaSetDefEnvString ("MenuBgColor", "Grey", FALSE);
 
@@ -1129,13 +1156,23 @@ static void         InitEnviron ()
    ** set up the defaul TMPDIR 
    */
 
-   /* set up a default TMPDIR */
-   TtaSetDefEnvString ("TMPDIR", DEF_TMPDIR, TRUE);
    /* get the tmpdir from the registry or use the default name if it
       doesn't exist */
    pT = TtaGetEnvString ("TMPDIR");
+#ifdef _WINDOWS
+   /* on WIN32, the default directory may be defined on several environment variables */
+   if (!pT)
+	   pT = TtaGetEnvString ("TMP");
+   if (!pT)
+	   pT = TtaGetEnvString ("TEMP");
+   if (!pT)
+	   pT = TtaGetEnvString ("TEMPDIR");
+#endif /* _WINDOWS */
    if (!pT)
      pT = DEF_TMPDIR;
+   /* set up a default TMPDIR in the registry in case it wasn't defined before */
+   TtaSetDefEnvString ("TMPDIR", pT, FALSE);
+
    /* create the TMPDIR dir if it doesn't exist */
    if (!TtaMakeDirectory (pT))
      {
@@ -1144,10 +1181,7 @@ static void         InitEnviron ()
        if (!TtaMakeDirectory (pT))
 	 {
 	   fprintf (stderr, "Couldn't create directory %s\n", pT);
-#ifdef _GTK
-	   gtk_exit (1);
-#endif /* _GTK */	
-	   exit (1);
+	   ThotExit (1);
 	 }	 
        /* update the registry entry */
        TtaSetEnvString ("TMPDIR", pT, TRUE);
@@ -1165,24 +1199,25 @@ static void         InitEnviron ()
   ----------------------------------------------------------------------*/
 void TtaInitializeAppRegistry (char *appArgv0)
 {
-  char      app_home[MAX_PATH];
-  char      filename[MAX_PATH];
-  char     *my_path;
-  char     *dir_end = NULL;
-  char     *appName;
-  char     *ptr;
-#ifdef _WINDOWS
+  char        app_home[MAX_PATH];
+  char        filename[MAX_PATH];
+  char       *my_path;
+  char       *dir_end = NULL;
+  char       *appName;
+  char       *ptr;
+#ifdef _WINGUI
   /* name in Windows NT 4 is 20 chars */
-  char      username[MAX_LENGTH];
-  char      windir[MAX_PATH+1];
+  char        username[MAX_LENGTH];
+  char        windir[MAX_PATH+1];
   DWORD       dwSize;
   ThotBool    status;
-#else /*  _WINDOWS */
+  char       *ptr2, *ptr3;
+#else /*  _WINGUI */
   struct stat stat_buf;
   char        c_execname[MAX_LENGTH];
   char        c_filename[MAX_LENGTH];
   char       *c_end;
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
   int         execname_len;
   int         len, round;
   ThotBool    found, ok;
@@ -1198,15 +1233,12 @@ void TtaInitializeAppRegistry (char *appArgv0)
    */
   if ((appArgv0 == NULL) || (*appArgv0 == EOS))
     {
-#ifdef _WINDOWS
+#ifdef _WINGUI
       MessageBox (NULL, "TtaInitializeAppRegistry called with invalid argv[0] value", "Amaya", MB_OK);
-#else  /* _WINDOWS */
+#else  /* _WINGUI */
       fprintf (stderr, "TtaInitializeAppRegistry called with invalid argv[0] value\n");
-#endif /* _WINDOWS */
-#ifdef _GTK
-      gtk_exit (1);
-#endif /* _GTK */	
-      exit (1);
+#endif /* _WINGUI */
+      ThotExit (1);
     }
 
   /*
@@ -1245,10 +1277,7 @@ void TtaInitializeAppRegistry (char *appArgv0)
       if (my_path == NULL)
 	{
 	  fprintf (stderr, "TtaInitializeAppRegistry cannot found PATH environment\n");
-#ifdef _GTK
-	  gtk_exit (1);
-#endif /* _GTK */	
-	  exit (1);
+	  ThotExit (1);
 	}
       /*
        * make our own copy of the string, in order to preserve the
@@ -1269,10 +1298,7 @@ void TtaInitializeAppRegistry (char *appArgv0)
 	{
 	  fprintf (stderr, "TtaInitializeAppRegistry internal error\n");
 	  fprintf (stderr, "\tcannot find path to binary : %s\n", appArgv0);
-#ifdef _GTK
-	  gtk_exit (1);
-#endif /* _GTK */	
-	  exit (1);
+	  ThotExit (1);
 	}
     }
    
@@ -1299,7 +1325,7 @@ void TtaInitializeAppRegistry (char *appArgv0)
   ptr = appName;
   while (*ptr)
     {
-      *ptr = utolower (*ptr);
+      *ptr = tolower (*ptr);
       ptr++;
     }
 #endif /* _WINDOWS */
@@ -1318,7 +1344,8 @@ void TtaInitializeAppRegistry (char *appArgv0)
 	 S_ISLNK (stat_buf.st_mode) &&
 	 len > 0)
     {
-      len = readlink (c_execname, c_filename, sizeof (filename) / sizeof (char));
+      len = readlink (c_execname, c_filename,
+		      sizeof (filename) / sizeof (char));
       if (len > 0)
 	{
 	  /*
@@ -1366,7 +1393,8 @@ void TtaInitializeAppRegistry (char *appArgv0)
        found = TRUE;
        *dir_end = EOS;
        /* save the binary directory in BinariesDirectory */
-       strncpy (BinariesDirectory, execname, sizeof (BinariesDirectory) / sizeof (char));
+       strncpy (BinariesDirectory, execname,
+		sizeof (BinariesDirectory) / sizeof (char));
        /* remove the binary directory */
        found = FALSE;
        ok = FALSE;
@@ -1402,7 +1430,8 @@ void TtaInitializeAppRegistry (char *appArgv0)
 	 {
 	   *dir_end = EOS;
 	   if (IsThotDir (execname))
-	     AddRegisterEntry ("System", "THOTDIR", execname, REGISTRY_INSTALL, TRUE);
+	     AddRegisterEntry ("System", "THOTDIR", execname,
+			       REGISTRY_INSTALL, TRUE);
 	 }
 #ifdef COMPILED_IN_THOTDIR
        /* Check a compiled-in value */
@@ -1410,7 +1439,8 @@ void TtaInitializeAppRegistry (char *appArgv0)
 	 {
            strcpy (UCOMPILED_IN_THOTDIR, COMPILED_IN_THOTDIR);
            strcpy (execname, UCOMPILED_IN_THOTDIR);
-           AddRegisterEntry ("System", "THOTDIR", UCOMPILED_IN_THOTDIR, REGISTRY_INSTALL, TRUE);
+           AddRegisterEntry ("System", "THOTDIR", UCOMPILED_IN_THOTDIR,
+			     REGISTRY_INSTALL, TRUE);
 	 } 
 #else /* COMPILED_IN_THOTDIR */
 #ifdef COMPILED_IN_THOTDIR2
@@ -1419,17 +1449,15 @@ void TtaInitializeAppRegistry (char *appArgv0)
 	 {
            strcpy (UCOMPILED_IN_THOTDIR2, COMPILED_IN_THOTDIR2);
            strcpy (execname, COMPILED_IN_THOTDIR2);
-           AddRegisterEntry ("System", "THOTDIR", UCOMPILED_IN_THOTDIR2, REGISTRY_INSTALL, TRUE);
+           AddRegisterEntry ("System", "THOTDIR", UCOMPILED_IN_THOTDIR2,
+			     REGISTRY_INSTALL, TRUE);
 	 }
 #endif /* COMPILED_IN_THOTDIR2 */
 #endif /* COMPILED_IN_THOTDIR */
       else
 	{
 	  fprintf (stderr, "Cannot find THOTDIR\n");
-#ifdef _GTK
-	  gtk_exit (1);
-#endif /* _GTK */	
-	  exit (1);
+	  ThotExit (1);
 	} 
      }
    
@@ -1440,7 +1468,8 @@ void TtaInitializeAppRegistry (char *appArgv0)
 #endif
 
    /* load the system settings, stored in THOTDIR/config/thot.ini */
-   sprintf (filename, "%s%c%s%c%s", execname, DIR_SEP, THOT_CONFIG_FILENAME, DIR_SEP, THOT_INI_FILENAME);
+   sprintf (filename, "%s%c%s%c%s", execname, DIR_SEP, THOT_CONFIG_FILENAME,
+	    DIR_SEP, THOT_INI_FILENAME);
    if (TtaFileExist (filename))
      {
 #ifdef DEBUG_REGISTRY
@@ -1451,60 +1480,139 @@ void TtaInitializeAppRegistry (char *appArgv0)
        dir_end -= 3;
      }
    else
-     fprintf (stderr, "System wide %s not found at %s\n", THOT_INI_FILENAME, &filename[0]);
+     fprintf (stderr, "System wide %s not found at %s\n", THOT_INI_FILENAME,
+	      &filename[0]);
    
    /*
    ** find the APP_HOME directory name:
    ** Unix: $HOME/.appname,
    ** Win95: $THOTDIR/users/login-name/
    ** WinNT: c:\WINNT\profiles\login-name
+   ** Win2000/XP: $HOMEDRIVE\$HOMEPATH
+   **              or
+   **             Documents and settings\All Users\login-name
+   **              or
+   **             the same thing as WinNT.
    */
-   /* No this should NOT be a call to TtaGetEnvString */
-#ifdef _WINDOWS
-   /* compute the default app_home value from the username and thotdir */
-   dwSize = sizeof (username);
-   status = GetUserName (username, &dwSize);
-   if (status)
-     ptr = username;
-   else
-     /* under win95, there may be no user name */
-     ptr = WIN_DEF_USERNAME;
-   if (IS_NT)
-     /* winnt: apphome is windowsdir\profiles\username\appname */
-     {
-       dwSize = MAX_PATH;
-       GetWindowsDirectory (windir, dwSize);
-       sprintf (app_home, "%s\\profiles\\%s\\%s", windir, ptr, AppNameW);
-     }
-   else
-     /* win95: apphome is  thotdir\users\username */
-     sprintf (app_home, "%s\\%s\\%s", execname, WIN_USERS_HOME_DIR, ptr);   
-#else /* _WINDOWS */
-   ptr = getenv ("HOME");
-   sprintf (app_home, "%s%c.%s", ptr, DIR_SEP, AppNameW); 
-#endif _WINDOWS
-   /* store the value of APP_HOME in the registry */
-   AddRegisterEntry (AppRegistryEntryAppli, "APP_HOME", app_home, REGISTRY_SYSTEM, FALSE);
 
-   /* get the app_home again from the registry, as the user may have overriden it
-      using the global configuration files */
+   app_home[0] = EOS;
+   /* IV 18/08/2003 Check the variable AMAYA_USER_HOME first */
+   ptr = getenv ("AMAYA_USER_HOME");
+   if (ptr && TtaDirExists (ptr))
+       strncpy (app_home, ptr, MAX_PATH);
+
+   if (app_home[0] == EOS)
+     {
+#ifdef _WINGUI
+       /* compute the default app_home value from the username and thotdir */
+       dwSize = sizeof (username);
+       status = GetUserName (username, &dwSize);
+       if (status)
+	 {
+	   if (strchr (username, '*'))
+	     /* don't use a username that include stars */
+	     ptr = WIN_DEF_USERNAME;
+	   else
+	     ptr = username;
+	 }
+       else
+	 /* under win95, there may be no user name */
+	 ptr = WIN_DEF_USERNAME;
+
+       if (IS_NT)
+	 /* winnt: apphome is windowsdir\profiles\username\appname */
+	 {
+	   dwSize = MAX_PATH;
+#if 0
+	   /* JK: commented as it needs installing a SP */
+	   /* this function will fail if dwSize is inferior to the buffer size.
+	      As we gave it the maximum possible value, we don't control
+	      the return code */
+	   GetUserProfileDirectory (AccessTokenHandle, app_home, &dwSize);
+#endif
+
+	   /* check if a previous app_home directory existed. If yes, use it. If
+	      it didn't exist, then we try to create it using the new conventions. */
+	   GetWindowsDirectory (windir, dwSize);
+	   /* the Windows NT convention */
+	   sprintf (app_home, "%s\\profiles\\%s\\%s", windir, ptr, AppNameW);
+	   if (!TtaDirExists (app_home))
+	     app_home[0] = EOS;
+	   
+	   if (app_home[0] == EOS)
+	     {
+	       /* use the HOMEDRIVE and HOMEPATH environment variables first */
+	       ptr2 = getenv ("HOMEDRIVE");
+	       ptr3 = getenv ("HOMEPATH");
+	       if (ptr2 && *ptr2 && ptr3)
+		 {
+		     sprintf (app_home, "%s%s\\%s", ptr2, ptr3, AppNameW);
+	     if (!TtaDirExists (app_home))
+		     app_home[0] = EOS;
+		 }
+	     }
+	   if (app_home[0] == EOS)
+	     {
+	       /* another possible Windows 2000/XP convention */
+           sprintf (app_home, "%s\\Documents and Settings\\%s\\%s", windir, ptr, AppNameW);
+	       if (!TtaDirExists (app_home))
+		 app_home[0] = EOS;
+	     }
+
+	   /* At this point app_home has a value if the directory existed. Otherwise,
+	      we'll try to create a new one */
+	   if (app_home[0] == EOS)
+	     {
+	       /* try to use one of the system home dirs */
+	       /* the Windows 2000/XP convention */
+	       if (ptr2 && *ptr2 && ptr3)
+	         sprintf (app_home, "%s%s", ptr2, ptr3);
+		   else
+             sprintf (app_home, "%s\\Documents and Settings\\%s", windir, ptr);
+
+	       if (! TtaDirExists (app_home))
+		 {
+		   /* the Windows NT convention */
+		   sprintf (app_home, "%s\\profiles\\%s", windir, ptr);
+		 }
+	       /* add the end suffix */
+	       sprintf (windir, "\\%s", AppNameW);
+	       strcat (app_home, windir);
+	     }
+	 }
+       else
+	 /* win95: apphome is  thotdir\users\username */
+	 sprintf (app_home, "%s\\%s\\%s", execname, WIN_USERS_HOME_DIR, ptr);
+#else /* _WINGUI */
+#ifdef _UNIX
+       ptr = getenv ("HOME");
+       sprintf (app_home, "%s%c.%s", ptr, DIR_SEP, AppNameW); 
+#else /* _UNIX */
+#if defined(_WX) /* SG TODO : a valider */
+	   wxString homedir = wxGetHomeDir();
+	   wxCSConv conv_ascii(_T("ISO-8859-1"));
+#ifdef _WINDOWS
+	   sprintf (app_home, "%s%c%s", homedir.mb_str(conv_ascii) , DIR_SEP, AppNameW);
+#else  /* _WINDOWS */
+	   sprintf (app_home, "%s%c.%s", homedir.mb_str(conv_ascii) , DIR_SEP, AppNameW);
+#endif /* _WINDOWS */
+#endif /* _WX */
+#endif /* _UNIX */
+#endif /*_WINGUI */
+     }
+   /* store the value of APP_HOME in the registry */
+   AddRegisterEntry (AppRegistryEntryAppli, "APP_HOME", app_home,
+		     REGISTRY_SYSTEM, FALSE);
+
+   /* get the app_home again from the registry, as the user may have
+      overriden it using the global configuration files */
    ptr = TtaGetEnvString ("APP_HOME");
    if (ptr)
      strcpy (app_home, ptr);
    else
      app_home[0] = EOS;
 
-   /* set the default APP_TMPDIR */
-#ifdef _WINDOWS
-   /* the tmpdir is DEF_TMPDIR\app-name */
-   sprintf (filename, "%s%c%s", DEF_TMPDIR, DIR_SEP, AppNameW);
-   AddRegisterEntry (AppRegistryEntryAppli, "APP_TMPDIR", filename, REGISTRY_SYSTEM, TRUE);
-#else
-   /* under Unix, APP_TMPDIR == APP_HOME */
-   AddRegisterEntry (AppRegistryEntryAppli, "APP_TMPDIR", app_home,
-		     REGISTRY_SYSTEM, TRUE);
-#endif /* _WINDOWS */
-   /* read the user's preferences (if they exist) */
+    /* read the user's preferences (if they exist) */
    if (app_home != NULL && *app_home != EOS)
      {
        sprintf (filename, "%s%c%s", app_home, DIR_SEP, THOT_RC_FILENAME);
@@ -1529,7 +1637,22 @@ void TtaInitializeAppRegistry (char *appArgv0)
 #ifdef DEBUG_REGISTRY
    PrintEnv (stderr);
 #endif
+   /* initialize the standard environment (i.e global variables) with 
+	  values stored in the registry. */
    InitEnviron ();
+  /* set the default APP_TMPDIR */
+#ifdef _WINDOWS
+   /* the tmpdir is DEF_TMPDIR\app-name */
+   ptr = TtaGetEnvString ("TMPDIR");
+   sprintf (filename, "%s%c%s", ptr, DIR_SEP, AppNameW);
+   AddRegisterEntry (AppRegistryEntryAppli, "APP_TMPDIR", filename,
+		     REGISTRY_SYSTEM, TRUE);
+#else
+   /* under Unix, APP_TMPDIR == APP_HOME */
+   AddRegisterEntry (AppRegistryEntryAppli, "APP_TMPDIR", app_home,
+		     REGISTRY_SYSTEM, TRUE);
+#endif /* _WINDOWS */
+   /* reset the status flag to say we don't need to save the registry right now */
    AppRegistryModified = 0;
 }
 
@@ -1570,9 +1693,9 @@ void                TtaFreeAppRegistry (void)
    0 otherwise.                                   
    Depending on dir value, the file is looked for in:
    - 0 : /                                                 
-   - 1 : ThotDir                                           
-   - 2 : ThotDir/bin                                       
-   - 3 : ThotDir/compil                                    
+   - 1 : ThotDir/document paths
+   - 2 : ThotDir/config     
+   - 3 : ThotDir/batch
   ----------------------------------------------------------------------*/
 int SearchFile (char *fileName, int dir, char *fullName)
 {
@@ -1623,7 +1746,6 @@ int SearchFile (char *fileName, int dir, char *fullName)
 		    ret = TtaFileExist (fullName);
 		 }
 	       break;
-
 	    case 2:
 	       /* lookup in config */
 	       strcat (fullName, DIR_STR);
@@ -1631,7 +1753,6 @@ int SearchFile (char *fileName, int dir, char *fullName)
 	       strcat (fullName, DIR_STR);
 	       strcat (fullName, fileName);
 	       break;
-
 	    case 3:
 	       /* lookup in batch */
 	       strcat (fullName, DIR_STR);
@@ -1639,7 +1760,6 @@ int SearchFile (char *fileName, int dir, char *fullName)
 	       strcat (fullName, DIR_STR);
 	       strcat (fullName, fileName);
 	       break;
-
 	    default:
 	       strcat (fullName, DIR_STR);
 	       strcat (fullName, fileName);
