@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -12,11 +12,12 @@
  *
  */
 
+#include "thot_gui.h"
 #include "thot_sys.h"
 #include "constmedia.h"
 #include "typemedia.h"
 #include "frame.h"
-#define THOT_EXPORT
+#define THOT_EXPORT extern
 #include "units_tv.h"
 #undef THOT_EXPORT
 #define THOT_EXPORT extern
