@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -12,7 +12,7 @@
  *          I. Vatton (INRIA) - Polylines
  *
  */
-
+#include "thot_gui.h"
 #include "thot_sys.h"
 #include "constmedia.h"
 #include "typemedia.h"
@@ -123,6 +123,7 @@ void SplitTextElement (PtrElement pEl, int rank, PtrDocument pDoc,
 	     notifyEl.event = TteElemNew;
 	     notifyEl.document = (Document) IdentDocument (pDoc);
 	     notifyEl.element = (Element) (pEl->ElParent);
+	     notifyEl.info = 0; /* not sent by undo */
 	     notifyEl.elementType.ElTypeNum = pEl->ElTypeNumber;
 	     notifyEl.elementType.ElSSchema = (SSchema) (pEl->ElStructSchema);
 	     pSibling = pEl;
@@ -153,105 +154,112 @@ void SplitTextElement (PtrElement pEl, int rank, PtrDocument pDoc,
 		    }
 	       }
 	  }
-	/* copie l'element de texte avec attributs et presentation specifique */
-	pEl2 = ReplicateElement (pEl, pDoc);
-	pEl2->ElVolume = 0;
-	pEl2->ElTextLength = 0;
-	*pSecondPart = pEl2;
-	/* acquiert un buffer de texte pour la 2eme partie */
-	CreateTextBuffer (pEl2);
-	/* chaine le nouvel element */
-	InsertElementAfter (pEl, pEl2);
-	if (rank <= pEl->ElTextLength)
-	   /* le nouvel element n'a pas une longueur nulle */
+	if (!dontCut)
 	  {
-	     /* longueur du nouvel element */
-	     pEl2->ElTextLength = pEl->ElTextLength - rank + 1;
-	     pEl2->ElVolume = pEl2->ElTextLength;
-	     /* nouvelle longueur de l'ancien */
-	     pEl->ElTextLength = rank - 1;
-	     pEl->ElVolume = pEl->ElTextLength;
-	     /* cherche le buffer (pointe' par pBuf) ou a lieu la coupure */
-	     pBuf = pEl->ElText;
-	     i = 0;
-	     while (i + pBuf->BuLength < rank)
-	       {
-		  i += pBuf->BuLength;
-		  pBuf = pBuf->BuNext;
-	       }
-	     /* longueur +1 du texte qui restera dans le dernier buffer (pBuf)
-	        de la premiere partie */
-	     len = rank - i;
-	     pBuf1 = pEl2->ElText;
-	     pBuf1->BuNext = pBuf->BuNext;
-	     if (pBuf1->BuNext != NULL)
-		pBuf1->BuNext->BuPrevious = pEl2->ElText;
-	     /* copie la 2eme partie du buffer coupe' dans le nouveau buffer */
-	     i = 0;
-	     do
-	       {
-		  pBuf1->BuContent[i] = pBuf->BuContent[len + i - 1];
-		  i++;
-	       }
-	     while (pBuf1->BuContent[i - 1] != EOS);
-	     pBuf1->BuLength = i - 1;
-	     pBuf1 = pBuf;
-	     /* met a jour le dernier buffer de la premiere partie */
-	     pBuf1->BuNext = NULL;
-	     pBuf1->BuContent[len - 1] = EOS;
-	     pBuf1->BuLength = len - 1;
-	     /* essaie de vider ce dernier buffer dans le precedent */
-	     if (pBuf1->BuPrevious != NULL)
-		/* il y a un buffer precedent */
-		if (pBuf1->BuPrevious->BuLength + pBuf1->BuLength < THOT_MAX_CHAR)
-		   /* ca peut tenir */
+	    /* copie l'element de texte avec attributs et presentation
+	       specifique */
+	    pEl2 = ReplicateElement (pEl, pDoc);
+	    pEl2->ElVolume = 0;
+	    pEl2->ElTextLength = 0;
+	    *pSecondPart = pEl2;
+	    /* acquiert un buffer de texte pour la 2eme partie */
+	    CreateTextBuffer (pEl2);
+	    /* chaine le nouvel element */
+	    InsertElementAfter (pEl, pEl2);
+	    if (rank <= pEl->ElTextLength)
+	      /* le nouvel element n'a pas une longueur nulle */
+	      {
+		/* longueur du nouvel element */
+		pEl2->ElTextLength = pEl->ElTextLength - rank + 1;
+		pEl2->ElVolume = pEl2->ElTextLength;
+		/* nouvelle longueur de l'ancien */
+		pEl->ElTextLength = rank - 1;
+		pEl->ElVolume = pEl->ElTextLength;
+		/* cherche le buffer (pointe' par pBuf) ou a lieu la coupure */
+		pBuf = pEl->ElText;
+		i = 0;
+		while (i + pBuf->BuLength < rank)
 		  {
-		     i = 0;
-		     while (i <= pBuf1->BuLength)
-		       {
-			  pBuf1->BuPrevious->BuContent[pBuf1->BuPrevious->BuLength + i] =
-			     pBuf1->BuContent[i];
-			  i++;
-		       }
-		     pBuf1->BuPrevious->BuLength += pBuf1->BuLength;
-		     /* supprime le dernier buffer de la premiere partie : il est */
-		     /* vide */
-		     pBuf1->BuPrevious->BuNext = NULL;
-		     FreeTextBuffer (pBuf);
+		    i += pBuf->BuLength;
+		    if (pBuf->BuNext)
+		      pBuf = pBuf->BuNext;
+		    else
+		      rank = i;
 		  }
-	     /* accroche les buffers de la deuxieme partie a leur element */
-	     pBuf = pEl2->ElText;
-	     while (pBuf != NULL)
-		pBuf = pBuf->BuNext;
-	     if (withAppEvent)
-	       {
-		  /* envoie l'evenement ElemTextModify.Post a qui le demande */
-		  pAsc = pEl;
-		  while (pAsc != NULL)
-		    {
-		       notifyTxt.event = TteElemTextModify;
-		       notifyTxt.document = (Document) IdentDocument (pDoc);
-		       notifyTxt.element = (Element) pAsc;
-		       notifyTxt.target = (Element) pEl;
-		       if (elBreak)
-		         notifyTxt.targetdocument = 0;
-		       else
-		         notifyTxt.targetdocument = (Document) IdentDocument (pDoc);
-		       CallEventType ((NotifyEvent *) & notifyTxt, FALSE);
-		       pAsc = pAsc->ElParent;
-		    }
-	       }
-	  }
-	if (withAppEvent)
-	  {
-	     /* envoie l'evenement ElemNew.Post au nouvel element de texte */
-	     notifyEl.event = TteElemNew;
-	     notifyEl.document = (Document) IdentDocument (pDoc);
-	     notifyEl.element = (Element) pEl2;
-	     notifyEl.elementType.ElTypeNum = pEl2->ElTypeNumber;
-	     notifyEl.elementType.ElSSchema = (SSchema) (pEl2->ElStructSchema);
-	     notifyEl.position = 0;
-	     CallEventType ((NotifyEvent *) & notifyEl, FALSE);
+		/* longueur+1 du texte qui restera dans le dernier buffer
+		   (pBuf) de la premiere partie */
+		len = rank - i - 1;
+		pBuf1 = pEl2->ElText;
+		pBuf1->BuNext = pBuf->BuNext;
+		if (pBuf1->BuNext != NULL)
+		  pBuf1->BuNext->BuPrevious = pEl2->ElText;
+		/* copie la 2eme partie du buffer coupe' dans le nouveau
+		   buffer */
+		i = 0;
+		do
+		  {
+		    pBuf1->BuContent[i] = pBuf->BuContent[len + i];
+		    i++;
+		  }
+		while (pBuf1->BuContent[i - 1] != EOS);
+		pBuf1->BuLength = i - 1;
+		/* met a jour le dernier buffer de la premiere partie */
+		pBuf->BuNext = NULL;
+		pBuf->BuContent[len] = EOS;
+		pBuf->BuLength = len;
+		/* essaie de vider ce dernier buffer dans le precedent */
+		pBuf1 = pBuf->BuPrevious;
+		if (pBuf1 &&
+		    pBuf1->BuLength + pBuf->BuLength <= FULL_BUFFER)
+		  /* il y a un buffer precedent et ca peut tenir */
+		  {
+		    i = 0;
+		    while (i <= pBuf->BuLength)
+		      {
+			pBuf1->BuContent[pBuf1->BuLength + i] = pBuf->BuContent[i];
+			i++;
+		      }
+		    pBuf1->BuLength += pBuf->BuLength;
+		    /* supprime le dernier buffer de la premiere partie :
+		       il est vide */
+		    pBuf1->BuNext = NULL;
+		    FreeTextBuffer (pBuf);
+		  }
+		/* accroche les buffers de la deuxieme partie a leur element */
+		pBuf = pEl2->ElText;
+		while (pBuf)
+		  pBuf = pBuf->BuNext;
+		if (withAppEvent)
+		  {
+		    /* envoie l'evenement ElemTextModify.Post a qui le demande */
+		    pAsc = pEl;
+		    while (pAsc != NULL)
+		      {
+			notifyTxt.event = TteElemTextModify;
+			notifyTxt.document = (Document) IdentDocument (pDoc);
+			notifyTxt.element = (Element) pAsc;
+			notifyTxt.target = (Element) pEl;
+			if (elBreak)
+			  notifyTxt.targetdocument = 0;
+			else
+			  notifyTxt.targetdocument = (Document)IdentDocument (pDoc);
+			CallEventType ((NotifyEvent *) & notifyTxt, FALSE);
+			pAsc = pAsc->ElParent;
+		      }
+		  }
+	      }
+	    if (withAppEvent)
+	      {
+		/* envoie l'evenement ElemNew.Post au nouvel element de texte*/
+		notifyEl.event = TteElemNew;
+		notifyEl.document = (Document) IdentDocument (pDoc);
+		notifyEl.element = (Element) pEl2;
+		notifyEl.info = 0; /* not sent by undo */
+		notifyEl.elementType.ElTypeNum = pEl2->ElTypeNumber;
+		notifyEl.elementType.ElSSchema = (SSchema)(pEl2->ElStructSchema);
+		notifyEl.position = 0;
+		CallEventType ((NotifyEvent *) & notifyEl, FALSE);
+	      }
 	  }
      }
 }
@@ -260,7 +268,7 @@ void SplitTextElement (PtrElement pEl, int rank, PtrDocument pDoc,
 /*--------------------------------------------------------------------
   MergeTextElements   si l'element pEl est une feuille de texte et si
   son suivant est egalement une feuille de texte dans le meme
-  alphabet, reunit les deux elements sucessifs en un seul.
+  script, reunit les deux elements sucessifs en un seul.
   Le deuxieme element, devenu vide, n'est pas libere' et un
   pointeur sur cet element est retourne dans pFreeEl.
   Le parametre booleen removeAbsBox indique s'il faut liberer les
@@ -296,8 +304,8 @@ ThotBool MergeTextElements (PtrElement pEl, PtrElement *pFreeEl, PtrDocument pDo
 	  pEl2->ElLeafType == LtText && pEl2->ElTerminal &&
 	  pEl2->ElLanguage == pEl->ElLanguage &&
 	  pEl1->ElSource == NULL && pEl2->ElSource == NULL &&
-	  pEl1->ElStructSchema->SsRule[pEl1->ElTypeNumber - 1].SrConstruct != CsConstant &&
-	  pEl2->ElStructSchema->SsRule[pEl2->ElTypeNumber - 1].SrConstruct != CsConstant)
+	  pEl1->ElStructSchema->SsRule->SrElem[pEl1->ElTypeNumber - 1]->SrConstruct != CsConstant &&
+	  pEl2->ElStructSchema->SsRule->SrElem[pEl2->ElTypeNumber - 1]->SrConstruct != CsConstant)
 	{
 	  merge = TRUE;
 	  if (withAppEvent)
@@ -307,6 +315,7 @@ ThotBool MergeTextElements (PtrElement pEl, PtrElement *pFreeEl, PtrDocument pDo
 	      notifyEl.event = TteElemDelete;
 	      notifyEl.document = (Document) IdentDocument (pDoc);
 	      notifyEl.element = (Element) pEl2;
+	      notifyEl.info = 0; /* not sent by undo */
 	      notifyEl.elementType.ElTypeNum = pEl2->ElTypeNumber;
 	      notifyEl.elementType.ElSSchema = (SSchema) (pEl2->ElStructSchema);
 	      notifyEl.position = TTE_MERGE_DELETE_ITEM;
@@ -350,7 +359,7 @@ ThotBool MergeTextElements (PtrElement pEl, PtrElement *pFreeEl, PtrDocument pDo
 		pBuf1 = pBuf1->BuNext;
 	      /* premier buffer du deuxieme element */
 	      pBuf2 = pEl2->ElText;
-	      if (pBuf1->BuLength + pBuf2->BuLength < THOT_MAX_CHAR)
+	      if (pBuf1->BuLength + pBuf2->BuLength <= FULL_BUFFER)
 		/* copie le contenu du premier buffer du 2eme element */
 		/* a la fin du dernier buffer du 1er element */
 		{
@@ -433,6 +442,7 @@ ThotBool MergeTextElements (PtrElement pEl, PtrElement *pFreeEl, PtrDocument pDo
 		  notifyEl.event = TteElemDelete;
 		  notifyEl.document = (Document) IdentDocument (pDoc);
 		  notifyEl.element = (Element) (pEl2->ElParent);
+		  notifyEl.info = 0; /* not sent by undo */
 		  notifyEl.elementType.ElTypeNum = pEl2->ElTypeNumber;
 		  notifyEl.elementType.ElSSchema = (SSchema) (pEl2->ElStructSchema);
 		  nSiblings = 0;
@@ -443,7 +453,6 @@ ThotBool MergeTextElements (PtrElement pEl, PtrElement *pFreeEl, PtrDocument pDo
 		      pSibling = pSibling->ElPrevious;
 		    }
 		  notifyEl.position = nSiblings;
-		  notifyEl.info = 0;
 		}
 	      /* dechaine le 2eme element */
 	      RemoveElement (pEl2);
@@ -544,54 +553,53 @@ PtrPathSeg CopyPath (PtrPathSeg firstPathEl)
 }
 
 /*----------------------------------------------------------------------
-   StringAndTextEqual compare une chaine de caracteres (String) et 
-   un texte contenu dans une suite de buffers. Retourne TRUE si    
-   les contenus sont egaux.                                        
+   StringAndTextEqual compare a character string (text) and the contents
+   of text buffers.
+   Return TRUE if equals.
   ----------------------------------------------------------------------*/
-ThotBool StringAndTextEqual (STRING String, PtrTextBuffer pBuf)
+ThotBool StringAndTextEqual (char *text, PtrTextBuffer pBuf)
 {
-   ThotBool            equal;
-   int                 l, lenRest;
-   STRING              pChar;
+  unsigned char      *ptr;
+  wchar_t             wc;
+  int                 i;
+  int                 l, length;
+  ThotBool            equal;
 
-   equal = FALSE;
-   if (pBuf == NULL && String == NULL)
-      /* les deux sont vides, egalite' */
-      equal = TRUE;
-   else if (pBuf != NULL && String != NULL)
-     {
-      if (String[0] == EOS && pBuf->BuLength == 0)
-	 equal = TRUE;
+  equal = FALSE;
+  if (pBuf == NULL && text == NULL)
+    /* les deux sont vides, egalite' */
+    equal = TRUE;
+  else if (pBuf && text)
+    {
+      if (text[0] == EOS && pBuf->BuLength == 0)
+	equal = TRUE;
       else
 	{
-	   pChar = String;
-	   lenRest = ustrlen (String);
-	   equal = TRUE;
-	   /* parcourt les buffers de texte successifs */
-	   while (pBuf != NULL && lenRest > 0 && equal)
-	     {
-		if (pBuf->BuLength > 0)
-		  {
-		     if (pBuf->BuLength > lenRest)
-			equal = FALSE;
-		     else
-		       {
-			  if (pBuf->BuLength < lenRest)
-			     l = pBuf->BuLength;
-			  else
-			     l = lenRest;
-			  equal = (ustrncmp (pChar, pBuf->BuContent, l) == 0);
-			  pChar += l;
-			  lenRest -= l;
-		       }
-		  }
-		pBuf = pBuf->BuNext;
-	     }
-	   if (pBuf != NULL || lenRest > 0)
-	      equal = FALSE;
+	  length = strlen (text);
+	  ptr = (unsigned char *)text;
+	  equal = TRUE;
+	  /* parcourt les buffers de texte successifs */
+	  while (pBuf && length > 0 && equal)
+	    {
+	      if (pBuf->BuLength > 0)
+		{
+		  l = 0;
+		  i = 0;
+		  while (equal && l < length && i < pBuf->BuLength)
+		    {
+		      l += TtaMBstringToWC (&ptr, &wc);
+		      equal = (wc == pBuf->BuContent[i]);
+		      i++;
+		    }
+		  length -= l;
+		}
+	      pBuf = pBuf->BuNext;
+	    }
+	  if (pBuf || length > 0)
+	    equal = FALSE;
 	}
-     }
-   return equal;
+    }
+  return equal;
 }
 
 
@@ -601,62 +609,62 @@ ThotBool StringAndTextEqual (STRING String, PtrTextBuffer pBuf)
   ----------------------------------------------------------------------*/
 ThotBool TextsEqual (PtrTextBuffer pBuf1, PtrTextBuffer pBuf2)
 {
-   ThotBool            equal;
-   PtrTextBuffer       pTB1, pTB2;
-   STRING              pChar1;
-   STRING              pChar2;
-   int                 len, lenRest1, lenRest2;
+  PtrTextBuffer       pTB1, pTB2;
+  STRING              pChar1;
+  STRING              pChar2;
+  int                 len, length1, length2;
+  ThotBool            equal;
 
-   equal = FALSE;
-   if (pBuf1 == NULL && pBuf2 == NULL)
+  equal = FALSE;
+  if (pBuf1 == NULL && pBuf2 == NULL)
+    equal = TRUE;
+  else if (pBuf1 != NULL && pBuf2 != NULL)
+    {
+      pTB1 = pBuf1;
+      length1 = pTB1->BuLength;
+      pChar1 = &pTB1->BuContent[0];
+      pTB2 = pBuf2;
+      length2 = pTB2->BuLength;
+      pChar2 = &pTB2->BuContent[0];
       equal = TRUE;
-   else if (pBuf1 != NULL && pBuf2 != NULL)
-     {
-	pTB1 = pBuf1;
-	lenRest1 = pTB1->BuLength;
-	pChar1 = &pTB1->BuContent[0];
-	pTB2 = pBuf2;
-	lenRest2 = pTB2->BuLength;
-	pChar2 = &pTB2->BuContent[0];
-	equal = TRUE;
-	/* parcourt les deux chaines de buffers */
-	while (pTB1 != NULL && pTB2 != NULL && equal)
-	  {
-	     if (lenRest1 == 0)
-	       {
-		  pTB1 = pTB1->BuNext;
-		  if (pTB1 != NULL)
-		    {
-		       lenRest1 = pTB1->BuLength;
-		       pChar1 = &pTB1->BuContent[0];
-		    }
-	       }
-	     if (lenRest2 == 0)
-	       {
-		  pTB2 = pTB2->BuNext;
-		  if (pTB2 != NULL)
-		    {
-		       lenRest2 = pTB2->BuLength;
-		       pChar2 = &pTB2->BuContent[0];
-		    }
-	       }
-	     if (lenRest1 > 0 && lenRest2 > 0)
-	       {
-		  if (lenRest1 > lenRest2)
-		     len = lenRest2;
-		  else
-		     len = lenRest1;
-		  equal = (ustrncmp (pChar1, pChar2, len) == 0);
-		  lenRest1 -= len;
-		  lenRest2 -= len;
-		  pChar1 += len;
-		  pChar2 += len;
-	       }
-	  }
-	if (pTB1 != NULL || pTB2 != NULL || lenRest1 > 0 || lenRest2 > 0)
-	   equal = FALSE;
-     }
-   return equal;
+      /* parcourt les deux chaines de buffers */
+      while (pTB1 != NULL && pTB2 != NULL && equal)
+	{
+	  if (length1 == 0)
+	    {
+	      pTB1 = pTB1->BuNext;
+	      if (pTB1 != NULL)
+		{
+		  length1 = pTB1->BuLength;
+		  pChar1 = &pTB1->BuContent[0];
+		}
+	    }
+	  if (length2 == 0)
+	    {
+	      pTB2 = pTB2->BuNext;
+	      if (pTB2 != NULL)
+		{
+		  length2 = pTB2->BuLength;
+		  pChar2 = &pTB2->BuContent[0];
+		}
+	    }
+	  if (length1 > 0 && length2 > 0)
+	    {
+	      if (length1 > length2)
+		len = length2;
+	      else
+		len = length1;
+	      equal = (ustrncmp (pChar1, pChar2, len) == 0);
+	      length1 -= len;
+	      length2 -= len;
+	      pChar1 += len;
+	      pChar2 += len;
+	    }
+	}
+      if (pTB1 != NULL || pTB2 != NULL || length1 > 0 || length2 > 0)
+	equal = FALSE;
+    }
+  return equal;
 }
 
 
@@ -670,131 +678,152 @@ ThotBool TextsEqual (PtrTextBuffer pBuf1, PtrTextBuffer pBuf2)
   ----------------------------------------------------------------------*/
 void CopyTextToText (PtrTextBuffer pSrceBuf, PtrTextBuffer pCopyBuf, int *len)
 {
-   PtrTextBuffer       pTBSrce, pTBDest;
+  PtrTextBuffer       pTBSrce, pTBDest;
 
-   *len = 0;
-   if (pSrceBuf != NULL && pCopyBuf != NULL)
-     {
-	/* cherche le dernier buffer de la chaine des buffers destination */
-	pTBDest = pCopyBuf;
-	while (pTBDest->BuNext != NULL)
-	   pTBDest = pTBDest->BuNext;
-	pTBSrce = pSrceBuf;
-	/* parcourt la chaine des buffers source */
-	while (pTBSrce != NULL)
-	  {
-	     if (pTBSrce->BuLength > 0)
-		/* ce buffer source n'est pas vide, on le copie */
-	       {
-		  if (THOT_MAX_CHAR - 1 - pTBDest->BuLength <= pTBSrce->BuLength)
-		     /* pas assez de place dans le buffer destination */
-		     /* pour copier tout le texte du buffer source, on */
-		     /* prend un nouveau buffer destination */
-		     pTBDest = NewTextBuffer (pTBDest);
-		  ustrcpy (pTBDest->BuContent + pTBDest->BuLength,
-			   pTBSrce->BuContent);
-		  pTBDest->BuLength += pTBSrce->BuLength;
-		  *len += pTBSrce->BuLength;
-	       }
-	     /* buffer source suivant */
-	     pTBSrce = pTBSrce->BuNext;
-	  }
-     }
+  *len = 0;
+  if (pSrceBuf && pCopyBuf)
+    {
+      /* cherche le dernier buffer de la chaine des buffers destination */
+      pTBDest = pCopyBuf;
+      while (pTBDest->BuNext)
+	pTBDest = pTBDest->BuNext;
+      pTBSrce = pSrceBuf;
+      /* parcourt la chaine des buffers source */
+      while (pTBSrce)
+	{
+	  if (pTBSrce->BuLength > 0)
+	    /* ce buffer source n'est pas vide, on le copie */
+	    {
+	      if (FULL_BUFFER - pTBDest->BuLength < pTBSrce->BuLength)
+		/* pas assez de place dans le buffer destination */
+		/* pour copier tout le texte du buffer source, on */
+		/* prend un nouveau buffer destination */
+		pTBDest = NewTextBuffer (pTBDest);
+	      ustrcpy (pTBDest->BuContent + pTBDest->BuLength,
+		       pTBSrce->BuContent);
+	      pTBDest->BuLength += pTBSrce->BuLength;
+	      *len += pTBSrce->BuLength;
+	    }
+	  /* buffer source suivant */
+	  pTBSrce = pTBSrce->BuNext;
+	}
+    }
 }
 
 
 /*----------------------------------------------------------------------
-   CopyStringToText copie a la fin de la suite de buffers      
-   commencant au buffer pCopyBuf le contenu de la chaine srceStrn.	
-   Ajoute eventuellement de nouveaux buffers a la fin de la suite  
-   de buffers.                                                     
-   Retourne dans LgCopiee le nombre de caracteres copie's.         
+  CopyMBs2Buffer copies max characters of the src string at the position pos in
+  the buffer pBuf.
+  In _I18N_ mode max is a UTF-8 string length;
+  Adds new buffers if necessary and converts multibytes to wide characters
+  if necessary.
+  Returns the number of (wide )characters added in buffers.
   ----------------------------------------------------------------------*/
-void CopyStringToText (CHAR_T* srceStrn, PtrTextBuffer pCopyBuf, int *LgCopiee)
+int CopyMBs2Buffer (unsigned char *src, PtrTextBuffer pBuf, int pos, int max)
 {
-   PtrTextBuffer       pTBDest;
-   CHAR_T*             pSrce;
-   int                 lenRest, len;
+  PtrTextBuffer       pNext;
+  int                 l, length;
 
-   *LgCopiee = 0;
-   if (pCopyBuf != NULL && srceStrn != NULL)
-     {
-	pTBDest = pCopyBuf;
-	/* longueur restant a copier */
-	lenRest = ustrlen (srceStrn);
-	*LgCopiee = lenRest;
-	pSrce = srceStrn;
-	/* cherche le dernier buffer de la chaine de destination */
-	while (pTBDest->BuNext != NULL)
-	   pTBDest = pTBDest->BuNext;
-	/* boucle tant qu'il reste du texte a copier */
-	while (lenRest > 0)
-	  {
-	     if (THOT_MAX_CHAR - 1 - pTBDest->BuLength <= lenRest)
-		/* le texte restant ne tient pas dans le buffer destination */
-		if (pTBDest->BuLength < THOT_MAX_CHAR - 2)
-		   /* il reste un peu de place, on copiera ce qu'on peut */
-		   len = THOT_MAX_CHAR - 1 - pTBDest->BuLength;
-		else
-		   /* on prend un nouveau buffer destination */
-		  {
-		     pTBDest = NewTextBuffer (pTBDest);
-		     len = 0;
-		  }
-	     else
-		/* le texte restant tient dans le buffer destination */
-		len = lenRest;
-	     if (len > 0)
-		/* on fait la copie */
-	       {
-		  ustrncpy (pTBDest->BuContent + pTBDest->BuLength, pSrce, len);
-		  pSrce += len;
-		  pTBDest->BuLength += len;
-		  pTBDest->BuContent[pTBDest->BuLength] = WC_EOS;
-		  lenRest -= len;
-	       }
-	  }
-     }
+  /* number of wide characters added */
+  length = 0;
+  /* continue while there is still text */
+  while (max > 0 && pBuf && src && src[0] != EOS)
+    {
+      l = 0;
+      while (pos < FULL_BUFFER && max > l && src[0] != EOS)
+	{
+	  l += TtaMBstringToWC (&src, &pBuf->BuContent[pos]);
+	  pos++;
+	  length++;
+	}
+      if (src[0] == EOS)
+	max = 0;
+      pBuf->BuLength = pos;
+      pBuf->BuContent[pos] = EOS;
+      max -= l;
+      if (max > 0)
+	{
+	  if (pBuf->BuNext == NULL)
+	    { 
+	      /* add a new buffer */
+	      GetTextBuffer (&pNext);
+	      if (pNext)
+		{
+		  pBuf->BuNext = pNext;
+		  pNext->BuPrevious = pBuf;
+		}
+	    }
+	  pBuf = pBuf->BuNext;
+	  pos = 0;
+	}
+    }
+  return length;
 }
 
 /*----------------------------------------------------------------------
-   CopyTextToString copie dans la chaine pStrCpy le texte	
-   contenu dans le buffer pSrceBuf et dans les buffers suivants.	
-   A l'appel, len contient la longueur maximum a copier.           
-   Au retour, len contient le nombre de caracteres copie's.        
+  CopyBuffer2MBs copies max characters from the position pos in the
+  buffer pBuf to dest string.
+  In _I18N_ mode max a UTF-8 string length
+  Returns the number of Multi Byte characters copied in des.
   ----------------------------------------------------------------------*/
-void CopyTextToString (PtrTextBuffer pSrceBuf, CHAR_T* pStrCpy, int *len)
+int CopyBuffer2MBs (PtrTextBuffer pBuf, int pos, unsigned char *des, int max)
 {
-   int                 LgMax;
-   int                 l;
-   PtrTextBuffer       pTBSrce;
-   STRING              pDest;
+  unsigned char       s[10], *ptr;
+  int                 l, length;
 
-   if (pSrceBuf == NULL || pStrCpy == NULL || *len <= 0)
-      *len = 0;
-   else
-     {
-	LgMax = *len;
-	*len = 0;
-	pTBSrce = pSrceBuf;
-	pDest = pStrCpy;
-	while (pTBSrce != NULL && LgMax > 0)
-	  {
-	     if (pTBSrce->BuLength > 0)
-	       {
-		  if (pTBSrce->BuLength < LgMax)
-		     l = pTBSrce->BuLength;
-		  else
-		     l = LgMax;
-		  ustrncpy (pDest, pTBSrce->BuContent, l);
-		  pDest += l;
-		  *pDest = EOS;
-		  LgMax -= l;
-		  *len += l;
-	       }
-	     pTBSrce = pTBSrce->BuNext;
-	  }
-     }
+  length = 0;
+  /* continue while there is still text */
+  while (max > 0 && pBuf)
+    {
+      l = 0;
+      while (pos < pBuf->BuLength && max > 0)
+	{
+	  ptr = s;
+	  l = TtaWCToMBstring (pBuf->BuContent[pos], &ptr);
+	  pos++;
+	  if (l <= max)
+	    {
+	      /* there is enough space to insert these characters */
+	      strncpy ((char *)&des[length], (char *)s, l);
+	      length += l;
+	    }
+	  max -= l;
+	}
+
+      if (max > 0)
+	{
+	  /* move to the new buffer */
+	  pBuf = pBuf->BuNext;
+	  pos = 0;
+	}
+    }
+  des[length] = EOS;
+  return length;
+}
+
+
+/*----------------------------------------------------------------------
+  CopyStringToBuffer adds the src string at the end of buffers.
+  Adds new buffers if necessary and converts into wide characters.    
+  Returns the number of copied characters.
+  ----------------------------------------------------------------------*/
+void CopyStringToBuffer (unsigned char *src, PtrTextBuffer pDestBuf, int *length)
+{
+  PtrTextBuffer       pBuf;
+  int                 max;
+
+  *length = 0;
+  if (pDestBuf && src)
+    {
+      pBuf = pDestBuf;
+      /* string length to copy */
+      max = strlen ((char *)src);
+      *length = max;
+      /* look for the end of buffers */
+      while (pBuf->BuNext != NULL)
+	pBuf = pBuf->BuNext;
+      CopyMBs2Buffer (src, pBuf, pBuf->BuLength, max);
+    }
 }
 
 
@@ -1000,17 +1029,15 @@ void ModifyPointInPolyline (PtrTextBuffer firstBuffer, int rank, int x, int y)
    element of type Text.
    Return value:
    textLength: the number of characters contained in the element.
-   In _I18N mode the length corresponds to the UTF-8 string.
+   In _I18N_ mode the length corresponds to the UTF-8 string.
   ----------------------------------------------------------------------*/
 int TtaGetTextLength (Element element)
 {
   PtrElement          pEl;
   int                 length;
-#ifdef _I18N
   PtrTextBuffer       pBuf;
   unsigned char       c[10], *ptr;
   int                 i, l;
-#endif /* _I18N */
 
   UserErrorCode = 0;
   length = 0;
@@ -1022,27 +1049,26 @@ int TtaGetTextLength (Element element)
   else if (pEl->ElLeafType != LtText && pEl->ElLeafType != LtPicture)
     TtaError (ERR_invalid_element_type);
   else
-    length = pEl->ElTextLength;
-#ifdef _I18N
-  pBuf = pEl->ElText;
-  l = 0;
-  while (pBuf != NULL && length > 0)
     {
-      i = 0;
-      while (i < pBuf->BuLength && l < length)
+      length = pEl->ElTextLength;
+      pBuf = pEl->ElText;
+      l = 0;
+      while (pBuf != NULL && length > 0)
 	{
-	  ptr = c;
-	  l += TtaWC2MBs ((wchar_t *)&pBuf->BuContent[i], &ptr, UTF_8);
-	  i++;
+	  i = 0;
+	  while (i < pBuf->BuLength && i < length)
+	    {
+	      ptr = c;
+	      l += TtaWCToMBstring (pBuf->BuContent[i], &ptr);
+	      i++;
+	    }
+	  /* next buffer */
+	  length -= pBuf->BuLength;
+	  pBuf = pBuf->BuNext;
 	}
-      /* next buffer */
-      length -= pBuf->BuLength;
-      pBuf = pBuf->BuNext;
+      length = l;
     }
-  return l;
-#else /* _I18N */
   return length;
-#endif /* _I18N */
 }
 
 /*----------------------------------------------------------------------
@@ -1055,12 +1081,12 @@ int TtaGetTextLength (Element element)
    buffer: the buffer that will contain the text. This buffer
    must be at least of size length.
    length: maximum length of that buffer.
-   In _I18N mode the length corresponds to the UTF-8 string.
+   In _I18N_ mode the length corresponds to the UTF-8 string.
    Return parameters:
    buffer: (the buffer contains the text).
    length: actual length of the text in the buffer.
    language: language of the text.
-   In _I18N mode returns a UTF-8 string.
+   In _I18N_ mode returns a UTF-8 string.
   ----------------------------------------------------------------------*/
 void TtaGiveTextContent (Element element, unsigned char *buffer, int *length,
 			 Language *language)
@@ -1069,9 +1095,6 @@ void TtaGiveTextContent (Element element, unsigned char *buffer, int *length,
   PtrTextBuffer       pBuf;
   unsigned char      *ptr;
   int                 len, l;
-#ifdef _I18N
-  int                 i;
-#endif /* _I18N */
 
   UserErrorCode = 0;
   pEl = (PtrElement) element;
@@ -1088,22 +1111,12 @@ void TtaGiveTextContent (Element element, unsigned char *buffer, int *length,
       ptr = buffer;
       while (pBuf != NULL && len < (*length) - 1)
 	{
-#ifdef _I18N
 	  l = 0;
 	  while (l < pBuf->BuLength && len < *length)
 	    {
-	      len += TtaWC2MBs ((wchar_t *) &pBuf->BuContent[l], &ptr, UTF_8);
+	      len += TtaWCToMBstring (pBuf->BuContent[l], &ptr);
 	      l++;
 	    }
-#else /* _I18N */
-	  if ((*length) < len + pBuf->BuLength + 1)
-	    l = (*length) - len;
-	  else
-	    l = pBuf->BuLength + 1;
-	  ustrncpy (ptr, pBuf->BuContent, l);
-	  ptr = ptr + (l - 1);
-	  len = len + (l - 1);
-#endif /* _I18N */
 	  pBuf = pBuf->BuNext;
 	}
       *length = len;
@@ -1111,5 +1124,348 @@ void TtaGiveTextContent (Element element, unsigned char *buffer, int *length,
       *language = pEl->ElLanguage;
     }
 }
+/*----------------------------------------------------------------------
+   TtaCopyPath : Copy Path linked list 
+  ----------------------------------------------------------------------*/
+void *TtaCopyPath (void *void_src)
+{
+#ifdef _GL
+  int npoints;
 
+  AnimPath *pop_path = (AnimPath *) void_src;
+  AnimPath *new_path = (AnimPath *) TtaGetMemory (sizeof(AnimPath));
 
+  memset (new_path, 0, sizeof(AnimPath));
+
+  npoints = pop_path->npoints + 1;
+
+  new_path->npoints = pop_path->npoints;
+  new_path->length = pop_path->length;
+
+  new_path->Path = (ThotPoint *)TtaGetMemory (npoints * sizeof(ThotPoint));
+  new_path->Proportion = (float *)TtaGetMemory (npoints * sizeof(float));
+  new_path->Tangent_angle = (float *)TtaGetMemory (npoints * sizeof(float));
+
+  npoints --;
+
+  memcpy (new_path->Path, pop_path->Path, npoints*sizeof(ThotPoint));
+  memcpy (new_path->Proportion, pop_path->Proportion, npoints * sizeof(float));
+  memcpy (new_path->Tangent_angle, pop_path->Tangent_angle, npoints * sizeof(float));
+
+  return new_path;
+#else /*_GL*/
+  return NULL;
+#endif /* _GL */
+}
+/*----------------------------------------------------------------------
+   TtaCopyAnim : Copy anim linked list 
+  ----------------------------------------------------------------------*/
+void *TtaCopyAnim (void *void_src)
+{
+  Animated_Element *dest, *current;
+  Animated_Element *src = (Animated_Element *) void_src;
+  
+  dest = (Animated_Element*)TtaNewAnimation ();
+  current = dest; 
+  while (src)
+    {
+      current->duration = src->duration;
+      current->start = src->start;
+      current->action_time = src->action_time;
+      current->AnimType = src->AnimType;
+      current->Fill = src->Fill;
+      current->repeatCount = src->repeatCount;
+
+      if (current->AnimType == Motion)
+	{
+	  if (src->from)
+	    current->from = TtaCopyPath (src->from);
+	}
+      else
+	{
+	  if (src->from)
+	    {	 
+	      current->from = TtaGetMemory (strlen ((char*)src->from) + 1);     
+	      strcpy ((char*)current->from, (char*)src->from); 
+	    }
+	  if (src->to)
+	    {
+	      current->to = TtaGetMemory (strlen ((char*)src->to) + 1);     
+	      strcpy ((char*)current->to, (char*)src->to);
+	    }
+	}
+      if (src->AttrName)
+	switch (current->AnimType)
+	  {
+	  case Motion:
+	    break;
+	  case Color:
+	    current->AttrName = (char*)TtaGetMemory (strlen ((char*)src->AttrName) + 1);     
+	    strcpy ((char*)current->AttrName, (char*)src->AttrName);  
+	    break;   
+	  case Set:
+	    current->AttrName = (char*)TtaGetMemory (strlen ((char*)src->AttrName) + 1);     
+	    strcpy ((char*)current->AttrName, (char*)src->AttrName);  
+	    break;
+	  case Transformation:
+	    current->AttrName = (char*)TtaGetMemory (sizeof (int));      
+	    *(current->AttrName) = *(src->AttrName);      
+	    break; 
+	  case Animate:
+	    current->AttrName = (char*)TtaGetMemory (strlen ((char*)src->AttrName) + 1);     
+	    strcpy ((char*)current->AttrName, (char*)src->AttrName);
+	    break;
+	  default:
+	    break; 
+	  } 
+      if (src->next)
+	{
+	  current->next = (Animated_Element*)TtaNewAnimation ();
+	  current = current->next; 
+	}
+      else
+	current->next = NULL;
+     
+      src = src->next;
+    } 
+  return (void *) dest;
+}
+/*----------------------------------------------------------------------
+   TtaReplaceTransform
+
+   Insert or Replace a Transform at the beginning of a Graphics basic element
+   if a transformation of the same type exists in the list, it is replaced
+   Parameters:
+   element: the Graphics element to be modified.
+   transform: the path segment to be inserted.
+   document: the document containing the element.
+  ----------------------------------------------------------------------*/
+void TtaReplaceTransform (Element element, void *transform, 
+			 Document document)
+{
+   PtrTransform       pPa, pPrevPa;
+
+   UserErrorCode = 0;
+   if (element == NULL)
+      TtaError (ERR_invalid_parameter);
+   else
+     /* verifies the parameter document */
+     if (document < 1 || document > MAX_DOCUMENTS)
+       TtaError (ERR_invalid_document_parameter);
+     else if (LoadedDocument[document - 1] == NULL)
+       TtaError (ERR_invalid_document_parameter);
+   else
+      /* parameter document is correct */
+     {
+       if (element && transform)
+	 {
+	  pPa = ((PtrElement) element)->ElTransform;
+	   pPrevPa = NULL;
+	   while (pPa)
+	     {
+	       if (pPa->TransType == ((PtrTransform)transform)->TransType)
+		 {
+		   if (pPrevPa == NULL)
+		     ((PtrElement) element)->ElTransform = (PtrTransform) transform;
+		   else
+		     pPrevPa->Next = (PtrTransform) transform;
+		   ((PtrTransform) transform)->Next = pPa->Next;
+		   TtaFreeMemory (pPa);	
+		  return;
+		 }	       
+	       pPrevPa = pPa;
+	       pPa = pPa->Next;
+	     }
+	   if (pPrevPa == NULL)
+	     ((PtrElement) element)->ElTransform = (PtrTransform) transform;
+	   else
+	     pPrevPa->Next = (PtrTransform) transform;
+	 }
+     }
+}
+
+/*----------------------------------------------------------------------
+  TtaCopyTransform
+  Copy a Linked List of transforms into another struct
+  ----------------------------------------------------------------------*/
+void *TtaCopyTransform(void *void_pPa)
+{
+  PtrTransform result_first, current;  
+  PtrTransform pPa = (PtrTransform) void_pPa;
+  
+  result_first = (Transform*)TtaNewTransform ();
+  current = result_first;      
+  while (pPa)
+    {      
+      current->TransType = pPa->TransType;    
+      switch (pPa->TransType)
+	{
+	case PtElRotate:
+	  current->XRotate = pPa->XRotate;	  
+	  current->YRotate = pPa->YRotate;	  
+	  current->TrAngle = pPa->TrAngle;
+	  break;  
+	case PtElMatrix:
+	  current->AMatrix = pPa->AMatrix;
+	  current->BMatrix = pPa->BMatrix;
+	  current->CMatrix = pPa->CMatrix;
+	  current->DMatrix = pPa->DMatrix;
+	  current->EMatrix = pPa->EMatrix;
+	  current->FMatrix = pPa->FMatrix;
+	  break;	  
+	case PtElSkewX:
+	case PtElSkewY:
+	  current->TrFactor = pPa->TrFactor;
+	  break;	  
+	default:
+	  current->XScale = pPa->XScale;	  
+	  current->YScale = pPa->YScale;	  
+	  break;
+	}	       
+	if (pPa->Next)
+	  {
+	    current->Next = (Transform*)TtaGetMemory (sizeof (Transform));
+	    current = current->Next;
+	  }
+	else
+	  current->Next = NULL;
+      pPa = pPa->Next;
+    }
+  return (void *) result_first;  
+}
+/*----------------------------------------------------------------------
+   TtaNewBoxTransformTranslate
+   ---------------------------------------------------------------------- */
+void *TtaNewBoxTransformTranslate (float x, float y)   
+{
+   PtrTransform pPa;
+
+   pPa = (Transform*)TtaNewTransform ();
+   pPa->TransType = PtElBoxTranslate;
+   pPa->XScale = x;
+   pPa->YScale = y;
+   return (pPa);
+}
+
+/*----------------------------------------------------------------------
+   TtaNewTransformTranslate
+   ---------------------------------------------------------------------- */
+void *TtaNewTransformTranslate (float x, float y, ThotBool viewbox)   
+{
+   PtrTransform pPa;
+
+   pPa = (Transform*)TtaNewTransform ();
+   if (viewbox)
+     pPa->TransType = PtElviewboxTranslate;
+   else
+     pPa->TransType = PtElTranslate;
+   pPa->XScale = x;
+   pPa->YScale = y;
+   return (pPa);
+}
+/*----------------------------------------------------------------------
+   TtaNewTransformAnimTranslate
+   ---------------------------------------------------------------------- */
+void *TtaNewTransformAnimTranslate (float x, float y)   
+{
+   PtrTransform pPa;
+
+   pPa = (Transform*)TtaNewTransform ();
+   pPa->TransType = PtElAnimTranslate;
+   pPa->XScale = x;
+   pPa->YScale = y;
+   return (pPa);
+}
+/*----------------------------------------------------------------------
+   TtaNewTransformAnimRotate
+   ---------------------------------------------------------------------- */
+void *TtaNewTransformAnimRotate (float angle, float x_scale, 
+			     float y_scale)				   
+{
+   PtrTransform pPa;
+
+   pPa = (Transform*)TtaNewTransform ();
+   pPa->TransType = PtElAnimRotate;
+   pPa->TrAngle = angle;
+   pPa->XRotate = x_scale;
+   pPa->YRotate = y_scale;
+   return (pPa);
+}
+
+/*----------------------------------------------------------------------
+   TtaNewTransformRotate
+   ---------------------------------------------------------------------- */
+void *TtaNewTransformRotate (float angle, float x_scale, 
+			     float y_scale)				   
+{
+   PtrTransform pPa;
+
+   pPa = (Transform*)TtaNewTransform ();
+   pPa->TransType = PtElRotate;
+   pPa->TrAngle = angle;
+   pPa->XRotate = x_scale;
+   pPa->YRotate = y_scale;
+   return (pPa);
+}
+
+/*----------------------------------------------------------------------
+   TtaNewTransformSkewX
+   ---------------------------------------------------------------------- */
+void *TtaNewTransformSkewX (float factor)
+				   
+{
+   PtrTransform pPa;
+
+   pPa = (Transform*)TtaNewTransform ();
+   pPa->TransType = PtElSkewX;
+   pPa->TrFactor = factor;
+   return (pPa);
+}
+
+/*----------------------------------------------------------------------
+  TtaNewTransformSkewY
+  ---------------------------------------------------------------------- */
+void *TtaNewTransformSkewY (float factor)
+{
+   PtrTransform pPa;
+
+   pPa = (Transform*)TtaNewTransform ();
+   pPa->TransType = PtElSkewY;
+   pPa->TrFactor = factor;
+   return (pPa);
+}
+/*----------------------------------------------------------------------
+   TtaNewTransformScale
+   ---------------------------------------------------------------------- */
+void *TtaNewTransformScale (float x_scale, float y_scale, ThotBool viewbox)	   
+{
+   PtrTransform pPa;
+
+   pPa = (Transform*)TtaNewTransform ();
+   if (viewbox)
+     pPa->TransType = PtElviewboxScale;
+   else
+     pPa->TransType = PtElScale;   
+   pPa->XScale = x_scale;
+   pPa->YScale = y_scale;
+   return (pPa);
+}
+
+/*----------------------------------------------------------------------
+  TtaNewTransformMatrix
+  ---------------------------------------------------------------------- */
+void *TtaNewTransformMatrix (float a, float b, float c,
+			     float d, float e, float f)
+{
+   PtrTransform pPa;
+
+   pPa = (Transform*)TtaNewTransform ();
+   pPa->TransType = PtElMatrix;
+   pPa->AMatrix = a;
+   pPa->BMatrix = b;
+   pPa->CMatrix = c;
+   pPa->DMatrix = d;
+   pPa->EMatrix = e;
+   pPa->FMatrix = f;
+   return (pPa);
+}
