@@ -1,13 +1,12 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996.
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
  
 /*
    This module processes regular expressions.
-
  */
 
 #ifdef RS
@@ -41,52 +40,51 @@
 
 /* This data structure is used to represent a compiled pattern. */
 struct re_pattern_buffer
-  {
-     char               *buffer;	/* Space holding the compiled pattern commands. */
-     int                 allocated;	/* Size of space that  buffer  points to */
-     int                 used;	/* Length of portion of buffer actually occupied */
-     char               *fastmap;	/* Pointer to fastmap, if any, or zero if none. */
-     /* re_search uses the fastmap, if there is one, to
-        skip quickly over totally implausible characters */
-     char               *translate;	/* Translate table to apply to all characters before
-					   comparing.
-					   Or zero for no translation.
-					   The translation is applied to a pattern when it is
-					   compiled and to data when it is matched. */
-     char                fastmap_accurate;
-     /* Set to zero when a new pattern is stored,
-        set to one when the fastmap is updated from it. */
-     char                can_be_null;	/* Set to one by compiling fastmap
-					   if this pattern might match the null string.
-					   It does not necessarily match the null string
-					   in that case, but if this is zero, it cannot.
-					   2 as value means can match null string
-					   but at end of range or before a character
-					   listed in the fastmap.  */
-  };
+{
+  char               *buffer;       /* Space holding the compiled pattern commands. */
+  int                 allocated;    /* Size of space that  buffer  points to */
+  int                 used;	/* Length of portion of buffer actually occupied */
+  char               *fastmap;      /* Pointer to fastmap, if any, or zero if none. */
+  /* re_search uses the fastmap, if there is one, to
+     skip quickly over totally implausible characters */
+  char               *translate;    /* Translate table to apply to all characters
+				       before comparing.
+				       Or zero for no translation.
+				       The translation is applied to a pattern when it is
+				       compiled and to data when it is matched. */
+  char                fastmap_accurate;
+  /* Set to zero when a new pattern is stored,
+     set to one when the fastmap is updated from it. */
+  char                can_be_null;  /* Set to one by compiling fastmap
+				       if this pattern might match the null string.
+				       It does not necessarily match the null string
+				       in that case, but if this is zero, it cannot.
+				       2 as value means can match null string
+				       but at end of range or before a character
+				       listed in the fastmap.  */
+};
 
 
 /* Structure to store "register" contents data in.
-
    Pass the address of such a structure as an argument to re_match, etc.,
    if you want this information back.
-
+   
    start[i] and end[i] record the string matched by \( ... \) grouping i,
    for i from 1 to RE_NREGS - 1.
    start[0] and end[0] record the entire string matched. */
 
 struct re_registers
-  {
-     int                 start[RE_NREGS];
-     int                 end[RE_NREGS];
-  };
+{
+  int                 start[RE_NREGS];
+  int                 end[RE_NREGS];
+};
 
 struct t_stack
-  {
-     unsigned char      *pchar;
-     PtrElement          pEl;
-     int                 index;
-  };
+{
+  unsigned char      *pchar;
+  PtrElement          pEl;
+  int                 index;
+};
 
 
 /* These are the command codes that appear in compiled regular expressions,
@@ -96,89 +94,89 @@ struct t_stack
    Zero-bytes may appear in the compiled regular expression. */
 
 enum regexpcode
-  {
-     unused,
-     exactn,			/* followed by one byte giving n, and then by n literal bytes */
-     begline,			/* fails unless at beginning of line */
-     endline,			/* fails unless at end of line */
-     jump,			/* followed by two bytes giving relative address to jump to */
-     on_failure_jump,		/* followed by two bytes giving relative address of
+{
+  unused,
+  exactn,			/* followed by one byte giving n, and then by n literal bytes */
+  begline,			/* fails unless at beginning of line */
+  endline,			/* fails unless at end of line */
+  jump,			/* followed by two bytes giving relative address to jump to */
+  on_failure_jump,		/* followed by two bytes giving relative address of
 				   place to resume at in case of failure. */
-     finalize_jump,		/* Throw away latest failure point and then jump to
+  finalize_jump,		/* Throw away latest failure point and then jump to
 				   address. */
-     maybe_finalize_jump,	/* Like jump but finalize if safe to do so.
-				   This is used to jump back to the beginning
-				   of a repeat.  If the command that follows
-				   this jump is clearly incompatible with the
-				   one at the beginning of the repeat, such that
-				   we can be sure that there is no use backtracking
-				   out of repetitions already completed,
-				   then we finalize. */
-     dummy_failure_jump,	/* jump, and push a dummy failure point.
-				   This failure point will be thrown away
-				   if an attempt is made to use it for a failure.
-				   A + construct makes this before the first repeat */
-     anychar,			/* matches any one character */
-     charset,			/* matches any one char belonging to specified set.
-				   First following byte is # bitmap bytes.
-				   Then come bytes for a bit-map saying which chars are in.
-				   Bits in each byte are ordered low-bit-first.
-				   A character is in the set if its bit is 1.
-				   A character too large to have a bit in the map
-				   is automatically not in the set */
-     charset_not,		/* similar but match any character that is NOT one of those
-				   specified */
-     start_memory,		/* starts remembering the text that is matched
-				   and stores it in a memory register.
-				   followed by one byte containing the register number.
-				   Register numbers must be in the range 0 through NREGS. */
-     stop_memory,		/* stops remembering the text that is matched
-				   and stores it in a memory register.
-				   followed by one byte containing the register number.
-				   Register numbers must be in the range 0 through NREGS. */
-     duplicate,			/* match a duplicate of something remembered.
+  maybe_finalize_jump,	/* Like jump but finalize if safe to do so.
+			   This is used to jump back to the beginning
+			   of a repeat.  If the command that follows
+			   this jump is clearly incompatible with the
+			   one at the beginning of the repeat, such that
+			   we can be sure that there is no use backtracking
+			   out of repetitions already completed,
+			   then we finalize. */
+  dummy_failure_jump,	/* jump, and push a dummy failure point.
+			   This failure point will be thrown away
+			   if an attempt is made to use it for a failure.
+			   A + construct makes this before the first repeat */
+  anychar,		   /* matches any one character */
+  charset,		   /* matches any one char belonging to specified set.
+			      First following byte is # bitmap bytes.
+			      Then come bytes for a bit-map saying which chars are in.
+			      Bits in each byte are ordered low-bit-first.
+			      A character is in the set if its bit is 1.
+			      A character too large to have a bit in the map
+			      is automatically not in the set */
+  charset_not,		/* similar but match any character that is NOT one of those
+			   specified */
+  start_memory,		/* starts remembering the text that is matched
+			   and stores it in a memory register.
+			   followed by one byte containing the register number.
+			   Register numbers must be in the range 0 through NREGS. */
+  stop_memory,		/* stops remembering the text that is matched
+			   and stores it in a memory register.
+			   followed by one byte containing the register number.
+			   Register numbers must be in the range 0 through NREGS. */
+  duplicate,			/* match a duplicate of something remembered.
 				   Followed by one byte containing the index of the memory
 				   register. */
-     before_dot,		/* Succeeds if before dot */
-     at_dot,			/* Succeeds if at dot */
-     after_dot,			/* Succeeds if after dot */
-     begbuf,			/* Succeeds if at beginning of buffer */
-     endbuf,			/* Succeeds if at end of buffer */
-     wordchar,			/* Matches any word-constituent character */
-     notwordchar,		/* Matches any char that is not a word-constituent */
-     wordbeg,			/* Succeeds if at word beginning */
-     wordend,			/* Succeeds if at word end */
-     wordbound,			/* Succeeds if at a word boundary */
-     notwordbound,		/* Succeeds if not at a word boundary */
-     syntaxspec,		/* Matches any character whose syntax is specified.
-				   followed by a byte which contains a syntax code, Sword or
-				   such like */
-     notsyntaxspec		/* Matches any character whose syntax differs from the
-				   specified. */
-  };
+  before_dot,		/* Succeeds if before dot */
+  at_dot,			/* Succeeds if at dot */
+  after_dot,			/* Succeeds if after dot */
+  begbuf,			/* Succeeds if at beginning of buffer */
+  endbuf,			/* Succeeds if at end of buffer */
+  wordchar,			/* Matches any word-constituent character */
+  notwordchar,		/* Matches any char that is not a word-constituent */
+  wordbeg,			/* Succeeds if at word beginning */
+  wordend,			/* Succeeds if at word end */
+  wordbound,			/* Succeeds if at a word boundary */
+  notwordbound,		/* Succeeds if not at a word boundary */
+  syntaxspec,		/* Matches any character whose syntax is specified.
+			   followed by a byte which contains a syntax code, Sword or
+			   such like */
+  notsyntaxspec		/* Matches any character whose syntax differs from the
+			   specified. */
+};
 
 
 /* A syntax table is a Lisp vector of length 0400, whose elements are integers.
    The low 8 bits of the integer is a code, as follows: */
 
 enum syntaxcode
-  {
-     Sspace,			/* for a separator character */
-     Swhitespace,		/* for a whitespace character */
-     Spunct,			/* for random punctuation characters */
-     Sword,			/* for a word constituent */
-     Ssymbol,			/* symbol constituent but not word constituent */
-     Sopen,			/* for a beginning delimiter */
-     Sclose,			/* for an ending delimiter */
-     Squote,			/* for a prefix character like Lisp ' */
-     Sstring,			/* for a string-grouping character like Lisp " */
-     Sinter,			/* for a high punctuation ? or ! */
-     Sescape,			/* for a character that begins a C-style escape */
-     Scharquote,		/* for a character that quotes the following character */
-     Scomment,			/* for a comment-starting character */
-     Sendcomment,		/* for a comment-ending character */
-     Smax			/* Upper bound on codes that are meaningful */
-  };
+{
+  Sspace,			/* for a separator character */
+  Swhitespace,		/* for a whitespace character */
+  Spunct,			/* for random punctuation characters */
+  Sword,			/* for a word constituent */
+  Ssymbol,			/* symbol constituent but not word constituent */
+  Sopen,			/* for a beginning delimiter */
+  Sclose,			/* for an ending delimiter */
+  Squote,			/* for a prefix character like Lisp ' */
+  Sstring,			/* for a string-grouping character like Lisp " */
+  Sinter,			/* for a high punctuation ? or ! */
+  Sescape,			/* for a character that begins a C-style escape */
+  Scharquote,		/* for a character that quotes the following character */
+  Scomment,			/* for a comment-starting character */
+  Sendcomment,		/* for a comment-ending character */
+  Smax			/* Upper bound on codes that are meaningful */
+};
 
 
 /* Indexed by a character, gives the upper case equivalent of the character */
@@ -281,7 +279,7 @@ char                syntax_code_spec[14] =
 #endif
 #endif
 
-#ifdef _WINDOWS
+#ifdef _WINGUI
 #define alloca(x) _alloca(x)
 #endif
 
@@ -1288,8 +1286,10 @@ static int ReMatch (struct re_pattern_buffer *pbufp, PtrElement pEl1,
 			 {
 			    /* go to next text element */
 			    *pEl2 = forw ?
-			       FwdSearchTypedElem (*pEl2, CharString + 1, NULL) :
-			       BackSearchTypedElem (*pEl2, CharString + 1, NULL);
+			       FwdSearchTypedElem (*pEl2, CharString + 1, NULL,
+						   NULL) :
+			       BackSearchTypedElem (*pEl2, CharString + 1,
+						    NULL, NULL);
 			    if ((*pEl2) != NULL)
 			       *pos2 = forw ? 1 : (*pEl2)->ElTextLength;
 			    else
@@ -1301,7 +1301,7 @@ static int ReMatch (struct re_pattern_buffer *pbufp, PtrElement pEl1,
 			    if (pos1 == 0 && !forw)
 			      {
 				 /* go to previous text element */
-				 *pEl2 = BackSearchTypedElem (*pEl2, CharString + 1, NULL);
+				 *pEl2 = BackSearchTypedElem (*pEl2, CharString + 1, NULL, NULL);
 				 if ((*pEl2) != NULL)
 				    *pos2 = forw ? 1 : (*pEl2)->ElTextLength;
 				 else
@@ -1311,7 +1311,7 @@ static int ReMatch (struct re_pattern_buffer *pbufp, PtrElement pEl1,
 			    else if (pos1 > pEl1->ElTextLength && forw)
 			      {
 				 /* go to next text */
-				 *pEl2 = FwdSearchTypedElem (*pEl2, CharString + 1, NULL);
+				 *pEl2 = FwdSearchTypedElem (*pEl2, CharString + 1, NULL, NULL);
 				 if ((*pEl2) != NULL)
 				    *pos2 = forw ? 1 : (*pEl2)->ElTextLength;
 				 else
@@ -1878,8 +1878,8 @@ ThotBool SearchRegularExpression (PtrElement *firstEl, int *firstChar,
 		  if (pEl1 != NULL && pEl1->ElTypeNumber != 1)
 		    {
 		       pEl1 = forward ?
-			  FwdSearchTypedElem (pEl1, CharString + 1, NULL) :
-			  BackSearchTypedElem (NextElement (pEl1), CharString + 1, NULL);
+			  FwdSearchTypedElem (pEl1, CharString + 1, NULL, NULL) :
+			  BackSearchTypedElem (NextElement (pEl1), CharString + 1, NULL, NULL);
 		       if (pEl1 != NULL)
 			  inx1 = forward ? 1 : pEl1->ElTextLength;
 		    }
@@ -1899,9 +1899,8 @@ ThotBool SearchRegularExpression (PtrElement *firstEl, int *firstChar,
 				 if (*textchar == -1)
 				   {
 				      pEl1 = forward ?
-					 FwdSearchTypedElem (pEl1, CharString + 1, NULL) :
-					 BackSearchTypedElem (NextElement (pEl1), CharString + 1,
-							      NULL);
+					 FwdSearchTypedElem (pEl1, CharString + 1, NULL, NULL) :
+					 BackSearchTypedElem (NextElement (pEl1), CharString + 1, NULL, NULL);
 				      if (pEl1 == NULL)
 					{
 					   stop = TRUE;
@@ -1985,9 +1984,9 @@ ThotBool SearchRegularExpression (PtrElement *firstEl, int *firstChar,
 			    if (*textchar == -1)
 			      {
 				 pEl1 = forward ?
-				    FwdSearchTypedElem (pEl1old, CharString + 1, NULL) :
+				    FwdSearchTypedElem (pEl1old, CharString + 1, NULL, NULL) :
 				    BackSearchTypedElem (pEl1old, CharString + 1,
-							 NULL);
+							 NULL, NULL);
 				 if (pEl1 == NULL)
 				    goto Lstop;
 				 else
