@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1999-2001
+ *  (c) COPYRIGHT INRIA, 1999-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -15,8 +15,9 @@
  * Author: V. Quint (INRIA)
  *
  */
-
+#include "thot_gui.h"
 #include "thot_sys.h"
+#include "ustring.h"
 #include "constmedia.h"
 #include "typemedia.h"
 #include "fileaccess.h"
@@ -25,6 +26,7 @@
 #include "message.h"
 #include "typecorr.h"
 #include "appdialogue.h"
+#include "application.h"
 
 #define THOT_EXPORT extern
 #include "platform_tv.h"
@@ -51,7 +53,9 @@
 #include "structmodif_f.h"
 #include "structschema_f.h"
 #include "structselect_f.h"
+#include "thotmsg_f.h"
 #include "tree_f.h"
+#include "uconvert_f.h"
 #include "views_f.h"
 
 /*----------------------------------------------------------------------
@@ -62,14 +66,14 @@
    Selectionne la chaine remplace'e si select est True.		
   ----------------------------------------------------------------------*/
 void ReplaceString (PtrDocument pDoc, PtrElement pEl, int firstChar,
-		    int stringLen, CHAR_T replaceStr[THOT_MAX_CHAR],
+		    int stringLen, CHAR_T *replaceStr,
 		    int replaceLen, ThotBool select)
 {
-  int                 ibuf1, ibuf2, len, diff, dvol, view, i;
   PtrTextBuffer       pBuf1, pBuf2, pBufn;
   PtrAbstractBox      pAb;
   PtrElement          pAsc;
   NotifyOnTarget      notifyEl;
+  int                 ibuf1, ibuf2, len, diff, dvol, view, i;
   ThotBool            DontReplace;
   ThotBool            visible;
 
@@ -92,58 +96,57 @@ void ReplaceString (PtrDocument pDoc, PtrElement pEl, int firstChar,
   if (DontReplace)
     return;
 
-  /* cherche le buffer du premier caractere a remplacer: pBuf1 */
+  /* buffer of the first character to be replaced */
   pBuf1 = pEl->ElText;
   len = 0;
-  /* longueur cumulee des buffers de texte precedant le */
-  /* buffer contenant le 1er caractere */
   while (len + pBuf1->BuLength < firstChar)
     {
       len += pBuf1->BuLength;
       pBuf1 = pBuf1->BuNext;
     }
-  ibuf1 = firstChar - len;
-  /* index dans le buffer de texte pointe par pBuf1 */
-  /* du 1er caractere a remplacer */
-  /* cherche le buffer du dernier caractere a remplacer: pBuf2 */
+  /* index of the first character to be replaced */
+  ibuf1 = firstChar - len - 1;
+  /* buffer of the character after the replaced string */
   pBuf2 = pBuf1;
-  while (len + pBuf2->BuLength < firstChar + stringLen - 1)
+  while (len + pBuf2->BuLength < firstChar + stringLen && pBuf2->BuNext)
     {
       len += pBuf2->BuLength;
       pBuf2 = pBuf2->BuNext;
     }
+  /* index of the character after the replaced string */
   ibuf2 = firstChar + stringLen - 1 - len;
-  /* index dans le buffer de texte */
-  /* pointe' par pBuf2 du dernier caractere a remplacer */
   if (replaceLen > stringLen)
     /* la chaine de remplacement est plus longue que la chaine */
     /* a remplacer */
     {
       diff = replaceLen - stringLen;
-      if (pBuf2->BuLength + diff > THOT_MAX_CHAR - 1)
+      if (pBuf2->BuLength + diff > FULL_BUFFER)
 	/* il n'y a pas assez de place */
 	/* ajoute un buffer apres le dernier buffer de la chaine */
 	/* a remplacer */
 	{
+	  /* add a new buffer */
 	  pBufn = pBuf2->BuNext;
 	  GetTextBuffer (&pBuf2->BuNext);
 	  pBuf2->BuNext->BuPrevious = pBuf2;
 	  pBuf2->BuNext->BuNext = pBufn;
-	  if (pBufn != NULL)
+	  if (pBufn)
 	    pBufn->BuPrevious = pBuf2->BuNext;
 	  pBufn = pBuf2->BuNext;
-	  /* recopie la fin du buffer dans le nouveau */
-	  for (i = 0; i <= pBuf2->BuLength + diff - THOT_MAX_CHAR; i++)
-	    pBufn->BuContent[i] = pBuf2->BuContent[THOT_MAX_CHAR - 1 - diff+i];
-	  pBufn->BuLength = pBuf2->BuLength + diff - THOT_MAX_CHAR + 1;
+	  /* move the end of the buffer into the new buffer */
+	  ustrncpy (&pBufn->BuContent[0], &pBuf2->BuContent[ibuf2],
+		    pBuf2->BuLength - ibuf2);
+	  pBufn->BuLength = pBuf2->BuLength - ibuf2;
 	  pBufn->BuContent[pBufn->BuLength] = EOS;
 	  pBuf2->BuLength -= pBufn->BuLength;
 	  pBuf2->BuContent[pBuf2->BuLength] = EOS;
 	}
-      /* decale a droite les caracteres qui suivent la chaine */
-      /* a remplacer dans le buffer pBuf2 */
-      for (i = pBuf2->BuLength; i >= ibuf2; i--)
-	pBuf2->BuContent[i + diff] = pBuf2->BuContent[i];
+      else
+	{
+	  /* move rigth the text that follows the selection */
+	  for (i = pBuf2->BuLength - 1; i >= ibuf2; i--)
+	    pBuf2->BuContent[i + diff] = pBuf2->BuContent[i];
+	}
       pBuf2->BuLength += diff;
       diff = 0;
     }
@@ -164,26 +167,28 @@ void ReplaceString (PtrDocument pDoc, PtrElement pEl, int firstChar,
   else
     /* les deux chaines ont meme longueur */
     diff = 0;
-  /* copie la chaine de remplacement */
+
+  /* copy the remplacing string (dialogue is supposed ISO_8859_1) */
   len = 0;
   pBufn = pBuf1;
   i = ibuf1;
   while (len < replaceLen)
     {
-      pBufn->BuContent[i - 1] = replaceStr[len++];
+      pBufn->BuContent[i] = replaceStr[len++];
       i++;
-      if (i > pBufn->BuLength)
+      if (i >= pBufn->BuLength)
 	{
-	  pBufn->BuContent[i - 1] = EOS;
-	  pBufn->BuLength = i - 1;
+	  pBufn->BuContent[i] = EOS;
+	  pBufn->BuLength = i;
 	  pBufn = pBufn->BuNext;
-	  i = 1;
+	  i = 0;
 	}
     }
+
   if (diff > 0)
     {
-      pBufn->BuContent[i - 1] = EOS;
-      pBufn->BuLength = i - 1;
+      pBufn->BuContent[i] = EOS;
+      pBufn->BuLength = i;
     }
   /* met a jour le volume de l'element */
   dvol = replaceLen - stringLen;
@@ -191,7 +196,7 @@ void ReplaceString (PtrDocument pDoc, PtrElement pEl, int firstChar,
   pEl->ElVolume = pEl->ElTextLength;
   /* met a jour le volume de tous les elements ascendants */
   pAsc = pEl->ElParent;
-  while (pAsc != NULL)
+  while (pAsc)
     {
       pAsc->ElVolume = pAsc->ElVolume + dvol;
       pAsc = pAsc->ElParent;
@@ -223,10 +228,10 @@ void ReplaceString (PtrDocument pDoc, PtrElement pEl, int firstChar,
     {
       /* Update Abstract views */
       if (ThotLocalActions[T_AIupdate] != NULL)
-	(*ThotLocalActions[T_AIupdate]) (pDoc);
+	(*(Proc1)ThotLocalActions[T_AIupdate]) ((void *)pDoc);
       /* Redisplay views */
       if (ThotLocalActions[T_redisplay] != NULL)
-	(*ThotLocalActions[T_redisplay]) (pDoc);
+	(*(Proc1)ThotLocalActions[T_redisplay]) ((void *)pDoc);
     }
 
   /* si l'element TEXTE modifie' appartient soit a un element copie' */
@@ -240,7 +245,7 @@ void ReplaceString (PtrDocument pDoc, PtrElement pEl, int firstChar,
       if (replaceLen < 2)
 	i = firstChar;
       else
-	i = firstChar + replaceLen - 1;
+	i = firstChar + replaceLen;
       SelectStringWithEvent (pDoc, pEl, firstChar, i);
     }
   /* le document a ete modifie' */
@@ -265,12 +270,16 @@ void ReplaceString (PtrDocument pDoc, PtrElement pEl, int firstChar,
   ----------------------------------------------------------------------*/
 static ThotBool EquivalentChar (CHAR_T c1, CHAR_T c2, ThotBool caseEquiv)
 {
+  int                 v1, v2;
+
   if (caseEquiv)
     {
-      if ((c1 >= 'A' && c1 <= 'Z') || (c1 >= '\300' && c1 <= '\336'))
-	c1 = (CHAR_T) ((int) (c1) + 32);
-      if ((c2 >= 'A' && c2 <= 'Z') || (c2 >= '\300' && c2 <= '\336'))
-	c2 = (CHAR_T) ((int) (c2) + 32);
+      v1 = (int) c1;
+      v2 = (int) c2;
+      if ((v1 >= 65 && v1 <= 90) || (v1 >= 192 && v1 <= 294))
+	c1 = (CHAR_T) (v1 + 32);
+      if ((v2 >= 65 && v2 <= 90) || (c2 >= 192 && c2 <= 294))
+	c2 = (CHAR_T) (v2 + 32);
     }
   return (c1 == c2);
 }
@@ -281,7 +290,7 @@ static ThotBool EquivalentChar (CHAR_T c1, CHAR_T c2, ThotBool caseEquiv)
    indiquee par (pBuf,ind).                                
   ----------------------------------------------------------------------*/
 static ThotBool SameString (PtrTextBuffer pBuf, int ind, ThotBool caseEquiv,
-			    CHAR_T strng[THOT_MAX_CHAR], int strngLen)
+			    CHAR_T *strng, int strngLen)
 {
   ThotBool            equal, stop;
   int                 ind2;
@@ -320,7 +329,7 @@ static ThotBool SameString (PtrTextBuffer pBuf, int ind, ThotBool caseEquiv,
 ThotBool ContentAndStringEqual (PtrElement firstEl, int firstChar,
 				PtrElement lastEl, int lastChar,
 				ThotBool caseEquiv,
-				CHAR_T strng[THOT_MAX_CHAR], int strngLen)
+				CHAR_T *strng, int strngLen)
 {
   PtrTextBuffer       pBuf;
   ThotBool            ok;
@@ -348,9 +357,9 @@ ThotBool ContentAndStringEqual (PtrElement firstEl, int firstChar,
    texte du a des regles de presentation specifiques ou des        
    attributs differents.                                           
   ----------------------------------------------------------------------*/
-static void FwdSearchString (PtrTextBuffer pBuf, int ind, ThotBool * found,
+static void FwdSearchString (PtrTextBuffer pBuf, int ind, ThotBool *found,
 			     int *firstChar, ThotBool caseEquiv,
-			     CHAR_T strng[THOT_MAX_CHAR])
+			     CHAR_T *strng)
 {
   int                 ir;
   int                 ix;
@@ -359,66 +368,52 @@ static void FwdSearchString (PtrTextBuffer pBuf, int ind, ThotBool * found,
   ThotBool            stop;
 
   stop = FALSE;
-  ix = 0;
-  ir = 1;
+  *found = FALSE;
+  ix = -1;
+  ir = 0;
   icx = 0;
   pBufx = NULL;
   /* index dans ChaineCherchee du caractere a comparer */
   do
     {
-      if (pBuf->BuContent[ind - 1] == EOS)
+      if (pBuf->BuContent[ind] == EOS || ind >= pBuf->BuLength)
 	{
 	  pBuf = pBuf->BuNext;
-	  if (pBuf != NULL)
-	    /* saute les buffers vides */
-	    {
-	      ind = 0;
-	      do
-		if (pBuf->BuLength > 0)
-		  /* buffer non vide */
-		  ind = 1;
-		else
-		  /* buffer vide */
-		  {
-		    /* buffer suivant */
-		    pBuf = pBuf->BuNext;
-		    /* fin si dernier buffer */
-		    stop = pBuf == NULL;
-		  }
-	      while (!stop && ind != 1);
-	    }
-	  else
-	    stop = TRUE;
+	  ind = 0;
+	  while (pBuf && pBuf->BuLength == 0)
+	    /* skip the empty buffer */
+	    pBuf = pBuf->BuNext;
+	  stop = (pBuf == NULL);
 	}
       if (!stop)
 	{
-	  if (EquivalentChar (pBuf->BuContent[ind - 1], strng[ir - 1],
-			      caseEquiv))
+	  if (EquivalentChar (pBuf->BuContent[ind], strng[ir], caseEquiv))
 	    {
-	      if (ir == 1)
+	      if (ir == 0)
 		{
+		  /* remember the current position in buffers */
 		  ix = ind;
 		  pBufx = pBuf;
 		  icx = *firstChar;
 		}
+	      /* check if the end of the string is reached */
+	      ir++;
 	      if (strng[ir] == EOS)
 		{
 		  *found = TRUE;
 		  stop = TRUE;
 		}
 	      else
-		{
-		  ind++;
-		  ir++;
-		}
+		ind++;
 	    }
-	  else if (ix != 0)
+	  else if (pBufx)
 	    {
+	      /* restart the search */
 	      ind = ix + 1;
 	      pBuf = pBufx;
 	      *firstChar = icx + 1;
-	      ix = 0;
-	      ir = 1;
+	      pBufx = NULL;
+	      ir = 0;
 	    }
 	  else
 	    {
@@ -433,9 +428,9 @@ static void FwdSearchString (PtrTextBuffer pBuf, int ind, ThotBool * found,
 /*----------------------------------------------------------------------
    BackSearchString recherche une chaine en arriere.               
   ----------------------------------------------------------------------*/
-static void BackSearchString (PtrTextBuffer pBuf, int ind, ThotBool * found,
+static void BackSearchString (PtrTextBuffer pBuf, int ind, ThotBool *found,
 			      int *firstChar, ThotBool caseEquiv,
-			      CHAR_T strng[THOT_MAX_CHAR], int strngLen)
+			      CHAR_T *strng, int strngLen)
 {
   int                 ir;
   int                 ix;
@@ -444,33 +439,38 @@ static void BackSearchString (PtrTextBuffer pBuf, int ind, ThotBool * found,
   ThotBool            stop;
 
   stop = FALSE;
-  ix = 0;
+  *found = FALSE;
+  ix = -1;
+  strngLen--;
   ir = strngLen;
   icx = 0;
   pBufx = NULL;
   /* index dans ChaineCherchee du caractere a comparer */
   do
     {
-      if (ind < 1)
+      if (ind < 0)
 	{
 	  pBuf = pBuf->BuPrevious;
+	  while (pBuf && pBuf->BuLength == 0)
+	    /* skip the empty buffer */
+	    pBuf = pBuf->BuPrevious;
+	  stop = (pBuf == NULL);
 	  if (pBuf != NULL)
-	    ind = pBuf->BuLength;
-	  else
-	    stop = TRUE;
+	    ind = pBuf->BuLength - 1;
 	}
-      if (!stop && ind > 0)
+      if (!stop)
 	{
-	  if (EquivalentChar (pBuf->BuContent[ind - 1], strng[ir - 1],
-			      caseEquiv))
+	  if (EquivalentChar (pBuf->BuContent[ind], strng[ir], caseEquiv))
 	    {
 	      if (ir == strngLen)
 		{
+		  /* remember the current position in buffers */
 		  ix = ind;
 		  pBufx = pBuf;
 		  icx = *firstChar;
 		}
-	      if (ir == 1)
+	      /* check if the beginning of the string is reached */
+	      if (ir == 0)
 		{
 		  *found = TRUE;
 		  stop = TRUE;
@@ -481,12 +481,13 @@ static void BackSearchString (PtrTextBuffer pBuf, int ind, ThotBool * found,
 		  ir--;
 		}
 	    }
-	  else if (ix != 0)
+	  else if (pBufx)
 	    {
+	      /* restart the search */
 	      ind = ix - 1;
 	      pBuf = pBufx;
 	      *firstChar = icx - 1;
-	      ix = 0;
+	      pBufx = NULL;
 	      ir = strngLen;
 	    }
 	  else
@@ -500,29 +501,26 @@ static void BackSearchString (PtrTextBuffer pBuf, int ind, ThotBool * found,
 }
 
 /*----------------------------------------------------------------------
-   SearchText recherche dans le document pDoc la chaine de		
-   caracteres decrite par les variables strng et strngLen.		
-   A l'appel, les variables firstEl, firstChar, lastEl,            
-   lastChar indiquent la region ou on cherche.                     
-   Au retour, elles indiquent la chaine trouvee si la fonction     
-   retourne Vrai. (La fonction retourne faux en cas d'echec).      
+  SearchText looks for the string described by text and textLen.
+  Parameters firstEl, firstChar, lastEl, lastChar give the search context.
+  If the string is found the function returns TRUE and points that string.
   ----------------------------------------------------------------------*/
-ThotBool SearchText (PtrDocument pDoc, PtrElement * firstEl, int *firstChar,
-		     PtrElement * lastEl, int *lastChar, ThotBool forward,
-		     ThotBool caseEquiv, CHAR_T strng[THOT_MAX_CHAR],
-		     int strngLen)
+ThotBool SearchText (PtrDocument pDoc, PtrElement *firstEl, int *firstChar,
+		     PtrElement *lastEl, int *lastChar, ThotBool forward,
+		     ThotBool caseEquiv, CHAR_T *text,
+		     int textLen)
 {
   PtrElement          pEl;
+  PtrElement          pAncest;
+  PtrTextBuffer       pBuf;
   int                 i;
   int                 ibuf;
   int                 ichar;
   ThotBool            found;
-  PtrTextBuffer       pBuf;
   ThotBool            result;
-  PtrElement          pAncest;
 
   result = FALSE;
-  if (*firstEl != NULL)
+  if (*firstEl)
     {
       /* on n'a pas encore trouve' la chaine cherchee */
       found = FALSE;
@@ -550,13 +548,13 @@ ThotBool SearchText (PtrDocument pDoc, PtrElement * firstEl, int *firstChar,
 		  }
 		/* index dans le buffer de texte pBuf du 1er caractere
 		   a tester */
-		ibuf = *firstChar - i;
+		ibuf = *firstChar - 1 - i;
 		ichar = *firstChar;
-		FwdSearchString (pBuf, ibuf, &found, &ichar, caseEquiv, strng);
+		FwdSearchString (pBuf, ibuf, &found, &ichar, caseEquiv, text);
 	      }
-	  while (!found && pEl != NULL)
+	  while (!found && pEl)
 	    {
-	      pEl = FwdSearchTypedElem (pEl, CharString + 1, NULL);
+	      pEl = FwdSearchTypedElem (pEl, CharString + 1, NULL, NULL);
 	      if (pEl != NULL)
 		/* on a trouve un element de texte */
 		{
@@ -572,8 +570,8 @@ ThotBool SearchText (PtrDocument pDoc, PtrElement * firstEl, int *firstChar,
 		      /* l'element n'est pas dans une partie cachee */
 		      {
 			ichar = 1;
-			FwdSearchString (pEl->ElText, 1, &found, &ichar,
-					 caseEquiv, strng);
+			FwdSearchString (pEl->ElText, 0, &found, &ichar,
+					 caseEquiv, text);
 		      }
 		}
 	    }
@@ -596,14 +594,14 @@ ThotBool SearchText (PtrDocument pDoc, PtrElement * firstEl, int *firstChar,
 		ibuf = *firstChar - i - 1;
 		/* index dans le buffer de texte */
 		/* pointe par pBuf du 1er caractere a tester */
-		ichar = *firstChar - 1;
+		ichar = *firstChar;
 		BackSearchString (pBuf, ibuf, &found, &ichar, caseEquiv,
-				  strng, strngLen);
+				  text, textLen);
 	      }
 	  while (!found && pEl != NULL)
 	    {
-	      pEl = BackSearchTypedElem (pEl, CharString + 1, NULL);
-	      if (pEl != NULL)
+	      pEl = BackSearchTypedElem (pEl, CharString + 1, NULL, NULL);
+	      if (pEl)
 		/* on a trouve' un element de texte */
 		{
 		  /* on verifie que cet element ne fait pas partie d'une */
@@ -621,13 +619,13 @@ ThotBool SearchText (PtrDocument pDoc, PtrElement * firstEl, int *firstChar,
 			while (pBuf->BuNext != NULL)
 			  pBuf = pBuf->BuNext;
 			ichar = pEl->ElTextLength;
-			BackSearchString (pBuf, pBuf->BuLength, &found, &ichar,
-					  caseEquiv, strng, strngLen);
+			BackSearchString (pBuf, pBuf->BuLength - 1, &found, &ichar,
+					  caseEquiv, text, textLen);
 		      }
 		}
 	    }
 	  if (found)
-	    ichar = ichar - strngLen + 1;
+	    ichar = ichar - textLen + 1;
 	}
       if (found)
 	/* on a trouve' la chaine cherchee */
@@ -641,7 +639,7 @@ ThotBool SearchText (PtrDocument pDoc, PtrElement * firstEl, int *firstChar,
 	      {
 		if (forward)
 		  {
-		    if (ichar + strngLen - 1 > *lastChar)
+		    if (ichar + textLen - 1 > *lastChar)
 		      /* la chaine trouvee se termine au-dela du caractere ou
 		         il faut s'arreter, on fait comme si on n'avait pas
 			 trouve' */
@@ -671,13 +669,55 @@ ThotBool SearchText (PtrDocument pDoc, PtrElement * firstEl, int *firstChar,
 	  *firstChar = ichar;
 	  /* *lastChar est le rang du caractere qui suit le dernier */
 	  /* caractere de la chaine trouvee dans le buffer */
-	  *lastChar = ichar + strngLen;
+	  *lastChar = ichar + textLen;
 	  result = TRUE;
 	}
     }
   return result;
 }
 
+/*----------------------------------------------------------------------
+  TtaSearchText looks for the text forward or backward in the document.
+  If the string is found the function returns the element that includes
+  it. It return NULL if the string is not found.
+  ----------------------------------------------------------------------*/
+Element TtaSearchText (Document document, Element element, ThotBool forward,
+		       char *text, CHARSET encoding)
+{
+  PtrDocument         pDoc = NULL;
+  PtrElement          elFound;
+  PtrElement          pfirstEl, plastEl;
+  int                 firstChar = 0, lastChar = 0;
+  ThotBool            found;
+  CHAR_T             *ptr;
+
+   UserErrorCode = 0;
+   /* Checks the parameter document */
+   elFound = NULL;
+   if (document < 1 || document > MAX_DOCUMENTS)
+     TtaError (ERR_invalid_document_parameter);
+   else if (LoadedDocument[document - 1] == NULL)
+     TtaError (ERR_invalid_document_parameter);
+   else if (element == NULL)
+     TtaError (ERR_invalid_parameter);
+   else if (text == NULL)
+     TtaError (ERR_invalid_parameter);
+   else
+     {
+       pfirstEl = (PtrElement) element;
+       plastEl = NULL;
+       ptr = (CHAR_T *)TtaConvertByteToWC ((unsigned char *)text, encoding);
+       found = SearchText (pDoc, &pfirstEl, &firstChar, &plastEl,
+			   &lastChar, forward, TRUE, ptr, ustrlen (ptr));
+       if (ptr)
+	 TtaFreeMemory (ptr);
+       if (found)
+	 elFound = pfirstEl;
+     }
+  return ((Element) elFound);
+}
+
+
 /*----------------------------------------------------------------------
    SearchPageBreak recherche a partir de l'element pEl un element
    	BreakPage concernant la vue view. Si relative est faux, pageNum
@@ -703,7 +743,7 @@ PtrElement SearchPageBreak (PtrElement pEl, int view, int pageNum,
       pElP = pEl;
       while (pageNum > 0 && pElP != NULL)
 	{
-	  pElP = FwdSearchTypedElem (pEl, PageBreak + 1, NULL);
+	  pElP = FwdSearchTypedElem (pEl, PageBreak + 1, NULL, NULL);
 	  if (pElP != NULL)
 	    {
 	      if (pElP->ElViewPSchema == view)
@@ -728,7 +768,7 @@ PtrElement SearchPageBreak (PtrElement pEl, int view, int pageNum,
   else
     while (pageNum < 0 && pElP != NULL)
       {
-	pElP = BackSearchTypedElem (pEl, PageBreak + 1, NULL);
+	pElP = BackSearchTypedElem (pEl, PageBreak + 1, NULL, NULL);
 	if (pElP != NULL)
 	  {
 	    if (pElP->ElViewPSchema == view)
