@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -11,7 +11,7 @@
  * Author: V. Quint (INRIA)
  *
  */
-
+#include "thot_gui.h"
 #include "thot_sys.h"
 #include "libmsg.h"
 
@@ -288,8 +288,9 @@ void                FindReferredEl ()
 	     {
 		pRefTable[nMenuItems] = firstSel->ElReference;
 		nMenuItems++;
-		ustrcpy (menuBuf + menuBufLen, firstSel->ElStructSchema->SsRule[firstSel->ElTypeNumber - 1].SrName);
-		menuBufLen += ustrlen (menuBuf) + 1;
+		strcpy (menuBuf + menuBufLen,
+			firstSel->ElStructSchema->SsRule->SrElem[firstSel->ElTypeNumber - 1]->SrName);
+		menuBufLen += strlen (menuBuf) + 1;
 	     }
 	if (firstSel->ElSource != NULL)
 	   /* la selection est un element inclus */
@@ -311,8 +312,9 @@ void                FindReferredEl ()
 		     {
 			pRefTable[nMenuItems] = pAttr->AeAttrReference;
 			nMenuItems++;
-			ustrcpy (menuBuf + menuBufLen, pAttr->AeAttrSSchema->SsAttribute[pAttr->AeAttrNum - 1].AttrName);
-			menuBufLen += ustrlen (pAttr->AeAttrSSchema->SsAttribute[pAttr->AeAttrNum - 1].AttrName) + 1;
+			strcpy (menuBuf + menuBufLen,
+				pAttr->AeAttrSSchema->SsAttribute->TtAttr[pAttr->AeAttrNum - 1]->AttrName);
+			menuBufLen += strlen (pAttr->AeAttrSSchema->SsAttribute->TtAttr[pAttr->AeAttrNum - 1]->AttrName) + 1;
 		     }
 	     pAttr = pAttr->AeNext;
 	  }
@@ -381,7 +383,7 @@ void                CheckReferences (PtrElement pRoot, PtrDocument pDoc)
    PtrReference        pRef, pNextRef;
    PtrAttribute        pAttr, pNextAttr;
    PtrDocument         pDocRef, pNextDocRef;
-   ThotBool            attrRef, delAttr;
+   ThotBool            delAttr;
    PtrExternalDoc      pExtDoc;
    PtrElement          pElemRef;
    int                 l;
@@ -462,13 +464,6 @@ void                CheckReferences (PtrElement pRoot, PtrDocument pDoc)
 		     pElemRef = pRef->RdElement;
 		  SetReference (pElemRef, pRef->RdAttribute, pRoot, pDocRef,
 				pDoc, FALSE, FALSE);
-		  /* si c'est une reference par attribut, verifie la */
-		  /* validite de l'attribut dans le cas des extensions de */
-		  /* cellule des tableaux */
-		  if (pRef->RdAttribute != NULL)
-		     if (ThotLocalActions[T_checkextens] != NULL)
-			(*ThotLocalActions[T_checkextens])
-			   (pRef->RdAttribute, pRef->RdElement, pRef->RdElement, TRUE);
 	       }
 	     pRef = pNextRef;	/* passe a la reference suivante */
 	     pDocRef = pNextDocRef;
@@ -497,26 +492,14 @@ void                CheckReferences (PtrElement pRoot, PtrDocument pDoc)
 		       if (!IsASavedElement (pElRef))
 			 if (!IsWithinANewElement (pElRef))
 			   {
-			     /* verifie la validite de l'attribut dans le cas des */
-			     /* extensions de cellule des tableaux */
-			     if (ThotLocalActions[T_checkextens] != NULL)
-			       (*ThotLocalActions[T_checkextens])
-				 (pAttr, pRoot, pRoot, TRUE);
 			     if (DocOfSavedElements != pDoc)
 			       /* reference et objet reference' sont */
 			       /* dans des documents differents, on */
 			       /* supprime l'attribut, sauf dans le */
 			       /* cas particulier des tableaux. */
 			       {
-				 attrRef = TRUE;
-				 if (ThotLocalActions[T_refattr] != NULL)
-				   (*ThotLocalActions[T_refattr])
-				     (pAttr, &attrRef);
-				 if (!attrRef)
-				   {
-				   RemoveAttribute (pRoot, pAttr);
-				   delAttr = TRUE;
-				   }
+				 RemoveAttribute (pRoot, pAttr);
+				 delAttr = TRUE;
 			       }
 			   }
 		     if (delAttr)
