@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -12,11 +12,15 @@
  *
  */
 
+#include "thot_gui.h"
 #include "thot_sys.h"
 #include "constmedia.h"
 #include "typemedia.h"
 #include "language.h"
 
+#define THOT_EXPORT extern
+#include "select_tv.h"
+
 static PtrDocument  DocSDomain;
 static PtrElement   FirstElSDomain;
 static PtrElement   LastElSDomain;
@@ -33,10 +37,10 @@ static int          LastCharSDomain;
    S'il n'y a pas de caractere suivant retourne le         
    caractere NULL.                                         
   ----------------------------------------------------------------------*/
-UCHAR_T NextCharacter (PtrTextBuffer *buffer, int *rank)
+CHAR_T NextCharacter (PtrTextBuffer *buffer, int *rank)
 {
   if (*buffer == NULL)
-    return (WC_EOS);
+    return EOS;
   else
     {
       (*rank)++;
@@ -46,12 +50,12 @@ UCHAR_T NextCharacter (PtrTextBuffer *buffer, int *rank)
 	  *rank = 0;
 	  *buffer = (*buffer)->BuNext;
 	  if (*buffer == NULL)
-	    return (0);
+	    return EOS;
 	  else
 	    return ((*buffer)->BuContent[*rank]);
 	}
       else
-	return ((*buffer)->BuContent[*rank]);
+	return (*buffer)->BuContent[*rank];
     }
 }
 
@@ -61,10 +65,10 @@ UCHAR_T NextCharacter (PtrTextBuffer *buffer, int *rank)
    S'il n'y a pas de caractere precedent retourne le       
    caractere `\0`.                                         
   ----------------------------------------------------------------------*/
-UCHAR_T PreviousCharacter (PtrTextBuffer *buffer, int *rank)
+CHAR_T PreviousCharacter (PtrTextBuffer *buffer, int *rank)
 {
   if (*buffer == NULL)
-    return (WC_EOS);
+    return EOS;
   else
     {
       (*rank)--;
@@ -74,15 +78,15 @@ UCHAR_T PreviousCharacter (PtrTextBuffer *buffer, int *rank)
 	  *rank = 0;
 	  *buffer = (*buffer)->BuPrevious;
 	  if (*buffer == NULL)
-	    return (0);
+	    return EOS;
 	  else
 	    {
 	      *rank = (*buffer)->BuLength - 1;
-	      return ((*buffer)->BuContent[*rank]);
+	      return (*buffer)->BuContent[*rank];
 	    }
 	}
       else
-	return ((*buffer)->BuContent[*rank]);
+	return (*buffer)->BuContent[*rank];
     }
 }
 
@@ -113,6 +117,14 @@ ThotBool InitSearchDomain (int domain, PtrSearchContext context)
   ok = GetCurrentSelection (&DocSDomain,
 			    &FirstElSDomain, &LastElSDomain,
 			    &FirstCharSDomain, &LastCharSDomain);
+  if (LastElSDomain && LastCharSDomain == 0 &&
+      LastElSDomain == FirstElSDomain && FirstCharSDomain == 0)
+    /* the whole element is selected */
+    FirstCharSDomain = LastElSDomain->ElVolume;
+  if (FirstElSDomain && FirstCharSDomain == 0)
+    /* the whole element is selected */
+    FirstCharSDomain = 1;
+
   /* store the word search domain */
   if (domain == 3)
     /* the whole document */
@@ -121,8 +133,8 @@ ThotBool InitSearchDomain (int domain, PtrSearchContext context)
       context->STree = 0;
       context->SWholeDocument = TRUE;
       context->SEndElement = NULL;
-      context->SStartChar = 0;
-      context->SEndChar = 0;
+      context->SStartChar = 1;
+      context->SEndChar = 1;
       context->SStartToEnd = TRUE;
     }
   else if (!ok)
@@ -138,7 +150,7 @@ ThotBool InitSearchDomain (int domain, PtrSearchContext context)
 	/* Search before selection */
 	{
 	  context->SStartElement = NULL;
-	  context->SStartChar = 0;
+	  context->SStartChar = 1;
 	  context->SEndElement = FirstElSDomain;
 	  context->SEndChar = FirstCharSDomain;
 	  context->SStartToEnd = FALSE;
@@ -147,27 +159,27 @@ ThotBool InitSearchDomain (int domain, PtrSearchContext context)
 	/* search within selection */
 	{
 	  context->SStartElement = FirstElSDomain;
-	  context->SStartChar = FirstCharSDomain - 1;
+	  context->SStartChar = FirstCharSDomain;
 	  context->SEndElement = LastElSDomain;
-	  context->SEndChar = LastCharSDomain - 1;
+	  context->SEndChar = LastCharSDomain;
 	  context->SStartToEnd = TRUE;
 	}
       else if (domain == 2)
 	/* search after selection */
 	{
 	  context->SStartElement = LastElSDomain;
-	  context->SStartChar = LastCharSDomain - 1;
-	  if (context->SStartChar < 0)
-	    context->SStartChar = 0;
+	  context->SStartChar = LastCharSDomain;
+	  if (context->SStartChar < 1)
+	    context->SStartChar = 1;
 	  if (LastElSDomain && !LastElSDomain->ElTerminal &&
 	      (LastCharSDomain == 0 ||
 	       LastCharSDomain > LastElSDomain->ElTextLength))
 	    {
 	      context->SStartElement = NextElement (LastElSDomain);
-	      context->SStartChar = 0;
+	      context->SStartChar = 1;
 	    }
 	  context->SEndElement = NULL;
-	  context->SEndChar = 0;
+	  context->SEndChar = 1;
 	  context->SStartToEnd = TRUE;
 	}
     }
@@ -177,8 +189,8 @@ ThotBool InitSearchDomain (int domain, PtrSearchContext context)
       /* no search domain */
       context->SStartElement = NULL;
       context->SEndElement = NULL;
-      context->SStartChar = 0;
-      context->SEndChar = 0;
+      context->SStartChar = 1;
+      context->SEndChar = 1;
       context->STree = 0;
       context->SWholeDocument = FALSE;
       return (FALSE);
@@ -234,12 +246,12 @@ void RestoreAfterSearch ()
   The parameter context points to the search domain.
   ----------------------------------------------------------------------*/
 ThotBool SearchNextWord (PtrElement *curEl, int *beginning, int *end,
-			 CHAR_T word[MAX_WORD_LEN], PtrSearchContext context)
+			 CHAR_T *word, PtrSearchContext context)
 {
   PtrElement          pEl, endEl;
   PtrElement          pAncestor;
   PtrTextBuffer       pBuf;
-  UCHAR_T             charact;
+  CHAR_T             charact;
   int                 iChar, endChar;
   int                 len;
   int                 index;
@@ -247,27 +259,36 @@ ThotBool SearchNextWord (PtrElement *curEl, int *beginning, int *end,
   pEl = *curEl;
   iChar = *end;
   *beginning = iChar;
-  word[0] = WC_EOS;
+  word[0] = EOS;
 
-  if (pEl == NULL && context != NULL)
+  if (pEl == NULL && context)
     {
       /* search start */
       pEl = context->SStartElement;
-      iChar = context->SStartChar;
-      if (pEl == NULL && context->SDocument != NULL)
+      if (!AbsBoxSelectedAttr)
+	/* don't change this value when an attribute value is selected */
+	iChar = context->SStartChar;
+      else if (iChar == 0)
+	iChar = 1;
+      if (pEl == NULL && context->SDocument)
 	pEl = context->SDocument->DocDocElement;
     }
 
   /* Check that element is a Text element and that the research does */
   /* not start on the last character */
-  if (pEl != NULL)
+  if (pEl)
     {
-      if (pEl->ElTypeNumber != CharString + 1 ||
-	  iChar + 1 >= pEl->ElTextLength)
+      if (pEl->ElTypeNumber != CharString + 1 || iChar >= pEl->ElTextLength)
 	{
-	  /* there is no buffer */
-	  pBuf = NULL;
-	  iChar = 0;
+	  if (AbsBoxSelectedAttr)
+	    /* selection within an attribute */
+	    pBuf = AbsBoxSelectedAttr->AbText;
+	  else
+	    {
+	      /* there is no buffer */
+	      pBuf = NULL;
+	      iChar = 1;
+	    }
 	}
       else
 	/* first buffer of element */
@@ -278,7 +299,7 @@ ThotBool SearchNextWord (PtrElement *curEl, int *beginning, int *end,
   if (context == NULL)
     {
       endEl = NULL;
-      endChar = 0;
+      endChar = 1;
     }
   else
     {
@@ -286,24 +307,24 @@ ThotBool SearchNextWord (PtrElement *curEl, int *beginning, int *end,
       endChar = context->SEndChar;
     }
 
-  while (pBuf == NULL && pEl != NULL)
+  while (pBuf == NULL && pEl)
     {
       /* Search the first non-empty and non-protected element */
-      pEl = FwdSearchTypedElem (pEl, CharString + 1, NULL);
-      iChar = 0;
-      if (pEl != NULL)
+      pEl = FwdSearchTypedElem (pEl, CharString + 1, NULL, NULL);
+      iChar = 1;
+      if (pEl)
 	{
-	  if (endEl != NULL && ElemIsBefore (endEl, pEl))
+	  if (endEl && ElemIsBefore (endEl, pEl))
 	    /* the element found is after the end of the domain */
 	    pEl = NULL;
 	}
 
-      if (pEl != NULL)
+      if (pEl)
 	/* on verifie que cet element ne fait pas partie d'une inclusion et */
 	/* n'est pas cache' a l'utilisateur */
 	{
 	  pAncestor = pEl;
-	  while (pAncestor->ElParent != NULL && pAncestor->ElSource == NULL)
+	  while (pAncestor->ElParent && pAncestor->ElSource == NULL)
 	    pAncestor = pAncestor->ElParent;
 	  if (pAncestor->ElSource == NULL && !ElementIsHidden (pEl) &&
 	      pEl->ElTextLength != 0)
@@ -313,7 +334,7 @@ ThotBool SearchNextWord (PtrElement *curEl, int *beginning, int *end,
 	}
     }
 
-  if (pEl == NULL)
+  if (pEl == NULL || iChar < 0)
     return (FALSE);
   else if (pEl == endEl && iChar >= endChar)
     /* On est arrive a la fin du domaine de recherche */
@@ -321,17 +342,23 @@ ThotBool SearchNextWord (PtrElement *curEl, int *beginning, int *end,
   else
     {
       /* Calcule l'index de depart de la recherche */
-      len = iChar;
+      if (iChar > 0)
+	len = iChar - 1;
+      else
+	len = iChar;
       while (len >= pBuf->BuLength)
 	{
 	  /* On passe au buffer suivant */
 	  len -= pBuf->BuLength;
-	  pBuf = pBuf->BuNext;
+	  if (pBuf->BuNext == NULL)
+	    len = pBuf->BuLength - 1;
+	  else
+	    pBuf = pBuf->BuNext;
 	}
       index = len;
       charact = pBuf->BuContent[index];
       /* On se place au debut du mot */
-      while (charact != WC_EOS && IsSeparatorChar (charact) &&
+      while (charact != EOS && IsSeparatorChar (charact) &&
 	     (pEl != endEl || iChar < endChar))
 	{
 	  charact = NextCharacter (&pBuf, &index);
@@ -342,7 +369,7 @@ ThotBool SearchNextWord (PtrElement *curEl, int *beginning, int *end,
       /* On verifie que l'on ne depasse pas la fin du domaine de recherche */
       len = 0;
       *beginning = iChar;
-      while (len < MAX_WORD_LEN-1 && charact != WC_EOS &&
+      while (len < MAX_WORD_LEN-1 && charact != EOS &&
 	     !IsSeparatorChar (charact) &&
 	     (pEl != endEl || iChar < endChar))
 	{
@@ -352,7 +379,7 @@ ThotBool SearchNextWord (PtrElement *curEl, int *beginning, int *end,
 	}
 
       /* positionne les valeurs de retour */
-      word[len] = WC_EOS;
+      word[len] = EOS;
       *curEl = pEl;
       *end = iChar;
       /* Si on a trouve effectivement un mot */
@@ -373,12 +400,12 @@ ThotBool SearchNextWord (PtrElement *curEl, int *beginning, int *end,
   The parameter context points to the search domain.
   ----------------------------------------------------------------------*/
 ThotBool SearchPreviousWord (PtrElement *curEl, int *beginning, int *end,
-			     CHAR_T word[MAX_WORD_LEN], PtrSearchContext context)
+			     CHAR_T *word, PtrSearchContext context)
 {
   PtrElement          pEl, endEl;
   PtrElement          pAncestor;
   PtrTextBuffer       pBuf;
-  UCHAR_T             charact;
+  CHAR_T             charact;
   CHAR_T              reverse[MAX_WORD_LEN];
   int                 iChar, endChar, j;
   int                 len;
@@ -387,35 +414,50 @@ ThotBool SearchPreviousWord (PtrElement *curEl, int *beginning, int *end,
   pEl = *curEl;
   iChar = *beginning;
   *end = iChar;
-  word[0] = WC_EOS;
-  if (pEl == NULL)
+  word[0] = EOS;
+  if (pEl == NULL && context)
     {
       /* C'est le debut de la recherche */
       pEl = context->SEndElement;
-      iChar = context->SEndChar;
-      if (iChar > 2)
-	/* La fin de selection pointe toujours sur un caratere plus loin */
-	iChar--;
+      if (!AbsBoxSelectedAttr)
+	{
+	  /* don't change this value when an attribute value is selected */
+	  iChar = context->SEndChar;
+	  if (iChar > 2)
+	    /* La fin de selection pointe toujours sur un caratere plus loin */
+	    iChar--;
+	}
+      if (pEl == NULL && context->SDocument)
+	pEl = context->SDocument->DocDocElement;
     }
   iChar--;
 
-  /* Verifie que l'element est de type texte */
-  /* et que la recherche ne debute pas sur le dernier caractere */
-  if (pEl->ElTypeNumber != CharString + 1 || iChar <= 0)
+  /* Check that element is a Text element and that the research does */
+  /* not start on the last character */
+  if (pEl)
     {
-      /* On n'a pas trouve de buffer */
-      pBuf = NULL;
-      iChar = 0;
+      if (pEl->ElTypeNumber != CharString + 1 || iChar < 1)
+	{
+	  if (AbsBoxSelectedAttr)
+	    /* selection within an attribute */
+	    pBuf = AbsBoxSelectedAttr->AbText;
+	  else
+	    {
+	      /* there is no buffer */
+	      pBuf = NULL;
+	      iChar = 1;
+	    }
+	}
+      else
+	/* first buffer of element */
+	pBuf = pEl->ElText;
     }
-  else
-    /* 1er Buffer de l'element */
-    pBuf = pEl->ElText;
 
   /* Determine l'element limite du contexte de recherche */
   if (context == NULL)
     {
       endEl = NULL;
-      endChar = 0;
+      endChar = 1;
     }
   else
     {
@@ -423,37 +465,37 @@ ThotBool SearchPreviousWord (PtrElement *curEl, int *beginning, int *end,
       endChar = context->SStartChar;
     }
 
-  while (pBuf == NULL && pEl != NULL)
+  while (pBuf == NULL && pEl)
     {
       /* Recherche le premier element texte non vide */
-      pEl = BackSearchTypedElem (pEl, CharString + 1, NULL);
-      if (pEl != NULL)
+      pEl = BackSearchTypedElem (pEl, CharString + 1, NULL, NULL);
+      if (pEl)
 	{
-	  if (endEl != NULL && ElemIsBefore (pEl, endEl))
+	  if (endEl && ElemIsBefore (pEl, endEl))
 	    /* l'element trouve' est avant l'element de debut, */
 	    /* on fait comme si on n'avait pas trouve' */
 	    pEl = NULL;
 	}
 
       /* On saute les elements vides */
-      if (pEl != NULL)
+      if (pEl)
 	/* on verifie que cet element ne fait pas partie d'une inclusion et */
 	/* n'est pas cache' a l'utilisateur */
 	{
 	  pAncestor = pEl;
-	  while (pAncestor->ElParent != NULL && pAncestor->ElSource == NULL)
+	  while (pAncestor->ElParent && pAncestor->ElSource == NULL)
 	    pAncestor = pAncestor->ElParent;
 	  if (pAncestor->ElSource == NULL && !ElementIsHidden (pEl) &&
 	      pEl->ElTextLength != 0)
 	    {
 	      /* on n'est pas dans une inclusion et l'element n'est pas cache' */
-	      iChar = pEl->ElTextLength - 1;
+	      iChar = pEl->ElTextLength;
 	      pBuf = pEl->ElText;
 	    }
 	}
     }
   
-  if (pEl == NULL)
+  if (pEl == NULL || iChar < 0)
     return (FALSE);
   else if (pEl == endEl && iChar < endChar)
     /* On est arrive a la fin du domaine de recherche */
@@ -461,18 +503,21 @@ ThotBool SearchPreviousWord (PtrElement *curEl, int *beginning, int *end,
   else
     {
       /* Calcule l'index de depart de la recherche */
-      len = iChar;
+      len = iChar - 1;
       while (len >= pBuf->BuLength)
 	{
 	  /* On passe au buffer suivant */
 	  len -= pBuf->BuLength;
-	  pBuf = pBuf->BuNext;
+	  if (pBuf->BuNext == NULL)
+	    len = pBuf->BuLength - 1;
+	  else
+	    pBuf = pBuf->BuNext;
 	}
 
       index = len;
       charact = pBuf->BuContent[index];
       /* On se place a la fin du mot */
-      while (charact != WC_EOS && iChar >= 0 &&
+      while (charact != EOS && iChar >= 0 &&
 	     IsSeparatorChar (charact) &&
 	     (pEl != endEl || iChar >= endChar))
 	{
@@ -482,7 +527,7 @@ ThotBool SearchPreviousWord (PtrElement *curEl, int *beginning, int *end,
 
       /* On se place au debut du mot et recupere le mot a l'envers */
       len = 0;
-      while (len < MAX_WORD_LEN-1 && charact != WC_EOS && iChar >= 0 &&
+      while (len < MAX_WORD_LEN-1 && charact != EOS && iChar >= 0 &&
 	     !IsSeparatorChar (charact) &&
 	     (pEl != endEl || iChar >= endChar))
 	{
@@ -501,7 +546,7 @@ ThotBool SearchPreviousWord (PtrElement *curEl, int *beginning, int *end,
 	}
 
       /* positionne les valeurs de retour */
-      word[len] = WC_EOS;
+      word[len] = EOS;
       *curEl = pEl;
       *beginning = iChar;
       *end = iChar + len;
