@@ -29,8 +29,8 @@ Thanks to Contributers:
 J Satchell, Eric Marttila 
 */
 
-#include "ustring.h"
-#ifndef _WINDOWS
+#include "string.h"
+#ifdef _MOTIF
 #include "thot_gui.h"
 #include "thot_sys.h"
 #include "LiteClueP.h"
@@ -53,7 +53,7 @@ struct liteClue_context_str
 	XcgLiteClueWidget cw;	/* pointer back to the liteClue widget */
 	Position  abs_x, abs_y;
 	Boolean sensitive;	/* if False, liteClue is suppressed */
-	STRING  text;		/* text to display */
+	char *text;		/* text to display */
 	short text_size;	/* its size */
 };
 
@@ -67,20 +67,20 @@ Widget resources: eg to set LiteClue box background:
 static XtResource LC_resources[] =
 {
 	{XtNforeground, XtCForeground, XtRPixel, sizeof(Pixel),
-		offset(liteClue.foreground), XtRString, "black"},
+		offset(liteClue.foreground), XtRString, (void *)"black"},
 #if XtSpecificationRelease < 5 || defined(NO_FONT_SET)
 	{XtNfont, XtCFont, XtRFontStruct, sizeof(XFontStruct *),
 		offset(liteClue.fontset), XtRString, 
-          "-adobe-new century schoolbook-bold-r-normal-*-14-*-*-*-*-*-*-*"},
+          (void *)"-adobe-new century schoolbook-bold-r-normal-*-14-*-*-*-*-*-*-*"},
 #else
 	{XtNfontSet, XtCFontSet, XtRFontSet, sizeof(XFontSet),
-		offset(liteClue.fontset), XtRString, "-adobe-new century schoolbook-bold-r-normal-*-12-*"},
+		offset(liteClue.fontset), XtRString, (void *)"-adobe-new century schoolbook-bold-r-normal-*-12-*"},
 #endif
 	{XgcNwaitPeriod, XgcCWaitPeriod, XtRInt , sizeof(int),
-		offset(liteClue.waitPeriod),XtRString, "500" },
+		offset(liteClue.waitPeriod),XtRString, (void *)"500" },
 
 	{XgcNcancelWaitPeriod, XgcCCancelWaitPeriod, XtRInt , sizeof(int),
-		offset(liteClue.cancelWaitPeriod),XtRString, "2000" },
+		offset(liteClue.cancelWaitPeriod),XtRString, (void *)"2000" },
 };
 
 #undef offset
@@ -206,7 +206,7 @@ The font_information is derived
 Return XFontSet for passed font_string. 
 return status
 */
-static int string_to_FontSet (XcgLiteClueWidget cw, STRING  font_string, XFontStruct ** out) 
+static int string_to_FontSet (XcgLiteClueWidget cw, char *font_string, XFontStruct ** out) 
 {
 	Boolean sts;
 	XrmValue from;
@@ -214,7 +214,7 @@ static int string_to_FontSet (XcgLiteClueWidget cw, STRING  font_string, XFontSt
 	
 	to.size = sizeof(out);
 	to.addr = (void *) out;
-	from.size = ustrlen(from.addr = font_string );
+	from.size = strlen(from.addr = font_string );
 	sts = XtConvertAndStore((Widget) cw, XtRString, &from, XtRFontStruct, &to);
 	return sts;
 }
@@ -246,15 +246,15 @@ static void compute_font_info(XcgLiteClueWidget cw)
 Return XFontSet for passed font_string. 
 return status
 */
-static int string_to_FontSet (XcgLiteClueWidget cw, STRING  font_string, XFontSet * out) 
+static int string_to_FontSet (XcgLiteClueWidget cw, char *font_string, XFontSet * out) 
 {
 	Boolean sts;
 	XrmValue from;
 	XrmValue to;
 	
 	to.size = sizeof(out);
-	to.addr = (void *) out;
-	from.size = ustrlen(from.addr = font_string );
+	to.addr = (char*) out;
+	from.size = strlen(from.addr = font_string );
 	sts = XtConvertAndStore((Widget) cw, XtRString, &from, XtRFontSet, &to);
 	return sts;
 }
@@ -291,7 +291,7 @@ static void create_GC(XcgLiteClueWidget cw )
 
 
 	valuemask = GCForeground | GCBackground | GCFillStyle;
-	myXGCV.foreground = cw->liteClue.foreground;
+	myXGCV.foreground = ColorPixel(ColorNumber("Black")) /*cw->liteClue.foreground */;
 	myXGCV.background = ColorPixel(ColorNumber("Yellow")) /* cw->core.background_pixel */;
 	myXGCV.fill_style = FillSolid; 
 
@@ -310,7 +310,7 @@ static void create_GC(XcgLiteClueWidget cw )
 a core dump for debugging analysis	
 when a public routine is called with the wrong class of widget
 */
-static void wrong_widget(STRING  routine)
+static void wrong_widget(char *routine)
 {
 	int mypid = getpid(); 
 	fprintf(stderr, "Wrong class of widget passed to %s\n", routine);
@@ -481,7 +481,7 @@ static void Enter_event(Widget w, XtPointer client_data, XEvent * xevent, Boolea
 	   help for next watched widget
 	*/
 	if ((event->time -  cw->liteClue.HelpPopDownTime) > 
-			cw->liteClue.cancelWaitPeriod ) 
+			(unsigned) cw->liteClue.cancelWaitPeriod ) 
 		current_waitPeriod = cw->liteClue.waitPeriod,timeout_event;
 	else
 		current_waitPeriod = 0;
@@ -548,7 +548,7 @@ Return:
 
 ;-
 */
-void XcgLiteClueAddWidget(Widget w, Widget watch,  STRING  text, int size, int option )
+void XcgLiteClueAddWidget(Widget w, Widget watch,  char *text, int size, int option )
 {
 #	define ROUTINE "XcgLiteClueAddWidget"
 	XcgLiteClueWidget cw = (XcgLiteClueWidget) w;
@@ -576,7 +576,7 @@ void XcgLiteClueAddWidget(Widget w, Widget watch,  STRING  text, int size, int o
 	if (text && !(obj->text))
 	{
 		if (!size)
-			size = ustrlen(text);
+			size = strlen(text);
 		obj->text = XtMalloc(size+1);
 		memcpy(obj->text, text, size);
 		obj->text[size] = 0;
@@ -801,4 +801,4 @@ Boolean XcgLiteClueDispatchEvent(Widget w, XEvent  *event)
 }
 
 
-#endif /* ! _WINDOWS */
+#endif /* ! _MOTIF */
