@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001.
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -11,16 +11,15 @@
  * Author: I. Vatton (INRIA)
  *
  */
-
+#include "thot_gui.h"
 #include "thot_sys.h"
 #include "constmedia.h"
 #include "typemedia.h"
-#include "modif.h"
 #include "appstruct.h"
 #include "appdialogue.h"
 
 #undef THOT_EXPORT
-#define THOT_EXPORT
+#define THOT_EXPORT extern
 #include "appevents_tv.h"
 #include "appdialogue_tv.h"
 
@@ -208,8 +207,9 @@ void TteAddActionEvent (PtrEventsSet eventsList, int typeId,
   ----------------------------------------------------------------------*/
 static void TteFreeEventsList (PtrEventsSet eventsList)
 {
-  PtrEventsSet current;
-  int event;
+  PtrEventsSet   current;
+  PtrActionEvent actevent, nextactevent;
+  int            event;
 
   current = eventsList;
   while (current)
@@ -217,7 +217,16 @@ static void TteFreeEventsList (PtrEventsSet eventsList)
    for (event = 0; event <= TteExit; event++)
    {
       if (current->EvSList[event])
-        TtaFreeMemory (current->EvSList[event]);
+	{
+	  /* free lists of couples action/event */
+	  actevent = current->EvSList[event];
+	  while (actevent)
+	    {
+	      nextactevent = actevent->AEvNext;
+	      TtaFreeMemory (actevent);
+	      actevent = nextactevent;
+	    }
+	}
   }
    eventsList = current->EvSNext;
    TtaFreeMemory (current);
