@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -40,9 +40,9 @@ static ThotBool     SaveBeforeClosing;
 #include "docs_f.h"
 #include "views_f.h"
 
-#ifdef _WINDOWS
-#include "wininclude.h"
-#endif /* _WINDOWS */
+#ifdef _WINGUI
+  #include "wininclude.h"
+#endif /* _WINGUI */
 
 /*----------------------------------------------------------------------
    CallbackCloseDocMenu 
@@ -50,7 +50,6 @@ static ThotBool     SaveBeforeClosing;
   ----------------------------------------------------------------------*/
 void CallbackCloseDocMenu (int ref, int typedata, char *data)
 {
-#ifndef _WINDOWS
   switch ((int) data)
     {
     case 0:
@@ -66,8 +65,9 @@ void CallbackCloseDocMenu (int ref, int typedata, char *data)
       CloseDontSave = FALSE;
       break;
     }
-  TtaDestroyDialogue (NumFormClose);
-#endif /* _WINDOWS */
+#if defined(_GTK) || defined(_MOTIF)
+   TtaDestroyDialogue (NumFormClose);
+#endif /* #if defined(_GTK) || defined(_MOTIF) */
 }
 
 /*----------------------------------------------------------------------
@@ -79,10 +79,10 @@ void AskToConfirm (PtrDocument pDoc, Document document, View view,
 		   ThotBool *confirmation, ThotBool *save)
 {
   char              buftext[300];
-#ifndef _WINDOWS
+#if defined(_GTK) || defined(_MOTIF)
   char              bufbutton[300];
   int                 i;
-#endif /* _WINDOWS */
+#endif /* #if defined(_GTK) || defined(_MOTIF) */
 
   CloseDontSave = TRUE;
   SaveBeforeClosing = FALSE;
@@ -95,7 +95,7 @@ void AskToConfirm (PtrDocument pDoc, Document document, View view,
   strcat (buftext, " ");
   strcat (buftext, TtaGetMessage (LIB, TMSG_BEFORE_CLOSING));
 
-#ifndef _WINDOWS
+#if defined(_GTK) || defined(_MOTIF)
   /* Feuille de dialogue Fermer */
   strcpy (bufbutton, TtaGetMessage (LIB, TMSG_SAVE_DOC));
   i = strlen (TtaGetMessage (LIB, TMSG_SAVE_DOC)) + 1;
@@ -110,10 +110,11 @@ void AskToConfirm (PtrDocument pDoc, Document document, View view,
   TtaShowDialogue (NumFormClose, FALSE);
   /* attend le retour de ce formulaire (traite' par CallbackCloseDocMenu) */
   TtaWaitShowDialogue ();
-#else  /* _WINDOWS */
-  CreateCloseDocDlgWindow (TtaGetViewFrame(document,view), buftext,
-			   &SaveBeforeClosing, &CloseDontSave);
-#endif /* _WINDOWS */
+#endif /* #if defined(_GTK) || defined(_MOTIF) */
+
+#ifdef _WINGUI
+  CreateCloseDocDlgWindow (TtaGetViewFrame(document,view), buftext);
+#endif /* _WINGUI */
   *save = SaveBeforeClosing;
   *confirmation = !CloseDontSave;
 }
@@ -140,7 +141,8 @@ ThotBool            CloseADocument (Document document, Document docform,
 	{
 	  /* si le document a ete modifie', demande a l'utilisateur s'il */
 	  /* veut le sauver. L'utilisateur peut aussi annuler la commande.*/
-	  if (!pDoc->DocReadOnly && pDoc->DocModified)
+	  /* if (!pDoc->DocReadOnly && pDoc->DocModified) */
+	  if (pDoc->DocModified)
 	    {
 	      /* Faut-il creer le formulaire TtcCloseDocument */
 	      if (ThotLocalActions[T_confirmclose] == NULL)
@@ -155,7 +157,9 @@ ThotBool            CloseADocument (Document document, Document docform,
 		{
 		  if (save && ThotLocalActions[T_writedocument])
 		    /* l'utilisateur demande a sauver le document */
-		    ok = (* (Func)ThotLocalActions[T_writedocument]) (pDoc, 4);
+		    ok = (* (Func2)ThotLocalActions[T_writedocument]) (
+			(void *)pDoc,
+			(void *)4);
 		  if (ok)
 		    /* tout va bien, on ferme ce document */
 		    TCloseDocument (pDoc);
