@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -21,9 +21,10 @@
 #include "appdialogue.h"
 #include "dialog.h"
 #include "message.h"
-#ifdef _WINDOWS
-#include "wininclude.h"
-#endif /* _WINDOWS */
+
+#ifdef _WINGUI
+  #include "wininclude.h"
+#endif /* _WINGUI */
 
 #undef THOT_EXPORT
 #define THOT_EXPORT extern
@@ -33,24 +34,30 @@
 #include "select_tv.h"
 #include "appdialogue_tv.h"
 
+#ifdef _GTK
+  #include "gtk-functions.h"
+#endif /* _GTK */
+
 #define MAX_ARGS 20
 #define MAX_KEYBOARD 4
 
 
 typedef struct _item
   {
-     char          name;
+     char          index;
      unsigned char value;
-     char*         legend;
+     char         *legend;
   }
 ITEM;
 
 
 static ThotWidget   Keyboards[MAX_KEYBOARD];
-static ptrfont      KbFonts[MAX_KEYBOARD];
+static ThotFont     KbFonts[MAX_KEYBOARD];
 static int          KbX = 800;
 static int          KbY = 100;
-static ThotGC       GCkey;
+#if defined(_MOTIF) || defined(_WINGUI)
+  static ThotGC       GCkey;
+#endif /* #if defined(_MOTIF) || defined(_WINGUI) */
 
 /* data for the keyboards */
 /*iso */
@@ -315,33 +322,6 @@ static ITEM         Items_Isol[] =
 };
 
 
-static ITEM         Items_Symb[] =
-{
-   {83, 'S', NULL},		/* sum */
-   {80, 'P', NULL},		/* product */
-   {85, 'U', NULL},		/* union */
-   {73, 'I', NULL},		/* intersection */
-   {105, 'i', NULL},		/* integral */
-   {99, 'c', NULL},		/* circle integral */
-   {100, 'd', NULL},		/* double integral */
-   {114, 'r', NULL},		/* root */
-   {82, 'R', NULL},             /* right arrow */
-   {76, 'L', NULL},             /* left arrow */
-   {118, 'V', NULL},		/* down arrow */
-   {94, '^', NULL},		/* up arrow */
-   {91, '[', NULL},		/* [ */
-   {93, ']', NULL},		/* ] */
-   {40, '(', NULL},		/* ( */
-   {41, ')', NULL},		/* ) */
-   {123, '{', NULL},		/* { */
-   {125, '}', NULL},		/* } */
-   {124, '|', NULL},            /* | */
-   {68, 'D', NULL},             /* || */
-   {60, '<', NULL},		/* < */
-   {62, '>', NULL},		/* > */
-};
-
-
 static ITEM         Items_Graph[] =
 {
    {'C', 'C', NULL},		/* oval circle */
@@ -392,6 +372,8 @@ static ITEM         Items_Graph[] =
 #include "textcommands_f.h"
 #include "xwindowdisplay_f.h"
 
+
+#if defined(_MOTIF) || defined(_WINGUI)
 /*----------------------------------------------------------------------
    WChar
    displays character ch at position <x,y> of window w using the character
@@ -399,121 +381,141 @@ static ITEM         Items_Graph[] =
    Function func indicates if it's an active box (1) or not (0).
   ----------------------------------------------------------------------*/
 static void WChar (ThotWindow w, char ch, int x, int y, int func,
-		   ptrfont font, int disp, ThotGC GClocal)
+		   ThotFont font, int disp, ThotGC GClocal)
 {
-#ifndef _GTK
    int                 length;
 
    length = 1;
-#ifdef _WINDOWS
+   
+#ifdef _WINGUI
    /* DrawTextEx or some such thing - @@@ */
-#else
+#endif /* _WINGUI */
+
+#ifdef _MOTIF
    XSetFont (TtDisplay, GClocal, ((XFontStruct *) font)->fid);
    FontOrig (font, ch, &x, &y);
    XDrawString (TtDisplay, w, GClocal, x, y, &ch, length);
-#endif
-#endif /* _GTK */
+#endif /* _MOTIF*/
+   
 }
 
+#endif /* #if defined(_MOTIF) || defined(_WINGUI) */
+
 /*----------------------------------------------------------------------
    KbdEndDisplay
    Ends the display of a keyboard.
   ----------------------------------------------------------------------*/
-static void         KbdEndDisplay (ThotWidget w, int index, caddr_t call_d)
+static void KbdEndDisplay (ThotWidget w, int index, caddr_t call_d)
 {
-#ifndef _GTK
-#ifndef _WINDOWS
+#ifdef _MOTIF
    XtPopdown (Keyboards[index]);
-#endif /* _WINDOWS */
-#endif /* _GTK */
+#endif /* _MOTIF */
+   
+#ifdef _GTK
+   gtk_widget_hide (GTK_WIDGET(Keyboards[index]));
+#endif /* !_GTK */
 }
 
 /*----------------------------------------------------------------------
-   KbdCallbackHandler
-   handles the keyboard keys.
+  KbdCallbackHandler handles the keyboard keys.
   ----------------------------------------------------------------------*/
-void         KbdCallbackHandler (ThotWidget w, int param, caddr_t call_d)
+void KbdCallbackHandler (ThotWidget w, unsigned int param, caddr_t call_d)
 {
-#ifndef _GTK
-   unsigned char       car;
+   CHAR_T              car;
    ThotWidget          wp;
-   int                 i;
+   int                 i, kbd;
 
    /* Recupere la table des items */
-   car = (unsigned char) param % 256;
+   car = (CHAR_T) (param % 256);
    /* Recupere le widget de la palette */
-#ifndef _WINDOWS
+
+#ifdef _MOTIF
    wp=XtParent(XtParent(XtParent(XtParent(w))));
-#else  /* _WINDOWS */
+#endif /* #ifdef _MOTIF */
+   
+#ifdef _WINGUI
    wp = GetParent (GetParent (GetParent (GetParent (w))));
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
+
+#ifdef _GTK
+   wp = GTK_WIDGET (w)->parent->parent->parent->parent;
+#endif /* !_GTK */
+   
    /* met a jour l'indicateur de palette */
-   if(Keyboards[KeyboardMode] != wp)
+   if (Keyboards[KeyboardMode] != wp)
      {
      CloseTextInsertion();
-     for(i = 0; i < (MAX_KEYBOARD + 1); i++)
-       if(Keyboards[i] == wp)
+     for (i = 0; i < (MAX_KEYBOARD + 1); i++)
+       if (Keyboards[i] == wp)
          KeyboardMode = i;
      }
-   /* Insere le caractere selectionne */
+   kbd = KeyboardMode;
+   /* Insert the selected character */
+   if (KeyboardMode == 3)
+     /* character entered through the Symbol/Greek palette */
+     {
+       /* give the unicode value instead of the Symbol index */
+       car = TtaGetWCFromChar ((unsigned char) car, ISO_SYMBOL);
+       if (car < 256)
+	 /* the character entered is an ISO-Latin 1 character. It's the
+	    same as entering the character using the real keyboard */
+	 kbd = -1;
+     }
+
    if (ThotLocalActions[T_insertchar] != NULL)
-      (*ThotLocalActions[T_insertchar]) (ActiveFrame, car, KeyboardMode);
-#endif /* _GTK */
+      (*(Proc3)ThotLocalActions[T_insertchar]) (
+		(void *)ActiveFrame,
+		(void *)car,
+		(void *)kbd);
+   if (KeyboardMode == 3)
+     /* character entered through the Symbol/Greek palette */
+     CloseTextInsertion();
 }
 
-
-#ifndef _WINDOWS
-#ifndef _GTK
+#ifdef _MOTIF
 /*----------------------------------------------------------------------
-   ExposeKbd
-   displays the keyboard keys
+  ExposeKbd displays the keyboard keys
   ----------------------------------------------------------------------*/
-static void         ExposeKbd (ThotWidget w, int param, XmDrawnButtonCallbackStruct * infos)
+static void ExposeKbd (ThotWidget w, int param,
+		       XmDrawnButtonCallbackStruct *infos)
 {
-   int                 y;
-   int                 i, kb;
-   ITEM               *it;
+  int                 y;
+  int                 i, kb;
+  ITEM               *it;
 
-   /* Recupere la table des items */
-   kb = param / 256;
-   switch (kb)
-     {
-     case 0:
-       it = Items_Symb;
-       break;
-     case 1:
-       it = Items_Graph;
-       break;
-     case 2:
-       it = Items_Isol;
-       break;
-     case 4:
-       it = Items_Grec;
-       break;
-     default:
-       return;
-     }
-   y = 4;
-   i = param % 256;		/* indice dans la table des items */
-   it = (ITEM *) ((int) it + (sizeof (ITEM) * i));
-   WChar (infos->window, it->name, CharacterWidth (87, FontDialogue), y, GXcopy, KbFonts[kb], 0, GCkey);
-   if (it->legend)
-     {
-	y = FontHeight (KbFonts[kb]);
-	WChaine (infos->window, it->legend, 4, y, FontDialogue, GCkey);
-     }
+  /* Recupere la table des items */
+  kb = param / 256;
+  switch (kb)
+    {
+    case 1:
+      it = Items_Graph;
+      break;
+    case 2:
+      it = Items_Isol;
+      break;
+    case 4:
+      it = Items_Grec;
+      break;
+    default:
+      return;
+    }
+  y = 4;
+  i = param % 256;		/* indice dans la table des items */
+  it = (ITEM *) ((int) it + (sizeof (ITEM) * i));
+  WChar (infos->window, it->index, CharacterWidth (87, DialogFont), y, GXcopy, KbFonts[kb], 0, GCkey);
+  if (it->legend)
+    {
+      y = FontHeight (KbFonts[kb]);
+      WChaine (infos->window, it->legend, 4, y, DialogFont, GCkey);
+    }
 }
-#endif /* _GTK */
-
 
 /*----------------------------------------------------------------------
-   CreateKeyboard
-   creates a keyboard.
+   CreateKeyboard creates a keyboard.
   ----------------------------------------------------------------------*/
-static void CreateKeyboard (int number, char *title, ptrfont pFont,
+static void CreateKeyboard (int number, char *title, ThotFont pFont,
 			    int col, int x, int y, ITEM *items, int nbitem)
 {
-#ifndef _GTK
    int                 n;
    int                 i;
    register ITEM      *it;
@@ -536,18 +538,14 @@ static void CreateKeyboard (int number, char *title, ptrfont pFont,
    n++;
    XtSetArg (args[n], XmNuseAsyncGeometry, TRUE);
    n++;
-#ifdef OLD
-   w = XtCreateWidget (title, topLevelShellWidgetClass, XtParent (FrameTable[0].WdFrame), args, n);
-#else
    w = XtCreatePopupShell (title, applicationShellWidgetClass, RootShell, args, n);
-#endif
-/*** Cree le clavier dans sa frame ***/
+   /*** Cree le clavier dans sa frame ***/
    n = 0;
    XtSetArg (args[n], XmNbackground, BgMenu_Color);
    n++;
    XtSetArg (args[n], XmNborderColor, BgMenu_Color);
    n++;
-   xfont = XmFontListCreate ((XFontStruct *) FontDialogue, XmSTRING_DEFAULT_CHARSET);
+   xfont = XmFontListCreate ((XFontStruct *) DialogFont, XmSTRING_DEFAULT_CHARSET);
    XtSetArg (args[n], XmNfontList, xfont);
    n++;
    title_string = XmStringCreateSimple (title);
@@ -555,7 +553,7 @@ static void CreateKeyboard (int number, char *title, ptrfont pFont,
    n++;
    XtSetArg (args[n], XmNautoUnmanage, FALSE);
    n++;
-/**XtSetArg(args[n], XmNdefaultPosition, FALSE); n++;**/
+   /**XtSetArg(args[n], XmNdefaultPosition, FALSE); n++;**/
    XtSetArg (args[n], XmNmarginWidth, 0);
    n++;
    XtSetArg (args[n], XmNmarginHeight, 0);
@@ -567,8 +565,8 @@ static void CreateKeyboard (int number, char *title, ptrfont pFont,
    XtManageChild (w);
    XmStringFree (title_string);
 
-/*** Cree un Row-Column pour mettre le bouton Quit ***/
-/*** en dessous des touches du clavier.    ***/
+   /*** Cree un Row-Column pour mettre le bouton Quit ***/
+   /*** en dessous des touches du clavier.    ***/
    n = 3;
    XtSetArg (args[n], XmNadjustLast, FALSE);
    n++;
@@ -583,7 +581,7 @@ static void CreateKeyboard (int number, char *title, ptrfont pFont,
    w = XmCreateRowColumn (w, "Dialogue", args, n);
    XtManageChild (w);
 
-/*** Cree un Row-Column pour contenir les touches du clavier ***/
+   /*** Cree un Row-Column pour contenir les touches du clavier ***/
    n = 3;
    XtSetArg (args[n], XmNadjustLast, FALSE);
    n++;
@@ -602,7 +600,7 @@ static void CreateKeyboard (int number, char *title, ptrfont pFont,
    row = XmCreateRowColumn (w, "Dialogue", args, n);
    XtManageChild (row);
 
-/*** Cree un Row-Column pour contenir le bouton Quit ***/
+   /*** Cree un Row-Column pour contenir le bouton Quit ***/
    n = 3;
    XtSetArg (args[n], XmNorientation, XmHORIZONTAL);
    n++;
@@ -618,7 +616,7 @@ static void CreateKeyboard (int number, char *title, ptrfont pFont,
    GCmodel.background = BgMenu_Color;
    GCkey = XCreateGC (TtDisplay, TtRootWindow, GCForeground | GCBackground | GCFunction, &GCmodel);
 
-/*** Cree le bouton Quit ***/
+   /*** Cree le bouton Quit ***/
    n = 0;
    XtSetArg (args[n], XmNbackground, BgMenu_Color);
    n++;
@@ -664,7 +662,7 @@ static void CreateKeyboard (int number, char *title, ptrfont pFont,
 	n++;
 	for (i = 0; i < nbitem; i++, it++)
 	  {
-	     string[0] = it->name;
+	     string[0] = it->index;
 	     string[1] = EOS;
 	     w = XmCreatePushButton (row, string, args, n);
 	     XtManageChild (w);
@@ -679,14 +677,14 @@ static void CreateKeyboard (int number, char *title, ptrfont pFont,
 	n++;
 	XtSetArg (args[n], XmNmarginHeight, 0);
 	n++;
-	XtSetArg (args[n], XmNwidth, (Dimension) CharacterWidth (87, FontDialogue) * 3);
+	XtSetArg (args[n], XmNwidth, (Dimension) CharacterWidth (87, DialogFont) * 3);
 	n++;
-	XtSetArg (args[n], XmNheight, (Dimension) FontHeight (pFont) + FontHeight (FontDialogue) + 4);
+	XtSetArg (args[n], XmNheight, (Dimension) FontHeight (pFont) + FontHeight (DialogFont) + 4);
 	n++;
 
 	for (i = 0; i < nbitem; i++, it++)
 	  {
-	     string[0] = it->name;
+	     string[0] = it->index;
 	     string[1] = '\n';
 	     w = XmCreateDrawnButton (row, "", args, n);
 	     XtManageChild (w);
@@ -694,32 +692,156 @@ static void CreateKeyboard (int number, char *title, ptrfont pFont,
 	     XtAddCallback (w, XmNexposeCallback, (XtCallbackProc) ExposeKbd, (XtPointer) (param + i));
 	  }
      }
-#endif /* _GTK */
 }
+#endif /*#ifdef _MOTIF*/
 
+#ifdef _GTK
+/*----------------------------------------------------------------------
+  CreateKeyboard creates a keyboard.
+  ----------------------------------------------------------------------*/
+static void CreateKeyboard (int number, char *title, ThotFont pFont,
+			    int col, int x, int y, ITEM *items, int nbitem)
+{
+  int                 i,j;
+  register ITEM      *it;
+  ThotWidget          w;
+  GtkWidget *vbox1;
+  GtkWidget *vbox5=None;
+  GtkWidget *frame2;
+  GtkWidget *hbox1;
+  GtkWidget *hbox2;
+  GtkWidget *button1;
+  GtkWidget *label;
+  char                string[10];
+  int                 param;  
+  GtkStyle *current_style;
+  
+
+
+  w = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_title (GTK_WINDOW (w), title);
+  gtk_window_set_position (GTK_WINDOW (w), GTK_WIN_POS_MOUSE);
+  ConnectSignalGTK (GTK_OBJECT (w),
+		    "delete_event",
+		    GTK_SIGNAL_FUNC(gtk_true),
+		    (gpointer)NULL);
+  
+  vbox1 = gtk_vbox_new (FALSE, 0);
+  gtk_widget_show (vbox1);
+  gtk_container_add (GTK_CONTAINER (w), vbox1);
+   
+  frame2 = gtk_frame_new (NULL);
+  gtk_widget_show (frame2);
+  gtk_box_pack_start (GTK_BOX (vbox1), frame2, TRUE, TRUE, 0);
+  gtk_container_set_border_width (GTK_CONTAINER (frame2), 5);
+  
+  hbox1 = gtk_hbox_new (FALSE, 5);
+  gtk_widget_show (hbox1);
+  gtk_container_add (GTK_CONTAINER (frame2), hbox1);
+  gtk_container_set_border_width (GTK_CONTAINER (hbox1), 5);
+  
+  hbox2 = gtk_hbox_new (FALSE, 5);
+  gtk_widget_show (hbox2);
+  gtk_container_add (GTK_CONTAINER (vbox1), hbox2);
+   
+  button1 = gtk_button_new_with_label (TtaGetMessage (LIB, TMSG_CANCEL));
+  gtk_widget_show (button1);
+  GTK_WIDGET_SET_FLAGS (GTK_WIDGET(button1), GTK_CAN_DEFAULT);
+  gtk_widget_grab_default (GTK_WIDGET(button1));
+  gtk_box_pack_start (GTK_BOX (hbox2), button1, TRUE, TRUE, 2); 
+  gtk_container_set_border_width (GTK_CONTAINER (button1), 2);
+  ConnectSignalGTK (GTK_OBJECT (button1),
+		    "clicked",
+		    GTK_SIGNAL_FUNC (KbdEndDisplay),
+		    (gpointer)number);
+
+  sprintf (string, "+%d+%d", x, y);
+  /* Affiche les differents boutons du clavier */
+  it = items;
+  /* Prepare les parametres des procedures KbdCallbackHandler et ExposeKbd */
+  param = number * 256;	/* indice du clavier */
+  
+  if (it->legend == 0)
+    {
+      j = 0;
+      /* Un clavier sans legende */
+      for (i = 0; i < nbitem; i++, it++)
+	{
+	  if (i == 10 * j)
+	    {
+	      /* vbox witch contain one color column */
+	      vbox5 = gtk_vbox_new (TRUE, 5);
+	      gtk_widget_show (vbox5);
+	      gtk_box_pack_start (GTK_BOX (hbox1), vbox5, TRUE, TRUE, 0);
+	      j++;
+	    }
+	  string[0] = it->index;
+	  string[1] = EOS;
+	  button1 = gtk_button_new ();
+	  gtk_box_pack_start (GTK_BOX(vbox5), button1, TRUE, TRUE, 0);
+	  label = gtk_label_new (string);
+	  current_style = gtk_style_copy(gtk_widget_get_style(label));
+	  current_style->font = pFont;
+	  gtk_widget_set_style(label, current_style);
+	  gtk_widget_show (label);
+	  gtk_container_add (GTK_CONTAINER (button1), label);
+	  ConnectSignalGTK (GTK_OBJECT (button1),
+			    "clicked",
+			    GTK_SIGNAL_FUNC (KbdCallbackHandler),
+			    (gpointer)(param + (int)(it->value)));
+	  gtk_widget_show (button1);
+	}
+    }
+  else
+    {
+      j = 0;
+      /* Un clavier avec legende */
+      for (i = 0; i < nbitem; i++, it++)
+	{
+	  if (i == 10 * j)
+	    {
+	      /* vbox witch contain one color column */
+	      vbox5 = gtk_vbox_new (TRUE, 5);
+	      gtk_widget_show (vbox5);
+	      gtk_box_pack_start (GTK_BOX (hbox1), vbox5, TRUE, TRUE, 0);
+	      j++;
+	    }
+	  
+	  string[0] = it->index;
+	  string[1] = '\n';
+	  button1 = gtk_button_new ();
+	  gtk_widget_show (button1);
+	  gtk_box_pack_start (GTK_BOX(vbox5), button1, TRUE, TRUE, 0);
+	  label = gtk_label_new (string);
+	  current_style = gtk_style_copy(gtk_widget_get_style(label));
+	  current_style->font = pFont;
+	  gtk_widget_set_style(label, current_style);
+	  gtk_widget_show (label);
+	  ConnectSignalGTK (GTK_OBJECT (button1),
+			    "clicked",
+			    GTK_SIGNAL_FUNC (KbdCallbackHandler),
+			    (gpointer)(param + (int)(it->value)));
+	}
+    }
+  gtk_widget_show_all (w);
+  Keyboards[number] = w;
+}
+#endif /* _GTK */
+
+#if defined(_GTK) || defined(_MOTIF)
 
 /*----------------------------------------------------------------------
    LoadKbd
    loads a keyboard.
   ----------------------------------------------------------------------*/
-static void         LoadKbd (int number)
+static void LoadKbd (int number)
 {
-#ifndef _GTK
-  ptrfont             pFontAc;
-  ptrfont             pFontIg;
+  ThotFont            pFontAc;
+  ThotFont            pFontIg;
 
   ConfigKeyboard (&KbX, &KbY);
   switch (number)
     {
-    case 0:		/* Symboles */
-      if (SymbolIcons != NULL)
-	{
-	  KbFonts[number] = SymbolIcons;
-	  CreateKeyboard (number, TtaGetMessage (LIB, TMSG_MATH_SYMBOLS),
-			  SymbolIcons, 3, KbX, KbY, Items_Symb,
-			  sizeof (Items_Symb) / sizeof (ITEM));
-	}
-      break;
     case 1:		/* Graphiques */
       if (GraphicsIcons != NULL)
 	{
@@ -731,50 +853,60 @@ static void         LoadKbd (int number)
 	}
       break;
     case 2:		/* ISO latin 1 */
-      pFontAc = ReadFont ('L', 'T', 0, 14, UnPoint);
+      pFontAc = ReadFont ('L', 1, 0, 14, UnPoint);
       if (!pFontAc)
-	pFontAc = FontDialogue;
+	pFontAc = DialogFont;
       KbFonts[number] = pFontAc;
       if (pFontAc != NULL)
 	CreateKeyboard (number, TtaGetMessage (LIB, TMSG_LATIN_ALPHABET), pFontAc, 13,
 			KbX, KbY, Items_Isol, sizeof (Items_Isol) / sizeof (ITEM));
       break;
-    case 3:		/* Grec */
-      pFontIg = ReadFont ('G', 'T', 0, 14, UnPoint);
+    case 3:		/* Greek */
+      pFontIg = ReadFont ('G', 1, 0, 14, UnPoint);
+      /*pFontIg = LoadFont ("-ttf-esstixone-*-*-*-*-14-*-*-*-*-*-*-*");*/
       if (!pFontIg)
-	pFontIg = FontDialogue;
+	pFontIg = DialogFont;
       if (pFontIg != NULL)
-	KbFonts[number] = pFontIg;	/* Grec */
+	KbFonts[number] = pFontIg;	/* Greek */
       CreateKeyboard (number, TtaGetMessage (LIB, TMSG_GREEK_ALPHABET), pFontIg, 16,
 		      KbX, KbY, Items_Grec, sizeof (Items_Grec) / sizeof (ITEM));
       break;
     }
-#endif /* _GTK */
 }
-#endif /* _WINDOWS */
+
+#endif /* #if defined(_GTK) || defined(_MOTIF) */
 
 
 /*----------------------------------------------------------------------
    KeyboardMap
    maps a keyboard.
   ----------------------------------------------------------------------*/
-void                KeyboardMap (int kb)
+void KeyboardMap (int kb)
 {
-#ifndef _GTK
-#ifndef _WINDOWS
-   if (kb >= 0 && kb < MAX_KEYBOARD)
-     {
+#if defined(_GTK) || defined(_MOTIF)  
+
+  if (kb >= 0 && kb < MAX_KEYBOARD)
+  {
 	/* Faut-il charger le clavier avant de l'afficher ? */
 	if (Keyboards[kb] == 0)
 	   LoadKbd (kb);
 	if (Keyboards[kb] != 0)
 	  {
+      
+#ifdef _MOTIF
 	    XtPopup (Keyboards[kb], XtGrabNonexclusive);
 	    XMapRaised (TtDisplay, XtWindowOfObject (Keyboards[kb]));
+#endif /* #ifdef _MOTIF */
+      
+#ifdef _GTK
+	    gtk_widget_show_all (GTK_WIDGET(Keyboards[kb]));
+	    gdk_window_raise (GTK_WIDGET(Keyboards[kb])->window);
+#endif /* !_GTK */
+      
 	  }
-     }
-#endif /* _WINDOWS */
-#endif /* _GTK */
+  }
+
+#endif /* #if defined(_GTK) || defined(_MOTIF) */
 }
 
 /*----------------------------------------------------------------------
@@ -783,7 +915,6 @@ void                KeyboardMap (int kb)
   ----------------------------------------------------------------------*/
 void GraphicsLoadResources ()
 {
-#ifndef _GTK
    int                 i;
 
    if (ThotLocalActions[T_keyboard] == NULL)
@@ -793,29 +924,26 @@ void GraphicsLoadResources ()
 	/* Initialise la table des claviers */
 	for (i = 0; i < MAX_KEYBOARD; i++)
 	   Keyboards[i] = 0;
-	SymbolIcons = NULL;
-
-	if (SmallFontDialogue == NULL)
-	   SmallFontDialogue = ReadFont ('L', 'H', 0, 9, UnPoint);
-	if (SmallFontDialogue == NULL)
-	   SmallFontDialogue = FontDialogue;
+	if (SmallDialogFont == NULL)
+	   SmallDialogFont = ReadFont ('L', 2, 0, 9, UnPoint);
+	if (SmallDialogFont == NULL)
+	   SmallDialogFont = DialogFont;
      }
 
-#ifndef _WINDOWS
+#if defined(_GTK) || defined(_MOTIF)
    GraphicsIcons = LoadFont ("ivgraf");
-#endif /* _WINDOWS */
+#endif /*  #if defined(_GTK) || defined(_MOTIF) */
+   
    if (GraphicsIcons == NULL)
-     GraphicsIcons = FontDialogue;
-#endif /* _GTK */
+     GraphicsIcons = DialogFont;
 }
 
 /*----------------------------------------------------------------------
   KeyboardsLoadResource
   Initializes the keyboards.
   ----------------------------------------------------------------------*/
-void                KeyboardsLoadResources ()
+void KeyboardsLoadResources ()
 {
-#ifndef _GTK
   int                 i;
 
   if (ThotLocalActions[T_keyboard] == NULL)
@@ -826,19 +954,11 @@ void                KeyboardsLoadResources ()
 	Keyboards[i] = 0;
       GraphicsIcons = NULL;
 
-      if (SmallFontDialogue == NULL)
-	SmallFontDialogue = ReadFont ('L', 'H', 0, 9, UnPoint);
-      if (SmallFontDialogue == NULL)
-	SmallFontDialogue = FontDialogue;
+      if (SmallDialogFont == NULL)
+	SmallDialogFont = ReadFont ('L', 2, 0, 9, UnPoint);
+      if (SmallDialogFont == NULL)
+	SmallDialogFont = DialogFont;
     }
-
-#ifndef _WINDOWS
-  SymbolIcons = LoadFont ("ivsymb");
-  if (SymbolIcons == NULL)
-    /* Fonte 'ivsymb' inaccessible */
-    SymbolIcons = FontDialogue;
-#endif /* _WINDOWS */
-#endif /* _GTK */
 }
 
 
@@ -848,14 +968,12 @@ void                KeyboardsLoadResources ()
   ----------------------------------------------------------------------*/
 void TtcDisplayMathKeyboard (Document document, View view)
 {
-#ifndef _GTK
    KeyboardsLoadResources ();
    /* Enregistre la position pour le dialogue */
-#ifndef _WINDOWS
+#if defined(_GTK) || defined(_MOTIF)
    TtaSetDialoguePosition ();
-#endif /* !_WINDOWS */
+#endif /* #if defined(_GTK) || defined(_MOTIF) */
    TtaSetCurrentKeyboard (0);
-#endif /* _GTK */
 }
 
 
@@ -865,14 +983,12 @@ void TtcDisplayMathKeyboard (Document document, View view)
   ----------------------------------------------------------------------*/
 void TtcDisplayGraphicsKeyboard (Document document, View view)
 {
-#ifndef _GTK
    KeyboardsLoadResources ();
    /* Enregistre la position pour le dialogue */
-#ifndef _WINDOWS
+#if defined(_GTK) || defined(_MOTIF)
    TtaSetDialoguePosition ();
-#endif /* !_WINDOWS */
+#endif /* #if defined(_GTK) || defined(_MOTIF) */
    TtaSetCurrentKeyboard (1);
-#endif /* _GTK */
 }
 
 
@@ -880,33 +996,29 @@ void TtcDisplayGraphicsKeyboard (Document document, View view)
    TtcDisplayLatinKeyboard
    displays the latin keyboard
   ----------------------------------------------------------------------*/
-void                TtcDisplayLatinKeyboard (Document document, View view)
+void TtcDisplayLatinKeyboard (Document document, View view)
 {
-#ifndef _GTK
    KeyboardsLoadResources ();
    /* Enregistre la position pour le dialogue */
-#ifndef _WINDOWS 
+#if defined(_GTK) || defined(_MOTIF)
    TtaSetDialoguePosition ();
-#endif /* !_WINDOWS */
+#endif /* #if defined(_GTK) || defined(_MOTIF) */
    TtaSetCurrentKeyboard (2);
-#endif /* _GTK */
 }
 
 /*----------------------------------------------------------------------
    TtcDisplayGreekKeyboard
    displays the greek keyboard 
   ----------------------------------------------------------------------*/
-void                TtcDisplayGreekKeyboard (Document document, View view)
+void TtcDisplayGreekKeyboard (Document document, View view)
 {
-#ifndef _GTK
    KeyboardsLoadResources ();
    /* Enregistre la position pour le dialogue */
-#ifndef _WINDOWS 
+#if defined(_GTK) || defined(_MOTIF)
    TtaSetDialoguePosition ();
-#endif /* !_WINDOWS */
+#endif /* #if defined(_GTK) || defined(_MOTIF) */
    TtaSetCurrentKeyboard (3);
-#ifdef _WINDOWS
+#ifdef _WINGUI
    CreateGreekKeyboardDlgWindow (NULL);
-#endif /* _WINDOWS */
-#endif /* _GTK */
+#endif /* _WINGUI */
 }
