@@ -1,14 +1,14 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
  
 /*
-   nodialog : empty functions for building the process print
+   nodialog : empty functions/globals variables for building the process print
  */
-
+#include "thot_gui.h"
 #include "thot_sys.h"
 #include "constmedia.h"
 #include "typemedia.h"
@@ -16,8 +16,28 @@
 #include "frame.h"
 #include "application.h"
 #include "memory_f.h"
+#include "nodialog.h"
 
+/*----------------------------------------------------------------------
+  DisplayPRule displays the presentation rule pointed by RP.
+  ----------------------------------------------------------------------*/
+void DisplayPRule (PtrPRule RP, FILE *fileDescriptor,
+		   PtrElement pEl, PtrPSchema pSchP)
+{
+}
 
+/*----------------------------------------------------------------------
+  ----------------------------------------------------------------------*/
+void ShowBox (int frame, PtrBox pBox, int position, int percent)
+{
+}
+
+/*----------------------------------------------------------------------
+  ----------------------------------------------------------------------*/
+void ExtendSelection (PtrElement pEl, int rank, ThotBool fixed, ThotBool begin,
+		      ThotBool drag)
+{
+}
 
 /*----------------------------------------------------------------------
    ConfigFree
@@ -35,6 +55,13 @@ void TtaFreeAllCatalogs (void)
 {
 }
 
+/*----------------------------------------------------------------------
+   InitDialogueFont initializes the dialogue font
+  ----------------------------------------------------------------------*/
+void InitDialogueFont ()
+{
+}
+
 /*----------------------------------------------------------------------
    TteFreeAllEventsList frees all event lists and the event/action
    context pointers.			                        
@@ -63,6 +90,12 @@ ThotBool             IsW3Path (const char* path)
   return FALSE;
 }
 
+/*----------------------------------------------------------------------
+  ----------------------------------------------------------------------*/
+void CloseTextInsertion ()
+{
+}
+
 /*----------------------------------------------------------------------
    NormalizeURL
    The function returns the new complete and normalized URL 
@@ -79,7 +112,7 @@ void NormalizeURL (char* orgName, Document doc, char* newName, char* docName, ch
    When returning, the parameter completeURL contains the normalized url
    and the parameter localfile the path of the local copy of the file.
   ----------------------------------------------------------------------*/
-ThotBool LoadRemoteStyleSheet (char *url, Document doc, Element el, void *css, char *completeURL, char *localfile)
+ThotBool LoadRemoteStyleSheet (char *url, Document doc, Element el, CSSInfoPtr css, char *completeURL, char *localfile)
 {
   strcpy (completeURL, url);
   strcpy (localfile, url);
@@ -90,10 +123,16 @@ ThotBool LoadRemoteStyleSheet (char *url, Document doc, Element el, void *css, c
    FetchImage loads an IMG from local file or from the web. The flags
    may indicate extra transfer parameters, for example bypassing the cache.		
   ----------------------------------------------------------------------*/
-void FetchImage (Document doc, Element el, char *URL, int flags, Proc callback, void *extra)
+void FetchImage (Document doc, Element el, char *URL, int flags,
+		 LoadedImageCallback callback, void *extra)
 {
   if (callback && URL)
-    callback (doc, el, URL, extra);
+	(*callback) (
+		doc,
+		el,
+		URL,
+		extra,
+		FALSE);
 }
 
 /*----------------------------------------------------------------------
@@ -104,7 +143,7 @@ void Prof_FreeTable ()
 
 /*----------------------------------------------------------------------
   ----------------------------------------------------------------------*/
-Pixmap TtaCreatePixmapLogo (char** data)
+ThotIcon TtaCreatePixmapLogo (char** data)
 {
    return None;
 }
@@ -154,7 +193,9 @@ void FreeTranslations ()
 
 /*----------------------------------------------------------------------
   ----------------------------------------------------------------------*/
-void RedisplayCopies ()
+void RedisplayCopies ( PtrElement pEl,
+                       PtrDocument pDoc,
+                       ThotBool redisplay )
 {
 }
 
@@ -167,7 +208,7 @@ ThotBool IsASavedElement (PtrElement pEl)
 
 /*----------------------------------------------------------------------
   ----------------------------------------------------------------------*/
-ThotBool CallEventType (int *notifyEvent, ThotBool pre)
+ThotBool CallEventType (NotifyEvent * notifyEvent, ThotBool pre)
 {
    return FALSE;
 }
@@ -179,7 +220,6 @@ ThotBool CallEventAttribute (NotifyAttribute * notifyAttr, ThotBool pre)
    return FALSE;
 }
 
-
 /*----------------------------------------------------------------------
    TtaSetStatus affiche le status de la vue du document.                      
   ----------------------------------------------------------------------*/
@@ -211,7 +251,7 @@ void UpdateScrollbars (int frame)
 
 /*----------------------------------------------------------------------
   ----------------------------------------------------------------------*/
-void ChangeFrameTitle (int frame, char *texte)
+void ChangeFrameTitle (int frame, char *texte, CHARSET encoding)
 {
 }
 
@@ -279,7 +319,7 @@ void SelectElement (PtrDocument pDoc, PtrElement pEl, ThotBool Debut, ThotBool C
 
 /*----------------------------------------------------------------------
   ----------------------------------------------------------------------*/
-void ComputeViewSelMarks (ViewSelection * marque)
+void ComputeViewSelMarks (ViewSelection * marque, int frame)
 {
 }
 
@@ -297,7 +337,8 @@ void CancelSelection ()
 
 /*----------------------------------------------------------------------
   ----------------------------------------------------------------------*/
-void SelectRange (PtrDocument SelDoc, PtrElement PremSel, PtrElement DerSel, int premcar, int dercar)
+void SelectRange (PtrDocument SelDoc, PtrElement PremSel, PtrElement DerSel,
+		  int premcar, int dercar)
 {
 }
 
@@ -327,8 +368,10 @@ void SetNewSelectionStatus (int frame, PtrAbstractBox pAb, ThotBool Etat)
 /*----------------------------------------------------------------------
   DisplayStringSelection the selection on a substring of text
   between leftX and rightX.
+  The parameter t gives the top extra margin of the box.
   ----------------------------------------------------------------------*/
-void DisplayStringSelection (int frame, int leftX, int rightX, PtrBox pBox)
+void DisplayStringSelection (int frame, int leftX, int rightX, int t,
+			     PtrBox pBox)
 {
 }
 
@@ -377,3 +420,4 @@ ThotBool OpenParsingErrors (Document document)
 {
   return TRUE;
 }
+
