@@ -1,10 +1,10 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996.
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
-
+#include "thot_gui.h"
 #include "thot_sys.h"
 #include "constmedia.h"
 #include "typemedia.h"
@@ -21,7 +21,8 @@
 #include "memory_f.h"
 #include "readpivot_f.h"
 
-/* procedure bidon declarees localement et exportees */
+/* dummy procedures to make the link editor happy */
+
 int TtaGetThotColor (unsigned short red, unsigned short green, unsigned short blue)
 {
   return 1;
@@ -37,8 +38,7 @@ ThotBool CallEventAttribute (NotifyAttribute * notifyAttr, ThotBool pre)
    return FALSE;
 }
 
-
-void FreePictInfo (PictInfo *esc)
+void CleanPictInfo (PictInfo *esc)
 {
 }
 
@@ -49,7 +49,7 @@ void AddInSelection (PtrElement pEl, ThotBool dernier)
 
 
 /* FreeAbEl est appele' par le module arbabs */
-void FreeAbEl (PtrElement pEl)
+void FreeAbEl (PtrElement pEl,PtrDocument pDoc)
 {
 }
 
