@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -19,7 +19,6 @@
 #include "fileaccess.h"
 #include "appdialogue.h"
 #include "fileaccess.h"
-#include "thotdir.h"
 #include "application.h"
 
 #undef THOT_EXPORT
@@ -158,8 +157,6 @@ Document TtaInitDocument (char *structureSchema, char *documentName,
 		  /* Create required attributes for the whole created tree */
 		  AttachMandatoryAttributes (pDoc->DocDocElement, pDoc);
 #endif
-		  /* dealing with exceptions */
-		  CreateWithException (pEl, pDoc);
 		  /* An attribut Language is stored in the root */
 		  CheckLanguageAttr (pDoc, pEl);
 		  /* The document is named */
@@ -248,8 +245,8 @@ void NewDocument (PtrDocument *pDoc, char *SSchemaName, char *docName,
 	 (*pDoc)->DocDirectory[i] = EOS;
 	 /* on suppose que le mon de schema est dans la langue de */
 	 /* l'utilisateur: on le traduit en nom interne */
-	 ConfigSSchemaInternalName (SSchemaName, docType, TRUE);
-	 if (docType[0] == EOS)
+	 /*ConfigSSchemaInternalName (SSchemaName, docType, TRUE);
+	   if (docType[0] == EOS)*/
 	    /* ce nom n'est pas dans le fichier langue, on le prend */
 	    /* tel quel */
 	    strncpy (docType, SSchemaName, MAX_NAME_LENGTH);
@@ -354,15 +351,12 @@ void NewDocument (PtrDocument *pDoc, char *SSchemaName, char *docName,
 	       /* le document n'a pas ete ferme' pendant l'attente */
 	       /* des attributs requis */
 	       {
-	       /* traitement des exceptions */
-	       if (ThotLocalActions[T_createtable] != NULL)
-		  (*ThotLocalActions[T_createtable]) (pEl, *pDoc);
 	       /* ouvre les vues du document cree' */
 	       OpenDefaultViews (*pDoc);
 	       /* selectionne la 1ere feuille du document */
 	       pEl = FwdSearchTypedElem ((*pDoc)->DocDocElement,
 					 (*pDoc)->DocSSchema->SsRootElem,
-					 (*pDoc)->DocSSchema);
+					 (*pDoc)->DocSSchema, NULL);
 	       if (pEl)
 		 {
 	          pEl = FirstLeaf (pEl);
@@ -404,7 +398,7 @@ void                PaginateDocument (PtrDocument pDoc)
    UpdateIncludedElement met a` jour et reaffiche l'element pEl inclus dans  
    le document pDoc.                                       
   ----------------------------------------------------------------------*/
-void                UpdateIncludedElement (PtrElement pEl, PtrDocument pDoc)
+void UpdateIncludedElement (PtrElement pEl, PtrDocument pDoc)
 {
    PtrElement          pChild, pNext;
    PtrTextBuffer       pBuf, pNextBuf;
@@ -419,7 +413,7 @@ void                UpdateIncludedElement (PtrElement pEl, PtrDocument pDoc)
    DestroyAbsBoxes (pEl, pDoc, FALSE);
    /* Update Abstract views */
    if (ThotLocalActions[T_AIupdate] != NULL)
-     (*ThotLocalActions[T_AIupdate]) (pDoc);
+     (*(Proc1)ThotLocalActions[T_AIupdate]) ((void *)pDoc);
    if (pEl->ElTerminal)
      switch (pEl->ElLeafType)
        {
@@ -491,10 +485,10 @@ void                UpdateIncludedElement (PtrElement pEl, PtrDocument pDoc)
    /* reaffiche l'element dans toutes les vues ou il existe */
    /* Update Abstract views */
    if (ThotLocalActions[T_AIupdate] != NULL)
-     (*ThotLocalActions[T_AIupdate]) (pDoc);
+     (*(Proc1)ThotLocalActions[T_AIupdate]) ((void *)pDoc);
    /* Redisplay views */
    if (ThotLocalActions[T_redisplay] != NULL)
-     (*ThotLocalActions[T_redisplay]) (pDoc);
+     (*(Proc1)ThotLocalActions[T_redisplay]) ((void *)pDoc);
    /* Reaffiche les numeros suivants qui changent */
    UpdateNumbers (NextElement (pEl), pEl, pDoc, TRUE);
 }
@@ -692,7 +686,9 @@ void BackupAll()
       /* il y a un document pour cette entree de la table */
       if (LoadedDocument[doc]->DocModified)
 	if (ThotLocalActions[T_writedocument] != NULL)
-	   (* ThotLocalActions[T_writedocument]) (LoadedDocument[doc], 3);
+	   (*(Proc2)ThotLocalActions[T_writedocument]) (
+		(void *)LoadedDocument[doc],
+		(void *)3);
 }
 
 /*----------------------------------------------------------------------
