@@ -1,10 +1,10 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
-
+#include "thot_gui.h"
 #include "thot_sys.h"
 #include "constmedia.h"
 #include "typemedia.h"
@@ -14,7 +14,6 @@
 #include "constpiv.h"
 #include "typecorr.h"
 #include "appdialogue.h"
-#include "thotdir.h"
 #include "fileaccess.h"
 
 #ifdef NODISPLAY
@@ -22,7 +21,7 @@
      LastSelectedElement are defined here, to be used at the end of
      function MergeTextElements  ***/
 #undef THOT_EXPORT
-#define THOT_EXPORT
+#define THOT_EXPORT extern
 #include "select_tv.h"
 #endif
 
@@ -61,18 +60,53 @@
 #include "views_f.h"
 #include "writepivot_f.h"
 
-static char nameBuffer[MAX_NATURES_DOC];
-static char ISObuffer[MAX_NATURES_DOC];
+static char nameBuffer[100];
+static char ISObuffer[400];
 
 /*----------------------------------------------------------------------
-  ----------------------------------------------------------------------*/
-void CreateWithException (PtrElement pEl, PtrDocument pDoc)
+  TtaGetDocumentProfile
+  Gets the document profile
+ ----------------------------------------------------------------------*/
+int TtaGetDocumentProfile (Document document)
 {
-   /* If table creation */
-   if (ThotLocalActions[T_createtable] != NULL)
-      (*ThotLocalActions[T_createtable]) (pEl, pDoc);
+  PtrDocument pDoc;
+
+  UserErrorCode = 0;
+  /* verifies the parameter document */
+  if (document < 1 || document > MAX_DOCUMENTS || LoadedDocument[document - 1] == NULL)
+    {
+      TtaError (ERR_invalid_document_parameter);
+      return (UNDEFINED_CHARSET);
+    }
+  else
+    {
+      pDoc = LoadedDocument[document - 1];
+      return (pDoc->DocProfile);
+    }
 }
 
+/*----------------------------------------------------------------------
+  TtaSetDocumentProfile
+  Sets the document profile
+ ----------------------------------------------------------------------*/
+void TtaSetDocumentProfile (Document document, int profile)
+{
+  PtrDocument pDoc;
+
+  UserErrorCode = 0;
+  /* verifies the parameter document */
+  if (document < 1 || document > MAX_DOCUMENTS)
+    TtaError (ERR_invalid_document_parameter);
+  else if (LoadedDocument[document - 1] == NULL)
+    TtaError (ERR_invalid_document_parameter);
+  else
+    {
+      pDoc = LoadedDocument[document - 1];
+      pDoc->DocProfile = profile;
+    }
+}
+
+
 /*----------------------------------------------------------------------
   TtaGetDocumentCharset gets the document charset
   Returns UNDEFINED_CHARSET when the document uses the default charset.
@@ -99,10 +133,11 @@ CHARSET TtaGetDocumentCharset (Document document)
 }
 
 
-/*----------------------------------------------------------------------
+/*------------------------------------------------------------------------------
   TtaSetDocumentCharset sets the document charset
- ----------------------------------------------------------------------*/
-void TtaSetDocumentCharset (Document document, CHARSET charSet)
+  The boolean 'default' indicates we are setting the default document charset
+ -------------------------------------------------------------------------------*/
+void TtaSetDocumentCharset (Document document, CHARSET charSet, ThotBool defaultCharset)
 {
   PtrDocument pDoc;
 
@@ -116,7 +151,8 @@ void TtaSetDocumentCharset (Document document, CHARSET charSet)
     {
       pDoc = LoadedDocument[document - 1];
       pDoc->DocCharset = charSet;
-      pDoc->DocDefaultCharset = FALSE;
+      if (!defaultCharset)
+	pDoc->DocDefaultCharset = FALSE;
     }
 }
 
@@ -190,11 +226,11 @@ void UnloadTree (Document document)
   PtrDocument      pDoc;
 
   pDoc = LoadedDocument[document - 1];
-  if (pDoc != NULL)
+  if (pDoc)
      {
        /* remove the selection on the document */
        if (ThotLocalActions[T_resetsel])
-	 (*ThotLocalActions[T_resetsel]) (pDoc);
+	 (*(Proc1)ThotLocalActions[T_resetsel]) (pDoc);
 #ifndef NODISPLAY
        if (DocOfSavedElements == pDoc)
 	 DocOfSavedElements = NULL;
@@ -216,7 +252,7 @@ void UnloadDocument (PtrDocument * pDoc)
     /* look for the current document in the documents table */
     {
       d = 0;
-      while (LoadedDocument[d] != *pDoc && d < MAX_DOCUMENTS - 1)
+      while (d < MAX_DOCUMENTS - 1 && LoadedDocument[d] != *pDoc)
 	d++;
       if (LoadedDocument[d] == *pDoc)
 	{
@@ -262,7 +298,7 @@ void TtaCloseDocument (Document document)
     {
       pDoc = LoadedDocument[document - 1];
       if (ThotLocalActions[T_clearhistory] != NULL)
-	(*ThotLocalActions[T_clearhistory]) (pDoc);
+	(*(Proc1)ThotLocalActions[T_clearhistory]) (pDoc);
 #ifndef NODISPLAY
       /* Closing all opened views relating to the document */
       /* close the views */
@@ -460,7 +496,7 @@ SSchema TtaNewNature (Document document, SSchema schema, char *natureName,
 	TtaError (ERR_invalid_parameter);
       else
 	{
-	  natureSchema = ((PtrSSchema) schema)->SsRule[natureRule - 1].SrSSchemaNat;
+	  natureSchema = ((PtrSSchema) schema)->SsRule->SrElem[natureRule - 1]->SrSSchemaNat;
 #ifndef NODISPLAY
 	  if (document > 0)
 	    AddSchemaGuestViews (LoadedDocument[document - 1], natureSchema);
@@ -630,9 +666,15 @@ void TtaSetDocumentAccessMode (Document document, int accessMode)
 	   if (accessMode == 0)
 	     /* disable the Paste command */
 	     SwitchPaste (pDoc, FALSE);
+#ifdef _WINGUI
+	   else
+	     /* enable the Paste command */
+	     SwitchPaste (pDoc, TRUE);
+#else /* _WINGUI */
 	   else if (FirstSavedElement != NULL || ClipboardThot.BuLength != 0)
 	     /* enable the Paste command */
 	     SwitchPaste (pDoc, TRUE);
+#endif /* _WINGUI */
 	   SetAccessMode (pDoc, accessMode);
 #endif
 	 }
@@ -643,27 +685,15 @@ void TtaSetDocumentAccessMode (Document document, int accessMode)
 /*----------------------------------------------------------------------
    TtaSetDocumentBackUpInterval
 
-   Sets the backup interval for a document.
-   Parameters:
-   document: the document whose backup interval is changed.
-   interval:
-   0 : the backup mechanism must be disabled
-   positive integer : number of characters typed which triggers
-   automatic save of the document into a .BAK file.
+   Sets the backup interval of documents.
+   Parameter:
+   interval: 0 if the backup mechanism is disabled, or gives the number
+   of characters which triggers an automatic save into a .BAK file.
   ----------------------------------------------------------------------*/
-void TtaSetDocumentBackUpInterval (Document document, int interval)
+void TtaSetDocumentBackUpInterval (int interval)
 {
   UserErrorCode = 0;
-  /* verifies the parameter document */
-  if (document < 1 || document > MAX_DOCUMENTS)
-    TtaError (ERR_invalid_document_parameter);
-  else if (LoadedDocument[document - 1] == NULL)
-    TtaError (ERR_invalid_document_parameter);
-  else if (interval < 0)
-    /* parameter document is correct */
-    TtaError (ERR_invalid_parameter);
-  else
-    LoadedDocument[document - 1]->DocBackUpInterval = interval;
+  DocBackUpInterval = interval;
 }
 
 /*----------------------------------------------------------------------
@@ -690,8 +720,8 @@ void  TtaSetNotificationMode (Document document, int notificationMode)
 }
 
 /*----------------------------------------------------------------------
-   SetDocumentModified set the document flags DocUpdated DocModified to
-   the status.
+   SetDocumentModified sets the document flags DocUpdated and DocModified 
+   to the status value.
    The parameter length gives the number of characters added.
    If the previous status of DocUpdated was FALSE the function notifies
    the application.
@@ -703,18 +733,18 @@ void  SetDocumentModified (PtrDocument pDoc, ThotBool status, int length)
       if (status)
 	{
 	  if (!pDoc->DocUpdated && ThotLocalActions[T_docmodified])
-	    (* ThotLocalActions[T_docmodified]) (IdentDocument (pDoc), TRUE);
+	    (*(Proc2)ThotLocalActions[T_docmodified]) ((void *)IdentDocument (pDoc), (void *)TRUE);
 	  pDoc->DocModified = TRUE;
 	  pDoc->DocUpdated = TRUE;
-	  pDoc->DocNTypedChars += length;
+	  /* pDoc->DocNTypedChars += length; */
 	}
       else
 	{
 	  if (pDoc->DocUpdated && ThotLocalActions[T_docmodified])
-	    (* ThotLocalActions[T_docmodified]) (IdentDocument (pDoc), FALSE);
+	    (*(Proc2)ThotLocalActions[T_docmodified]) ((void *)IdentDocument (pDoc), (void *)FALSE);
 	  pDoc->DocModified = FALSE;
 	  pDoc->DocUpdated = FALSE;
-	  pDoc->DocNTypedChars = 0;
+	  /* pDoc->DocNTypedChars = 0; */
 	}
     }
 }
@@ -916,7 +946,7 @@ char *TtaGetSSchemaName (SSchema schema)
        TtaError (ERR_invalid_parameter);
      }
    else
-     strcpy (ISObuffer, ((PtrSSchema) schema)->SsName);
+     strncpy (ISObuffer, ((PtrSSchema) schema)->SsName, 400);
    return ISObuffer;
 }
 
@@ -939,7 +969,7 @@ char *TtaGetPSchemaName (SSchema schema)
        TtaError (ERR_invalid_parameter);
      }
    else
-     strcpy (ISObuffer, ((PtrSSchema) schema)->SsDefaultPSchema);
+     strncpy (ISObuffer, ((PtrSSchema) schema)->SsDefaultPSchema, 400);
    return ISObuffer;
 }
 
@@ -986,13 +1016,13 @@ SSchema TtaGetSSchema (char *name, Document document)
    the structure schema having this URI, or NULL if this structure
    schema is not loaded or not used by the document.
   ----------------------------------------------------------------------*/
-SSchema             TtaGetSSchemaByUri (CHAR_T* uriName, Document document)
+SSchema TtaGetSSchemaByUri (char *uriName, Document document)
 {
    SSchema          schema;
 
    UserErrorCode = 0;
    schema = NULL;
-   if (uriName == NULL || uriName[0] == WC_EOS)
+   if (uriName == NULL || uriName[0] == EOS)
      TtaError (ERR_invalid_parameter);
    /* verifies the parameter document */
    else if (document < 1 || document > MAX_DOCUMENTS)
@@ -1071,17 +1101,17 @@ void TtaGiveSchemasOfDocument (char *documentName, char *structureName,
      {
 	error = FALSE;
 	/* Gets the version number if it exists */
-	if (!TtaReadByte (file, &charGotten))
+	if (!TtaReadByte (file, (unsigned char *)&charGotten))
 	   error = TRUE;
 	if (charGotten == (char) C_PIV_VERSION)
 	  {
-	     if (!TtaReadByte (file, &charGotten))
+	     if (!TtaReadByte (file, (unsigned char *)&charGotten))
 		error = TRUE;
-	     if (!TtaReadByte (file, &charGotten))
+	     if (!TtaReadByte (file, (unsigned char *)&charGotten))
 		error = TRUE;
 	     else
 		currentVersion = (int) charGotten;
-	     if (!TtaReadByte (file, &charGotten))
+	     if (!TtaReadByte (file, (unsigned char *)&charGotten))
 		error = TRUE;
 	  }
 	/* Gets the label max. of the document if it is present */
@@ -1091,7 +1121,7 @@ void TtaGiveSchemasOfDocument (char *documentName, char *structureName,
 	     charGotten == (char) C_PIV_LABEL))
 	  {
 	     ReadLabel (charGotten, lab, file);
-	     if (!TtaReadByte (file, &charGotten))
+	     if (!TtaReadByte (file, (unsigned char *)&charGotten))
 		error = TRUE;
 	  }
 
@@ -1101,14 +1131,14 @@ void TtaGiveSchemasOfDocument (char *documentName, char *structureName,
 	     while (charGotten == (char) C_PIV_LANG && !error)
 	       {
 		  do
-		     if (!TtaReadByte (file, &charGotten))
+		     if (!TtaReadByte (file, (unsigned char *)&charGotten))
 			error = TRUE;
 		  while (!(error || charGotten == EOS)) ;
 		  if (charGotten != EOS)
 		     error = TRUE;
 		  else
 		     /* Gets the byte following the language name */
-		  if (!TtaReadByte (file, &charGotten))
+		  if (!TtaReadByte (file, (unsigned char *)&charGotten))
 		     error = TRUE;
 	       }
 	  }
@@ -1120,7 +1150,7 @@ void TtaGiveSchemasOfDocument (char *documentName, char *structureName,
 	  {
 	     i = 0;
 	     do
-		if (!TtaReadByte (file, &structureName[i++]))
+		if (!TtaReadByte (file, (unsigned char *)&structureName[i++]))
 		   error = TRUE;
 	     while (!(error || structureName[i - 1] == EOS || i == MAX_NAME_LENGTH)) ;
 	     if (structureName[i - 1] != EOS)
@@ -1136,7 +1166,7 @@ void TtaGiveSchemasOfDocument (char *documentName, char *structureName,
 		       /* Gets the name of the associated presentation schema */
 		       i = 0;
 		       do
-			  if (!TtaReadByte (file, &presentationName[i++]))
+			  if (!TtaReadByte (file, (unsigned char *)&presentationName[i++]))
 			     error = TRUE;
 		       while (!(error || presentationName[i - 1] == EOS || i == MAX_NAME_LENGTH)) ;
 		    }
@@ -1351,29 +1381,15 @@ int TtaGetDocumentAccessMode (Document document)
 /*----------------------------------------------------------------------
    TtaGetDocumentBackUpInterval
 
-   Returns backup interval for a document.
-   Parameters:
-   document: the document whose backup interval is asked.
+   Returns backup interval of documents.
    Return value:
-   0 : the backup mechanism is disabled
-   positive integer : number of typed characters which trigger an autamatic
-   save of the document into a .BAK file.
+   0 if the backup mechanism is disabled or the number of characters
+   which trigger an automatic save into a .BAK file.
   ----------------------------------------------------------------------*/
-int TtaGetDocumentBackUpInterval (Document document)
+int TtaGetDocumentBackUpInterval ()
 {
-  int                 result;
-
   UserErrorCode = 0;
-  result = 0;
-  /* verifies the parameter document */
-  if (document < 1 || document > MAX_DOCUMENTS)
-    TtaError (ERR_invalid_document_parameter);
-  else if (LoadedDocument[document - 1] == NULL)
-    TtaError (ERR_invalid_document_parameter);
-  else
-    /* parameter document is correct */
-    result = LoadedDocument[document - 1]->DocBackUpInterval;
-  return result;
+  return DocBackUpInterval;
 }
 
 
@@ -1478,3 +1494,289 @@ PtrDocument         DocToPtr (Document document)
    return LoadedDocument[document - 1];
 }
 
+/*----------------------------------------------------------------------
+  TtaSetNamespaceDeclaration
+  Sets a namespace declaration for an element 
+ ----------------------------------------------------------------------*/
+void TtaSetANamespaceDeclaration (Document document, Element element,
+				 char *NsPrefix, char *NsUri)
+{
+  PtrDocument pDoc;
+
+  UserErrorCode = 0;
+  /* verifies the parameter document */
+  if (document < 1 || document > MAX_DOCUMENTS)
+    TtaError (ERR_invalid_document_parameter);
+  else if (LoadedDocument[document - 1] == NULL)
+    TtaError (ERR_invalid_document_parameter);
+  else
+    {
+      /* parameter document is correct */
+      pDoc = LoadedDocument[document - 1];
+      SetNamespaceDeclaration (pDoc, (PtrElement) element, NsPrefix, NsUri);
+    }
+}
+
+/*----------------------------------------------------------------------
+  TtaShowElemNamespaceDeclarations
+  Show the namespaces declarations related to an element
+ ----------------------------------------------------------------------*/
+void TtaShowNamespaceDeclarations (Document document)
+{
+  PtrDocument pDoc;
+
+  UserErrorCode = 0;
+  /* verifies the parameter document */
+  if (document < 1 || document > MAX_DOCUMENTS)
+    TtaError (ERR_invalid_document_parameter);
+  else if (LoadedDocument[document - 1] == NULL)
+    TtaError (ERR_invalid_document_parameter);
+  else
+    {
+      /* parameter document is correct */
+      pDoc = LoadedDocument[document - 1];
+      ShowNamespaceDeclarations (pDoc);
+    }
+}
+
+/*----------------------------------------------------------------------
+  TtaFreeElemNamespaceDeclarations
+  Free the namespaces declarations related to an element
+ ----------------------------------------------------------------------*/
+void TtaFreeElemNamespaceDeclarations (Document document, Element element)
+{
+  PtrDocument pDoc;
+
+  UserErrorCode = 0;
+  /* verifies the parameter document */
+  if (document < 1 || document > MAX_DOCUMENTS)
+    TtaError (ERR_invalid_document_parameter);
+  else if (LoadedDocument[document - 1] == NULL)
+    TtaError (ERR_invalid_document_parameter);
+  else
+    {
+      /* parameter document is correct */
+      pDoc = LoadedDocument[document - 1];
+      FreeElemNamespaceDeclarations (pDoc, (PtrElement) element);
+    }
+}
+
+/*----------------------------------------------------------------------
+  TtaFreeNamespaceDeclarations
+  Free all the namespaces declarations of a document
+ ----------------------------------------------------------------------*/
+void TtaFreeNamespaceDeclarations (Document document)
+{
+  PtrDocument pDoc;
+
+  UserErrorCode = 0;
+  /* verifies the parameter document */
+  if (document < 1 || document > MAX_DOCUMENTS)
+    TtaError (ERR_invalid_document_parameter);
+  else if (LoadedDocument[document - 1] == NULL)
+    TtaError (ERR_invalid_document_parameter);
+  else
+    {
+      /* parameter document is correct */
+      pDoc = LoadedDocument[document - 1];
+      FreeNamespaceDeclarations (pDoc);
+    }
+}
+
+/*----------------------------------------------------------------------
+   TtaAppendXmlAttribute
+   Add a new xml global attribute
+  ----------------------------------------------------------------------*/
+void TtaAppendXmlAttribute (char *xmlName, AttributeType *attrType, Document document)
+{
+  PtrDocument pDoc;
+
+  UserErrorCode = 0;
+  /* verifies the parameter document */
+  if (document < 1 || document > MAX_DOCUMENTS)
+    TtaError (ERR_invalid_document_parameter);
+  else if (LoadedDocument[document - 1] == NULL)
+    TtaError (ERR_invalid_document_parameter);
+  else
+    {
+      /* parameter document is correct */
+      pDoc = LoadedDocument[document - 1];
+      AppendXmlAttribute (xmlName, attrType, pDoc);
+    }
+}
+
+/*----------------------------------------------------------------------
+  TtaGetXmlAttributeType
+  Search in attrType->AttrSSchema if not NULL otherwise,
+  search in the different loaded natures.
+  ----------------------------------------------------------------------*/
+void TtaGetXmlAttributeType (char* xmlName, AttributeType *attrType, Document document)
+{
+  PtrDocument pDoc;
+
+  UserErrorCode = 0;
+  /* verifies the parameter document */
+  if (document < 1 || document > MAX_DOCUMENTS)
+    TtaError (ERR_invalid_document_parameter);
+  else if (LoadedDocument[document - 1] == NULL)
+    TtaError (ERR_invalid_document_parameter);
+  else
+    {
+      /* parameter document is correct */
+      pDoc = LoadedDocument[document - 1];
+      GetXmlAttributeType (xmlName, attrType, pDoc);
+    }
+}
+
+#ifndef NODISPLAY
+/*-----------------------------------------------------------------------------
+   TtaAddEmptyBox
+   Add the specific presentation rule Createlast(EmptyBox) to an empty element
+ -----------------------------------------------------------------------------*/
+void TtaAddEmptyBox (Element element)
+{
+  AddEmptyBox ((PtrElement) element);
+}
+
+/*----------------------------------------------------------------------
+   TtaHasXmlInLineRule
+   Retuns TRUE if the element type has a 'Line' presentation rule
+  ----------------------------------------------------------------------*/
+ThotBool TtaHasXmlInLineRule (ElementType elType, Document document)
+{
+  PtrDocument pDoc;
+
+  UserErrorCode = 0;
+  /* verifies the parameter document */
+  if (document < 1 || document > MAX_DOCUMENTS)
+    TtaError (ERR_invalid_document_parameter);
+  else if (LoadedDocument[document - 1] == NULL)
+    TtaError (ERR_invalid_document_parameter);
+  else
+    {
+      /* parameter document is correct */
+      pDoc = LoadedDocument[document - 1];
+      return HasXmlInLineRule (elType, pDoc);
+    }
+  return FALSE;
+}
+
+/*----------------------------------------------------------------------
+   TtaSetXmlInLineRule
+   Add an InLine generic rule to an element type
+  ----------------------------------------------------------------------*/
+void TtaSetXmlInLineRule (ElementType elType, Document document)
+{
+  PtrDocument pDoc;
+
+  UserErrorCode = 0;
+  /* verifies the parameter document */
+  if (document < 1 || document > MAX_DOCUMENTS)
+    TtaError (ERR_invalid_document_parameter);
+  else if (LoadedDocument[document - 1] == NULL)
+    TtaError (ERR_invalid_document_parameter);
+  else
+    {
+      /* parameter document is correct */
+      pDoc = LoadedDocument[document - 1];
+      SetXmlInLineRule (elType, pDoc);
+    }
+}
+#endif 
+
+/*----------------------------------------------------------------------
+   TtaAppendXmlElement
+   Add a new element to the schema
+  ----------------------------------------------------------------------*/
+void TtaAppendXmlElement (char *xmlName, ElementType *elType,
+			  char **mappedName, Document document)
+{
+  PtrDocument pDoc;
+
+  UserErrorCode = 0;
+  /* verifies the parameter document */
+  if (document < 1 || document > MAX_DOCUMENTS)
+    TtaError (ERR_invalid_document_parameter);
+  else if (LoadedDocument[document - 1] == NULL)
+    TtaError (ERR_invalid_document_parameter);
+  else
+    {
+      /* parameter document is correct */
+      pDoc = LoadedDocument[document - 1];
+      AppendXmlElement (xmlName, elType, mappedName, pDoc);
+    }
+}
+
+/*----------------------------------------------------------------------
+  TtaGetXmlElementType
+  Search in elType->ElSSchema if not NULL otherwise,
+  search in the different loaded natures.
+  ----------------------------------------------------------------------*/
+void TtaGetXmlElementType (char *xmlName, ElementType *elType,
+			   char **mappedName, Document document)
+{
+  PtrDocument pDoc;
+
+  UserErrorCode = 0;
+  /* verifies the parameter document */
+  if (document < 1 || document > MAX_DOCUMENTS)
+    TtaError (ERR_invalid_document_parameter);
+  else if (LoadedDocument[document - 1] == NULL)
+    TtaError (ERR_invalid_document_parameter);
+  else
+    {
+      /* parameter document is correct */
+      pDoc = LoadedDocument[document - 1];
+      GetXmlElementType (xmlName, elType, mappedName, pDoc);
+    }
+}
+
+/*----------------------------------------------------------------------
+  TtaSetUriSSchema
+  Set the namespace uri associated with that schema
+  ----------------------------------------------------------------------*/
+void TtaSetUriSSchema (SSchema sSchema, char *sSchemaUri)
+{
+  UserErrorCode = 0;
+  if (sSchema == NULL)
+    TtaError (ERR_invalid_parameter);
+  else
+    SetUriSSchema ((PtrSSchema) sSchema, sSchemaUri);
+}
+
+/*----------------------------------------------------------------------
+  TtaChangeGenericSchemaNames
+  Change the name of a generic xml schema
+  ----------------------------------------------------------------------*/
+void TtaChangeGenericSchemaNames (char *sSchemaUri, char *sSchemaName,
+				  Document document)
+{
+  PtrDocument pDoc;
+
+  UserErrorCode = 0;
+  /* verifies the parameter document */
+  if (document < 1 || document > MAX_DOCUMENTS)
+    TtaError (ERR_invalid_document_parameter);
+  else if (LoadedDocument[document - 1] == NULL)
+    TtaError (ERR_invalid_document_parameter);
+  else
+    {
+      /* parameter document is correct */
+      pDoc = LoadedDocument[document - 1];
+      ChangeGenericSchemaNames (sSchemaUri, sSchemaName, pDoc);
+    }
+}
+
+/*----------------------------------------------------------------------
+  TtaIsXmlSSchema
+  Returns yes if schema corresponds to a generic xml structure schema
+  Parameter:
+  schema: the structure schema of interest.
+  ----------------------------------------------------------------------*/
+ThotBool TtaIsXmlSSchema (SSchema schema)
+{
+  UserErrorCode = 0;
+  if (!schema)
+    TtaError (ERR_invalid_parameter);
+  return ((PtrSSchema) schema)->SsIsXml;
+}
