@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -8,7 +8,7 @@
 /*
    This module loads a structure schema from a .STR file
  */
-
+#include "thot_gui.h"
 #include "thot_sys.h"
 #include "libmsg.h"
 #include "message.h"
@@ -17,11 +17,10 @@
 #include "fileaccess.h"
 
 /* variables importees */
+#undef THOT_EXPORT
 #define THOT_EXPORT extern
-#include "platform_tv.h"
-
-
 #include "fileaccess_f.h"
+#include "platform_tv.h"
 #include "readstr_f.h"
 #include "memory_f.h"
 
@@ -33,7 +32,7 @@ static ThotBool ReadAttribType (BinFile file, AttribType *attrType)
 {
    char c;
 
-   TtaReadByte (file, &c);
+   TtaReadByte (file, (unsigned char *)&c);
    switch (c)
 	 {
 	    case C_INT_ATTR:
@@ -64,7 +63,7 @@ static ThotBool ReadRConstruct (BinFile file, RConstruct *constr)
 {
    char c;
 
-   TtaReadByte (file, &c);
+   TtaReadByte (file, (unsigned char *)&c);
    switch (c)
 	 {
 	    case C_IDENTITY_CONSTR:
@@ -124,7 +123,7 @@ static ThotBool ReadBasicType (BinFile file, BasicType *typ)
 {
    char c;
 
-   TtaReadByte (file, &c);
+   TtaReadByte (file, (unsigned char *)&c);
    switch (c)
 	 {
 	    case C_CHAR_STRING:
@@ -137,7 +136,7 @@ static ThotBool ReadBasicType (BinFile file, BasicType *typ)
 	       *typ = Symbol;
 	       break;
 	    case C_PICTURE:
-	       *typ = Picture;
+	       *typ = tt_Picture;
 	       break;
 	    case C_REFER:
 	       *typ = Refer;
@@ -145,8 +144,8 @@ static ThotBool ReadBasicType (BinFile file, BasicType *typ)
 	    case C_PAGE_BREAK:
 	       *typ = PageBreak;
 	       break;
-	    case C_UNUSED:
-	       *typ = UnusedBasicType;
+	    case C_ANY_TYPE:
+	       *typ = AnyType;
 	       break;
 	    default:
 	       *typ = CharString;
@@ -158,13 +157,15 @@ static ThotBool ReadBasicType (BinFile file, BasicType *typ)
 /*----------------------------------------------------------------------
    ReadAttribute                                			
   ----------------------------------------------------------------------*/
-static ThotBool     ReadAttribute (BinFile file, TtAttribute *pAttr)
+static ThotBool     ReadAttribute (BinFile file, PtrTtAttribute pAttr)
 {
    AttribType          attrType;
    int                 j;
+   Name                buffer;
 
-   TtaReadName (file, pAttr->AttrName);
-   strcpy (pAttr->AttrOrigName, pAttr->AttrName);
+   TtaReadName (file, (unsigned char *)buffer);
+   pAttr->AttrName = TtaStrdup (buffer);
+   pAttr->AttrOrigName = TtaStrdup (buffer);
    TtaReadBool (file, &pAttr->AttrGlobal);
    TtaReadShort (file, &pAttr->AttrFirstExcept);
    TtaReadShort (file, &pAttr->AttrLastExcept);
@@ -178,13 +179,13 @@ static ThotBool     ReadAttribute (BinFile file, TtAttribute *pAttr)
 		    break;
 		 case AtReferenceAttr:
 		    TtaReadShort (file, &pAttr->AttrTypeRef);
-		    TtaReadName (file, pAttr->AttrTypeRefNature);
+		    TtaReadName (file, (unsigned char *)pAttr->AttrTypeRefNature);
 		    break;
 		 case AtEnumAttr:
 		    TtaReadShort (file, &pAttr->AttrNEnumValues);
 		    for (j = 0; j < pAttr->AttrNEnumValues; j++)
 		      {
-			TtaReadName (file, pAttr->AttrEnumValue[j]);
+			TtaReadName (file, (unsigned char *)pAttr->AttrEnumValue[j]);
 			strcpy (pAttr->AttrEnumOrigValue[j], pAttr->AttrEnumValue[j]);
 		      }
 		    break;
@@ -198,13 +199,15 @@ static ThotBool     ReadAttribute (BinFile file, TtAttribute *pAttr)
 /*----------------------------------------------------------------------
    ReadSRule                                  			
   ----------------------------------------------------------------------*/
-static ThotBool     ReadSRule (BinFile file, SRule *pSRule)
+static ThotBool     ReadSRule (BinFile file, PtrSRule pSRule)
 {
    RConstruct          constr;
    int                 j;
+   Name                buffer;
 
-   TtaReadName (file, pSRule->SrName);
-   strcpy (pSRule->SrOrigName, pSRule->SrName);
+   TtaReadName (file, (unsigned char *)buffer);
+   pSRule->SrName = TtaStrdup (buffer); 
+   pSRule->SrOrigName = TtaStrdup (buffer); 
    TtaReadShort (file, &pSRule->SrNDefAttrs);
    for (j = 0; j < pSRule->SrNDefAttrs; j++)
       TtaReadShort (file, &pSRule->SrDefAttr[j]);
@@ -214,10 +217,20 @@ static ThotBool     ReadSRule (BinFile file, SRule *pSRule)
       TtaReadBool (file, &pSRule->SrDefAttrModif[j]);
 
    TtaReadShort (file, &pSRule->SrNLocalAttrs);
-   for (j = 0; j < pSRule->SrNLocalAttrs; j++)
-      TtaReadShort (file, &pSRule->SrLocalAttr[j]);
-   for (j = 0; j < pSRule->SrNLocalAttrs; j++)
-      TtaReadBool (file, &pSRule->SrRequiredAttr[j]);
+   if (pSRule->SrNLocalAttrs <= 0)
+     {
+       pSRule->SrLocalAttr = NULL;
+       pSRule->SrRequiredAttr = NULL;
+     }
+   else
+     {
+       pSRule->SrLocalAttr = (NumTable*) TtaGetMemory (pSRule->SrNLocalAttrs * sizeof (int));
+       for (j = 0; j < pSRule->SrNLocalAttrs; j++)
+	 TtaReadShort (file, &pSRule->SrLocalAttr->Num[j]);
+       pSRule->SrRequiredAttr = (BoolTable*) TtaGetMemory (pSRule->SrNLocalAttrs * sizeof (ThotBool));
+       for (j = 0; j < pSRule->SrNLocalAttrs; j++)
+	 TtaReadBool (file, &pSRule->SrRequiredAttr->Bln[j]);
+     }
 
    TtaReadBool (file, &pSRule->SrUnitElem);
    TtaReadBool (file, &pSRule->SrRecursive);
@@ -227,7 +240,7 @@ static ThotBool     ReadSRule (BinFile file, SRule *pSRule)
    if (pSRule->SrExportedElem)
      {
 	TtaReadShort (file, &pSRule->SrExportContent);
-	TtaReadName (file, pSRule->SrNatExpContent);
+	TtaReadName (file, (unsigned char *)pSRule->SrNatExpContent);
      }
 
    TtaReadShort (file, &pSRule->SrFirstExcept);
@@ -257,7 +270,7 @@ static ThotBool     ReadSRule (BinFile file, SRule *pSRule)
 	       break;
 	    case CsReference:
 	       TtaReadShort (file, &pSRule->SrReferredType);
-	       TtaReadName (file, pSRule->SrRefTypeNat);
+	       TtaReadName (file, (unsigned char *)pSRule->SrRefTypeNat);
 	       break;
 	    case CsIdentity:
 	       TtaReadShort (file, &pSRule->SrIdentRule);
@@ -308,11 +321,11 @@ static ThotBool ReadConstants (BinFile file, PtrSSchema pSS)
      {
 	do
 	  {
-	     TtaReadByte (file, &c);
+	     TtaReadByte (file, (unsigned char *)&c);
 	     pSS->SsConstBuffer[i++] = c;
 	  }
 	while (c != EOS && i < MAX_LEN_ALL_CONST);
-	TtaReadByte (file, &c);
+	TtaReadByte (file, (unsigned char *)&c);
 	pSS->SsConstBuffer[i++] = c;
      }
    while (c != EOS && i < MAX_LEN_ALL_CONST);
@@ -335,7 +348,7 @@ ThotBool ReadStructureSchema (Name fileName, PtrSSchema pSS)
    BinFile             file;
    PathBuffer          buf;
    PathBuffer          dirBuffer;
-   int                 i;
+   int                 i, num;
 
    /* compose le nom du fichier a ouvrir */
    strncpy (dirBuffer, SchemaPath, MAX_PATH);
@@ -356,13 +369,14 @@ ThotBool ReadStructureSchema (Name fileName, PtrSSchema pSS)
      {
 	pSS->SsActionList = NULL;
 	/* lit la partie fixe du schema de structure */
-	TtaReadName (file, pSS->SsName);
+	TtaReadName (file, (unsigned char *)pSS->SsName);
 	TtaReadShort (file, &pSS->SsCode);
-	TtaReadName (file, pSS->SsDefaultPSchema);
+	TtaReadName (file, (unsigned char *)pSS->SsDefaultPSchema);
 	TtaReadBool (file, &pSS->SsExtension);
 	pSS->SsUriName = NULL;
 	pSS->SsNExtensRules = 0;
 	pSS->SsExtensBlock = NULL;
+	pSS->SsIsXml = FALSE;
 	TtaReadShort (file, &pSS->SsDocument);
 	TtaReadShort (file, &pSS->SsRootElem);
 	TtaReadShort (file, &pSS->SsNAttributes);
@@ -385,23 +399,52 @@ ThotBool ReadStructureSchema (Name fileName, PtrSSchema pSS)
 	/* il n'y a pas encore de nature chargee dynamiquement */
 	pSS->SsFirstDynNature = 0;
 
+	/* allocate the attribute table */
+	if (!pSS->SsAttribute)
+	  {
+	    num = pSS->SsNAttributes;         /* table size */
+	    pSS->SsAttribute = (TtAttrTable*) malloc (num * sizeof (PtrTtAttribute));
+	    pSS->SsAttrTableSize = num;
+	    for (i = 0; i < num; i++)
+	      pSS->SsAttribute->TtAttr[i] = NULL;
+	  }
+
 	/* lit les attributs */
 	for (i = 0; i < pSS->SsNAttributes; i++)
-	   if (!ReadAttribute (file, &pSS->SsAttribute[i]))
-	     {
+	  {
+	    pSS->SsAttribute->TtAttr[i] = (PtrTtAttribute) malloc (sizeof (TtAttribute));
+	    if (!ReadAttribute (file, pSS->SsAttribute->TtAttr[i]))
+	      {
 		/* message 'Fichier .STR incorrect ' */
-		TtaDisplayMessage (INFO, TtaGetMessage (LIB, TMSG_INCORRECT_STR_FILE), fileName);
+		TtaDisplayMessage (INFO, TtaGetMessage (LIB,
+					   TMSG_INCORRECT_STR_FILE), fileName);
 		return FALSE;
-	     }
+	      }
+	  }
+
+	/* allocate the element table */
+	if (!pSS->SsRule)
+	  {
+	    /* reserve 2 additional entries for counter aliases (see function
+	       MakeAliasTypeCount in presvariables.c) */
+	    num = pSS->SsNRules + 2;         /* table size */
+	    pSS->SsRule = (SrRuleTable*) malloc (num * sizeof (PtrSRule));
+	    pSS->SsRuleTableSize = num;
+	    for (i = 0; i < num; i++)
+	      pSS->SsRule->SrElem[i] = NULL;
+	  }
 
 	/* lit les elements */
 	for (i = 0; i < pSS->SsNRules; i++)
-	   if (!ReadSRule (file, &pSS->SsRule[i]))
-	     {
+	  {
+	    pSS->SsRule->SrElem[i] = (PtrSRule) malloc (sizeof (SRule));
+	    if (!ReadSRule (file, pSS->SsRule->SrElem[i]))
+	      {
 		/* message 'Fichier .STR incorrect ' */
 		TtaDisplayMessage (INFO, TtaGetMessage (LIB, TMSG_INCORRECT_STR_FILE), fileName);
 		return FALSE;
-	     }
+	      }
+	  }
 	if (pSS->SsExtension)
 	  {
 	     if (TtaReadShort (file, &pSS->SsNExtensRules))
