@@ -1,23 +1,14 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996.
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
 
-/*
- * Warning:
- * This module is part of the Thot library, which was originally
- * developed in French. That's why some comments are still in
- * French, but their translation is in progress and the full module
- * will be available in English in the next release.
- * 
- */
- 
 /*
    Ce module charge un schema de traduction depuis un fichier .TRA
  */
-
+#include "thot_gui.h"
 #include "thot_sys.h"
 #include "libmsg.h"
 #include "message.h"
@@ -40,14 +31,9 @@ static ThotBool     error = FALSE;
 /*----------------------------------------------------------------------
    TSchemaError positionne error dans le cas d'une erreur de lecture
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                TSchemaError (int err)
-#else  /* __STDC__ */
-void                TSchemaError (err)
-int                 err;
-#endif /* __STDC__ */
+void TSchemaError (int err)
 {
-   TtaDisplayMessage (INFO, TEXT("Error %d in translation schema\n"), err);
+   TtaDisplayMessage (INFO, "Error %d in translation schema\n", err);
    error = TRUE;
 }
 
@@ -56,18 +42,13 @@ int                 err;
    ReadTransCondition lit un type de condition d'application de	
    regle de traduction.					
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static TransCondition ReadTransCondition (BinFile file)
-#else  /* __STDC__ */
-static TransCondition ReadTransCondition (file)
-BinFile             file;
-#endif /* __STDC__ */
 {
-   CHAR_T                c;
+   char                c;
    TransCondition      cond;
 
    cond = TcondFirst;
-   if (!TtaReadByte (file, &c))
+   if (!TtaReadByte (file, (unsigned char *)&c))
       TSchemaError (1);
    else
       switch (c)
@@ -78,9 +59,6 @@ BinFile             file;
 	       case C_TR_LAST:
 		  cond = TcondLast;
 		  break;
-	       case C_TR_DEFINED:
-		  cond = TcondDefined;
-		  break;
 	       case C_TR_REFERRED:
 		  cond = TcondReferred;
 		  break;
@@ -105,11 +83,8 @@ BinFile             file;
 	       case C_TR_PRULE:
 		  cond = TcondPRule;
 		  break;
-	       case C_TR_COMMENT:
-		  cond = TcondComment;
-		  break;
 	       case C_TR_ALPHABET:
-		  cond = TcondAlphabet;
+		  cond = TcondScript;
 		  break;
 	       case C_TR_ATTRIBUTES:
 		  cond = TcondAttributes;
@@ -153,18 +128,13 @@ BinFile             file;
 /*----------------------------------------------------------------------
    ReadTRuleType lit un type de regle de traduction.             
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static TRuleType    ReadTRuleType (BinFile file)
-#else  /* __STDC__ */
-static TRuleType    ReadTRuleType (file)
-BinFile             file;
-#endif /* __STDC__ */
 {
-   CHAR_T                c;
+   char                c;
    TRuleType           ruleType;
 
    ruleType = TRemove;
-   if (!TtaReadByte (file, &c))
+   if (!TtaReadByte (file, (unsigned char *)&c))
       TSchemaError (3);
    else
       switch (c)
@@ -181,6 +151,9 @@ BinFile             file;
 	       case C_TR_REMOVE:
 		  ruleType = TRemove;
 		  break;
+	       case C_TR_IGNORE:
+		  ruleType = TIgnore;
+		  break;
 	       case C_TR_WRITE:
 		  ruleType = TWrite;
 		  break;
@@ -227,18 +200,13 @@ BinFile             file;
 /*----------------------------------------------------------------------
    ReadRelatNAscend                                                
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static RelatNAscend ReadRelatNAscend (BinFile file)
-#else  /* __STDC__ */
-static RelatNAscend ReadRelatNAscend (file)
-BinFile             file;
-#endif /* __STDC__ */
 {
-   CHAR_T                c;
+   char                c;
    RelatNAscend        relat;
 
    relat = RelEquals;		/* valeur par defaut */
-   if (!TtaReadByte (file, &c))
+   if (!TtaReadByte (file, (unsigned char *)&c))
       TSchemaError (3);
    else
       switch (c)
@@ -261,18 +229,13 @@ BinFile             file;
    ReadTOrder lit la position ou` il faut creer les chaines	
    produites par le traducteur.                            
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static TOrder       ReadTOrder (BinFile file)
-#else  /* __STDC__ */
-static TOrder       ReadTOrder (file)
-BinFile             file;
-#endif /* __STDC__ */
 {
-   CHAR_T                c;
+   char                c;
    TOrder              order;
 
    order = TAfter;
-   if (!TtaReadByte (file, &c))
+   if (!TtaReadByte (file, (unsigned char *)&c))
       TSchemaError (5);
    else
       switch (c)
@@ -297,18 +260,13 @@ BinFile             file;
    ReadCreatedObject lit le type de chose a` creer dans le fichier 
    de sortie.                                  		
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static CreatedObject ReadCreatedObject (BinFile file)
-#else  /* __STDC__ */
-static CreatedObject ReadCreatedObject (file)
-BinFile             file;
-#endif /* __STDC__ */
 {
-   CHAR_T                c;
+   char                c;
    CreatedObject       obj;
 
    obj = ToConst;
-   if (!TtaReadByte (file, &c))
+   if (!TtaReadByte (file, (unsigned char *)&c))
       TSchemaError (7);
    else
       switch (c)
@@ -328,9 +286,6 @@ BinFile             file;
 	       case C_OB_CONTENT:
 		  obj = ToContent;
 		  break;
-	       case C_OB_COMMENT:
-		  obj = ToComment;
-		  break;
 	       case C_OB_ATTRIBUTES:
 		  obj = ToAllAttr;
 		  break;
@@ -389,22 +344,13 @@ BinFile             file;
 /*----------------------------------------------------------------------
    ReadTRelatPosition lit la position relative d'un element a` prendre 
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static TRelatPosition ReadTRelatPosition (BinFile file)
-
-#else  /* __STDC__ */
-static TRelatPosition ReadTRelatPosition (file)
-BinFile             file;
-
-#endif /* __STDC__ */
-
 {
-   CHAR_T                c;
+   char                c;
    TRelatPosition      position;
 
    position = RpSibling;
-   if (!TtaReadByte (file, &c))
+   if (!TtaReadByte (file, (unsigned char *)&c))
       TSchemaError (9);
    else
       switch (c)
@@ -418,9 +364,6 @@ BinFile             file;
 	       case C_TR_REFERRED_EL:
 		  position = RpReferred;
 		  break;
-	       case C_TR_ASSOC:
-		  position = RpAssoc;
-		  break;
 	       default:
 		  TSchemaError (10);
 		  position = RpSibling;
@@ -433,22 +376,13 @@ BinFile             file;
 /*----------------------------------------------------------------------
    ReadIndentType	lit un type de regle d'indentation
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static TIndentType ReadIndentType (BinFile file)
-
-#else  /* __STDC__ */
-static TIndentType ReadIndentType (file)
-BinFile             file;
-
-#endif /* __STDC__ */
-
 {
-   CHAR_T                c;
+   char                c;
    TIndentType		 typ;
 
    typ = ItAbsolute;
-   if (!TtaReadByte (file, &c))
+   if (!TtaReadByte (file, (unsigned char *)&c))
       TSchemaError (23);
    else
       switch (c)
@@ -476,19 +410,13 @@ BinFile             file;
 /*----------------------------------------------------------------------
    ReadTCounterOp lit le type< d'une operation sur un compteur.    
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static TCounterOp   ReadTCounterOp (BinFile file)
-#else  /* __STDC__ */
-static TCounterOp   ReadTCounterOp (file)
-BinFile             file;
-#endif /* __STDC__ */
-
 {
-   CHAR_T                c;
+   char                c;
    TCounterOp          op;
 
    op = TCntrNoOp;
-   if (!TtaReadByte (file, &c))
+   if (!TtaReadByte (file, (unsigned char *)&c))
       TSchemaError (11);
    else
       switch (c)
@@ -518,18 +446,13 @@ BinFile             file;
    ReadTranslVarType lit le type des elements de variables de      
    traduction.                                             
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static TranslVarType ReadTranslVarType (BinFile file)
-#else  /* __STDC__ */
-static TranslVarType ReadTranslVarType (file)
-BinFile             file;
-#endif /* __STDC__ */
 {
-   CHAR_T                c;
+   char                c;
    TranslVarType       varType;
 
    varType = VtText;
-   if (!TtaReadByte (file, &c))
+   if (!TtaReadByte (file, (unsigned char *)&c))
       TSchemaError (13);
    else
       switch (c)
@@ -575,18 +498,13 @@ BinFile             file;
    ReadCounterStyle  lit un style de compteur dans le fichier et   
    retourne sa valeur.                                     
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static CounterStyle ReadCounterStyle (BinFile file)
-#else  /* __STDC__ */
-static CounterStyle ReadCounterStyle (file)
-BinFile             file;
-#endif /* __STDC__ */
 {
-   CHAR_T                c;
+   char                c;
    CounterStyle        style;
 
 
-   if (!TtaReadByte (file, &c))
+   if (!TtaReadByte (file, (unsigned char *)&c))
      {
 	c = SPACE;
 	TSchemaError (15);
@@ -620,18 +538,12 @@ BinFile             file;
    ReadPtrTRule retourne un pointeur sur la regle suivante ou NULL   
    s'il n'y a pas de regle suivante.                       
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static PtrTRule     ReadPtrTRule (BinFile file, PtrTRule * pNextTRule)
-#else  /* __STDC__ */
-static PtrTRule     ReadPtrTRule (file, pNextTRule)
-BinFile             file;
-PtrTRule           *pNextTRule;
-#endif /* __STDC__ */
+static PtrTRule ReadPtrTRule (BinFile file, PtrTRule *pNextTRule)
 {
-   CHAR_T                c;
+   char                c;
    PtrTRule            pTRule;
 
-   if (!TtaReadByte (file, &c))
+   if (!TtaReadByte (file, (unsigned char *)&c))
       TSchemaError (17);
    if (c == EOS)
       pTRule = NULL;
@@ -645,14 +557,7 @@ PtrTRule           *pNextTRule;
    ReadTRules ecrit une suite de regles chainees et fait pointer	
    pFirstTRule sur la premiere regle de la suite de regles lues.   
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void         ReadTRules (BinFile file, PtrTRule * pFirstTRule, PtrTRule * pNextTRule)
-#else  /* __STDC__ */
-static void         ReadTRules (file, pFirstTRule, pNextTRule)
-BinFile             file;
-PtrTRule           *pFirstTRule;
-PtrTRule           *pNextTRule;
-#endif /* __STDC__ */
+static void ReadTRules (BinFile file, PtrTRule *pFirstTRule, PtrTRule *pNextTRule)
 {
    PtrTRule            pTRule;
 
@@ -677,21 +582,22 @@ PtrTRule           *pNextTRule;
 		       case TWrite:
 			  pTRule->TrObject = ReadCreatedObject (file);
 			  TtaReadShort (file, &pTRule->TrObjectNum);
-			  TtaReadName (file, (STRING) &pTRule->TrObjectNature);
+			  TtaReadName (file, (unsigned char *)pTRule->TrObjectNature);
 			  TtaReadBool (file, &pTRule->TrReferredObj);
 			  TtaReadShort (file, &pTRule->TrFileNameVar);
 			  break;
 		       case TGet:
 		       case TCopy:
 			  TtaReadShort (file, &pTRule->TrElemType);
-			  TtaReadName (file, (STRING) &pTRule->TrElemNature);
+			  TtaReadName (file, (unsigned char *)pTRule->TrElemNature);
 			  pTRule->TrRelPosition = ReadTRelatPosition (file);
 			  break;
 		       case TUse:
-			  TtaReadName (file, (STRING) &pTRule->TrNature);
-			  TtaReadName (file, (STRING) &pTRule->TrTranslSchemaName);
+			  TtaReadName (file, (unsigned char *)pTRule->TrNature);
+			  TtaReadName (file, (unsigned char *)pTRule->TrTranslSchemaName);
 			  break;
 		       case TRemove:
+		       case TIgnore:
 		       case TNoTranslation:
 		       case TNoLineBreak:
 
@@ -732,17 +638,11 @@ PtrTRule           *pNextTRule;
    ReadPtrTRuleBlock retourne un pointeur sur le bloc de regles	
    	suivant ou NULL s'il n'y a pas de bloc suivant.                 
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static PtrTRuleBlock ReadPtrTRuleBlock (BinFile file, PtrTRuleBlock * pNextBlock)
-#else  /* __STDC__ */
-static PtrTRuleBlock ReadPtrTRuleBlock (file, pNextBlock)
-BinFile             file;
-PtrTRuleBlock      *pNextBlock;
-#endif /* __STDC__ */
+static PtrTRuleBlock ReadPtrTRuleBlock (BinFile file, PtrTRuleBlock *pNextBlock)
 {
-   CHAR_T                c;
+   char c;
 
-   TtaReadByte (file, &c);
+   TtaReadByte (file, (unsigned char *)&c);
    if (c == EOS)
       return NULL;
    else
@@ -753,18 +653,8 @@ PtrTRuleBlock      *pNextBlock;
 /*----------------------------------------------------------------------
    ReadBlocks lit une suite de blocs de regles.                    
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void         ReadBlocks (BinFile file, PtrTRuleBlock * pBlock, PtrTRule * pNextTRule, PtrSSchema * pSS, PtrTRuleBlock * pNextBlock)
-#else  /* __STDC__ */
-static void         ReadBlocks (file, pBlock, pNextTRule, pSS, pNextBlock)
-BinFile             file;
-PtrTRuleBlock      *pBlock;
-PtrTRule           *pNextTRule;
-PtrSSchema         *pSS;
-PtrTRuleBlock      *pNextBlock;
-
-#endif /* __STDC__ */
-
+static void ReadBlocks (BinFile file, PtrTRuleBlock *pBlock, PtrTRule *pNextTRule,
+			PtrSSchema *pSS, PtrTRuleBlock *pNextBlock)
 {
    PtrTRuleBlock       pBl;
    TranslCondition    *pCond;
@@ -791,17 +681,17 @@ PtrTRuleBlock      *pNextBlock;
 		  TtaReadBool (file, &pCond->TcNegativeCond);
 		  TtaReadBool (file, &pCond->TcTarget);
 		  TtaReadShort (file, &pCond->TcAscendType);
-		  TtaReadName (file, (STRING) &pCond->TcAscendNature);
+		  TtaReadName (file, (unsigned char *)pCond->TcAscendNature);
 		  TtaReadSignedShort (file, &pCond->TcAscendRelLevel);
 		  switch (pCond->TcCondition)
 			{
-			   case TcondAlphabet:
-			      TtaReadByte (file, &pCond->TcAlphabet);
+			   case TcondScript:
+			      TtaReadByte (file, (unsigned char *)&pCond->TcScript);
 			      break;
 			   case TcondWithin:
 			   case TcondFirstWithin:
 			      TtaReadShort (file, &pCond->TcElemType);
-			      TtaReadName (file, (STRING) &pCond->TcElemNature);
+			      TtaReadName (file, (unsigned char *)pCond->TcElemNature);
 			      TtaReadBool (file, &pCond->TcImmediatelyWithin);
 			      pCond->TcAscendRel = ReadRelatNAscend (file);
 			      TtaReadShort (file, &pCond->TcAscendLevel);
@@ -809,14 +699,14 @@ PtrTRuleBlock      *pNextBlock;
 			   case TcondAttr:
 			      TtaReadShort (file, &pCond->TcAttr);
 			      if (!error)
-				 switch ((*pSS)->SsAttribute[pCond->TcAttr - 1].AttrType)
+				 switch ((*pSS)->SsAttribute->TtAttr[pCond->TcAttr - 1]->AttrType)
 				       {
 					  case AtNumAttr:
 					     TtaReadSignedShort (file, &pCond->TcLowerBound);
 					     TtaReadSignedShort (file, &pCond->TcUpperBound);
 					     break;
 					  case AtTextAttr:
-					     TtaReadName (file, (STRING) &pCond->TcTextValue);
+					     TtaReadName (file, (unsigned char *)pCond->TcTextValue);
 					     break;
 					  case AtReferenceAttr:
 
@@ -832,19 +722,24 @@ PtrTRuleBlock      *pNextBlock;
 			   case TcondPRule:
 			      TtaReadShort (file, &pCond->TcAttr);
 			      if (!error)
+				{
 				 if (pCond->TcAttr == PtSize ||
 				     pCond->TcAttr == PtIndent ||
 				     pCond->TcAttr == PtLineSpacing ||
 				     pCond->TcAttr == PtLineWeight ||
 				     pCond->TcAttr == PtFillPattern ||
+				     pCond->TcAttr == PtOpacity ||
+				     pCond->TcAttr == PtFillOpacity ||
+				     pCond->TcAttr == PtStrokeOpacity ||
 				     pCond->TcAttr == PtBackground ||
 				     pCond->TcAttr == PtForeground)
-				   {
+				     {
 				      TtaReadSignedShort (file, &pCond->TcLowerBound);
 				      TtaReadSignedShort (file, &pCond->TcUpperBound);
-				   }
+				     }
 				 else
-				    TtaReadByte (file, &pCond->TcPresValue);
+				    TtaReadByte (file, (unsigned char *)&pCond->TcPresValue);
+				}
 			      break;
 			   case TcondElementType:
 			      TtaReadShort (file, &pCond->TcAttr);
@@ -867,12 +762,7 @@ PtrTRuleBlock      *pNextBlock;
    FreeBlocks     libere une suite de blocs de regles ainsi que    
    les regles attachees.                                   
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void         FreeBlocks (PtrTRuleBlock pBlock)
-#else  /* __STDC__ */
-static void         FreeBlocks (pBlock)
-PtrTRuleBlock       pBlock;
-#endif /* __STDC__ */
+static void FreeBlocks (PtrTRuleBlock pBlock)
 {
    PtrTRuleBlock       pNextBlock;
    PtrTRule            pTRule, nextRule;
@@ -898,27 +788,19 @@ PtrTRuleBlock       pBlock;
    ReadAttrTRules lit les regles de traduction de l'attribut de    
    numero att appartenant au schema de structure pSS.      
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void         ReadAttrTRules (BinFile file, int att, PtrTRuleBlock * pNextBlock, PtrTRule * pNextTRule, PtrSSchema * pSS, PtrTSchema * pTSch)
-#else  /* __STDC__ */
-static void         ReadAttrTRules (file, att, pNextBlock, pNextTRule, pSS, pTSch)
-BinFile             file;
-int                 att;
-PtrTRuleBlock      *pNextBlock;
-PtrTRule           *pNextTRule;
-PtrSSchema         *pSS;
-PtrTSchema         *pTSch;
-#endif /* __STDC__ */
+static void ReadAttrTRules (BinFile file, int att, PtrTRuleBlock *pNextBlock,
+			    PtrTRule *pNextTRule, PtrSSchema *pSS,
+			    PtrTSchema *pTSch)
 {
-   AttributeTransl    *pAttrT;
+   PtrAttributeTransl  pAttrT;
    TranslNumAttrCase  *pCase;
    int                 i;
 
    if (!error)
      {
-	pAttrT = &(*pTSch)->TsAttrTRule[att];
+	pAttrT = (*pTSch)->TsAttrTRule->TsAttrTransl[att];
 	TtaReadShort (file, &pAttrT->AtrElemType);
-	switch ((*pSS)->SsAttribute[att].AttrType)
+	switch ((*pSS)->SsAttribute->TtAttr[att]->AttrType)
 	      {
 		 case AtNumAttr:
 		    TtaReadShort (file, &pAttrT->AtrNCases);
@@ -933,7 +815,7 @@ PtrTSchema         *pTSch;
 			 }
 		    break;
 		 case AtTextAttr:
-		    TtaReadName (file, (STRING) &pAttrT->AtrTextValue);
+		    TtaReadName (file, (unsigned char *)pAttrT->AtrTextValue);
 		    ReadBlocks (file, &pAttrT->AtrTxtTRuleBlock, pNextTRule,
 				pSS, pNextBlock);
 		    break;
@@ -942,7 +824,7 @@ PtrTSchema         *pTSch;
 				pSS, pNextBlock);
 		    break;
 		 case AtEnumAttr:
-		    for (i = 0; i <= (*pSS)->SsAttribute[att].AttrNEnumValues; i++)
+		    for (i = 0; i <= (*pSS)->SsAttribute->TtAttr[att]->AttrNEnumValues; i++)
 		       ReadBlocks (file, &pAttrT->AtrEnuTRuleBlock[i],
 				   pNextTRule, pSS, pNextBlock);
 		    break;
@@ -955,13 +837,7 @@ PtrTSchema         *pTSch;
 /*----------------------------------------------------------------------
    FreeTRulesAttr libere les regles de traduction d'un attribut.   
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void         FreeTRulesAttr (AttributeTransl * pAttrT, TtAttribute * pAttr)
-#else  /* __STDC__ */
-static void         FreeTRulesAttr (pAttrT, pAttr)
-AttributeTransl    *pAttrT;
-TtAttribute        *pAttr;
-#endif /* __STDC__ */
+static void FreeTRulesAttr (PtrAttributeTransl pAttrT, PtrTtAttribute pAttr)
 {
    int                 i;
 
@@ -990,17 +866,9 @@ TtAttribute        *pAttr;
    ReadPresTRules lit les regles de traduction de la presentation  
    de numero pres appartenant au schema de structure pSS.  
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void         ReadPresTRules (BinFile file, int pres, PtrTRuleBlock * pNextBlock, PtrTRule * pNextTRule, PtrSSchema * pSS, PtrTSchema * pTSch)
-#else  /* __STDC__ */
-static void         ReadPresTRules (file, pres, pNextBlock, pNextTRule, pSS, pTSch)
-BinFile             file;
-int                 pres;
-PtrTRuleBlock      *pNextBlock;
-PtrTRule           *pNextTRule;
-PtrSSchema         *pSS;
-PtrTSchema         *pTSch;
-#endif /* __STDC__ */
+static void ReadPresTRules (BinFile file, int pres, PtrTRuleBlock *pNextBlock,
+			    PtrTRule *pNextTRule, PtrSSchema *pSS,
+			    PtrTSchema *pTSch)
 {
    PRuleTransl        *pPruleTr;
    TranslNumAttrCase  *pCase;
@@ -1010,6 +878,7 @@ PtrTSchema         *pTSch;
      {
 	pPruleTr = &(*pTSch)->TsPresTRule[pres];
 	if (pPruleTr->RtExist)
+	  {
 	   if (pres == PtSize || pres == PtIndent ||
 	       pres == PtLineSpacing || pres == PtLineWeight ||
 	       pres == PtFillPattern || pres == PtBackground ||
@@ -1030,10 +899,11 @@ PtrTSchema         *pTSch;
 	   else
 	      for (i = 0; i <= MAX_TRANSL_PRES_VAL; i++)
 		{
-		   TtaReadByte (file, &pPruleTr->RtPRuleValue[i]);
+		   TtaReadByte (file, (unsigned char *)&pPruleTr->RtPRuleValue[i]);
 		   ReadBlocks (file, &pPruleTr->RtPRuleValueBlock[i],
 			       pNextTRule, pSS, pNextBlock);
 		}
+	  }
      }
 }
 
@@ -1042,17 +912,12 @@ PtrTSchema         *pTSch;
    FreeTRulesPres libere les regles de traduction d'une regle de	
    presentation                                            
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void         FreeTRulesPres (int pres, PRuleTransl * pPruleTr)
-#else  /* __STDC__ */
-static void         FreeTRulesPres (pres, pPruleTr)
-int                 pres;
-PRuleTransl        *pPruleTr;
-#endif /* __STDC__ */
+static void FreeTRulesPres (int pres, PRuleTransl *pPruleTr)
 {
    int                 i;
 
    if (pPruleTr->RtExist)
+     {
       if (pres == PtSize + 1 || pres == PtIndent + 1 ||
 	  pres == PtLineSpacing + 1 || pres == PtLineWeight + 1 ||
 	  pres == PtFillPattern + 1 || pres == PtBackground + 1 ||
@@ -1063,6 +928,7 @@ PRuleTransl        *pPruleTr;
       else
 	 for (i = 0; i <= MAX_TRANSL_PRES_VAL; i++)
 	    FreeBlocks (pPruleTr->RtPRuleValueBlock[i]);
+     }
 }
 
 
@@ -1075,13 +941,7 @@ PRuleTransl        *pPruleTr;
    - fileName: nom du fichier a lire, sans le suffixe .TRA 
    - pSS: schema de structure correspondant, deja rempli.  
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-PtrTSchema          ReadTranslationSchema (Name fileName, PtrSSchema pSS)
-#else  /* __STDC__ */
-PtrTSchema          ReadTranslationSchema (fileName, pSS)
-Name                fileName;
-PtrSSchema          pSS;
-#endif /* __STDC__ */
+PtrTSchema ReadTranslationSchema (Name fileName, PtrSSchema pSS)
 {
    BinFile             file;
    PtrTRule            pNextTRule;
@@ -1090,27 +950,28 @@ PtrSSchema          pSS;
    TCounter           *pCntr;
    TranslVariable     *pVar;
    TranslVarItem      *pVarItem;
-   AttributeTransl    *pAttrTr;
-   AlphabetTransl     *pAlphTr;
+   PtrAttributeTransl  pAttrTr;
+   ScriptTransl     *pAlphTr;
    StringTransl       *pStringTr;
    PRuleTransl        *pPRuleTr;
    PathBuffer          dirBuffer;
-   CHAR_T                buf[MAX_TXT_LEN];
-   int                 InitialNElems, i, j;
+   char                buf[MAX_TXT_LEN];
+   int                 InitialNElems, i, j, size;
 
    error = FALSE;
    pTSch = NULL;
    /* compose le nom du fichier a ouvrir */
-   ustrncpy (dirBuffer, SchemaPath, MAX_PATH);
-   MakeCompleteName (fileName, TRA_EXT2, dirBuffer, buf, &i);
+   strncpy (dirBuffer, SchemaPath, MAX_PATH);
+   MakeCompleteName (fileName, "TRA", dirBuffer, buf, &i);
 
    /* ouvre le fichier */
    file = TtaReadOpen (buf);
    if (file == 0)
      {
-	ustrncpy (buf, fileName, MAX_PATH);
-	ustrcat (buf, TRA_EXT);
-	TtaDisplayMessage (INFO, TtaGetMessage (LIB, TMSG_LIB_MISSING_FILE), buf);
+       strncpy (buf, fileName, MAX_PATH);
+       strcat (buf, ".TRA");
+       TtaDisplayMessage (INFO, TtaGetMessage (LIB, TMSG_TRA_FILE_INCORRECT),
+			  buf);
      }
    else
      {
@@ -1120,6 +981,24 @@ PtrSSchema          pSS;
 	     TSchemaError (20);
 	     return NULL;
 	  }
+	/* allocate the element translation table */
+	size = pSS->SsNRules * sizeof (PtrTRuleBlock);
+	pTSch->TsElemTRule = (ElemTransTable*) TtaGetMemory (size);
+	if (pTSch->TsElemTRule)
+	  memset (pTSch->TsElemTRule, 0, size);
+
+	/* allocate the attribute inherit table */
+	size = pSS->SsNRules * sizeof (ThotBool);
+	pTSch->TsInheritAttr = (BlnTable*) TtaGetMemory (size);
+	if (pTSch->TsInheritAttr)
+	  memset (pTSch->TsInheritAttr, 0, size);
+
+	/* allocate the attribute translation table */
+	size = pSS->SsNAttributes * sizeof (PtrAttributeTransl);
+	pTSch->TsAttrTRule = (AttrTransTable*) TtaGetMemory (size);
+	if (pTSch->TsAttrTRule)
+	  memset (pTSch->TsAttrTRule, 0, size);
+
 	GetTRule (&pNextTRule);
 	if (pNextTRule == NULL)
 	  {
@@ -1135,11 +1014,11 @@ PtrSSchema          pSS;
 	  memset (pNextBlock, 0, sizeof (TRuleBlock));
 
 	/* lit la partie fixe du schema de traduction */
-	TtaReadName (file, (STRING) &pTSch->TsStructName);
+	TtaReadName (file, (unsigned char *)pTSch->TsStructName);
 	TtaReadShort (file, &pTSch->TsStructCode);
 	TtaReadShort (file, &pTSch->TsLineLength);
-	TtaReadName (file, (STRING) &pTSch->TsEOL);
-	TtaReadName (file, (STRING) &pTSch->TsTranslEOL);
+	TtaReadName (file, (unsigned char *)pTSch->TsEOL);
+	TtaReadName (file, (unsigned char *)pTSch->TsTranslEOL);
 	TtaReadShort (file, &pTSch->TsNConstants);
 	TtaReadShort (file, &pTSch->TsNCounters);
 	TtaReadShort (file, &pTSch->TsNVariables);
@@ -1185,6 +1064,17 @@ PtrSSchema          pSS;
 		     }
 	     }
 	TtaReadShort (file, &pTSch->TsPictureBuffer);
+	TtaReadShort (file, &pTSch->TsNVarBuffers);
+	for (i = 0; i < MAX_TRANSL_BUFFER; i++)
+	  {
+	    pTSch->TsVarBuffer[i].VbIdent[0] = EOS;
+	    pTSch->TsVarBuffer[i].VbNum = 0;
+	  }
+	for (i = 0; i < pTSch->TsNVarBuffers; i++)
+	  {
+	    TtaReadName (file, (unsigned char *)pTSch->TsVarBuffer[i].VbIdent);
+	    TtaReadShort (file, &pTSch->TsVarBuffer[i].VbNum);
+	  }
 	for (i = 0; i < MAX_TRANSL_BUFFER; i++)
 	   pTSch->TsBuffer[i][0] = EOS;
 	if (pSS->SsFirstDynNature == 0)
@@ -1193,45 +1083,51 @@ PtrSSchema          pSS;
 	  {
 	     InitialNElems = pSS->SsFirstDynNature - 1;
 	     for (j = InitialNElems; j < pSS->SsNRules; j++)
-		pTSch->TsElemTRule[j] = NULL;
+		pTSch->TsElemTRule->TsElemTransl[j] = NULL;
 	  }
 	if (!error)
-	   for (i = 0; i < InitialNElems; i++)
-	      pTSch->TsElemTRule[i] = ReadPtrTRuleBlock (file, &pNextBlock);
+	  for (i = 0; i < InitialNElems; i++)
+	    pTSch->TsElemTRule->TsElemTransl[i] =
+	                                ReadPtrTRuleBlock (file, &pNextBlock);
 	if (!error)
-	   for (i = 0; i < InitialNElems; i++)
-	      TtaReadBool (file, &pTSch->TsInheritAttr[i]);
+	  for (i = 0; i < InitialNElems; i++)
+	    TtaReadBool (file, &pTSch->TsInheritAttr->Bln[i]);
 	if (!error)
-	   for (i = 0; i < pSS->SsNAttributes; i++)
-	      if (!error)
+	  for (i = 0; i < pSS->SsNAttributes && !error; i++)
+	    {
+	      GetAttributeTransl (&pAttrTr);
+	      if (!pAttrTr)
+		TSchemaError (25);
+	      else
 		{
-		   pAttrTr = &pTSch->TsAttrTRule[i];
-		   switch (pSS->SsAttribute[i].AttrType)
-			 {
-			    case AtNumAttr:
-			       TtaReadShort (file, &pAttrTr->AtrNCases);
-			       if (!error)
-				  for (j = 0; j < pAttrTr->AtrNCases; j++)
-				     pAttrTr->AtrCase[j].TaTRuleBlock =
-					ReadPtrTRuleBlock (file, &pNextBlock);
-			       break;
-			    case AtTextAttr:
-			       pAttrTr->AtrTxtTRuleBlock =
-				  ReadPtrTRuleBlock (file, &pNextBlock);
-			       break;
-			    case AtReferenceAttr:
-			       pAttrTr->AtrRefTRuleBlock =
-				  ReadPtrTRuleBlock (file, &pNextBlock);
-			       break;
-			    case AtEnumAttr:
-			       for (j = 0; j <= pSS->SsAttribute[i].AttrNEnumValues; j++)
-				  pAttrTr->AtrEnuTRuleBlock[j] =
-				     ReadPtrTRuleBlock (file, &pNextBlock);
-			       break;
-			    default:
-			       break;
-			 }
+		  pTSch->TsAttrTRule->TsAttrTransl[i] = pAttrTr;
+		  switch (pSS->SsAttribute->TtAttr[i]->AttrType)
+		    {
+		    case AtNumAttr:
+		      TtaReadShort (file, &pAttrTr->AtrNCases);
+		      if (!error)
+			for (j = 0; j < pAttrTr->AtrNCases; j++)
+			  pAttrTr->AtrCase[j].TaTRuleBlock =
+			    ReadPtrTRuleBlock (file, &pNextBlock);
+		      break;
+		    case AtTextAttr:
+		      pAttrTr->AtrTxtTRuleBlock =
+			ReadPtrTRuleBlock (file, &pNextBlock);
+		      break;
+		    case AtReferenceAttr:
+		      pAttrTr->AtrRefTRuleBlock =
+			ReadPtrTRuleBlock (file, &pNextBlock);
+		      break;
+		    case AtEnumAttr:
+		      for (j = 0; j <= pSS->SsAttribute->TtAttr[i]->AttrNEnumValues; j++)
+			pAttrTr->AtrEnuTRuleBlock[j] =
+			  ReadPtrTRuleBlock (file, &pNextBlock);
+		      break;
+		    default:
+		      break;
+		    }
 		}
+	    }
 	if (!error)
 	   for (i = 0; i < MAX_TRANSL_PRULE; i++)
 	      if (!error)
@@ -1239,6 +1135,7 @@ PtrSSchema          pSS;
 		   pPRuleTr = &pTSch->TsPresTRule[i];
 		   TtaReadBool (file, &pPRuleTr->RtExist);
 		   if (pPRuleTr->RtExist)
+		     {
 		      if (i == PtSize || i == PtIndent || i == PtLineSpacing ||
 			  i == PtLineWeight || i == PtFillPattern ||
 			  i == PtBackground || i == PtForeground)
@@ -1254,13 +1151,14 @@ PtrSSchema          pSS;
 			 for (j = 0; j <= MAX_TRANSL_PRES_VAL; j++)
 			    pPRuleTr->RtPRuleValueBlock[j] =
 			       ReadPtrTRuleBlock (file, &pNextBlock);
+		     }
 		}
-	TtaReadShort (file, &pTSch->TsNTranslAlphabets);
+	TtaReadShort (file, &pTSch->TsNTranslScripts);
 	if (!error)
-	   for (i = 0; i < pTSch->TsNTranslAlphabets; i++)
+	   for (i = 0; i < pTSch->TsNTranslScripts; i++)
 	     {
-		pAlphTr = &pTSch->TsTranslAlphabet[i];
-		TtaReadByte (file, &pAlphTr->AlAlphabet);
+		pAlphTr = &pTSch->TsTranslScript[i];
+		TtaReadByte (file, (unsigned char *)&pAlphTr->AlScript);
 		TtaReadShort (file, &pAlphTr->AlBegin);
 		TtaReadShort (file, &pAlphTr->AlEnd);
 	     }
@@ -1277,12 +1175,12 @@ PtrSSchema          pSS;
 		/* lit la chaine source */
 		j = 0;
 		do
-		   TtaReadByte (file, &pStringTr->StSource[j++]);
+		   TtaReadByte (file, (unsigned char *)&pStringTr->StSource[j++]);
 		while (pStringTr->StSource[j - 1] != EOS);
 		/* lit la chaine cible */
 		j = 0;
 		do
-		   TtaReadByte (file, &pStringTr->StTarget[j++]);
+		   TtaReadByte (file, (unsigned char *)&pStringTr->StTarget[j++]);
 		while (pStringTr->StTarget[j - 1] != EOS);
 	     }
 	if (!error)
@@ -1291,14 +1189,14 @@ PtrSSchema          pSS;
 	     {
 		j = pTSch->TsConstBegin[i] - 1;
 		do
-		   TtaReadByte (file, &pTSch->TsConstant[j++]);
+		   TtaReadByte (file, (unsigned char *)&pTSch->TsConstant[j++]);
 		while (pTSch->TsConstant[j - 1] != EOS);
 	     }
 	/* lit les blocs de regles des elements */
 	if (!error)
 	   for (i = 0; i < InitialNElems; i++)
-	      ReadBlocks (file, &pTSch->TsElemTRule[i], &pNextTRule, &pSS,
-			  &pNextBlock);
+	      ReadBlocks (file, &pTSch->TsElemTRule->TsElemTransl[i],
+			  &pNextTRule, &pSS, &pNextBlock);
 	/* lit les blocs de regles des attributs */
 	for (i = 0; i < pSS->SsNAttributes; i++)
 	   if (!error)
@@ -1330,16 +1228,7 @@ PtrSSchema          pSS;
    FreeTranslationSchema   libere le schema de traduction pTSch    
    correspondant au schema de structure pSS.               
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                FreeTranslationSchema (PtrTSchema pTSch, PtrSSchema pSS)
-
-#else  /* __STDC__ */
-void                FreeTranslationSchema (pTSch, pSS)
-PtrTSchema          pTSch;
-PtrSSchema          pSS;
-
-#endif /* __STDC__ */
-
+void FreeTranslationSchema (PtrTSchema pTSch, PtrSSchema pSS)
 {
    int                 InitialNElems;
    int                 i;
@@ -1350,13 +1239,32 @@ PtrSSchema          pSS;
       InitialNElems = pSS->SsFirstDynNature - 1;
    /* libere les blocs de regles  des elements */
    for (i = 0; i < InitialNElems; i++)
-      FreeBlocks (pTSch->TsElemTRule[i]);
+      FreeBlocks (pTSch->TsElemTRule->TsElemTransl[i]);
    /* libere les blocs de regles des attributs */
    for (i = 0; i < pSS->SsNAttributes; i++)
-      FreeTRulesAttr (&pTSch->TsAttrTRule[i], &pSS->SsAttribute[i]);
+     {
+      FreeTRulesAttr (pTSch->TsAttrTRule->TsAttrTransl[i],
+		      pSS->SsAttribute->TtAttr[i]);
+      FreeAttributeTransl (pTSch->TsAttrTRule->TsAttrTransl[i]);
+     }
    /* libere les blocs de regles des presentations */
    for (i = 0; i < MAX_TRANSL_PRULE; i++)
       FreeTRulesPres (i + 1, &pTSch->TsPresTRule[i]);
+   if (pTSch->TsElemTRule)
+     {
+       TtaFreeMemory (pTSch->TsElemTRule);
+       pTSch->TsElemTRule = NULL;
+     }
+   if (pTSch->TsInheritAttr)
+     {
+       TtaFreeMemory (pTSch->TsInheritAttr);
+       pTSch->TsInheritAttr = NULL;
+     }
+   if (pTSch->TsAttrTRule)
+     {
+       TtaFreeMemory (pTSch->TsAttrTRule);
+       pTSch->TsAttrTRule = NULL;
+     }
    /* libere le schema de traduction lui-meme */
-   TtaFreeMemory ( pTSch);
+   FreeSchTra (pTSch, pSS);
 }
