@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -9,6 +9,9 @@
  * Module de gestion des vues des documents
  *
  */
+#ifdef _WX
+  #include "wx/wx.h"
+#endif /* _WX */
 
 #include "thot_gui.h"
 #include "thot_sys.h"
@@ -20,7 +23,6 @@
 #include "fileaccess.h"
 #include "appdialogue.h"
 #include "fileaccess.h"
-#include "thotdir.h"
 #include "application.h"
 
 #undef THOT_EXPORT
@@ -32,6 +34,7 @@
 #include "edit_tv.h"
 #include "frame_tv.h"
 #include "appdialogue_tv.h"
+#include "boxes_tv.h"
 
 
 #include "absboxes_f.h"
@@ -73,9 +76,11 @@
 #include "viewapi_f.h"
 #include "writepivot_f.h"
 
-#ifdef _WINDOWS
+#include "appdialogue_wx_f.h"
+
+#ifdef _WINGUI
 #include "wininclude.h"
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
 
 static AvailableView AllViews;
 static int          ViewMenuItem[MAX_VIEW_OPEN];
@@ -101,28 +106,7 @@ ThotBool            PaginatedView (PtrDocument pDoc, int view)
   return paginate;
 }
 
-/*----------------------------------------------------------------------
-   GetViewFromFrame retourne le pointeur sur le numero de vue (viewNum)      
-   dans le document pDoc, correspondant a`                 	
-   la fenetre de numero nframe.
-  ----------------------------------------------------------------------*/
-void GetViewFromFrame (int nframe, PtrDocument pDoc, int *viewNum)
-{
-  int                 view;
 
-  *viewNum = 0;
-  view = 0;
-  /* cherche d'abord dans les vues de l'arbre principal */
-  do
-    {
-      if (pDoc->DocView[view].DvPSchemaView > 0 &&
-	  pDoc->DocViewFrame[view] == nframe)
-	*viewNum = view + 1;
-      else
-	view++;
-    }
-  while (view < MAX_VIEW_DOC && *viewNum == 0);
-}
 
 /*----------------------------------------------------------------------
    GetDocAndView retourne le pointeur sur le document (pDoc) et le
@@ -157,16 +141,18 @@ static void BuildSSchemaViewList (PtrDocument pDoc, PtrSSchema pSS,
 {
    PtrPSchema          pPSchema, pPS;
    DocViewDescr       *pView;
-   SRule              *pSRule;
+   PtrSRule            pSRule;
    int                 view, i;
    ThotBool            open, viewOK, present;
 
    if (pSS != NULL)
       {
-      pSRule = &pSS->SsRule[pSS->SsRootElem - 1];
+      pSRule = pSS->SsRule->SrElem[pSS->SsRootElem - 1];
       if (!pSRule->SrRefImportedDoc || pSS->SsExtension)
 	 {
 	 pPSchema = PresentationSchema (pSS, pDoc);
+	 if (!pPSchema)
+	   return;
 	 if (nature || pSS->SsExtension)
 	    view = 1;
 	 else
@@ -267,7 +253,8 @@ void ChangeDocumentName (PtrDocument pDoc, char *newName)
 	 /* met dans le buffer le nom de la vue */
 	{
 	   pView = &pDoc->DocView[view];
-	   ChangeFrameTitle (pDoc->DocViewFrame[view], buffer);
+	   ChangeFrameTitle (pDoc->DocViewFrame[view], (unsigned char *)buffer,
+			     TtaGetDefaultCharset ());
 	}
 }
 
@@ -275,14 +262,12 @@ void ChangeDocumentName (PtrDocument pDoc, char *newName)
    TtaSetDocumentName
 
    Sets or changes the name of a document. The document must be loaded.
-
    Parameters:
    document: the document whose name is set.
    documentName: new document name. This is only the name, without any
    suffix, without directory name. See function TtaSetDocumentDirectory
    for changing the directory of a document.
    The name must not exceed 31 characters.
-
   ----------------------------------------------------------------------*/
 void TtaSetDocumentName (Document document, char *documentName)
 {
@@ -339,7 +324,7 @@ void OpenDefaultViews (PtrDocument pDoc)
   PtrSSchema        pSS;
   NotifyDialog      notifyDoc;
   int               view, i, X, Y, width, height, schView;
-  ThotBool          bool, skeleton;
+  ThotBool          b, skeleton;
 
   /* si le document a ete charge' sous le forme de ses seuls elements 
      exporte's, on ouvre la vue export sinon, on ouvre la premiere vue. */
@@ -373,7 +358,7 @@ void OpenDefaultViews (PtrDocument pDoc)
      pDoc->DocViewFrame[0] = MakeFrame (pSS->SsName, schView, pDoc->DocDName,
 					X, Y, width, height,
 					&pDoc->DocViewVolume[0],
-					IdentDocument (pDoc));
+					IdentDocument (pDoc), TRUE, TRUE);
      }
   if (pDoc->DocViewFrame[0] == 0)
      /* echec creation fenetre */
@@ -393,13 +378,13 @@ void OpenDefaultViews (PtrDocument pDoc)
      pDoc->DocViewFreeVolume[0] = pDoc->DocViewVolume[0];
      /* met a jour les menus variables de la fenetre */
      if (ThotLocalActions[T_chselect] != NULL)
-	(*ThotLocalActions[T_chselect]) (pDoc);
+	(*(Proc1)ThotLocalActions[T_chselect]) ((void*)pDoc);
      if (ThotLocalActions[T_chattr] != NULL)
-	(*ThotLocalActions[T_chattr]) (pDoc);
+	(*(Proc1)ThotLocalActions[T_chattr]) ((void*)pDoc);
      if (pDoc->DocDocElement != NULL)
 	{
 	pDoc->DocViewRootAb[0] = AbsBoxesCreate (pDoc->DocDocElement, pDoc,
-						 1, TRUE, TRUE, &bool);
+						 1, TRUE, TRUE, &b);
 	i = 0;
 	/* on ne s'occupe pas de la hauteur de page */
 	ChangeConcreteImage (pDoc->DocViewFrame[0], &i,pDoc->DocViewRootAb[0]);
@@ -435,7 +420,7 @@ int CreateAbstractImage (PtrDocument pDoc, int v, PtrSSchema pSS,
    PtrAbstractBox      pAb;
    int                 view, freeView, volume, firstChar, lastChar,
                        ret;
-   ThotBool            stop, sel, selInMainTree, bool;
+   ThotBool            stop, sel, selInMainTree, b;
    ThotBool            truncHead;
 
    freeView = 0;
@@ -466,7 +451,7 @@ int CreateAbstractImage (PtrDocument pDoc, int v, PtrSSchema pSS,
      /* on cree la nouvelle image depuis le debut du document */
      pDoc->DocViewRootAb[freeView-1] = AbsBoxesCreate (pDoc->DocDocElement,
 						       pDoc, freeView, TRUE,
-						       TRUE, &bool);
+						       TRUE, &b);
    else
      {
        /* cree l'image de la meme partie du document que */
@@ -500,7 +485,7 @@ int CreateAbstractImage (PtrDocument pDoc, int v, PtrSSchema pSS,
 	     /* document, on cree la nouvelle image depuis */
 	     /* le debut du document */
 	     pDoc->DocViewRootAb[freeView - 1] = AbsBoxesCreate (pDoc->DocDocElement,
-					   pDoc, freeView, TRUE, TRUE, &bool);
+					   pDoc, freeView, TRUE, TRUE, &b);
 	   else
 	     {
 	       /* cherche dans la vue designee le premier pave dont le debut
@@ -544,7 +529,8 @@ int CreateAbstractImage (PtrDocument pDoc, int v, PtrSSchema pSS,
    		     fenetre en mm.					
   ----------------------------------------------------------------------*/
 void OpenCreatedView (PtrDocument pDoc, int view, int X, int Y,
-		      int width, int height)
+		      int width, int height,
+		      ThotBool withMenu, ThotBool withButton)
 {
   PtrSSchema          pSS;
   int                 volume = 0;
@@ -559,8 +545,26 @@ void OpenCreatedView (PtrDocument pDoc, int view, int X, int Y,
       schView = pDoc->DocView[view - 1].DvPSchemaView;
       /* creation d'une fenetre pour la vue */
       pSS = pDoc->DocSSchema;
+
+#ifdef _WX
+      int doc_id = IdentDocument(pDoc);
+      frame = TtaMakeFrame( pSS->SsName,
+	                    schView,
+			    doc_id,
+			    pDoc->DocDName,
+			    width, height, &volume );
+      TtaAttachFrame( frame,
+		      TtaGetWindowId( doc_id ),
+		      TtaGetPageId( doc_id ),
+		      schView > 1 ? 2 : 1 /* TODO: 1=up 2=down ajouter la vue en haut ou en bas, au choix de l'utilisateur */ );
+#endif /* _WX */
+      
+#if defined(_MOTIF) || defined(_GTK) || defined(_WINGUI)
       frame = MakeFrame (pSS->SsName, schView,  pDoc->DocDName, X, Y,
-			     width, height, &volume, IdentDocument (pDoc));
+			 width, height, &volume, IdentDocument (pDoc),
+			 withMenu, withButton);
+#endif /* #if defined(_MOTIF) || defined(_GTK) || defined(_WINGUI) */
+      
     } 
   if (frame == 0)
     {
@@ -577,15 +581,23 @@ void OpenCreatedView (PtrDocument pDoc, int view, int X, int Y,
       pDoc->DocViewFrame[view - 1] = frame;
       pDoc->DocViewVolume[view - 1] = volume;
       ChangeConcreteImage (frame, &h, pDoc->DocViewRootAb[view - 1]);
-      ShowSelection (pDoc->DocViewRootAb[view - 1], TRUE);
       DisplayFrame (frame);
+      ShowSelection (pDoc->DocViewRootAb[view - 1], TRUE);
+
       /* Update Paste entry in menu */
+#ifdef _WINGUI
+	  if (pDoc->DocReadOnly)
+	SwitchPaste (pDoc, FALSE);
+      else
+	SwitchPaste(pDoc, TRUE);
+#else /* _WINGUI */
       if ((FirstSavedElement == NULL && ClipboardThot.BuLength == 0) ||
 	  pDoc->DocReadOnly)
 	SwitchPaste (pDoc, FALSE);
       else
 	SwitchPaste(pDoc, TRUE);
-      
+#endif /* _WINGUI */
+
       /* check the Undo state of the document */
       if (pDoc->DocNbEditsInHistory == 0)
 	SwitchUndo (pDoc, FALSE);
@@ -600,9 +612,14 @@ void OpenCreatedView (PtrDocument pDoc, int view, int X, int Y,
       
       /* met a jour les menus de la fenetre */
       if (ThotLocalActions[T_chselect] != NULL)
-	(*ThotLocalActions[T_chselect]) (pDoc);
+	(*(Proc1)ThotLocalActions[T_chselect]) ((void*)pDoc);
       if (ThotLocalActions[T_chattr] != NULL)
-	(*ThotLocalActions[T_chattr]) (pDoc);
+	(*(Proc1)ThotLocalActions[T_chattr]) ((void*)pDoc);
+
+#ifdef _WX
+      /* wait for frame initialisation (needed by opengl) */
+      TtaHandlePendingEvents();
+#endif /* _WX */
     }
 }
 
@@ -640,7 +657,8 @@ static ThotBool GetViewByName (PtrDocument pDoc, Name viewName, int *view,
 /*----------------------------------------------------------------------
    OpenViewByName ouvre la vue de nom viewName			
   ----------------------------------------------------------------------*/
-int OpenViewByName (PtrDocument pDoc, Name viewName, int X, int Y, int width, int height)
+int OpenViewByName (PtrDocument pDoc, Name viewName, int X, int Y,
+		    int width, int height)
 {
    PtrSSchema          pSS;
    NotifyDialog        notifyDoc;
@@ -667,7 +685,7 @@ int OpenViewByName (PtrDocument pDoc, Name viewName, int X, int Y, int width, in
 	     if (!CallEventType ((NotifyEvent *) & notifyDoc, TRUE))
 	        {
 		ret = CreateAbstractImage (pDoc, view, pSS, 1, FALSE, NULL);
-		OpenCreatedView (pDoc, ret, X, Y, width, height);
+		OpenCreatedView (pDoc, ret, X, Y, width, height, TRUE, TRUE);
 		notifyDoc.event = TteViewOpen;
 		notifyDoc.document = (Document) IdentDocument (pDoc);
 		notifyDoc.view = ret;
@@ -710,7 +728,7 @@ void OpenViewByMenu (PtrDocument pDoc, int menuItem, PtrElement subTree,
 	 view = CreateAbstractImage (pDoc, AllViews[theView - 1].VdView,
 				     AllViews[theView - 1].VdSSchema,
 				     selectedView, FALSE, subTree);
-	 OpenCreatedView (pDoc, view, X, Y, width, height);
+	 OpenCreatedView (pDoc, view, X, Y, width, height, TRUE, TRUE);
 	 if (viewHasBeenOpen)
 	    {
 	    notifyDoc.event = TteViewOpen;
@@ -811,10 +829,16 @@ void CloseView (PtrDocument pDoc, int viewNb)
 		  TteConnectAction (T_rconfirmclose,
 				    (Proc) CallbackCloseDocMenu);
 		  }
-	       (*ThotLocalActions[T_confirmclose]) (pDoc, document, view, &ok,
-						    &Save);
+	       (*(Proc5)ThotLocalActions[T_confirmclose]) (
+			(void*)pDoc,
+		       	(void*)document,
+			(void*)view,
+			(void*)&ok,
+			(void*)&Save);
 	       if (Save)
-		  ok = (*(Func)ThotLocalActions[T_writedocument]) (pDoc, 0);
+		  ok = (*(Func2)ThotLocalActions[T_writedocument]) (
+		      (void*)pDoc,
+		      (void*)0);
 	       }
 	    else
 	       ok = TRUE;
