@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -15,7 +15,6 @@
 #include "fileaccess.h"
 #include "appdialogue.h"
 #include "fileaccess.h"
-#include "thotdir.h"
 #include "application.h"
 
 #undef THOT_EXPORT
@@ -335,7 +334,7 @@ static ThotBool     interactiveSave (PtrDocument pDoc)
   ThotBool            status;
   
   status = FALSE;
-  if (!pDoc->DocReadOnly && pDoc->DocSSchema)
+  if (pDoc->DocSSchema)
     status = StoreDocument (pDoc, SaveFileName, SaveDirectoryName,SaveDocWithCopy, SaveDocWithMove);
   
   if (status)
