@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -12,6 +12,7 @@
 
  */
 
+#include "thot_gui.h"
 #include "thot_sys.h"
 #include "libmsg.h"
 #include "message.h"
@@ -23,7 +24,6 @@
 #include "appaction.h"
 #include "appstruct.h"
 #include "typecorr.h"
-#include "thotdir.h"
 #include "fileaccess.h"
 #include "labelAllocator.h"
 #include "application.h"
@@ -90,6 +90,13 @@ static void PutInteger (BinFile pivFile, int n)
    PutShort (pivFile, n % 65536);
 }
 
+/*----------------------------------------------------------------------
+   PutInteger ecrit un entier long dans le fichier, sur 4 octets      
+  ----------------------------------------------------------------------*/
+static void PutFloat (BinFile pivFile, float n)
+{
+   TtaWrite4Byte (pivFile, (unsigned char*) (&n));
+}
 /*----------------------------------------------------------------------
    PutDimensionType ecrit dans le fichier un type de dimension sur	
    1 octet.								
@@ -392,8 +399,8 @@ void PutAttribut (BinFile pivFile, PtrAttribute pAttr, PtrDocument pDoc)
 	    while (pBuf != NULL)
 	      {
 		i = 0;
-		while (pBuf->BuContent[i] != WC_EOS)
-		  TtaWriteWideChar (pivFile, pBuf->BuContent[i++], UTF_8);
+		while (pBuf->BuContent[i] != EOS)
+		  TtaWriteWideChar (pivFile, pBuf->BuContent[i++]);
 		pBuf = pBuf->BuNext;
 	      }
 	    TtaWriteByte (pivFile, EOS);
@@ -438,14 +445,20 @@ static void   PutPresRule (BinFile pivFile, PtrPRule pPRule)
 	rType == PtBorderRightStyle ||
 	rType == PtBorderBottomStyle ||
 	rType == PtBorderLeftStyle ||
+	rType == PtDisplay ||
+	rType == PtFloat ||
+	rType == PtClear ||
+	rType == PtVisibility ||
 	rType == PtSize || rType == PtStyle ||
 	rType == PtWeight || rType == PtFont ||
 	rType == PtUnderline || rType == PtThickness ||
 	rType == PtIndent || rType == PtLineSpacing ||
 	rType == PtDepth ||
 	rType == PtAdjust ||
+	rType == PtDirection || rType == PtUnicodeBidi ||
 	rType == PtLineStyle || rType == PtLineWeight ||
-	rType == PtFillPattern ||
+	rType == PtFillPattern || rType == PtOpacity || 
+	rType == PtFillOpacity || rType == PtStrokeOpacity ||
 	rType == PtBackground || rType == PtForeground ||
 	rType == PtHyphenate ||
 	rType == PtBreak1 || rType == PtBreak2 ||
@@ -469,6 +482,9 @@ static void   PutPresRule (BinFile pivFile, PtrPRule pPRule)
       TtaWriteByte (pivFile, (char) C_PIV_PRESENT);
       /* ecrit le numero de vue */
       PutShort (pivFile, pPRule->PrViewNum);
+      /* ecrit la specificite et l'importance */
+      PutShort (pivFile, pPRule->PrSpecificity);
+      PutBoolean (pivFile, pPRule->PrImportant);
       /* ecrit le numero de la boite de presentation concernee */
       PutShort (pivFile, 0);
       /* ecrit le type de la regle */
@@ -516,9 +532,27 @@ static void   PutPresRule (BinFile pivFile, PtrPRule pPRule)
 	case PtThickness:
 	  TtaWriteByte (pivFile, C_PR_UNDER_THICK);
 	  break;
+	case PtDirection:
+	  TtaWriteByte (pivFile, C_PR_DIRECTION);
+	  break;
+	case PtUnicodeBidi:
+	  TtaWriteByte (pivFile, C_PR_UNICODEBIDI);
+	  break;
 	case PtLineStyle:
 	  TtaWriteByte (pivFile, C_PR_LINESTYLE);
 	  break;
+	case PtDisplay:
+	  TtaWriteByte (pivFile, C_PR_DISPLAY);
+	  break;
+	case PtFloat:
+	  TtaWriteByte (pivFile, C_PR_FLOAT);
+	  break;
+	case PtClear:
+	  TtaWriteByte (pivFile, C_PR_CLEAR);
+	  break;
+	case PtVisibility:
+	  TtaWriteByte (pivFile, C_PR_VISIBILITY);
+	  break;
         case PtBorderTopStyle:
 	  TtaWriteByte (pivFile, C_PR_BORDERTOPSTYLE);
 	  break;
@@ -600,6 +634,15 @@ static void   PutPresRule (BinFile pivFile, PtrPRule pPRule)
 	case PtFillPattern:
 	  TtaWriteByte (pivFile, C_PR_FILLPATTERN);
 	  break;
+	case PtOpacity:
+	  TtaWriteByte (pivFile, C_PR_OPACITY);
+	  break;
+	case PtFillOpacity:
+	  TtaWriteByte (pivFile, C_PR_FILL_OPACITY);
+	  break;
+	case PtStrokeOpacity:
+	  TtaWriteByte (pivFile, C_PR_STROKE_OPACITY);
+	  break;
 	case PtBackground:
 	  TtaWriteByte (pivFile, C_PR_BACKGROUND);
 	  break;
@@ -619,6 +662,7 @@ static void   PutPresRule (BinFile pivFile, PtrPRule pPRule)
 	  TtaWriteByte (pivFile, C_PR_BORDERLEFTCOLOR);
 	  break;
 	default:
+          printf ("\nERROR: wrong pres rule *****\n");
 	  break;
 	}
       /* ecrit les parametres de la regle */
@@ -646,7 +690,14 @@ static void   PutPresRule (BinFile pivFile, PtrPRule pPRule)
 	case PtHorizPos:
 	  PutShort (pivFile, pPRule->PrPosRule.PoPosDef);
 	  PutShort (pivFile, pPRule->PrPosRule.PoPosRef);
+	  PutBoolean (pivFile, pPRule->PrPosRule.PoDistAttr);
 	  PutShort (pivFile, pPRule->PrPosRule.PoRelation);
+	  PutBoolean (pivFile, pPRule->PrPosRule.PoNotRel);
+	  PutShort (pivFile, pPRule->PrPosRule.PoRefKind);
+	  PutShort (pivFile, pPRule->PrPosRule.PoRefIdent);
+	  PutShort (pivFile, abs (pPRule->PrPosRule.PoDistDelta));
+	  PutUnit (pivFile, pPRule->PrPosRule.PoDeltaUnit);
+	  PutSign (pivFile, (ThotBool)(pPRule->PrPosRule.PoDistDelta >= 0));
 	  PutShort (pivFile, abs (pPRule->PrPosRule.PoDistance));
 	  PutUnit (pivFile, pPRule->PrPosRule.PoDistUnit);
 	  PutSign (pivFile, (ThotBool)(pPRule->PrPosRule.PoDistance >= 0));
@@ -698,6 +749,9 @@ static void   PutPresRule (BinFile pivFile, PtrPRule pPRule)
 	    }
 	  break;
 	case PtDepth:
+	case PtOpacity:
+	case PtFillOpacity:
+	case PtStrokeOpacity:
 	case PtFillPattern:
 	  PutShort (pivFile, pPRule->PrIntValue);
 	  break;
@@ -713,7 +767,11 @@ static void   PutPresRule (BinFile pivFile, PtrPRule pPRule)
 	  else
 	    /* negative value */
 	    PutShort (pivFile, 1);
+#ifdef NODISPLAY
+	  red = green = blue = 0;
+#else /* NODISPLAY */
 	  TtaGiveThotRGB (pPRule->PrIntValue, &red, &green, &blue);
+#endif /* NODISPLAY */
 	  PutShort (pivFile, red);
 	  PutShort (pivFile, green);
 	  PutShort (pivFile, blue);
@@ -723,7 +781,13 @@ static void   PutPresRule (BinFile pivFile, PtrPRule pPRule)
 	case PtWeight:
 	case PtUnderline:
 	case PtThickness:
+	case PtDirection:
+	case PtUnicodeBidi:
+	case PtVisibility:
 	case PtLineStyle:
+	case PtDisplay:
+	case PtFloat:
+	case PtClear:
         case PtBorderTopStyle:
         case PtBorderRightStyle:
         case PtBorderBottomStyle:
@@ -758,12 +822,13 @@ void Externalise (BinFile pivFile, PtrElement *pEl, PtrDocument pDoc,
   PtrTextBuffer       pBuf;
   PtrPathSeg          pPa;
   PtrAttribute        pAttr;
-  PtrPRule            pPRule;
+  PtrPRule            pPRule = NULL;
   PtrSSchema          pSS;
   NotifyElement       notifyEl;
   NotifyAttribute     notifyAttr;
   int                 i, c;
   ThotBool            stop;
+  PtrTransform        Trans;
 
   /* on ecrit effectivement la forme pivot de l'element */
   pEl1 = *pEl;
@@ -802,6 +867,7 @@ void Externalise (BinFile pivFile, PtrElement *pEl, PtrDocument pDoc,
       notifyAttr.event = TteAttrSave;
       notifyAttr.document = (Document) IdentDocument (pDoc);
       notifyAttr.element = (Element) pEl1;
+      notifyAttr.info = 0; /* not sent by undo */
       notifyAttr.attribute = (Attribute) pAttr;
       notifyAttr.attributeType.AttrTypeNum = pAttr->AeAttrNum;
       notifyAttr.attributeType.AttrSSchema = (SSchema) (pAttr->AeAttrSSchema);
@@ -814,6 +880,7 @@ void Externalise (BinFile pivFile, PtrElement *pEl, PtrDocument pDoc,
 	  notifyAttr.event = TteAttrSave;
 	  notifyAttr.document = (Document) IdentDocument (pDoc);
 	  notifyAttr.element = (Element) pEl1;
+	  notifyAttr.info = 0; /* not sent by undo */
 	  notifyAttr.attribute = (Attribute) pAttr;
 	  notifyAttr.attributeType.AttrTypeNum = pAttr->AeAttrNum;
 	  notifyAttr.attributeType.AttrSSchema = (SSchema) (pAttr->AeAttrSSchema);
@@ -826,20 +893,21 @@ void Externalise (BinFile pivFile, PtrElement *pEl, PtrDocument pDoc,
   /*****TO BE CHANGED**** */
   /* write a specific rule for each picture element */
   if (pEl1->ElTerminal && pEl1->ElLeafType == LtPicture &&
-      pEl1->ElPictInfo != NULL)
+      pEl1->ElPictInfo)
     {
       /* write the rule mark */
       TtaWriteByte (pivFile, (char) C_PIV_PRESENT);
       /* write the view number */
       PutShort (pivFile, 1);
+      /* write the specificity and the importance of the rule */
+      PutShort (pivFile, 0);
+      PutBoolean (pivFile, TRUE);
       /* write the presentation box number */
       PutShort (pivFile, 0);
       /* write the specific rule and its parameters */
       TtaWriteByte (pivFile, C_PR_PICTURE);
       PutShort (pivFile, 0);
       PutShort (pivFile, 0);
-      PutShort (pivFile, 0);
-      PutShort (pivFile, 0);
       switch (((PictInfo *)(pEl1->ElPictInfo))->PicPresent)
 	{
 	case RealSize:
@@ -880,7 +948,7 @@ void Externalise (BinFile pivFile, PtrElement *pEl, PtrDocument pDoc,
 	{
 	  /* feuille terminale: on ecrit son contenu entre C_PIV_BEGIN et
 	     C_PIV_END */
-	  if (pSS->SsRule[pEl1->ElTypeNumber - 1].SrConstruct != CsConstant)
+	  if (pSS->SsRule->SrElem[pEl1->ElTypeNumber - 1]->SrConstruct != CsConstant)
 	    /* on n'ecrit pas le texte des constantes, puisqu'il est cree
 	       automatiquement */
 	    {
@@ -914,14 +982,15 @@ void Externalise (BinFile pivFile, PtrElement *pEl, PtrDocument pDoc,
 		      while (c < pEl1->ElTextLength && pBuf != NULL)
 			{
 			  i = 0;
-			  while (pBuf->BuContent[i] != WC_EOS && i < pBuf->BuLength)
-			    TtaWriteWideChar (pivFile, pBuf->BuContent[i++], UTF_8);
+			  while (pBuf->BuContent[i] != EOS && i < pBuf->BuLength)
+			    TtaWriteWideChar (pivFile, pBuf->BuContent[i++]);
 			  c = c + i;
 			  /* buffer suivant du meme element */
 			  pBuf = pBuf->BuNext;
 			}
 		      /* peut-on concatener l'element suivant ? */
 		      stop = TRUE;
+#ifndef NODISPLAY
 		      if (pEl1->ElLeafType == LtText && pEl1->ElNext != NULL &&
 			  /* c'est du texte, il y a un suivant.. */
 			  pEl1->ElNext->ElTerminal &&
@@ -930,8 +999,8 @@ void Externalise (BinFile pivFile, PtrElement *pEl, PtrDocument pDoc,
 			  pEl1->ElNext->ElLanguage == pEl1->ElLanguage &&
 			  pEl1->ElNext->ElSource == NULL &&
 			  /* le suivant n'est pas une inclusion */
-			  pEl1->ElStructSchema->SsRule[pEl1->ElTypeNumber - 1].SrConstruct != CsConstant &&
-			  pEl1->ElNext->ElStructSchema->SsRule[pEl1->ElNext->ElTypeNumber - 1].SrConstruct != CsConstant &&
+			  pEl1->ElStructSchema->SsRule->SrElem[pEl1->ElTypeNumber - 1]->SrConstruct != CsConstant &&
+			  pEl1->ElNext->ElStructSchema->SsRule->SrElem[pEl1->ElNext->ElTypeNumber - 1]->SrConstruct != CsConstant &&
 			  SameAttributes (*pEl, pEl1->ElNext))
 			/* il a les memes attributs */
 			if (BothHaveNoSpecRules (*pEl, pEl1->ElNext))
@@ -941,6 +1010,7 @@ void Externalise (BinFile pivFile, PtrElement *pEl, PtrDocument pDoc,
 			    stop = FALSE;
 			    pEl1 = pEl1->ElNext;
 			  }
+#endif /* NODISPLAY */
 		    }
 		  while (!stop);
 		  /* update pEl value */
@@ -1056,6 +1126,51 @@ void Externalise (BinFile pivFile, PtrElement *pEl, PtrDocument pDoc,
 		default:
 		  break;
 		}
+	      /*Add transformation informations in pivot file*/
+	      if (pEl1->ElParent && pEl1->ElParent->ElTransform)
+		{
+		  Trans = pEl1->ElParent->ElTransform;
+		  while (Trans)
+		    {
+		      TtaWriteByte (pivFile, (char) C_PIV_TRANS_START);
+		      PutInteger (pivFile, Trans->TransType);
+		      switch (Trans->TransType)
+			{
+			case PtElBoxTranslate:
+			case PtElviewboxScale:
+			case PtElviewboxTranslate:
+			case PtElScale:
+			case PtElAnimTranslate:
+			case PtElTranslate:
+			  PutFloat (pivFile, Trans->XScale);  
+			  PutFloat (pivFile, Trans->YScale); 		
+			  break;
+			case PtElAnimRotate:
+			case PtElRotate:
+			  PutFloat (pivFile, Trans->XRotate);
+			  PutFloat (pivFile, Trans->YRotate);
+			  PutFloat (pivFile, Trans->TrAngle);	    
+			  break;
+			case PtElMatrix:
+			  PutFloat (pivFile, Trans->AMatrix);
+			  PutFloat (pivFile, Trans->BMatrix);
+			  PutFloat (pivFile, Trans->CMatrix);
+			  PutFloat (pivFile, Trans->DMatrix);
+			  PutFloat (pivFile, Trans->EMatrix);
+			  PutFloat (pivFile, Trans->FMatrix);
+			  break;
+			case PtElSkewX:
+			case PtElSkewY:
+			  PutFloat (pivFile, Trans->TrFactor);
+			  break;	  
+			default:
+			  break;	  
+			}
+		      Trans = Trans->Next;
+		    }
+		  TtaWriteByte (pivFile, (char) C_PIV_TRANS_END);
+		}
+
 	      if (pEl1->ElLeafType != LtReference)
 		TtaWriteByte (pivFile, (char) C_PIV_END);
 	    }
@@ -1077,6 +1192,7 @@ void Externalise (BinFile pivFile, PtrElement *pEl, PtrDocument pDoc,
 		notifyEl.event = TteElemSave;
 		notifyEl.document = (Document) IdentDocument (pDoc);
 		notifyEl.element = (Element) pChild;
+		notifyEl.info = 0; /* not sent by undo */
 		notifyEl.elementType.ElTypeNum = pChild->ElTypeNumber;
 		notifyEl.elementType.ElSSchema = (SSchema) (pChild->ElStructSchema);
 		notifyEl.position = 0;
@@ -1095,6 +1211,7 @@ void Externalise (BinFile pivFile, PtrElement *pEl, PtrDocument pDoc,
 		    notifyEl.event = TteElemSave;
 		    notifyEl.document = (Document) IdentDocument (pDoc);
 		    notifyEl.element = (Element) pChild;
+		    notifyEl.info = 0; /* not sent by undo */
 		    notifyEl.elementType.ElTypeNum = pChild->ElTypeNumber;
 		    notifyEl.elementType.ElSSchema = (SSchema) (pChild->ElStructSchema);
 		    notifyEl.position = 0;
@@ -1128,6 +1245,7 @@ static void PutName (BinFile pivFile, Name N)
   ----------------------------------------------------------------------*/
 void WriteSchemaNamesOfDoc (BinFile pivFile, PtrDocument pDoc)
 {
+#ifndef NODISPLAY
    int                 nat;
 
    BuildDocNatureTable (pDoc);
@@ -1146,6 +1264,7 @@ void WriteSchemaNamesOfDoc (BinFile pivFile, PtrDocument pDoc)
 	/* ecrit le nom du schema de presentation associe' */
 	PutName (pivFile, pDoc->DocNatureSSchema[nat]->SsDefaultPSchema);
      }
+#endif /* NODISPLAY */
 }
 
 /*----------------------------------------------------------------------
@@ -1160,7 +1279,7 @@ static void UpdateLanguageTable (PtrDocument pDoc, PtrElement pEl)
 
    while (pEl != NULL)
      {
-	pEl = FwdSearchTypedElem (pEl, CharString + 1, NULL);
+	pEl = FwdSearchTypedElem (pEl, CharString + 1, NULL, NULL);
 	if (pEl != NULL)
 	  {
 	     found = FALSE;
@@ -1234,6 +1353,7 @@ void SauveDoc (BinFile pivFile, PtrDocument pDoc)
 	notifyEl.event = TteElemSave;
 	notifyEl.document = (Document) IdentDocument (pDoc);
 	notifyEl.element = (Element) pEl;
+	notifyEl.info = 0; /* not sent by undo */
 	notifyEl.elementType.ElTypeNum = pEl->ElTypeNumber;
 	notifyEl.elementType.ElSSchema = (SSchema) (pEl->ElStructSchema);
 	notifyEl.position = 0;
@@ -1248,6 +1368,7 @@ void SauveDoc (BinFile pivFile, PtrDocument pDoc)
 	     notifyEl.event = TteElemSave;
 	     notifyEl.document = (Document) IdentDocument (pDoc);
 	     notifyEl.element = (Element) pEl;
+	     notifyEl.info = 0; /* not sent by undo */
 	     notifyEl.elementType.ElTypeNum = pEl->ElTypeNumber;
 	     notifyEl.elementType.ElSSchema = (SSchema) (pEl->ElStructSchema);
 	     notifyEl.position = 0;
