@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -12,6 +12,11 @@
  *
  */	
 
+#ifdef _WX
+  #include "wx/wx.h"
+#endif /* _WX */
+
+#include "thot_gui.h"
 #include "thot_sys.h"
 #include "constmenu.h"
 #include "constmedia.h"
@@ -32,19 +37,19 @@
 #endif /* NODISPLAY */
 #include "appdialogue.h"
 #include "dictionary.h"
+#include "picture.h"
 
 #undef THOT_EXPORT
 #define THOT_EXPORT extern
 #include "platform_tv.h"
 #include "appevents_tv.h"
 #include "units_tv.h"
-#undef THOT_EXPORT
-#define THOT_EXPORT
+
 #include "print_tv.h"
 #include "edit_tv.h"
 #include "thotcolor.h"
 #include "thotcolor_tv.h"
-#include "thotpalette_tv.h"
+//#include "thotpalette_tv.h"
 #include "frame_tv.h"
 #include "appdialogue_tv.h"
 
@@ -75,11 +80,11 @@
 
 static Proc         AppClosingFunction = NULL;
 #define VersionId "V3.1"
-ThotBool            PrintErrorMessages = TRUE;
+static ThotBool            PrintErrorMessages = TRUE;
 
-#ifdef _WINDOWS
-#include "wininclude.h"
-#endif /* _WINDOWS */
+#ifdef _WINGUI
+  #include "wininclude.h"
+#endif /* _WINGUI */
 
 /*----------------------------------------------------------------------
   ----------------------------------------------------------------------*/
@@ -110,13 +115,13 @@ void CloseInsertion ()
 #ifndef NODISPLAY
 
 /*----------------------------------------------------------------------
-   GetWindowNumber returns the window corresponding to the view of 
+   GetWindowNumber returns the frame corresponding to the view of 
    the document.                                           
    Parameters:                                                     
-   document: the document.                                 
-   view: the view.                                         
-   Return value:                                                   
-   corresponding window.                                   
+     document: the document.                                 
+     view: the view.                                         
+   Return value:
+     the corresponding frame id                                                   
   ----------------------------------------------------------------------*/
 int GetWindowNumber (Document document, View view)
 {
@@ -166,11 +171,18 @@ ThotWidget TtaGetViewFrame (Document document, View view)
       return 0;
     }
   else
-#ifdef _WINDOWS
+
+#ifdef _WINGUI
     return (FrMainRef[frame]);
-#else  /* _WINDOWS */
-    return (FrameTable[frame].WdFrame);
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
+  
+#if defined(_MOTIF) || defined(_GTK) || defined(_WX)
+    return ((ThotWidget)FrameTable[frame].WdFrame);
+#endif /* #if defined(_MOTIF) || defined(_GTK) || defined(_WX) */
+
+#if defined(_NOGUI)
+    return 0;
+#endif /* #ifdef _NOGUI */
 }
 #endif
 
@@ -193,10 +205,14 @@ static void ErrorHandler ()
    if (ThotLocalActions [T_backuponfatal] != NULL)
        (*ThotLocalActions [T_backuponfatal]) ();
    {
-#ifdef _GTK
+#if defined(_GTK) && !defined(NODISPLAY)
      gtk_exit (1);
-#endif /* _GTK */	
+#endif /* _GTK && !NODISPLAY */	
+#if defined(_WX) && !defined(NODISPLAY)
+     wxExit();
+#else /* _WX && !defined(NODISPLAY) */     
      exit (1);
+#endif /* _WX && !defined(NODISPLAY) */     
    }
 }
 
@@ -206,24 +222,28 @@ static void ErrorHandler ()
   ----------------------------------------------------------------------*/
 static void QuitHandler ()
 {
-   signal (SIGINT, ErrorHandler);
-#  ifndef _WINDOWS 
+   signal (SIGINT, (void (*)(int))ErrorHandler);
+#if defined(_MOTIF) || defined(_GTK) || defined(_WX) && !defined(_WINDOWS)
    signal (SIGQUIT, SIG_DFL);
-#  endif /* _WINDOWS */
-   signal (SIGTERM, ErrorHandler);
+#endif /* #if defined(_MOTIF) || defined(_GTK) || defined(_WX) && !defined(_WINDOWS) */
+   signal (SIGTERM, (void (*)(int))ErrorHandler);
    if (ThotLocalActions [T_backuponfatal] != NULL)
      (*ThotLocalActions [T_backuponfatal]) ();
    {
-#ifdef _GTK
+#if defined(_GTK) && !defined(NODISPLAY)
      gtk_exit (1);
-#endif /* _GTK */	
+#endif /* _GTK && !NODISPLAY */	
+#if defined(_WX) && !defined(NODISPLAY)
+     wxExit();
+#else /* _WX && !defined(NODISPLAY) */     
      exit (1);
+#endif /* _WX && !defined(NODISPLAY) */     
    }
-#  ifndef _WINDOWS
-   signal (SIGINT, QuitHandler);
-   signal (SIGQUIT, QuitHandler);
-   signal (SIGTERM, QuitHandler);
-#  endif /* _WINDOWS */
+#if defined(_MOTIF) || defined(_GTK) || defined(_WX) && !defined(_WINDOWS)
+   signal (SIGINT, (void (*)(int))QuitHandler);
+   signal (SIGQUIT, (void (*)(int))QuitHandler);
+   signal (SIGTERM, (void (*)(int))QuitHandler);
+#endif /* #if defined(_MOTIF) || defined(_GTK) || defined(_WX) && !defined(_WINDOWS) */
 }
 
 /*----------------------------------------------------------------------
@@ -231,21 +251,29 @@ static void QuitHandler ()
   ----------------------------------------------------------------------*/
 void InitErrorHandler ()
 {
-#  ifndef _WINDOWS
-   signal (SIGBUS, ErrorHandler);
-   signal (SIGHUP, ErrorHandler);
-   signal (SIGQUIT, QuitHandler);
-#  endif /* _WINDOWS */
+#if defined(_MOTIF) || defined(_GTK) || defined(_WX) && !defined(_WINDOWS)
+   signal (SIGBUS, (void (*)(int))ErrorHandler);
+   signal (SIGHUP, (void (*)(int))ErrorHandler);
+   signal (SIGQUIT, (void (*)(int))QuitHandler);
+#endif /* #if defined(_MOTIF) || defined(_GTK) || defined(_WX) && !defined(_WINDOWS) */
 
-   signal (SIGSEGV, ErrorHandler);
+   signal (SIGSEGV, (void (*)(int))ErrorHandler);
 #  ifdef SIGABRT
-   signal (SIGABRT, ErrorHandler);
+   signal (SIGABRT, (void (*)(int))ErrorHandler);
 #  else
-   signal (SIGIOT, ErrorHandler);
+   signal (SIGIOT, (void (*)(int))ErrorHandler);
 #  endif
 
-   signal (SIGINT, QuitHandler);
-   signal (SIGTERM, QuitHandler);
+   signal (SIGINT, (void (*)(int))QuitHandler);
+   signal (SIGTERM, (void (*)(int))QuitHandler);
+}
+
+/*----------------------------------------------------------------------
+  TtaUseDotForFloat returns TRUE if floats take the form xx.yy
+ ----------------------------------------------------------------------*/
+ThotBool TtaUseDotForFloat ()
+{
+  return UseDotForFloat;
 }
 
 /*----------------------------------------------------------------------
@@ -261,12 +289,19 @@ void InitErrorHandler ()
 void TtaInitialize (char *applicationName)
 {
    int                 i;
+   float               val;
 
    UserErrorCode = 0;
    strcpy (DefaultDocumentName, "");
    InitEditorMemory ();	      /* Initializes the memory managment of the editor */
    InitNatures ();	      /* Initializes the table of Natures */
+
+   /* test if the system uses dot or comma in the float syntax */
+   sscanf (".5", "%f", &val);
+   UseDotForFloat = (val == 0.5);
+
    FullStructureChecking = FALSE;
+   ShowOnePage = FALSE;
    /* Initializes the table of documents */
    for (i = 0; i < MAX_DOCUMENTS; i++)
       LoadedDocument[i] = NULL;
@@ -283,10 +318,10 @@ void TtaInitialize (char *applicationName)
    InitLanguage ();
 
 #ifndef NODISPLAY
-#ifdef _WINDOWS
+#ifdef _WINGUI
    TtPrinterDC = NULL;
    WIN_Main_Wd = NULL;
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
    DOT_PER_INCH = 72;
    numOfJobs = 0;
    /* Initializes patterns */
@@ -301,16 +336,24 @@ void TtaInitialize (char *applicationName)
        break;
      case -1:
        fprintf (stderr, "cannot find messages table\n");
-#ifdef _GTK
-	gtk_exit (1);
-#endif /* _GTK */	
+#if defined(_GTK) && !defined(NODISPLAY)
+       gtk_exit (1);
+#endif /* _GTK && !NODISPLAY */	
+#if defined(_WX) && !defined(NODISPLAY)
+       wxExit();
+#else /* _WX && !defined(NODISPLAY) */     
        exit (1);
+#endif /* _WX && !defined(NODISPLAY) */     
      default:
        fprintf (stderr, "Previous messages table loaded\n");
-#ifdef _GTK
-	gtk_exit (1);
-#endif /* _GTK */	
+#if defined(_GTK) && !defined(NODISPLAY)
+       gtk_exit (1);
+#endif /* _GTK && !NODISPLAY */	
+#if defined(_WX) && !defined(NODISPLAY)
+       wxExit();
+#else /* _WX && !defined(NODISPLAY) */     
        exit (1);
+#endif /* _WX && !defined(NODISPLAY) */     
      }
 
    /* init the system error signal handler */
@@ -367,6 +410,9 @@ void TtaQuit ()
   /* remove the contents of the cut buffer related to the document */
   if (ThotLocalActions[T_freesavedel])
     (*ThotLocalActions[T_freesavedel]) ();
+  /* free all context related to search commands */
+  if (ThotLocalActions[T_freesearch])
+    (*ThotLocalActions[T_freesearch]) ();
   FreeDocColors ();
   FreeAllMessages ();
   Prof_FreeTable ();
@@ -376,16 +422,21 @@ void TtaQuit ()
   TteFreeAllEventsList ();
   TtaFreeAllCatalogs ();
   ConfigFree ();
+  FreeAllPicCache ();
 #endif /* NODISPLAY */
   FreeAll ();
   TtaFreeAppRegistry ();
   if (AppClosingFunction)
     (*AppClosingFunction) ();
   {
-#ifdef _GTK
-    gtk_exit (0);
-#endif /* _GTK */	
-    exit (0);
+#if defined(_GTK) && !defined(NODISPLAY)
+     gtk_exit (0);
+#endif /* _GTK && !NODISPLAY */	
+#if defined(_WX) && !defined(NODISPLAY)
+     /*     wxExit();*/
+#else /* _WX && !defined(NODISPLAY) */     
+     exit (0);
+#endif /* _WX && !defined(NODISPLAY) */     
   }
 }
 
@@ -562,10 +613,14 @@ void ThotExit (int result)
       ErrorHandler ();
    else
      {
-#ifdef _GTK
-       gtk_exit (result);
-#endif /* _GTK */
-       exit (result);
+#if defined(_GTK) && !defined(NODISPLAY)
+     gtk_exit (result);
+#endif /* _GTK && !NODISPLAY */	
+#if defined(_WX) && !defined(NODISPLAY)
+     wxExit();
+#else /* _WX && !defined(NODISPLAY) */     
+     exit (result);
+#endif /* _WX && !defined(NODISPLAY) */     
      }
 }
 
