@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -15,7 +15,6 @@
 
 #include "thot_gui.h"
 #include "thot_sys.h"
-
 #include "corrmsg.h"
 #include "libmsg.h"
 #include "message.h"
@@ -29,10 +28,10 @@
 #include "corrmenu.h"
 #include "appdialogue.h"
 #include "frame.h"
-#ifdef _WINDOWS
+#ifdef _WINGUI
 #include "wininclude.h"
 #include "resource.h"
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
 
 #undef THOT_EXPORT
 #define THOT_EXPORT extern
@@ -40,7 +39,7 @@
 #include "appdialogue_tv.h"
 #include "frame_tv.h"
 #undef THOT_EXPORT
-#define THOT_EXPORT
+#define THOT_EXPORT extern
 #include "spell_tv.h"
 
 /* les variables locales */
@@ -64,70 +63,70 @@ static PtrDocument  pDocSel;
 #include "views_f.h"
 #include "word_f.h"
 
-#ifdef _WINDOWS
+#ifdef _WINGUI
 #define IDC_WORDBUTTON    20000
 #define IDC_LANGEDIT      20002
 static ThotWindow   SpellChecker = NULL;
-static ThotWindow   hwnListWords;
-static ThotWindow   hwndCurrentWord;
+static ThotWindow   WordList;
+static ThotWindow   EditWord;
 static ThotWindow   hwndLanguage;
 static char         currentWord [MAX_WORD_LEN];
 static ThotWindow   wordButton;
 static UINT         itemIndex;
 static int          iLocation;
 static int          iIgnore;
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
 
 
 /*----------------------------------------------------------------------
    RemoveSpellForm fait disparaitre les formulaires correction    
    s'ils concernent le document pDoc et s'ils sont a l'ecran.      
   ----------------------------------------------------------------------*/
-static void         RemoveSpellForm ()
+static void RemoveSpellForm ()
 {
   if (ChkrRange != NULL)
     {
       FreeSearchContext (&ChkrRange);
-#ifdef _WINDOWS
+#ifdef _WINGUI
       EndDialog (SpellChecker, ID_DONE);
       hwndLanguage = NULL;
       SpellChecker = NULL;
-#else /* _WINDOWS */
+#else /* _WINGUI */
       TtaDestroyDialogue (SpellingBase + ChkrFormCorrect);
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
     }
 }
 
 /*----------------------------------------------------------------------
    DisplayWords prepare le selecteur                             
   ----------------------------------------------------------------------*/
-static void         DisplayWords (void)
+static void DisplayWords (void)
 {
-#ifdef _WINDOWS
+#ifdef _WINGUI
   int i;
   
   SetWindowText (wordButton, ChkrCorrection[0]);
-  SendMessage (hwnListWords, LB_RESETCONTENT, 0, 0);
+  SendMessage (WordList, LB_RESETCONTENT, 0, 0);
   if ((strcmp (ChkrCorrection[1], "$")) == 0)
     {
       currentWord [0] = EOS;
-      SetWindowText (hwndCurrentWord, "");
-      SendMessage (hwnListWords, LB_ADDSTRING, 0, (LPARAM) (""));  
+      SetWindowText (EditWord, "");
+      SendMessage (WordList, LB_ADDSTRING, 0, (LPARAM) (""));  
     }
   else
     {
       sprintf (currentWord, "%s", ChkrCorrection[1]);
-      SetWindowText (hwndCurrentWord, ChkrCorrection[1]);
+      SetWindowText (EditWord, ChkrCorrection[1]);
       for (i = 1; (i <= NC && strcmp (ChkrCorrection[i], "$") != 0);
 	   i++)
 	{
-	  SendMessage (hwnListWords, LB_INSERTSTRING, i - 1,
+	  SendMessage (WordList, LB_INSERTSTRING, i - 1,
 		       (LPARAM) ((LPCTSTR)ChkrCorrection[i]));  
 	}
     }
   OldNC = NC;
 
-#else /* _WINDOWS */
+#else /* _WINGUI */
    int                 i, indx, length;
    char               *entry;
    char                BufMenu[MAX_TXT_LEN];
@@ -152,7 +151,7 @@ static void         DisplayWords (void)
    TtaNewSelector (SpellingBase + ChkrSelectProp,
 		   SpellingBase + ChkrFormCorrect,
 		   TtaGetMessage (CORR, Correct), i - 1,
-		   ((i < 2) ? "" : BufMenu), 3, entry, TRUE, FALSE);
+		   ((i < 2) ? (char *)"" : BufMenu), 3, entry, TRUE, FALSE);
    /* selectionner la proposition 0 dans le selecteur - si elle existe */
    if (strcmp (ChkrCorrection[1], "$") != 0)
       TtaSetSelector (SpellingBase + ChkrSelectProp, -1, ChkrCorrection[1]);
@@ -161,15 +160,16 @@ static void         DisplayWords (void)
 
    /* le formulaire est maintenant pret a etre affiche' */
    OldNC = NC;
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
 }
 
 
-#ifdef _WINDOWS
+#ifdef _WINGUI
 /*-----------------------------------------------------------------------
  SpellCheckDlgProc
  ------------------------------------------------------------------------*/
-static LRESULT CALLBACK SpellCheckDlgProc (ThotWindow hwnDlg, UINT msg, WPARAM wParam, LPARAM lParam)
+static LRESULT CALLBACK SpellCheckDlgProc (ThotWindow hwnDlg, UINT msg,
+					   WPARAM wParam, LPARAM lParam)
 {
   ThotBool ok;	  
   int  val;
@@ -181,7 +181,7 @@ static LRESULT CALLBACK SpellCheckDlgProc (ThotWindow hwnDlg, UINT msg, WPARAM w
       SpellChecker = hwnDlg;
       hwndLanguage = GetDlgItem (hwnDlg, IDC_LANG);
       wordButton = GetDlgItem (hwnDlg, IDC_CURWORD);
-	  SetWindowText (hwnDlg, TtaGetMessage (CORR, Correct));
+      SetWindowText (hwnDlg, TtaGetMessage (CORR, Correct));
       SetWindowText (GetDlgItem (hwnDlg, IDC_LABEL), TtaGetMessage (CORR, Correct));
       SetWindowText (GetDlgItem (hwnDlg, IDC_BEFORE), TtaGetMessage (LIB, TMSG_BEFORE_SEL));
       SetWindowText (GetDlgItem (hwnDlg, IDC_WITHIN), TtaGetMessage (LIB, TMSG_WITHIN_SEL));
@@ -204,16 +204,12 @@ static LRESULT CALLBACK SpellCheckDlgProc (ThotWindow hwnDlg, UINT msg, WPARAM w
       SetWindowText (GetDlgItem (hwnDlg, IDC_IGNOREGROUP), TtaGetMessage (CORR, Ignore));
       
       SetWindowText (GetDlgItem (hwnDlg, ID_DONE), TtaGetMessage (LIB, TMSG_DONE));
-      
-      hwnListWords = CreateWindow ("listbox", NULL, WS_CHILD | WS_VISIBLE | LBS_STANDARD,
-				   13, 72, 150, 70, hwnDlg, (HMENU) 1, 
-				   (HINSTANCE) GetWindowLong (hwnDlg, GWL_HINSTANCE), NULL);
-      
-      hwndCurrentWord = CreateWindow ("EDIT", NULL, WS_CHILD | WS_VISIBLE | ES_LEFT | WS_BORDER,
-				      13, 146, 150, 20, hwnDlg, (HMENU) IDC_LANGEDIT, 
-				      (HINSTANCE) GetWindowLong (hwnDlg, GWL_HINSTANCE), NULL);
-      
-      
+      WordList = GetDlgItem (hwnDlg, IDC_WORDLIST);
+	  WIN_SetDialogfont(WordList);
+      /*BuildWordList ();*/
+	  EditWord = GetDlgItem (hwnDlg, IDC_WORDEDIT);
+      WIN_SetDialogfont(EditWord);
+       
       CheckRadioButton (hwnDlg, IDC_BEFORE, IDC_WHOLEDOC, IDC_AFTER);
       CheckDlgButton (hwnDlg, IDC_IGNORE1, BST_CHECKED);
       CheckDlgButton (hwnDlg, IDC_IGNORE2, BST_CHECKED);
@@ -223,7 +219,7 @@ static LRESULT CALLBACK SpellCheckDlgProc (ThotWindow hwnDlg, UINT msg, WPARAM w
       SetDlgItemInt (hwnDlg, IDC_EDITPROPOSALS, 3, FALSE);
       SetDlgItemText (hwnDlg, IDC_EDITIGNORE, RejectedChar);
       iLocation = 2;
-      SetWindowText (hwndCurrentWord, "");
+      SetWindowText (EditWord, "");
       DisplayWords ();
       break;
       
@@ -235,43 +231,39 @@ static LRESULT CALLBACK SpellCheckDlgProc (ThotWindow hwnDlg, UINT msg, WPARAM w
       break;      
       
     case WM_COMMAND:
-      if (LOWORD (wParam) == 1 && HIWORD (wParam) == LBN_SELCHANGE) {
-	itemIndex = SendMessage (hwnListWords, LB_GETCURSEL, 0, 0);
-	itemIndex = SendMessage (hwnListWords, LB_GETTEXT, itemIndex, (LPARAM) currentWord);
-	SetDlgItemText (hwnDlg, IDC_LANGEDIT, currentWord);
-      } else if (LOWORD (wParam) == 1 && HIWORD (wParam) == LBN_DBLCLK) {
-	if (LB_ERR == (itemIndex = SendMessage (hwnListWords, LB_GETCURSEL, 0, 0L)))
-	  break;
-	itemIndex = SendMessage (hwnListWords, LB_GETTEXT, itemIndex, (LPARAM) currentWord);
-	SetDlgItemText (hwnDlg, IDC_LANGEDIT, currentWord);
-	ThotCallback (SpellingBase + ChkrSelectProp, STRING_DATA, currentWord);
-	ThotCallback (SpellingBase + ChkrMenuOR, INTEGER_DATA, (char*) iLocation);
-	ThotCallback (SpellingBase + ChkrFormCorrect, INTEGER_DATA, (char*) 3);
-	if (iLocation == 3) {
-	  CheckRadioButton (hwnDlg, IDC_BEFORE, IDC_WHOLEDOC, IDC_AFTER);
-	  iLocation = 2;
-	}
-	return 0;
-      } 
-      if (HIWORD (wParam) == EN_UPDATE)
-	{
-	  if (LOWORD (wParam) == IDC_EDITPROPOSALS)
-	    {
-	      val = GetDlgItemInt (hwnDlg, IDC_EDITPROPOSALS, &ok, TRUE);
-	      if (ok)
-		ThotCallback (SpellingBase + ChkrCaptureNC, INTEGER_DATA, (char*) val);
-	    }
-	  else if (LOWORD (wParam) == IDC_EDITIGNORE)
-	    {
-	      GetDlgItemText (hwnDlg, IDC_EDITIGNORE, currentRejectedchars, sizeof (currentRejectedchars) + 1);
-	      ThotCallback (SpellingBase + ChkrSpecial, STRING_DATA, currentRejectedchars);
-	    }
-	  else if (LOWORD (wParam) == IDC_LANGEDIT) 
-	    GetDlgItemText (hwnDlg, IDC_LANGEDIT, currentWord, sizeof (currentWord) + 1);
-	}
-      
       switch (LOWORD (wParam))
 	{
+	case IDC_EDITPROPOSALS:
+	  val = GetDlgItemInt (hwnDlg, IDC_EDITPROPOSALS, &ok, TRUE);
+	  if (ok)
+		ThotCallback (SpellingBase + ChkrCaptureNC, INTEGER_DATA, (char*) val);
+	  break;
+	case IDC_EDITIGNORE:
+	  GetDlgItemText (hwnDlg, IDC_EDITIGNORE, currentRejectedchars, sizeof (currentRejectedchars) + 1);
+	  ThotCallback (SpellingBase + ChkrSpecial, STRING_DATA, currentRejectedchars);
+	  break;
+	case IDC_WORDLIST:
+	  itemIndex = SendMessage (WordList, LB_GETCURSEL, 0, 0);
+	  itemIndex = SendMessage (WordList, LB_GETTEXT, itemIndex,
+				   (LPARAM) currentWord);
+	  SetDlgItemText (hwnDlg, IDC_WORDEDIT, currentWord);
+	  if (HIWORD (wParam) == LBN_DBLCLK)
+	  {
+	    /* double click activate the replace */
+	    ThotCallback (SpellingBase + ChkrSelectProp, STRING_DATA, currentWord);
+	    ThotCallback (SpellingBase + ChkrMenuOR, INTEGER_DATA, (char*) iLocation);
+	    ThotCallback (SpellingBase + ChkrFormCorrect, INTEGER_DATA, (char*) 3);
+	    if (iLocation == 3)
+		{
+	      CheckRadioButton (hwnDlg, IDC_BEFORE, IDC_WHOLEDOC, IDC_AFTER);
+	      iLocation = 2;
+		}
+	    return 0;
+	  }
+	  break;
+	case IDC_WORDEDIT:
+	    GetDlgItemText (hwnDlg, IDC_WORDEDIT, currentWord, sizeof (currentWord) + 1);
+	  break;
 	case IDC_BEFORE:
 	  iLocation = 0;
 	  break;
@@ -308,7 +300,8 @@ static LRESULT CALLBACK SpellCheckDlgProc (ThotWindow hwnDlg, UINT msg, WPARAM w
 	  ThotCallback (SpellingBase + ChkrSelectProp, STRING_DATA, currentWord);
 	  ThotCallback (SpellingBase + ChkrMenuOR, INTEGER_DATA, (char*) iLocation);
 	  ThotCallback (SpellingBase + ChkrFormCorrect, INTEGER_DATA, (char*) 1);
-	  if (iLocation == 3) {
+	  if (iLocation == 3)
+	  {
 	    CheckRadioButton (hwnDlg, IDC_BEFORE, IDC_WHOLEDOC, IDC_AFTER);
 	    iLocation = 2;
 	  }
@@ -349,21 +342,23 @@ static LRESULT CALLBACK SpellCheckDlgProc (ThotWindow hwnDlg, UINT msg, WPARAM w
 	  
 	case IDC_WORDBUTTON:
 	  GetWindowText (wordButton, currentWord, MAX_WORD_LEN);
-	  SetWindowText (hwndCurrentWord, currentWord);
+	  SetWindowText (EditWord, currentWord);
 	  break;
 	  
+	case IDCANCEL:
 	case ID_DONE:
 	  SpellChecker = NULL;
 	  hwndLanguage = NULL;
 	  EndDialog (hwnDlg, ID_DONE);
 	  break;	  
-	}
+
+	  }
       break;
     default: return FALSE;
     }
   return TRUE;
 }
-#else /* _WINDOWS */
+#else /* _WINGUI */
 
 /*----------------------------------------------------------------------
   UnsetEntryMenu
@@ -378,11 +373,11 @@ void UnsetEntryMenu (int ref, int ent)
       TtaRedrawMenuEntry (ref, ent, NULL, InactiveB_Color, 0);
    else
      {
-	FontIdentifier ('L', 'T', 2, 11, 1, text, fontname);
-	TtaRedrawMenuEntry (ref, ent, fontname, -1, 0);
+	GetFontIdentifier ('L', 'T', 2, 11, (TypeUnit)1, text, fontname);
+	TtaRedrawMenuEntry (ref, ent, fontname, (ThotColor)-1, 0);
      }
 }
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
 
 /*----------------------------------------------------------------------
   TtcSpellCheck  active le formulaire de correction                
@@ -391,13 +386,13 @@ void TtcSpellCheck (Document doc, View view)
 {
    PtrDocument         document;
    int                 i;
-#ifndef _WINDOWS
+#ifndef _WINGUI
    PtrElement          pEl1, pElN;
    int                 c1, cN;
    int                 indx;
    char                BufMenu[MAX_TXT_LEN];
    ThotBool            ok;
-#endif /* !_WINDOWS */
+#endif /* !_WINGUI */
 
    /* SpecialChars = "@#$&+~"; */
 
@@ -407,7 +402,7 @@ void TtcSpellCheck (Document doc, View view)
       return;
    SpellCheckLoadResources ();
 
-#ifndef _WINDOWS 
+#ifndef _WINGUI 
    if (ChkrRange != NULL)
       RemoveSpellForm ();
    TtaDestroyDialogue (SpellingBase + ChkrFormCorrect);
@@ -425,18 +420,18 @@ void TtcSpellCheck (Document doc, View view)
    TtaNewSheet (SpellingBase + ChkrFormCorrect, TtaGetViewFrame (doc, view), 
 		TtaGetMessage (CORR, Correct), 4, BufMenu, FALSE, 4, 'L',
 		D_DONE);
-
+   TtaSetDefaultButton (SpellingBase + ChkrFormCorrect, 3);
    /* initialise le champ langue de correction courante */
    TtaNewLabel (SpellingBase + ChkrLabelLanguage,
 		SpellingBase + ChkrFormCorrect, " ");
-#endif /* !_WINDOWS */
+#endif /* !_WINGUI */
 
    /* Afficher une liste de mots EMPTY */
    strcpy (ChkrCorrection[0], " ");
    for (i = 1; i <= NC; i++)
       strcpy (ChkrCorrection[i], "$");
 
-#ifndef _WINDOWS 
+#ifndef _WINGUI 
    DisplayWords ();
 
    /* creer le sous-menu OU dans la feuille OPTIONS */
@@ -465,9 +460,9 @@ void TtcSpellCheck (Document doc, View view)
    else
      {
 
-	TtaRedrawMenuEntry (SpellingBase + ChkrMenuOR, 0, NULL, -1, 1);
-	TtaRedrawMenuEntry (SpellingBase + ChkrMenuOR, 1, NULL, -1, 1);
-	TtaRedrawMenuEntry (SpellingBase + ChkrMenuOR, 2, NULL, -1, 1);
+	TtaRedrawMenuEntry (SpellingBase + ChkrMenuOR, 0, NULL, (ThotColor)-1, 1);
+	TtaRedrawMenuEntry (SpellingBase + ChkrMenuOR, 1, NULL, (ThotColor)-1, 1);
+	TtaRedrawMenuEntry (SpellingBase + ChkrMenuOR, 2, NULL, (ThotColor)-1, 1);
      }
 
    /* initialise le champ termine de correction courante */
@@ -486,7 +481,7 @@ void TtcSpellCheck (Document doc, View view)
    indx += strlen (&BufMenu[indx]) + 1;
    sprintf (&BufMenu[indx], "B%s", TtaGetMessage (CORR, Romans));
    indx += strlen (&BufMenu[indx]) + 1;
-   printf (&BufMenu[indx], "B%s", TtaGetMessage (CORR, Specials));
+   sprintf (&BufMenu[indx], "B%s", TtaGetMessage (CORR, Specials));
    TtaNewToggleMenu (SpellingBase + ChkrMenuIgnore,
 		     SpellingBase + ChkrFormCorrect,
 		     TtaGetMessage (CORR, Ignore),
@@ -496,7 +491,7 @@ void TtcSpellCheck (Document doc, View view)
    TtaNewTextForm (SpellingBase + ChkrSpecial, SpellingBase + ChkrFormCorrect,
 		   NULL, 20, 1, TRUE);
    TtaSetTextForm (SpellingBase + ChkrSpecial, "@#$&+~");
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
 
    /* ne pas ignorer les mots en capitale, chiffres romains */
    /* ou contenant des chiffres arabes,  certains car. speciaux */
@@ -508,7 +503,7 @@ void TtcSpellCheck (Document doc, View view)
    IgnoreRoman = TRUE;
    IgnoreSpecial = TRUE;
 
-#ifndef _WINDOWS 
+#ifndef _WINGUI 
    /* Selectionne les types de mots a ne ignorer par defaut */
    TtaSetToggleMenu (SpellingBase + ChkrMenuIgnore, 0, IgnoreUppercase);
    TtaSetToggleMenu (SpellingBase + ChkrMenuIgnore, 1, IgnoreArabic);
@@ -520,7 +515,7 @@ void TtcSpellCheck (Document doc, View view)
 
    /* Et enfin, afficher le formulaire de CORRECTION */
    TtaShowDialogue (SpellingBase + ChkrFormCorrect, TRUE);
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
 
    /* Indique que c'est une nouvelle correction qui debute */
    FirstStep = TRUE;
@@ -532,7 +527,7 @@ void TtcSpellCheck (Document doc, View view)
    /* ne cree pas inutilement le dictionnaire fichier */
    TtaLoadDocumentDictionary (document, (int*) &ChkrFileDict, FALSE);
 
-#ifdef _WINDOWS  
+#ifdef _WINGUI  
   /* to have the same behavior as under Unix, we need to destroy the
      dialog if it already existed */
   if (SpellChecker) 
@@ -542,7 +537,7 @@ void TtcSpellCheck (Document doc, View view)
       hwndLanguage = NULL;
     }
   DialogBox (hInstance, MAKEINTRESOURCE (SPELLCHECKDIALOG), NULL, (DLGPROC) SpellCheckDlgProc);
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
 }
 
 
@@ -550,7 +545,7 @@ void TtcSpellCheck (Document doc, View view)
   ResetCheckInDocument
   cleans up the checker domain if it refers a closed document.
   ----------------------------------------------------------------------*/
-void         ResetCheckInDocument (PtrDocument pDoc)
+void ResetCheckInDocument (PtrDocument pDoc)
 {
   if (ChkrRange && pDoc == ChkrRange->SDocument)
     RemoveSpellForm ();
@@ -560,7 +555,7 @@ void         ResetCheckInDocument (PtrDocument pDoc)
 /*----------------------------------------------------------------------
    SetProposals                                                 
   ----------------------------------------------------------------------*/
-static void         SetProposals (Language language)
+static void SetProposals (Language language)
 {
    char             Lab[200];
 
@@ -577,19 +572,19 @@ static void         SetProposals (Language language)
    /* afficher la langue de correction courante */
    sprintf (Lab, "%s: %s", TtaGetMessage (LIB, TMSG_LANGUAGE),
 	     TtaGetLanguageName (ChkrLanguage));
-#ifdef _WINDOWS
+#ifdef _WINGUI
    SetWindowText (hwndLanguage, Lab);
-#else  /* !_WINDOWS */
+#else  /* !_WINGUI */
    TtaNewLabel (SpellingBase + ChkrLabelLanguage,
 		SpellingBase + ChkrFormCorrect, Lab);
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
 }
 
 
 /*----------------------------------------------------------------------
    StartSpellChecker retourne 0 si c'est OK et -1 en cas d'erreur          
   ----------------------------------------------------------------------*/
-static ThotBool     StartSpellChecker ()
+static ThotBool StartSpellChecker ()
 {
    ThotBool            ok;
 
@@ -603,27 +598,23 @@ static ThotBool     StartSpellChecker ()
      {
 	FirstStep = FALSE;
 	ChkrElement = NULL;
-	ChkrIndChar = 0;
+	ChkrIndChar = 1;
      }
 
    /* en tenant compte des options choisies (par defaut) */
-   NextSpellingError (CurrentWord, ChkrFileDict);
+   NextSpellingError ((unsigned char*)CurrentWord, ChkrFileDict);
    if (CurrentWord[0] != EOS)
       /* calculer la 1ere liste des propositions ds selecteur */
       SetProposals (ChkrLanguage);
    else
      {
        /* Correction TERMINEE */
-#ifdef _WINDOWS
-       MessageBox (NULL, TtaGetMessage (LIB, TMSG_NOT_FOUND), \
+#ifdef _WINGUI
+       MessageBox (NULL, TtaGetMessage (CORR, END_CHECK), \
 		   "Spell checking", MB_OK | MB_ICONINFORMATION);
-#else  /* _WINDOWS */
-       TtaDisplaySimpleMessage (INFO, CORR, END_CHECK);
-       /* message 'Pas trouve' dans le formulaire */
-       TtaNewLabel (SpellingBase + ChkrLabelNotFound,
-		    SpellingBase + ChkrFormCorrect,
-		    TtaGetMessage (LIB, TMSG_NOT_FOUND));
-#endif /* _WINDOWS */
+#else  /* _WINGUI */
+       TtaDisplayMessage (CONFIRM, TtaGetMessage (CORR, END_CHECK), NULL);
+#endif /* _WINGUI */
        FirstStep = TRUE;
        ok = FALSE;
      }
@@ -634,7 +625,7 @@ static ThotBool     StartSpellChecker ()
 /*----------------------------------------------------------------------
    ApplyCommand traite l'action de correction demandee.             
   ----------------------------------------------------------------------*/
-static void         ApplyCommand (int val)
+static void ApplyCommand (int val)
 {
    ThotBool            change;
    PtrDocument         document;
@@ -667,15 +658,15 @@ static void         ApplyCommand (int val)
 	     if (CorrectWord[0] != EOS &&
 		 (strcmp (CorrectWord, ChkrCorrection[1]) != 0))
 	       {
-		 /* ajouter le mot corroge dans le dictionaire */
-		 if (!CheckWord (CorrectWord, ChkrLanguage, ChkrFileDict))
-		   AddWord (CorrectWord, &ChkrFileDict);
+		 /* ajouter le mot corrige dans le dictionaire */
+		 if (!CheckWord ((unsigned char*)CorrectWord, ChkrLanguage, ChkrFileDict))
+		   AddWord ((unsigned char*)CorrectWord, &ChkrFileDict);
 	       }
 	     else
 	       {
 		 /* ajouter le mot courant dans le dictionnaire */
-		 if (!CheckWord (CurrentWord, ChkrLanguage, ChkrFileDict))
-		   AddWord (CurrentWord, &ChkrFileDict);
+		 if (!CheckWord ((unsigned char*)CurrentWord, ChkrLanguage, ChkrFileDict))
+		   AddWord ((unsigned char*)CurrentWord, &ChkrFileDict);
 	       }
 	     }
 	   break;
@@ -684,7 +675,7 @@ static void         ApplyCommand (int val)
 	   if (ToReplace)	/* CorrectWord est rempli */
 	     {		/* et ce n'est pas 1er lancement */
 	       if (ChkrElement != NULL && CurrentWord[0] != EOS)
-		 WordReplace (CurrentWord, CorrectWord);
+		 WordReplace ((unsigned char*)CurrentWord, (unsigned char*)CorrectWord);
 	       ToReplace = FALSE;
 	     }
 	   break;
@@ -694,11 +685,11 @@ static void         ApplyCommand (int val)
 	     {		/* et ce n'est pas 1er lancement */
 	       if (ChkrElement != NULL && CurrentWord[0] != EOS)
 		 {
-		   WordReplace (CurrentWord, CorrectWord);
+		   WordReplace ((unsigned char*)CurrentWord, (unsigned char*)CorrectWord);
 		   /* mettre CorrectWord dans le dictionnaire */
-		   if (!CheckWord (CorrectWord, ChkrLanguage, ChkrFileDict))
+		   if (!CheckWord ((unsigned char*)CorrectWord, ChkrLanguage, ChkrFileDict))
 		     /* mettre ce nouveau mot dans le dictionnaire */
-		     AddWord (CorrectWord, &ChkrFileDict);
+		     AddWord ((unsigned char*)CorrectWord, &ChkrFileDict);
 		 }
 	       ToReplace = FALSE;
 	     }
@@ -754,9 +745,9 @@ void CallbackChecker (int ref, int dataType, char *data)
 	if (!IgnoreSpecial)
 	  {
 	    IgnoreSpecial = TRUE;
-#ifndef _WINDOWS 
+#ifndef _WINGUI 
 	    TtaSetToggleMenu (SpellingBase + ChkrMenuIgnore, 3, IgnoreSpecial);
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
 	  }
 	break;
       case ChkrCaptureNC:
@@ -789,12 +780,12 @@ void CallbackChecker (int ref, int dataType, char *data)
 	  {
 	    InitSearchDomain ((int) data, ChkrRange);
 	    /* On prepare la recheche suivante */
-#ifndef _WINDOWS
+#ifndef _WINGUI
 	    if (ChkrRange->SStartToEnd)
 	      TtaSetMenuForm (SpellingBase + ChkrMenuOR, 2);
 	    else
 	      TtaSetMenuForm (SpellingBase + ChkrMenuOR, 0);
-#endif /* !_WINDOWS */
+#endif /* !_WINGUI */
 	  }
 	else if (ChkrRange != NULL)
 	  /* Est-ce que le document vient de recevoir la selection */
@@ -802,14 +793,15 @@ void CallbackChecker (int ref, int dataType, char *data)
 	    {
 	      /* Est-ce encore vrai */
 	      GetCurrentSelection (&pDocSel, &pEl1, &pElN, &c1, &cN);
-#ifndef _WINDOWS
-	      if (pDocSel == ChkrRange->SDocument) {
-		/* Il faut reactiver les entree */
-		TtaRedrawMenuEntry (SpellingBase + ChkrMenuOR, 0, NULL, -1, 1);
-		TtaRedrawMenuEntry (SpellingBase + ChkrMenuOR, 1, NULL, -1, 1);
-		TtaRedrawMenuEntry (SpellingBase + ChkrMenuOR, 2, NULL, -1, 1);
-	      } 
-#endif /* !_WINDOWS */
+#ifndef _WINGUI
+	      if (pDocSel == ChkrRange->SDocument)
+		{
+		  /* Il faut reactiver les entree */
+		  TtaRedrawMenuEntry (SpellingBase + ChkrMenuOR, 0, NULL, (ThotColor)-1, 1);
+		  TtaRedrawMenuEntry (SpellingBase + ChkrMenuOR, 1, NULL, (ThotColor)-1, 1);
+		  TtaRedrawMenuEntry (SpellingBase + ChkrMenuOR, 2, NULL, (ThotColor)-1, 1);
+		} 
+#endif /* !_WINGUI */
 	    }
 	break;
       case ChkrSelectProp:
@@ -833,12 +825,12 @@ void CallbackChecker (int ref, int dataType, char *data)
 /*----------------------------------------------------------------------
   SpellCheckLoadResources loads resouces of the spellchecker
   ----------------------------------------------------------------------*/
-void                SpellCheckLoadResources ()
+void SpellCheckLoadResources ()
 {
    if (SpellingBase == 0)
      {
 	CORR = TtaGetMessageTable ("corrdialogue", MSG_MAX_CHECK);
-	SpellingBase = TtaSetCallback (CallbackChecker, ChkrMaxDialogue);
+	SpellingBase = TtaSetCallback ((Proc)CallbackChecker, ChkrMaxDialogue);
 	/* Initialisation des variables globales */
 	ParametrizeChecker ();
 	ChkrFileDict = NULL;
