@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -11,11 +11,12 @@
  * Author: I. Vatton (INRIA)
  *
  */
-
+#include "thot_gui.h"
 #include "thot_sys.h"
 #include "constmedia.h"
 #include "typemedia.h"
 #include "appdialogue.h"
+#include "picture.h"
 
 #define THOT_EXPORT extern
 #include "boxes_tv.h"
@@ -32,16 +33,16 @@
 #include "textcommands_f.h"
 #include "xwindowdisplay_f.h"
 
+#ifdef _GL
+#include "glwindowdisplay.h"
+#endif /*_GL*/
 
 /*----------------------------------------------------------------------
-   VerticalScroll effectue un scroll en avant (delta > 0) ou en arriere   
-   (delta<0).                                              
-   Deplacement par raster-op d'une partie de la fenetre et 
-   reaffichage du reste de la fenetre :                    
-   - soit en avant par RedrawFrameBottom,                       
-   - soit en arriere par RedrawFrameTop.                     
-   Le parametre selection indique s'il faut ge'rer la      
-   selection (valeur 1) ou non (valeur 0).                 
+   VerticalScroll scrolls forward (delta > 0) or backward (delta < 0).
+   Moves by raster-op a part of the window and displays the new part:                  
+   - forwardr by RedrawFrameBottom,                       
+   - backward by RedrawFrameTop.                     
+   The parameter selection is 1 if threre is a selection, 0 if not.
   ----------------------------------------------------------------------*/
 void VerticalScroll (int frame, int delta, int selection)
 {
@@ -90,7 +91,7 @@ void VerticalScroll (int frame, int delta, int selection)
 		    {
 		     if (delta > 0)
 		       {
-			  /* SCROLL AVANT */
+			  /* SCROLL forward */
 			  if (delta > max)
 			     delta = max;
 			  y = delta;
@@ -99,12 +100,13 @@ void VerticalScroll (int frame, int delta, int selection)
 			  Scroll (frame, width, height, 0, y, 0, 0);
 			  height = pFrame->FrYOrg + hframe;
 			  DefClip (frame, pFrame->FrXOrg, height,
-				   pFrame->FrXOrg + lframe, height + delta);
+				   pFrame->FrXOrg + lframe, 
+				   height + delta);
 			  add = RedrawFrameBottom (frame, delta, NULL);
 		       }
 		     else
 		       {
-			  /* SCROLL ARRIERE */
+			  /* SCROLL backward */
 			  if (delta < y)
 			     delta = y;
 			  height = hframe + delta;
@@ -116,12 +118,11 @@ void VerticalScroll (int frame, int delta, int selection)
 				   pFrame->FrXOrg + lframe, height);
 			  add = RedrawFrameTop (frame, -delta);
 		       }
+		     /* recompute scrolls */
+		     CheckScrollingWidth (frame);
+		     UpdateScrollbars (frame);
 		    }
 
-		  /* recompute scrolls */
-		  CheckScrollingWidth (frame);
-		  UpdateScrollbars (frame);
-
 		  if (selection != 0)
 		     if (add)
 		       {
@@ -349,7 +350,7 @@ int PositionAbsBox (int frame, int *nbCharBegin, int *nbCharEnd, int *total)
    dans la fenetree^tre frame.                                     
    La proce'dure rend :                                            
    - Xpos : la position en X du premier cararte`re visualise'.     
-   - Ypos : la position en X du premier cararte`re visualise'.     
+   - Ypos : la position en Y du premier cararte`re visualise'.     
    - width : la largeur de la portion de l'arbre visualise'e.    
    - height : la hauteur de la portion de l'arbre visualise'e.    
   ----------------------------------------------------------------------*/
@@ -384,6 +385,8 @@ void ComputeDisplayedChars (int frame, int *Xpos, int *Ypos, int *width, int *he
        pBox = pFrame->FrAbstractBox->AbBox;
        if (pBox == NULL)
 	 return;
+       if (pFrame->FrAbstractBox->AbElement == NULL)
+	 return;
      }
    else
       return;
@@ -442,22 +445,21 @@ void ComputeDisplayedChars (int frame, int *Xpos, int *Ypos, int *width, int *he
       /* there is nothing on the left */
       *width = l - *Xpos;
 
-
    /***     Vertical scrolling             ***/
    /*      +-------------+                   */
-   /*      |  Document   |a                  */
+   /*      |  Document   |min                */
    /*    +-+-------------+-+     +-+         */
    /*    |    Concrete     |\    | |         */
-   /*    |                 | \   | |a        */
+   /*    |                 | \   | |max      */
    /*    |+---------------+|  \  | |         */
    /*    ||               ||\  --|-|         */
    /*    ||   ViewFrame   || ====|=|slider   */
    /*    ||               ||/  --|-|         */
    /*    |+---------------+|  /  | |         */
-   /*    |                 | /   | |b        */
+   /*    |                 | /   | |max      */
    /*    |     Image       |/    | |         */
    /*    +-+-------------+-+     +-+         */
-   /*      |             |b                  */
+   /*      |             |max                */
    /*      +-------------+                   */
 
    /** Compute the part of the document which is not formatted **/
@@ -465,25 +467,8 @@ void ComputeDisplayedChars (int frame, int *Xpos, int *Ypos, int *width, int *he
        (!pFrame->FrAbstractBox->AbTruncatedTail))
      {
 	/* The whole document is formatted */
-	if (pFrame->FrYOrg - pBox->BxYOrg <= 0)
-	   /* The beginning of the document */
-	   min = 0;
-	else if (pFrame->FrYOrg >= pBox->BxYOrg)
-	   /* le debordement du haut n'est pas visualible */
-	   min = 0;
-	else
-	   min = - pFrame->FrYOrg + pBox->BxYOrg;	/* marge haut */
-
-	if (pBox->BxYOrg + pBox->BxHeight <= pFrame->FrYOrg + h)
-	   /* Le bas de l'Picture Concrete + le debordement du bas sont
-	      visibles */
-	   max = 0;
-	else if (pBox->BxYOrg + pBox->BxHeight >= pFrame->FrYOrg + h)
-	   /* le debordement du bas n'est pas visualible */
-	   max = 0;
-	else
-	   /* marge restant en bas */
-	   max = pBox->BxYOrg + pBox->BxHeight - pFrame->FrYOrg - h;
+       min = 0;
+       max = 0;
      }
    else
      {
@@ -501,9 +486,11 @@ void ComputeDisplayedChars (int frame, int *Xpos, int *Ypos, int *width, int *he
 	     while (pBoxFirst->BxAbstractBox->AbPresentationBox &&
 		    pBoxFirst != pBoxLast)
 		pBoxFirst = pBoxFirst->BxNext;
-	     while (pBoxLast->BxAbstractBox->AbPresentationBox &&
-		    pBoxFirst != pBoxLast)
-		pBoxLast = pBoxLast->BxPrevious;
+	     if (pBoxFirst != pBoxLast)
+	       while (pBoxLast->BxAbstractBox->AbPresentationBox &&
+		      pBoxLast->BxPrevious->BxAbstractBox &&
+		      pBoxFirst != pBoxLast)
+		 pBoxLast = pBoxLast->BxPrevious;
 
 	     /* Evalue la hauteur et la position Y par rapport a l'image
 		abstraite */
@@ -557,33 +544,64 @@ void ComputeDisplayedChars (int frame, int *Xpos, int *Ypos, int *width, int *he
 }
 
 /*----------------------------------------------------------------------
-   ShowBox force la position de la boi^te dans la fenetree^tre        
-   affiche'e sur l'e'cran.                                 
-   La parame`tre pBox identifie la boi^te a` montrer.      
-   Le parame`tre position indique s'il faut afficher en    
-   haut (0), au milieu (1) ou en bas (2).                  
-   Si postion = 0, percent indique la distance entre le   
-   haut de la fenetre et le haut de la boite, exprimee en  
-   pourcentage de la hauteur de la fenetre.                
+  ShowBox displays the box pBox at the requested position in the window.
+  The parameter position is:
+    0 for the top of the window
+    1 for the middle of the window
+    2 for the bottom of the window
+   When the postion = 0, percent give the percent from the top of the
+   window.
   ----------------------------------------------------------------------*/
 void ShowBox (int frame, PtrBox pBox, int position, int percent)
 {
+   PtrAbstractBox      pBlock;
+   PtrBox              pBox1;
+   PtrLine             pLine;
+   ViewFrame          *pFrame;
    int                 ymin, ymax;
    int                 width, height;
    int                 y, dy, h;
-   ViewFrame          *pFrame;
 
    if (pBox == NULL)
       return;
-   while (pBox->BxType == BoGhost)
-     pBox = pBox->BxAbstractBox->AbEnclosing->AbBox;
+   pBox1 = pBox;
+   pBlock = NULL;
+   pLine = NULL;
+   if (pBox->BxType == BoGhost || pBox->BxType == BoFloatGhost)
+     {
+       while (pBox && (pBox->BxType == BoGhost || pBox->BxType == BoFloatGhost))
+	 pBox = pBox->BxAbstractBox->AbFirstEnclosed->AbBox;
+       if (!pBox)
+	 {
+	   pBox = pBox1;
+	   while (pBox->BxType == BoGhost || pBox->BxType == BoFloatGhost)
+	     pBox = pBox->BxAbstractBox->AbEnclosing->AbBox;
+	 }
+       /* manage the line instead of the box itself */
+       pLine = SearchLine (pBox, frame);
+       if (pLine)
+	 {
+	   pBlock = pBox->BxAbstractBox;
+	   while (pBlock && pBlock->AbBox->BxType != BoBlock &&
+		  pBlock->AbBox->BxType != BoFloatBlock)
+	     pBlock = pBlock->AbEnclosing;
+	 }
+     }
 
    if (pBox->BxType == BoSplit)
      pBox = pBox->BxNexChild;
 
    pFrame = &ViewFrameTable[frame - 1];
-   y = pBox->BxYOrg;
-   h = pBox->BxHeight;
+   if (pBlock && pBlock->AbBox)
+     {
+       y = pBlock->AbBox->BxYOrg + pLine->LiYOrg;
+       h = pLine->LiHeight;
+     }
+   else
+     {
+       y = pBox->BxYOrg;
+       h = pBox->BxHeight;
+     }
    /* largeur et hauteur de la fenetre */
    GetSizesFrame (frame, &width, &height);
    ymin = pFrame->FrYOrg;
@@ -679,61 +697,70 @@ ThotBool IsScrolled (int frame, int selection)
 
 
 /*----------------------------------------------------------------------
-   ShowSelectedBox montre le de'but de la se'lection dans la      
-   frame et rend cette frame active si l'indicateur actif     
-   est Vrai.                                               
+  ShowSelectedBox shows the beginning of the selection within the frame
+  and makes this frame active if the parameter active is TRUE.
   ----------------------------------------------------------------------*/
-void ShowSelectedBox (int frame, ThotBool actif)
+void ShowSelectedBox (int frame, ThotBool active)
 {
    ViewFrame          *pFrame;
    PtrBox              pBo1;
    int                 xmin, xmax;
    int                 ymin, ymax;
-   int                 x, y, dx, dy;
+   int                 x, y, dx, dy, w, h;
 
-   /* Faut-il rendre la fenetre active (frame d'insertion de texte) */
-   if (actif)
+   if (active)
+     /* make the window active */
       ChangeSelFrame (frame);
-
    pFrame = &ViewFrameTable[frame - 1];
    if (pFrame->FrSelectionBegin.VsBox != NULL && pFrame->FrReady)
      {
 	pBo1 = pFrame->FrSelectionBegin.VsBox;
-
-	/* Verifie qu'une des boites selectionnees est visible */
-	while (pBo1 != NULL && pBo1->BxAbstractBox != NULL &&
+	/* Check if almost one box is displayed */
+	while (pBo1 && pBo1->BxAbstractBox &&
 	       pBo1->BxAbstractBox->AbVisibility < pFrame->FrVisibility)
 	  {
 	     if (pBo1->BxAbstractBox->AbSelected ||
 		 pBo1 == pFrame->FrSelectionBegin.VsBox)
 		pBo1 = pBo1->BxNext;
 	     else
-		return;		/* aucune boite n'est visible */
+		/* no box found */
+		return;
 	  }
 	if (pBo1 != NULL)
 	  {
+#ifndef _GL
 	     x = pBo1->BxXOrg;
 	     y = pBo1->BxYOrg;
-	     GetSizesFrame (frame, &dx, &dy);
+	     GetSizesFrame (frame, &w, &h);
 	     xmin = pFrame->FrXOrg;
-	     xmax = xmin + dx;
+	     xmax = xmin + w;
 	     ymin = pFrame->FrYOrg;
-	     ymax = ymin + dy;
-	     /* MilieuX de la fenetre */
+	     ymax = ymin + h;
+#else /* _GL */
+	     x = pBo1->BxClipX;
+	     y = pBo1->BxClipY;
+	     GetSizesFrame (frame, &w, &h);
+	     xmin = 0;
+	     xmax = w;
+	     ymin = 0;
+	     ymax = h;
+#endif /* _GL*/
+	     /* center in the window */
 	     dx = pFrame->FrSelectionBegin.VsXPos;
 	     dy = 13;
-
-	     if (pBo1->BxAbstractBox != NULL)
+	     w /= 2;
+	     h /= 2;
+	     if (pBo1->BxAbstractBox)
 	       {
 		 if (!pBo1->BxAbstractBox->AbHorizPos.PosUserSpecified)
 		   /* the box position is not given by the user */
 		   {
 		   if (x + dx < xmin + 10)
 		     /* scroll the window */
-		     HorizontalScroll (frame, x + dx - xmin - 10, 0);
+		     HorizontalScroll (frame, x + dx - xmin - w, 0);
 		   else if (x + dx > xmax - 10)
 		     /* scroll the window */
-		     HorizontalScroll (frame, x + dx - xmax + 10, 0);
+		     HorizontalScroll (frame, x + dx - xmax + w, 0);
 		   }
 
 		 if (!pBo1->BxAbstractBox->AbVertPos.PosUserSpecified)
@@ -741,10 +768,10 @@ void ShowSelectedBox (int frame, ThotBool actif)
 		   {
 		   if (y + pBo1->BxHeight < ymin + dy)
 		     /* scroll the window */
-		     VerticalScroll (frame, y + pBo1->BxHeight - ymin - dy, 0);
+		     VerticalScroll (frame, y + pBo1->BxHeight - ymin - h, 0);
 		   else if (y > ymax - dy)
 		     /* scroll the window */
-		     VerticalScroll (frame, y - ymax + dy, 0);
+		     VerticalScroll (frame, y - ymax + h, 0);
 		   }
 	       }
 	  }
@@ -758,26 +785,33 @@ void ShowSelectedBox (int frame, ThotBool actif)
   ----------------------------------------------------------------------*/
 ThotBool IsAbsBoxVisible (int frame, PtrAbstractBox pAb)
 {
-   ViewFrame          *pFrame;
-   int                 y, ymax;
-   int                 dx, dy;
+  ViewFrame          *pFrame;
+  int                 y, ymax;
+  int                 dx, dy;
 
-   pFrame = &ViewFrameTable[frame - 1];
-   if (pAb == NULL)
-      return (FALSE);		/* pas de pave a tester */
-   else if (pAb->AbBox == NULL)
-      return (FALSE);		/* pas de boite a tester */
-   else if (pAb->AbVisibility < pFrame->FrVisibility ||
-	    pAb->AbBox->BxType == BoGhost)
-      return (FALSE);		/* la boite n'est pas visible par definition */
-   else
-     {
-	y = pAb->AbBox->BxYOrg;
-	GetSizesFrame (frame, &dx, &dy);
-	ymax = pFrame->FrYOrg + dy;
-	if (y <= ymax)
-	   return (FALSE);
-	else
-	   return (TRUE);
-     }
+  pFrame = &ViewFrameTable[frame - 1];
+  if (pAb == NULL)
+    return (FALSE);		/* pas de pave a tester */
+  else if (pAb->AbBox == NULL)
+    return (FALSE);		/* pas de boite a tester */
+  else if (pAb->AbVisibility < pFrame->FrVisibility ||
+	   pAb->AbBox->BxType == BoGhost ||
+	   pAb->AbBox->BxType == BoFloatGhost)
+    return (FALSE);		/* la boite n'est pas visible par definition */
+  else
+    {
+#ifndef _GL
+      y = pAb->AbBox->BxYOrg;
+      GetSizesFrame (frame, &dx, &dy);
+      ymax = pFrame->FrYOrg + dy;
+#else /* _GL */
+      y = pAb->AbBox->BxClipY;
+      GetSizesFrame (frame, &dx, &dy);
+      ymax = dy;
+#endif /* _GL */
+      if (y <= ymax)
+	return (FALSE);
+      else
+	return (TRUE);
+    }
 }
