@@ -1,10 +1,10 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
-
+#include "thot_gui.h"
 #include "thot_sys.h"
 #include "constmedia.h"
 #include "typemedia.h"
@@ -18,6 +18,7 @@
 #undef THOT_EXPORT
 #define THOT_EXPORT extern
 #include "edit_tv.h"
+#include "modif_tv.h"
 #include "select_tv.h"
 #include "appdialogue_tv.h"
 
@@ -31,13 +32,14 @@
 /*----------------------------------------------------------------------
    TtaIsDocumentSelected answers TRUE if the document is selected.  
   ----------------------------------------------------------------------*/
-ThotBool            TtaIsDocumentSelected (Document document)
+ThotBool TtaIsDocumentSelected (Document document)
 {
    PtrDocument         docsel;
    PtrElement          firstSelection, lastSelection;
    int                 firstChar, lastChar;
 
-   if (!GetCurrentSelection (&docsel, &firstSelection, &lastSelection, &firstChar, &lastChar))
+   if (!GetCurrentSelection (&docsel, &firstSelection, &lastSelection,
+			     &firstChar, &lastChar))
       return (FALSE);
    else if (LoadedDocument[document - 1] != docsel)
       return (FALSE);
@@ -45,6 +47,27 @@ ThotBool            TtaIsDocumentSelected (Document document)
       return (TRUE);
 }
 
+/*----------------------------------------------------------------------
+  TtaIsColumnRowSaved answers TRUE if there is a column or a row in
+  the Thot Clicboard.
+  ----------------------------------------------------------------------*/
+ThotBool TtaIsColumnRowSelected (Document document)
+{
+   PtrDocument         docsel;
+   PtrElement          firstSelection, lastSelection;
+   int                 firstChar, lastChar;
+
+   if (!GetCurrentSelection (&docsel, &firstSelection, &lastSelection,
+			     &firstChar, &lastChar))
+      return (FALSE);
+   else if (LoadedDocument[document - 1] != docsel)
+      return (FALSE);
+   else if (WholeColumnSaved || TableRowsSaved)
+       return (TRUE);
+   else
+      return (FALSE);
+}
+
 /*----------------------------------------------------------------------
    TtaSelectElement
 
@@ -57,7 +80,7 @@ ThotBool            TtaIsDocumentSelected (Document document)
    selectedElement: the element to be selected. NULL for cancelling the
    selection in the document.
   ----------------------------------------------------------------------*/
-void   TtaSelectElement (Document document, Element selectedElement)
+void TtaSelectElement (Document document, Element selectedElement)
 {
    DisplayMode         dispMode;
 
@@ -75,32 +98,36 @@ void   TtaSelectElement (Document document, Element selectedElement)
        if (dispMode == DisplayImmediately)
 	 if (selectedElement == NULL && ThotLocalActions[T_resetsel])
 	   /* Abort the selection */
-	 (*ThotLocalActions[T_resetsel]) (LoadedDocument[document - 1]);
+	 (*(Proc1)ThotLocalActions[T_resetsel]) ((void *)LoadedDocument[document - 1]);
 	 else
-	   SelectElement (LoadedDocument[document - 1], (PtrElement) selectedElement, TRUE, FALSE);
+	   SelectElement (LoadedDocument[document - 1],
+			  (PtrElement) selectedElement, TRUE, FALSE);
        else
-	 NewSelection (document, selectedElement, 0, 0);
+	 NewSelection (document, selectedElement, NULL, 0, 0);
      }
 }
 
 /*----------------------------------------------------------------------
    TtaSelectView posts the selected view in the document
   ----------------------------------------------------------------------*/
-void                TtaSelectView (Document document, View view)
+void TtaSelectView (Document document, View view)
 {
   PtrDocument       pDoc;
   int               oldView;
 
   UserErrorCode = 0;
-  GetActiveView (&pDoc, &oldView);
-  if (LoadedDocument[document - 1] == pDoc && oldView != view)
-    SelectedView = view;
+  if (document)
+    {
+      GetActiveView (&pDoc, &oldView);
+      if (LoadedDocument[document - 1] == pDoc && oldView != view)
+	SelectedView = view;
+    }
 }
 
 /*----------------------------------------------------------------------
    TtaGetSelectedDocument returns the current selected document
   ----------------------------------------------------------------------*/
-Document   TtaGetSelectedDocument ()
+Document TtaGetSelectedDocument ()
 {
   PtrDocument       pDoc;
   int               oldView;
@@ -113,6 +140,26 @@ Document   TtaGetSelectedDocument ()
     return (IdentDocument (pDoc));
 }
 
+/*----------------------------------------------------------------------
+   TtaGetActiveView returns the selected document and the active view
+   in that document
+  ----------------------------------------------------------------------*/
+void TtaGetActiveView (Document *doc, View *view)
+{
+  PtrDocument     pDoc;
+  int             docView;
+
+  UserErrorCode = 0;
+  *doc = 0;
+  *view = 0;
+  GetActiveView (&pDoc, &docView);
+  if (pDoc)
+    {
+      *doc = IdentDocument (pDoc);
+      *view = (View) docView;
+    }
+}
+
 /*----------------------------------------------------------------------
    TtaSelectString
 
@@ -140,7 +187,9 @@ void  TtaSelectString (Document document, Element textElement,
       TtaError (ERR_invalid_parameter);
    else if (!((PtrElement) textElement)->ElTerminal)
       TtaError (ERR_invalid_element_type);
-   else if (((PtrElement) textElement)->ElLeafType != LtText)
+   else if (((PtrElement) textElement)->ElLeafType != LtText &&
+	    ((PtrElement) textElement)->ElLeafType != LtSymbol &&
+	    ((PtrElement) textElement)->ElLeafType != LtPicture)
       TtaError (ERR_invalid_element_type);
    /* Checks the parameter document */
    else if (document < 1 || document > MAX_DOCUMENTS)
@@ -154,7 +203,7 @@ void  TtaSelectString (Document document, Element textElement,
 	 SelectString (LoadedDocument[document - 1],
 		       (PtrElement) textElement, firstCharacter, lastCharacter);
        else
-	 NewSelection (document, textElement, firstCharacter, lastCharacter);
+	 NewSelection (document, textElement, NULL, firstCharacter, lastCharacter);
      }
 }
 
@@ -170,53 +219,51 @@ void  TtaSelectString (Document document, Element textElement,
   ----------------------------------------------------------------------*/
 void TtaExtendSelection (Document document, Element element, int lastCharacter)
 {
-   PtrDocument         pDoc;
-   PtrElement          firstSelection, lastSelection;
-   DisplayMode         dispMode;
-   int                 firstChar, lastChar;
-   ThotBool            ok;
-   ThotBool            abort;
+  PtrDocument         pDoc;
+  PtrElement          firstSelection, lastSelection;
+  DisplayMode         dispMode;
+  int                 firstChar, lastChar;
+  ThotBool            ok;
+  ThotBool            abort;
 
-   UserErrorCode = 0;
-   if (element == NULL)
-      TtaError (ERR_invalid_parameter);
-   else if (((PtrElement) element)->ElParent == NULL)
-      TtaError (ERR_invalid_parameter);
-   /* Checks the parameter document */
-   else if (document < 1 || document > MAX_DOCUMENTS)
-      TtaError (ERR_invalid_document_parameter);
-   else if (LoadedDocument[document - 1] == NULL)
-      TtaError (ERR_invalid_document_parameter);
-   else
-      /* Parameter document is correct */
-     {
-	/* verifies if there is a selection */
-	ok = GetCurrentSelection (&pDoc, &firstSelection, &lastSelection, &firstChar, &lastChar);
-	dispMode = TtaGetDisplayMode (document);
-	if (dispMode == DisplayImmediately || dispMode == DeferredDisplay)
-	   /* The command can be executed */
-	  {
-	     if (ok)
-		/* verifies that the selection is into the document */
-		ok = (pDoc == LoadedDocument[document - 1]);
-	  }
-	/* verifies if a selection is applied */
-	else if (IsSelectionRegistered (document, &abort))
-	   /* There is an application, the extension is accepted if there is 
-	      a request which is not for aborting */
-	   ok = !abort;
-
-	if (!ok)
-	   /* Error: no selection */
-	   TtaError (ERR_no_selection_in_document);
-	else
-	  {
-	    if (dispMode == DisplayImmediately || dispMode == DeferredDisplay)
-	      ExtendSelection ((PtrElement) element, lastCharacter, FALSE, FALSE, FALSE);
-	    else
-	      NewSelectionExtension (document, element, lastCharacter);
-	  }
-     }
+  UserErrorCode = 0;
+  if (element == NULL)
+    TtaError (ERR_invalid_parameter);
+  else if (((PtrElement) element)->ElParent == NULL)
+    TtaError (ERR_invalid_parameter);
+  /* Checks the parameter document */
+  else if (document < 1 || document > MAX_DOCUMENTS)
+    TtaError (ERR_invalid_document_parameter);
+  else if (LoadedDocument[document - 1] == NULL)
+    TtaError (ERR_invalid_document_parameter);
+  else
+    /* Parameter document is correct */
+    {
+      /* verifies if there is a selection */
+      ok = GetCurrentSelection (&pDoc, &firstSelection, &lastSelection,
+				&firstChar, &lastChar);
+      dispMode = TtaGetDisplayMode (document);
+      if (dispMode == DisplayImmediately)
+	/* The command can be executed */
+	{
+	  if (ok)
+	    /* verifies that the selection is into the document */
+	    ok = (pDoc == LoadedDocument[document - 1]);
+	}
+      /* verifies if a selection is applied */
+      else if (IsSelectionRegistered (document, &abort))
+	/* There is an application, the extension is accepted if there is 
+	   a request which is not for aborting */
+	ok = !abort;
+      
+      if (!ok)
+	/* Error: no selection */
+	TtaError (ERR_no_selection_in_document);
+      else if (dispMode == DisplayImmediately)
+	ExtendSelection ((PtrElement) element, lastCharacter, TRUE, FALSE, FALSE);
+      else
+	NewSelectionExtension (document, element, lastCharacter);
+    }
 }
 
 /*----------------------------------------------------------------------
@@ -230,7 +277,7 @@ void TtaExtendSelection (Document document, Element element, int lastCharacter)
 	to TtaAddElementToSelection and by the last call to TtaSelectElement.
    element: the element to be added to the current selection
   ----------------------------------------------------------------------*/
-void      TtaAddElementToSelection (Document document, Element element)
+void TtaAddElementToSelection (Document document, Element element)
 {
    PtrDocument         pDoc;
    PtrElement          firstSelection, lastSelection;
@@ -283,7 +330,7 @@ void      TtaAddElementToSelection (Document document, Element element)
    Return value:
    No return value
   ----------------------------------------------------------------------*/
-void                TtaSelectInterval ()
+void TtaSelectInterval ()
 {
    SelectPairInterval ();
 }
@@ -298,7 +345,7 @@ void                TtaSelectInterval ()
    Return value:
    No return value
   ----------------------------------------------------------------------*/
-void                TtaUnselect (Document document)
+void TtaUnselect (Document document)
 {
    if (document < 1 || document > MAX_DOCUMENTS)
      TtaError (ERR_invalid_document_parameter);
@@ -320,7 +367,7 @@ void                TtaUnselect (Document document)
    Parameter:
    withMenu: the new selection mode.
   ----------------------------------------------------------------------*/
-void                TtaSetSelectionMode (ThotBool withMenu)
+void TtaSetSelectionMode (ThotBool withMenu)
 {
    SelectionUpdatesMenus = withMenu;
 }
@@ -331,67 +378,78 @@ void                TtaSetSelectionMode (ThotBool withMenu)
    Returns TRUE if there is a current selection and this selection is
    empty (a caret).
   ----------------------------------------------------------------------*/
-ThotBool            TtaIsSelectionEmpty ()
+ThotBool TtaIsSelectionEmpty ()
 {
-  if (SelectedDocument == NULL)
+  if (SelectedDocument == NULL && DocSelectedAttr == NULL)
     return (FALSE);
-  else if (SelPosition != TRUE &&
+  else if (SelPosition)
+    return (TRUE);
+  else if (FirstSelectedElement &&
 	   FirstSelectedElement == LastSelectedElement &&
 	   FirstSelectedElement->ElVolume == 0)
     return (TRUE);
   else
-    return (SelPosition);
+    return (FALSE);
 }
 
 /*----------------------------------------------------------------------
-   TtaGiveFirstSelectedElement
+  TtaGiveFirstSelectedElement
 
-   Returns the first element in the current selection in a given document.
-   If this element is a Text element and if only a substring is selected,
-   return also the rank of the first and last characters in the selection.
-   Parameter:
-   document: the document for which the selection is asked.
-   Return values:
-   selectedElement: the first selected element, NULL if no element is
-   selected in the document.
-   firstCharacter: rank of the first character in the selection, or 0 if the
-   whole element is in the selection.
-   lastCharacter: rank of the last character in the selection, or 0 if the
-   whole element is in the selection.
+  Returns the first element in the current selection in a given document.
+  If this element is a Text element and if only a substring is selected,
+  return also the rank of the first and last characters in the selection.
+  Parameter:
+  document: the document for which the selection is asked.
+  Return values:
+  selectedElement: the first selected element, NULL if no element is
+  selected in the document.
+  firstCharacter: rank of the first selected character in the element, or
+  0 if the whole element is selected.
+  lastCharacter: rank of the last selected character in the element, or
+  0 if the whole element is in the selection, or firstCharacter - 1 for
+  a position.
   ----------------------------------------------------------------------*/
-void  TtaGiveFirstSelectedElement (Document document, Element *selectedElement,
-				   int *firstCharacter, int *lastCharacter)
+void TtaGiveFirstSelectedElement (Document document, Element *selectedElement,
+				  int *firstCharacter, int *lastCharacter)
 {
-   PtrDocument         pDoc;
-   PtrElement          firstSelection, lastSelection;
-   int                 firstChar, lastChar;
-   ThotBool            ok;
+  PtrDocument         pDoc;
+  PtrElement          firstSelection, lastSelection;
+  int                 firstChar, lastChar;
+  ThotBool            ok;
 
-   UserErrorCode = 0;
-   *selectedElement = NULL;
-   *firstCharacter = 0;
-   *lastCharacter = 0;
-   /* Checks the parameter document */
-   if (document < 1 || document > MAX_DOCUMENTS)
-     TtaError (ERR_invalid_document_parameter);
-   else if (LoadedDocument[document - 1] == NULL)
-     TtaError (ERR_invalid_document_parameter);
-   else
+  UserErrorCode = 0;
+  *selectedElement = NULL;
+  *firstCharacter = 0;
+  *lastCharacter = 0;
+  /* Checks the parameter document */
+  if (document < 1 || document > MAX_DOCUMENTS)
+    TtaError (ERR_invalid_document_parameter);
+  else if (LoadedDocument[document - 1] == NULL)
+    TtaError (ERR_invalid_document_parameter);
+  else
+    {
       /* Parameter document is correct */
-     {
-       ok = GetCurrentSelection (&pDoc, &firstSelection, &lastSelection, &firstChar, &lastChar);
-       if (ok)
-	 if (pDoc == LoadedDocument[document - 1])
-	   {
-	     *selectedElement = (Element) firstSelection;
-	     *firstCharacter = firstChar;
-	     if (lastSelection == firstSelection)
-	       if (lastChar > 1)
-		 *lastCharacter = lastChar - 1;
-	     if (firstChar != 0 && lastChar == 0)
-	       *lastCharacter = firstSelection->ElVolume;
-	   }
-     }
+      ok = GetCurrentSelection (&pDoc, &firstSelection, &lastSelection,
+				&firstChar, &lastChar);
+      if (ok)
+	if (pDoc == LoadedDocument[document - 1])
+	  {
+	    *selectedElement = (Element) firstSelection;
+	    *firstCharacter = firstChar;
+	    if (lastSelection == firstSelection)
+	      {
+		/* only one element */
+		if (SelPosition)
+		  /* a position */
+		  *lastCharacter = firstChar - 1;
+		else
+		  *lastCharacter = lastChar;
+	      }
+	    else if (firstChar > 0 && lastChar == 0)
+	      /* the selection starts in the middle of the element */
+	      *lastCharacter = firstSelection->ElVolume;
+	  }
+    }
 }
 
 
@@ -413,42 +471,43 @@ void  TtaGiveFirstSelectedElement (Document document, Element *selectedElement,
    lastCharacter: rank of the last character in the selection, or 0 if the
    whole element is in the selection.
   ----------------------------------------------------------------------*/
-void  TtaGiveNextSelectedElement (Document document, Element *selectedElement,
-				  int *firstCharacter, int *lastCharacter)
+void TtaGiveNextSelectedElement (Document document, Element *selectedElement,
+				 int *firstCharacter, int *lastCharacter)
 {
-   PtrDocument         pDoc;
-   PtrElement          pEl;
-   PtrElement          firstSelection, lastSelection;
-   int                 firstChar, lastChar;
-   ThotBool            ok;
+  PtrDocument         pDoc;
+  PtrElement          pEl;
+  PtrElement          firstSelection, lastSelection;
+  int                 firstChar, lastChar;
+  ThotBool            ok;
 
-   UserErrorCode = 0;
-   *firstCharacter = 0;
-   *lastCharacter = 0;
-   /* Checks the parameter document */
-   if (document < 1 || document > MAX_DOCUMENTS)
-     TtaError (ERR_invalid_document_parameter);
-   else if (LoadedDocument[document - 1] == NULL)
-     TtaError (ERR_invalid_document_parameter);
-   else
+  UserErrorCode = 0;
+  *firstCharacter = 0;
+  *lastCharacter = 0;
+  /* Checks the parameter document */
+  if (document < 1 || document > MAX_DOCUMENTS)
+    TtaError (ERR_invalid_document_parameter);
+  else if (LoadedDocument[document - 1] == NULL)
+    TtaError (ERR_invalid_document_parameter);
+  else
+    {
       /* Parameter document is correct */
-     {
-	ok = GetCurrentSelection (&pDoc, &firstSelection, &lastSelection, &firstChar, &lastChar);
-	if (!ok)
-	   *selectedElement = NULL;
-	else if (pDoc != LoadedDocument[document - 1])
-	   *selectedElement = NULL;
-	else
-	  {
-	     pEl = NextInSelection ((PtrElement) * selectedElement, lastSelection);
-	     *selectedElement = (Element) pEl;
-	     if (pEl == lastSelection && lastChar > 0)
-	       {
-		  *firstCharacter = 1;
-		  *lastCharacter = lastChar - 1;
-	       }
-	  }
-     }
+      ok = GetCurrentSelection (&pDoc, &firstSelection, &lastSelection,
+				&firstChar, &lastChar);
+      if (!ok)
+	*selectedElement = NULL;
+      else if (pDoc != LoadedDocument[document - 1])
+	*selectedElement = NULL;
+      else
+	{
+	  pEl = NextInSelection ((PtrElement) *selectedElement, lastSelection);
+	  *selectedElement = (Element) pEl;
+	  if (pEl == lastSelection && lastChar > 0)
+	    {
+	      *firstCharacter = 1;
+	      *lastCharacter = lastChar;
+	    }
+	}
+    }
 }
 
 
@@ -463,7 +522,7 @@ void  TtaGiveNextSelectedElement (Document document, Element *selectedElement,
    Return parameters:
    element: the next element in the selection order. NULL if not found.
   ----------------------------------------------------------------------*/
-void  TtaGiveNextElement (Document document, Element *element, Element last)
+void TtaGiveNextElement (Document document, Element *element, Element last)
 {
    PtrElement          pEl;
 
@@ -485,54 +544,66 @@ void  TtaGiveNextElement (Document document, Element *element, Element last)
 
 
 /*----------------------------------------------------------------------
-   TtaGiveLastSelectedElement
+  TtaGiveLastSelectedElement
 
-   Returns the last element in the current selection in a given document.
-   If this element is a Text element and if only a substring is selected,
-   return also the rank of the first and last characters in the selection.
-   Parameter:
-   document: the document for which the selection is asked.
-   Return values:
-   selectedElement: the last selected element, NULL if no element is
-   selected in the document.
-   firstCharacter: rank of the first character in the selection, or 0 if the
-   whole element is in the selection.
-   lastCharacter: rank of the last character in the selection, or 0 if the
-   whole element is in the selection.
+  Returns the last element in the current selection in a given document.
+  If this element is a Text element and if only a substring is selected,
+  return also the rank of the first and last characters in the selection.
+  Parameter:
+  document: the document for which the selection is asked.
+  Return values:
+  selectedElement: the last selected element, NULL if no element is
+  selected in the document.
+  firstCharacter: rank of the first selected character in the element, or
+  0 if the whole element is in the selection.
+  lastCharacter: rank of the last selected character in the element, or
+  0 if the whole element is in the selection, or firstCharacter - 1 for
+  a position.
   ----------------------------------------------------------------------*/
-void   TtaGiveLastSelectedElement (Document document, Element *selectedElement,
-				   int *firstCharacter, int *lastCharacter)
+void TtaGiveLastSelectedElement (Document document, Element *selectedElement,
+				 int *firstCharacter, int *lastCharacter)
 {
-   PtrDocument         pDoc;
-   PtrElement          firstSelection, lastSelection;
-   int                 firstChar, lastChar;
-   ThotBool            ok;
+  PtrDocument         pDoc;
+  PtrElement          firstSelection, lastSelection;
+  int                 firstChar, lastChar;
+  ThotBool            ok;
 
-   UserErrorCode = 0;
-   *selectedElement = NULL;
-   *firstCharacter = 0;
-   *lastCharacter = 0;
-   /* Checks the parameter document */
-   if (document < 1 || document > MAX_DOCUMENTS)
-     TtaError (ERR_invalid_document_parameter);
-   else if (LoadedDocument[document - 1] == NULL)
-     TtaError (ERR_invalid_document_parameter);
-   else
+  UserErrorCode = 0;
+  *selectedElement = NULL;
+  *firstCharacter = 0;
+  *lastCharacter = 0;
+  /* Checks the parameter document */
+  if (document < 1 || document > MAX_DOCUMENTS)
+    TtaError (ERR_invalid_document_parameter);
+  else if (LoadedDocument[document - 1] == NULL)
+    TtaError (ERR_invalid_document_parameter);
+  else
+    {
       /* Parameter document is correct */
-     {
-       ok = GetCurrentSelection (&pDoc, &firstSelection, &lastSelection, &firstChar, &lastChar);
-       if (ok)
-	 if (pDoc == LoadedDocument[document - 1])
-	   {
-	     *selectedElement = (Element) lastSelection;
-	     if (lastSelection == firstSelection)
-	       {
-		 *firstCharacter = firstChar;
-		 if (lastChar > 1)
-		   *lastCharacter = lastChar - 1;
-	       }
-	     if (lastChar > 0)
-	       *lastCharacter = lastChar - 1;
-	   }
-     }
+      ok = GetCurrentSelection (&pDoc, &firstSelection, &lastSelection,
+				&firstChar, &lastChar);
+      if (ok)
+	if (pDoc == LoadedDocument[document - 1])
+	  {
+	    *selectedElement = (Element) lastSelection;
+	    if (lastSelection == firstSelection)
+	      {
+		/* only one element */
+		*firstCharacter = firstChar;
+		if (SelPosition)
+		  /* a position */
+		  *lastCharacter = firstChar - 1;
+		else
+		  *lastCharacter = lastChar;
+	      }
+	    else if (lastChar > 0)
+	      {
+		/* the selection ends in the middle of the element */
+		*firstCharacter = 1;
+		*lastCharacter = lastChar;
+	      }
+	  }
+    }
 }
+
+
