@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001.
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -11,7 +11,7 @@
  * Authors: I. Vatton (INRIA), H. Richy (INRIA)
  *
  */
-
+#include "thot_gui.h"
 #include "thot_sys.h"
 #include "constmedia.h"
 #include "typemedia.h"
@@ -28,9 +28,9 @@
 #undef THOT_EXPORT
 #define THOT_EXPORT extern
 #include "spell_tv.h"
-#undef THOT_EXPORT
-#define THOT_EXPORT
-#include "word_tv.h"
+
+extern short _cType_[]; /* defined in word_tv.h */
+
 /* description des chiffres romains (majuscule) */
 static char         NRomain[] = {
   'M', 'C', 'D', 'L', 'X', 'V', 'I', EOS
@@ -61,6 +61,7 @@ static int           type_err;	/* Type du mot errone (majuscule, SmallLettering)
 #include "exceptions_f.h"
 #include "fileaccess_f.h"
 #include "hyphen_f.h"
+#include "memory_f.h"
 #include "registry_f.h"
 #include "search_f.h"
 #include "spellchecker_f.h"
@@ -81,7 +82,7 @@ ThotBool TtaLoadDocumentDictionary (PtrDocument document, int *pDictionary,
   char               path[MAX_PATH], dictname[MAX_PATH];
 
   /* dictionary name = document name */
-  *pDictionary = (int) NULL;
+  *pDictionary = 0;
   
   extenddic = TtaGetEnvString ("EXTENDDICT");
   if (extenddic != NULL)
@@ -106,7 +107,7 @@ ThotBool TtaLoadDocumentDictionary (PtrDocument document, int *pDictionary,
 
 /*----------------------------------------------------------------------
   ----------------------------------------------------------------------*/
-static void         Asci2Code (char *string)
+static void Asci2Code (char *string)
 {
    int                 i;
 
@@ -121,7 +122,7 @@ static void         Asci2Code (char *string)
 
 /*----------------------------------------------------------------------
   ----------------------------------------------------------------------*/
-static void         Code2Asci (char *string)
+static void Code2Asci (char *string)
 {
    int                 i = 0;
 
@@ -132,42 +133,10 @@ static void         Code2Asci (char *string)
      }
 }
 
-#ifdef IV
-/*----------------------------------------------------------------------
-  minMAJ returns TRUE if the string doesn't contains only lowercases 
-  ----------------------------------------------------------------------*/
-ThotBool            minMAJ (char *string)
-{
-   int                 maj = 0;
-   int                 i = 0;
-   char                c;
-
-   while ((c = string[i]) != EOS && (maj == 0))
-     {
-	maj = isimaj (string[i]);
-	i++;
-     }
-   return (maj == 1) ? TRUE : FALSE;
-}
-
 
 /*----------------------------------------------------------------------
   ----------------------------------------------------------------------*/
-char                tomin (char caract)
-{
-   char                c;
-
-   c = caract;
-   if (isimaj (caract) != 0)
-      c = tolower (caract);
-
-   return (c);
-}
-#endif /* IV */
-
-/*----------------------------------------------------------------------
-  ----------------------------------------------------------------------*/
-void         SetUpperCase (char *string)
+void SetUpperCase (char *string)
 {
    int                 i = 0;
 
@@ -182,7 +151,7 @@ void         SetUpperCase (char *string)
 
 /*----------------------------------------------------------------------
   ----------------------------------------------------------------------*/
-void         SetCapital (char *string)
+void SetCapital (char *string)
 {
    if (isimin (string[0]) != 0)
       string[0] = toupper (string[0]);
@@ -192,7 +161,7 @@ void         SetCapital (char *string)
 
 /*----------------------------------------------------------------------
   ----------------------------------------------------------------------*/
-ThotBool     IsUpperCase (char *string)
+ThotBool IsUpperCase (char *string)
 {
    int                 maj = 1;
    int                 i = 0;
@@ -209,7 +178,7 @@ ThotBool     IsUpperCase (char *string)
 
 /*----------------------------------------------------------------------
   ----------------------------------------------------------------------*/
-ThotBool     IsCapital (char *string)
+ThotBool IsCapital (char *string)
 {
    int                 cap = 0;
    int                 i = 0;
@@ -230,7 +199,7 @@ ThotBool     IsCapital (char *string)
 
 /*----------------------------------------------------------------------
   ----------------------------------------------------------------------*/
-ThotBool            IsIso (char *string)
+ThotBool IsIso (char *string)
 {
    int                 iso = 1;
    int                 i = 0;
@@ -251,22 +220,23 @@ ThotBool            IsIso (char *string)
    retourne -3  si le mot est present dans le dict                          
    retourne >=0 si le mot n'est pas dans le dict (indice de sa place future) 
   ----------------------------------------------------------------------*/
-int WordInDictionary (char word[MAX_WORD_LEN], PtrDict dict)
+int WordInDictionary (unsigned char *word, PtrDict dict)
 {
    int                 inf, sup, med, rescomp, size;
 
    if (dict == NULL)
      /* dictionnaire n'existe pas */
      return (-2);
-   else if (dict->DictNbWords < 0)
+   else if (dict->DictNbWords <= 0)
      /* dictionnaire vide */
      return (-1);
 
-   size = strlen (word);
+   size = strlen ((char*)word);
    /* premier mot */
    inf = dict->DictLengths[size];
-   if (size >= MAX_WORD_LEN || (dict->DictLengths[size + 1] - 1 > dict->DictNbWords))
-     sup = dict->DictNbWords;
+   if (size >= MAX_WORD_LEN ||
+       (dict->DictLengths[size + 1] - 1 > dict->DictNbWords))
+     sup = dict->DictNbWords - 1;
    else
      /* dernier mot */
      sup = dict->DictLengths[size + 1] - 1;
@@ -275,7 +245,7 @@ int WordInDictionary (char word[MAX_WORD_LEN], PtrDict dict)
      /* Recherche dichotomique */
      {
 	med = (sup + inf) / 2;
-	rescomp = strcmp (&dict->DictString[dict->DictWords[med]], word);
+	rescomp = strcmp ((char*)&dict->DictString[dict->DictWords[med]], (char*)word);
 	if (rescomp == 0)
 	  /* Mot present dans le dict */
 	  return (-3);
@@ -297,79 +267,62 @@ int WordInDictionary (char word[MAX_WORD_LEN], PtrDict dict)
    0 si le mot est inconnu ou vide                         
    -1 s'il n'y a pas de dictionnaire pour verifier ce mot   
   ----------------------------------------------------------------------*/
-int CheckWord (char word[MAX_WORD_LEN], Language language, PtrDict dict)
+int CheckWord (unsigned char *word, Language language, PtrDict dict)
 {
-   ThotBool            present = TRUE;
-   int                 res;
-   char                word1[MAX_WORD_LEN];
-   char                wordmin[MAX_WORD_LEN];
-   PtrDict             globalDict;
-   PtrDict             personalDict;
-   PtrDict             dictsigle;
-   PtrDict             dictname;
+  PtrDict             globalDict;
+  PtrDict             personalDict;
+  int                 res;
+  unsigned char       word1[MAX_WORD_LEN];
 
-   /* On refuse de corriger l'ISOlatin-1 */
-   if (language == 0)
-      return (-1);
-
-   if (word[0] == EOS)
-      return (0);		/* mot vide */
-
-   if (strlen (word) >= 2)
-     {
-	strcpy (word1, word);
-	SmallLettering (word1);
-	strcpy (wordmin, word1);
-	Asci2Code (word1);
-
-	globalDict = (PtrDict) TtaGetPrincipalDictionary (language);
-	personalDict = (PtrDict) TtaGetSecondaryDictionary (language);
-	res = WordInDictionary (word1, dict);
-
-	if (res != -3 || res == -2)
-	   /* mot absent de dict ou dict vide ou pas de dict */
-	  {
-	     res = WordInDictionary (word1, globalDict);
-	     if (res == -2)
-		return (-1);	/* PAS de globalDict */
-	     else
-	       {
-		  if (res != -3)	/* mot absent de globalDict */
-		    {		/* word1 n'appartient pas a ces 2 dicts obligatoires */
-		       present = FALSE;
-		       dictsigle = (PtrDict) TtaGetPrincipalDictionary (0);
-		       dictname = (PtrDict) TtaGetSecondaryDictionary (0);
-		       if (WordInDictionary (word1, personalDict) == -3
-			   || WordInDictionary (word1, dictsigle) == -3
-			   || WordInDictionary (word1, dictname) == -3)
-			  /* word1 appartient a l'un de ces dict facultatifs */
-			  present = TRUE;
-		    }
-	       }
-	  }
-
-	if (present == FALSE)
-	  {			/* calculer le type du mot errone' */
-	     if (IsUpperCase (word) == TRUE)
-		type_err = 3;	/* MAJ */
-	     else
-	       {
-		  if (IsCapital (word) == TRUE)
-		     type_err = 2;	/* Capitale */
-		  else
-		     type_err = 1;	/* SmallLettering ou melange */
-	       }
-	     /* recopier ce mot errone' dans ChkrErrWord */
-	     strcpy (ChkrErrWord, word);
-	  }
-     }
-   res = (present == TRUE) ? 1 : 0;
-   return (res);
+  /* On refuse de corriger l'ISOlatin-1 */
+  if (language == 0)
+    return -1;
+  if (word[0] == EOS)
+    return 0;
+  if (strlen ((char*)word) < 2)
+    return 1;
+  else
+    {
+      strcpy ((char*)word1, (char*)word);
+      SmallLettering (word1);
+      Asci2Code ((char*)word1);
+      globalDict = (PtrDict) TtaGetPrincipalDictionary (language);
+      personalDict = (PtrDict) TtaGetSecondaryDictionary (language);
+      res = WordInDictionary (word1, dict);
+      if (res != -3)
+	{
+	  /* not found */
+	  res = WordInDictionary (word1, globalDict);
+	  if (res == -2)
+	    return (-1);	/* PAS de globalDict */
+          else if (res != -3)
+            {
+              /* check extra dictionaries */
+              if (WordInDictionary (word1, personalDict) == -3)
+                /* found */
+                res = -3;
+	    }
+	}
+      if (res != -3)
+	{			/* calculer le type du mot errone' */
+	  if (IsUpperCase ((char*)word))
+	    type_err = 3;	/* MAJ */
+	  else if (IsCapital ((char*)word))
+	    type_err = 2;	/* Capitale */
+	  else
+	    type_err = 1;	/* SmallLettering ou melange */
+	  /* recopier ce mot errone' dans ChkrErrWord */
+	  strcpy ((char*)ChkrErrWord, (char*)word);
+	  return 0;
+	}
+      else
+	return 1;
+    }
 }
 
 /*----------------------------------------------------------------------
   ----------------------------------------------------------------------*/
-static int          Insert (int x, int pWord, PtrDict dict)
+static int Insert (int x, int pWord, PtrDict dict)
 {
    int                 i, k;
 
@@ -400,9 +353,10 @@ static int          Insert (int x, int pWord, PtrDict dict)
    1 si le mot a pu etre ajoute dans le dictionnaire 
    par defaut : tous les mots nouveaux sont insecables        
   ----------------------------------------------------------------------*/
-static int          InsertWord (PtrDict dict, char word[MAX_WORD_LEN])
+static int InsertWord (PtrDict dict, unsigned char *word)
 {
-   int                 size, place, i, k;
+   int          size, place, i, k;
+   int          first, last, index;
 
    place = WordInDictionary (word, dict);
    if (place == -2)
@@ -411,29 +365,27 @@ static int          InsertWord (PtrDict dict, char word[MAX_WORD_LEN])
       return (0);		/* mot deja present dans le dictionnaire */
    if (place == -1)		/* dictionnaire vide */
       place = 0;		/* indice pour inserer dans ce dictionaire */
-   size = strlen (word) + 1;
-   /* JK: Changed the limit to - 2 instead of -1 because we had a
-      memory overwrite problem otherwise */
-   if ((dict->DictNbWords >= dict->DictMaxWords - 2) || (dict->DictNbChars >= dict->DictMaxChars + size))
+   size = strlen ((char*)word) + 1;
+   /* IV: prevents an overwrite problem */
+   if (dict->DictNbWords + 2 >= dict->DictMaxWords ||
+       dict->DictNbChars >= dict->DictMaxChars + size)
       return (-1);
    else
      {
-	for (i = dict->DictNbWords; i >= place; i--)
-	  /* deplacement des mots */
+	for (i = dict->DictNbWords - 1; i >= place; i--)
 	  {
-	     int                 debut, fin, index;
-
-	     debut = dict->DictWords[i];
-	     fin = dict->DictWords[i + 1] - 1;
-	     index = fin + size;
-	     for (k = fin; k >= debut; k--)	/* deplacement d'un mot */
+	    /* move previous words */
+	     first = dict->DictWords[i];
+	     last = dict->DictWords[i + 1] - 1;
+	     index = last + size;
+	     for (k = last; k >= first; k--)	/* deplacement d'un mot */
 		dict->DictString[index--] = dict->DictString[k];
 	  }
-	/* insertion nouveau mot */
-	strcpy (&dict->DictString[dict->DictWords[place]], word);
-	/* mise a jour des pointeurs sur les mots */
-	for (i = dict->DictNbWords + 1; i >= place; i--)
-	   dict->DictWords[i + 1] = dict->DictWords[i] + size;
+	/* insert the new word */
+	strcpy ((char*)&dict->DictString[dict->DictWords[place]], (char*)word);
+	/* update next entries including the first empty entry */
+	for (i = dict->DictNbWords + 1; i > place; i--)
+	   dict->DictWords[i] = dict->DictWords[i - 1] + size;
 	/* mise a jour des pointeurs sur les longueurs de mots */
 	for (i = size; i < MAX_WORD_LEN; i++)
 	   dict->DictLengths[i]++;
@@ -448,24 +400,24 @@ static int          InsertWord (PtrDict dict, char word[MAX_WORD_LEN])
 
 /*----------------------------------------------------------------------
   ----------------------------------------------------------------------*/
-static void         Cmp (char wordtest[], PtrDict dict)
+static void Cmp (unsigned char *wordtest, PtrDict dict)
 {
   int                 dist[MAX_WORD_LEN][MAX_WORD_LEN];
   unsigned char       wordcmp[MAX_WORD_LEN];
-  int                 Lg, idx, sup, pWord, seuilCourant;
-  int                 i, j, k, x, y, z;
-  int                 difference, iteration, size, largeur, word;
   unsigned char       currentWord[MAX_WORD_LEN];
+  int                 lg, idx, sup, pWord, seuilCourant;
+  int                 i, j, k, x, y, z;
+  int                 difference, iteration, size, width, word;
   int                 minimum;
-  int                 deb, fin;
-  int                 derniere_ligne = MAX_WORD_LEN; /* last computed line */
+  int                 first, last;
+  int                 lastline = MAX_WORD_LEN; /* last computed line */
   int                 dist_mini;
   int                 dernier_liste;
   
   x = 0;
   if (dict == NULL)
     return;
-  else if (dict->DictNbWords < 0)
+  else if (dict->DictNbWords <= 0)
     /* empty dictionary */
     return;
   
@@ -475,13 +427,13 @@ static void         Cmp (char wordtest[], PtrDict dict)
     dist[i][0] = dist[i - 1][0] + KO;
   for (i = 1; i < MAX_WORD_LEN; i++)
     dist[0][i] = dist[0][i - 1] + KI;
-  
-  strcpy (wordcmp, wordtest);
+  idx = 0;
+  strcpy ((char*)wordcmp, (char*)wordtest);
   SmallLettering (wordcmp);
-  Asci2Code (wordcmp);
-  Lg = strlen (wordcmp);
-  seuilCourant = Seuil[Lg];
-  largeur = Delta[Lg];
+  Asci2Code ((char*)wordcmp);
+  lg = strlen ((char*)wordcmp);
+  seuilCourant = Seuil[lg];
+  width = Delta[lg];
   
   /* parcours du dictionnaire
      avec d'abord les mots de meme size,
@@ -489,10 +441,10 @@ static void         Cmp (char wordtest[], PtrDict dict)
      puis EVENTUELLEMENT les mots de +- 2 lettres etc.
      */
   difference = 0;
-  for (iteration = 0; iteration <= 2 * largeur; iteration++)
+  for (iteration = 0; iteration <= 2 * width; iteration++)
     {
       difference = (difference > 0) ? difference - iteration : difference + iteration;
-      size = Lg - difference;
+      size = lg - difference;
       if (size >= 0 && size < MAX_WORD_LEN)
 	{
 	  /*
@@ -502,61 +454,60 @@ static void         Cmp (char wordtest[], PtrDict dict)
 	    ces mots
 	  */
 	  dist_mini = (difference > 0) ? difference * KI : -difference * KO;
-	  if (dist_mini > seuilCourant)
-	    continue;
+	  /*if (dist_mini > seuilCourant)
+	    continue;*/
 	  
 	  /* calcul des indices de debut et fin de dictionnaire */
 	  word = dict->DictLengths[size];
 	  idx = dict->DictWords[word];
-	  if (size >= MAX_WORD_LEN || (dict->DictLengths[size + 1] - 1 > dict->DictNbWords))
-	    sup = dict->DictWords[dict->DictNbWords];
+	  if (dict->DictLengths[size + 1] - 1 > dict->DictNbWords)
+	    sup = dict->DictWords[dict->DictNbWords] - 1;
 	  else
-	    sup = dict->DictWords[dict->DictLengths[size + 1]];
+	    sup = dict->DictWords[dict->DictLengths[size + 1]] - 1;
 	  
 	  /* initialisation des valeurs en dehors des diagonales de calculs effectifs */
-	  
 	  for (j = 1; j <= size; j++)
 	    {
-	      i = j - largeur - 1;
+	      i = j - width - 1;
 	      if (difference > 0)
 		i = i + difference;
 	      if (i > 0)
 		dist[i][j] = seuilCourant + 1;
-	      i = j + largeur + 1;
+	      i = j + width + 1;
 	      if (difference < 0)
 		i = i + difference;
-	      if (i <= Lg)
+	      if (i <= lg)
 		dist[i][j] = seuilCourant + 1;
 	    }
 	  
 	  /* parcours du dictionnaire */
-	  while (idx < sup)
+	  while (idx < sup && idx < dict->DictNbChars)
 	    {
 	      pWord = idx;
 	      k = dict->DictCommon[word++];
 	      /* si le calcul du mot precedent a ete stoppe a l'indice derniere_ligne
 		 et que le mot courant possede un nombre de lettres communes superieur
 		 a cette valeur, il est inutile de faire le calcul */
-	      if (k <= derniere_ligne)
+	      if (k <= lastline)
 		{
-		  strcpy (currentWord, &dict->DictString[pWord]);
+		  strcpy ((char*)currentWord, (char*)&dict->DictString[pWord]);
 		  
 		  /* calcul */
 		  for (j = k; j <= size; j++)
 		    {
 		      minimum = dist[0][j];
-		      derniere_ligne = j;
-		      deb = j - largeur;
+		      lastline = j;
+		      first = j - width;
 		      if (difference > 0)
-			deb = deb + difference;
-		      if (deb < 1)
-			deb = 1;
-		      fin = j + largeur;
+			first = first + difference;
+		      if (first < 1)
+			first = 1;
+		      last = j + width;
 		      if (difference < 0)
-			fin = fin + difference;
-		      if (fin > Lg)
-			fin = Lg;
-		      for (i = deb; i <= fin; i++)
+			last = last + difference;
+		      if (last > lg)
+			last = lg;
+		      for (i = first; i <= last; i++)
 			{
 			  x = dist[i][j - 1] + KI;
 			  y = dist[i - 1][j] + KO;
@@ -624,7 +575,7 @@ static void         LoadSpellChecker ()
 
 /*----------------------------------------------------------------------
   ----------------------------------------------------------------------*/
-static void         SaveDictFile (PtrDict docDict)
+static void SaveDictFile (PtrDict docDict)
 {
    FILE               *f;
    int                 i, j;
@@ -632,7 +583,7 @@ static void         SaveDictFile (PtrDict docDict)
    char                word[MAX_WORD_LEN];
 
    FindCompleteName (docDict->DictName, "", docDict->DictDirectory, tempbuffer, &i);
-   if (docDict->DictNbWords >= 0)
+   if (docDict->DictNbWords > 0)
      {
 	f = fopen (tempbuffer, "w");
 	if (f != NULL)
@@ -642,7 +593,7 @@ static void         SaveDictFile (PtrDict docDict)
 	     j = docDict->DictNbChars;
 	     fprintf (f, "%d %d\n", i, j);
 
-	     for (i = 0; i <= docDict->DictNbWords; i++)
+	     for (i = 0; i < docDict->DictNbWords; i++)
 	       {
 		  strcpy (word, &docDict->DictString[docDict->DictWords[i]]);
 		  Code2Asci (word);
@@ -669,22 +620,22 @@ static void         SaveDictFile (PtrDict docDict)
    (s'il n'y a plus de place : ferme et ouvre avec plus d'espace) 
    creation eventuelle et initialisation de ChkrFileDict             
   ----------------------------------------------------------------------*/
-void                AddWord (char word[MAX_WORD_LEN], PtrDict * pDict)
+void AddWord (unsigned char *word, PtrDict * pDict)
 {
-   char                word1[MAX_WORD_LEN];
+   unsigned char       word1[MAX_WORD_LEN];
    int                 ret;
    Name                DiNom;
    ThotBool            OKinsere = TRUE;
    PtrDict             docDict;
 
    docDict = *pDict;
-   strcpy (word1, word);
+   strcpy ((char*)word1, (char*)word);
    /* verifier que ce mot est bien en caracteres iso */
-   if (IsIso (word1))
+   if (IsIso ((char*)word1))
      {
 	/* ajout d'un mot dans le dictionnaire docDict */
 	SmallLettering (word1);
-	Asci2Code (word1);
+	Asci2Code ((char*)word1);
 	ret = InsertWord (docDict, word1);
 	switch (ret)
 	      {
@@ -744,7 +695,7 @@ void                AddWord (char word[MAX_WORD_LEN], PtrDict * pDict)
 
 /*----------------------------------------------------------------------
   ----------------------------------------------------------------------*/
-static void            InitChecker ()
+static void InitChecker ()
 {
    int                 j;
 
@@ -768,7 +719,7 @@ static void            InitChecker ()
    met dans ChkrCorrection les propositions de correction du mot       
    qui se trouve dans ChkrErrWord                                   
   ----------------------------------------------------------------------*/
-void                GiveProposal (Language language, PtrDict docDict)
+void GiveProposal (Language language, PtrDict docDict)
 {
    PtrDict             globalDict;
    PtrDict             personalDict;
@@ -784,23 +735,23 @@ void                GiveProposal (Language language, PtrDict docDict)
    nameDict = (PtrDict) TtaGetSecondaryDictionary (0);
 
    /* on suppose que ChkrErrWord contient la description du mot errone' */
-   Cmp (ChkrErrWord, globalDict);
+   Cmp ((unsigned char *)ChkrErrWord, globalDict);
 
    /* calcul local avec le dictionnaire personnel s'il existe */
    if (personalDict != NULL)
-      Cmp (ChkrErrWord, personalDict);
+      Cmp ((unsigned char *)ChkrErrWord, personalDict);
 
    /* calcul local avec le dictionnaire du document s'il n'est pas vide */
    if (docDict != NULL)
-      Cmp (ChkrErrWord, docDict);
+      Cmp ((unsigned char *)ChkrErrWord, docDict);
 
    /* calcul local avec le dictionnaire de sigles s'il n'est pas vide */
    if (acronymDict != NULL)
-      Cmp (ChkrErrWord, acronymDict);
+      Cmp ((unsigned char *)ChkrErrWord, acronymDict);
 
    /* calcul local avec le dictionnaire de noms s'il n'est pas vide */
    if (nameDict != NULL)
-      Cmp (ChkrErrWord, nameDict);
+      Cmp ((unsigned char *)ChkrErrWord, nameDict);
    /* remplissage de ChkrCorrection  */
    LoadSpellChecker ();
 }
@@ -822,9 +773,9 @@ static void init_Tsub (FILE *ftsub)
 	 Tsub[i][j] = KD;
    while (fscanf (ftsub, "%s%s%s", ch1, ch2, ch3) != EOF)
      {
-	sscanf (ch1, "%c", &x);
-	sscanf (ch2, "%c", &y);
-	sscanf (ch3, "%c", &valeur);
+	sscanf ((char *)ch1, "%c", (char*)&x);
+	sscanf ((char *)ch2, "%c", (char*)&y);
+	sscanf ((char *)ch3, "%c", (char*)&valeur);
 	switch (valeur)
 	  {
 	  case 'b':
@@ -845,7 +796,7 @@ static void init_Tsub (FILE *ftsub)
 
 /*----------------------------------------------------------------------
   ----------------------------------------------------------------------*/
-static void         init_param (FILE * fd)
+static void init_param (FILE * fd)
 {
    int                 i, cc, ii, oo, pp, bb, mm, dd, ss, rr;
 
@@ -889,7 +840,7 @@ static void         init_param (FILE * fd)
 
 /*----------------------------------------------------------------------
   ----------------------------------------------------------------------*/
-static void         DefaultParams (int lettres)
+static void DefaultParams (int lettres)
 {
    int                 i;
 
@@ -907,7 +858,7 @@ static void         DefaultParams (int lettres)
    1 si OK                                         
    positionne Clavier_charge                                   
   ----------------------------------------------------------------------*/
-int                 ParametrizeChecker ()
+int ParametrizeChecker ()
 {
    FILE               *fparam;
    FILE               *ftsub;
@@ -963,52 +914,55 @@ int                 ParametrizeChecker ()
 /*----------------------------------------------------------------------
    WordReplace                                               
   ----------------------------------------------------------------------*/
-void WordReplace (char orgWord[MAX_WORD_LEN], char newWord[MAX_WORD_LEN])
+void WordReplace (unsigned char *orgWord,  unsigned char *newWord)
 {
    int                 idx;
    int                 stringLength;	/* longueur de cette chaine */
-   char                pChaineRemplace[MAX_WORD_LEN]; /* la chaine de remplacement */
-   int                 LgChaineRempl;	/* longueur de cette chaine */
+   CHAR_T             *nString, *oString;
+   int                 newStringLen;	/* longueur de cette chaine */
 
-   /* remplacer le mot errone par le mot corrige */
-   stringLength = strlen (orgWord);
-   /* initialiser LgChaineRempl et pChaineRemplace */
-   LgChaineRempl = strlen (newWord);
-   strcpy (pChaineRemplace, newWord);
+   /* replace the wrong word by the right word */
+   oString = TtaConvertByteToCHAR (orgWord, ISO_8859_1);
+   nString = TtaConvertByteToCHAR (newWord, ISO_8859_1);
+   stringLength = ustrlen (oString);
+   /* initialiser newStringLen et pChaineRemplace */
+   newStringLen = ustrlen (nString);
 
    /* substitue la nouvelle chaine et la selectionne */
    if (ChkrRange->SStartToEnd)
      {
-	idx = ChkrIndChar - stringLength + 1;
+	idx = ChkrIndChar - stringLength;
 	OpenHistorySequence (ChkrRange->SDocument, ChkrElement, ChkrElement,
-			     idx, idx+stringLength-1);
+			     NULL, idx, idx + stringLength);
 	AddEditOpInHistory (ChkrElement, ChkrRange->SDocument, TRUE, TRUE);
 	CloseHistorySequence (ChkrRange->SDocument);
 	ReplaceString (ChkrRange->SDocument, ChkrElement, idx,
-		       stringLength, pChaineRemplace, LgChaineRempl, TRUE);
+		       stringLength, nString, newStringLen, TRUE);
 	/* met a jour ChkrIndChar */
-	ChkrIndChar = idx + LgChaineRempl - 1;
+	ChkrIndChar = idx + newStringLen;
 
 	/* met eventuellement a jour la borne de fin du domaine de recherche */
 	if (ChkrElement == ChkrRange->SEndElement)
 	   /* la borne est dans l'element ou` on a fait le remplacement */
-	   if (ChkrRange->SEndChar != 0)
+	   if (ChkrRange->SEndChar > 1)
 	      /* la borne n'est pas a la fin de l'element, on decale la borne */
-	      ChkrRange->SEndChar += LgChaineRempl - stringLength;
+	      ChkrRange->SEndChar += newStringLen - stringLength;
      }
    else
      {
-	idx = ChkrIndChar + 1;
+	idx = ChkrIndChar;
 	OpenHistorySequence (ChkrRange->SDocument, ChkrElement, ChkrElement,
-			     idx, idx+stringLength-1);
+			     NULL, idx, idx+stringLength-1);
 	AddEditOpInHistory (ChkrElement, ChkrRange->SDocument, TRUE, TRUE);
 	CloseHistorySequence (ChkrRange->SDocument);
 	ReplaceString (ChkrRange->SDocument, ChkrElement, idx,
-		       stringLength, pChaineRemplace, LgChaineRempl, TRUE);
+		       stringLength, nString, newStringLen, TRUE);
      }
 
+   TtaFreeMemory (nString);
+   TtaFreeMemory (oString);
    /* met eventuellement a jour la selection initiale */
-   UpdateDuringSearch (ChkrElement, LgChaineRempl - stringLength);
+   UpdateDuringSearch (ChkrElement, newStringLen - stringLength);
 }
 
 
@@ -1016,7 +970,7 @@ void WordReplace (char orgWord[MAX_WORD_LEN], char newWord[MAX_WORD_LEN])
    CheckChangeSelection retourne vrai si la selection a change      
    depuis la derniere recherche d'erreur.                       
   ----------------------------------------------------------------------*/
-ThotBool            CheckChangeSelection ()
+ThotBool CheckChangeSelection ()
 {
    PtrDocument         docsel;
    PtrElement          pEl1, pElN;
@@ -1033,13 +987,24 @@ ThotBool            CheckChangeSelection ()
      {
 	pEl1 = ChkrElement;
 	c1 = ChkrIndChar;
-	ok = GetCurrentSelection (&ChkrRange->SDocument, &ChkrElement, &pElN, &cN, &ChkrIndChar);
+	ok = GetCurrentSelection (&ChkrRange->SDocument, &ChkrElement, &pElN,
+				  &cN, &ChkrIndChar);
+	if (ChkrIndChar < cN)
+	  /* selection is just a caret */
+	  ChkrIndChar = cN;
+	if (pElN && ChkrIndChar == 0)
+	  /* the whole element is selected */
+	  ChkrIndChar = pElN->ElVolume;
      }
    else
      {
 	pEl1 = ChkrElement;
 	c1 = ChkrIndChar;
-	ok = GetCurrentSelection (&ChkrRange->SDocument, &pElN, &ChkrElement, &ChkrIndChar, &cN);
+	ok = GetCurrentSelection (&ChkrRange->SDocument, &pElN, &ChkrElement,
+				  &ChkrIndChar, &cN);
+	if (pElN && ChkrIndChar == 0)
+	  /* the whole element is selected */
+	  ChkrIndChar = 1;
      }
 
    if (!ok)
@@ -1059,8 +1024,6 @@ ThotBool            CheckChangeSelection ()
      {
 	ok = (ChkrElement != pEl1 || ChkrElement != pElN || ChkrIndChar != c1);
 	/* S'il s'agit d'une nouvelle selection dans le document */
-	if (ok && ChkrIndChar != 0)
-	   ChkrIndChar--;
 	return (ok);
      }
 }
@@ -1071,139 +1034,134 @@ ThotBool            CheckChangeSelection ()
    retourne TRUE si oui                               
    FALSE sinon                               
   ----------------------------------------------------------------------*/
-static ThotBool     CheckCharList (char car, char *listcar)
+static ThotBool CheckCharList (unsigned char car, unsigned char *listcar)
 {
-   int                 i;
+  int                 i, l;
 
-   for (i = 0; (size_t) i < strlen (listcar); i++)
-     {
-	if (car == listcar[i])
-	   return (TRUE);
-     }
-   return (FALSE);
+  l = strlen ((char *)listcar);
+  for (i = 0; i < l; i++)
+    {
+      if (car == listcar[i])
+	return (TRUE);
+    }
+  return (FALSE);
 }
 
 /*----------------------------------------------------------------------
-   ACeCar retourne TRUE si le mot contient un des caracteres       
-   contenu dans ListCar.                                    
+   IncludeAChar returns TRUE if the word includes a character of the
+   list ListCar.
   ----------------------------------------------------------------------*/
-static ThotBool     ACeCar (char word[MAX_WORD_LEN])
+static ThotBool IncludeAChar (unsigned char *word)
 {
+  int                 i;
+  int                 len;
+  ThotBool            result;
 
-   ThotBool            result;
-   int                 i;
-   int                 longueur;
-
-   result = FALSE;
-   longueur = strlen (word);
-   if (longueur > 0)
-     {
-	for (i = 0; i < longueur && (result == FALSE); i++)
-	   if (CheckCharList (word[i], RejectedChar) == TRUE)
-	      result = TRUE;
-     }
-   return result;
+  result = FALSE;
+  len = strlen ((char *)word);
+  if (len > 0)
+    {
+      for (i = 0; i < len && (result == FALSE); i++)
+	if (CheckCharList (word[i], (unsigned char *)RejectedChar) == TRUE)
+	  result = TRUE;
+    }
+  return result;
 }
 
 /*----------------------------------------------------------------------
-   IncludeANumber                                                       
-   retourne TRUE si le mot contient au moins un chiffre arabe.     
+   IncludeANumber returns TRUE if the word include a number.
   ----------------------------------------------------------------------*/
-static ThotBool     IncludeANumber (char word[MAX_WORD_LEN])
+static ThotBool IncludeANumber (unsigned char *word)
 {
+  int                 i;
+  int                 len;
+  ThotBool            result;
 
-   ThotBool            result;
-   int                 i;
-   int                 longueur;
-
-   result = FALSE;
-   longueur = strlen (word);
-   if (longueur > 0)
-     {
-	for (i = 0; i < longueur && (result == FALSE); i++)
-	   if (word[i] >= '0' && word[i] <= '9')
-	      result = TRUE;
-     }
-   return result;
+  result = FALSE;
+  len = strlen ((char *)word);
+  if (len > 0)
+    {
+      for (i = 0; i < len && (result == FALSE); i++)
+	if (word[i] >= '0' && word[i] <= '9')
+	  result = TRUE;
+    }
+  return result;
 }
 
 /*----------------------------------------------------------------------
    IsANumber retourne TRUE si le mot est forme' uniquement de    
    chiffres decimaux arabes.                                       
   ----------------------------------------------------------------------*/
-static ThotBool     IsANumber (char word[MAX_WORD_LEN])
+static ThotBool IsANumber (unsigned char *word)
 {
+  int                 i;
+  int                 len;
+  ThotBool            result;
 
-   ThotBool            result;
-   int                 i;
-   int                 longueur;
-
-   result = FALSE;
-   longueur = strlen (word);
-   if (longueur > 0)
-     {
-
-	result = TRUE;
-	for (i = 0; i < longueur && result; i++)
-	   if (word[i] < '0' || word[i] > '9')
-	      result = FALSE;
-     }
-   return result;
+  result = FALSE;
+  len = strlen ((char *)word);
+  if (len > 0)
+    {
+      
+      result = TRUE;
+      for (i = 0; i < len && result; i++)
+	if (word[i] < 48 || word[i] > 57)
+	  result = FALSE;
+    }
+  return result;
 }
 
 /*----------------------------------------------------------------------
    InRoman retourne TRUE si le mot est forme' uniquement de       
    chiffres romains.                                               
   ----------------------------------------------------------------------*/
-static ThotBool     InRoman (char word[MAX_WORD_LEN])
+static ThotBool InRoman (unsigned char *word)
 {
-   ThotBool            result;
-   int                 i, j, nbcar;
-   int                 longueur, lg1;
-   char                cecar;
+  int                 i, j, nbcar;
+  int                 len, lg1;
+  unsigned char       c;
+  ThotBool            result;
 
-   result = FALSE;
-   longueur = strlen (word);
-   if (longueur > 0)
-     {
-	result = TRUE;
-	for (i = 0; i < longueur && result; i++)
-	   if (CheckCharList (word[i], NRomain) != TRUE)
-	      result = FALSE;
-     }
-   if (result == TRUE)
-      /* analyse plus fine de ce "possible" chiffre romain */
-     {
-	/* pas plus de 3 fois la meme lettre successivement dans ce nombre */
-	lg1 = longueur - 3;
-	for (i = 0; i < lg1 && result; i++)
-	  {
-	     cecar = word[i];
-	     nbcar = 1;
-	     for (j = i + 1; j < longueur && word[j] == cecar; j++)
-		nbcar++;
-	     if (nbcar > 3)
-		/* ce n'est pas un "bon" chiffre romain */
-		result = FALSE;
-	  }
+  result = FALSE;
+  len = strlen ((char *)word);
+  if (len > 0)
+    {
+      result = TRUE;
+      for (i = 0; i < len && result; i++)
+	if (CheckCharList (word[i], (unsigned char *)NRomain) != TRUE)
+	  result = FALSE;
+    }
+  if (result == TRUE)
+    /* analyse plus fine de ce "possible" chiffre romain */
+    {
+      /* pas plus de 3 fois la meme lettre successivement dans ce nombre */
+      lg1 = len - 3;
+      for (i = 0; i < lg1 && result; i++)
+	{
+	  c = word[i];
+	  nbcar = 1;
+	  for (j = i + 1; j < len && word[j] == c; j++)
+	    nbcar++;
+	  if (nbcar > 3)
+	    /* ce n'est pas un "bon" chiffre romain */
+	    result = FALSE;
+	}
 
-	/* pas plus de 1 fois V, L ou D successivement dans ce nombre */
-	for (i = 0; i < longueur - 1 && result; i++)
-	  {
-	     if (CheckCharList (word[i], NRomainIsole) == TRUE)
-	       {
-		  if (word[i + 1] == word[i])
-		     result = FALSE;
-	       }
-	  }
-
-	/* ne pas considerer "M" comme un romain */
- if (strlen (word) == 1 && word[0] == 'M')
- result = FALSE;
-	/* verifier aussi l'ordre des I V X L C D M */
-	/* A FAIRE */
-     }
-   return result;
+      /* pas plus de 1 fois V, L ou D successivement dans ce nombre */
+      for (i = 0; i < len - 1 && result; i++)
+	{
+	  if (CheckCharList (word[i], (unsigned char *)NRomainIsole) == TRUE &&
+	      word[i + 1] == word[i])
+	    result = FALSE;
+	}
+      
+      /* ne pas considerer "M" comme un romain */
+      if (strlen ((char *)word) == 1 && word[0] == 'M')
+	result = FALSE;
+      /* verifier aussi l'ordre des I V X L C D M */
+      /* A FAIRE */
+    }
+  return result;
 }
 
 
@@ -1213,31 +1171,24 @@ static ThotBool     InRoman (char word[MAX_WORD_LEN])
    chiffre arabe, car. special)                                       
    retourne FALSE sinon : le mot sera alors verifie par le correcteur 
   ----------------------------------------------------------------------*/
-static ThotBool     IgnoreWord (char word[MAX_WORD_LEN])
+static ThotBool IgnoreWord (unsigned char *word)
 {
 
-   ThotBool            result = FALSE;
+  ThotBool            result = FALSE;
 
-   /* les mots en capitale */
-   if (IgnoreUppercase)
-      if (IsUpperCase (word))
-	 return (TRUE);
-
-   /* les mots contenant un chiffre arabe */
-   if (IgnoreArabic)
-      if (IncludeANumber (word))
-	 return (TRUE);
-
-   /* les chiffres romains */
-   if (IgnoreRoman)
-      if (InRoman (word))
-	 return (TRUE);
-
-   /* les mots contenant au moins l'un des caracteres de RejectedChar[] */
-   if (IgnoreSpecial)
-      result = ACeCar (word);
-
-   return result;
+  /* les mots en capitale */
+  if (IgnoreUppercase && IsUpperCase ((char *)word))
+    return (TRUE);
+  /* les mots contenant un chiffre arabe */
+  if (IgnoreArabic && IncludeANumber (word))
+    return (TRUE);
+  /* les chiffres romains */
+  if (IgnoreRoman && InRoman (word))
+    return (TRUE);
+  /* les mots contenant au moins l'un des caracteres de RejectedChar[] */
+  if (IgnoreSpecial)
+    result = IncludeAChar (word);
+  return result;
 }
 
 
@@ -1246,13 +1197,23 @@ static ThotBool     IgnoreWord (char word[MAX_WORD_LEN])
    selectionne dans la vue courante du document.                
    Le mot errone' est mis dans ChkrErrWord.                           
   ----------------------------------------------------------------------*/
-void NextSpellingError (char word[MAX_WORD_LEN], PtrDict docDict)
+void NextSpellingError (unsigned char *word, PtrDict docDict)
 {
   Language            language;
-  ThotBool            ok, novalid;
+  CHAR_T              s[MAX_WORD_LEN];
+  int                 j;
   int                 i;
+  ThotBool            ok, novalid;
 
-  i = 0;
+  /* get the CHAR_T string */
+  j = 0;
+  while (word[j] != EOS)
+    {
+      s[j] = TtaGetWCFromChar (word[j], ISO_8859_1);
+      j++;
+    }
+  s[j] = EOS;
+  i = 1;
   do
     {
       /* Recherche un mot a corriger */
@@ -1261,13 +1222,24 @@ void NextSpellingError (char word[MAX_WORD_LEN], PtrDict docDict)
       while (ok && novalid)
 	{
 	  if (ChkrRange->SStartToEnd)
-	    ok = SearchNextWord (&ChkrElement, &i, &ChkrIndChar, word, ChkrRange);
+	    ok = SearchNextWord (&ChkrElement, &i, &ChkrIndChar, s, ChkrRange);
 	  else
-	    ok = SearchPreviousWord (&ChkrElement, &ChkrIndChar, &i, word, ChkrRange);
+	    ok = SearchPreviousWord (&ChkrElement, &ChkrIndChar, &i, s, ChkrRange);
 	  /* Is it a valid selection ? */
 	  if (ok && ChkrElement->ElParent != NULL)
-	    novalid = TypeHasException (ExcNoSpellCheck, ChkrElement->ElParent->ElTypeNumber, ChkrElement->ElParent->ElStructSchema);
+	    novalid = (TypeHasException (ExcNoSpellCheck,
+					ChkrElement->ElParent->ElTypeNumber,
+					ChkrElement->ElParent->ElStructSchema));
 	}
+
+      /* get back the Iso string */
+      j = 0;
+      while (s[j] != 0)
+	{
+	  word[j] = TtaGetCharFromWC (s[j], ISO_8859_1);
+	  j++;
+	}
+      word[j] = EOS;
       
       if (ok)
 	{
@@ -1295,16 +1267,17 @@ void NextSpellingError (char word[MAX_WORD_LEN], PtrDict docDict)
     }
   /* saute les mots qui sont dans une langue SANS dictionnaire */
   while (ok && (i > 0 || i == -1));
-  
-  i = strlen (word);
+
+  i = strlen ((char *)word);
   if (i > 0)
     {
-      /* on a trouve un mot */
-      /* selectionner le mot a corriger */
+      /* a word is found, select it */
       if (ChkrRange->SStartToEnd)
-	SelectString (ChkrRange->SDocument, ChkrElement, ChkrIndChar - i + 1, ChkrIndChar);
+	SelectString (ChkrRange->SDocument, ChkrElement,
+		      ChkrIndChar - i, ChkrIndChar);
       else
-	SelectString (ChkrRange->SDocument, ChkrElement, ChkrIndChar + 1, ChkrIndChar + i);
+	SelectString (ChkrRange->SDocument, ChkrElement,
+		      ChkrIndChar, ChkrIndChar + i);
     }
-  strcpy (ChkrErrWord, word);
+  strcpy ((char *)ChkrErrWord, (char *)word);
 }
