@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA 1996-2001
+ *  (c) COPYRIGHT INRIA 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -11,7 +11,7 @@
  * Authors: I. Vatton, S. Bonhomme (INRIA)
  *          
  */
-
+#include "thot_gui.h"
 #include "thot_sys.h"
 #include "constmedia.h"
 #include "typemedia.h"
@@ -31,7 +31,7 @@
 /* ----------------------------------------------------------------------
    TtaPrepareUndo returns TRUE if a undo sequence is opened
   ----------------------------------------------------------------------*/
-ThotBool      TtaPrepareUndo (Document document)
+ThotBool TtaPrepareUndo (Document document)
 {
 
   if (document < 1 || document > MAX_DOCUMENTS)
@@ -48,7 +48,6 @@ ThotBool      TtaPrepareUndo (Document document)
    Open a sequence of editing operations in the history.
 
    Parameters:
-
    document: the concerned document
    firstSel: indicate the selection that must be set when the operation 
    will be undone. If null, the current selection is recorded.
@@ -59,11 +58,11 @@ ThotBool      TtaPrepareUndo (Document document)
    lastSelChar: indicate the selection that must be set when the operation 
    will be undone.
   ----------------------------------------------------------------------*/
-void          TtaOpenUndoSequence (Document document, Element firstSel,
-				   Element lastSel, int firstSelChar,
+void TtaOpenUndoSequence (Document document, Element firstSel,
+			  Element lastSel, int firstSelChar,
 				   int lastSelChar)
 {
-  int i;
+  int           i;
 
   if (document < 1 || document > MAX_DOCUMENTS)
       TtaError (ERR_invalid_document_parameter);
@@ -75,31 +74,48 @@ void          TtaOpenUndoSequence (Document document, Element firstSel,
 	{
 	  /* gets the current selection */
 	  TtaGiveFirstSelectedElement (document, &firstSel, &firstSelChar, &i);
-	  /* gets the last selected element */
-	  TtaGiveLastSelectedElement (document, &lastSel, &i, &lastSelChar);
 	  if (SelPosition)
-	    firstSelChar = lastSelChar + 1;
+	    {
+	      lastSelChar = i;
+	      lastSel = firstSel;
+	    }
+	  else
+	    /* gets the last selected element */
+	    TtaGiveLastSelectedElement (document, &lastSel, &i, &lastSelChar);
 	}
 
       /* inits the history sequence */
       OpenHistorySequence (LoadedDocument [document - 1], 
-			   (PtrElement)firstSel, (PtrElement)lastSel, 
+			   (PtrElement)firstSel, (PtrElement)lastSel, NULL,
 			   firstSelChar, lastSelChar);	     
     }
 }
 
+/*----------------------------------------------------------------------
+   TtaExtendUndoSequence
+
+   Reopen the latest sequence of editing operations in the history.
+  ----------------------------------------------------------------------*/
+void TtaExtendUndoSequence (Document document)
+{
+  if (document < 1 || document > MAX_DOCUMENTS)
+    TtaError (ERR_invalid_document_parameter);
+  else if (LoadedDocument [document - 1] == NULL)
+    TtaError (ERR_invalid_document_parameter);
+  else 
+    LoadedDocument [document - 1]->DocEditSequence = TRUE;
+}
+
 /* ----------------------------------------------------------------------
    TtaCloseUndoSequence
 
    Close a sequence of editing operations in the history.
-
    Parameters:
    document: the concerned document
-
    Return value:
        FALSE if the closed sequence is empty, TRUE otherwise
   ----------------------------------------------------------------------*/
-ThotBool      TtaCloseUndoSequence (Document document)
+ThotBool TtaCloseUndoSequence (Document document)
 {
   ThotBool	result;
 
@@ -118,13 +134,11 @@ ThotBool      TtaCloseUndoSequence (Document document)
     
    Register a single element creation in the editing history
    Registration must be performed AFTER the element is inserted
-
    Parameters:
-
    element: the created element
    document: the concerned document
   ----------------------------------------------------------------------*/
-void         TtaRegisterElementCreate (Element element, Document document)
+void TtaRegisterElementCreate (Element element, Document document)
 {
   if (document < 1 || document > MAX_DOCUMENTS)
     TtaError (ERR_invalid_document_parameter);
@@ -143,13 +157,11 @@ void         TtaRegisterElementCreate (Element element, Document document)
    Register a single element Deletion in the editing history
    Registration must be performed BEFORE the element is actually
    removed from the structure.
-
    Parameters:
-
    element: the element to be deleted
    document: the concerned document
   ----------------------------------------------------------------------*/
-void         TtaRegisterElementDelete (Element element, Document document)
+void TtaRegisterElementDelete (Element element, Document document)
 {
   if (document < 1 || document > MAX_DOCUMENTS)
     TtaError (ERR_invalid_document_parameter);
@@ -168,12 +180,11 @@ void         TtaRegisterElementDelete (Element element, Document document)
    Register a single element replacement in the editing history
    Registration must be performed BEFORE the element is actually
    replaced.
-   
    Parameters:
    element: the replaced element
    document: the concerned document
   ----------------------------------------------------------------------*/
-void         TtaRegisterElementReplace (Element element, Document document)
+void TtaRegisterElementReplace (Element element, Document document)
 {
   if (document < 1 || document > MAX_DOCUMENTS)
     TtaError (ERR_invalid_document_parameter);
@@ -186,19 +197,39 @@ void         TtaRegisterElementReplace (Element element, Document document)
 			);
 }
 
+/* ----------------------------------------------------------------------
+   TtaRegisterElementTypeChange
+    
+   Register a changing of element type in the editing history
+   Parameters:
+   element: the element concerned
+   oldElementType: the element type to be restored when undoing
+   document: the concerned document
+  ----------------------------------------------------------------------*/
+void TtaRegisterElementTypeChange (Element element, int oldElementType,
+				   Document document)
+{
+  if (document < 1 || document > MAX_DOCUMENTS)
+    TtaError (ERR_invalid_document_parameter);
+  else if (LoadedDocument [document - 1] == NULL)
+    TtaError (ERR_invalid_document_parameter);
+  else 
+    AddChangeTypeOpInHistory ((PtrElement)element, oldElementType,
+			      LoadedDocument[document-1]); 
+}
+
 /* ----------------------------------------------------------------------
    TtaRegisterAttributeCreate
     
    Register a single attribute creation in the editing history
    Registration must be performed AFTER the attribute is inserted
-
    Parameters:
    attribute: the created attribute
    element: the element to which the attribute has been attached
    document: the concerned document
   ----------------------------------------------------------------------*/
-void         TtaRegisterAttributeCreate (Attribute attribute, Element element,
-					 Document document)
+void TtaRegisterAttributeCreate (Attribute attribute, Element element,
+				 Document document)
 {
   if (document < 1 || document > MAX_DOCUMENTS)
     TtaError (ERR_invalid_document_parameter);
@@ -218,14 +249,13 @@ void         TtaRegisterAttributeCreate (Attribute attribute, Element element,
    Register a single attribute Deletion in the editing history
    Registration must be performed BEFORE the attribute is actually
    removed from the structure.
-
    Parameters:
    attribute: the attribute to be deleted
    element: the element to which this attribute is attached
    document: the concerned document
   ----------------------------------------------------------------------*/
-void         TtaRegisterAttributeDelete (Attribute attribute, Element element,
-					 Document document)
+void TtaRegisterAttributeDelete (Attribute attribute, Element element,
+				 Document document)
 {
   if (document < 1 || document > MAX_DOCUMENTS)
     TtaError (ERR_invalid_document_parameter);
@@ -245,14 +275,13 @@ void         TtaRegisterAttributeDelete (Attribute attribute, Element element,
    Register a single attribute replacement in the editing history
    Registration must be performed BEFORE the element is actually
    replaced.
-   
    Parameters:
    attribute: the attribute
    element: the element to which the attribute is attached
    document: the concerned document
   ----------------------------------------------------------------------*/
-void         TtaRegisterAttributeReplace (Attribute attribute, Element element,
-					  Document document)
+void TtaRegisterAttributeReplace (Attribute attribute, Element element,
+				  Document document)
 {
   if (document < 1 || document > MAX_DOCUMENTS)
     TtaError (ERR_invalid_document_parameter);
@@ -271,7 +300,7 @@ void         TtaRegisterAttributeReplace (Attribute attribute, Element element,
 
    Clears all editing operations registered in the editing history of document.
   ----------------------------------------------------------------------*/
-void         TtaClearUndoHistory (Document document)
+void TtaClearUndoHistory (Document document)
 {
   if (document < 1 || document > MAX_DOCUMENTS)
     TtaError (ERR_invalid_document_parameter);
@@ -287,7 +316,7 @@ void         TtaClearUndoHistory (Document document)
    Cancel the latest sequence of editing operations registered in the
    editing history of document.
   ----------------------------------------------------------------------*/
-void         TtaCancelLastRegisteredSequence (Document document)
+void TtaCancelLastRegisteredSequence (Document document)
 {
   if (document < 1 || document > MAX_DOCUMENTS)
     TtaError (ERR_invalid_document_parameter);
@@ -304,9 +333,9 @@ void         TtaCancelLastRegisteredSequence (Document document)
    of document, only if it's an attribute operation for element oldEl.
    In that case, make it related to element newEl and attribute newAttr.
   ----------------------------------------------------------------------*/
-void         TtaChangeLastRegisteredAttr (Element oldEl, Element newEl,
-					  Attribute oldAttr, Attribute newAttr,
-					  Document document)
+void TtaChangeLastRegisteredAttr (Element oldEl, Element newEl,
+				  Attribute oldAttr, Attribute newAttr,
+				  Document document)
 {
   if (document < 1 || document > MAX_DOCUMENTS)
     TtaError (ERR_invalid_document_parameter);
@@ -324,7 +353,7 @@ void         TtaChangeLastRegisteredAttr (Element oldEl, Element newEl,
    Replace the latest operation registered in the editing history of document
    from an attribute value modification to an attribute value deletion.
   ----------------------------------------------------------------------*/
-void         TtaReplaceLastRegisteredAttr (Document document)
+void TtaReplaceLastRegisteredAttr (Document document)
 {
   if (document < 1 || document > MAX_DOCUMENTS)
     TtaError (ERR_invalid_document_parameter);
@@ -339,7 +368,7 @@ void         TtaReplaceLastRegisteredAttr (Document document)
 
    Cancel the latest operation registered in the editing history of document.
   ----------------------------------------------------------------------*/
-void         TtaCancelLastRegisteredOperation (Document document)
+void TtaCancelLastRegisteredOperation (Document document)
 {
   if (document < 1 || document > MAX_DOCUMENTS)
     TtaError (ERR_invalid_document_parameter);
@@ -349,6 +378,57 @@ void         TtaCancelLastRegisteredOperation (Document document)
     CancelLastEditFromHistory (LoadedDocument [document - 1]);
 }
 
+/* ----------------------------------------------------------------------
+   TtaCancelLastRegisteredAttrOperation
+
+   Cancel the latest attribute registered in the editing history of document.
+  ----------------------------------------------------------------------*/
+void TtaCancelLastRegisteredAttrOperation (Document document)
+{
+  if (document < 1 || document > MAX_DOCUMENTS)
+    TtaError (ERR_invalid_document_parameter);
+  else if (LoadedDocument [document - 1] == NULL)
+    TtaError (ERR_invalid_document_parameter);
+  else 
+    CancelLastAttrOperationFromHistory (LoadedDocument [document - 1]);
+}
+
+/* ----------------------------------------------------------------------
+   TtaGetLastCreatedElemInHistory
+
+   If the last operation recorded in the history is the creation of an
+   element, return that element, otherwise return NULL.
+  ----------------------------------------------------------------------*/
+Element TtaGetLastCreatedElemInHistory (Document document)
+{
+  Element   el;
+
+  el = NULL;
+  if (document < 1 || document > MAX_DOCUMENTS)
+    TtaError (ERR_invalid_document_parameter);
+  else if (LoadedDocument [document - 1] == NULL)
+    TtaError (ERR_invalid_document_parameter);
+  else 
+    el = (Element) GetLastCreatedElemInHistory (LoadedDocument [document - 1]);
+  return el;
+}
+
+/* ----------------------------------------------------------------------
+   TtaChangeInfoLastRegisteredElem
+
+   Change the info field associated with the latest operation registered in
+   the editing history of document
+  ----------------------------------------------------------------------*/
+void TtaChangeInfoLastRegisteredElem (Document document, int newInfo)
+{
+  if (document < 1 || document > MAX_DOCUMENTS)
+    TtaError (ERR_invalid_document_parameter);
+  else if (LoadedDocument [document - 1] == NULL)
+    TtaError (ERR_invalid_document_parameter);
+  else
+    ChangeInfoLastRegisteredElem (LoadedDocument [document - 1], newInfo);
+}
+
 /* ----------------------------------------------------------------------
    TtaUndoNoRedo
 
@@ -356,7 +436,7 @@ void         TtaCancelLastRegisteredOperation (Document document)
    of document and forget about this sequence: it won't be redone by
    the next Redo command issued by the user.
   ----------------------------------------------------------------------*/
-void         TtaUndoNoRedo (Document document)
+void TtaUndoNoRedo (Document document)
 {
   if (document < 1 || document > MAX_DOCUMENTS)
     TtaError (ERR_invalid_document_parameter);
