@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001.
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -12,7 +12,7 @@
  *          R. Guetari (W3C/INRIA) - Windows version
  *
  */
-
+#include "thot_gui.h"
 #include "thot_sys.h"
 #include "constmedia.h"
 #include "typemedia.h"
@@ -26,6 +26,48 @@
 #include "memory_f.h"
 #include "picture_f.h"
 
+#ifdef _GL
+#include "glwindowdisplay.h"
+#endif /*_GL*/
+
+
+/*----------------------------------------------------------------------
+  CleanPictInfo frees the picture information but not the structure itself
+  ----------------------------------------------------------------------*/
+void CleanPictInfo (PictInfo *imageDesc)
+{
+  if (imageDesc)
+    {
+#ifndef _GL
+      if (imageDesc->PicPixmap != None)
+	{
+	  FreePixmap (imageDesc->PicPixmap);
+	  imageDesc->PicPixmap = None;
+#if defined(_MOTIF) || defined(_WINGUI)
+	  FreePixmap (imageDesc->PicMask);
+#endif /* #if defined(_MOTIF) || defined(_WINGUI)    */ 
+#ifdef _GTK
+	  /*Frees imlib struct that contains the real ref to pics */
+	  if (imageDesc->im)
+	    gdk_imlib_destroy_image (imageDesc->im);
+	  imageDesc->im = NULL;
+#endif /* _GTK */
+	  imageDesc->PicMask = None;
+	}
+
+#ifdef _WINGUI
+	imageDesc->PicBgMask = -1;
+#endif /* _WINGUI */
+#else /*_GL*/
+	FreeGlTexture (imageDesc);
+#endif /*_GL*/
+	imageDesc->PicWArea = 0;
+	imageDesc->PicHArea = 0;
+	imageDesc->PicWidth = 0;
+	imageDesc->PicHeight = 0;
+    }
+}
+
 
 /*----------------------------------------------------------------------
    NewPictInfo cree un descripteur par element image.       
@@ -37,15 +79,13 @@
   ----------------------------------------------------------------------*/
 void NewPictInfo (PtrAbstractBox pAb, PathBuffer filename, int imagetype)
 {
-  PtrTextBuffer       pBuffer;
   PictInfo           *imageDesc = NULL;
   char               *ptr = NULL;
   PictureScaling      picPresent;
   int                 len;
 
   picPresent = DefaultPres;
-  if (!pAb->AbPresentationBox &&
-      pAb->AbElement->ElTerminal && pAb->AbElement->ElLeafType == LtPicture)
+  if (!pAb->AbPresentationBox && pAb->AbElement->ElLeafType == LtPicture)
     {
       /* image element -> attach the element descriptor to the abtract box */
       imageDesc = (PictInfo *) pAb->AbElement->ElPictInfo;
@@ -55,37 +95,31 @@ void NewPictInfo (PtrAbstractBox pAb, PathBuffer filename, int imagetype)
 	  imageDesc = (PictInfo *) TtaGetMemory (sizeof (PictInfo));
 	  memset (imageDesc, 0, sizeof (PictInfo));
 	  pAb->AbElement->ElPictInfo = (int *) imageDesc;
+	  pAb->AbPictInfo = (int *) imageDesc;
 	}
       else
-	  /* don't reset the presentation value */
-	  picPresent = imageDesc->PicPresent;
-
-      pAb->AbPictInfo = pAb->AbElement->ElPictInfo;
-      if (filename == NULL)
 	{
-	  GetTextBuffer (&pBuffer);
-	  pAb->AbElement->ElText = pBuffer;
-	  ptr = &pBuffer->BuContent[0];
+	  /* don't reset the presentation value */
+	  pAb->AbPictInfo = pAb->AbElement->ElPictInfo;
+	  return;
+	}
+      ptr = filename;
+    }
+  else if (pAb->AbPresentationBox)
+    {
+      /*  It's a presentation box -> Create the descriptor */
+      imageDesc = (PictInfo *) pAb->AbPictInfo;
+      if (imageDesc == NULL)
+	{
+	  imageDesc = (PictInfo *) TtaGetMemory (sizeof (PictInfo));
+	  memset (imageDesc, 0, sizeof (PictInfo));
+	  pAb->AbPictInfo = (int *) imageDesc;
 	}
       else
-	ptr = filename;
-     }
-   else if (pAb->AbPresentationBox)
-     {
-       /*  It's a presentation box -> Create the descriptor */
-       imageDesc = (PictInfo *) pAb->AbPictInfo;
-       if (imageDesc == NULL)
-	 {
-	   imageDesc = (PictInfo *) TtaGetMemory (sizeof (PictInfo));
-	   memset (imageDesc, 0, sizeof (PictInfo));
-	   pAb->AbPictInfo = (int *) imageDesc;
-	 }
-       else
-	 /* don't reset the presentation value */
-	 picPresent = imageDesc->PicPresent;
-       
-       ptr = filename;
-     }
+	/* don't reset the presentation value */
+	picPresent = imageDesc->PicPresent;
+      ptr = filename;
+    }
   else if (pAb->AbLeafType == LtCompound)
     {
       /*  It's a background image -> Create the descriptor */
@@ -106,7 +140,7 @@ void NewPictInfo (PtrAbstractBox pAb, PathBuffer filename, int imagetype)
        /* create the text buffer */
       if (filename == NULL)
 	{
-	  if (ptr != NULL)
+	  if (ptr)
 	    {
 	      TtaFreeMemory (ptr);
 	      ptr = NULL;
@@ -117,26 +151,21 @@ void NewPictInfo (PtrAbstractBox pAb, PathBuffer filename, int imagetype)
 	  len = strlen (filename) + 1;
 	  if (ptr == NULL || len > (int) strlen (ptr) + 1)
 	    {
+	      if (imageDesc->PicFileName == ptr)
+		imageDesc->PicFileName = NULL;
 	      TtaFreeMemory (ptr);
-	      ptr = TtaGetMemory (len);
+	      ptr = (char *)TtaGetMemory (len);
 	    }
 	  strcpy (ptr, filename);
 	}
     }
+  else
+    return;
 
-  if (imageDesc && imageDesc->PicFileName != ptr)
+  if (imageDesc && (imageDesc->PicFileName != ptr || ptr == NULL))
     {
-      /* Initialize image descriptor */
-      /* use the buffer allocated by the picture content */
-#ifndef _GTK
-#ifdef _WINDOWS
-      imageDesc->PicMask = -1;
-#else /* _WINDOWS */
-      FreePixmap (imageDesc->PicMask);
-      imageDesc->PicMask = None;
-#endif /* _WINDOWS */
-      FreePixmap (imageDesc->PicPixmap);
-#endif /* !_GTK */
+      /* reinitialize the image descriptor */
+      CleanPictInfo (imageDesc);
       imageDesc->PicFileName = ptr;
       imageDesc->PicType    = imagetype;
       imageDesc->PicPresent = picPresent;
@@ -144,60 +173,27 @@ void NewPictInfo (PtrAbstractBox pAb, PathBuffer filename, int imagetype)
 }
 
 
-/*----------------------------------------------------------------------
-  FreePictInfo  frees the picture information but not the structure itself
-  ----------------------------------------------------------------------*/
-void FreePictInfo (PictInfo *imageDesc)
-{
-   if (imageDesc)
-     {
-#ifndef _GTK
-       if (imageDesc->PicPixmap != None)
-#endif /* !_GTK */
-	 {
-#ifndef _GTK
-#ifdef _WINDOWS
-	   imageDesc->PicMask = -1;
-#else /* _WINDOWS */
-	   FreePixmap (imageDesc->PicMask);
-	   imageDesc->PicMask = None;
-#endif /* _WINDOWS */
-	   FreePixmap (imageDesc->PicPixmap);
-	   imageDesc->PicPixmap = None;
-#endif /* !_GTK */
-	   imageDesc->PicXArea = 0;
-	   imageDesc->PicYArea = 0;
-	   imageDesc->PicWArea = 0;
-	   imageDesc->PicHArea = 0;
-	   imageDesc->PicWidth = 0;
-	   imageDesc->PicHeight = 0;
-	 }
-
-       if ((imageDesc->PicType >= InlineHandlers) &&
-	   (PictureHandlerTable[imageDesc->PicType].FreePicture != NULL))
-	 (*(PictureHandlerTable[imageDesc->PicType].FreePicture)) (imageDesc);
-     }
-}
-
-
-
 /*----------------------------------------------------------------------
    Copie d'un PictInfo                                      
   ----------------------------------------------------------------------*/
-void                CopyPictInfo (int *Imdcopie, int *Imdsource)
+void CopyPictInfo (int *Imdcopie, int *Imdsource)
 {
    PictInfo           *imagec;
    PictInfo           *images;
 
    imagec = (PictInfo *) Imdcopie;
    images = (PictInfo *) Imdsource;
-   imagec->PicXArea = images->PicXArea;
-   imagec->PicYArea = images->PicYArea;
    imagec->PicWArea = images->PicWArea;
    imagec->PicHArea = images->PicHArea;
    imagec->PicWidth = images->PicWidth;
    imagec->PicHeight = images->PicHeight;
    imagec->PicPresent = images->PicPresent;
    imagec->PicType = images->PicType;
+#ifdef _GL
+   imagec->TextureBind = images->TextureBind;
+#endif /*_GL*/
 }
 
+
+
+
