@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001.
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -11,7 +11,7 @@
  * Authors: I. Vatton, N. Layaida (INRIA)
  *
  */
-
+#include "thot_gui.h"
 #include "thot_sys.h"
 #include "constmedia.h"
 #include "libmsg.h"
@@ -32,10 +32,10 @@
 #include "units_f.h"
 #include "memory_f.h"
 
+#ifndef ABS
 #define ABS(x) (x<0?-x:x)
-#define MAX(x,y) (x>y?x:y)
-extern Pixmap       EpsfPictureLogo;
-
+#endif
+extern ThotPixmap       EpsfPictureLogo;
 
 /*----------------------------------------------------------------------
    Find EPS picture bounding box.      		             
@@ -62,7 +62,7 @@ static void GetPictureBoundaries (char *fn, int *xif, int *yif, int *wif,
       for (c = getc (fin); c != EOF; c = getc (fin))
 	{
 	  if (pt - buff < BUFSIZE - 2)
-	    *pt++ = (CHAR_T) c;
+	    *pt++ = (char) c;
 	  if (c == '\n')
 	    {
 	      *(--pt) = EOS;
@@ -84,18 +84,18 @@ static void GetPictureBoundaries (char *fn, int *xif, int *yif, int *wif,
 /*----------------------------------------------------------------------
    Read the bounding box of an eps file  no picture to produce here
   ----------------------------------------------------------------------*/
-Drawable EpsCreate (char *fn, PictInfo *imageDesc, int *xif, int *yif,
+ThotDrawable EpsCreate (char *fn, PictInfo *imageDesc, int *xif, int *yif,
 		    int *wif, int *hif, int bgColor, int *width,
 		    int *height, int zoom)
 {
-#ifdef _WINDOWS
+#ifdef _WINGUI
    return (NULL);
-#else  /* _WINDOWS */
+#else  /* _WINGUI */
    GetPictureBoundaries (fn, xif, yif, wif, hif);
    *width = *wif;
    *height = *hif;
-   return ((ThotBitmap) EpsfPictureLogo);
-#endif /* _WINDOWS */
+   return ((ThotDrawable) EpsfPictureLogo);
+#endif /* _WINGUI */
 }			
 
 /*----------------------------------------------------------------------
@@ -104,11 +104,11 @@ Drawable EpsCreate (char *fn, PictInfo *imageDesc, int *xif, int *yif,
 void EpsPrint (char *fn, PictureScaling pres, int xif, int yif, int wif,
 	       int hif, FILE *fd, int bgColor)
 {
-#ifndef _WINDOWS
+#ifndef _WINGUI
    float               Scx, Scy;
    FILE               *fin;
    int                 x, y;
-  int                  picX, picY, picW, picH;
+   int                 picX, picY, picW, picH;
    int                 c;
 
    /* Read the picture boundaries */
@@ -172,7 +172,7 @@ void EpsPrint (char *fn, PictureScaling pres, int xif, int yif, int wif,
    fprintf (fd, "\n");
    fprintf (fd, "%%%% end of file %s\n", fn);
    fprintf (fd, "ENDEPSFILE\n");
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
 }			
 
 /*----------------------------------------------------------------------
