@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001.
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -18,9 +18,10 @@
 #include "picture.h"
 #include "frame.h"
 #include "message.h"
-#ifdef _WINDOWS
-#include "wininclude.h"
-#endif /* _WINDOWS */
+
+#ifdef _WINGUI
+  #include "wininclude.h"
+#endif /* _WINGUI */
 
 #define THOT_EXPORT extern
 #include "edit_tv.h"
@@ -101,14 +102,139 @@ int Magic64[256] =    /* for 4 levels of red and blue */
 };
 
 static png_color        Std_color_cube[128];
-#ifdef _WINDOWS     
-extern ThotBool         pic2print;
-#endif /* _WINDOWS */
+
+#ifdef _WINGUI     
+  extern ThotBool         pic2print;
+  extern int              PngTransparentColor;
+#endif /* _WINGUI */
 
 char  *typecouleur[] = {"grayscale", "undefined type", "RGB",
 			"colormap", "grayscale+alpha",
 			"undefined type", "RGB+alpha"};
 
+#if defined(_MOTIF) || defined(_GTK) 
+extern ThotPixmap MakeMask (Display *dsp, unsigned char *pixels, int w, int h,
+		 unsigned int bg, int bperpix);
+#endif /* #if defined(_MOTIF) || defined(_GTK) */
+
+#ifdef _GL
+
+
+/*----------------------------------------------------------------------
+  ReadPng : reads from a file pointer a png file into a RGBA buffer
+  All png specifications is supported : http://www.w3.org/Graphics/PNG/
+  lib manual http://www.libpng.org/pub/png/libpng-manual.txt
+  ----------------------------------------------------------------------*/
+static unsigned char *ReadPng (FILE *pfFile, unsigned int *width, unsigned int *height, 
+			       int *ncolors, int *cpp, ThotColorStruct **colrs,
+			       int *bg, ThotBool *withAlpha, ThotBool *grayScale)
+{
+  png_structp png_ptr;
+  png_infop info_ptr;
+  png_byte pbSig[8];
+  png_uint_32 ulChannels;
+  png_uint_32 ulRowBytes;
+  png_byte **ppbRowPointers;
+  unsigned char *pixels;
+  unsigned int i, j, passes;
+  int iBitDepth, iColorType;
+  double dGamma;
+
+    /* Checks the eight byte PNG signature*/
+    fread (pbSig, 1, 8, pfFile);
+    if (!png_check_sig (pbSig, 8))
+        return NULL;	
+    /* create the two png(-info) structures*/
+    png_ptr = png_create_read_struct (PNG_LIBPNG_VER_STRING, 
+				      NULL, 
+				      (png_error_ptr) NULL, 
+				      (png_error_ptr) NULL);
+    if (!png_ptr)
+      return NULL;	
+    info_ptr = png_create_info_struct (png_ptr);
+    if (!info_ptr)
+      {
+        png_destroy_read_struct (&png_ptr, NULL, NULL);
+        return NULL;
+      }   
+    if (setjmp (png_ptr->jmpbuf))
+      {
+	/* Free all of the memory associated with the png_ptr and info_ptr */
+	png_destroy_read_struct (&png_ptr, &info_ptr, (png_infopp)NULL);
+	return NULL;
+      }
+    png_init_io (png_ptr, pfFile);
+    png_set_sig_bytes (png_ptr, 8);
+    png_read_info (png_ptr, info_ptr);
+    png_get_IHDR(png_ptr, info_ptr, (unsigned long *) width, 
+		 (unsigned long *) height, 
+		 &iBitDepth,
+		 &iColorType, NULL, NULL, NULL);	
+   /* if less than 8 bits /channels => 8 bits / channels*/
+    if (iBitDepth < 8)
+      png_set_packing (png_ptr);
+    /* 16 bits /channels => 8 bits / channels*/
+    if (iBitDepth == 16)
+      png_set_strip_16 (png_ptr);
+    /* Grayscale =>RGB or RGBA */
+    if (iColorType == PNG_COLOR_TYPE_GRAY || iColorType == PNG_COLOR_TYPE_GRAY_ALPHA)
+      png_set_gray_to_rgb (png_ptr);
+    /* Palette indexed colors to RGB */
+    if (iColorType == PNG_COLOR_TYPE_PALETTE)
+      png_set_palette_to_rgb (png_ptr);
+    /* 8 bits / channel is needed */
+    if (iColorType == PNG_COLOR_TYPE_GRAY && iBitDepth < 8) 
+      png_set_gray_1_2_4_to_8(png_ptr);
+    /* all transparency type : 1 color, indexed => alpha channel*/
+    if (png_get_valid (png_ptr, info_ptr,PNG_INFO_tRNS)) 
+      png_set_tRNS_to_alpha (png_ptr);
+    /* RGB => RGBA*/
+    if (iColorType != PNG_COLOR_TYPE_RGBA)
+      png_set_filler (png_ptr, 0xff, PNG_FILLER_AFTER);
+    /* Gives us the good number of passes to make : 
+       if not interlaced -> 1 passes else often 7 passes*/
+    passes = png_set_interlace_handling (png_ptr);
+    /* Gamma settings 
+       2.2 => A good guess for a  PC monitor in a bright office or a dim room 
+       2.0 => A good guess for a PC monitor in a dark room 
+       1.7 => A good guess for Mac systems    
+    */
+    if (png_get_gAMA (png_ptr, info_ptr, &dGamma))
+      png_set_gamma (png_ptr, (double) 2.2, dGamma);
+    /* Update the png in order to reach ou out pixels sprcification*/
+    png_read_update_info (png_ptr, info_ptr);
+    /* get again width, height and the new bit-depth and color-type*/
+    png_get_IHDR (png_ptr, info_ptr, (unsigned long *) width, 
+		  (unsigned long *) height, 
+		  &iBitDepth, 
+		  &iColorType, 
+		  NULL, NULL, NULL);
+    /* row_bytes is the width x number of channels => the length of a line */
+    ulRowBytes = png_get_rowbytes (png_ptr, info_ptr);
+    ulChannels = png_get_channels (png_ptr, info_ptr);
+    pixels = (png_byte *) TtaGetMemory (ulRowBytes * (*height) * sizeof(png_byte));
+    /* Row pointers give a pointer on each line */
+    ppbRowPointers = (png_bytepp) TtaGetMemory  ((*height) * sizeof(png_bytep));
+    /* Opengl Texture inversion */   
+    for (i = 0; i < (*height); i++)
+      ppbRowPointers[i] = pixels + ((*height) - (i+1)) * ulRowBytes;    
+    png_start_read_image (png_ptr); 
+    /* depending on interlacing, reading the data*/
+    for (i = 0; i < passes; i++)
+      for (j = 0; j < (*height); j++)
+	png_read_row (png_ptr, ppbRowPointers[j], NULL);
+    png_read_end(png_ptr, NULL);
+    TtaFreeMemory (ppbRowPointers);
+    ppbRowPointers = NULL; 
+    /* clean up after the read, and free any memory allocated */
+    png_read_destroy (png_ptr, info_ptr, (png_info*) NULL);
+    /* Free all of the memory associated with the png_ptr and info_ptr */
+    png_destroy_read_struct (&png_ptr, &info_ptr, (png_infopp)NULL);
+    return pixels;
+}
+
+#else /* _GL */
+
 /*----------------------------------------------------------------------
   ----------------------------------------------------------------------*/
 static void PError (png_struct *png_ptr, char *message)
@@ -130,22 +256,22 @@ static void PWarning (png_struct *png_ptr, char *message)
   ----------------------------------------------------------------------*/
 static unsigned char *ReadPng (FILE *infile, int *width, int *height,
 			       int *ncolors, int *cpp, ThotColorStruct **colrs,
-			       int *bg, ThotBool *withAlpha)
+			       int *bg, ThotBool *withAlpha, ThotBool *grayScale)
 {
   png_byte        *pp;
   png_byte         buf[8];
   png_struct      *png_ptr = NULL;
   png_info        *info_ptr = NULL;
-  png_byte        *png_pixels = NULL;
-  png_byte       **row_pointers = NULL;
-  ThotColorStruct *colors = NULL;
-  unsigned char   *pixels = NULL;
+  png_byte        *png_pixels;
+  png_byte       **row_pointers;
+  ThotColorStruct *colors;
+  unsigned char   *pixels;
   unsigned short  *spixels = NULL;
   double           gamma_correction;
   unsigned int     bytesPerExpandedLine;
   int              alpha;
   int              color_type;
-  int              ret, gr, isgrey;
+  int              ret, gr;
   int              i, j, passes;
   int              xpos, ypos, ind;
   int              xr, xg, xb;
@@ -153,18 +279,25 @@ static unsigned char *ReadPng (FILE *infile, int *width, int *height,
   int              bit_depth, interlace_type;
   int              cr, cg, cb, cgr;
   int              row, col;
+  ThotBool         isgrey;
 
   *withAlpha = FALSE;
+  *grayScale = FALSE;
+  *colrs = NULL;
+  *ncolors = 0;
+
+  /* Checks the eight byte PNG signature*/
   ret = fread (buf, 1, 8, infile);
   if (ret != 8)
     return NULL;
-  
   ret = png_check_sig (buf, 8);	
   if (!ret)
     return NULL;
   rewind (infile);
+
+  /* create the two png(-info) structures*/
   png_ptr = png_create_read_struct (PNG_LIBPNG_VER_STRING, (png_voidp)PError,
-				    (png_voidp)PError, (png_voidp)PWarning);
+				    (png_error_ptr)PError, (png_error_ptr)PWarning);
   if (png_ptr == NULL)
     return NULL;
 
@@ -175,7 +308,12 @@ static unsigned char *ReadPng (FILE *infile, int *width, int *height,
       png_destroy_read_struct(&png_ptr, (png_infopp)NULL, (png_infopp)NULL);
       return NULL;
     }
-
+  png_pixels = NULL;
+  row_pointers = NULL;
+  colors = NULL;
+  pixels = NULL;
+  isgrey = FALSE;
+  cr = cg = cb = 0;
   if (setjmp (png_ptr->jmpbuf))
     {
       /* Free all of the memory associated with the png_ptr and info_ptr */
@@ -241,7 +379,8 @@ static unsigned char *ReadPng (FILE *infile, int *width, int *height,
   for (i = 0; i < *height; i++)
     row_pointers[i] = png_pixels + (bytesPerExpandedLine * i);    
     
-  /*   png_read_image(png_ptr, row_pointers);*/
+
+
   color_type = info_ptr->color_type;
   alpha = color_type & PNG_COLOR_MASK_ALPHA;
   color_type &= ~PNG_COLOR_MASK_ALPHA;
@@ -251,22 +390,24 @@ static unsigned char *ReadPng (FILE *infile, int *width, int *height,
 	*ncolors = 256;
       else
 	*ncolors = info_ptr->num_palette;
-      colors = TtaGetMemory ((*ncolors) * sizeof (ThotColorStruct));
+      colors = (ThotColorStruct *)TtaGetMemory ((*ncolors) * sizeof (ThotColorStruct));
       for (i = 0; i < *ncolors; i++)
 	{
-#ifdef _WINDOWS
+#ifdef _WINGUI
 	  colors[i].red   = info_ptr->palette[i].red;
 	  colors[i].green = info_ptr->palette[i].green;
 	  colors[i].blue  = info_ptr->palette[i].blue;
-#else /* _WINDOWS */
+#endif /* _WINGUI */
+
+#if defined(_MOTIF) || defined(_GTK) || defined(_WX)
 	  colors[i].red   = info_ptr->palette[i].red << 8;
 	  colors[i].green = info_ptr->palette[i].green << 8;
 	  colors[i].blue  = info_ptr->palette[i].blue << 8;
 	  colors[i].pixel = i;
-#ifndef _GTK
+#ifdef _MOTIF
 	  colors[i].flags = DoRed|DoGreen|DoBlue;
-#endif /* _GTK */
-#endif /* _WINDOWS */
+#endif /* _MOTIF */
+#endif /* #if defined(_MOTIF) || defined(_GTK) || defined(_WX) */
 	}
     }
   else if (color_type == PNG_COLOR_TYPE_RGB)
@@ -274,54 +415,62 @@ static unsigned char *ReadPng (FILE *infile, int *width, int *height,
       if (TtWDepth > 8)
 	{
 	  /* Generate the image palette */
-	  colors = TtaGetMemory (512 * sizeof (ThotColorStruct));
+	  colors = (ThotColorStruct *)TtaGetMemory (512 * sizeof (ThotColorStruct));
 	  *ncolors = 0; 
 	}
       else
 	{
 	  /* Use the standard palette */
-	  colors = TtaGetMemory (128 * sizeof (ThotColorStruct));
+	  colors = (ThotColorStruct *)TtaGetMemory (128 * sizeof (ThotColorStruct));
 	  *ncolors = 128;
 	  for (i = 0; i < *ncolors ; i++)
 	    {
-#ifdef _WINDOWS
+#ifdef _WINGUI
 	      colors[i].red = Std_color_cube[i].red;
 	      colors[i].green = Std_color_cube[i].green;
 	      colors[i].blue = Std_color_cube[i].blue;
-#else /* _WINDOWS */
+#endif /* _WINGUI */
+        
+#if defined(_MOTIF) || defined(_GTK) || defined(_WX)    
 	      colors[i].red = Std_color_cube[i].red << 8;
 	      colors[i].green = Std_color_cube[i].green << 8;
 	      colors[i].blue = Std_color_cube[i].blue << 8;
-#ifndef _GTK
+#ifdef _MOTIF
 	      colors[i].flags = DoRed|DoGreen|DoBlue;
-#endif /* ! _GTK */
-#endif /* _WINDOWS */
+#endif /* _MOTIF */
+#endif /* #if defined(_MOTIF) || defined(_GTK) || defined(_WX) */
 	    }
 	}
     }
   else if (color_type == PNG_COLOR_TYPE_GRAY)
     {
+      isgrey = TRUE;
      if (TtWDepth > 8)
 	{
 	  /* Generate the image palette */
-	  colors = TtaGetMemory (512 * sizeof (ThotColorStruct));
+	  colors = (ThotColorStruct *)TtaGetMemory (512 * sizeof (ThotColorStruct));
+	  *grayScale = TRUE;
 	  *ncolors = 0;
 	}
       else
 	{
 	  /* Use the standard palette */
-	  colors = TtaGetMemory (16 * sizeof (ThotColorStruct));
+	  colors = (ThotColorStruct *)TtaGetMemory (16 * sizeof (ThotColorStruct));
+	  *grayScale = TRUE;
 	  *ncolors = 16; 
-	  for (i = 0; i < 15; i++)
+	  for (i = 0; i <= 15; i++)
 	    {
-#ifdef _WINDOWS
+#if defined(_WINGUI)
 	      colors[i].red = colors[i].green = colors[i].blue = i;
-#else /* _WINDOWS */
-	      colors[i].red = colors[i].green = colors[i].blue = i << 8;
-#ifndef _GTK
+#endif /* _WINGUI */
+        
+#if defined(_MOTIF) || defined(_GTK) || defined(_WX)
+	      colors[i].red = colors[i].green = colors[i].blue = i << 4; 
+              /* this is 4 and not 8 because the palette should be between 0 and 255 */
+#ifdef _MOTIF
 	      colors[i].flags = DoRed|DoGreen|DoBlue;
-#endif /* _GTK */
-#endif /* _WINDOWS */
+#endif /* _MOTIF */
+#endif /* #if defined(_MOTIF) || defined(_GTK) || defined(_WX) */
 	    }
 	}
     }
@@ -335,7 +484,7 @@ static unsigned char *ReadPng (FILE *infile, int *width, int *height,
       png_read_row (png_ptr, NULL, row_pointers[j]);
 
   /* Generate the table that'll store an index in the color table for each pixel */
-  pixels = (char *) TtaGetMemory ((*width) * (*height) * sizeof (char));
+  pixels = (unsigned char *) TtaGetMemory ((*width) * (*height) * sizeof (char));
   if (pixels == NULL)
     png_error (png_ptr, "not enough memory ");
   pp = png_pixels;
@@ -364,6 +513,7 @@ static unsigned char *ReadPng (FILE *infile, int *width, int *height,
       break;
 
     case PNG_COLOR_TYPE_RGB:
+    case  PNG_COLOR_TYPE_GRAY:
       if (TtWDepth > 8)
 	{
 	  /* True color -> Keep the image descriptor as it is */
@@ -373,85 +523,82 @@ static unsigned char *ReadPng (FILE *infile, int *width, int *height,
 	  TtaFreeMemory (colors);
 	  colors = NULL;
 	  *withAlpha = (alpha != 0);
-	  break;
 	}
-    case  PNG_COLOR_TYPE_GRAY:
-      ind = 0; /* pixel index */
-      for (ypos = 0; ypos < *height; ypos ++)
+      else
 	{
-	  col = ypos & 0x0f;
-	  pp = row_pointers[ypos];
-	  for (xpos = 0; xpos < *width; xpos ++)
+	  ind = 0; /* pixel index */
+	  for (ypos = 0; ypos < *height; ypos ++)
 	    {
-	      cgr = 0;
-	      a = 0;
-	      row = xpos & 0x0f;
-	      if (color_type == PNG_COLOR_TYPE_GRAY)
+	      col = ypos & 0x0f;
+	      pp = row_pointers[ypos];
+	      for (xpos = 0; xpos < *width; xpos ++)
 		{
-		  cr = cg = cb = cgr = (*pp++);
-		  isgrey = 1;
-		}
-	      else if (color_type == PNG_COLOR_TYPE_RGB)
-		{		
-		  cr = (*pp++);
-		  cg = (*pp++);
-		  cb = (*pp++);
-		  isgrey = 0;
-		}
+		  cgr = 0;
+		  a = 0;
+		  row = xpos & 0x0f;
+		  if (isgrey)
+		    cr = cg = cb = cgr = (*pp++);
+		  else if (color_type == PNG_COLOR_TYPE_RGB)
+		    {		
+		      cr = (*pp++);
+		      cg = (*pp++);
+		      cb = (*pp++);
+		    }
 	      
-	      /* the alpha channel is not yet handled :) */
-	      if (alpha)
-		{
-		  a = (*pp++);
-		  cr  = (int) ((a/255.0) * cr  + ((255.0-a)/255.0) * 211.0);
-		  cg  = (int) ((a/255.0) * cg  + ((255.0-a)/255.0) * 211.0);
-		  cb  = (int) ((a/255.0) * cb  + ((255.0-a)/255.0) * 211.0);
-		  cgr = (int) ((a/255.0) * cgr + ((255.0-a)/255.0) * 211.0);
-		}
+		  /* the alpha channel is not yet handled :) */
+		  if (alpha)
+		    {
+		      a = (*pp++);
+		      cr  = (int) ((a/255.0) * cr  + ((255.0-a)/255.0) * 211.0);
+		      cg  = (int) ((a/255.0) * cg  + ((255.0-a)/255.0) * 211.0);
+		      cb  = (int) ((a/255.0) * cb  + ((255.0-a)/255.0) * 211.0);
+		      cgr = (int) ((a/255.0) * cgr + ((255.0-a)/255.0) * 211.0);
+		    }
    
-	      if (isgrey)
-		{
-		  /* Use the palette of 16 colors */
-		  gr = cgr & 0xF0;
-		  if (cgr - gr > Magic16[(row << 4) + col])
-		    gr += 16;
-		  gr = min(gr, 0xF0);
-		  pixels[ind++] = gr >> 4;
-		}
-	      else
-		{
-		  /* Use the palette of 128 colors */
-		  r = cr & 0xC0;
-		  g = cg & 0xE0;
-		  b = cb & 0xC0;
-		  v = (row << 4) + col;
-		  if (cr - r > Magic64[v])
-		    r += 64;
-		  if (cg - g > Magic32[v])
-		    g += 32;
-		  if (cb - b > Magic64[v])
-		    b += 64;
-		  r = min(r, 255) & 0xC0;
-		  g = min(g, 255) & 0xE0;
-		  b = min(b, 255) & 0xC0;
-		  pixels[ind++] = (unsigned char) ((r >> 6) | (g >> 3) | (b >> 1));
+		  if (isgrey)
+		    {
+		      /* Use the palette of 16 colors */
+		      gr = cgr & 0xF0;
+		      if (cgr - gr > Magic16[(row << 4) + col])
+			gr += 16;
+		      gr = min(gr, 0xF0);
+		      pixels[ind++] = gr >> 4; /* this is the color index in the palette (16 colors) */
+		    }
+		  else
+		    {
+		      /* Use the palette of 128 colors */
+		      r = cr & 0xC0;
+		      g = cg & 0xE0;
+		      b = cb & 0xC0;
+		      v = (row << 4) + col;
+		      if (cr - r > Magic64[v])
+			r += 64;
+		      if (cg - g > Magic32[v])
+			g += 32;
+		      if (cb - b > Magic64[v])
+			b += 64;
+		      r = min(r, 255) & 0xC0;
+		      g = min(g, 255) & 0xE0;
+		      b = min(b, 255) & 0xC0;
+		      pixels[ind++] = (unsigned char) ((r >> 6) | (g >> 3) | (b >> 1));
+		    }
 		}
 	    }
-	}
-      if (info_ptr->valid & PNG_INFO_tRNS)
-	{
-	  if (isgrey)
-	    *bg = ((info_ptr->trans_values.gray) & 0xff) >> 4;
-	  else
+	  if (info_ptr->valid & PNG_INFO_tRNS)
 	    {
-	      xr = info_ptr->trans_values.red;
-	      xg = info_ptr->trans_values.green;
-	      xb = info_ptr->trans_values.blue;
-	      xr = xr & 0xff; xg = xg & 0xff; xb = xb & 0xff;
-	      xr = min(xr, 255) & 0xC0;
-	      xg = min(xg, 255) & 0xE0;
-	      xb = min(xb, 255) & 0xC0;
-	      *bg = (unsigned char) ((xr >> 6) | (xg >> 3) | (xb >> 1));
+	      if (isgrey)
+		*bg = ((info_ptr->trans_values.gray) & 0xff) >> 4;
+	      else
+		{
+		  xr = info_ptr->trans_values.red;
+		  xg = info_ptr->trans_values.green;
+		  xb = info_ptr->trans_values.blue;
+		  xr = xr & 0xff; xg = xg & 0xff; xb = xb & 0xff;
+		  xr = min(xr, 255) & 0xC0;
+		  xg = min(xg, 255) & 0xE0;
+		  xb = min(xb, 255) & 0xC0;
+		  *bg = (unsigned char) ((xr >> 6) | (xg >> 3) | (xb >> 1));
+		}
 	    }
 	}
       break;
@@ -474,7 +621,7 @@ static unsigned char *ReadPng (FILE *infile, int *width, int *height,
   else
     return (pixels);
 }
-
+#endif /* _GL */
 /*----------------------------------------------------------------------
   InitPngColors
   ----------------------------------------------------------------------*/
@@ -500,19 +647,28 @@ void InitPngColors ()
 static unsigned char *ReadPngToData (char *datafile, int *w, int *h,
 				     int *ncolors, int *cpp,
 				     ThotColorStruct **colrs, int *bg,
-				     ThotBool *withAlpha)
+				     ThotBool *withAlpha, ThotBool *grayScale)
 {
   unsigned char *bit_data;
   FILE           *fp;
       
-#ifndef _WINDOWS  
+#if defined(_MOTIF) || defined(_GTK) || defined(_WX)
   fp = fopen (datafile, "r");
-#else  /* _WINDOWS */
+#endif /* #if defined(_MOTIF) || defined(_GTK) || defined(_WX) */
+  
+#ifdef _WINGUI
   fp = fopen (datafile, "rb");
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
+
   if (fp != NULL)
     {
-      bit_data = ReadPng (fp, w, h, ncolors, cpp, colrs, bg, withAlpha);
+#ifdef _GL
+      bit_data = ReadPng (fp, (unsigned int *)w, (unsigned int *)h, ncolors, cpp, colrs, bg, withAlpha,
+			  grayScale);
+#else /* #ifdef _GL */
+      bit_data = ReadPng (fp, w, h, ncolors, cpp, colrs, bg, withAlpha,
+			  grayScale);      
+#endif /* #ifdef _GL */      
       if (bit_data != NULL)
 	{
 	  if (fp != stdin) 
@@ -528,22 +684,25 @@ static unsigned char *ReadPngToData (char *datafile, int *w, int *h,
 
 /*----------------------------------------------------------------------
   ----------------------------------------------------------------------*/
-Drawable PngCreate (char *fn, PictInfo *imageDesc, int *xif, int *yif,
+ThotDrawable PngCreate (char *fn, PictInfo *imageDesc, int *xif, int *yif,
 		    int *wif, int *hif, int bgColor, int *width,
 		    int *height, int zoom)
 {
-  Pixmap           pixmap = (Pixmap) 0;
-  ThotColorStruct *colrs;
-#ifdef _WINDOWS
+  ThotPixmap           pixmap = (ThotPixmap) NULL;
+  ThotColorStruct *colrs = NULL;
+#if defined (_WINGUI) && !defined (_GL)
   unsigned short   red, green, blue;
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
   unsigned char   *buffer = NULL; 
+#ifndef _GL
   unsigned char   *buffer2 = NULL;
-  int              ncolors, cpp, bg = -1;
+#endif /*_GL*/
+  int              ncolors = 1, cpp, bg = -1;
   int              w, h, bperpix;
-  ThotBool         withAlpha;
+  ThotBool         withAlpha, grayScale;
 
-  buffer = ReadPngToData (fn, &w, &h, &ncolors, &cpp, &colrs, &bg, &withAlpha);
+  buffer = ReadPngToData (fn, &w, &h, &ncolors, &cpp, &colrs, &bg,
+			  &withAlpha, &grayScale);
   if (ncolors == 0)
     {
       /* one byte per component RGB */
@@ -558,7 +717,7 @@ Drawable PngCreate (char *fn, PictInfo *imageDesc, int *xif, int *yif,
   *width = w;
   *height = h;
   if (buffer == NULL) 
-     return ((Drawable) NULL);
+     return ((ThotDrawable) NULL);
 
   if (zoom != 0 && *xif == 0 && *yif == 0)
     {
@@ -575,6 +734,7 @@ Drawable PngCreate (char *fn, PictInfo *imageDesc, int *xif, int *yif,
     }
 
 #ifndef _WIN_PRINT
+#ifndef _GL
   if ((*xif != 0 && *yif != 0) && (w != *xif || h != *yif))
     {
       /* xif and yif contain width and height of the box */
@@ -585,19 +745,21 @@ Drawable PngCreate (char *fn, PictInfo *imageDesc, int *xif, int *yif,
       w = *xif;
       h = *yif;
     }
+#endif /*_GL*/
 #endif /* _WINPRINT */
     
   if (buffer == NULL)
     {
-#ifdef _WINDOWS
+#ifdef _WINGUI
       WinErrorBox (NULL, "PngCreate: (1)");
-#endif /* _WINDOWS */
-      return ((Drawable) NULL);
+#endif /* _WINGUI */
+      return ((ThotDrawable) NULL);
     }
 
+#ifndef _GL
   if (bg >= 0 && colrs)
     {
-#ifdef _WINDOWS
+#ifdef _WINGUI
       if (Printing)
 	{
 	  TtaGiveThotRGB (bgColor, &red, &green, &blue);
@@ -609,34 +771,37 @@ Drawable PngCreate (char *fn, PictInfo *imageDesc, int *xif, int *yif,
 	}
       else
 	/* register the transparent color index */
-	bg = TtaGetThotColor (colrs[bg].red, colrs[bg].green,
-			      colrs[bg].blue);
-      imageDesc->PicMask = bg;
-#else  /* _WINDOWS */
+	bg = TtaGetThotColor (colrs[bg].red, colrs[bg].green, colrs[bg].blue);
+      imageDesc->PicBgMask = bg;
+#endif /* _WINGUI */
+      
+#if defined(_MOTIF) || defined(_GTK)      
       /* register the transparent mask */
       imageDesc->PicMask = MakeMask (TtDisplay, buffer, w, h, bg, bperpix);
-#endif /* _WINDOWS */
+#endif /* #if defined(_MOTIF) || defined(_GTK) */
+      
     }
-
-  pixmap = DataToPixmap (buffer, w, h, ncolors, colrs, withAlpha);
+  pixmap = DataToPixmap (buffer, w, h, ncolors, colrs, withAlpha, grayScale);
   TtaFreeMemory (buffer);
+#ifdef _WINGUI
+  if (withAlpha && bg == -1 && PngTransparentColor != -1)
+    imageDesc->PicBgMask = PngTransparentColor;
+#endif /* _WINGUI */
+#else /* _GL */
+  /* GL buffer are display independant, 
+  and already in the good format RGB, or RGBA*/
+  pixmap = (ThotPixmap) buffer;
+#endif /*_GL*/
   /* free the table of colors */
   TtaFreeMemory (colrs);
-  if (pixmap == None)
-    {
-#ifdef _WINDOWS
-      WinErrorBox (NULL, "PngCreate: (2)");
-#endif /* _WINDOWS */
-      return ((Drawable) NULL); 
-    }
-  else
+  if (pixmap != None)
     { 
       *wif = w;
       *hif = h;      
       *xif = 0;
       *yif = 0;
-      return (Drawable) pixmap;
     }
+  return (ThotDrawable) pixmap;
 }
 
 
@@ -645,23 +810,23 @@ Drawable PngCreate (char *fn, PictInfo *imageDesc, int *xif, int *yif,
 void PngPrint (char *fn, PictureScaling pres, int xif, int yif, int wif,
 	       int hif, FILE *fd, int bgColor)
 {
-#ifndef _WINDOWS
+#if defined(_MOTIF) || defined(_GTK)
   ThotColorStruct *colrs;
   unsigned char   *data;
   int              picW, picH;
   int              ncolors, cpp, transparent;
-  ThotBool         withAlpha;
+  ThotBool         withAlpha, grayScale;
 
   transparent = -1;
   data = ReadPngToData (fn, &picW, &picH, &ncolors, &cpp, &colrs, &transparent,
-			&withAlpha);
+			&withAlpha, &grayScale);
   if (data)
     DataToPrint (data, pres, xif, yif, wif, hif, picW, picH, fd, ncolors,
-		 transparent, bgColor, colrs, withAlpha);
+		 transparent, bgColor, colrs, withAlpha, grayScale);
   TtaFreeMemory (data);
   /* free the table of colors */
   TtaFreeMemory (colrs);
-#endif /* !_WINDOWS */
+#endif /* #if defined(_MOTIF) || defined(_GTK) */
 }
 
 /*----------------------------------------------------------------------
@@ -679,7 +844,88 @@ ThotBool IsPngFormat(char *fn)
    fclose(fp);
    if (ret != 8)
       return FALSE;
-   ret = png_check_sig(buf, 8);
+   ret = png_check_sig((png_byte*)buf, 8);
    if (ret) return (TRUE);
    return(FALSE);
 }
+
+/*----------------------------------------------------------------------
+  SavePng : Wth Alpha, compression in 32 bit
+----------------------------------------------------------------------*/
+ThotBool SavePng (const char *filename, 
+		 unsigned char *m_pData,
+		 unsigned int m_width,
+		 unsigned int m_height)
+{
+  png_structp                png = 0;
+  png_infop                  pngInfo = 0;
+  FILE                       *pngFile;
+  unsigned char              **rowPtrs;
+  register int      rowcount, linesize, imagesize;
+
+  if (!m_pData)
+    return FALSE; 
+  png = png_create_write_struct (PNG_LIBPNG_VER_STRING, 0, 0, 0);
+  if (!png)
+    return FALSE;
+  pngInfo = png_create_info_struct (png);
+  if (!pngInfo)
+    {
+      png_destroy_write_struct(&png, (png_infopp) NULL);
+      return FALSE;
+    }
+  pngFile = fopen (filename, "wb");
+  if (!(pngFile))
+    {
+      png_destroy_write_struct(&png, (png_infopp) NULL);
+      return FALSE;
+    }
+  if (setjmp(png->jmpbuf)) 
+    {
+        png_destroy_write_struct(&png, &pngInfo);
+        fclose(pngFile);
+        return FALSE;
+    }
+  png_init_io (png, pngFile);
+  /* set the zlib compression level */
+  png_set_compression_level(png, Z_BEST_COMPRESSION);
+  /* set other zlib parameters */
+  png_set_compression_mem_level(png, 8);
+  png_set_compression_strategy(png, Z_DEFAULT_STRATEGY);
+  png_set_compression_window_bits(png, 15);
+  png_set_compression_method(png, 8);
+
+  png_set_IHDR (png, pngInfo, 
+		(int) m_width, (int) m_height, 8, 
+		PNG_COLOR_TYPE_RGB_ALPHA, 
+		PNG_INTERLACE_NONE, 
+		PNG_COMPRESSION_TYPE_DEFAULT,
+		PNG_FILTER_TYPE_DEFAULT);
+  /* png_set_invert_alpha(png_ptr);*/
+  png_write_info (png, pngInfo);  
+  rowPtrs = (unsigned char**)malloc (sizeof(unsigned char *) * m_height);
+  if (!rowPtrs)
+    {
+      fclose (pngFile);
+      png_destroy_read_struct (&png, &pngInfo, 0);
+      return FALSE;
+    }
+  rowcount = ((int) m_height) - 1;
+  linesize = ((int) m_width) * 4;
+  imagesize = 0;
+  /* invert image order*/
+  while (rowcount >= 0)
+    {
+      rowPtrs[rowcount] = (unsigned char *) m_pData + imagesize;
+      imagesize += linesize;
+      rowcount--;
+    }  
+  png_write_image (png, rowPtrs);
+  png_write_end (png, pngInfo);
+  png_write_flush (png);  
+  free (rowPtrs);  
+  png_destroy_read_struct (&png, &pngInfo, 0);  
+  fclose (pngFile);
+  return TRUE;
+}
+
