@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001.
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -9,7 +9,7 @@
  * Author: I. Vatton, N. Layaida (INRIA)
  *         R. Guetari (W3C/INRIA) Windows version
  */
-
+#include "thot_gui.h"
 #include "thot_sys.h"
 #include "constmedia.h"
 #include "typemedia.h"
@@ -28,20 +28,20 @@
    XbmCreate reads and produces the bitmap read from the file      
    fn. updates the wif, hif, xif , yif                     
   ----------------------------------------------------------------------*/
-Drawable XbmCreate (char *fn, PictInfo *imageDesc, int *xif, int *yif,
+ThotDrawable XbmCreate (char *fn, PictInfo *imageDesc, int *xif, int *yif,
 		    int *wif, int *hif, unsigned long bgPixel, int *width,
 		    int *height, int zoom)
 {
-#ifndef _WINDOWS
+#ifdef _MOTIF
   Pixmap              pixmap;
   Pixmap              bitmap;
   int                 status;
   int                 w, h;
   int                 xHot, yHot;
 
-  status = XReadBitmapFile (TtDisplay, TtRootWindow, fn, &w, &h, &bitmap, &xHot, &yHot);
+  status = XReadBitmapFile (TtDisplay, TtRootWindow, fn, (unsigned int *)&w, (unsigned int *)&h, &bitmap, &xHot, &yHot);
   if (status != BitmapSuccess)
-    return ((Drawable) None);
+    return ((ThotDrawable) None);
   else
     {
       *xif = 0;
@@ -50,16 +50,29 @@ Drawable XbmCreate (char *fn, PictInfo *imageDesc, int *xif, int *yif,
       *hif = h;
       *width = w;
       *height = h;
-#ifndef _GTK
       pixmap = XCreatePixmap (TtDisplay, TtRootWindow, w, h, TtWDepth);
       XCopyPlane (TtDisplay, bitmap, pixmap, GCpicture, 0, 0, w, h, 0, 0, 1);
       XFreePixmap (TtDisplay, bitmap);
-#endif /* _GTK */
       return (pixmap);
     }
-#else /* _WINDOWS */
+#endif /* _MOTIF */
+  
+#ifdef _GTK
+  return (ThotDrawable)NULL;
+#endif /* _GTK */
+  
+#ifdef _WINGUI
   return NULL;
-#endif /* !_WINDOWS */
+#endif /* _WINGUI */
+
+#ifdef _NOGUI
+  return NULL;
+#endif /* #ifdef _NOGUI */ 
+
+#ifdef _WX
+  return NULL;
+#endif /* #ifdef _WX */ 
+
 }
 
 
@@ -69,7 +82,7 @@ Drawable XbmCreate (char *fn, PictInfo *imageDesc, int *xif, int *yif,
 void XbmPrint (char *fn, PictureScaling pres, int xif, int yif, int wif,
 	       int hif, FILE *fd, unsigned int bgPixel)
 {
-#ifndef _WINDOWS
+#ifdef _MOTIF
   XImage             *pict;
   Pixmap              pix;
   int                 delta;
@@ -80,7 +93,7 @@ void XbmPrint (char *fn, PictureScaling pres, int xif, int yif, int wif,
   register char      *pt, *pt1;
   int                 wim, him;
 
-  i = XReadBitmapFile (TtDisplay, TtRootWindow, fn, &picW, &picH, &pix,
+  i = XReadBitmapFile (TtDisplay, TtRootWindow, fn, (unsigned int *)&picW, (unsigned int *)&picH, &pix,
 		       &xtmp, &ytmp);
   if (i != BitmapSuccess)
     return;
@@ -159,7 +172,7 @@ void XbmPrint (char *fn, PictureScaling pres, int xif, int yif, int wif,
       XDestroyImage (pict);
       XFreePixmap (TtDisplay, pix);
     }
-#endif /* !_WINDOWS */
+#endif /* _MOTIF */
 }
 
 /*----------------------------------------------------------------------
@@ -167,17 +180,33 @@ void XbmPrint (char *fn, PictureScaling pres, int xif, int yif, int wif,
   ----------------------------------------------------------------------*/
 ThotBool IsXbmFormat (char *fn)
 {
-#ifdef _WINDOWS
+#ifdef _WINGUI
    return (FALSE);
-#else  /* _WINDOWS */
+#endif /* _WINGUI */
+
+#ifdef _MOTIF   
    Pixmap              bitmap = None;
    int                 status;
    int                 w, h;
    int                 xHot, yHot;
 
-   status = XReadBitmapFile (TtDisplay, TtRootWindow, fn, &w, &h, &bitmap, &xHot, &yHot);
+   status = XReadBitmapFile (TtDisplay, TtRootWindow, fn, (unsigned int *)&w, (unsigned int *)&h, &bitmap, &xHot, &yHot);
    if (bitmap != None)
       XFreePixmap (TtDisplay, bitmap);
    return (status == BitmapSuccess);
-#endif /* !_WINDOWS */
+#endif /* _MOTIF */
+   
+#ifdef _GTK
+   return FALSE;
+#endif /* _GTK */
+
+#ifdef _NOGUI
+  return FALSE;
+#endif /* #ifdef _NOGUI*/   
+
+#ifdef _WX
+  return FALSE;
+#endif /* #ifdef _WX */ 
+
 }
+
