@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001.
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -38,19 +38,11 @@
    XpmCreate reads and produces the bitmap read from the file      
    fn. updates the wif, hif, xif , yif                     
   ----------------------------------------------------------------------*/
-Drawable XpmCreate (char *fn, PictInfo *imageDesc, int *xif, int *yif,
+ThotDrawable XpmCreate (char *fn, PictInfo *imageDesc, int *xif, int *yif,
 		    int *wif, int *hif, int bgColor, int *width,
 		    int *height, int zoom)
 {
-#ifdef _WINDOWS
-  *width = 0;
-  *height = 0;
-  *wif = 0;
-  *hif = 0;
-  *xif = 0;
-  *yif = 0;
-  return (NULL);
-#else /* !_WINDOWS */
+#ifdef _MOTIF
   int                 status;
   Pixmap              pixmap;
   XpmAttributes       att;
@@ -73,7 +65,7 @@ Drawable XpmCreate (char *fn, PictInfo *imageDesc, int *xif, int *yif,
   *height = att.height;
   
   if (status != XpmSuccess)
-    return ((Drawable) None);
+    return ((ThotDrawable) None);
   else
     {
       *wif = att.width;
@@ -84,9 +76,24 @@ Drawable XpmCreate (char *fn, PictInfo *imageDesc, int *xif, int *yif,
       /* frees the library's internal structures */
       XpmFreeAttributes (&att);
       att.valuemask = valuemask;/* reinitialises the value mask */
-      return (Drawable) pixmap;
+      return (ThotDrawable) pixmap;
     }
-#endif  /* _WINDOWS */
+#endif /* _MOTIF */
+
+
+#if defined(_GTK) || defined(_WINGUI) || defined(_WX)
+  *width = 0;
+  *height = 0;
+  *wif = 0;
+  *hif = 0;
+  *xif = 0;
+  *yif = 0;
+  return (ThotDrawable) (NULL);
+#endif /* defined(_GTK) OR defined(_WINGUI) || defined(_WX) */
+
+#ifdef _NOGUI
+  return NULL;
+#endif /* #ifdef _NOGUI */  
 }
 
 
@@ -96,7 +103,7 @@ Drawable XpmCreate (char *fn, PictInfo *imageDesc, int *xif, int *yif,
 void XpmPrint (char *fn, PictureScaling pres, int xif, int yif, int wif,
 	       int hif, FILE *fd, int bgColor)
 {
-#ifndef _WINDOWS 
+#ifdef _MOTIF
    register int        i;
    XpmAttributes       att;
    XpmInfo             info;
@@ -145,11 +152,11 @@ void XpmPrint (char *fn, PictureScaling pres, int xif, int yif, int wif,
      }
    DataToPrint ((unsigned char *)image.data, pres, xif, yif, wif, hif, picW,
 		picH, fd, image.ncolors, -1, bgColor,
-		(ThotColorStruct *)image.colorTable, FALSE);
+		(ThotColorStruct *)image.colorTable, FALSE, FALSE);
    XpmFreeXpmInfo (&info);
    XpmFreeXpmImage (&image);
    att.valuemask = valuemask;
-#endif /* _WINDOWS */
+#endif /* _MOTIF */
 }			
 
 /*----------------------------------------------------------------------
