@@ -55,12 +55,11 @@ char *xpmColorKeys[] = {
 };
 
 int
-xpmParseValues(data, width, height, ncolors, cpp,
-	    x_hotspot, y_hotspot, hotspot, extensions)
-    xpmData *data;
-    unsigned int *width, *height, *ncolors, *cpp;
-    unsigned int *x_hotspot, *y_hotspot, *hotspot;
-    unsigned int *extensions;
+xpmParseValues(
+    xpmData *data,
+    unsigned int *width, unsigned int *height, unsigned int *ncolors, unsigned int *cpp,
+    unsigned int *x_hotspot, unsigned int *y_hotspot, unsigned int *hotspot,
+    unsigned int *extensions)
 {
     unsigned int l;
     char buf[BUFSIZ];
@@ -165,12 +164,12 @@ xpmParseValues(data, width, height, ncolors, cpp,
 }
 
 int
-xpmParseColors(data, ncolors, cpp, colorTablePtr, hashtable)
-    xpmData *data;
-    unsigned int ncolors;
-    unsigned int cpp;
-    XpmColor **colorTablePtr;
-    xpmHashTable *hashtable;
+xpmParseColors(
+    xpmData *data,
+    unsigned int ncolors,
+    unsigned int cpp,
+    XpmColor **colorTablePtr,
+    xpmHashTable *hashtable)
 {
     unsigned int key, l, a, b;
     unsigned int curkey;		/* current color key */
@@ -325,15 +324,15 @@ xpmParseColors(data, ncolors, cpp, colorTablePtr, hashtable)
 }
 
 static int
-ParsePixels(data, width, height, ncolors, cpp, colorTable, hashtable, pixels)
-    xpmData *data;
-    unsigned int width;
-    unsigned int height;
-    unsigned int ncolors;
-    unsigned int cpp;
-    XpmColor *colorTable;
-    xpmHashTable *hashtable;
-    unsigned int **pixels;
+ParsePixels(
+    xpmData *data,
+    unsigned int width,
+    unsigned int height,
+    unsigned int ncolors,
+    unsigned int cpp,
+    XpmColor *colorTable,
+    xpmHashTable *hashtable,
+    unsigned int **pixels)
 {
     unsigned int *iptr, *iptr2;
     unsigned int a, x, y;
@@ -481,10 +480,10 @@ if (cidx[f]) XpmFree(cidx[f]);}
 }
 
 int
-xpmParseExtensions(data, extensions, nextensions)
-    xpmData *data;
-    XpmExtension **extensions;
-    unsigned int *nextensions;
+xpmParseExtensions(
+    xpmData *data,
+    XpmExtension **extensions,
+    unsigned int *nextensions)
 {
     XpmExtension *exts = NULL, *ext;
     unsigned int num = 0;
@@ -602,10 +601,10 @@ xpmParseExtensions(data, extensions, nextensions)
  * in an an XpmImage structure which is returned.
  */
 int
-xpmParseData(data, image, info)
-    xpmData *data;
-    XpmImage *image;
-    XpmInfo *info;
+xpmParseData(
+    xpmData *data,
+    XpmImage *image,
+    XpmInfo *info)
 {
     /* variables to return */
     unsigned int width, height, ncolors, cpp;
@@ -694,7 +693,9 @@ xpmParseData(data, image, info)
      * parse extensions
      */
     if (info && (info->valuemask & XpmReturnExtensions))
-	if (extensions) {
+      {
+	if (extensions)
+	  {
 	    ErrorStatus = xpmParseExtensions(data, &info->extensions,
 					     &info->nextensions);
 	    if (ErrorStatus != XpmSuccess)
@@ -703,6 +704,7 @@ xpmParseData(data, image, info)
 	    info->extensions = NULL;
 	    info->nextensions = 0;
 	}
+      }
 
     /*
      * store found informations in the XpmImage structure
