@@ -19,7 +19,7 @@
 /* The list RegisteredAppEvents within the module app.c have to be conform to
    this type enum APPevent */
 /* NUMBER_OF_APP_EVENTS gives the number of entries in the list */
-#define NUMBER_OF_APP_EVENTS 48
+#define NUMBER_OF_APP_EVENTS 49
 typedef enum
   {
      TteAttrMenu,
@@ -38,6 +38,7 @@ typedef enum
      TteElemSelect,
      TteElemExtendSelect,
      TteElemClick,
+     TteElemLClick,
      TteElemMClick,
      TteElemRClick,
      TteElemActivate,
@@ -105,6 +106,7 @@ typedef struct
      Element             element;
      Attribute           attribute;
      AttributeType       attributeType;
+     int		 info; /* 1 if event sent by undo/redo */
   }
 NotifyAttribute;
 
@@ -115,7 +117,7 @@ typedef struct
      Element             element;
      ElementType         elementType;
      int                 position;
-     int		 info;
+     int		 info; /* 1 if event sent by undo/redo */
   }
 NotifyElement;
 
@@ -156,6 +158,7 @@ typedef struct
      Element             element;
      PRule               pRule;
      int                 pRuleType;
+     int                 value;
   }
 NotifyPresentation;
 
