@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT MIT and INRIA, 1996-2001.
+ *  (c) COPYRIGHT MIT and INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -15,8 +15,22 @@
 #include "appaction.h"
 
 typedef void        (*Proc) ();
+typedef void        (*Proc1) (void * );
+typedef void        (*Proc2) (void *,void *);
+typedef void        (*Proc3) (void *,void *,void *);
+typedef void        (*Proc4) (void *,void *,void *,void *);
+typedef void        (*Proc5) (void *,void *,void *,void *,void *);
+typedef void        (*Proc6) (void *,void *,void *,void *,void *,void *);
+typedef void        (*Proc7) (void *,void *,void *,void *,void *,void *,void *);
+typedef void        (*Proc8) (void *,void *,void *,void *,void *,void *,void *,void *);
+typedef void        (*Proc9) (void *,void *,void *,void *,void *,void *,void *,void *,void *);
+typedef void        (*Proc10) (void *,void *,void *,void *,void *,void *,void *,void *,void *,void *);
 typedef int         (*UserProc) ();
 typedef             ThotBool (*Func) ();
+/* for c++ compatibility */
+typedef             ThotBool (*Func1) (void *); 
+typedef             ThotBool (*Func2) (void *, void *);
+typedef             ThotBool (*Func3) (void *, void *, void *);
 
 /* Type definition for windows */
 typedef enum
@@ -51,7 +65,7 @@ typedef struct _EventsSet *PtrEventsSet;
 typedef struct _EventsSet
 {
   int                 EvSStructId;/* Identifier of SSchema */
-  CHAR_T*             EvSName;	/* Name of the events set */
+  char               *EvSName;	/* Name of the events set */
   PtrActionEvent      EvSList[NUMBER_OF_APP_EVENTS];
   PtrEventsSet        EvSNext;	/* Next EventsSet in the list */
 } EventsSet;
@@ -74,19 +88,20 @@ extern void         TableHLoadResources ();
 #ifndef __CEXTRACT__
 extern void         TteConnectAction (int id, Proc procedure);
 extern void         TtaSetBackup (Proc procedure);
+extern void         TtaSetAutoSave (Proc procedure);
 extern void         TtaSetDocStatusUpdate (Proc procedure);
-extern void         TteZeroMenu (WindowType windowtype, CHAR_T* schemaName);
-extern void         TteOpenMainWindow (CHAR_T* name, Pixmap logo, Pixmap icon);
-extern void         TteInitMenus (CHAR_T* name, int number);
+extern void         TteZeroMenu (WindowType windowtype, char *schemaName);
+extern void         TteOpenMainWindow (char *name, ThotIcon logo, ThotPixmap icon);
+extern void         TteInitMenus (char *name, int number);
 extern void         TteAddMenuAction (char* actionName, Proc procedure, ThotBool state);
-extern void         TteAddMenu (WindowType windowtype, CHAR_T* schemaName, int view, int menuID, int itemsNumber, char* menuName);
-extern void         TteAddSubMenu (WindowType windowtype, CHAR_T* schemaName, int menuID, int itemID, int itemsNumber);
-extern void         TteAddMenuItem (WindowType windowtype, CHAR_T* schemaName, int menuID, int subMenuID, int itemID, char* actionName, char itemType);
+extern void         TteAddMenu (WindowType windowtype, char *schemaName, int view, int menuID, int itemsNumber, char* menuName);
+extern void         TteAddSubMenu (WindowType windowtype, char *schemaName, int menuID, int itemID, int itemsNumber);
+extern void         TteAddMenuItem (WindowType windowtype, char *schemaName, int menuID, int subMenuID, int itemID, char* actionName, char itemType);
 
 extern void         TteAddAction (char* actionName, Proc doIt);
 extern void         TteAddActionEvent (PtrEventsSet eventsList, int typeId, APPevent event, ThotBool pre, char* actionName);
-extern PtrEventsSet TteGetEventsSet (CHAR_T* name);
-extern PtrEventsSet TteNewEventsSet (int structureId, CHAR_T* name);
+extern PtrEventsSet TteGetEventsSet (char *name);
+extern PtrEventsSet TteNewEventsSet (int structureId, char *name);
 extern void         TtcStandardPresentation (Document document, View view);
 extern void         TtcStandardGeometry (Document document, View view);
 extern void         TtaSetTransformCallback (Func function);
@@ -102,6 +117,7 @@ extern void         TtcCloseDocument (Document document, View view);
 extern void         TtcCloseView (Document document, View view);
 extern void         TtcCopySelection (Document document, View view);
 extern void         TtcCopyToClipboard (Document document, View view);
+extern void         TtcCreateElement (Document doc, View view);
 extern void         TtcCutSelection (Document document, View view);
 extern void         TtcDeletePreviousChar (Document document, View view);
 extern void         TtcDeleteSelection (Document document, View view);
@@ -115,8 +131,8 @@ extern void         TtcHolophrast (Document document, View view);
 extern void         TtcInclude (Document document, View view);
 extern void         TtcIndex (Document document, View view);
 extern void         TtcInsert (Document document, View view);
-extern void         TtcInsertChar (Document document, View view, UCHAR_T c);
-extern void         TtcInsertGraph (Document document, View view, UCHAR_T c);
+extern void         TtcInsertChar (Document document, View view, CHAR_T c);
+extern void         TtcInsertGraph (Document document, View view, unsigned char c);
 extern void         TtcInsertLineBreak (Document document, View view);
 extern void         TtcInsertPageBreak (Document document, View view);
 extern void         TtcLineDown (Document document, View view);
@@ -150,6 +166,8 @@ extern void         TtcQuit (Document document, View view);
 extern void         TtcScrollLeft (Document document, View view);
 extern void         TtcScrollRight (Document document, View view);
 extern void         TtcSearchText (Document document, View view);
+extern void         TtcSelStartOfLine (Document document, View view);
+extern void         TtcSelEndOfLine (Document document, View view);
 extern void         TtcSetBackupInterval (Document document, View view);
 extern void         TtcSetDocumentPath (Document document, View view);
 extern void         TtcSetSchemaPath (Document document, View view);
