@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT MIT and INRIA, 1996-2001
+ *  (c) COPYRIGHT MIT and INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -12,8 +12,8 @@
 #include "tree.h"
 typedef enum _PicType
   {
-    xbm_type, eps_type, xpm_type, gif_type, png_type, jpeg_type, svg_type, 
-    unknown_type
+    xbm_type, eps_type, xpm_type, gif_type, png_type, jpeg_type,
+    svg_type, html_type, mathml_type, unknown_type
   }
 PicType;
 
@@ -35,7 +35,8 @@ typedef int     *PathSegment;
    language: language of that Text element.
    document: the document containing that element.
   ----------------------------------------------------------------------*/
-extern void TtaSetTextContent (Element element, unsigned char *content, Language language, Document document);
+extern void TtaSetTextContent (Element element, unsigned char *content,
+			       Language language, Document document);
 
 /*----------------------------------------------------------------------
    TtaSetPictureContent
@@ -51,7 +52,8 @@ extern void TtaSetTextContent (Element element, unsigned char *content, Language
    document: the document containing that element.
    mime_type: MIME type of the picture, if known. NULL, otherwise
   ----------------------------------------------------------------------*/
-extern void TtaSetPictureContent (Element element, unsigned char *content, Language language, Document document, char *mime_type);
+extern void TtaSetPictureContent (Element element, unsigned char *content,
+				  Language language, Document document, char *mime_type);
 
 /*----------------------------------------------------------------------
    TtaAppendTextContent
@@ -62,9 +64,50 @@ extern void TtaSetPictureContent (Element element, unsigned char *content, Langu
    content: the character string to be appended coded in ISO-Latin or UTF-8.
    document: the document containing that element.
   ----------------------------------------------------------------------*/
-extern void TtaAppendTextContent (Element element, unsigned char *content, Document document);
+extern void TtaAppendTextContent (Element element, unsigned char *content,
+				  Document document);
 
 /*----------------------------------------------------------------------
+   TtaHasFinalSpace
+
+   Returns TRUE if the text element ends with a space or a NEWLINE
+   Parameters:
+   element: the Text element to be modified.
+   document: the document containing that element.
+  ----------------------------------------------------------------------*/
+extern ThotBool TtaHasFinalSpace (Element element, Document document);
+
+/*----------------------------------------------------------------------
+   TtaRemoveInitialSpaces
+
+   Removes spaces and NEWLINE at the beginning of the text element
+   Parameters:
+   element: the Text element to be modified.
+   document: the document containing that element.
+  ----------------------------------------------------------------------*/
+extern void TtaRemoveInitialSpaces (Element element, Document document);
+
+/*----------------------------------------------------------------------
+   TtaRemoveFinalSpaces
+
+   Removes spaces and NEWLINE at the end of the text element
+   Parameters:
+   element: the Text element to be modified.
+   document: the document containing that element.
+   all: when TRUE removes all spaces, when FALSE only the last space.
+  ----------------------------------------------------------------------*/
+extern void TtaRemoveFinalSpaces (Element element, Document document,
+				  ThotBool all);
+
+/*----------------------------------------------------------------------
+  TtaSearchText looks for the text forward or backward in the document.
+  If the string is found the function returns the element that includes
+  it. It return NULL if the string is not found.
+  ----------------------------------------------------------------------*/
+extern Element TtaSearchText (Document document, Element element, ThotBool forward,
+			      char *text, CHARSET encoding);
+
+/* ----------------------------------------------------------------------
    TtaInsertTextContent
 
    Inserts a character string in a text basic element.
@@ -75,7 +118,8 @@ extern void TtaAppendTextContent (Element element, unsigned char *content, Docum
    content: the character string to be inserted coded in ISO-Latin or UTF-8.
    document: the document containing the text element.
   ----------------------------------------------------------------------*/
-extern void TtaInsertTextContent (Element element, int position, unsigned char *content, Document document);
+extern void TtaInsertTextContent (Element element, int position,
+				  unsigned char *content, Document document);
 
 /*----------------------------------------------------------------------
    TtaDeleteTextContent
@@ -87,7 +131,8 @@ extern void TtaInsertTextContent (Element element, int position, unsigned char *
    length: length of the character string to be deleted.
    document: the document containing the text element.
   ----------------------------------------------------------------------*/
-extern void TtaDeleteTextContent (Element element, int position, int length, Document document);
+extern void TtaDeleteTextContent (Element element, int position, int length,
+				  Document document);
 
 /*----------------------------------------------------------------------
    TtaSplitText
@@ -96,7 +141,7 @@ extern void TtaDeleteTextContent (Element element, int position, int length, Doc
    Parameters:
    element: the text element to be divided. A new text element containing
    the second part of the text is created as the next sibling.
-   position: rank of the character after which the element must be cut.
+   position: rank of the character before which the element must be cut.
    document: the document to which the element belongs.
   ----------------------------------------------------------------------*/
 extern void TtaSplitText (Element element, int position, Document document);
@@ -154,7 +199,8 @@ extern void TtaSetSymbolCode (Element element, wchar_t code, Document document);
    must be positive or null.
    unit: UnPixel or UnPoint.
   ----------------------------------------------------------------------*/
-extern void TtaAddPointInPolyline (Element element, int rank, TypeUnit unit, int x, int y, Document document);
+extern void TtaAddPointInPolyline (Element element, int rank, TypeUnit unit,
+				   int x, int y, Document document);
 
 /*----------------------------------------------------------------------
    TtaDeletePointInPolyline
@@ -186,7 +232,8 @@ extern void TtaDeletePointInPolyline (Element element, int rank, Document docume
    document: the document containing the polyline element.
    unit: UnPixel or UnPoint.
   ----------------------------------------------------------------------*/
-extern void TtaModifyPointInPolyline (Element element, int rank, TypeUnit unit, int x, int y, Document document);
+extern void TtaModifyPointInPolyline (Element element, int rank, TypeUnit unit,
+				      int x, int y, Document document);
 
 /*----------------------------------------------------------------------
    TtaChangeLimitOfPolyline
@@ -203,7 +250,8 @@ extern void TtaModifyPointInPolyline (Element element, int rank, TypeUnit unit,
    points of the polyline.
    document: the document containing the polyline element.
   ----------------------------------------------------------------------*/
-extern void TtaChangeLimitOfPolyline (Element element, TypeUnit unit, int x, int y, Document document);
+extern void TtaChangeLimitOfPolyline (Element element, TypeUnit unit, int x,
+				      int y, Document document);
 
 /*----------------------------------------------------------------------
    TtaTransformCurveIntoPath
@@ -230,7 +278,8 @@ extern char *TtaTransformCurveIntoPath (Element el);
    Return value:
    the created path segment.
    ---------------------------------------------------------------------- */
-extern PathSegment TtaNewPathSegLine (int xstart, int ystart, int xend, int yend, ThotBool newSubpath);
+extern PathSegment TtaNewPathSegLine (int xstart, int ystart, int xend,
+				      int yend, ThotBool newSubpath);
 
 /*----------------------------------------------------------------------
    TtaNewPathSegCubic
@@ -249,7 +298,9 @@ extern PathSegment TtaNewPathSegLine (int xstart, int ystart, int xend, int yend
    Return value:
    the created path segment.
    ---------------------------------------------------------------------- */
-extern PathSegment TtaNewPathSegCubic (int xstart, int ystart, int xend, int yend, int xctrl1, int yctrl1, int xctrl2, int yctrl2, ThotBool newSubpath);
+extern PathSegment TtaNewPathSegCubic (int xstart, int ystart, int xend,
+				       int yend, int xctrl1, int yctrl1,
+				       int xctrl2, int yctrl2, ThotBool newSubpath);
 
 /*----------------------------------------------------------------------
    TtaNewPathSegQuadratic
@@ -269,7 +320,9 @@ extern PathSegment TtaNewPathSegCubic (int xstart, int ystart, int xend, int yen
    the created path segment.
 
    ---------------------------------------------------------------------- */
-extern PathSegment   TtaNewPathSegQuadratic (int xstart, int ystart, int xend, int yend, int xctrl, int yctrl, ThotBool newSubpath);
+extern PathSegment TtaNewPathSegQuadratic (int xstart, int ystart, int xend,
+					   int yend, int xctrl, int yctrl,
+					   ThotBool newSubpath);
 
 /*----------------------------------------------------------------------
    TtaNewPathSegArc
@@ -290,7 +343,10 @@ extern PathSegment   TtaNewPathSegQuadratic (int xstart, int ystart, int xend, i
    Return value:
    the created path segment.
    ---------------------------------------------------------------------- */
-extern PathSegment TtaNewPathSegArc (int xstart, int ystart, int xend, int yend, int xradius, int yradius, int angle, ThotBool largearc, ThotBool sweep, ThotBool newSubpath);
+extern PathSegment TtaNewPathSegArc (int xstart, int ystart, int xend, int yend,
+				     int xradius, int yradius, int angle,
+				     ThotBool largearc, ThotBool sweep,
+				     ThotBool newSubpath);
 
 /*----------------------------------------------------------------------
    TtaAppendPathSeg
@@ -301,8 +357,109 @@ extern PathSegment TtaNewPathSegArc (int xstart, int ystart, int xend, int yend,
    segment: the path segment to be appended.
    document: the document containing the element.
   ----------------------------------------------------------------------*/
-extern void TtaAppendPathSeg (Element element, PathSegment segment, Document document);
+extern void TtaAppendPathSeg (Element element, PathSegment segment,
+			      Document document);
 
+/*----------------------------------------------------------------------
+   TtaTransform
+  ----------------------------------------------------------------------*/
+extern void TtaInsertTransform (Element element, void *transform, 
+				 Document document);
+extern void TtaReplaceTransform (Element element, void *transform, 
+				 Document document);
+extern void TtaAppendTransform (Element element, void *transform, 
+				Document document);
+
+extern void *TtaNewTransformScale (float x_scale, float y_scale, ThotBool viewbox);
+extern void *TtaNewTransformTranslate (float x, float y, ThotBool viewbox);
+extern void *TtaNewTransformRotate (float angle, float x_scale, float y_scale);
+extern void *TtaNewTransformSkewX (float factor);
+extern void *TtaNewTransformSkewY (float factor);
+extern void *TtaNewTransformMatrix (float a, float b, float c,
+				    float d, float e, float f);
+
+extern void TtaFreeTransform (void *transform);
+
+extern void TtaSetElCoordinateSystem (Element element);
+
+extern void TtaPlay (Document doc, View view);
+extern void TtaNoPlay (int frame);
+/*----------------------------------------------------------------------
+   TtaAppendTransform
+   Copy a Linked List of transform into another struct
+  ----------------------------------------------------------------------*/
+void *TtaCopyTransform (void *pPa);
+void *TtaNewTransformAnimTranslate (float x, float y);
+void *TtaNewTransformAnimRotate (float angle, float x_scale, 
+				 float y_scale);
+/*----------------------------------------------------------------------
+   TtaAnim
+  ----------------------------------------------------------------------*/
+extern void *TtaNewAnimInfo ();
+
+extern void *TtaNewAnimation ();
+extern void TtaAddAnimPath (void *info, void *anim);
+
+extern void TtaAddAnimMotionPath (void *info, void *anim);
+extern void TtaAddAnimMotionFromTo (void *info, void *anim);
+extern void TtaAddAnimMotionValues (void *info, void *anim);
+
+extern void TtaAnimPathAddPoint (void *anim, float x, float y);
+
+extern void *TtaCopyTransform ( void *pPa );
+extern void TtaAppendAnim (Element element, void *anim);
+
+extern void TtaSetAnimationTime (void *anim_info, double start, double duration);
+
+extern void TtaSetAnimReplace (void *anim, ThotBool is_replace);
+
+extern void TtaAddAnimFreeze (void *anim);
+extern void TtaAddAnimRemove (void *anim);
+
+extern void TtaAddAnimRepeatCount (int repeat, void *anim);
+
+extern void TtaAddAnimTo (void *info, void *anim);
+extern void TtaAddAnimFrom (void *info, void *anim);
+extern void TtaAddAnimAttrName (void *info, void *anim);
+
+extern void TtaSetAnimTypetoSet (void *anim);
+
+extern void TtaAppendPathSegToAnim (void *anim, PathSegment segment, Document doc);
+extern void *TtaNewAnimPath (Document doc);
+
+
+extern void TtaSetAnimTypetoColor (void *anim);
+extern void TtaSetAnimTypetoAnimate (void *anim);
+extern void TtaSetAnimTypetoTransform (void *anim);
+extern void TtaSetAnimTypetoMotion (void *anim);
+
+extern void TtaFreeAnimation (void *anim);
+extern void TtaSetDocumentCurrentTime (double current_time, Document basedoc);
+extern void TtaDisableScrollbars (Document doc, View view);
+
+extern void TtaRegisterTimeEvent(void (*pfunc) (Document doc, double current_time));
+/*----------------------------------------------------------------------
+  TtaCopyTransform
+  Copy a Linked List of transforms into another struct
+  ----------------------------------------------------------------------*/
+extern void *TtaCopyTransform (void *void_pPa);
+/*----------------------------------------------------------------------
+   TtaCopyAnim : Copy anim linked list 
+  ----------------------------------------------------------------------*/
+extern void *TtaCopyAnim (void *src);
+
+/*----------------------------------------------------------------------
+   TtaSetLinearGradient
+   ----------------------------------------------------------------------*/
+void TtaSetLinearGradient (Element el);
+void TtaSetLinearx1Gradient (int value, Element el);
+void TtaSetLineary1Gradient (int value, Element el);
+void TtaSetLinearx2Gradient (int value, Element el);
+void TtaSetLineary2Gradient (int value, Element el);
+void TtaSetStopColorGradient (unsigned short red, unsigned short green,
+			      unsigned short blue, Element el);
+void TtaSetStopOffsetColorGradient (float offset, Element el);
+     
 /*----------------------------------------------------------------------
    TtaCopyPage
 
@@ -334,18 +491,6 @@ extern PicType TtaGetPictureType (Element element);
   ----------------------------------------------------------------------*/
 extern void TtaSetPictureType (Element element, char *mime_type);
 
-
-/*----------------------------------------------------------------------
-   TtaGetVolume
-
-   Returns the current volume of the current element.
-   Parameter:
-   element: the element of interest.
-   Return value:
-   volume contained in the element.
-  ----------------------------------------------------------------------*/
-extern int TtaGetVolume (Element element);
-
 /*----------------------------------------------------------------------
    TtaGetTextLength
 
@@ -376,7 +521,66 @@ extern int TtaGetTextLength (Element element);
    language: language of the text.
    In _I18N mode returns a UTF-8 string.
   ----------------------------------------------------------------------*/
-extern void TtaGiveTextContent (Element element, unsigned char *buffer, int *length, Language *language);
+extern void TtaGiveTextContent (Element element, unsigned char *buffer,
+				int *length, Language *language);
+
+/*----------------------------------------------------------------------
+   TtaGiveBufferContent
+
+   Returns a sub buffer from a Text basic element limited to the first
+   Buffer length.
+   Parameters:
+   element: the element of interest. This element must be a basic
+   element of type Text.
+   buffer: the buffer that will contain the returned string. This buffer
+   must be at least of size length.
+   The length corresponds to the buffer length.
+   length: the length of the substring. Must be strictly positive.
+   Return parameter:
+   buffer: (the buffer contains the substring).
+   language: language of the text.
+  ----------------------------------------------------------------------*/
+extern void TtaGiveBufferContent (Element element, CHAR_T *buffer, int length,
+				  Language *language);
+
+/*----------------------------------------------------------------------
+   TtaSetBufferContent
+
+   Changes the content of a Text basic element. The full content (if any) is
+   deleted and replaced by the new one.
+   Parameters:
+   element: the Text element to be modified.
+   content: new content for that element coded as a string of CHAR_T
+   characters.
+   language: language of that Text element.
+   document: the document containing that element.
+  ----------------------------------------------------------------------*/
+extern void TtaSetBufferContent (Element element, CHAR_T *content,
+				 Language language, Document document);
+
+/*----------------------------------------------------------------------
+   TtaGetFirstBufferContent
+
+   Returns the first CHAR_T character of the string.
+   Parameters:
+   element: the element of interest. This element must be a basic
+   element of type Text.
+   Return parameter:
+   buffer: the first character.
+  ----------------------------------------------------------------------*/
+extern CHAR_T TtaGetFirstBufferContent (Element element);
+
+/*----------------------------------------------------------------------
+   TtaGetLastBufferContent
+
+   Returns the last CHAR_T character of the string.
+   Parameters:
+   element: the element of interest. This element must be a basic
+   element of type Text.
+   Return parameter:
+   buffer: the last character.
+  ----------------------------------------------------------------------*/
+extern CHAR_T TtaGetLastBufferContent (Element element);
 
 /*----------------------------------------------------------------------
    TtaGiveSubString
@@ -395,7 +599,8 @@ extern void TtaGiveTextContent (Element element, unsigned char *buffer, int *len
    buffer: (the buffer contains the substring).
    In _I18N mode returns a UTF-8 string.
   ----------------------------------------------------------------------*/
-extern void TtaGiveSubString (Element element, unsigned char *buffer, int position, int length);
+extern void TtaGiveSubString (Element element, unsigned char *buffer,
+			      int position, int length);
 
 /*----------------------------------------------------------------------
    TtaGetGraphicsShape
@@ -435,7 +640,8 @@ extern int TtaGetPolylineLength (Element element);
    x, y: coordinates of the point, in unit, relatively to
    the upper left corner of the enclosing rectangle.
   ----------------------------------------------------------------------*/
-extern void TtaGivePolylinePoint (Element element, int rank, TypeUnit unit, /*OUT*/ int *x, /*OUT*/ int *y);
+extern void TtaGivePolylinePoint (Element element, int rank, TypeUnit unit,
+				  /*OUT*/ int *x, /*OUT*/ int *y);
 
 /*----------------------------------------------------------------------
    TtaGetPageNumber
