@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT MIT and INRIA, 1996-2001.
+ *  (c) COPYRIGHT MIT and INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -44,7 +44,8 @@ typedef int        *SSchema;
    the document that has been created or 0 if the document has not
    been created.
   ----------------------------------------------------------------------*/
-extern Document     TtaInitDocument (CHAR_T* structureSchema, CHAR_T* documentName, Document document);
+extern Document TtaInitDocument (char *structureSchema, char *documentName,
+				 Document document);
 
 /*----------------------------------------------------------------------
    TtaNewDocument
@@ -65,7 +66,7 @@ extern Document     TtaInitDocument (CHAR_T* structureSchema, CHAR_T* documentNa
    the document that has been created or 0 if the document has not
    been created.
   ----------------------------------------------------------------------*/
-extern Document     TtaNewDocument (CHAR_T* structureSchema, CHAR_T* documentName);
+extern Document TtaNewDocument (char *structureSchema, char *documentName);
 
 /*----------------------------------------------------------------------
    TtaOpenDocument
@@ -82,7 +83,7 @@ extern Document     TtaNewDocument (CHAR_T* structureSchema, CHAR_T* documentNam
    the opened document, or 0 if the document cannot be open.
 
   ----------------------------------------------------------------------*/
-extern Document     TtaOpenDocument (STRING documentName, int accessMode);
+extern Document TtaOpenDocument (char *documentName, int accessMode);
 
 /*----------------------------------------------------------------------
    TtaSaveDocument
@@ -102,7 +103,7 @@ extern Document     TtaOpenDocument (STRING documentName, int accessMode);
    removed by the function TtaRemoveDocument.
 
   ----------------------------------------------------------------------*/
-extern void         TtaSaveDocument (Document document, STRING documentName);
+extern void TtaSaveDocument (Document document, char *documentName);
 
 /*----------------------------------------------------------------------
    TtaExportDocument
@@ -121,7 +122,8 @@ extern void         TtaSaveDocument (Document document, STRING documentName);
    
    Return: TRUE if done correctly
   ----------------------------------------------------------------------*/
-extern ThotBool     TtaExportDocument (Document document, STRING fileName, STRING TSchemaName);
+extern ThotBool TtaExportDocument (Document document, char *fileName,
+				   char *TSchemaName);
 
 /*----------------------------------------------------------------------
    TtaExportDocumentWithNewLineNumbers
@@ -142,9 +144,9 @@ extern ThotBool     TtaExportDocument (Document document, STRING fileName, STRIN
 
    Return: TRUE if done correctly
   ----------------------------------------------------------------------*/
-extern ThotBool     TtaExportDocumentWithNewLineNumbers (Document document,
-							 STRING fileName,
-							 STRING TSchemaName);
+extern ThotBool TtaExportDocumentWithNewLineNumbers (Document document,
+						     char *fileName,
+						     char *TSchemaName);
 
 /*----------------------------------------------------------------------
    TtaCloseDocument
@@ -156,7 +158,7 @@ extern ThotBool     TtaExportDocumentWithNewLineNumbers (Document document,
    document: the document to be closed.
 
   ----------------------------------------------------------------------*/
-extern void         TtaCloseDocument (Document document);
+extern void TtaCloseDocument (Document document);
 
 /*----------------------------------------------------------------------
    TtaRemoveDocument
@@ -168,7 +170,7 @@ extern void         TtaCloseDocument (Document document);
    Parameter:
    document: the document to be removed.
   ----------------------------------------------------------------------*/
-extern void         TtaRemoveDocument (Document document);
+extern void TtaRemoveDocument (Document document);
 
 /*----------------------------------------------------------------------
    TtaSetDocumentPath
@@ -184,7 +186,7 @@ extern void         TtaRemoveDocument (Document document);
    the character PATH_SEP.
 
   ----------------------------------------------------------------------*/
-extern void         TtaSetDocumentPath (STRING path);
+extern void TtaSetDocumentPath (char *path);
 
 /*----------------------------------------------------------------------
    TtaCheckPath
@@ -199,7 +201,7 @@ extern void         TtaSetDocumentPath (STRING path);
    accessed.
 
   ----------------------------------------------------------------------*/
-extern ThotBool     TtaCheckPath (char* path );
+extern ThotBool TtaCheckPath (char *path);
 
 /*----------------------------------------------------------------------
    TtaIsInDocumentPath
@@ -210,7 +212,7 @@ extern ThotBool     TtaCheckPath (char* path );
    directory: the new directory name.
 
   ----------------------------------------------------------------------*/
-extern ThotBool     TtaIsInDocumentPath (CHAR_T* aDirectory);
+extern ThotBool TtaIsInDocumentPath (char *aDirectory);
 
 /*----------------------------------------------------------------------
    TtaAppendDocumentPath
@@ -222,7 +224,7 @@ extern ThotBool     TtaIsInDocumentPath (CHAR_T* aDirectory);
    directory: the new directory name.
 
   ----------------------------------------------------------------------*/
-extern void         TtaAppendDocumentPath (CHAR_T* aDirectory);
+extern void TtaAppendDocumentPath (char *aDirectory);
 
 /*----------------------------------------------------------------------
    TtaSetSchemaPath
@@ -235,7 +237,7 @@ extern void         TtaAppendDocumentPath (CHAR_T* aDirectory);
    the character PATH_SEP.
 
   ----------------------------------------------------------------------*/
-extern void         TtaSetSchemaPath (STRING path);
+extern void TtaSetSchemaPath (char *path);
 
 /*----------------------------------------------------------------------
    TtaNewNature
@@ -258,7 +260,8 @@ extern void         TtaSetSchemaPath (STRING path);
    has not been loaded.
 
   ----------------------------------------------------------------------*/
-extern SSchema      TtaNewNature (Document document, SSchema schema, CHAR_T* natureName, CHAR_T* presentationName);
+extern SSchema TtaNewNature (Document document, SSchema schema,
+			     char *natureName, char *presentationName);
 
 /*----------------------------------------------------------------------
    TtaNewSchemaExtension
@@ -277,7 +280,8 @@ extern SSchema      TtaNewNature (Document document, SSchema schema, CHAR_T* nat
    the extension schema, NULL if the extension schema has not been loaded.
 
   ----------------------------------------------------------------------*/
-extern SSchema      TtaNewSchemaExtension (Document document, CHAR_T* extensionName, CHAR_T* presentationName);
+extern SSchema TtaNewSchemaExtension (Document document, char *extensionName,
+				      char *presentationName);
 
 /*----------------------------------------------------------------------
    TtaGetSchemaExtension
@@ -293,7 +297,7 @@ extern SSchema      TtaNewSchemaExtension (Document document, CHAR_T* extensionN
    NULL if the extension is not associated with the document.
 
   ----------------------------------------------------------------------*/
-extern SSchema      TtaGetSchemaExtension (Document document, STRING NomExtension);
+extern SSchema TtaGetSchemaExtension (Document document, char *NomExtension);
 
 /*----------------------------------------------------------------------
    TtaRemoveSchemaExtension
@@ -311,7 +315,8 @@ extern SSchema      TtaGetSchemaExtension (Document document, STRING NomExtensio
    removedAttributes: number of attributes actually removed.
 
   ----------------------------------------------------------------------*/
-extern void         TtaRemoveSchemaExtension (Document document, SSchema extension, int *removedElements, int *removedAttributes);
+extern void TtaRemoveSchemaExtension (Document document, SSchema extension,
+				      int *removedElements, int *removedAttributes);
 
 /*----------------------------------------------------------------------
    TtaSetPSchema
@@ -325,7 +330,7 @@ extern void         TtaRemoveSchemaExtension (Document document, SSchema extensi
    with the document.
 
   ----------------------------------------------------------------------*/
-extern void         TtaSetPSchema (Document document, CHAR_T* presentationName);
+extern void TtaSetPSchema (Document document, char *presentationName);
 
 /*----------------------------------------------------------------------
    TtaSetDocumentDirectory
@@ -337,7 +342,7 @@ extern void         TtaSetPSchema (Document document, CHAR_T* presentationName);
    directory: new document directory.
 
   ----------------------------------------------------------------------*/
-extern void         TtaSetDocumentDirectory (Document document, CHAR_T* directory);
+extern void TtaSetDocumentDirectory (Document document, char *directory);
 
 /*----------------------------------------------------------------------
    TtaRemoveSchemaExtension
@@ -349,40 +354,31 @@ extern void         TtaSetDocumentDirectory (Document document, CHAR_T* director
    Parameters:
    document: the document.
    extension: the structure schema extension to be removed.
-
    Return parameters:
    removedElements: number of elements actually removed.
    removedAttributes: number of attributes actually removed.
-
   ----------------------------------------------------------------------*/
-extern void         TtaSetDocumentName (Document document, STRING documentName);
+extern void TtaSetDocumentName (Document document, char *documentName);
 
 /*----------------------------------------------------------------------
    TtaSetDocumentAccessMode
 
    Sets the access mode for a document.
-
    Parameters:
    document: the document whose access mode is changed.
    accessMode: 0 = read only, 1 = read-write.
-
   ----------------------------------------------------------------------*/
-extern void         TtaSetDocumentAccessMode (Document document, int accessMode);
+extern void TtaSetDocumentAccessMode (Document document, int accessMode);
 
 /*----------------------------------------------------------------------
    TtaSetDocumentBackUpInterval
 
-   Sets the backup interval for a document.
-
-   Parameters:
-   document: the document whose backup interval is changed.
-   interval:
-   0 : the backup mechanism must be disabled
-   positive integer : number of characters typed which triggers
-   automatic save of the document into a .BAK file.
-
+   Sets the backup interval of documents.
+   Parameter:
+   interval: 0 if the backup mechanism is disabled, or gives the number
+   of characters which triggers an automatic save into a .BAK file.
   ----------------------------------------------------------------------*/
-extern void         TtaSetDocumentBackUpInterval (Document document, int interval);
+extern void TtaSetDocumentBackUpInterval (int interval);
 
 /*----------------------------------------------------------------------
    TtaSetNotificationMode
@@ -395,19 +391,17 @@ extern void         TtaSetDocumentBackUpInterval (Document document, int interva
    be notified.
 
   ----------------------------------------------------------------------*/
-extern void         TtaSetNotificationMode (Document document, int notificationMode);
+extern void TtaSetNotificationMode (Document document, int notificationMode);
 
 /*----------------------------------------------------------------------
    TtaSetDocumentModified
 
    Notifies the tool kit that a document has been modified by the application.
    As a consequence, the user will be asked to save the document when closing it.
-
    Parameter:
    document: the document.
-
   ----------------------------------------------------------------------*/
-extern void         TtaSetDocumentModified (Document document);
+extern void TtaSetDocumentModified (Document document);
 
 /*----------------------------------------------------------------------
    TtaSetDocumentUnmodified
@@ -416,12 +410,10 @@ extern void         TtaSetDocumentModified (Document document);
    by the application or by the user. As a consequence, if no further modification
    is made to that document, the user will not be asked to save the document
    when closing it.
-
    Parameter:
    document: the document.
-
   ----------------------------------------------------------------------*/
-extern void         TtaSetDocumentUnmodified (Document document);
+extern void TtaSetDocumentUnmodified (Document document);
 
 /*----------------------------------------------------------------------
    TtaSetDocumentUnupdated
@@ -430,148 +422,120 @@ extern void         TtaSetDocumentUnmodified (Document document);
    by the application or by the user. That will allow the application to
    detect if any change will be made on the document
    (see TtaIsDocumentUpdated).
-
    Parameter:
    document: the document.
-
   ----------------------------------------------------------------------*/
-extern void         TtaSetDocumentUnupdated (Document document);
+extern void TtaSetDocumentUnupdated (Document document);
 
 /*----------------------------------------------------------------------
    TtaGetDocumentName
 
    Returns the name of a document.
-
    Parameter:
    document: the document whose name is asked.
-
    Return value:
    name of that document.
-
   ----------------------------------------------------------------------*/
-extern CHAR_T*       TtaGetDocumentName (Document document);
+extern char *TtaGetDocumentName (Document document);
 
 /*----------------------------------------------------------------------
    TtaGetDocumentFromName
 
    Returns the document having a given name.
-
    Parameter:
    documentName: the document name.
-
    Return value:
    the document having that name.
-
   ----------------------------------------------------------------------*/
-extern Document     TtaGetDocumentFromName (STRING documentName);
+extern Document TtaGetDocumentFromName (char *documentName);
 
 /*----------------------------------------------------------------------
    TtaGetDocumentDirectory
 
    Returns the directory to which the document is supposed to be saved.
-
    Parameters:
    document: the document whose directory is asked.
    buffer: a buffer provided by the caller.
    bufferLength: the length of that buffer.
-
    Return parameter:
    buffer: the document directory.
-
   ----------------------------------------------------------------------*/
-extern void         TtaGetDocumentDirectory (Document document, STRING buffer, int bufferLength);
+extern void TtaGetDocumentDirectory (Document document, char *buffer,
+				     int bufferLength);
 
 /*----------------------------------------------------------------------
    TtaGetDocumentSSchema
 
    Returns the main structure schema of a document.
-
    Parameter:
    document: the document for which the structure schema is asked.
-
    Return value:
    the structure schema of that document.
-
   ----------------------------------------------------------------------*/
-extern SSchema      TtaGetDocumentSSchema (Document document);
+extern SSchema TtaGetDocumentSSchema (Document document);
 
 /*----------------------------------------------------------------------
    TtaGetSSchemaName
 
    Returns the name of a structure schema.
-
    Parameter:
    schema: the structure schema of interest.
-
    Return value:
    name of that structure schema.
-
   ----------------------------------------------------------------------*/
-extern CHAR_T*        TtaGetSSchemaName (SSchema schema);
+extern char *TtaGetSSchemaName (SSchema schema);
 
 /*----------------------------------------------------------------------
    TtaGetPSchemaName
 
    Returns the name of the presentation schema currently associated
    with a given structure schema.
-
    Parameter:
    schema: the structure schema of interest.
-
    Return value:
    name of the associated presentation schema.
-
   ----------------------------------------------------------------------*/
-extern CHAR_T*       TtaGetPSchemaName (SSchema schema);
+extern char *TtaGetPSchemaName (SSchema schema);
 
 /*----------------------------------------------------------------------
    TtaGetSSchema
 
    Returns a structure schema whose name is known and that is used in a
    given document.
-
    Parameters:
    name: the name of the structure schema of interest.
    document: the document that uses this structure schema.
-
    Return value:
    the structure schema having this name, or NULL if this structure
    schema is not loaded or not used by the document.
-
   ----------------------------------------------------------------------*/
-extern SSchema      TtaGetSSchema (CHAR_T* name, Document document);
+extern SSchema TtaGetSSchema (char *name, Document document);
 
 /*----------------------------------------------------------------------
    TtaGetSSchemaByUri
 
    Returns a structure schema whose URI is known and that is used in a
    given document.
-
    Parameters:
    uriName: the URI of the structure schema of interest.
    document: the document that uses this structure schema.
-
    Return value:
    the structure schema having this URI, or NULL if this structure
    schema is not loaded or not used by the document.
-
   ----------------------------------------------------------------------*/
-SSchema             TtaGetSSchemaByUri (CHAR_T* uriName, Document document);
+SSchema TtaGetSSchemaByUri (char *uriName, Document document);
 
 /*----------------------------------------------------------------------
    TtaSameSSchemas
 
    Compares two structure schemas.
-
    Parameters:
    schema1: first structure schema.
    schema2: second structure schema.
-
    Return value:
    0 if both schemas are different, 1 if they are identical.
-
   ----------------------------------------------------------------------*/
-extern int          TtaSameSSchemas (SSchema schema1, SSchema schema2);
+extern int TtaSameSSchemas (SSchema schema1, SSchema schema2);
 
 /*----------------------------------------------------------------------
    TtaGiveSchemasOfDocument
@@ -579,25 +543,23 @@ extern int          TtaSameSSchemas (SSchema schema1, SSchema schema2);
    Returns the names of the main structure schema and presentation schema
    associated with a given document. The document does not need to be open
    and the schemas are not loaded by this function.
-
    Parameters:
    documentName: Name of the document to be checked (maximum length
    19 characters).
    structureName: buffer.
    presentationName: buffer.
-
    Return value:
    structureName: Name of the document structure schema.
    presentationName: Name of the document presentation schema.
-
   ----------------------------------------------------------------------*/
-extern void         TtaGiveSchemasOfDocument (STRING documentName, /*OUT*/ char* structureName, /*OUT*/ char* presentationName);
+extern void TtaGiveSchemasOfDocument (char *documentName,
+				      /*OUT*/ char* structureName,
+				      /*OUT*/ char* presentationName);
 
 /*----------------------------------------------------------------------
    TtaNextSchemaExtension
 
    Returns a structure schema extension associated with a given document.
-
    Parameters:
    document: the document of interest.
    extension: a schema extension of that document. NULL for accessing
@@ -606,15 +568,13 @@ extern void         TtaGiveSchemasOfDocument (STRING documentName, /*OUT*/ char*
    extension: the schema extension that follows or the first schema
    extension of the document if parameter extension is NULL.
    NULL if there is no more schema extension.
-
   ----------------------------------------------------------------------*/
-extern void         TtaNextSchemaExtension (Document document, /*INOUT*/ SSchema * extension);
+extern void TtaNextSchemaExtension (Document document, /*INOUT*/ SSchema * extension);
 
 /*----------------------------------------------------------------------
    TtaNextNature
 
    Returns the structure schema of a nature used in a given document.
-
    Parameters:
    document: the document of interest.
    nature: the structure schema of a nature for that document. NULL for
@@ -624,9 +584,8 @@ extern void         TtaNextSchemaExtension (Document document, /*INOUT*/ SSchema
    of the first nature of the document if parameter nature was NULL
    when calling.
    NULL if there is no more nature for the document.
-
   ----------------------------------------------------------------------*/
-extern void         TtaNextNature (Document document, /*INOUT*/ SSchema * nature);
+extern void TtaNextNature (Document document, /*INOUT*/ SSchema *nature);
 
 /*----------------------------------------------------------------------
    TtaIsDocumentModified
@@ -634,16 +593,13 @@ extern void         TtaNextNature (Document document, /*INOUT*/ SSchema * nature
    Indicates whether a document has been modified by the user or not.
    Modifications made by the application program are not considered,
    except when explicitely notified by TtaDocumentModified.
-
    Parameter:
    document: the document.
-
    Return value:
    1 if the document has been modified by the user since it has been saved,
    loaded or created, 0 if it has not been modified.
-
   ----------------------------------------------------------------------*/
-extern int          TtaIsDocumentModified (Document document);
+extern int TtaIsDocumentModified (Document document);
 
 /*----------------------------------------------------------------------
    TtaIsDocumentUpdated
@@ -652,107 +608,86 @@ extern int          TtaIsDocumentModified (Document document);
    since the last TtaSetDocumentUnupdated or TtaSetDocumentUnmodified.
    Modifications made by the application program are not considered,
    except when explicitely notified by TtaDocumentModified.
-
    Parameter:
    document: the document.
-
    Return value:
    1 if the document has been modified by the user since it has been saved,
    loaded or created, 0 if it has not been modified.
-
   ----------------------------------------------------------------------*/
-extern int          TtaIsDocumentUpdated (Document document);
+extern int TtaIsDocumentUpdated (Document document);
 
 /*----------------------------------------------------------------------
    TtaGetDocumentAccessMode
 
    Returns the access mode for a document.
-
    Parameter:
    document: the document whose access mode is asked.
    Return value:
    0 if access mode is read only, 1 if access mode is read-write.
-
   ----------------------------------------------------------------------*/
-extern int          TtaGetDocumentAccessMode (Document document);
+extern int TtaGetDocumentAccessMode (Document document);
 
 /*----------------------------------------------------------------------
    TtaGetDocumentBackUpInterval
 
-   Returns backup interval for a document.
-
-   Parameters:
-   document: the document whose backup interval is asked.
+   Returns backup interval of documents.
    Return value:
-   0 : the backup mechanism is disabled
-   positive integer : number of typed characters which trigger an autamatic
-   save of the document into a .BAK file.
-
+   0 if the backup mechanism is disabled or the number of characters
+   which trigger an automatic save into a .BAK file.
   ----------------------------------------------------------------------*/
-extern int          TtaGetDocumentBackUpInterval (Document document);
+extern int TtaGetDocumentBackUpInterval ();
 
 /*----------------------------------------------------------------------
    TtaGetNotificationMode
 
    Returns the ECF notification mode for a document.
-
    Parameters:
    document: the document whose notification mode is asked.
    Return value:
    0 = if only roots of created and deleted subtrees must be notified,
    1 = all elements of created and deleted subtrees must be notified.
-
   ----------------------------------------------------------------------*/
-extern int          TtaGetNotificationMode (Document document);
+extern int TtaGetNotificationMode (Document document);
 
 /*----------------------------------------------------------------------
    TtaGetDocumentPath
 
    Returns the current list of the directories used when a document is open
    (see TtaOpenDocument).
-
    Parameters:
    buffer: a buffer provided by the caller.
    bufferLength: the length of that buffer.
-
    Return parameter:
    buffer: the list of directories. Directory names are separated by
    the character PATH_SEP.
-
   ----------------------------------------------------------------------*/
-extern void         TtaGetDocumentPath (/*OUT*/ STRING buffer, int bufferLength);
+extern void TtaGetDocumentPath (/*OUT*/ char *buffer, int bufferLength);
 
 /*----------------------------------------------------------------------
    TtaGetSchemaPath
 
    Returns the current list of directories used for accessing schemas.
-
    Parameters:
    buffer: a buffer provided by the caller.
    bufferLength: the length of that buffer.
-
    Return parameter:
    buffer: the list of directories. Directory names are separated by
    the character PATH_SEP.
-
   ----------------------------------------------------------------------*/
-extern void         TtaGetSchemaPath (/*OUT*/ STRING buffer, int bufferLength);
+extern void TtaGetSchemaPath (/*OUT*/ char *buffer, int bufferLength);
 
 /*----------------------------------------------------------------------
    TtaGetDocumentOfSavedElements
 
    Returns the document for which the last Copy or Cut command has been
    issued.
-
    Parameters:
    no parameter.
-
    Return value:
    the document for which the last Copy or Cut command has been issued.
    0 if the clipboard is empty.
-
   ----------------------------------------------------------------------*/
-extern Document     TtaGetDocumentOfSavedElements ();
+extern Document TtaGetDocumentOfSavedElements ();
 
 
 /*----------------------------------------------------------------------
@@ -760,7 +695,7 @@ extern Document     TtaGetDocumentOfSavedElements ();
    structure (documents, natures et extensions) qui ont    
    des fichiers de langue dans les directories de schemas. 
   ----------------------------------------------------------------------*/
-extern void TtaConfigReadConfigFiles ( CHAR_T* aSchemaPath );
+extern void TtaConfigReadConfigFiles (char *aSchemaPath);
 
 /*----------------------------------------------------------------------
    TtaConfigSSchemaExternalName retourne dans nameUser le nom     
@@ -769,7 +704,8 @@ extern void TtaConfigReadConfigFiles ( CHAR_T* aSchemaPath );
    Typ indique s'il s'agit d'un schema de document (1), de         
    nature (2) ou d'extension (3).                                  
   ----------------------------------------------------------------------*/
-extern void TtaConfigSSchemaExternalName (/*OUT*/ CHAR_T* nameUser, CHAR_T* nameSchema, int Typ );
+extern void TtaConfigSSchemaExternalName (/*OUT*/ char *nameUser,
+					  char *nameSchema, int Typ );
 
 /*----------------------------------------------------------------------
   TtaGetDocumentCharset gets the document charset
@@ -781,8 +717,61 @@ extern CHARSET TtaGetDocumentCharset (Document document);
   TtaSetDocumentCharset sets the document charset
   ----------------------------------------------------------------------*/
 extern void  TtaSetDocumentCharset (Document document,
-				    CHARSET charSet);
+				    CHARSET charSet,
+				    ThotBool defaultCharset);
+
+/*----------------------------------------------------------------------
+  TtaGetDocumentProfile
+  Gets the document profile
+ ----------------------------------------------------------------------*/
+extern int TtaGetDocumentProfile (Document document);
+
+/*----------------------------------------------------------------------
+  TtaSetDocumentProfile
+  Sets the document profile
+ ----------------------------------------------------------------------*/
+extern void TtaSetDocumentProfile (Document document, int profile);
+
+/*----------------------------------------------------------------------
+  TtaSetNamespaceDeclaration
+  Sets a namespace declaration for an element 
+ ----------------------------------------------------------------------*/
+void TtaSetANamespaceDeclaration (Document document, Element element,
+				 char *Ns_Prefix, char *Ns_Uri);
+
+/*----------------------------------------------------------------------
+  TtaFreeElemNamespaceDeclarations
+  Free all the namespaces declarations of a document
+ ----------------------------------------------------------------------*/
+void TtaFreeElemNamespaceDeclarations (Document document, Element element);
+
+/*----------------------------------------------------------------------
+  TtaFreeNamespaceDeclarations
+  Free all the namespaces declarations of a document
+ ----------------------------------------------------------------------*/
+void TtaFreeNamespaceDeclarations (Document document);
+
+/*----------------------------------------------------------------------
+  TtaShowNamespaceDeclarations
+  Show the namespaces declarations related to a document
+ ----------------------------------------------------------------------*/
+void TtaShowNamespaceDeclarations (Document document);
+
+/*----------------------------------------------------------------------
+  TtaSetUriSSchema
+  Set the schema namespace declaration uri
+ ----------------------------------------------------------------------*/
+void TtaSetUriSSchema (SSchema sSchema, char *sSchemaUri);
 
 #endif /* __CEXTRACT__ */
 
+/*----------------------------------------------------------------------
+  TtaIsXmlSSchema
+  Returns yes if schema corresponds to a generic xml structure schema
+  Parameter:
+  schema: the structure schema of interest.
+  ----------------------------------------------------------------------*/
+ThotBool TtaIsXmlSSchema (SSchema schema);
+
+
 #endif
