@@ -139,6 +139,9 @@
 #define TMSG_DEL_ATTR		134
 #define TMSG_CPBUTTON_1		135
 #define TMSG_CPBUTTON_2		136
-#define TMSG_LIB_MSG_MAX		137
+#define TMSG_CPMOREBUTTON		137
+#define TMSG_CPGETBUTTON		138
+#define TMSG_CPCOLORBUTTON		139
+#define TMSG_LIB_MSG_MAX		140
 
 #endif
