@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT MIT and INRIA, 1996-2001.
+ *  (c) COPYRIGHT MIT and INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -16,67 +16,128 @@ typedef int        *PRule;
 
 #define PRVisibility 0
 #define PRFunction 1
-#define PRVertRef 2
-#define PRHorizRef 3
-#define PRHeight 4
-#define PRWidth 5
-#define PRVertPos 6
-#define PRHorizPos 7
-#define PRMarginTop 8
-#define PRMarginRight 9
-#define PRMarginBottom 10
-#define PRMarginLeft 11
-#define PRPaddingTop 12
-#define PRPaddingRight 13
-#define PRPaddingBottom 14
-#define PRPaddingLeft 15
-#define PRBorderTopWidth 16
-#define PRBorderRightWidth 17
-#define PRBorderBottomWidth 18
-#define PRBorderLeftWidth 19
-#define PRBorderTopColor 20
-#define PRBorderRightColor 21
-#define PRBorderBottomColor 22
-#define PRBorderLeftColor 23
-#define PRBorderTopStyle 24
-#define PRBorderRightStyle 25
-#define PRBorderBottomStyle 26
-#define PRBorderLeftStyle 27
-#define PRSize 28
-#define PRStyle 29
-#define PRWeight 30
-#define PRFont 31
-#define PRUnderline 32
-#define PRThickness 33
-#define PRIndent 34
-#define PRLineSpacing 35
-#define PRDepth 36
-#define PRAdjust 37
-#define PRLineStyle 38
-#define PRLineWeight 39
-#define PRFillPattern 40
-#define PRBackground 41
-#define PRForeground 42
-#define PRHyphenate 43
-#define PRVertOverflow 44
-#define PRHorizOverflow 45
+#define PRVertOverflow 2
+#define PRHorizOverflow 3
+#define PRVertRef 4
+#define PRHorizRef 5
+#define PRHeight 6
+#define PRWidth 7
+#define PRVertPos 8
+#define PRHorizPos 9
+#define PRMarginTop 10
+#define PRMarginRight 11
+#define PRMarginBottom 12
+#define PRMarginLeft 13
+#define PRPaddingTop 14
+#define PRPaddingRight 15
+#define PRPaddingBottom 16
+#define PRPaddingLeft 17
+#define PRBorderTopWidth 18
+#define PRBorderRightWidth 19
+#define PRBorderBottomWidth 20
+#define PRBorderLeftWidth 21
+#define PRBorderTopColor 22
+#define PRBorderRightColor 23
+#define PRBorderBottomColor 24
+#define PRBorderLeftColor 25
+#define PRBorderTopStyle 26
+#define PRBorderRightStyle 27
+#define PRBorderBottomStyle 28
+#define PRBorderLeftStyle 29
+#define PRSize 30
+#define PRStyle 31
+#define PRWeight 32
+#define PRFont 33
+#define PRUnderline 34
+#define PRThickness 35
+#define PRIndent 36
+#define PRLineSpacing 37
+#define PRDepth 38
+#define PRAdjust 39
+#define PRDirection 40
+#define PRUnicodeBidi 41
+#define PRLineStyle 42
+#define PRLineWeight 43
+#define PRFillPattern 44
+#define PRBackground 45
+#define PRForeground 46
+#define PRHyphenate 47
 /* PtPageBreak, PtLineBreak, PtGather */
-#define PRXRadius 49
-#define PRYRadius 50
-#define PRNoBreak1 51
-#define PRNoBreak2 52
-/* PtPictInfo */
-#define PRLine 54
-#define PRCreateEnclosing 55
-#define PRShowBox 56
-#define PRBackgroundPicture 57
-#define PRPictureMode 58
-#define PRNotInLine 59
-#define PRNone 60
-#define PRPageBefore 61
-#define PRPageAfter 62
-#define PRPageInside 63
-#define LAST_PRES_RULE_TYPE 63
+#define PRXRadius 51
+#define PRYRadius 52
+#define PRDisplay 53
+#define PRNoBreak1 54
+#define PRNoBreak2 55
+/* PtPictInfo, PRLine */
+#define PRCreateEnclosing 58
+#define PRShowBox 59
+#define PRBackgroundPicture 60
+#define PRPictureMode 61
+#define PRNotInLine 62
+#define PRNone 63
+#define PRPageBefore 64
+#define PRPageAfter 65
+#define PRPageInside 66
+#define PROpacity 67
+#define PRFillOpacity 68
+#define PRStrokeOpacity 69
+#define PRFloat 70
+#define PRClear 71
+#define LAST_PRES_RULE_TYPE 71
+
+
+/*
+ * A bunch of constants describing the most common values for presentation
+ * attributes, like font families, styles, etc ...
+ */
+
+/* Units */
+#define UNIT_INVALID	0	/* invalid unit : for parsing   */
+#define UNIT_REL	1	/* relative : e.g. to font size */
+#define UNIT_PT		2	/* typo points : 1/72 inches    */
+#define UNIT_PC		3	/* typo points : 1/6 inches     */
+#define UNIT_IN		4	/* inches                       */
+#define UNIT_CM		5	/* centimeters                  */
+#define UNIT_MM		6	/* millimeters                  */
+#define UNIT_EM		7	/* defined / size of char m     */
+#define UNIT_PX		8	/* defined in pixels            */
+#define UNIT_PERCENT	9	/* a relative size in percent   */
+#define UNIT_XHEIGHT	10	/* defined / size of x char     */
+#define UNIT_BOX	11	/* this is a box number         */
+#define VALUE_AUTO      12      /* not a unit: value = auto     */
+
+/* values for rules PRBorderStyle */
+#define BorderStyleNone 1
+#define BorderStyleHidden 2
+#define BorderStyleDotted 3
+#define BorderStyleDashed 4
+#define BorderStyleSolid 5
+#define BorderStyleDouble 6
+#define BorderStyleGroove 7
+#define BorderStyleRidge 8
+#define BorderStyleInset 9
+#define BorderStyleOutset 10
+
+/* values for rules PRFloat */
+#define FloatNone          1
+#define FloatLeft          2
+#define FloatRight	   3
+
+/* values for rules PRClear */
+#define ClearNone          1
+#define ClearLeft          2
+#define ClearRight	   3
+#define ClearBoth	   4
+
+/* values for PRHorizPos and PRVertPos */
+#define PositionTop 1
+#define PositionBottom 2
+#define PositionLeft 3
+#define PositionRight 4
+#define PositionHorizRef 5
+#define PositionVertRef 6
+#define PositionHorizMiddle 7
+#define PositionVertMiddle 8
 
 /* values for rule PRStyle */
 #define StyleRoman 1
@@ -91,6 +152,13 @@ typedef int        *PRule;
 #define FontTimes 1
 #define FontHelvetica 2
 #define FontCourier 3
+#define FontOther 4
+
+#define REALSIZE		1
+#define SCALE			2
+#define REPEAT			3
+#define XREPEAT			4
+#define YREPEAT			5
 
 /* values for rule PRUnderline */
 #define NoUnderline 1
@@ -106,19 +174,47 @@ typedef int        *PRule;
 #define AdjustLeft 1
 #define AdjustRight 2
 #define Centered 3
-#define LeftWithDots 4
-#define Justify 5
+#define Justify 4
+#define LeftWithDots 5
+
+/* values for rule PRDirection */
+#define LeftToRight 1
+#define RightToLeft 2
+
+/* values for rule PRUnicodeBidi */
+#define Normal 1
+#define Embed 2
+#define Override 3
 
 /* values for rule PRLineStyle */
 #define SolidLine 1
 #define DashedLine 2
 #define DottedLine 3
 
+/* values for rule PRDisplay */
+#define Undefined 1
+#define Inline 2
+#define Block 3
+#define ListItem 4
+#define RunIn 5
+#define Compact 6
+#define Marker 7
 
 /* values for rule PRHyphenate */
 #define Hyphenation 1
 #define NoHyphenation 2
 
+#define PageAuto			0
+#define PageAlways			1
+#define PageAvoid			2
+#define PageLeft			3
+#define PageRight			4
+#define PageInherit			5
+
+/* keep this in sync with $THOTDIR/config/thot.pattern */
+#define PATTERN_NONE		0
+#define PATTERN_BACKGROUND	2
+
 #ifndef __CEXTRACT__
 
 
@@ -137,127 +233,109 @@ extern void TtaSetFontZoom (int zoom);
    Parameter:
    presentationType: type of the presentation rule to be created. Available
    values are PRSize, PRStyle, PRWeight, PRFont, PRUnderline, PRThickness,
-   PRIndent, PRLineSpacing, PRDepth, PRAdjust, PRLineStyle,
-   PRLineWeight, PRFillPattern, PRBackground, PRForeground, PRHyphenate,
-   PRWidth, PRHeight, PRVertPos, PRHorizPos.
+   PRIndent, PRLineSpacing, PRDepth, PRAdjust, PRLineStyle, PRDirection,
+   PRUnicodeBidi, PRLineWeight, PRFillPattern, PRBackground, PRForeground,
+   PRHyphenate, PRWidth, PRHeight, PRVertPos, PRHorizPos.
    view: the view (this view must be open).
    document: the document.
-
    Return value:
    new presentation rule.
-
   ----------------------------------------------------------------------*/
-extern PRule        TtaNewPRule (int presentationType, View view, Document document);
+extern PRule TtaNewPRule (int presentationType, View view, Document document);
 
 /*----------------------------------------------------------------------
    TtaNewPRuleForView
 
    Creates a new presentation rule of a given type for a given view of a given
    document. The view is identified by its number.
-
    Parameter:
    presentationType: type of the presentation rule to be created. Available
    values are PRSize, PRStyle, PRWeight, PRFont, PRUnderline, PRThickness,
-   PRIndent, PRLineSpacing, PRDepth, PRAdjust, PRLineStyle,
-   PRLineWeight, PRFillPattern, PRBackground, PRForeground, PRHyphenate,
-   PRShowBox, PRNotInLine.
+   PRIndent, PRLineSpacing, PRDepth, PRAdjust, PRLineStyle, PRDirection,
+   PRUnicodeBidi, PRLineWeight, PRFillPattern, PRBackground, PRForeground,
+   PRHyphenate, PRShowBox, PRNotInLine.
    viewName: the name of the view (this view does not need to be open).
    document: the document.
-
    Return value:
    new presentation rule.
-
   ----------------------------------------------------------------------*/
-extern PRule        TtaNewPRuleForView (int presentationType, int view, Document document);
+extern PRule TtaNewPRuleForView (int presentationType, int view, Document document);
 
 /*----------------------------------------------------------------------
    TtaNewPRuleForNamedView
 
    Creates a new presentation rule of a given type for a given view of a given
    document. The view is identified by its name.
-
    Parameter:
    presentationType: type of the presentation rule to be created. Available
    values are PRSize, PRStyle, PRWeight, PRFont, PRUnderline, PRThickness,
-   PRIndent, PRLineSpacing, PRDepth, PRAdjust, PRLineStyle,
-   PRLineWeight, PRFillPattern, PRBackground, PRForeground, PRHyphenate,
-   PRShowBox, PRNotInLine.
+   PRIndent, PRLineSpacing, PRDepth, PRAdjust, PRLineStyle, PRDirection,
+   PrUnicodeBidi, PRLineWeight, PRFillPattern, PRBackground, PRForeground,
+   PRHyphenate, PRShowBox, PRNotInLine.
    viewName: the name of the view (this view does not need to be open).
    document: the document.
-
    Return value:
    new presentation rule.
-
   ----------------------------------------------------------------------*/
-extern PRule        TtaNewPRuleForNamedView (int presentationType, STRING viewName, Document document);
+extern PRule TtaNewPRuleForNamedView (int presentationType, char *viewName,
+				      Document document);
 
 /*----------------------------------------------------------------------
    TtaCopyPRule
 
    Creates a new presentation rule and initializes it with a copy of an existing
    presentation rule.
-
    Parameter:
    pRule: presentation rule to be copied.
-
    Return value:
    new presentation rule.
-
   ----------------------------------------------------------------------*/
-extern PRule        TtaCopyPRule (PRule pRule);
+extern PRule TtaCopyPRule (PRule pRule);
 
 /*----------------------------------------------------------------------
    TtaAttachPRule
 
    Attaches a presentation rule to an element.
-
    Parameters:
    element: the element to which the presentation rule
    has to be attached.
    pRule: the presentation rule to be attached.
    document: the document to which the element belongs.
-
   ----------------------------------------------------------------------*/
-extern void         TtaAttachPRule (Element element, PRule pRule, Document document);
+extern void TtaAttachPRule (Element element, PRule pRule, Document document);
 
 /*----------------------------------------------------------------------
    TtaIsCSSPRule
 
    Check whether a presentation rule is associated with a CSS rule
-
    Parameters:
    pRule: the presentation rule to be tested.
-
    Return value:
    TRUE if pRule is a CSS rule
   ----------------------------------------------------------------------*/
-extern ThotBool    TtaIsCSSPRule (PRule pRule);
+extern ThotBool TtaIsCSSPRule (PRule pRule);
 
 /*----------------------------------------------------------------------
    TtaRemovePRule
 
    Removes a presentation rule from an element and release that rule.
-
    Parameters:
    element: the element with which the presentation rule is associated.
    pRule: the presentation rule to be removed.
    document: the document to which the element belongs.
-
   ----------------------------------------------------------------------*/
-extern void         TtaRemovePRule (Element element, PRule pRule, Document document);
+extern void TtaRemovePRule (Element element, PRule pRule, Document document);
 
 /*----------------------------------------------------------------------
    TtaSetPRuleValue
 
    Changes the value of a presentation rule. The presentation rule must
    be attached to an element that is part of a document.
-
    Parameters:
    element: the element to which the presentation rule is attached.
    pRule: the presentation rule to be changed.
    value: the value to be set.
    document: the document to which the element belongs.
-
    Valid values according to rule type:
    PRDepth: an integer (z-index of the element).
    PRVisibility: an integer >= 0 (visibility level).
@@ -272,8 +350,10 @@ extern void         TtaRemovePRule (Element element, PRule pRule, Document docum
    PRWeight: WeightNormal, WeightBold.
    PRUnderline: NoUnderline, Underline, Overline, CrossOut.
    PRThickness: ThinUnderline, ThickUnderline.
+   PRDirection: LeftToRight, RightToLeft.
+   PRUnicodeBidi: Normal, Embed, Override.
    PRLineStyle, PRBorderTopStyle, PRBorderRightStyle, PRBorderBottomStyle,
-      PRBorderLeftStyle: SolidLine, DashedLine, DottedLine.
+   PRBorderLeftStyle: SolidLine, DashedLine, DottedLine.
    PRNoBreak1, PRNoBreak2: a positive integer (distance in points).
    PRIndent: a positive, null or negative integer (indentation in points).
    PRSize: an integer between 6 and 72 (body size in points).
@@ -288,9 +368,9 @@ extern void         TtaRemovePRule (Element element, PRule pRule, Document docum
       an integer > 0 (border width in points).
    PRHyphenate: Hyphenation, NoHyphenation.
    PRAdjust: AdjustLeft, AdjustRight, Centered, LeftWithDots, Justify.
-
   ----------------------------------------------------------------------*/
-extern void         TtaSetPRuleValue (Element element, PRule pRule, int value, Document document);
+extern void TtaSetPRuleValue (Element element, PRule pRule, int value,
+			      Document document);
 
 /*----------------------------------------------------------------------
    TtaSetPRuleValueWithUnit
@@ -298,14 +378,12 @@ extern void         TtaSetPRuleValue (Element element, PRule pRule, int value, D
    Changes the value and the length unit of a presentation rule.
    The presentation rule must be attached to an element that is part of
    a document.
-
    Parameters:
    element: the element to which the presentation rule is attached.
    pRule: the presentation rule to be changed.
    value: the value to be set.
    unit: the length unit in which the value is expressed.
    document: the document to which the element belongs.
-
    Valid values according to rule type:
    PRNoBreak1, PRNoBreak2: a positive integer (distance).
    PRIndent: a positive, null or negative integer (indentation).
@@ -318,29 +396,42 @@ extern void         TtaSetPRuleValue (Element element, PRule pRule, int value, D
       > 0 (padding).
    PRBorderTopWidth, PRBorderRightWidth, PRBorderBottomWidth, PRBorderLeftWidth:
       an integer > 0 (border width).
-
   ----------------------------------------------------------------------*/
-extern void         TtaSetPRuleValueWithUnit (Element element, PRule pRule, int value, TypeUnit unit, Document document);
+extern void TtaSetPRuleValueWithUnit (Element element, PRule pRule, int value,
+				      TypeUnit unit, Document document);
+
+/*----------------------------------------------------------------------
+   TtaSetPositionPRuleDelta
+
+   Changes the delta value and the unit of a position presentation rule.
+   The presentation rule must be attached to an element that is part of
+   a document.
+   Parameters:
+   element: the element to which the presentation rule is attached.
+   pRule: the presentation rule to be changed. It must be a position rule.
+   delta: the value to be set.
+   unit: the unit in which the delta value is expressed.
+   document: the document to which the element belongs.
+  ----------------------------------------------------------------------*/
+extern void TtaSetPositionPRuleDelta (Element element, PRule pRule, int delta,
+				      TypeUnit unit, Document document);
 
 /*----------------------------------------------------------------------
    TtaSetPRuleView
 
    Sets the view to which a presentation rule applies. The presentation rule
    must not be attached yet to an element.
-
    Parameters:
    pRule: the presentation rule to be changed.
    view: the value to be set.
-
   ----------------------------------------------------------------------*/
-extern void         TtaSetPRuleView (PRule pRule, int view);
+extern void TtaSetPRuleView (PRule pRule, int view);
 
 /*----------------------------------------------------------------------
    TtaChangeBoxSize
 
    Changes the height and width of the box corresponding to an element in
    a given view.
-
    Parameters:
    element: the element of interest.
    document: the document to which the element belongs.
@@ -348,15 +439,13 @@ extern void         TtaSetPRuleView (PRule pRule, int view);
    deltaX: width increment in units (positive, negative or zero).
    deltaY: height increment in units (positive, negative or zero).
    unit: the unit used for the values.
-
   ----------------------------------------------------------------------*/
-extern void         TtaChangeBoxSize (Element element, Document document, View view, int deltaX, int deltaY, TypeUnit unit);
+extern void TtaChangeBoxSize (Element element, Document document, View view, int deltaX, int deltaY, TypeUnit unit);
 
 /*----------------------------------------------------------------------
    TtaChangeBoxPosition
 
    Moves the box of an element in a given view.
-
    Parameters:
    element: the element of interest.
    document: the document to which the element belongs.
@@ -364,9 +453,8 @@ extern void         TtaChangeBoxSize (Element element, Document document, View v
    deltaX: horizontal shift in units (positive, negative or zero).
    deltaY: vertical shift in units (positive, negative or zero).
    unit: the unit used for the values.
-
   ----------------------------------------------------------------------*/
-extern void         TtaChangeBoxPosition (Element element, Document document, View view, int X, int Y, TypeUnit unit);
+extern void TtaChangeBoxPosition (Element element, Document document, View view, int X, int Y, TypeUnit unit);
 
 /*----------------------------------------------------------------------
    TtaGetDepth
@@ -377,25 +465,22 @@ extern void         TtaChangeBoxPosition (Element element, Document document, Vi
    document: the document of interest.
    view: the view.
   ----------------------------------------------------------------------*/
-extern int          TtaGetDepth (Element element, Document document, View view);
+extern int TtaGetDepth (Element element, Document document, View view);
 
 /*----------------------------------------------------------------------
    TtaGiveBoxSize
 
    Returns the height and width of the box corresponding to an element in
    a given view.
-
    Parameters:
    element: the element of interest.
    view: the view.
    unit: the unit used for the values.
-
    Return parameters:
    width: box width in units.
    height: box height in units.
-
   ----------------------------------------------------------------------*/
-extern void         TtaGiveBoxSize (Element element, Document document, View view, TypeUnit unit, /*OUT*/ int *width, /*OUT*/ int *height);
+extern void TtaGiveBoxSize (Element element, Document document, View view, TypeUnit unit, /*OUT*/ int *width, /*OUT*/ int *height);
 
 /*----------------------------------------------------------------------
    TtaGiveBoxPosition
@@ -404,20 +489,17 @@ extern void         TtaGiveBoxSize (Element element, Document document, View vie
    a given view. The returned coordinates indicate the distance in points
    between the upper left corner of the box and the upper left corner of its
    parent box.
-
    Parameters:
    element: the element of interest.
    view: the view.
    unit: the unit used for the values.
-
    Return parameters:
    xCoord: distance from the left edge of the parent box to the left
    edge of the box, in units.
    yCoord:  distance from the upper edge of the parent box to the upper
    edge of the box, in units.
-
   ----------------------------------------------------------------------*/
-extern void         TtaGiveBoxPosition (Element element, Document document, View view, TypeUnit unit, /*OUT*/ int *xCoord, /*OUT*/ int *yCoord);
+extern void TtaGiveBoxPosition (Element element, Document document, View view, TypeUnit unit, /*OUT*/ int *xCoord, /*OUT*/ int *yCoord);
 
 /*----------------------------------------------------------------------
    TtaGiveBoxAbsPosition
@@ -426,43 +508,37 @@ extern void         TtaGiveBoxPosition (Element element, Document document, View
    a given view. The returned coordinates indicate the distance
    between the upper left corner of the box and the upper left corner of its
    window.
- 
    Parameters:
    element: the element of interest.
    document: the document of interest.
    view: the view.
    unit: the unit used for the values.
- 
    Return parameters:
    xCoord: distance from the left edge of the window to the left
    edge of the box.
    yCoord:  distance from the upper edge of the window to the upper
    edge of the box.
- 
   ----------------------------------------------------------------------*/
-extern void         TtaGiveBoxAbsPosition (Element element, Document document, View view, TypeUnit unit, int *xCoord, int *yCoord);
+extern void TtaGiveBoxAbsPosition (Element element, Document document, View view, TypeUnit unit, int *xCoord, int *yCoord);
 
 /*----------------------------------------------------------------------
    TtaGiveWindowSize
  
    Returns the height and width of the window corresponding to a given view.
- 
    Parameters:
    document: the document of interest.
    view: the view.
    unit: the unit used for the values (UnPixel or UnPoint only)
- 
    Return parameters:
    width: window width in units.
    height: window height in units.
- 
   ----------------------------------------------------------------------*/
-extern void        TtaGiveWindowSize (Document document, View view, TypeUnit unit, int *width, int *height);
+extern void TtaGiveWindowSize (Document document, View view, TypeUnit unit, int *width, int *height);
 
 /*----------------------------------------------------------------------
  *   TtaGiveRGB returns the RGB of the color.
  ----------------------------------------------------------------------*/
-extern CHAR_T      *TtaGiveRGB ( CHAR_T* colname, /*OUT*/ unsigned short *red, /*OUT*/ unsigned short *green, /*OUT*/ unsigned short *blue );
+extern char *TtaGiveRGB (char *colname, /*OUT*/ unsigned short *red, /*OUT*/ unsigned short *green, /*OUT*/ unsigned short *blue );
 
 /*----------------------------------------------------------------------
    TtaGiveThotRGB returns the Red Green and Blue values corresponding
@@ -470,7 +546,7 @@ extern CHAR_T      *TtaGiveRGB ( CHAR_T* colname, /*OUT*/ unsigned short *red, /
    If the color doesn't exist the function returns the values
    for the default color.
   ----------------------------------------------------------------------*/
-extern void         TtaGiveThotRGB (int num, /*OUT*/ unsigned short *red, /*OUT*/ unsigned short *green, /*OUT*/ unsigned short *blue);
+extern void TtaGiveThotRGB (int num, /*OUT*/ unsigned short *red, /*OUT*/ unsigned short *green, /*OUT*/ unsigned short *blue);
 
 /*----------------------------------------------------------------------
    TtaNextPRule
@@ -478,64 +554,53 @@ extern void         TtaGiveThotRGB (int num, /*OUT*/ unsigned short *red, /*OUT*
    Returns the first presentation rule associated with a given
    element (if pRule is NULL) or the presentation rule that
    follows a given rule of a given element.
-
    Parameters:
    element: the element of interest.
    pRule: a presentation rule of that element, or NULL
    if the first rule is asked.
-
    Return parameter:
    pRule: the next presentation rule, or NULL if
    pRule is the last rule of the element.
-
   ----------------------------------------------------------------------*/
-extern void         TtaNextPRule (Element element, /*INOUT*/ PRule * pRule);
+extern void TtaNextPRule (Element element, /*INOUT*/ PRule * pRule);
 
 /*----------------------------------------------------------------------
    TtaGetPRule
 
    Returns a presentation rule of a given type associated with a given element.
- 
    Parameters:
    element: the element of interest.
    presentationType: type of the desired presentation rule. Available
    values are PRSize, PtStyle, PtWeight, PRFont, PRUnderline, PRThickness,
-   PRIndent, PRLineSpacing, PRDepth, PRAdjust, PRLineStyle,
-   PRLineWeight, PRFillPattern, PRBackground, PRForeground, PRHyphenate,
-   PRShowBox, PRNotInLine.
- 
+   PRIndent, PRLineSpacing, PRDepth, PRAdjust, PRLineStyle, PRDirection,
+   PRUnicodeBidi, PRLineWeight, PRFillPattern, PRBackground, PRForeground,
+   PRHyphenate, PRShowBox, PRNotInLine.
    Return value:
    the presentation rule found, or NULL if the element
    does not have this type of presentation rule.
- 
   ----------------------------------------------------------------------*/
-extern PRule        TtaGetPRule (Element element, int presentationType);
+extern PRule TtaGetPRule (Element element, int presentationType);
 
 /*----------------------------------------------------------------------
    TtaGetPRuleType
 
    Returns the type of a presentation rule.
-
    Parameters:
    pRule: the presentation rule of interest.
-
    Return value:
-   type of that presentation rule. Available values are RSize, PtStyle,
+   type of that presentation rule. Available values are RSize, RStyle,
    RFont, RUnderline, RThickness, PRIndent, RLineSpacing, RDepth, RAdjust,
-   RLineStyle, RLineWeight, RFillPattern, RBackground,
-   RForeground, RHyphenate, PRShowBox, PRNotInLine.
-
+   RLineStyle, RDirection, RUnicodeBidi, RLineWeight, RFillPattern,
+   RBackground, RForeground, RHyphenate, PRShowBox, PRNotInLine.
   ----------------------------------------------------------------------*/
-extern int          TtaGetPRuleType (PRule pRule);
+extern int TtaGetPRuleType (PRule pRule);
 
 /*----------------------------------------------------------------------
    TtaGetPRuleValue
 
    Returns the value of a presentation rule.
-
    Parameters:
    pRule: the presentation rule of interest.
-
    Return values according to rule type:
    PRVisibility: an integer >= 0 (visibility level).
    PRDepth: an integer (z-index of the element).
@@ -550,6 +615,8 @@ extern int          TtaGetPRuleType (PRule pRule);
    PRWeight: WeightNormal, WeightBold.
    PRUnderline: NoUnderline, Underline, Overline, CrossOut.
    PRThickness: ThinUnderline, ThickUnderline.
+   PRDirection: LeftToRight, RightToLeft.
+   PRUnicodeBidi: Normal, Embed, Override.
    PRLineStyle, PRBorderTopStyle, PRBorderRightStyle, PRBorderBottomStyle,
       PRBorderLeftStyle: SolidLine, DashedLine, DottedLine.
    PRNoBreak1, PRNoBreak2: a positive integer (distance in points).
@@ -565,55 +632,58 @@ extern int          TtaGetPRuleType (PRule pRule);
       an integer > 0 (border width in points).
    PRHyphenate: Hyphenation, NoHyphenation.
    PRAdjust: AdjustLeft, AdjustRight, Centered, LeftWithDots, Justify.
-
   ----------------------------------------------------------------------*/
-extern int          TtaGetPRuleValue (PRule pRule);
+extern int TtaGetPRuleValue (PRule pRule);
+
+/*----------------------------------------------------------------------
+   TtaGetPositionPRuleDelta
+
+   Returns the delta value of a position rule.
+
+   Parameters:
+   pRule: the presentation rule of interest.
+
+   Return:
+   delta value.
+  ----------------------------------------------------------------------*/
+extern int TtaGetPositionPRuleDelta (PRule pRule);
 
 /*----------------------------------------------------------------------
    TtaGetPRuleUnit
 
    Returns the unit of a presentation rule.
-
    Parameters:
    pRule: the presentation rule of interest.
-
    Return the unit of the rule type PRSize, PRIndent, PRLineSpacing, PRLineWeight,
    PRWidth, PRHeight, PRVertPos, PRHorizPos.
    This unit could be UnRelative, UnXHeight, UnPoint, UnPixel, UnPercent.
    Return UnRelative in other cases.
-
   ----------------------------------------------------------------------*/
-extern int         TtaGetPRuleUnit (PRule pRule);
+extern int TtaGetPRuleUnit (PRule pRule);
 
 /*----------------------------------------------------------------------
    TtaGetPRuleView
 
    Returns the number of the view to which a presentation rule applies.
-
    Parameters:
        pRule: the presentation rule of interest.
-
    Return value:
        number of the view to which the presentation rule applies.
-
   ----------------------------------------------------------------------*/
-extern int          TtaGetPRuleView (PRule pRule);
+extern int TtaGetPRuleView (PRule pRule);
 
 /*----------------------------------------------------------------------
    TtaSamePRules
 
    Compares two presentation rules associated with the same element or with
    different elements.
-
    Parameters:
    pRule1: first presentation rule to be compared.
    pRule2: second presentation rule to be compared.
-
    Return value:
    0 if both rules are different, 1 if they are identical.
-
   ----------------------------------------------------------------------*/
-extern int          TtaSamePRules (PRule pRule1, PRule pRule2);
+extern int TtaSamePRules (PRule pRule1, PRule pRule2);
 
 #endif /* __CEXTRACT__ */
 
