@@ -30,15 +30,20 @@ and must be removed at the end of the debug */
 #include <signal.h>
 #include <math.h>
 
+#ifndef M_PI
 #define M_PI            3.14159265358979323846  /* pi */
 #define M_PI_2          1.57079632679489661923  /* pi/2 */
+#endif
 /*
   A charater that starts an entity that Amaya cannot convert.
   It should be displayed as a & and should be exported as &
   instead of &amp;
 */
-#define START_ENTITY 26
-
+#define START_ENTITY        26
+/*
+  A character Unicode that cannot be displayed
+*/
+#define UNDISPLAYED_UNICODE 27
 /**********************************************************/
 #if defined(_WINDOWS) || defined(_CONSOLE)
 
@@ -114,8 +119,12 @@ and must be removed at the end of the debug */
 typedef char       *caddr_t;	/* may be TCHAR for UNICODE */
 /* added functions */
 void bzero (void *s, size_t n);
+#ifndef _WX /* SG there is warnings if I define this function */
 int _getpid (void);
+#endif /* _WX */
+#ifdef __STDC__ // allready defined if __STDC__ is not defined
 #define isascii(c) __isascii(c)
+#endif /* #ifdef __STDC__ */
 #define strcasecmp(a, b) _stricmp(a, b)
 #define index(str, ch) strchr(str, ch)
 /* function mappings */
@@ -124,6 +133,7 @@ int _getpid (void);
 #define getcwd(buffer, len) _getcwd(buffer, len)
 #define access(f,m) _access((f),(m))
 #define unlink(f) _unlink((f))
+#define snprintf _snprintf
 
 /* Constants for PATHs */
 #define DIR_SEP  '\\'
@@ -170,9 +180,14 @@ typedef BOOL        ThotBool;
 
 #define Dimension int
 
-#else /* _WINDOWS *//***********************************WINDOWS**/
+#endif /* _WINDOWS *//***********************************WINDOWS**/
+
+#if defined(_UNIX) || defined(MOTIF) || defined(_GTK) || defined(_NOGUI)
+
 /* Unix definitions */
+#define None	    0L	/* X11/X.h */
 typedef unsigned char   ThotBool;
+typedef ThotBool BOOL;
 #define ThotPid_get()	getpid()
 #define ThotPid		pid_t
 
@@ -187,14 +202,9 @@ typedef unsigned char   ThotBool;
 #define WC_DIR_STR  "/"
 #define WC_PATH_STR ":"
 
-#endif /* _WINDOWS */
-/********************************************************WINDOWS**/
-#ifdef _I18N_
-#define ___TEXT___(str) L##str
-#else  /* !_I18N_ */
-#define ___TEXT___(str) str
-#endif /* _I18N_ */
+#endif /* #if defined(_UNIX) || defined(_MOTIF) || defined(_GTK) */
 
+#define ___TEXT___(str) L##str
 #ifndef TEXT
 #define TEXT(str) ___TEXT___(str)
 #endif  /* TEXT */
@@ -206,15 +216,7 @@ typedef unsigned char   ThotBool;
 #define SPACE   ' '
 #define BSPACE  '\b'
 
-#define WC_CR     '\r'
-#define WC_EOS    '\0'
-#define WC_EOL    '\n'
-#define WC_TAB    '\t'
-#define WC_SPACE  ' '
-#define WC_BSPACE '\b'
-
 #define NBSP      0xA0
-
 #define MAX_BYTES 6 
 
 #endif /* THOT_SYS_H */
