@@ -2,10 +2,9 @@
 #define _USTRING_H
 
 #include <string.h>
-#ifdef _WINDOWS
+#ifdef _WINGUI
 #include <windows.h>
-#endif /* _WINDOWS */
-#include <wchar.h>
+#endif /* _WINGUI */
 
 #ifdef _WINDOWS
 #define strncasecmp _strnicmp
@@ -20,16 +19,12 @@ typedef enum {
    ISO_8859_4, 
    ISO_8859_5,
    ISO_8859_6, 
-   ISO_8859_6_E,
-   ISO_8859_6_I,
    ISO_8859_7, 
    ISO_8859_8,
-   ISO_8859_8_E,
-   ISO_8859_8_I,
    ISO_8859_9, 
    ISO_8859_10,
+   ISO_8859_13,
    ISO_8859_15,
-   ISO_8859_supp,
    UNICODE_1_1,
    UNICODE_1_1_UTF_7,
    UTF_7,
@@ -43,36 +38,41 @@ typedef enum {
    WINDOWS_1255,
    WINDOWS_1256,
    WINDOWS_1257,
-   WINDOWS_1258
+   WINDOWS_1258,
+   ISO_SYMBOL,
+   KOI8_R,
+   MAC_OS_ROMAN,
+   ISO_2022_JP,
+   EUC_JP,
+   SHIFT_JIS,
+   JIS_X_0201_ROMAN,
+   JIS_X_0201_KANA,
+   JIS_X_0208,
+   JIS_X_0212,
+   GB_2312,
+   KSC_5601,
+   UNSUPPORTED_CHARSET
 } CHARSET;
 
-#ifdef _I18N_
+#include <wchar.h>
 typedef wchar_t  CHAR_T;
-typedef wchar_t *PCHAR_T;
-typedef wchar_t  UCHAR_T;
-typedef wchar_t *PUCHAR_T;
 typedef wchar_t *STRING;
-typedef wchar_t *USTRING;
-#else  /* _I18N_ */
-typedef char            CHAR_T;
-typedef char           *PCHAR_T;
-typedef unsigned char   UCHAR_T;
-typedef unsigned char  *PUCHAR_T;
-typedef char           *STRING;
-typedef unsigned char  *USTRING;
-#endif /* _I18N_ */
 
 /*----------------------------------------------------------------------
   TtaGetCharset gives the charset 
   ----------------------------------------------------------------------*/
 extern CHARSET TtaGetCharset (char *charsetname);
 
+/*----------------------------------------------------------------------
+  TtaGetDefaultCharset gives the default charset 
+  ----------------------------------------------------------------------*/
+extern CHARSET TtaGetDefaultCharset ();
+
 /*----------------------------------------------------------------------
   TtaGetCharsetName gives the constant string of the charset ISO name.
   ----------------------------------------------------------------------*/
 extern char *TtaGetCharsetName (CHARSET charset);
 
-extern int uputchar (int);
 extern int ustrcasecmp (const CHAR_T *, const CHAR_T *);
 extern CHAR_T *ustrcat (CHAR_T *, const CHAR_T *);
 extern CHAR_T *ustrchr (const CHAR_T *, CHAR_T);
