@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -15,25 +15,23 @@
 
 #include "thot_sys.h"
 
-#ifdef _WINDOWS
+#ifdef _WINGUI
 
 #define MAX_NATURES 7		/* max. number of simultaneous natures */
 #define MAX_VIEW_DOC 6		/* max. number of views for a document */
 #define MAX_SEL_ELEM 150	/* max. number of elements that can be selected
 				   simultaneously */
 #define MAX_VIEW_OPEN 50	/* max. number of open views for all documents */
-#else /* _WINDOWS ---------------------------------- */
+#else /* _WINGUI ---------------------------------- */
 
 #define MAX_NATURES 7		/* max. number of simultaneous natures */
 #define MAX_VIEW_DOC 6		/* max. number of views for a document */
 #define MAX_SEL_ELEM 150	/* max. number of elements that can be selected
 				   simultaneously */
 #define MAX_VIEW_OPEN 50	/* max. number of open views for all documents */
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
 
 #define THOT_MAX_CHAR 100	/* max. number of character in a text buffer */
-#define DEF_SAVE_INTVL 500	/* number of typed characters triggering 
-				   automatic saving */
 #define MAX_LABEL_LEN 12       	/* max. length of labels */
 #define MAX_DOC_IDENT_LEN 32	/* max. length of a document identifier */
 
