@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -55,7 +55,6 @@
 #define NumMenuSearchNature	43
 #define NumSelTypeToSearch	44
 #define NumSelAttributeToSearch	45
-#define NumLabelAttributeValue	46
 #define NumToggleUpperEqualLower       47
 #define NumZoneScrollUp      	48
 #define NumZoneScrollLeft  	49
@@ -96,19 +95,16 @@
 #define NumMenuAttrEnumNeeded 95 /* numero du menu des valeurs d'un */
 #define NumMenuCharacters	96
 #define NumFormPrint		97
-#define NumFormSearchPage	98
-#define NumZoneSearchPage	99
 #define NumZoneRecess		100
 #define NumMenuRecessSense	101
 #define NumZoneLineSpacing	102
 #define NumMenuLineSpacing	103
-#define NumMenuWordBreak	104
-#define NumFormPresGraphics	155
-#define NumMenuStrokeStyle	106
-#define NumZoneStrokeWeight	107
-#define NumSelectPattern	108
+#define NumFormPresGraphics	104
+#define NumMenuStrokeStyle	105
+#define NumZoneStrokeWeight	106
+#define NumSelectPattern	107
 
-#define MAX_LocalMenu		109
+#define MAX_LocalMenu		108
 
 /* Number of references reserved by Thot       				*/
 /* MAX_ITEM >= MAX_FRAME for X-Windows applications			*/
