@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -19,6 +19,29 @@
 #include "typecorr.h"
 #include "typetra.h"
 
+typedef struct _FontSet      *SpecFont;
+typedef struct _FontSet {
+  SpecFont      NextFontSet;
+  ThotBool      specificFont; /* TRUE when the specific font is used */
+  int           FontFamily;
+  int           FontHighlight;
+  int           FontSize;
+  unsigned int  FontMask;
+  ThotFont      FontIso_1;
+  ThotFont      FontIso_2;
+  ThotFont      FontIso_3;
+  ThotFont      FontIso_4;
+  ThotFont      FontIso_5;
+  ThotFont      FontIso_6;
+  ThotFont      FontIso_7;
+  ThotFont      FontIso_8;
+  ThotFont      FontIso_9;
+  ThotFont      FontIso_13;
+  ThotFont      FontIso_15;
+  ThotFont      FontSymbol; /* Symbol of Stix fonts ?? */
+  ThotFont      FontUnicode;
+} FontSet;
+
 typedef unsigned char RuleSet[32];
 
 /* List of pointers used */
@@ -29,17 +52,21 @@ typedef struct _Line *PtrLine;
 /* Type of a described box */
 typedef enum
 {
-  BoComplete, 
-  BoSplit, 
-  BoPiece, 
-  BoBlock, 
-  BoPicture, 
+  BoComplete,
+  BoSplit,
+  BoPiece,
+  BoMulScript,
+  BoScript,
+  BoBlock,
+  BoPicture,
   BoGhost,
   BoDotted,
   BoTable,
   BoRow,
   BoColumn,
-  BoCell
+  BoCell,
+  BoFloatBlock,
+  BoFloatGhost
 } BoxType;
 
 /* Type of relation between boxes:
@@ -48,24 +75,24 @@ typedef enum
    Ref(H, V) -> the box is bound to a reference axis
    Dim(H, V) -> the box modifies the elastic dimension of the other box */
 typedef enum
-{
-  OpHorizDep, 
-  OpVertDep,
-  OpHorizInc,
-  OpVertInc,
-  OpHorizRef,
-  OpVertRef,
-  OpWidth,
-  OpHeight
-} OpRelation;
+  {
+    OpHorizDep, 
+    OpVertDep,
+    OpHorizInc,
+    OpVertInc,
+    OpHorizRef,
+    OpVertRef,
+    OpWidth,
+    OpHeight
+  } OpRelation;
 
 /* Domain of propagation of the modifications between boxes */
 typedef enum
-{
-  ToSiblings,
-  ToChildren,
-  ToAll
-} Propagation;
+  {
+    ToSiblings,
+    ToChildren,
+    ToAll
+  } Propagation;
 
 typedef struct _ViewSelection
 {
@@ -97,7 +124,17 @@ typedef struct _Line
   PtrBox          LiFirstPiece;	/* Index if split box */
   PtrBox          LiLastBox;	/* Index of last contained box */
   PtrBox          LiLastPiece;	/* Index if split box */
-} Line;
+  ThotBool        LiNoOverlap;  /* The line cannot overlap */
+} ThotLine;
+
+/* Desription of a list of float */
+typedef struct _BFloat *PtrFloat;
+typedef struct _BFloat
+{
+  PtrFloat       FlPrevious;	/* Floats linking */
+  PtrFloat       FlNext;
+  PtrBox         FlBox;	        /* the floating box */
+} BFloat;
 
 typedef struct _BoxRelation
 {
@@ -160,7 +197,6 @@ typedef struct _Box
   int             BxYOrg;	/* Y origin from the root */
   int             BxHeight;	        /* Box height including margins */
   int             BxWidth;	        /* Box width including margins */
-
   int             BxH;	        	/* Inner height */
   int             BxW;	        	/* Inner Width including spaces */
   int             BxTMargin;	        /* Top Margin */
@@ -176,7 +212,18 @@ typedef struct _Box
   int             BxLPadding;	        /* Left Padding */
   int             BxBPadding;	        /* Bottom Padding */
   int             BxRPadding;	        /* Right Padding */
-
+#ifdef _GL
+  int             BxClipX; 
+  int             BxClipY; 
+  int             BxClipW;
+  int             BxClipH;
+  int             DisplayList;          /*Video Memory index of precomputed visual*/
+  void            *Pre_computed_Pic;    /*Pic of the rendered content, used
+					  for group, for example*/
+  void            *Post_computed_Pic;    /*Pic of the rendered content, used
+					  for group, for example*/
+#endif /*_GL*/
+  
   int             BxHorizRef;	        /* Current base */
   int             BxVertRef;	        /* Current vertical reference */
   int             BxEndOfBloc;	        /* Fill length:
@@ -187,7 +234,7 @@ typedef struct _Box
   PtrPosRelations BxPosRelations;	/* Dependencies on positions */
   PtrDimRelations BxWidthRelations;	/* Dependencies in width */
   PtrDimRelations BxHeightRelations;	/* Dependencies in height */
-  ptrfont         BxFont;	        /* Font bound to the box */
+  SpecFont        BxFont;	        /* Font bound to the box */
   PtrBox          BxMoved;	        /* Linking of moved boxes */
   PtrBox          BxHorizInc;	        /* Box linking to the enclosing one */
   PtrBox          BxVertInc;	        /* Box linking to the enclosing one */
@@ -207,8 +254,15 @@ typedef struct _Box
   ThotBool	  BxContentHeight;	/* Real height is the content height */
   ThotBool	  BxContentWidth;	/* Real width is the content width */
   ThotBool        BxShadow;		/* Characters are showed as '*' */
-  ThotBool        BxFill;	        /* The box has borders or background */
-  ThotBool        BxDisplay;	        /* The box has borders or background */
+  ThotBool        BxDisplay;	        /* The box has borders or
+					   background */
+#ifdef _GL
+    ThotBool        BxEditable;           /* If Box is Rotated or Scaled,
+					     prevents edition */
+    ThotBool        BxBoundinBoxComputed;
+    ThotBool        BxTransformationComputed;
+    ThotBool        VisibleModification;    
+#endif /* _GL */
   int		  BxRuleHeigth;         /* Content height or minimum */
   int		  BxRuleWidth;	        /* Content width or minimum */
   PtrTextBuffer   BxBuffer;	        /* Pointer on the buffer list */
@@ -222,6 +276,9 @@ typedef struct _Box
       int	 _BxNPixels_;	/* Number of pixels to share out */
       int	 _BxSpaceWidth_;/* >0 of the box is justified */
       int	 _BxFirstChar_;	/* First character in buffer */
+      char       _BxScript_;    /* Script of the current text */
+      char       _BxFirstScript_;/*Script of the first element*/
+      char       _BxLastScript_;/*Script of the last element*/
     } s0;
     struct /* BoPicture - used for pictures, polylines and paths */
     {
@@ -244,6 +301,8 @@ typedef struct _Box
       int        _BxMinWidth_;  /* Mininmum width */
       int        _BxCycles_;    /* count reformatting cycles */
       int        _BxPacking_;   /* Packing */
+      PtrFloat   _BxLeftFloat_; /* list of left floated boxes */
+      PtrFloat   _BxRightFloat_;/* list of right floated boxes */
     } s2;
     struct /* BoTable BoColumn BoRow */
     {
@@ -268,6 +327,11 @@ typedef struct _Box
 #define BxNPixels u.s0._BxNPixels_
 #define BxSpaceWidth u.s0._BxSpaceWidth_
 #define BxFirstChar u.s0._BxFirstChar_
+#define BxScript u.s0._BxScript_
+
+#define BxFirstScript u.s0._BxFirstScript_
+#define BxLastScript u.s0._BxLastScript_
+
 #define BxPictInfo u.s1.u._BxPictInfo_
 #define BxFirstPathSeg u.s1.u._BxFirstPathSeg_
 #define BxXRatio u.s1._BxXRatio_
@@ -278,6 +342,8 @@ typedef struct _Box
 #define BxMinWidth u.s2._BxMinWidth_
 #define BxCycles u.s2._BxCycles_
 #define BxPacking u.s2._BxPacking_
+#define BxLeftFloat u.s2._BxLeftFloat_
+#define BxRightFloat u.s2._BxRightFloat_
 #define BxColumns u.s3._BxColumns_
 #define BxTable u.s3._BxColumns_
 #define BxRows u.s3._BxRows_
@@ -295,9 +361,11 @@ typedef struct _AbPosition
 {
   PtrAbstractBox         PosAbRef;	/* Pointer of the reference abstract box */
   int             PosDistance;	/* Distance between the two of them */
+  int             PosDistDelta; /* Distance increment */
   BoxEdge         PosEdge;	/* Positioned abstract box mark */
   BoxEdge         PosRefEdge;	/* Reference abstract box mark */
   TypeUnit        PosUnit;	/* Unit used for distance */
+  TypeUnit        PosDeltaUnit; /* Unit used for the distance increment */
   ThotBool	  PosUserSpecified;	/* User specified distance */
 } AbPosition;
 
@@ -384,6 +452,7 @@ typedef struct _AbstractBox
   int             AbIndent;	/* Indentation for line breaking */
   int             AbDepth;	/* Abstract box display plane */
   int	          AbVisibility; /* Abstract box visibility degree */
+  int             AbFont;	/* Font family: index in the family table */
   int             AbFontStyle;	/* Font style: roman, italic, oblique */
   int             AbFontWeight;	/* Font weight: normal, bold */
   int             AbSize;	/* Character logical/real size */
@@ -392,7 +461,9 @@ typedef struct _AbstractBox
   int             AbFillPattern;/* Fill pattern */	
   int             AbBackground; /* Background color */
   int             AbForeground; /* Drawing color */
-
+  int             AbFillOpacity;    /* opacity of the fill part of element*/
+  int             AbStrokeOpacity;    /* opacity of the stroke part element*/
+  int             AbOpacity;    /* opacity of all the element*/
   int             AbTopBColor;     /* Border Top color */
   int             AbRightBColor;   /* Border Right color */
   int             AbBottomBColor;  /* Border Bottom color */
@@ -401,7 +472,6 @@ typedef struct _AbstractBox
   int             AbRightStyle;    /* Border Right style */
   int             AbBottomStyle;   /* Border Bottom style */
   int             AbLeftStyle;     /* Border Left style */
-
   int             AbTopMargin;     /* Top Margin */
   int             AbRightMargin;   /* Right Margin */
   int             AbBottomMargin;  /* Bottom Margin */
@@ -414,19 +484,22 @@ typedef struct _AbstractBox
   int             AbRightBorder;   /* Right Border */
   int             AbBottomBorder;  /* Bottom Border */
   int             AbLeftBorder;    /* Left Border */
-
-  char            AbFont;	      /* Characteristics of the font used */
+  char            AbDirection;        /* Writing direction */
+  char            AbUnicodeBidi;      /* Unicode Bidi */
   char            AbLineStyle;        /* Line style */
+  char            AbDisplay;          /* Display mode */
+  char            AbFloat;            /* Float mode */
+  char            AbClear;            /* Clear mode */
   BAlignment      AbAdjust;	      /* Alignment of lines in the box */
   TypeUnit        AbSizeUnit;         /* Unit for the size */
   TypeUnit        AbIndentUnit;       /* Indentation unit */
   TypeUnit        AbLineWeightUnit;   /* Unit of thickness */
   TypeUnit        AbLineSpacingUnit;  /* Interlining unit */
 
-  TypeUnit        AbTopMarginUnit;    /* Top Margin Unit */
-  TypeUnit        AbRightMarginUnit;  /* Right Margin Unit */
-  TypeUnit        AbBottomMarginUnit; /* Bottom Margin Unit */
-  TypeUnit        AbLeftMarginUnit;   /* Left Margin Unit */
+  TypeUnit        AbTopMarginUnit;    /* Top Margin Unit (includes auto) */
+  TypeUnit        AbRightMarginUnit;  /* Right Margin Unit (includes auto) */
+  TypeUnit        AbBottomMarginUnit; /* Bottom Margin Unit (includes auto) */
+  TypeUnit        AbLeftMarginUnit;   /* Left Margin Unit (includes auto) */
   TypeUnit        AbTopPaddingUnit;   /* Top Padding Unit */
   TypeUnit        AbRightPaddingUnit; /* Right Padding Unit */
   TypeUnit        AbBottomPaddingUnit;/* Bottom Padding Unit */
@@ -465,6 +538,7 @@ typedef struct _AbstractBox
   ThotBool        AbAspectChange;     /* Change the graphical aspect: plane,
 				         color, pattern, line style */
   ThotBool        AbMBPChange;	      /* Change margins, borders, paddings */
+  ThotBool        AbFloatChange;      /* Change the floatting information */
   ThotBool        AbChange;	      /* Change of another type */
   LeafType        AbLeafType;
   union
@@ -481,12 +555,12 @@ typedef struct _AbstractBox
     struct /* AbLeafType = LtText */
     {
       PtrTextBuffer _AbText_;     /* On first text buffer */
-      Language      _AbLanguage_; /* Language used */
+      Language      _AbLang_;     /* Language or script used */
     } s1;
     struct /* AbLeafType = LtGraphics or LtSymbol */
     {
       char     _AbShape_;         /* Drawing code */
-      char     _AbGraphAlphabet_; /* Alphabet used*/
+      char     _AbGraphScript_; /* Script used*/
       char     _AbRealShape_;     /* Effective drawing code */
       int      _AbRx_;            /* Horizontal radius */
       int      _AbRy_;            /* Vertical radius */     
@@ -515,9 +589,9 @@ typedef struct _AbstractBox
 #define AbTruncatedHead u.s0._AbTruncatedHead_
 #define AbTruncatedTail u.s0._AbTruncatedTail_
 #define AbText u.s1._AbText_
-#define AbLanguage u.s1._AbLanguage_
+#define AbLang u.s1._AbLang_
 #define AbShape u.s2._AbShape_
-#define AbGraphAlphabet u.s2._AbGraphAlphabet_
+#define AbGraphScript u.s2._AbGraphScript_
 #define AbRealShape u.s2._AbRealShape_
 #define AbRx u.s2._AbRx_
 #define AbRy u.s2._AbRy_
@@ -533,6 +607,10 @@ typedef struct _ViewFrame
   PtrAbstractBox         FrAbstractBox;	/* Pointer on the root abstract box of the View */
   int             FrXOrg;	/* X origin from root */
   int             FrYOrg;       /* Y origin from root */
+#ifdef _GL
+  int             OldFrXOrg;	   /* X origin from root for not opengl computed boxes*/
+  int             OldFrYOrg;       /* Y origin from root for not opengl computed boxes*/
+#endif /*_GL*/
   int             FrVolume;	/* Character volume in the window */
   int             FrClipXBegin;	/* X beginning of the display rectangle */
   int             FrClipXEnd;	/* X ending of the display rectangle */
