@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -61,10 +61,10 @@ typedef enum
         CharString,
         GraphicElem,
         Symbol,
-        Picture,
+        tt_Picture,
         Refer,
         PageBreak,
-        UnusedBasicType
+        AnyType
 } BasicType;
 #define MAX_BASIC_TYPE 7        /* number of values for BasicType */
  
@@ -83,9 +83,9 @@ typedef enum
  
 typedef struct _TtAttribute
 {
-        Name       AttrName;          /* name of the attribute, may be
+        char      *AttrName;          /* name of the attribute, may be
                                          translated in the user's language */
-        Name       AttrOrigName;      /* real name of the attribute */
+        char      *AttrOrigName;      /* real name of the attribute */
         ThotBool   AttrGlobal;        /* the attribute can apply to all
                                          the elements defined in the schema */
         int        AttrFirstExcept;   /* index in SsException of the first
@@ -127,28 +127,44 @@ typedef struct _TtAttribute
 #define AttrEnumValue u.s3._AttrEnumValue_
 #define AttrEnumOrigValue u.s3._AttrEnumOrigValue_
 
+typedef TtAttribute *PtrTtAttribute;
+
+typedef struct _TtAttrTable
+{
+        PtrTtAttribute TtAttr[1];
+} TtAttrTable;
+
+typedef struct _NumTable
+{
+  int           Num[1];
+} NumTable;
+
+typedef struct _BoolTable
+{
+  ThotBool      Bln[1];
+} BoolTable;
 
 typedef struct _StructSchema *PtrSSchema;
 
-/* A rule defining a type in a structure schema */
+/* A rule defining an element type in a structure schema */
 typedef struct _SRule
 {
-	Name       SrName;	    /* left-hand symbol of the rule =
-				       type defined by the rule */
-        Name       SrOrigName;      /* real name of the rule */
+	char      *SrName;	    /* left-hand symbol of the rule =
+				       element type defined by the rule */
+        char      *SrOrigName;      /* real name of the rule */
 	int 	   SrNDefAttrs;     /* 0..MAX_DEFAULT_ATTR, number of
 				       attributes with a default value */
-        /* numbers of default value attributes */
-	int        SrDefAttr[MAX_DEFAULT_ATTR];  
-        /* default values of these attributes, in the same order */ 
-	int        SrDefAttrValue[MAX_DEFAULT_ATTR];  
+	int        SrDefAttr[MAX_DEFAULT_ATTR]; /* default value attributes */
+	int        SrDefAttrValue[MAX_DEFAULT_ATTR];/* default values of these
+				       attributes, in the same order */
 	ThotBool   SrDefAttrModif[MAX_DEFAULT_ATTR];
-        /* 0..MAX_LOCAL_ATTR, number of attributes that can apply to the type*/
-	int 	   SrNLocalAttrs;
-        /* numbers of the attributes that can apply to the type */ 
-	int        SrLocalAttr[MAX_LOCAL_ATTR];  
-        /* the local attribute of same rank is mandatory */
-	ThotBool   SrRequiredAttr[MAX_LOCAL_ATTR];
+	int 	   SrNLocalAttrs;   /* number of attributes that can apply to
+				       the element type, i.e. size of tables
+				       SrLocalAttr and SrRequiredAttr */
+	NumTable*  SrLocalAttr;     /* attributes that can apply to the
+				       element type */ 
+	BoolTable* SrRequiredAttr;  /* indicate whether the local attribute of
+				       same rank is mandatory */
 	ThotBool   SrUnitElem;      /* it is an exported unit */
 	ThotBool   SrRecursive;     /* recursive rule */
 	ThotBool   SrRecursDone;    /* already applied recursive rule */
@@ -250,6 +266,13 @@ typedef struct _SRule
 #define SrIndexConst u.s7._SrIndexConst_
 #define SrFirstOfPair u.s8._SrFirstOfPair_
 
+typedef SRule *PtrSRule;
+
+typedef struct _SrRuleTable
+{
+        PtrSRule SrElem[1];
+} SrRuleTable;
+
 typedef struct _ExtensBlock *PtrExtensBlock;
 
 /* block containing all the extension rules (EXTENS section) of an
@@ -280,6 +303,7 @@ typedef struct _StructSchema
    Name             SsName;              /* generic structure name*/
    char            *SsUriName;           /* Pointer to the URI associated with
 					    this schema */
+   ThotBool         SsIsXml;             /* Indicates a generix XML schema */
    int              SsCode;              /* code to identify the version */
    Name             SsDefaultPSchema;    /* name of the default presentation 
 					    schema associated with this
@@ -297,10 +321,11 @@ typedef struct _StructSchema
    int              SsRootElem;          /* number of the root rule */
    int              SsNAttributes;       /* number of attributes in the
 					    schema */
+   int              SsAttrTableSize;     /* actual size of table SsAttribute */
    int              SsNRules;            /* current number of rules defining
 					    the elements, including the nature
-					    rules */
-                                         /* added dynamically */
+					    rules added dynamically */
+   int              SsRuleTableSize;     /* actual size of table SsRule */
    ThotBool         SsExport;            /* this schema exports elements */
    int              SsNExceptions;       /* number of entries in SsException */
                                          /* All the exception numbers
@@ -311,12 +336,12 @@ typedef struct _StructSchema
    char             SsConstBuffer[MAX_LEN_ALL_CONST];/* number of the rule
 					    defining the first dynamically
 					    loaded nature */
-   int              SsFirstDynNature;    /* attributes for this schema */
-   TtAttribute      SsAttribute[MAX_ATTR_SSCHEMA];   /* structure rules
-							defining the elements*/
-   SRule            SsRule[MAX_RULES_SSCHEMA + 2];   /* +2 to be sure to have
-					    two free rules at the end of the
-					    arrays */
+   int              SsFirstDynNature;
+
+   TtAttrTable      *SsAttribute;        /* attributes for this schema */
+
+   SrRuleTable      *SsRule;             /* structure rules defining the
+					    elements for this schema */
 } StructSchema;
 
 #endif /* _THOTLIB_TYPESTR_H_ */
