@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996.
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -15,37 +15,44 @@ THOT_EXPORT Proc	       ThotLocalActions[MAX_LOCAL_ACTIONS];
 
 #ifndef NODISPLAY
 
-#ifdef _WINDOWS
-#include <windows.h>
-#ifndef __GNUC__
-#include <commctrl.h>
-#endif
+#ifdef _WINGUI
+  #include <windows.h>
+  #ifndef __GNUC__
+    #include <commctrl.h>
+  #endif /* __GNUC__ */
+  THOT_EXPORT WNDCLASSEX         RootShell;
+  THOT_EXPORT HMENU              currentMenu;
+  THOT_EXPORT HINSTANCE          hInstance;
+  THOT_EXPORT int                currentFrame;
+  THOT_EXPORT BOOL               buttonCommand;
+#endif /* _WINGUI */
 
-THOT_EXPORT CHAR_T             szTbStrings [4096];
-THOT_EXPORT WNDCLASSEX         RootShell;
-THOT_EXPORT HMENU              currentMenu;
-THOT_EXPORT HINSTANCE          hInstance;
-THOT_EXPORT int                currentFrame;
-THOT_EXPORT BOOL               buttonCommand;
-#else /* _WINDOWS */
-
-THOT_EXPORT ThotWidget	       RootShell;
-#endif /* !_WINDOWS */
+//#if defined(_MOTIF) || defined(_GTK)
+//THOT_EXPORT ThotWidget	       RootShell;
+//#endif /* #if defined(_MOTIF) || defined(_GTK) */
+#ifdef _MOTIF
+THOT_EXPORT ThotTranslations   TextTranslations;
+#endif /* #ifdef _MOTIF */
 
 THOT_EXPORT ThotAppContext     app_cont;
-THOT_EXPORT Proc	       CurrentCallbackAPI;
+/*THOT_EXPORT Proc	       CurrentCallbackAPI;*/
 
 /* Table of actions related to the menus attached to Application and	  */
 /* Documents windows.							  */
 /* For each action, the following items are stored: name, address,	  */
 /* keyboard equivalent, windows supposed to call the action.		  */
 /* Calling an action: (*actionptr->Call_Action)(document, view);	  */
-
 THOT_EXPORT Action_Ctl	      *MenuActionList;
 THOT_EXPORT int		       MaxMenuAction;
-
 /* return events for Application and Document windows			  */
-THOT_EXPORT int                ClickIsDone;
-THOT_EXPORT CHAR_T*            servername ;
+/*THOT_EXPORT int                ClickIsDone;*/
+THOT_EXPORT char              *servername ;
 #endif /* !NODISPLAY */
 #endif /* THOT_APPDIALOGUE_VAR */
+
+
+
+
+
+
+
