@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001.
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -8,12 +8,17 @@
 
 THOT_EXPORT int         UserErrorCode;
 
-THOT_EXPORT CHAR_T      DefaultDocumentName[MAX_PATH]; /* default document name */
+THOT_EXPORT char        DefaultDocumentName[MAX_PATH]; /* default document name */
 THOT_EXPORT PtrDocument LoadedDocument[MAX_DOCUMENTS]; /* loaded documents */
 THOT_EXPORT DisplayMode	documentDisplayMode[MAX_DOCUMENTS];
 
+THOT_EXPORT CHARSET     DialogCharset;/* the charset of dialog string */
 THOT_EXPORT PtrDocument	DocAutoSave;  /* document to be saved autoimatically */
 THOT_EXPORT int         InputSpace;   /* Spaces imput mode (0 or 1) */
 THOT_EXPORT int         ShowSpace;    /* Spaces display mode (0 or 1) */
+THOT_EXPORT int         DocBackUpInterval; /* number of characters which
+					      triggers an automatic save */
 THOT_EXPORT ThotBool	ChangeLabel;  /* Labels must be changed */
-THOT_EXPORT ThotBool    Printing;     /* TRUE if it's a printing application */
+THOT_EXPORT ThotBool    Printing;     /* TRUE for print, FALSE for amaya */
+THOT_EXPORT ThotBool    ShowOnePage;  /* TRUE when only a page is displayed */
+THOT_EXPORT ThotBool    UseDotForFloat; /* TRUE if floats take the form xx.yy */
