@@ -5,17 +5,23 @@
  *
  */
 
-THOT_EXPORT ThotGC          GCpicture;	 /* for bitmap */
-THOT_EXPORT ThotGC          GCimage;	 /* for pixmap */
-THOT_EXPORT char           *FileExtension[];
-THOT_EXPORT int             HandlersCounter;
-THOT_EXPORT int             InlineHandlers;
-#ifndef _WINDOWS
-THOT_EXPORT Visual         *theVisual;
-#else  /* _WINDOWS */
+/* instancied in picture.c */
+extern ThotGC          GCpicture;	 /* for bitmap */
+THOT_EXPORT ThotGC     GCimage;	 	/* for pixmap */
+extern char           *FileExtension[]; 
+extern int             HandlersCounter;
+extern int             InlineHandlers;
+
+#ifndef _WINGUI
+THOT_EXPORT ThotVisual     *theVisual;
+#else  /* _WINGUI */
 THOT_EXPORT void           *theVisual;
+#endif /* _WINGUI */
+
+#if defined _GTK || defined _WINGUI || defined(_GL)
 THOT_EXPORT char            LostPicturePath [512];
-#endif /* _WINDOWS */
+#endif 
+
 THOT_EXPORT PictureHandler  PictureHandlerTable[MAX_PICT_FORMATS];
 THOT_EXPORT int             PictureIdType[MAX_PICT_FORMATS];
 THOT_EXPORT int             PictureMenuType[MAX_PICT_FORMATS];
@@ -23,10 +29,10 @@ THOT_EXPORT int             InlineHandlers;
 THOT_EXPORT int             HandlersCounter;
 THOT_EXPORT int             currentExtraHandler;
 THOT_EXPORT ThotGC          GCpicture;
-THOT_EXPORT Pixmap          EpsfPictureLogo;
+THOT_EXPORT ThotPixmap      EpsfPictureLogo;
 
 /* bg colors used by Windows */
-#ifdef _WINDOWS
+#ifdef _WINGUI
 THOT_EXPORT ThotBool        bgAlpha;
 THOT_EXPORT HBITMAP         alphamap;
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
