@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -11,7 +11,7 @@
  * Author: V. Quint (INRIA)
  *
  */
-
+#include "thot_gui.h"
 #include "thot_sys.h"
 #include "constmedia.h"
 #include "typemedia.h"
@@ -28,67 +28,53 @@
 /*----------------------------------------------------------------------
    CreateInheritedAttrTable
 
-   Allocate and initialize an InheritAttrTable for element pEl.
+   Allocate and initialize an InheritAttrTable for element pEl in presentation
+   schema pSP.
    This table indicates attributes that transmit presentation rules
    to element pEl.
-   table[attr] = TRUE if the presentation schema contains in its
+   table[attr] != '\0' if the presentation schema contains in its
    ATTRIBUTES section a rule of the form
 
       AttrName(ElType): ...
-
-   AttrName is the name of attribute attr (rank of attribute definition
-   in structure schema) and ElType is the type of element pEl.
   ----------------------------------------------------------------------*/
-void  CreateInheritedAttrTable (PtrElement pEl, PtrDocument pDoc)
+void  CreateInheritedAttrTable (PtrElement pEl, PtrPSchema pPS,
+				PtrDocument pDoc)
 {
   int                 attr;
   int                 rule;
   AttributePres      *pAttrPR;
   InheritAttrTable   *table;
-  PtrPSchema          pSchP, pSP;
-  PtrHandlePSchema    pHd;
 
-  pSP = PresentationSchema (pEl->ElStructSchema, pDoc);
-  if (pSP != NULL)
+  if (pPS != NULL &&
+      pPS->PsInheritedAttr->ElInherit[pEl->ElTypeNumber - 1] == NULL)
     {
       /* table allocation and initialization */
-      if ((table = (InheritAttrTable *) TtaGetMemory (sizeof (InheritAttrTable))) == NULL)
+      if ((table = (InheritAttrTable*) TtaGetMemory (pEl->ElStructSchema->SsNAttributes * sizeof (char))) == NULL)
         /* memory exhausted */
         return;
-      for (attr = 0; attr < MAX_ATTR_SSCHEMA; (*table)[attr++] = FALSE);
-
-      pSP->PsInheritedAttr[pEl->ElTypeNumber - 1] = table;
+      pPS->PsInheritedAttr->ElInherit[pEl->ElTypeNumber - 1] = table;
       /* for all attributes defined in the structure schema */
       for (attr = 0; attr < pEl->ElStructSchema->SsNAttributes; attr++)
 	{
-	  /* check the main presentation schema and all its extensions */
-	  pSchP = pSP;
-	  pHd = NULL;
-	  while (pSchP)
-	    {
-	      pAttrPR = pSchP->PsAttrPRule[attr];
-	      if (pAttrPR != NULL)
-		/* check all presentation rules associated with that attr. */
-		for (rule = 0; rule < pSchP->PsNAttrPRule[attr]; rule++)
+	  (*table)[attr] = '\0';  /* no inheritance by default */
+	  pAttrPR = pPS->PsAttrPRule->AttrPres[attr];
+	  if (pAttrPR != NULL)
+	    /* check all presentation rules associated with that attr */
+	    for (rule = 0; rule < pPS->PsNAttrPRule->Num[attr]; rule++)
+	      {
+		if (pAttrPR->ApElemType == pEl->ElTypeNumber)
 		  {
-		    if (pAttrPR->ApElemType == pEl->ElTypeNumber)
-		      (*table)[attr] = TRUE;
-		    pAttrPR = pAttrPR->ApNextAttrPres;
+		    if (pAttrPR->ApElemInherits)
+		      /* the element inherits some presentation properties
+			 from its ancestors or from itself */
+		      (*table)[attr] = 'S';
+		    else
+		      /* the element inherits some presentation properties
+			 from its ancestors */
+		      (*table)[attr] = 'H';
 		  }
-	      /* next P schema */
-	      if (pHd == NULL)
-		/* extension schemas have not been checked yet */
-		/* get the first extension schema */
-		pHd = FirstPSchemaExtension (pEl->ElStructSchema, pDoc);
-	      else
-		/* get the next extension schema */
-		pHd = pHd->HdNextPSchema;
-	      if (pHd == NULL)
-		/* no more extension schemas. Stop */
-		pSchP = NULL;
-	      else
-		pSchP = pHd->HdPSchema;
-	    }
+		pAttrPR = pAttrPR->ApNextAttrPres;
+	      }
 	}
     }
 }
@@ -109,7 +95,8 @@ void  CreateInheritedAttrTable (PtrElement pEl, PtrDocument pDoc)
   AttrName is the name of attribute attr (rank of attribute definition
   in structure schema).
   ----------------------------------------------------------------------*/
-void CreateComparAttrTable (PtrAttribute pAttr, PtrDocument pDoc)
+void CreateComparAttrTable (PtrAttribute pAttr, PtrDocument pDoc,
+			    PtrElement pEl)
 {
   AttributePres      *pAttrPR;
   int                 attNum;
@@ -125,27 +112,28 @@ void CreateComparAttrTable (PtrAttribute pAttr, PtrDocument pDoc)
   pPS = PresentationSchema (pAttr->AeAttrSSchema, pDoc);
   if (!pPS)
     return;
-  if ((table = (ComparAttrTable *) TtaGetMemory (sizeof (ComparAttrTable))) == NULL)
+  if ((table = (ComparAttrTable *) TtaGetMemory (pAttr->AeAttrSSchema->SsNAttributes * sizeof (ThotBool))) == NULL)
     /* memory exhausted */
     return;
-  for (attr = 0; attr < MAX_ATTR_SSCHEMA; (*table)[attr++] = FALSE);
   attNum = pAttr->AeAttrNum;
-  pPS->PsComparAttr[attNum - 1] = table;
+  pPS->PsComparAttr->CATable[attNum - 1] = table;
   /* scan all attributes defined in the structure schema */
   for (attr = 0; attr < pAttr->AeAttrSSchema->SsNAttributes; attr++)
+    {
+    (*table)[attr] = FALSE;
     /* check only integer attributes */
-    if (pAttr->AeAttrSSchema->SsAttribute[attr].AttrType == AtNumAttr)
+    if (pAttr->AeAttrSSchema->SsAttribute->TtAttr[attr]->AttrType == AtNumAttr)
       {
 	/* check the main presentation schema and all its extensions */
 	pSchP = pPS;
 	pHd = NULL;
 	while (pSchP)
 	  {
-	    if (pSchP->PsNAttrPRule[attr] > 0)
+	    if (pSchP->PsNAttrPRule->Num[attr] > 0)
 	      {
-		pAttrPR = pSchP->PsAttrPRule[attr];
+		pAttrPR = pSchP->PsAttrPRule->AttrPres[attr];
 		/* check presentation rules associated with attribute attr */
-		for (rule = 0; rule < pSchP->PsNAttrPRule[attr]; rule++)
+		for (rule = 0; rule < pSchP->PsNAttrPRule->Num[attr]; rule++)
 		  {
 		    if (pAttrPR->ApElemType == 0)
 		      /* no inheritance */
@@ -169,7 +157,7 @@ void CreateComparAttrTable (PtrAttribute pAttr, PtrDocument pDoc)
 	    if (pHd == NULL)
 	      /* extension schemas have not been checked yet */
 	      /* get the first extension schema */
-	      pHd = FirstPSchemaExtension (pAttr->AeAttrSSchema, pDoc);
+	      pHd = FirstPSchemaExtension (pAttr->AeAttrSSchema, pDoc, pEl);
 	    else
 	      /* get the next extension schema */
 	      pHd = pHd->HdNextPSchema;
@@ -180,6 +168,7 @@ void CreateComparAttrTable (PtrAttribute pAttr, PtrDocument pDoc)
 	      pSchP = pHd->HdPSchema;
 	  }
       }
+    }
 }
 
 /*----------------------------------------------------------------------
@@ -190,7 +179,7 @@ void CreateComparAttrTable (PtrAttribute pAttr, PtrDocument pDoc)
   pDoc by the rule inclRule from the pSS structure schema.
   ----------------------------------------------------------------------*/
 void TransmitElementContent (PtrElement pEl, PtrDocument pDoc,
-			     CHAR_T* attrName, int inclRule, PtrSSchema pSS)
+			     char *attrName, int inclRule, PtrSSchema pSS)
 {
   PtrReferredDescr    pReferredD;
   PtrReference        pRef;
@@ -199,7 +188,7 @@ void TransmitElementContent (PtrElement pEl, PtrDocument pDoc,
   PtrAttribute        pAttr;
   DocumentIdentifier  IncludedDocIdent;
   int                 att;
-  TtAttribute        *AttrDef;
+  PtrTtAttribute         AttrDef;
   PtrElement          pChild;
   ThotBool            found;
   int                 len;
@@ -234,10 +223,11 @@ void TransmitElementContent (PtrElement pEl, PtrDocument pDoc,
 		     while (att < pIncludedEl->ElStructSchema->SsNAttributes &&
 			    !found)
 		       {
-			 AttrDef = &(pIncludedEl->ElStructSchema->SsAttribute[att++]);
+			 AttrDef = pIncludedEl->ElStructSchema->SsAttribute->TtAttr[att++];
 			 if (AttrDef->AttrType == AtTextAttr)
 			   /* that's a text attribute */
-			   if (ustrcmp (AttrDef->AttrOrigName, attrName) == 0)
+			   if (AttrDef->AttrOrigName != NULL &&
+			       strcmp (AttrDef->AttrOrigName, attrName) == 0)
 			     /* that's the rigth attribute */
 			     found = TRUE;
 		       }
@@ -268,7 +258,7 @@ void TransmitElementContent (PtrElement pEl, PtrDocument pDoc,
 					     pAttr->AeAttrText, &len);
 			     /* associate the attribute with the */
 			     /* of the included document */
-			     AttachAttrWithValue (pIncludedEl, pIncludedDoc, pAttr);
+			     AttachAttrWithValue (pIncludedEl, pIncludedDoc, pAttr, TRUE);
 			   }
 			 DeleteAttribute (NULL, pAttr);
 		       }
@@ -301,7 +291,7 @@ void                ApplyTransmitRules (PtrElement pEl, PtrDocument pDoc)
   Counter            *pCounter;
 
   if (pEl != NULL)
-    if (pEl->ElStructSchema->SsRule[pEl->ElTypeNumber - 1].SrRefImportedDoc)
+    if (pEl->ElStructSchema->SsRule->SrElem[pEl->ElTypeNumber - 1]->SrRefImportedDoc)
       {
 	/* it's an inclusion of an external document */
 	/* search the presentation schema that applies to the element */
@@ -320,14 +310,14 @@ void                ApplyTransmitRules (PtrElement pEl, PtrDocument pDoc)
 	      {
 		/* what element type does transmit its contents? */
 		for (srcNumType = 0; srcNumType < pSS->SsNRules &&
-		       pPSch->PsElemTransmit[srcNumType] != rule + 1; srcNumType++);
-		if (pPSch->PsElemTransmit[srcNumType] == rule + 1)
+		       pPSch->PsElemTransmit->Num[srcNumType] != rule + 1; srcNumType++);
+		if (pPSch->PsElemTransmit->Num[srcNumType] == rule + 1)
 		  {
 		    /* elements of type srcNumType+1 transmit their */
 		    /* content to the documents of interest */
 		    /* Search an element of that type in the document */
 		    pSrcEl = FwdSearchTypedElem (pDoc->DocDocElement,
-						 srcNumType + 1, pSS);
+						 srcNumType + 1, pSS, NULL);
 		    if (pSrcEl != NULL)
 		      /* apply the Transmit rule to the element found */
 		      TransmitElementContent (pSrcEl, pDoc,
@@ -367,7 +357,7 @@ void  RepApplyTransmitRules (PtrElement pTransmEl, PtrElement pEl,
     while (pEl != NULL)
       {
 	pEl = FwdSearchTypedElem (pEl, pTransmEl->ElTypeNumber,
-				  pTransmEl->ElStructSchema);
+				  pTransmEl->ElStructSchema, NULL);
 	ApplyTransmitRules (pEl, pDoc);
       }
 }
