@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001.
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -11,7 +11,7 @@
  * Author: V. Quint (INRIA)
  *
  */
-
+#include "thot_gui.h"
 #include "thot_sys.h"
 #include "libmsg.h"
 #include "constmedia.h"
@@ -65,86 +65,92 @@ static void   ApplyRuleSubTree (PtrElement pE, PRuleType ruleType,
 				PtrDocument pDoc, PtrPRule * pPRule,
 				int view, ThotBool display)
 {
-   PtrAbstractBox      pAbb, pAbbF;
-   PtrPSchema          pSPR;
-   PtrAttribute        pAttr;
+  PtrAbstractBox      pAbb, pAbbF;
+  PtrPSchema          pSPR;
+  PtrAttribute        pAttr;
+  ThotBool            complete;
 
-   if (pE->ElTerminal)
-      pE = NULL;
-   else
-      /* on passe au premier fils */
-      pE = pE->ElFirstChild;
-   /* traite tous les fils */
-   while (pE)
-     {
-	/* 1er pave de l'element dans la view */
-	pAbb = pE->ElAbstractBox[view - 1];
-	if (pAbb)
-	   if (pAbb->AbDead)
-	      pAbb = NULL;	/* on ne traite pas les paves morts */
-	if (pAbb == NULL)
-	   /* cet element n'a pas de pave, mais ses descendants en */
-	   /* ont peut etre... */
-	   ApplyRuleSubTree (pE, ruleType, pDoc, pPRule, view, display);
-	else
-	  {
-	     /* il y a un element descendant dont les paves peuvent heriter
-		de pAb. On parcourt ses paves dans la vue */
-	     do
-	       {
-		  *pPRule = SearchRulepAb (pDoc, pAbb, &pSPR, ruleType, FnAny,
-					   TRUE, &pAttr);
-		  if (*pPRule)
+  if (pE->ElTerminal)
+    pE = NULL;
+  else
+    /* on passe au premier fils */
+    pE = pE->ElFirstChild;
+  /* traite tous les fils */
+  while (pE)
+    {
+      /* 1er pave de l'element dans la view */
+      pAbb = pE->ElAbstractBox[view - 1];
+      if (pAbb == NULL)
+	/* no abstract box */
+	if (ruleType == PtVisibility)
+	  /* it's a visibility rule. Try to create the abstract box */
+	  pAbb = AbsBoxesCreate (pE, pDoc, view, True, True, &complete);
+      if (pAbb)
+	if (pAbb->AbDead)
+	  pAbb = NULL;	/* on ne traite pas les paves morts */
+      if (pAbb == NULL)
+	/* cet element n'a pas de pave, mais ses descendants en */
+	/* ont peut etre... */
+	ApplyRuleSubTree (pE, ruleType, pDoc, pPRule, view, display);
+      else
+	{
+	  /* il y a un element descendant dont les paves peuvent heriter
+	     de pAb. On parcourt ses paves dans la vue */
+	  do
+	    {
+	      *pPRule = SearchRulepAb (pDoc, pAbb, &pSPR, ruleType, FnAny,
+				       TRUE, &pAttr);
+	      if (*pPRule)
+		{
+		  if ((*pPRule)->PrPresMode == PresInherit &&
+		      ((*pPRule)->PrInheritMode == InheritParent ||
+		       (*pPRule)->PrInheritMode == InheritGrandFather))
+		    /* la regle de ce pave herite de l'ascendant, */
+		    /* on applique la regle */
+		    if (ApplyRule (*pPRule, pSPR, pAbb, pDoc, pAttr))
+		      {
+			SetChange (pAbb, ruleType, (FunctionType)0);
+			if (display)
+			  RedispAbsBox (pAbb, pDoc);
+			if (!pAbb->AbPresentationBox)
+			  ApplyInherit (ruleType, pAbb, pDoc, display);
+		      }
+		}
+	      if (!pAbb->AbPresentationBox)
+		/* c'est le pave principal de l'element, on traite */
+		/* les paves crees par l'element au niveau inferieur */
+		{
+		  pAbbF = pAbb->AbFirstEnclosed;
+		  while (pAbbF)
 		    {
-		       if ((*pPRule)->PrPresMode == PresInherit &&
-			   ((*pPRule)->PrInheritMode == InheritParent ||
-			    (*pPRule)->PrInheritMode == InheritGrandFather))
-			  /* la regle de ce pave herite de l'ascendant, */
-			  /* on applique la regle */
-			  if (ApplyRule (*pPRule, pSPR, pAbb, pDoc, pAttr))
+		      if (pAbbF->AbElement == pE)
+			{
+			  *pPRule = SearchRulepAb (pDoc, pAbbF, &pSPR,ruleType,
+						   FnAny, TRUE, &pAttr);
+			  if (*pPRule)
 			    {
-			       SetChange (pAbb, ruleType);
-			       if (display)
-				 RedispAbsBox (pAbb, pDoc);
-			       if (!pAbb->AbPresentationBox)
-				 ApplyInherit (ruleType, pAbb, pDoc, display);
+			      if ((*pPRule)->PrPresMode == PresInherit
+				  && (*pPRule)->PrInheritMode == InheritParent)
+				if (ApplyRule (*pPRule, pSPR, pAbbF,
+					       pDoc, pAttr))
+				  {
+				    SetChange (pAbbF, ruleType, (FunctionType)0);
+				    if (display)
+				      RedispAbsBox (pAbbF, pDoc);
+				  }
 			    }
+			}
+		      pAbbF = pAbbF->AbNext;
 		    }
-		  if (!pAbb->AbPresentationBox)
-		     /* c'est le pave principal de l'element, on traite */
-		     /* les paves crees par l'element au niveau inferieur */
-		    {
-		       pAbbF = pAbb->AbFirstEnclosed;
-		       while (pAbbF)
-			 {
-			    if (pAbbF->AbElement == pE)
-			      {
-				 *pPRule = SearchRulepAb (pDoc, pAbbF, &pSPR,
-						ruleType, FnAny, TRUE, &pAttr);
-				 if (*pPRule)
-				   {
-				      if ((*pPRule)->PrPresMode == PresInherit
-					  && (*pPRule)->PrInheritMode == InheritParent)
-					 if (ApplyRule (*pPRule, pSPR, pAbbF,
-							pDoc, pAttr))
-					   {
-					     SetChange (pAbbF, ruleType);
-					     if (display)
-					       RedispAbsBox (pAbbF, pDoc);
-					   }
-				   }
-			      }
-			    pAbbF = pAbbF->AbNext;
-			 }
-		    }
-		  pAbb = pAbb->AbNext;	/* passe au pave suivant */
-	       }
-	       /* on arrete s'il n'y a pas de pave suivant ou si le pave
-		  suivant n'appartient pas a l'element */
-	     while (pAbb && pAbb->AbElement == pE);
-	  }
-	pE = pE->ElNext;	/* on traite l'element suivant */
-     }
+		}
+	      pAbb = pAbb->AbNext;	/* passe au pave suivant */
+	    }
+	  /* on arrete s'il n'y a pas de pave suivant ou si le pave
+	     suivant n'appartient pas a l'element */
+	  while (pAbb && pAbb->AbElement == pE);
+	}
+      pE = pE->ElNext;	/* on traite l'element suivant */
+    }
 }
 
 /*----------------------------------------------------------------------
@@ -153,7 +159,8 @@ static void   ApplyRuleSubTree (PtrElement pE, PRuleType ruleType,
    		heritent de cette regle et si oui leur applique		
    		l'heritage et on les r'eaffiche si display est TRUE
   ----------------------------------------------------------------------*/
-void         ApplyInherit (PRuleType ruleType, PtrAbstractBox pAb, PtrDocument pDoc, ThotBool display)
+void         ApplyInherit (PRuleType ruleType, PtrAbstractBox pAb,
+			   PtrDocument pDoc, ThotBool display)
 {
    PtrElement          pEl;
    int                 view;
@@ -194,7 +201,7 @@ void         ApplyInherit (PRuleType ruleType, PtrAbstractBox pAb, PtrDocument p
 			on applique la regle */
 		     if (ApplyRule (pPRule, pSchP, pAbbCur, pDoc, pAttrib))
 		       {
-			  SetChange (pAbbCur, ruleType);
+			  SetChange (pAbbCur, ruleType, (FunctionType)0);
 			  ApplyInherit (ruleType, pAbbCur, pDoc, display);
 			  if (display)
 			    RedispAbsBox (pAbbCur, pDoc);
@@ -236,7 +243,7 @@ void         ApplyInherit (PRuleType ruleType, PtrAbstractBox pAb, PtrDocument p
 		   /* applique la regle */
 		   if (ApplyRule (pPRule, pSchP, pAbbCur, pDoc, pAttrib))
 		     {
-		       SetChange (pAbbCur, ruleType);
+		       SetChange (pAbbCur, ruleType, (FunctionType)0);
 		       ApplyInherit (ruleType, pAbbCur, pDoc, display);
 		       if (display)
 			 RedispAbsBox (pAbbCur, pDoc);
@@ -266,7 +273,7 @@ void         ApplyInherit (PRuleType ruleType, PtrAbstractBox pAb, PtrDocument p
 			 on l'applique */
 		      if (ApplyRule (pPRule, pSchP, pAbbCur, pDoc, pAttrib))
 			{
-			  SetChange (pAbbCur, ruleType);
+			  SetChange (pAbbCur, ruleType, (FunctionType)0);
 			  ApplyInherit (ruleType, pAbbCur, pDoc, display);
 			  if (display)
 			    RedispAbsBox (pAbbCur, pDoc);
@@ -291,7 +298,7 @@ void         ApplyInherit (PRuleType ruleType, PtrAbstractBox pAb, PtrDocument p
 			 on l'applique */
 		      if (ApplyRule (pPRule, pSchP, pAbbCur, pDoc, pAttrib))
 			{
-			  SetChange (pAbbCur, ruleType);
+			  SetChange (pAbbCur, ruleType, (FunctionType)0);
 			  ApplyInherit (ruleType, pAbbCur, pDoc, display);
 			  if (display)
 			    RedispAbsBox (pAbbCur, pDoc);
@@ -316,7 +323,7 @@ void         ApplyInherit (PRuleType ruleType, PtrAbstractBox pAb, PtrDocument p
 		       on l'applique */
 		    if (ApplyRule (pPRule, pSchP, pAbbCur, pDoc, pAttrib))
 		      {
-			SetChange (pAbbCur, ruleType);
+			SetChange (pAbbCur, ruleType, (FunctionType)0);
 			ApplyInherit (ruleType, pAbbCur, pDoc, display);
 			if (display)
 			  RedispAbsBox (pAbbCur, pDoc);
@@ -421,6 +428,15 @@ int          NumTypePRuleAPI (PtrPRule pRule)
 	    case PtBorderLeftStyle:
                return PRBorderLeftStyle;
                break;
+	    case PtDisplay:
+	       return PRDisplay;
+	       break;
+	    case PtFloat:
+	       return PRFloat;
+	       break;
+	    case PtClear:
+	       return PRClear;
+	       break;
 	    case PtSize:
 	       return PRSize;
 	       break;
@@ -451,6 +467,12 @@ int          NumTypePRuleAPI (PtrPRule pRule)
 	    case PtAdjust:
 	       return PRAdjust;
 	       break;
+	    case PtDirection:
+	       return PRDirection;
+	       break;
+	    case PtUnicodeBidi:
+	       return PRUnicodeBidi;
+	       break;
 	    case PtLineStyle:
 	       return PRLineStyle;
 	       break;
@@ -460,6 +482,15 @@ int          NumTypePRuleAPI (PtrPRule pRule)
 	    case PtFillPattern:
 	       return PRFillPattern;
 	       break;
+	    case PtOpacity:
+	       return PROpacity;
+	       break;
+	    case PtFillOpacity:
+	       return PRFillOpacity;
+	       break;
+	    case PtStrokeOpacity:
+	       return PRStrokeOpacity;
+	       break;
 	    case PtBackground:
 	       return PRBackground;
 	       break;
@@ -604,16 +635,9 @@ void    ApplyNewRule (PtrDocument pDoc, PtrPRule pPRule, PtrElement pEl)
 		  if (ApplyRule (pPRule, NULL, pAb, pDoc, NULL))
 		    {
 		      if (pPRule->PrType == PtFunction)
-			{
-			  if (pPRule->PrPresFunction == FnPictureMode
-			      || pPRule->PrPresFunction == FnBackgroundPicture
-			      || pPRule->PrPresFunction == FnShowBox)
-			    pAb->AbAspectChange = TRUE;
-			  else
-			    pAb->AbChange = TRUE;
-			}
+			SetChange (pAb, pPRule->PrType, (FunctionType)pPRule->PrPresFunction);
 		      else
-			SetChange (pAb, pPRule->PrType);
+			SetChange (pAb, pPRule->PrType, (FunctionType)0);
 		      ApplyInherit (pPRule->PrType, pAb, pDoc, TRUE);
 		      /* indique le pave a faire reafficher */
 		      RedispAbsBox (pAb, pDoc);
@@ -626,7 +650,8 @@ void    ApplyNewRule (PtrDocument pDoc, PtrPRule pPRule, PtrElement pEl)
   RemoveSpecifPres supprime toutes les regles de presentation specifiques
   associees a l'element pEl.				
   ----------------------------------------------------------------------*/
-static void RemoveSpecifPres (PtrElement pEl, PtrDocument pDoc, RuleSet rules, int viewToApply)
+static void RemoveSpecifPres (PtrElement pEl, PtrDocument pDoc,
+			      RuleSet rules, int viewToApply)
 {
    PtrPRule            pPRule, pR, pRS;
    NotifyPresentation  notifyPres;
@@ -638,13 +663,15 @@ static void RemoveSpecifPres (PtrElement pEl, PtrDocument pDoc, RuleSet rules, i
    /* type de cette view */
    viewSch = AppliedView (pEl, NULL, pDoc, viewToApply);
    pPRule = pEl->ElFirstPRule;
+   if (pPRule == NULL)
+     return;
    pR = NULL;
    found = FALSE;
    doc = IdentDocument (pDoc);
-   /* parcourt les regles de presentation specifiques de l'element */
-   while (pPRule != NULL)
+   /* manage all specific presentation rules of the element if it still exists */
+   while (pPRule && pEl->ElStructSchema)
       if (pPRule->PrViewNum != viewSch || !RuleSetIn (pPRule->PrType, rules))
-	 /* cette regle n'est pas concernee */
+	 /* skip this rule */
 	{
 	   pR = pPRule;
 	   pPRule = pR->PrNextPRule;
@@ -678,7 +705,7 @@ static void RemoveSpecifPres (PtrElement pEl, PtrDocument pDoc, RuleSet rules, i
 		notifyPres.pRule = NULL;
 		notifyPres.pRuleType = NumTypePRuleAPI (pPRule);
 		/* libere la regle */
-		FreePresentRule (pPRule);
+		FreePresentRule (pPRule, pEl->ElStructSchema);
 		CallEventType ((NotifyEvent *) & notifyPres, FALSE);
 	     }
 	   /* passe a la regle suivante */
@@ -716,16 +743,9 @@ static void ApplyPRuleAndRedisplay (PtrAbstractBox pAb, PtrDocument pDoc,
   ApplyRule (pRP, pSPR, pAb, pDoc, pAttr);
   /* marque que le pave a change' et doit etre reaffiche' */
   if (pRP->PrType == PtFunction)
-    {
-      if (pRP->PrPresFunction == FnPictureMode
-	  || pRP->PrPresFunction == FnBackgroundPicture
-	  || pRP->PrPresFunction == FnShowBox)
-	pAb->AbAspectChange = TRUE;
-      else
-	pAb->AbChange = TRUE;
-    }
+    SetChange (pAb, pRP->PrType, (FunctionType)pRP->PrPresFunction);
   else
-    SetChange (pAb, pRP->PrType);
+    SetChange (pAb, pRP->PrType, (FunctionType)0);
   RedispAbsBox (pAb, pDoc);
   /* applique la regle de meme type aux paves environnants */
   /* s'ils heritent de ce parametre de presentation */
@@ -821,23 +841,24 @@ static PtrAbstractBox     NextAbstractBox (PtrAbstractBox pAb)
 static void RemoveFunctionPRule (PtrPRule pPres, PtrAbstractBox pAb,
 				 PtrDocument pDoc)
 {
-   if (pPres->PrPresFunction == FnBackgroundPicture &&
-       pAb->AbPictBackground != NULL)
-     {
-       TtaFreeMemory ((((PictInfo *) (pAb->AbPictBackground))->PicFileName));
-       FreePictInfo ((PictInfo *) (pAb->AbPictBackground));
-       TtaFreeMemory (pAb->AbPictBackground);
-       pAb->AbPictBackground = NULL;
-     }
-   else if (pPres->PrPresFunction == FnPictureMode &&
-	    pAb->AbPictBackground != NULL)
-       ((PictInfo *) (pAb->AbPictBackground))->PicPresent = FillFrame;
-   else if (pPres->PrPresFunction == FnShowBox)
-       pAb->AbFillBox = FALSE;
+  PictInfo      *image;
 
-   pAb->AbAspectChange = TRUE;
-   RedispAbsBox (pAb, pDoc);
-   /* pDoc->DocViewModifiedAb[view] = pAb; */
+  if (pPres->PrPresFunction == FnBackgroundPicture && pAb->AbPictBackground)
+    {
+      image = (PictInfo *)pAb->AbPictBackground;
+      CleanPictInfo (image);
+      TtaFreeMemory (image->PicFileName);
+      TtaFreeMemory (pAb->AbPictBackground);
+      pAb->AbPictBackground = NULL;
+    }
+  else if (pPres->PrPresFunction == FnPictureMode &&
+	   pAb->AbPictBackground)
+    ((PictInfo *) (pAb->AbPictBackground))->PicPresent = FillFrame;
+  else if (pPres->PrPresFunction == FnShowBox)
+    pAb->AbFillBox = FALSE;
+  
+  pAb->AbAspectChange = TRUE;
+  RedispAbsBox (pAb, pDoc);
 }
 
 /*----------------------------------------------------------------------
@@ -856,7 +877,7 @@ void  ApplyASpecificStyleRule (PtrPRule pRule, PtrElement pEl,
   PRuleType	  ruleType;
   int             viewSch;
   int             view;
-  ThotBool	  done, enclosed;
+  ThotBool	  done, enclosed, complete;
 
   enclosed = FALSE;
   /* do nothing if the document no longer exists */
@@ -866,6 +887,11 @@ void  ApplyASpecificStyleRule (PtrPRule pRule, PtrElement pEl,
     {
       /* the abstract box of the root element */
       pAb = pEl->ElAbstractBox[view];
+      if (pAb == NULL)
+	/* no abstract box */
+	if (pRule->PrType == PtVisibility)
+	  /* it's a visibility rule. Try to create the abstract box */
+	  pAb = AbsBoxesCreate (pEl, pDoc, view+1, True, True, &complete);
       /* the schema view associatde with the current view */
       viewSch = pDoc->DocView[view].DvPSchemaView;
       while (pAb != NULL)
@@ -1004,7 +1030,7 @@ void  ApplyAGenericStyleRule (Document doc, PtrSSchema pSS, int elType,
 	  if (elType > 0)
 	    /* presentation rules are associated with an element type */
 	    found = (pAb->AbElement->ElTypeNumber == elType &&
-	             !ustrcmp (pAb->AbElement->ElStructSchema->SsName, pSS->SsName));
+	             !strcmp (pAb->AbElement->ElStructSchema->SsName, pSS->SsName));
 	  else if (attrType > 0)
 	    {
 	      /* presentation rules are associated with an attribute type */
@@ -1012,7 +1038,7 @@ void  ApplyAGenericStyleRule (Document doc, PtrSSchema pSS, int elType,
 	      while (!found && pAttr != NULL)
 		{
 		  found = (pAttr->AeAttrNum == attrType &&
-			   !ustrcmp (pAttr->AeAttrSSchema->SsName, pSS->SsName));
+			   !strcmp (pAttr->AeAttrSSchema->SsName, pSS->SsName));
 		  if (!found)
 		     pAttr = pAttr->AeNext;
 		}
@@ -1045,10 +1071,10 @@ void  ApplyAGenericStyleRule (Document doc, PtrSSchema pSS, int elType,
 			    {
 			      /* remove a PtFunction rule */
 			      RemoveFunctionPRule (pCurrentRule, pAb, pDoc);
-			      if (pRP != NULL && pRP != pCurrentRule)
+			      if (pRP && pRP != pCurrentRule)
 				ApplyPRuleAndRedisplay (pAb, pDoc, pAttr, pRP, pSPR);
 			    }
-			  else if (pRP != NULL)
+			  else if (pRP && (!remove || pRP != pCurrentRule))
 			    ApplyPRuleAndRedisplay (pAb, pDoc, pAttr, pRP, pSPR);
 			}
 		    }
@@ -1120,7 +1146,7 @@ ThotBool            TtaIsCSSPRule (PRule pRule)
      TtaError (ERR_invalid_parameter);
    else
      /* parameter pRule is correct */
-     ret = (((PtrPRule) pRule)->PrLevel != 0);
+     ret = (((PtrPRule) pRule)->PrSpecificity > 0);
    return ret;
 }
 
@@ -1172,7 +1198,7 @@ void    TtaRemovePRule (Element element, PRule pRule, Document document)
 	      RedisplayDefaultPresentation (document, (PtrElement) element,
 			pPres->PrType, pPres->PrPresFunction,pPres->PrViewNum);
 #endif
-	   FreePresentRule (pPres);
+	   FreePresentRule (pPres, ((PtrElement) element)->ElStructSchema);
 	 }
      }
 }
