@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -12,7 +12,7 @@
  * Author: V. Quint (INRIA)
  *
  */
-
+#include "thot_gui.h"
 #include "thot_sys.h"
 #include "constmedia.h"
 #include "typemedia.h"
@@ -27,56 +27,71 @@
 #include "structschema_f.h"
 #include "content_f.h"
 #include "presvariables_f.h"
+#include "schemas_f.h"
 
 /*----------------------------------------------------------------------
-   MakeAliasTypeCount cree un alias temporaire qui va servir a faire 
-   une recherche multiple sur les elements du compteur     
-   on cree l'alias  MAX_RULES_SSCHEMA  pour les elements reset   
-   du compteur.  On cree l'alias  MAX_RULES_SSCHEMA + 1 pour les 
-   elements add du compteur.                               
+   MakeAliasTypeCount
+   Create a temporary structure rule in schema pSchStr to allow
+   searching of multiple types used in a counter.
+   Create alias rule SsNRules+1 for reset elements
+   Create alias rule SsNRules+2 for add elements
   ----------------------------------------------------------------------*/
 static int MakeAliasTypeCount (Counter *pCounter, CounterOp op,
 			       PtrSSchema pSchStr)
 {
-   SRule              *NewAlias;
-   int                 i;
+  PtrSRule            NewAlias;
+  int                 i, rule;
 
-   if (op == CntrSet)
-      NewAlias = &pSchStr->SsRule[MAX_RULES_SSCHEMA];
-   else if (op == CntrAdd)
-      NewAlias = &pSchStr->SsRule[MAX_RULES_SSCHEMA + 1];
-   else				/* on s'est trompe c'est un CntrRank */
-      return pCounter->CnItem[0].CiElemType;
+  if (op == CntrSet)
+    rule = pSchStr->SsNRules + 1;
+  else if (op == CntrAdd)
+    rule = pSchStr->SsNRules + 2;
+  else				/* on s'est trompe c'est un CntrRank */
+    return pCounter->CnItem[0].CiElemType;
 
-   /* initialise l'alias temporaire */
-   ustrcpy (NewAlias->SrName, "Counter alias");
-   NewAlias->SrNDefAttrs = 0;
-   NewAlias->SrNLocalAttrs = 0;
-   NewAlias->SrUnitElem = FALSE;
-   NewAlias->SrRecursive = FALSE;
-   NewAlias->SrExportedElem = FALSE;
-   NewAlias->SrFirstExcept = 0;
-   NewAlias->SrLastExcept = 0;
-   NewAlias->SrNInclusions = 0;
-   NewAlias->SrNExclusions = 0;
-   NewAlias->SrConstruct = CsChoice;
-   NewAlias->SrNChoices = 0;
+  NewAlias = (PtrSRule) TtaGetMemory (sizeof (SRule));
+  pSchStr->SsRule->SrElem[rule -1] = NewAlias;
 
-   /* initialise le constructeur choix de l'alias */
-   for (i = 0; i < pCounter->CnNItems; i++)
-     {
-	if (pCounter->CnItem[i].CiCntrOp == op)
-	  {
-	     NewAlias->SrChoice[NewAlias->SrNChoices] =
-		pCounter->CnItem[i].CiElemType;
-	     NewAlias->SrNChoices += 1;
-	  }
-     }
+  /* initialise l'alias temporaire */
+  NewAlias->SrName = TtaStrdup ("Counter_alias");
+  NewAlias->SrNDefAttrs = 0;
+  NewAlias->SrNLocalAttrs = 0;
+  NewAlias->SrLocalAttr = NULL;
+  NewAlias->SrRequiredAttr = NULL;
+  NewAlias->SrUnitElem = FALSE;
+  NewAlias->SrRecursive = FALSE;
+  NewAlias->SrExportedElem = FALSE;
+  NewAlias->SrFirstExcept = 0;
+  NewAlias->SrLastExcept = 0;
+  NewAlias->SrNInclusions = 0;
+  NewAlias->SrNExclusions = 0;
+  NewAlias->SrConstruct = CsChoice;
+  NewAlias->SrNChoices = 0;
+  /* initialise le constructeur choix de l'alias */
+  for (i = 0; i < pCounter->CnNItems; i++)
+    {
+      if (pCounter->CnItem[i].CiCntrOp == op)
+	{
+	  NewAlias->SrChoice[NewAlias->SrNChoices] =
+	    pCounter->CnItem[i].CiElemType;
+	  NewAlias->SrNChoices += 1;
+	}
+    }
+  return rule;
+}
 
-   if (op == CntrSet)
-      return MAX_RULES_SSCHEMA + 1;
-   else				/* op == CntrAdd */
-      return MAX_RULES_SSCHEMA + 2;
+/*----------------------------------------------------------------------
+   ReleaseAliasTypeCount
+   Destroys a provisional structure rule created for evaluating
+   a counter.
+   typeInt is the rule number and pSS is the structure schema where
+   it was created.
+  ----------------------------------------------------------------------*/
+static void ReleaseAliasTypeCount (int typeNum, PtrSSchema pSS)
+{
+  TtaFreeMemory (pSS->SsRule->SrElem[typeNum -1]->SrName);
+  TtaFreeMemory (pSS->SsRule->SrElem[typeNum -1]);
+  pSS->SsRule->SrElem[typeNum -1] = NULL;
 }
 
 /*----------------------------------------------------------------------
@@ -126,7 +141,7 @@ static ThotBool InitCounterByAttribute (int *valinit, Counter * pCo,
 		if (pAttr == NULL)
 		   stop = TRUE;	/* dernier attribut */
 		else if (pAttr->AeAttrNum == pCo->CnItem[0].CiInitAttr &&
-			 !ustrcmp (pAttr->AeAttrSSchema->SsName, pSS->SsName))
+			 !strcmp (pAttr->AeAttrSSchema->SsName, pSS->SsName))
 		   stop = TRUE;	/* c'est l'attribut cherche' */
 		else
 		   pAttr = pAttr->AeNext;	/* au suivant */
@@ -167,7 +182,7 @@ static ThotBool CondAttrOK (CntrItem * pCountItem, PtrElement pEl,
 	if (pAttr == NULL)
 	   stop = TRUE;	/* dernier attribut */
 	else if (pAttr->AeAttrNum == pCountItem->CiCondAttr &&
-		 !ustrcmp (pAttr->AeAttrSSchema->SsName, pSS->SsName))
+		 !strcmp (pAttr->AeAttrSSchema->SsName, pSS->SsName))
 	   stop = TRUE;	/* c'est l'attribut cherche' */
 	else
 	   pAttr = pAttr->AeNext;	/* au suivant */
@@ -258,20 +273,20 @@ int CounterValMinMax (int counterNum, PtrSSchema pSS, PtrPSchema pSchP,
 		 stop = FALSE;
 		 do
 		   {
-		     pEl = BackSearchTypedElem (pEl, TypeRank, NULL);
+		     pEl = BackSearchTypedElem (pEl, TypeRank, NULL, NULL);
 		     if (pEl == NULL)
 		       /* pas de marque de page precedente */
 		       stop = TRUE;
 		     else
 		       {
-			 if (ustrcmp (pElNum->ElStructSchema->SsName, pSSpr->SsName))
+			 if (strcmp (pElNum->ElStructSchema->SsName, pSSpr->SsName))
 			   /* Saut de page d'une view squelette */
 			   /* on ne que compte les pages etrangeres au schema */
 			   /* principal */
-			   pstop = ustrcmp (pEl->ElStructSchema->SsName,
+			   pstop = strcmp (pEl->ElStructSchema->SsName,
 					    pSSpr->SsName);
 			 else
-			   pstop = !ustrcmp (pEl->ElStructSchema->SsName,
+			   pstop = !strcmp (pEl->ElStructSchema->SsName,
 					     pSSpr->SsName);
 
 			 /* on ignore les pages qui ne concernent pas la view */
@@ -298,7 +313,7 @@ int CounterValMinMax (int counterNum, PtrSSchema pSS, PtrPSchema pSchP,
 		  /* suivantes jusqu'a ne plus en trouver */
 		  while (pEl != NULL)
 		    {
-		       pEl = FwdSearchTypedElem (pEl, TypeRank, NULL);
+		       pEl = FwdSearchTypedElem (pEl, TypeRank, NULL, NULL);
 		       if (pEl != NULL)
 			  if ((pEl->ElViewPSchema == view) && (pEl->ElPageNumber > value))
 			     value = pEl->ElPageNumber - 1;
@@ -337,8 +352,8 @@ int CounterValMinMax (int counterNum, PtrSSchema pSS, PtrPSchema pSchP,
 			  /* l'element dont on veut le rang n'a pas de frere... */
 			  if (pEl->ElParent != NULL)
 			     /* ... mais il a un pere... */
-			     if (pEl->ElParent->ElStructSchema->SsRule[pEl->ElParent->
-				  ElTypeNumber - 1].SrConstruct == CsChoice)
+			     if (pEl->ElParent->ElStructSchema->SsRule->SrElem[pEl->ElParent->
+				  ElTypeNumber - 1]->SrConstruct == CsChoice)
 				/* ... et son pere est un choix */
 				/* on prendra le rang du pere */
 				pEl = pEl->ElParent;
@@ -405,41 +420,43 @@ int CounterValMinMax (int counterNum, PtrSSchema pSS, PtrPSchema pSchP,
 
 	if (Maximum)
 	  {
-	     /* On veut la valeur maximale du compteur     */
-	     /* a partir de l'element trouve', cherche en avant tous les */
-	     /* elements ayant le type qui incremente le compteur, */
-	     /* jusqu'a rencontrer un autre element qui reset le compteur */
-	     if (TypeIncr <= MAX_BASIC_TYPE)
-		/* c'est un type de base, on le cherche quel que soit son schema */
-		pSchIncr = NULL;
-	     else
-		pSchIncr = pSchStr; /* schema de struct. du type qui incremente */
-	     if (TypeIncr > 0)
-		do
-		  {
-		     pEl = FwdSearchElem2Types (pEl, TypeIncr, pElNum->ElTypeNumber, pSchIncr,
-						pElNum->ElStructSchema);
-		     if (pEl != NULL &&
-			 EquivalentType (pEl, TypeIncr, pSchIncr))
-		       {
-			   /* on a trouve' un element du type qui incremente */
-			if (!CondAttr)
-			   value += GetCounterValEl (pCo1, pEl, CntrAdd, pSS);
-			else
-			   /* check conditions on attributes */
-			   {
-			   i = GetCounterItem (pCo1, CntrAdd, pSS, pEl);
-			   if (i >= 0)
-			      if (CondAttrOK (&pCo1->CnItem[i], pEl, pSS))
-			         value += GetCounterValEl (pCo1, pEl, CntrAdd, pSS);
-			   }
-		       }
-		  }
-		while (pEl != NULL && !EquivalentType (pEl, TypeSet, pSS));
+	    /* On veut la valeur maximale du compteur a partir de l'element
+	       trouve', cherche en avant tous les elements ayant le type qui
+	       incremente le compteur, jusqu'a rencontrer un autre element qui
+	       reset le compteur */
+	    if (TypeIncr <= MAX_BASIC_TYPE)
+	      /* c'est un type de base, on le cherche quel que soit son schema */
+	      pSchIncr = NULL;
+	    else
+	      pSchIncr = pSchStr; /* schema de struct. du type qui incremente*/
+	    if (TypeIncr > 0)
+	      do
+		{
+		  pEl = FwdSearchElem2Types (pEl, TypeIncr, pElNum->ElTypeNumber,
+				       pSchIncr, pElNum->ElStructSchema, NULL);
+		  if (pEl != NULL &&
+		      EquivalentType (pEl, TypeIncr, pSchIncr))
+		    {
+		      /* on a trouve' un element du type qui incremente */
+		      if (!CondAttr)
+			value += GetCounterValEl (pCo1, pEl, CntrAdd, pSS);
+		      else
+			/* check conditions on attributes */
+			{
+			  i = GetCounterItem (pCo1, CntrAdd, pSS, pEl);
+			  if (i >= 0)
+			    if (CondAttrOK (&pCo1->CnItem[i], pEl, pSS))
+			      value += GetCounterValEl (pCo1, pEl, CntrAdd, pSS);
+			}
+		    }
+		}
+	      while (pEl != NULL && !EquivalentType (pEl, TypeSet, pSS));
 	  }
+	ReleaseAliasTypeCount (TypeSet, pSS);
+	ReleaseAliasTypeCount (TypeIncr, pSS);
      }
    if (value < 0)
-      value = 0;
+     value = 0;
 
    return value;
 }
@@ -488,7 +505,7 @@ int CounterVal (int counterNum, PtrSSchema pSS, PtrPSchema pSchP,
 	while (pEl != NULL)
 	  {
 	     if (pEl->ElTypeNumber == TypeRLevel &&
-		 !ustrcmp (pEl->ElStructSchema->SsName,
+		 !strcmp (pEl->ElStructSchema->SsName,
 			   pElNum->ElStructSchema->SsName))
 		/* cet element englobant a le type qui incremente le compteur*/
 		/* if there is a condition about an attribute attached to the
@@ -521,19 +538,19 @@ int CounterVal (int counterNum, PtrSSchema pSS, PtrPSchema pSchP,
 	     stop = FALSE;
 	     do
 	       {
-		  pEl = BackSearchTypedElem (pEl, TypeRank, NULL);
+		  pEl = BackSearchTypedElem (pEl, TypeRank, NULL, NULL);
 		  if (pEl == NULL)
 		     /* pas de marque de page precedente */
 		     stop = TRUE;
 		  else
 		    {
-		       if (ustrcmp (pElNum->ElStructSchema->SsName, pSSpr->SsName))
+		       if (strcmp (pElNum->ElStructSchema->SsName, pSSpr->SsName))
 			  /* Saut de page d'une view squelette */
 			  /*on ne que compte les pages etrangeres au schema principal */
-			  pstop = ustrcmp (pEl->ElStructSchema->SsName,
+			  pstop = strcmp (pEl->ElStructSchema->SsName,
 					   pSSpr->SsName);
 		       else
-			  pstop = !ustrcmp (pEl->ElStructSchema->SsName,
+			  pstop = !strcmp (pEl->ElStructSchema->SsName,
 					    pSSpr->SsName);
 
 		       /* on ignore les pages qui ne concernent pas la view */
@@ -588,7 +605,7 @@ int CounterVal (int counterNum, PtrSSchema pSS, PtrPSchema pSchP,
 		  while (level > 0 && pEl != NULL)
 		    {
 		       if (pEl->ElTypeNumber == TypeRank &&
-			   !ustrcmp (pEl->ElStructSchema->SsName,
+			   !strcmp (pEl->ElStructSchema->SsName,
 				     pElNum->ElStructSchema->SsName))
 			  /* cet element englobant a le type qui incremente le
 			     compteur */
@@ -611,7 +628,7 @@ int CounterVal (int counterNum, PtrSSchema pSS, PtrPSchema pSchP,
 		     /* l'element dont on veut le rang n'a pas de frere... */
 		     if (pEl->ElParent != NULL)
 			/* ... mais il a un pere... */
-			if (pEl->ElParent->ElStructSchema->SsRule[pEl->ElParent->ElTypeNumber - 1].SrConstruct
+			if (pEl->ElParent->ElStructSchema->SsRule->SrElem[pEl->ElParent->ElTypeNumber - 1]->SrConstruct
 			    == CsChoice)
 			   /* ... et son pere est un choix */
 			   /* on prendra le rang du pere */
@@ -632,7 +649,7 @@ int CounterVal (int counterNum, PtrSSchema pSS, PtrPSchema pSchP,
 			    if (pAttr == NULL)
 			      stop = TRUE;	/* dernier attribut */
 			    else if (pAttr->AeAttrNum == pCo1->CnItem[0].CiReinitAttr &&
-				     !ustrcmp (pAttr->AeAttrSSchema->SsName, pSS->SsName))
+				     !strcmp (pAttr->AeAttrSSchema->SsName, pSS->SsName))
 			      stop = TRUE;	/* c'est l'attribut cherche' */
 			    else
 			      pAttr = pAttr->AeNext;	/* au suivant */
@@ -708,33 +725,33 @@ int CounterVal (int counterNum, PtrSSchema pSS, PtrPSchema pSchP,
 	  /* elements ayant le type qui incremente le compteur, */
 	  /* jusqu'a rencontrer l'element qui a cree la boite compteur. */
 	  if (TypeIncr <= MAX_BASIC_TYPE)
-	     /* c'est un type de base, on le cherche quel que soit son schema*/
-	     pSchIncr = NULL;
+	    /* c'est un type de base, on le cherche quel que soit son schema*/
+	    pSchIncr = NULL;
 	  else
-	     pSchIncr = pSchStr; /* schema de struct. du type qui incremente */
+	    pSchIncr = pSchStr; /* schema de struct. du type qui incremente */
 	  if (TypeIncr > 0)
-	     do
-	       {
-		  pEl = FwdSearchElem2Types (pEl, TypeIncr,
-					     pElNum->ElTypeNumber, pSchIncr,
-					     pElNum->ElStructSchema);
-		  if (pEl != NULL &&
-		      EquivalentType (pEl, TypeIncr, pSchIncr))
-		    {
-		      /* on a trouve' un element du type qui incremente */
-		      if (!CondAttr)
-			value += GetCounterValEl (pCo1, pEl, CntrAdd, pSS);
-		      else
-			/* check conditions on attributes */
-			{
-			  i = GetCounterItem (pCo1, CntrAdd, pSS, pEl);
-			  if (i >= 0)
-			    if (CondAttrOK (&pCo1->CnItem[i], pEl, pSS))
-			      value += GetCounterValEl (pCo1, pEl, CntrAdd, pSS);
-			}
-		    }
-	       }
-	     while (pEl != NULL && pEl != pElNum);
+	    do
+	      {
+		pEl = FwdSearchElem2Types (pEl, TypeIncr,
+					   pElNum->ElTypeNumber, pSchIncr,
+					   pElNum->ElStructSchema, NULL);
+		if (pEl != NULL &&
+		    EquivalentType (pEl, TypeIncr, pSchIncr))
+		  {
+		    /* on a trouve' un element du type qui incremente */
+		    if (!CondAttr)
+		      value += GetCounterValEl (pCo1, pEl, CntrAdd, pSS);
+		    else
+		      /* check conditions on attributes */
+		      {
+			i = GetCounterItem (pCo1, CntrAdd, pSS, pEl);
+			if (i >= 0)
+			  if (CondAttrOK (&pCo1->CnItem[i], pEl, pSS))
+			    value += GetCounterValEl (pCo1, pEl, CntrAdd, pSS);
+		      }
+		  }
+	      }
+	    while (pEl != NULL && pEl != pElNum);
 	  }
 
 	else
@@ -779,7 +796,7 @@ int CounterVal (int counterNum, PtrSSchema pSS, PtrPSchema pSchP,
 	     do
 	       {
 		  pEl = BackSearchElem2Types (pEl, TypeSet, TypeIncr, pSchSet,
-					      pSchIncr);
+					      pSchIncr, NULL);
 		  if (pEl != NULL)
 		    {
 		     if (EquivalentType (pEl, TypeIncr, pSchIncr))
@@ -831,6 +848,8 @@ int CounterVal (int counterNum, PtrSSchema pSS, PtrPSchema pSchP,
 	     while (pEl != NULL);
 	  value += Nincr * incrVal;
 	  }
+	ReleaseAliasTypeCount (TypeSet, pSS);
+	ReleaseAliasTypeCount (TypeIncr, pSS);
      }
 
    if (value < 0)
@@ -856,7 +875,7 @@ ThotBool PresAbsBoxUserEditable (PtrAbstractBox pAb)
 	 /* c'est un pave de presentation */
 	{
 	   /* cherche la boite de presentation correspondant au pave' */
-	   pBo = &pAb->AbPSchema->PsPresentBox[pAb->AbTypeNum - 1];
+	   pBo = pAb->AbPSchema->PsPresentBox->PresBox[pAb->AbTypeNum - 1];
 	   if (pBo->PbContent == ContVariable)
 	      /* une boite dont le contenu est une variable de presentation */
 	     {
@@ -900,10 +919,11 @@ ThotBool NewVariable (int varNum, PtrSSchema pSS, PtrPSchema pSchP,
    PresVariable       *pPr1;
    PresVarItem        *pVa1;
    PresConstant       *pPres1;
-   TtAttribute        *pAttr1;
+   PtrTtAttribute         pAttr1;
    PtrElement          pEl;
    Counter            *pCo1;
    char                number[20];
+   char               *uri;
    PtrTextBuffer       pBTN, pBTA, pBTAPrec;
 
    /* sauve temporairement le contenu de ce pave de presentation */
@@ -925,9 +945,9 @@ ThotBool NewVariable (int varNum, PtrSSchema pSS, PtrPSchema pSchP,
 	  case VarText:
 	    /* constante textuelle */
 	    pPres1 = &pSchP->PsConstant[pVa1->ViConstant - 1];
-	    CopyStringToText (pPres1->PdString, pAb->AbText, &l);
+	    CopyStringToBuffer ((unsigned char *)pPres1->PdString, pAb->AbText, &l);
 	    pAb->AbVolume += l;
-	    pAb->AbLanguage = TtaGetLanguageIdFromAlphabet (pPres1->PdAlphabet);
+	    pAb->AbLang = 0; /* default */
 	    break;
 	    
 	  case VarAttrValue:
@@ -952,7 +972,7 @@ ThotBool NewVariable (int varNum, PtrSSchema pSS, PtrPSchema pSchP,
 		    pA = pEl->ElFirstAttr;	/* premier attribut */
 		    while (!found && pA != NULL)
 		      if (pA->AeAttrNum == pVa1->ViAttr &&
-			  !ustrcmp (pA->AeAttrSSchema->SsName, pSS->SsName))
+			  !strcmp (pA->AeAttrSSchema->SsName, pSS->SsName))
 			found = TRUE;
 		      else
 			pA = pA->AeNext;
@@ -969,7 +989,7 @@ ThotBool NewVariable (int varNum, PtrSSchema pSS, PtrPSchema pSchP,
 		  case AtNumAttr:
 		    /* traduit l'entier en ASCII selon le style voulu */
 		    GetCounterValue (pA->AeAttrValue, pVa1->ViStyle, number, &l);
-		    CopyStringToText (number, pAb->AbText, &l);
+		    CopyStringToBuffer ((unsigned char *)number, pAb->AbText, &l);
 		    pAb->AbVolume += l;
 		    pAb->AbCreatorAttr = pA;
 		    break;
@@ -983,12 +1003,12 @@ ThotBool NewVariable (int varNum, PtrSSchema pSS, PtrPSchema pSchP,
 		    pAb->AbCreatorAttr = pA;
 		    break;
 		  case AtReferenceAttr:
-		    CopyStringToText ("REF", pAb->AbText, &l);
+		    CopyStringToBuffer ((unsigned char *)"REF", pAb->AbText, &l);
 		    pAb->AbVolume += l;
 		    break;
 		  case AtEnumAttr:
-		    pAttr1 = &pSS->SsAttribute[pA->AeAttrNum - 1];
-		    CopyStringToText (pAttr1->AttrEnumValue[pA->AeAttrValue-1],
+		    pAttr1 = pSS->SsAttribute->TtAttr[pA->AeAttrNum - 1];
+		    CopyStringToBuffer ((unsigned char *)pAttr1->AttrEnumValue[pA->AeAttrValue-1],
 				      pAb->AbText, &l);
 		    pAb->AbVolume += l;
 		    pAb->AbCreatorAttr = pA;
@@ -1033,7 +1053,7 @@ ThotBool NewVariable (int varNum, PtrSSchema pSS, PtrPSchema pSchP,
 		}
 	    /* traduit l'entier en ASCII */
 	    GetCounterValue (i, pVa1->ViStyle, number, &l);
-	    CopyStringToText (number, pAb->AbText, &l);
+	    CopyStringToBuffer ((unsigned char *)number, pAb->AbText, &l);
 	    pAb->AbVolume += l;
 	    break;
 
@@ -1043,17 +1063,17 @@ ThotBool NewVariable (int varNum, PtrSSchema pSS, PtrPSchema pSchP,
 	    *pt = time (NULL);
 	    ptm = localtime (pt);
 	    GetCounterValue (ptm->tm_year, CntArabic, number, &l);
-	    CopyStringToText (number, pAb->AbText, &l);
+	    CopyStringToBuffer ((unsigned char *)number, pAb->AbText, &l);
 	    pAb->AbVolume += l;
-	    CopyStringToText ("/", pAb->AbText, &l);
+	    CopyStringToBuffer ((unsigned char *)"/", pAb->AbText, &l);
 	    pAb->AbVolume += l;
 	    GetCounterValue (ptm->tm_mon + 1, CntArabic, number, &l);
-	    CopyStringToText (number, pAb->AbText, &l);
+	    CopyStringToBuffer ((unsigned char *)number, pAb->AbText, &l);
 	    pAb->AbVolume += l;
-	    CopyStringToText ("/", pAb->AbText, &l);
+	    CopyStringToBuffer ((unsigned char *)"/", pAb->AbText, &l);
 	    pAb->AbVolume += l;
 	    GetCounterValue (ptm->tm_mday, CntArabic, number, &l);
-	    CopyStringToText (number, pAb->AbText, &l);
+	    CopyStringToBuffer ((unsigned char *)number, pAb->AbText, &l);
 	    pAb->AbVolume += l;
 	    break;
 
@@ -1063,43 +1083,59 @@ ThotBool NewVariable (int varNum, PtrSSchema pSS, PtrPSchema pSchP,
 	    *pt = time (NULL);
 	    ptm = localtime (pt);
 	    GetCounterValue (ptm->tm_mday, CntArabic, number, &l);
-	    CopyStringToText (number, pAb->AbText, &l);
+	    CopyStringToBuffer ((unsigned char *)number, pAb->AbText, &l);
 	    pAb->AbVolume += l;
-	    CopyStringToText ("/", pAb->AbText, &l);
+	    CopyStringToBuffer ((unsigned char *)"/", pAb->AbText, &l);
 	    pAb->AbVolume += l;
 	    GetCounterValue (ptm->tm_mon + 1, CntArabic, number, &l);
-	    CopyStringToText (number, pAb->AbText, &l);
+	    CopyStringToBuffer ((unsigned char *)number, pAb->AbText, &l);
 	    pAb->AbVolume += l;
-	    CopyStringToText ("/", pAb->AbText, &l);
+	    CopyStringToBuffer ((unsigned char *)"/", pAb->AbText, &l);
 	    pAb->AbVolume += l;
 	    GetCounterValue (ptm->tm_year, CntArabic, number, &l);
-	    CopyStringToText (number, pAb->AbText, &l);
+	    CopyStringToBuffer ((unsigned char *)number, pAb->AbText, &l);
 	    pAb->AbVolume += l;
 	    break;
 
 	  case VarDocName:
 	    /* Name du document */
-	    CopyStringToText (pDoc->DocDName, pAb->AbText, &l);
+	    CopyStringToBuffer ((unsigned char *)pDoc->DocDName, pAb->AbText, &l);
 	    pAb->AbVolume += l;
 	    break;
 
 	  case VarDirName:
 	    /* Name du document */
-	    CopyStringToText (pDoc->DocDirectory, pAb->AbText, &l);
+	    CopyStringToBuffer ((unsigned char *)pDoc->DocDirectory, pAb->AbText, &l);
 	    pAb->AbVolume += l;
 	    break;
 
 	  case VarElemName:
 	    /* Name de l'element */
 	    pEl = pAb->AbElement;
-	    CopyStringToText (pEl->ElStructSchema->SsRule[pEl->ElTypeNumber - 1].
-			      SrName, pAb->AbText, &l);
+	    CopyStringToBuffer ((unsigned char *)pEl->ElStructSchema->SsRule->SrElem[pEl->ElTypeNumber - 1]->SrName, pAb->AbText, &l);
 	    pAb->AbVolume += l;
+	    
+	    /* Show the namespace declaration for the root element and */
+	    /* when an element is not in the same namespace than its parent */
+	    if (pEl->ElStructSchema->SsRootElem == pEl->ElTypeNumber ||
+		(pEl->ElParent && pEl->ElStructSchema != pEl->ElParent->ElStructSchema))
+	      {
+		uri = GiveCurrentNsUri (pDoc, pEl);
+		if (uri != NULL)
+		  {
+		    CopyStringToBuffer ((unsigned char *)" xmlns=\"", pAb->AbText, &l);
+		    pAb->AbVolume += l;
+		    CopyStringToBuffer ((unsigned char *)uri, pAb->AbText, &l);
+		    pAb->AbVolume += l;
+		    CopyStringToBuffer ((unsigned char *)"\"", pAb->AbText, &l);
+		    pAb->AbVolume += l;
+		  }
+	      }
 	    break;
 
 	  case VarAttrName:
 	    /* Nom de l'attribut */
-	    CopyStringToText (pAttr->AeAttrSSchema->SsAttribute[pAttr->AeAttrNum - 1].AttrName, pAb->AbText, &l);
+	    CopyStringToBuffer ((unsigned char *)pAttr->AeAttrSSchema->SsAttribute->TtAttr[pAttr->AeAttrNum - 1]->AttrName, pAb->AbText, &l);
 	    pAb->AbVolume += l;
 	    break;
 
@@ -1111,7 +1147,7 @@ ThotBool NewVariable (int varNum, PtrSSchema pSS, PtrPSchema pSchP,
 	    found = FALSE;
 	    do
 	      {
-		pEl = BackSearchTypedElem (pEl, PageBreak + 1, NULL);
+		pEl = BackSearchTypedElem (pEl, PageBreak + 1, NULL, NULL);
 		if (pEl != NULL &&
 		    /* on ignore les pages qui ne concernent pas la view */
 		    pEl->ElViewPSchema == pVa1->ViView)
@@ -1124,7 +1160,7 @@ ThotBool NewVariable (int varNum, PtrSSchema pSS, PtrPSchema pSchP,
 	      i = pEl->ElPageNumber;	/* numero de la page trouvee */
 	    /* traduit le numero de page en ASCII selon le style voulu */
 	    GetCounterValue (i, pVa1->ViStyle, number, &l);
-	    CopyStringToText (number, pAb->AbText, &l);
+	    CopyStringToBuffer ((unsigned char *)number, pAb->AbText, &l);
 	    pAb->AbVolume += l;
 	    break;
 
