@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001.
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -11,7 +11,7 @@
  * Author: V. Quint (INRIA)
  *
  */
-
+#include "thot_gui.h"
 #include "libmsg.h"
 #include "thot_sys.h"
 #include "message.h"
@@ -46,6 +46,7 @@
 #include "documentapi_f.h"
 #include "exceptions_f.h"
 #include "frame_f.h"
+#include "language_f.h"
 #include "memory_f.h"
 #include "presvariables_f.h"
 #include "references_f.h"
@@ -256,7 +257,7 @@ PtrAttribute AttachAttrByExceptNum (int ExceptNum, PtrElement pEl,
 	pAttr->AeAttrSSchema = pSS;
 	pAttr->AeAttrNum = attrNum;
 	pAttr->AeDefAttr = FALSE;
-	pAttr->AeAttrType = pAttr->AeAttrSSchema->SsAttribute[pAttr->AeAttrNum - 1].AttrType;
+	pAttr->AeAttrType = pAttr->AeAttrSSchema->SsAttribute->TtAttr[pAttr->AeAttrNum - 1]->AttrType;
 	if (pAttr->AeAttrType == AtReferenceAttr)
 	  {
 	     if (pReferredEl != NULL)
@@ -297,7 +298,7 @@ static void RedisplayAbsBox (PtrAbstractBox pAbsBox, int boxNum,
       if (pAbsBox->AbTypeNum == boxNum && pAbsBox->AbPSchema == pPSchema)
 	/* c'est bien un pave du type cherche' */
 	/* recalcule la valeur de la variable de presentation */
-	if (NewVariable (pPSchema->PsPresentBox[boxNum - 1].PbContVariable,
+	if (NewVariable (pPSchema->PsPresentBox->PresBox[boxNum - 1]->PbContVariable,
 			 pAttr->AeAttrSSchema, pPSchema, pAbsBox, pAttr, pDoc))
 	  {
 	    /* la variable de presentation a change' de valeur */
@@ -358,8 +359,8 @@ void RedisplayAttribute (PtrAttribute pAttr, PtrElement pEl, PtrDocument pDoc)
 	      /* cette variable comme contenu */
 	     {
 		for (presBox = 0; presBox < pPSchema->PsNPresentBoxes; presBox++)
-		   if (pPSchema->PsPresentBox[presBox].PbContent == ContVariable
-		       && pPSchema->PsPresentBox[presBox].PbContVariable == varNum + 1)
+		   if (pPSchema->PsPresentBox->PresBox[presBox]->PbContent == ContVariable
+		       && pPSchema->PsPresentBox->PresBox[presBox]->PbContVariable == varNum + 1)
 		      /* cette boite a la variable comme contenu */
 		      /* cherche dans toutes les vues du document les paves */
 		      /* de l'element auquel correspond l'attribut qui sont */
@@ -463,7 +464,7 @@ void ApplyAttrPRulesToElem (PtrElement pEl, PtrDocument pDoc,
 
 
 /*----------------------------------------------------------------------
-   Applique au sous arbre nte' par pEl du document pDoc les     
+   Applique au sous-arbre pEl du document pDoc les     
    regles de presentation heritees de l'attribut pAttr		
    On arrete la recursion quand on rencontre un fils portant       
    lui-meme un attribut de meme type que pAttr			
@@ -471,46 +472,64 @@ void ApplyAttrPRulesToElem (PtrElement pEl, PtrDocument pDoc,
 void ApplyAttrPRulesToSubtree (PtrElement pEl, PtrDocument pDoc,
 			       PtrAttribute pAttr, PtrElement pElAttr)
 {
-   PtrElement          pChild;
-   PtrAttribute        pOldAttr;
-   PtrPSchema          pPS;
-   InheritAttrTable   *inheritedAttr;
+  PtrElement          pChild;
+  PtrAttribute        pOldAttr;
+  PtrPSchema          pPS;
+  InheritAttrTable   *inheritedAttr;
+  PtrHandlePSchema    pHd;
 
-   pOldAttr = GetAttributeOfElement (pEl, pAttr);
-   if (pOldAttr == NULL || pOldAttr == pAttr)
-     {
-	/* on traite d'abord tout le sous-arbre */
-	if (!pEl->ElTerminal)
-	   for (pChild = pEl->ElFirstChild; pChild != NULL; pChild = pChild->ElNext)
-	      if (pChild->ElStructSchema == pEl->ElStructSchema)
-		 /* same structure schema */
-		 ApplyAttrPRulesToSubtree (pChild, pDoc, pAttr, pElAttr);
+  pOldAttr = GetAttributeOfElement (pEl, pAttr);
+  if (pOldAttr == NULL || pOldAttr == pAttr)
+    {
+      /* on traite d'abord tout le sous-arbre */
+      if (!pEl->ElTerminal)
+	for (pChild = pEl->ElFirstChild; pChild != NULL;
+	     pChild = pChild->ElNext)
+	  if (pChild->ElStructSchema == pEl->ElStructSchema)
+	    /* same structure schema */
+	    ApplyAttrPRulesToSubtree (pChild, pDoc, pAttr, pElAttr);
 
-	/* on traite l'element lui-meme */
-	/* on cherche d'abord si pEl herite de pAttr */
-	pPS = PresentationSchema (pEl->ElStructSchema, pDoc);
-	if (pPS != NULL)
-	   if (pPS->PsNInheritedAttrs[pEl->ElTypeNumber - 1])
-	     {
-		/* pEl peut heriter d'un attribut */
-		if ((inheritedAttr = pPS->PsInheritedAttr[pEl->ElTypeNumber - 1]) == NULL)
-		  {
-		     /* la table d'heritage n'existe pas. On la cree */
-		     CreateInheritedAttrTable (pEl, pDoc);
-		     inheritedAttr = pPS->PsInheritedAttr[pEl->ElTypeNumber - 1];
-		  }
-		if ((*inheritedAttr)[pAttr->AeAttrNum - 1])
-		  {
-		     /* pEl herite de l'attribut pAttr */
-		     /* on lui applique la presentation correspondante */
-		     ApplyAttrPRulesToElem (pEl, pDoc, pAttr, pElAttr, TRUE);
-		  }
-	     }
-     }
+      /* on traite l'element lui-meme */
+      /* on cherche d'abord si pEl herite de pAttr */
+      pPS = PresentationSchema (pEl->ElStructSchema, pDoc);
+      pHd = NULL;
+      while (pPS)
+	{
+	  if  (pPS->PsNInheritedAttrs->Num[pEl->ElTypeNumber - 1] > 0)
+	    {
+	      /* pEl peut heriter d'au moins un attribut */
+	      if ((inheritedAttr = pPS->PsInheritedAttr->ElInherit[pEl->ElTypeNumber - 1]) == NULL)
+		{
+		  /* la table d'heritage n'existe pas. On la cree */
+		  CreateInheritedAttrTable (pEl, pPS, pDoc);
+		  inheritedAttr = pPS->PsInheritedAttr->ElInherit[pEl->ElTypeNumber - 1];
+		}
+	      if ((*inheritedAttr)[pAttr->AeAttrNum - 1])
+		{
+		  /* pEl herite de l'attribut pAttr */
+		  /* on lui applique la presentation correspondante */
+		  ApplyAttrPRulesToElem (pEl, pDoc, pAttr, pElAttr, TRUE);
+		}
+	    }
+	  /* next P schema */
+	  if (pHd == NULL)
+	    /* extension schemas have not been checked yet */
+	    /* get the first extension schema */
+	    pHd = FirstPSchemaExtension (pEl->ElStructSchema, pDoc, pEl);
+	  else
+	    /* get the next extension schema */
+	    pHd = pHd->HdNextPSchema;
+	  if (pHd == NULL)
+	    /* no more extension schemas. Stop */
+	    pPS = NULL;
+	  else
+	    pPS = pHd->HdPSchema;	
+	}
+    }
 }
 
 /*----------------------------------------------------------------------
-   Applique au sous arbre pointe' par pEl du document pDoc les     
+   Applique au sous-arbre pointe' par pEl du document pDoc les     
    regles de presentation des attributs dont les valeurs se	
    	comparent a pAttr.						
   ----------------------------------------------------------------------*/
@@ -537,11 +556,11 @@ void ApplyAttrPRules (PtrElement pEl, PtrDocument pDoc, PtrAttribute pAttr)
    pPS = PresentationSchema (pEl->ElStructSchema, pDoc);
    if (pPS != NULL)
      {
-	if ((attrValComp = pPS->PsComparAttr[pAttr->AeAttrNum - 1]) == NULL)
+	if ((attrValComp = pPS->PsComparAttr->CATable[pAttr->AeAttrNum - 1]) == NULL)
 	  {
 	     /* la table de comparaison n'existe pas. On la creee */
-	     CreateComparAttrTable (pAttr, pDoc);
-	     attrValComp = pPS->PsComparAttr[pAttr->AeAttrNum - 1];
+	     CreateComparAttrTable (pAttr, pDoc, pEl);
+	     attrValComp = pPS->PsComparAttr->CATable[pAttr->AeAttrNum - 1];
 	  }
 	if (attrValComp != NULL)
 	   for (i = pEl->ElStructSchema->SsNAttributes; i > 0; i--)
@@ -579,13 +598,12 @@ void ChangeLanguage (PtrDocument pDoc, PtrElement pEl, Language lang,
    int                 view;
    PtrElement          pChild;
    PtrAbstractBox      pAbsBox;
-   Language	       oldElLang;
 
    if (pEl == NULL)
       return;
 
    if (!pEl->ElTerminal)
-      /* l'element n'est pas une feuille, on traite son sous-arbre */
+      /* this element is not a leaf */
      {
 	pChild = pEl->ElFirstChild;
 	while (pChild != NULL)
@@ -596,48 +614,40 @@ void ChangeLanguage (PtrDocument pDoc, PtrElement pEl, Language lang,
 	     pChild = pChild->ElNext;
 	  }
      }
-   else
-      /* this element is a leaf */
-      if (pEl->ElLeafType == LtText && pEl->ElLanguage != lang)
-       /* That's a text leaf in a different language */
-       /* Make the change only if the leaf is empty or if its language uses
-	  the same alphabet as the new language, i.e. don't change the
-	  character set of some existing text. */
-    	 /* change the language of the element */
-       if (pEl->ElTextLength == 0 ||
-	   TtaGetAlphabet (pEl->ElLanguage) == TtaGetAlphabet (lang))
-	{
-	  oldElLang = pEl->ElLanguage;
-	   pEl->ElLanguage = lang;
-	   /* parcourt toutes les vues du document pour changer les */
-	   /* paves de l'element */
-	   for (view = 0; view < MAX_VIEW_DOC; view++)
-	      if (pEl->ElAbstractBox[view] != NULL)
-		 /* l'element a au moins un pave dans la vue */
-		{
-		   pAbsBox = pEl->ElAbstractBox[view];
-		   /* saute les paves de presentation de l'element */
-		   while (pAbsBox->AbPresentationBox &&
-			  pAbsBox->AbElement == pEl)
-		      pAbsBox = pAbsBox->AbNext;
-		   if (!pAbsBox->AbDead)
-		      /* traite le pave' principal de l'element */
-		      /* change la langue du pave */
-		     {
-			pAbsBox->AbLanguage = lang;
-			if (force ||
-			    TtaGetAlphabet (oldElLang) != TtaGetAlphabet(lang))
-			  /* cette langue s'ecrit dans un alphabet different */
-			  /* ou la langue est forcee */
-			  {
-			    pAbsBox->AbChange = TRUE;
-			    /* conserve le pointeur sur le pave a reafficher */
-			    pDoc->DocViewModifiedAb[view] =
-			      Enclosing (pAbsBox, pDoc->DocViewModifiedAb[view]);
-			  }
-		     }
-		}
-	}
+   else if (pEl->ElLeafType == LtText &&
+	    pEl->ElLanguage != lang &&
+	    (pEl->ElLanguage >= TtaGetFirstUserLanguage () || force))
+     /* That's a text leaf in a different language */
+     /* Changes the language of the element:
+	When lang < TtaGetFirstUserLanguage (), lang actually represents a
+	script, not a real language. */
+     if (pEl->ElTextLength == 0 ||
+	 TtaGetScript (pEl->ElLanguage) == TtaGetScript (lang))
+       {
+	 pEl->ElLanguage = lang;
+	 /* parcourt toutes les vues du document pour changer les */
+	 /* paves de l'element */
+	 for (view = 0; view < MAX_VIEW_DOC; view++)
+	   if (pEl->ElAbstractBox[view] != NULL)
+	     /* l'element a au moins un pave dans la vue */
+	     {
+	       pAbsBox = pEl->ElAbstractBox[view];
+	       /* saute les paves de presentation de l'element */
+	       while (pAbsBox->AbPresentationBox &&
+		      pAbsBox->AbElement == pEl)
+		 pAbsBox = pAbsBox->AbNext;
+	       if (!pAbsBox->AbDead)
+		 /* traite le pave' principal de l'element */
+		 /* change la langue du pave */
+		 {
+		   pAbsBox->AbLang = lang;
+		   pAbsBox->AbChange = TRUE;
+		   /* conserve le pointeur sur le pave a reafficher */
+		   pDoc->DocViewModifiedAb[view] =
+		     Enclosing (pAbsBox, pDoc->DocViewModifiedAb[view]);
+		 }
+	     }
+       }
 }
 
 
@@ -722,38 +732,56 @@ void RemoveInheritedAttrPresent (PtrElement pEl, PtrDocument pDoc,
   PtrElement          pChild;
   PtrPSchema          pPS;
   InheritAttrTable   *inheritedAttr;
+  PtrHandlePSchema    pHd;
 
   if (GetAttributeOfElement (pEl, pAttr) == NULL)
     {
       /* pEl does not have an attribute of that type */
       /* process the subtree */
       if (!pEl->ElTerminal)
-	for (pChild = pEl->ElFirstChild; pChild != NULL; pChild = pChild->ElNext)
+	for (pChild = pEl->ElFirstChild; pChild != NULL;
+	     pChild = pChild->ElNext)
 	  if (pChild->ElStructSchema == pEl->ElStructSchema)
 	    /* same structure schema */
 	    RemoveInheritedAttrPresent (pChild, pDoc, pAttr, pElAttr);
 
       /* process element pEl itself */
       pPS = PresentationSchema (pEl->ElStructSchema, pDoc);
-      if (pPS != NULL)
-	if (pPS->PsNInheritedAttrs[pEl->ElTypeNumber - 1])
-	  {
-	    /* pEl can inherit some presentation rules from attributes */
-	    if ((inheritedAttr = pPS->PsInheritedAttr[pEl->ElTypeNumber - 1]) == NULL)
-	      {
-		/* la table d'heritage n'existe pas. On la cree */
-		CreateInheritedAttrTable (pEl, pDoc);
-		inheritedAttr = pPS->PsInheritedAttr[pEl->ElTypeNumber - 1];
-	      }
-	    if ((*inheritedAttr)[pAttr->AeAttrNum - 1])
-	      RemoveAttrPresentation (pEl, pDoc, pAttr, pElAttr, TRUE, NULL);
-	  }
+      pHd = NULL;
+      while (pPS)
+	{
+	  if (pPS->PsNInheritedAttrs->Num[pEl->ElTypeNumber - 1])
+	    {
+	      /* pEl can inherit some presentation rules from attributes */
+	      if ((inheritedAttr = pPS->PsInheritedAttr->ElInherit[pEl->ElTypeNumber - 1]) == NULL)
+		{
+		  /* la table d'heritage n'existe pas. On la cree */
+		  CreateInheritedAttrTable (pEl, pPS, pDoc);
+		  inheritedAttr = pPS->PsInheritedAttr->ElInherit[pEl->ElTypeNumber - 1];
+		}
+	      if ((*inheritedAttr)[pAttr->AeAttrNum - 1])
+		RemoveAttrPresentation (pEl, pDoc, pAttr, pElAttr, TRUE, NULL);
+	    }
+	  /* next P schema */
+	  if (pHd == NULL)
+	    /* extension schemas have not been checked yet */
+	    /* get the first extension schema */
+	    pHd = FirstPSchemaExtension (pEl->ElStructSchema, pDoc, pEl);
+	  else
+	    /* get the next extension schema */
+	    pHd = pHd->HdNextPSchema;
+	  if (pHd == NULL)
+	    /* no more extension schemas. Stop */
+	    pPS = NULL;
+	  else
+	    pPS = pHd->HdPSchema;
+	}
     }
 }
 
 
 /*----------------------------------------------------------------------
-   Retire du sous arbre pointe' par pEl du document pDoc les     
+   Retire du sous-arbre pointe' par pEl du document pDoc les     
    regles de presentation provenant de la comparaison avec		
    l'attribut pAttr.						
    On arrete la recursion quand on rencontre un fils portant       
@@ -762,7 +790,7 @@ void RemoveInheritedAttrPresent (PtrElement pEl, PtrDocument pDoc,
 void RemoveComparAttrPresent (PtrElement pEl, PtrDocument pDoc, PtrAttribute pAttr)
 {
    PtrElement          pChild;
-   PtrPSchema            pPS;
+   PtrPSchema          pPS;
    ComparAttrTable    *attrValComp;
    int                 i;
    PtrAttribute        pCompAttr;
@@ -782,11 +810,11 @@ void RemoveComparAttrPresent (PtrElement pEl, PtrDocument pDoc, PtrAttribute pAt
    pPS = PresentationSchema (pEl->ElStructSchema, pDoc);
    if (pPS != NULL)
      {
-	if ((attrValComp = pPS->PsComparAttr[pAttr->AeAttrNum - 1]) == NULL)
+	if ((attrValComp = pPS->PsComparAttr->CATable[pAttr->AeAttrNum - 1]) == NULL)
 	  {
 	     /* la table de comparaison n'existe pas. On la creee */
-	     CreateComparAttrTable (pAttr, pDoc);
-	     attrValComp = pPS->PsComparAttr[pAttr->AeAttrNum - 1];
+	     CreateComparAttrTable (pAttr, pDoc, pEl);
+	     attrValComp = pPS->PsComparAttr->CATable[pAttr->AeAttrNum - 1];
 	  }
 	if (attrValComp != NULL)
 	   for (i = pEl->ElStructSchema->SsNAttributes; i > 0; i--)
@@ -883,229 +911,262 @@ PtrAttribute AddAttrToElem (PtrElement pEl, PtrAttribute pNewAttr,
    Egalement utilise' pour supprimer un attribut et desappliquer les
    regles de presentation correspondantes.
   ----------------------------------------------------------------------*/
-void AttachAttrWithValue (PtrElement pEl, PtrDocument pDoc, PtrAttribute pNewAttr)
+void AttachAttrWithValue (PtrElement pEl, PtrDocument pDoc,
+			  PtrAttribute pNewAttr, ThotBool history)
 {
-   PtrAttribute        pAttr, pAttrAsc, pAttrNext;
-   PtrElement          pChild, pElAttr;
-   PtrPSchema          pPS;
-   ThotBool            suppress, compare, inherit, mandatory, create, allowed;
-   NotifyAttribute     notifyAttr;
+  PtrAttribute        pAttr, pAttrAsc, pAttrNext;
+  PtrElement          pChild, pElAttr;
+  PtrPSchema          pPS;
+  ThotBool            suppress, compare, inherit, mandatory, create, allowed;
+  NotifyAttribute     notifyAttr;
 
-   if (pNewAttr == NULL || pNewAttr->AeAttrNum == 0)
-     /* it's not a valid attribute */
-     return;
-   mandatory = FALSE;
-   pAttrNext = NULL;
-   /* l'element porte-t-il deja un attribut du meme type ? */
-   pAttr = AttributeValue (pEl, pNewAttr);
-   create = (pAttr == NULL);
+  if (pNewAttr == NULL || pNewAttr->AeAttrNum == 0)
+    /* it's not a valid attribute */
+    return;
+  mandatory = FALSE;
+  pAttrNext = NULL;
 
-   /* peut-on appliquer l'attribut a l'element ? */
-   allowed = CanAssociateAttr (pEl, pAttr, pNewAttr, &mandatory);
-   if (!allowed && create)
-      return;
+  /* l'element porte-t-il deja un attribut du meme type ? */
+  pAttr = AttributeValue (pEl, pNewAttr);
+  create = (pAttr == NULL);
 
-   /* est-ce une suppression d'attribut */
-   suppress = TRUE;
-   switch (pNewAttr->AeAttrType)
-	 {
-	    case AtEnumAttr:
-	       if (pNewAttr->AeAttrValue != 0)
-		  suppress = FALSE;
-	       break;
-
-	    case AtNumAttr:
-	       if (pNewAttr->AeAttrValue >= -MAX_INT_ATTR_VAL
-		   && pNewAttr->AeAttrValue <= MAX_INT_ATTR_VAL)
-		  suppress = FALSE;
-	       break;
-
-	    case AtTextAttr:
-	       if (pNewAttr->AeAttrText != NULL)
-		  suppress = FALSE;
-	       break;
-
-	    case AtReferenceAttr:
-	       if (pNewAttr->AeAttrReference != NULL)
-		  if (pNewAttr->AeAttrReference->RdReferred != NULL)
-		     suppress = FALSE;
-	       break;
-
-	    default:
-	       break;
-	 }
-   if (suppress)
-     {
-	if (pAttr == NULL)
-	   /* suppression d'un attribut inexistant */
-	   return;
-	if (!mandatory)
-	   /* si c'est un attribut Langue sur un element racine, il est obligatoire */
-	   if (pEl->ElParent == NULL)
-	      if (pNewAttr->AeAttrNum == 1)
-		 mandatory = TRUE;
-	if (mandatory)
-	  /* suppression d'un attribut obligatoire. Interdit */
+  /* est-ce une suppression d'attribut */
+  suppress = FALSE;
+  switch (pNewAttr->AeAttrType)
+    {
+    case AtEnumAttr:
+      if (pNewAttr->AeAttrValue == 0)
+	suppress = TRUE;
+      else
+	if (pAttr && pNewAttr->AeAttrValue == pAttr->AeAttrValue)
+	  /* the existing attribute already has this value */
 	  return;
-     }
+      break;
 
-   /* faut-il traiter des heritages et comparaisons d'attributs */
-   inherit = FALSE;
-   compare = FALSE;
-   pPS = PresentationSchema (pNewAttr->AeAttrSSchema, pDoc);
-   if (pPS != NULL)
-     {
-       inherit = (pPS->PsNHeirElems[pNewAttr->AeAttrNum - 1] != 0);
-       compare = (pPS->PsNComparAttrs[pNewAttr->AeAttrNum - 1] != 0);
-     }
-   if (inherit || compare)
-      /* cherche le premier attribut de meme type sur un ascendant de pEl */
-      pAttrAsc = GetTypedAttrAncestor (pEl, pNewAttr->AeAttrNum,
-				       pNewAttr->AeAttrSSchema, &pElAttr);
-   else
-      pAttrAsc = NULL;
+    case AtNumAttr:
+      if (pNewAttr->AeAttrValue < -MAX_INT_ATTR_VAL ||
+	  pNewAttr->AeAttrValue > MAX_INT_ATTR_VAL)
+	suppress = TRUE;
+      else
+	if (pAttr && pNewAttr->AeAttrValue == pAttr->AeAttrValue)
+	  /* the existing attribut already has this value */
+	  return;
+      break;
 
-   /* prepare et envoie l'evenement pour l'application */
-   if (create)
-      notifyAttr.event = TteAttrCreate;
-   else if (suppress)
-      notifyAttr.event = TteAttrDelete;
-   else
-      notifyAttr.event = TteAttrModify;
-   notifyAttr.document = (Document) IdentDocument (pDoc);
-   notifyAttr.element = (Element) pEl;
-   notifyAttr.attribute = (Attribute) pAttr;
-   notifyAttr.attributeType.AttrSSchema = (SSchema) (pNewAttr->AeAttrSSchema);
-   notifyAttr.attributeType.AttrTypeNum = pNewAttr->AeAttrNum;
-   if (!CallEventAttribute (&notifyAttr, TRUE))
-     {
-	/* l'application accepte l'operation */
-	/* on supprime sur pEl et son sous arbre les regles de presentation
-	   anciennes liees a l'heritage d'un attribut du type pAttr porte'
-	   soit par pEl soit par un ascendant */
-	if (pAttr)
-	  {
-	    /* register the attribute in history */
-	    if (ThotLocalActions[T_attraddhistory] != NULL)
-	      (*ThotLocalActions[T_attraddhistory]) (pAttr, pEl, pDoc, TRUE, FALSE);
-	    /* detache l'attribut de l'element s'il y a lieu */
-	    pAttrNext = pAttr->AeNext;
-	    /* supprime l'attribut */
-	    RemoveAttribute (pEl, pAttr);
-	    /* heritage et comparaison sont lies a un attribut de pEl */
-	    /* On supprime d'abord les regles de presentation liees a
-	       l'attribut sur l'element lui-meme */
-	    RemoveAttrPresentation (pEl, pDoc, pAttr, pEl, FALSE, NULL);
-	    /* indique que le document a ete modifie' */
-	    /* un changement d'attribut vaut dix caracteres saisis */
-	    SetDocumentModified (pDoc, TRUE, 10);
-	    /* On supprime de pEl de son sous-arbre  la presentation venant
-	       de l'heritage de cet attribut par le sous-arbre, s'il existe
-	       des elements heritants de celui-ci */
-	    if (inherit)
-	       RemoveInheritedAttrPresent (pEl, pDoc, pAttr, pEl);
-	    /* On supprime des elements du sous arbre pEl la presentation
-	       venant de la comparaison d'un attribut du sous-arbre avec ce
-	       type d'attribut */
-	    if (!pEl->ElTerminal && compare)
-	      for (pChild = pEl->ElFirstChild; pChild != NULL; pChild = pChild->ElNext)
-		RemoveComparAttrPresent (pChild, pDoc, pAttr);
-	    DeleteAttribute (pEl, pAttr);
-	  }
+    case AtTextAttr:
+      if (pNewAttr->AeAttrText == NULL)
+	suppress = TRUE;
+      else
+	if (pAttr && TextsEqual (pAttr->AeAttrText, pNewAttr->AeAttrText))
+	  return;
+      break;
 
-	else if (pAttrAsc)
-	  {
-	     /* heritage et comparaison sont dus a un attribut d'un ascendant
-	        de pEl */
-	     /* on supprime du sous arbre pEl la presentation venant de
-	        l'heritage de cet attribut par le sous-arbre, s'il existe des
-	        elements heritants de celui-ci */
-	     if (inherit)
-		RemoveInheritedAttrPresent (pEl, pDoc, pAttrAsc, pElAttr);
-	     /* on supprime du sous-arbre pEl la presentation venant de la
-	        comparaison d'un attribut du sous-arbre avec ce type d'attribut */
-	     if (compare)
-		RemoveComparAttrPresent (pEl, pDoc, pAttrAsc);
-	  }
+    case AtReferenceAttr:
+      if (pNewAttr->AeAttrReference == NULL ||
+	  pNewAttr->AeAttrReference->RdReferred == NULL)
+	  suppress = TRUE;
+      else
+	if (pAttr && pAttr->AeAttrReference &&
+	    pAttr->AeAttrReference->RdReferred ==
+	                      pNewAttr->AeAttrReference->RdReferred)
+	  return;
+      break;
 
-	if (inherit || compare)
-	   /* cherche le premier attribut de meme type sur un ascendant de pEl */
-	   pAttrAsc = GetTypedAttrAncestor (pEl, pNewAttr->AeAttrNum,
+    default:
+      break;
+    }
+
+  /* peut-on appliquer l'attribut a l'element ? */
+  allowed = CanAssociateAttr (pEl, pAttr, pNewAttr, &mandatory);
+  if (!allowed && !suppress)
+    return;
+
+  if (suppress)
+    {
+      if (pAttr == NULL)
+	/* suppression d'un attribut inexistant */
+	return;
+      if (!mandatory)
+	/* si c'est un attribut Langue sur un element racine,
+	   il est obligatoire */
+	if (pEl->ElParent == NULL)
+	  if (pNewAttr->AeAttrNum == 1)
+	    mandatory = TRUE;
+      if (mandatory)
+	/* suppression d'un attribut obligatoire. Interdit */
+	return;
+    }
+
+  /* faut-il traiter des heritages et comparaisons d'attributs */
+  inherit = FALSE;
+  compare = FALSE;
+  pPS = PresentationSchema (pNewAttr->AeAttrSSchema, pDoc);
+  if (pPS != NULL)
+    {
+      inherit = (pPS->PsNHeirElems->Num[pNewAttr->AeAttrNum - 1] != 0);
+      compare = (pPS->PsNComparAttrs->Num[pNewAttr->AeAttrNum - 1] != 0);
+    }
+  if (inherit || compare)
+    /* cherche le premier attribut de meme type sur un ascendant de pEl */
+    pAttrAsc = GetTypedAttrAncestor (pEl, pNewAttr->AeAttrNum,
+				     pNewAttr->AeAttrSSchema, &pElAttr);
+  else
+    pAttrAsc = NULL;
+
+  /* prepare et envoie l'evenement pour l'application */
+  if (create)
+    notifyAttr.event = TteAttrCreate;
+  else if (suppress)
+    notifyAttr.event = TteAttrDelete;
+  else
+    notifyAttr.event = TteAttrModify;
+  notifyAttr.document = (Document) IdentDocument (pDoc);
+  notifyAttr.element = (Element) pEl;
+  notifyAttr.info = 0; /* not sent by undo */
+  notifyAttr.attribute = (Attribute) pAttr;
+  notifyAttr.attributeType.AttrSSchema = (SSchema) (pNewAttr->AeAttrSSchema);
+  notifyAttr.attributeType.AttrTypeNum = pNewAttr->AeAttrNum;
+  if (!CallEventAttribute (&notifyAttr, TRUE))
+    {
+      /* l'application accepte l'operation */
+      /* on supprime sur pEl et son sous arbre les regles de presentation
+	 anciennes liees a l'heritage d'un attribut du type pAttr porte'
+	 soit par pEl soit par un ascendant */
+      if (pAttr)
+	{
+	  /* register the attribute in history */
+	  if (history && ThotLocalActions[T_attraddhistory] != NULL)
+	    (*(Proc5)ThotLocalActions[T_attraddhistory]) (
+		(void *)pAttr,
+	       	(void *)pEl,
+	       	(void *)pDoc,
+	       	(void *)TRUE,
+		(void *)FALSE);
+	  /* detache l'attribut de l'element s'il y a lieu */
+	  pAttrNext = pAttr->AeNext;
+	  /* supprime l'attribut */
+	  RemoveAttribute (pEl, pAttr);
+	  /* heritage et comparaison sont lies a un attribut de pEl */
+	  /* On supprime d'abord les regles de presentation liees a
+	     l'attribut sur l'element lui-meme */
+	  RemoveAttrPresentation (pEl, pDoc, pAttr, pEl, FALSE, NULL);
+	  /* indique que le document a ete modifie' */
+	  /* un changement d'attribut vaut dix caracteres saisis */
+	  SetDocumentModified (pDoc, TRUE, 10);
+	  /* On supprime de pEl de son sous-arbre  la presentation venant
+	     de l'heritage de cet attribut par le sous-arbre, s'il existe
+	     des elements heritants de celui-ci */
+	  if (inherit)
+	    RemoveInheritedAttrPresent (pEl, pDoc, pAttr, pEl);
+	  /* On supprime des elements du sous arbre pEl la presentation
+	     venant de la comparaison d'un attribut du sous-arbre avec ce
+	     type d'attribut */
+	  if (!pEl->ElTerminal && compare)
+	    for (pChild = pEl->ElFirstChild; pChild != NULL;
+		 pChild = pChild->ElNext)
+	      RemoveComparAttrPresent (pChild, pDoc, pAttr);
+	  DeleteAttribute (pEl, pAttr);
+	}
+
+      else if (pAttrAsc)
+	{
+	  /* heritage et comparaison sont dus a un attribut d'un ascendant
+	     de pEl */
+	  /* on supprime du sous arbre pEl la presentation venant de
+	     l'heritage de cet attribut par le sous-arbre, s'il existe des
+	     elements heritants de celui-ci */
+	  if (inherit)
+	    RemoveInheritedAttrPresent (pEl, pDoc, pAttrAsc, pElAttr);
+	  /* on supprime du sous-arbre pEl la presentation venant de la
+	     comparaison d'un attribut du sous-arbre avec ce type d'attribut */
+	  if (compare)
+	    RemoveComparAttrPresent (pEl, pDoc, pAttrAsc);
+	}
+
+      if (inherit || compare)
+	/* cherche le premier attribut de meme type sur un ascendant de pEl */
+	pAttrAsc = GetTypedAttrAncestor (pEl, pNewAttr->AeAttrNum,
 					 pNewAttr->AeAttrSSchema, &pElAttr);
-	else
-	   pAttrAsc = NULL;
+      else
+	pAttrAsc = NULL;
 
-	/* on met l'attribut */
-	if (!suppress)
-	  {
-	    /* add a copy of the new attribute before pAttrNext */
-	    pAttr = AddAttrToElem (pEl, pNewAttr, pAttrNext);
-	    /* register the attribute in history */
-	    if (ThotLocalActions[T_attraddhistory] != NULL)
-	      (*ThotLocalActions[T_attraddhistory]) (pAttr, pEl, pDoc, FALSE, TRUE);
-	    /* indique que le document a ete modifie' */
-	    /* un changement d'attribut vaut dix caracteres saisis */
-	    SetDocumentModified (pDoc, TRUE, 10);
-	  }
+      /* on met l'attribut */
+      if (!suppress)
+	{
+	  /* add a copy of the new attribute before pAttrNext */
+	  pAttr = AddAttrToElem (pEl, pNewAttr, pAttrNext);
+	  /* register the attribute in history */
+	  if (history && ThotLocalActions[T_attraddhistory] != NULL)
+	    (*(Proc5)ThotLocalActions[T_attraddhistory]) (
+		(void *)pAttr,
+	       	(void *)pEl,
+	       	(void *)pDoc,
+	       	(void *)FALSE,
+		(void *)TRUE);
+	  /* indique que le document a ete modifie' */
+	  /* un changement d'attribut vaut dix caracteres saisis */
+	  SetDocumentModified (pDoc, TRUE, 10);
+	}
 
-	else
-	   /* suppression */
-	  {
-	     /* valide les modifications */
-	     AbstractImageUpdated (pDoc);
-	     /* la suppression est maintenant prise en compte dans les
-	        copies-inclusions de l'element */
-	     RedisplayCopies (pEl, pDoc, TRUE);
-	  }
+      else
+	/* suppression */
+	{
+	  /* valide les modifications */
+	  AbstractImageUpdated (pDoc);
+	  /* la suppression est maintenant prise en compte dans les
+	     copies-inclusions de l'element */
+	  RedisplayCopies (pEl, pDoc, TRUE);
+	}
 
-	/* on applique les regles de presentation */
-	if (pAttr != NULL && !suppress)
-	  {
-	     /* applique les regles du nouvel attribut */
-	     /* applique d'abord les regles de presentation associees a
-	        l'attribut sur l'element lui-meme */
-	     ApplyAttrPRulesToElem (pEl, pDoc, pAttr, pEl, FALSE);
-	     /* applique les regles de presentation venant de l'heritage de
-	        cet attribut par le sous-arbre s'il existe des elements qui
-	        heritent */
-	     if (inherit)
-		ApplyAttrPRulesToSubtree (pEl, pDoc, pAttr, pEl);
-	     /* applique sur les elements du sous-arbre les regles de
-	        presentation venant de la comparaison d'un attribut du
-	        sous-arbre avec ce type d'attribut */
-	     if (!pEl->ElTerminal && compare)
-		for (pChild = pEl->ElFirstChild; pChild != NULL; pChild = pChild->ElNext)
-		   ApplyAttrPRules (pChild, pDoc, pAttr);
-	     if (pAttr->AeAttrType == AtNumAttr)
-		/* s'il s'agit d'un attribut initialisant un compteur, il */
-		/* faut mettre a jour les boites utilisant ce compteur */
-		UpdateCountersByAttr (pEl, pNewAttr, pDoc);
+      /* on applique les regles de presentation */
+      if (pAttr != NULL && !suppress)
+	{
+	  /* applique les regles du nouvel attribut */
+	  /* applique d'abord les regles de presentation associees a
+	     l'attribut sur l'element lui-meme */
+	  ApplyAttrPRulesToElem (pEl, pDoc, pAttr, pEl, FALSE);
+	  /* applique les regles de presentation venant de l'heritage de
+	     cet attribut par le sous-arbre s'il existe des elements qui
+	     heritent */
+	  if (inherit)
+	    ApplyAttrPRulesToSubtree (pEl, pDoc, pAttr, pEl);
+	  /* applique sur les elements du sous-arbre les regles de
+	     presentation venant de la comparaison d'un attribut du
+	     sous-arbre avec ce type d'attribut */
+	  if (!pEl->ElTerminal && compare)
+	    for (pChild = pEl->ElFirstChild; pChild != NULL;
+		 pChild = pChild->ElNext)
+	      ApplyAttrPRules (pChild, pDoc, pAttr);
+	  if (pAttr->AeAttrType == AtNumAttr)
+	    /* s'il s'agit d'un attribut initialisant un compteur, il */
+	    /* faut mettre a jour les boites utilisant ce compteur */
+	    UpdateCountersByAttr (pEl, pNewAttr, pDoc);
 
-	     /* le nouvel attribut est pris en compte dans les 
-	        les copies-inclusions de l'element */
-	     RedisplayCopies (pEl, pDoc, TRUE);
-	  }
-	else if (pAttrAsc != NULL && suppress)
-	  {
-	     /* applique les regles venant de l'heritage ou de la comparaison
-	        avec un ascendant */
-	     /* applique sur les elements du sous-arbre les regles de
-	        presentation venant de l'heritage de cet attribut par le
-	        sous-arbre, s'il existe des elements qui heritent */
-	     ApplyAttrPRulesToSubtree (pEl, pDoc, pAttrAsc, pElAttr);
-	     /* applique sur les elements du sous-arbre les regles de
-	        presentation venant de la comparaison d'un attribut de
-	        sous-arbre avec ce type d'attribut */
-	     ApplyAttrPRules (pEl, pDoc, pAttrAsc);
-	  }
+	  /* le nouvel attribut est pris en compte dans les 
+	     les copies-inclusions de l'element */
+	  RedisplayCopies (pEl, pDoc, TRUE);
+	}
+      else if (pAttrAsc != NULL && suppress)
+	{
+	  /* applique les regles venant de l'heritage ou de la comparaison
+	     avec un ascendant */
+	  /* applique sur les elements du sous-arbre les regles de
+	     presentation venant de l'heritage de cet attribut par le
+	     sous-arbre, s'il existe des elements qui heritent */
+	  ApplyAttrPRulesToSubtree (pEl, pDoc, pAttrAsc, pElAttr);
+	  /* applique sur les elements du sous-arbre les regles de
+	     presentation venant de la comparaison d'un attribut de
+	     sous-arbre avec ce type d'attribut */
+	  ApplyAttrPRules (pEl, pDoc, pAttrAsc);
+	}
 
-	/* prepare et envoie a l'application l'evenement TteAttrCreate.Post */
-	if (notifyAttr.event == TteAttrCreate)
-	  notifyAttr.attribute = (Attribute) pAttr;
-	else if (notifyAttr.event == TteAttrDelete)
-	  notifyAttr.attribute = NULL;
-	CallEventAttribute (&notifyAttr, FALSE);
-     }
+      /* prepare et envoie a l'application l'evenement TteAttrCreate.Post */
+      if (notifyAttr.event == TteAttrCreate)
+	notifyAttr.attribute = (Attribute) pAttr;
+      else if (notifyAttr.event == TteAttrDelete)
+	notifyAttr.attribute = NULL;
+      CallEventAttribute (&notifyAttr, FALSE);
+    }
 }
 
 /*----------------------------------------------------------------------
@@ -1220,7 +1281,9 @@ Attribute TtaGetAttribute (Element element, AttributeType attributeType)
 int TtaGetTextAttributeLength (Attribute attribute)
 {
    int                 length;
-   PtrTextBuffer       pBT;
+  unsigned char       c[10], *ptr;
+  int                 i;
+   PtrTextBuffer       pBuf;
 
    UserErrorCode = 0;
    length = 0;
@@ -1230,11 +1293,17 @@ int TtaGetTextAttributeLength (Attribute attribute)
      TtaError (ERR_invalid_attribute_type);
    else
      {
-       pBT = ((PtrAttribute) attribute)->AeAttrText;
-       while (pBT != NULL)
+       pBuf = ((PtrAttribute) attribute)->AeAttrText;
+       while (pBuf != NULL)
 	 {
-	   length += pBT->BuLength;
-	   pBT = pBT->BuNext;
+	  i = 0;
+	  while (i < pBuf->BuLength)
+	    {
+	      ptr = c;
+	      length += TtaWCToMBstring (pBuf->BuContent[i], &ptr);
+	      i++;
+	    }
+	   pBuf = pBuf->BuNext;
 	 }
      }
    return length;
@@ -1261,5 +1330,47 @@ void TtaGiveTextAttributeValue (Attribute attribute, char *buffer, int *length)
    else if (((PtrAttribute) attribute)->AeAttrType != AtTextAttr)
     TtaError (ERR_invalid_attribute_type);
   else
-    CopyTextToString (((PtrAttribute) attribute)->AeAttrText, buffer, length);
+    *length = CopyBuffer2MBs (((PtrAttribute) attribute)->AeAttrText, 0,
+			     (unsigned char*)buffer, *length);
+}
+
+/* ----------------------------------------------------------------------
+   TtaGetAttributeKind
+
+   Returns the kind of the attribute: 0 = Enumerate, 1 = Integer, 2 = Text,
+   3 = CsReference.
+   Parameter:
+   attType: type of the attribute.
+   ---------------------------------------------------------------------- */
+int TtaGetAttributeKind (AttributeType attType)
+{
+  PtrSSchema          pSS;
+  int                 attrKind = 2;
+
+  UserErrorCode = 0;
+  if (attType.AttrSSchema == NULL || attType.AttrTypeNum <= 0 ||
+      attType.AttrTypeNum > ((PtrSSchema)(attType.AttrSSchema))->SsNAttributes)
+    TtaError (ERR_invalid_parameter);
+  else
+    {
+      pSS = (PtrSSchema)(attType.AttrSSchema);
+      switch (pSS->SsAttribute->TtAttr[attType.AttrTypeNum - 1]->AttrType)
+	{
+	case AtEnumAttr:
+	  attrKind = 0;
+	  break;
+	case AtNumAttr:
+	  attrKind = 1;
+	  break;
+	case AtTextAttr:
+	  attrKind = 2;
+	  break;
+	case AtReferenceAttr:
+	  attrKind = 3;
+	  break;
+	default:
+	  break;
+	}
+    } 
+  return attrKind;
 }
