@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -10,7 +10,7 @@
  * Author: V. Quint (INRIA)
  *
  */
-
+#include "thot_gui.h"
 #include "thot_sys.h"
 #include "constmedia.h"
 #include "typemedia.h"
@@ -24,7 +24,7 @@
 #include "message.h"
 
 #undef THOT_EXPORT
-#define THOT_EXPORT
+#define THOT_EXPORT extern /* to avoid redefinition */
 #include "edit_tv.h"
 
 #include "applicationapi_f.h"
@@ -59,7 +59,7 @@ void TtaSetReference (Element element, Document document, Element target,
    PtrDocument         pRefDoc;
    PtrReference       *pRef;
    ReferenceType       refType;
-   int                 saveNbCar;
+   /* int              saveNbCar; */
    ThotBool            ok;
   
    pRef = NULL;
@@ -124,7 +124,7 @@ void TtaSetReference (Element element, Document document, Element target,
 		     GetReference (pRef);
 		  (*pRef)->RdElement = (PtrElement) element;
 		  (*pRef)->RdTypeRef = refType;
-		  saveNbCar = LoadedDocument[document - 1]->DocNTypedChars;
+		  /* saveNbCar = LoadedDocument[document - 1]->DocNTypedChars; */
 		  ok = SetReference ((PtrElement) element, NULL,
 				     (PtrElement) target,
 				     LoadedDocument[document - 1], pRefDoc,
@@ -133,7 +133,7 @@ void TtaSetReference (Element element, Document document, Element target,
 		    CopyIncludedElem ((PtrElement)element, LoadedDocument[document - 1]);
 		  /* an API function is not supposed to change */
 		  /* the number of characters typed by the user */
-		  LoadedDocument[document - 1]->DocNTypedChars = saveNbCar;
+		  /* LoadedDocument[document - 1]->DocNTypedChars = saveNbCar; */
 		  if (!ok)
 		       TtaError (ERR_cannot_set_link);
 	       }
@@ -161,7 +161,7 @@ Element TtaNewInclusion (Document document, Element target, Document targetDocum
 {
    PtrElement          inclusion;
    PtrDocument         pRefDoc;
-   int                 saveNbCar;
+   /* int              saveNbCar; */
 
    UserErrorCode = 0;
    inclusion = NULL;
@@ -197,7 +197,7 @@ Element TtaNewInclusion (Document document, Element target, Document targetDocum
 	     ((PtrElement) target)->ElReferredDescr = NewReferredElDescr (pRefDoc);
 	     ((PtrElement) target)->ElReferredDescr->ReReferredElem = (PtrElement) target;
 	  }
-	saveNbCar = LoadedDocument[document - 1]->DocNTypedChars;
+	/* saveNbCar = LoadedDocument[document - 1]->DocNTypedChars; */
 	if (SetReference (inclusion, NULL, (PtrElement) target,
 			LoadedDocument[document - 1], pRefDoc, TRUE, FALSE))
 	   CopyIncludedElem (inclusion, LoadedDocument[document - 1]);
@@ -205,7 +205,7 @@ Element TtaNewInclusion (Document document, Element target, Document targetDocum
 	     TtaError (ERR_cannot_set_link);
 	/* an API function is not supposed to change */
 	/* the number of characters typed by the user */
-	LoadedDocument[document - 1]->DocNTypedChars = saveNbCar;
+	/* LoadedDocument[document - 1]->DocNTypedChars = saveNbCar; */
      }
    return ((Element) inclusion);
 }
@@ -264,7 +264,7 @@ void TtaSetAttributeReference (Attribute attribute, Element element,
    PtrDocument         pDoc, pRefDoc;
    PtrReference        ref;
    PtrAttribute        pAttr;
-   int                 saveNbCar;
+   /* int              saveNbCar; */
    ThotBool            ok;
 
    UserErrorCode = 0;
@@ -339,13 +339,13 @@ void TtaSetAttributeReference (Attribute attribute, Element element,
 		       ref->RdElement = (PtrElement) element;
 		       ref->RdAttribute = (PtrAttribute) attribute;
 		       ref->RdTypeRef = RefFollow;
-		       saveNbCar = LoadedDocument[document - 1]->DocNTypedChars;
+		       /* saveNbCar = LoadedDocument[document - 1]->DocNTypedChars; */
 		       ok = SetReference (NULL, (PtrAttribute) attribute,
 					  (PtrElement) target,
 					  pDoc, pRefDoc, TRUE, FALSE);
 		       /* an API function is not supposed to change */
 		       /* the number of characters typed by the user */
-		       LoadedDocument[document - 1]->DocNTypedChars = saveNbCar;
+		       /* LoadedDocument[document - 1]->DocNTypedChars = saveNbCar; */
 		       if (!ok)
 			  TtaError (ERR_cannot_set_link);
 		    }
@@ -404,7 +404,7 @@ void TtaCopyAttributeReference (Attribute attribute, Element element,
    element: the element to be up to dated.
    document: the document that contains the element.
    ---------------------------------------------------------------------- */
-void		TtaCopyIncludedElem (Element element, Document document)
+void TtaCopyIncludedElem (Element element, Document document)
 {
    UserErrorCode = 0;
 
@@ -532,7 +532,7 @@ void TtaGiveReferredElement (Element element, Element *target,
    Return value:
    1 = the type is a reference, 0 = the type is not a reference.
   ----------------------------------------------------------------------*/
-int                 TtaIsElementTypeReference (ElementType elementType)
+int TtaIsElementTypeReference (ElementType elementType)
 {
    int                 result;
 
@@ -545,7 +545,7 @@ int                 TtaIsElementTypeReference (ElementType elementType)
 	TtaError (ERR_invalid_element_type);
    else
      {
-	if (((PtrSSchema) (elementType.ElSSchema))->SsRule[elementType.ElTypeNum - 1].SrConstruct == CsReference)
+	if (((PtrSSchema) (elementType.ElSSchema))->SsRule->SrElem[elementType.ElTypeNum - 1]->SrConstruct == CsReference)
 	   result = 1;
      }
    return result;
@@ -681,12 +681,64 @@ int TtaIsElementReferred (Element element)
 	   /* The element has a referenced element descriptor */
 	   if (((PtrElement) element)->ElReferredDescr->ReFirstReference != NULL ||
 	       ((PtrElement) element)->ElReferredDescr->ReExtDocRef != NULL)
-	      /* The element is well referenced */
+	      /* The element is referenced */
 	      result = 1;
      }
    return result;
 }
 
+/*----------------------------------------------------------------------
+   TtaGetFirstReferringAttribute
+
+   Return the first element that refers to the given element through
+   an attribute of type attrType.
+   Parameter:
+   element: the element of interest
+   attrType: type of the reference attribute considered
+   Return value:
+   the referring element if it exists, NULL if element is not referred
+   by this type of attribute.
+  ----------------------------------------------------------------------*/
+Element TtaGetFirstReferringAttribute (Element element, AttributeType attrType)
+{
+  PtrElement      referringEl;
+  PtrReference    ref;
+
+  UserErrorCode = 0;
+  referringEl = NULL;
+  if (element == NULL)
+    TtaError (ERR_invalid_parameter);
+  else if (attrType.AttrSSchema == NULL)
+    TtaError (ERR_invalid_attribute_type);
+  else if (attrType.AttrTypeNum < 1 ||
+	   attrType.AttrTypeNum > ((PtrSSchema)(attrType.AttrSSchema))->SsNAttributes)
+    TtaError (ERR_invalid_attribute_type);
+  else
+    /* parameters are OK */
+    {
+      if (((PtrElement) element)->ElReferredDescr)
+	/* The element has a referenced element descriptor */
+	{
+	  ref = ((PtrElement) element)->ElReferredDescr->ReFirstReference;
+	  /* check all references to this element */
+	  while (ref && !referringEl)
+	    {
+	      if (ref->RdAttribute)
+		/* this is a reference from an attribute. Check the attribute*/
+		{
+		  if (ref->RdAttribute->AeAttrSSchema == (PtrSSchema)(attrType.AttrSSchema) &&
+		      ref->RdAttribute->AeAttrNum == attrType.AttrTypeNum)
+		    /* that's the attribute we are looking for */
+		    referringEl = ref->RdElement;
+		}
+	      if (!referringEl)
+		ref = ref->RdNext;
+	    }
+	}
+    }
+  return ((Element) referringEl);
+}
+
 /*----------------------------------------------------------------------
    TtaSameReferenceAttributes
 
