@@ -126,14 +126,14 @@ static ThotBool RestTransferContent (Element oldElem, Element newElem,
       case RestTypeTexte:
       case RestTypeImage:
         len = TtaGetTextLength (oldElem);
-        buffer = TtaGetMemory (len++);
-        TtaGiveTextContent (oldElem, buffer, &len, &lang); 
+        buffer = (char *)TtaGetMemory (len++);
+        TtaGiveTextContent (oldElem, (unsigned char *)buffer, &len, &lang); 
 #ifdef DEBUG
 	strncpy (msgbuf, buffer, 20);
 	strcat (msgbuf,"...");
 	printf("%s\n",msgbuf);
 #endif
-        TtaSetTextContent (newElem, buffer, lang, doc);
+        TtaSetTextContent (newElem, (unsigned char *)buffer, lang, doc);
         result = !TtaGetErrorCode();
         break;
       case RestTypeGraphique:
@@ -195,8 +195,13 @@ static ThotBool IsPresentInAggregate (Element elParent, int typeNum,
   
   typeParent = TtaGetElementType (elParent);
   card = TtaGetCardinalOfType (typeParent);
-  typesArray = TtaGetMemory (card * sizeof (ElementType));
-  TtaGiveConstructorsOfType (&typesArray, &card, typeParent);
+  if (card > 0)
+    {
+      typesArray = (ElementType *)TtaGetMemory (card * sizeof (ElementType));
+      TtaGiveConstructorsOfType (&typesArray, &card, typeParent);
+    }
+  else
+    typesArray = NULL;
   elChild = TtaGetFirstChild (elParent);
   if (elChild == NULL)
     {
@@ -470,12 +475,12 @@ static ThotBool RestTransformElement (Restruct restr, Element elSource,
   Element elTarget, prev, elParent;
   TypeTree target, targetParent;
   ThotBool result = FALSE;
-# ifndef _WINDOWS
+# ifndef _WINGUI
 #ifdef THIS_IS_GARBAGE
   Element elChild;
   TypeTree treeChild;
 # endif /* THIS_IS_GARBAGE */
-# endif /* _WINDOWS */
+# endif /* _WINGUI */
   
 #ifdef DEBUG
   ElementType dbgType;
@@ -609,11 +614,11 @@ static ThotBool RestTransformChildren (Restruct restr, Element srcElem,
   ElementType elType;
   ThotBool result = TRUE;
   TypeTree childTree;
-# ifndef _WINDOWS 
+# ifndef _WINGUI 
 #ifdef THIS_IS_GARBAGE
   Element elCopy, prev;
 #endif /* THIS_IS_GARBAGE */
-# endif /* _WINDOWS */
+# endif /* _WINGUI */
 
   /* traite succesivement chacun des fils de oldElem */
   elChild = TtaGetFirstChild (srcElem);
