@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001.
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -11,8 +11,7 @@
  * Author: I. Vatton (INRIA)
  *
  */
-
-#include "ustring.h"
+#include "thot_gui.h"
 #include "thot_sys.h"
 #include "constmedia.h"
 #include "typemedia.h"
@@ -37,7 +36,7 @@
    InitializeFrameParams initialise le seuil de visibilite et le facteur de     
    zoom de la fenetre frame.                                       
   ----------------------------------------------------------------------*/
-void     InitializeFrameParams (int frame, int Visibilite, int Zoom)
+void InitializeFrameParams (int frame, int Visibilite, int Zoom)
 {
    ViewFrame          *pFrame;
 
@@ -54,7 +53,7 @@ void     InitializeFrameParams (int frame, int Visibilite, int Zoom)
    GetFrameParams retourne les valeurs courantes du seuil de visibilite et
    du facteur de zoom de la fenetre.                       
   ----------------------------------------------------------------------*/
-void       GetFrameParams (int frame, int *Visibilite, int *Zoom)
+void GetFrameParams (int frame, int *Visibilite, int *Zoom)
 {
    ViewFrame          *pFrame;
 
@@ -71,7 +70,7 @@ void       GetFrameParams (int frame, int *Visibilite, int *Zoom)
    SetFrameParams reevalue la vue designee apres decalage des tailles     
    ou/et modification du seuil de visibilite des boites.   
   ----------------------------------------------------------------------*/
-void              SetFrameParams (int frame, int Visibilite, int Zoom)
+void SetFrameParams (int frame, int Visibilite, int Zoom)
 {
    PtrAbstractBox      pAb;
    PtrAbstractBox      pv1;
@@ -100,13 +99,13 @@ void              SetFrameParams (int frame, int Visibilite, int Zoom)
 	 /* On sauvegarde la selection courante dans la fenetre */
 	 {
 	   pViewSel = &pFrame->FrSelectionBegin;
-	   if (pViewSel->VsBox != NULL)
+	   if (pViewSel->VsBox)
 	     {
 	       pv1 = pViewSel->VsBox->BxAbstractBox;
-	       if (pViewSel->VsIndBuf == 0)
+	       if (pViewSel->VsIndBox == 0)
 		 c1 = 0;
 	       else
-		 c1 = pViewSel->VsBox->BxIndChar + pViewSel->VsIndBox + 1;
+		 c1 = pViewSel->VsBox->BxFirstChar + pViewSel->VsIndBox;
 	       /* On annule le debut de selection */
 	       pViewSel->VsBox = NULL;
 	     }
@@ -116,10 +115,10 @@ void              SetFrameParams (int frame, int Visibilite, int Zoom)
 	   if (pViewSel->VsBox != NULL)
 	     {
 	       pvN = pViewSel->VsBox->BxAbstractBox;
-	       if (pViewSel->VsIndBuf == 0)
+	       if (pViewSel->VsIndBox == 0)
 		 cN = 0;
 	       else
-		 cN = pViewSel->VsBox->BxIndChar + pViewSel->VsIndBox + 1;
+		 cN = pViewSel->VsBox->BxFirstChar + pViewSel->VsIndBox;
 	       /* On annule la fin de selection */
 	       pViewSel->VsBox = NULL;
 	     }
@@ -131,11 +130,13 @@ void              SetFrameParams (int frame, int Visibilite, int Zoom)
 	   /* On sauvegarde la position de la fenetre dans le document */
 	   x = pFrame->FrXOrg;
 	   y = pFrame->FrYOrg;
+	   pAb->AbDead = TRUE;
 	   RemoveBoxes (pAb, TRUE, frame);
 	   ThotFreeFont (frame);
 	   /* On libere les polices de caracteres utilisees */
 	   pFrame->FrAbstractBox = NULL;
 	   /* Recreation de la vue */
+	   pAb->AbDead = FALSE;
 	   h = 0;
 	   (void) ChangeConcreteImage (frame, &h, pAb);
 	   /* On restaure la position de la fenetre dans le document */
@@ -153,7 +154,7 @@ void              SetFrameParams (int frame, int Visibilite, int Zoom)
 		 InsertViewSelMarks (frame, pvN, 0, cN, FALSE, TRUE, FALSE);
 	       /* On visualise la selection que l'on vient de poser */
 	     }
-	   DefClip (frame, -1, -1, -1, -1);
+ 	   DefClip (frame, -1, -1, -1, -1);
 	   DisplayFrame (frame);
 	 }
        ReadyToDisplay = TRUE;
