@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001.
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -13,6 +13,11 @@
  *          R. Guetari (W3C/INRIA) - Unicode and Windows version
  */
 
+#ifdef _WX
+  #include "wx/wx.h"
+#endif /* _WX */
+
+#include "thot_gui.h"
 #include "thot_sys.h"
 #include "constmedia.h"
 #include "constmenu.h"
@@ -30,50 +35,86 @@
 #include "memory_f.h"
 #include "inites_f.h"
 #include "registry_f.h"
-#ifdef _WINDOWS
-#include "wininclude.h"
-#endif /* _WINDOWS*/
+
+#ifdef _WINGUI
+  #include "wininclude.h"
+#endif /* _WINGUI*/
+
+#ifdef _WX
+  #include "wx/colour.h"
+  #include "wx/memory.h"
+#endif /* _WX */
 
 static ThotColorStruct def_colrs[256];
 static int             allocation_index[256];
 static int             have_colors = 0;
 
 
-#ifndef _WINDOWS
+#if defined(_MOTIF) || defined(_GTK)
 /*----------------------------------------------------------------------
    FindOutColor finds the closest color by allocating it, or picking
    an already allocated color.
   ----------------------------------------------------------------------*/
-static void FindOutColor (Display *dsp, Colormap colormap,
+static void FindOutColor (ThotDisplay *dsp, Colormap colormap,
 			  ThotColorStruct *colr)
 {
-   int                 i, match;
+   int                 i;
    int                 rd, gd, bd, dist, mindist;
    int                 cindx;
    int                 NumCells;
+#ifdef _MOTIF
+   int                 match;
+#endif /* #ifdef _MOTIF */
+   
+#ifdef _GTK
+   gboolean            match;
+#endif /* _GTK */
 
 #ifdef _GTK
- match =  gdk_colormap_alloc_color (colormap, colr, FALSE, TRUE);
-#else /* _GTK */
+ match = gdk_colormap_alloc_color ((GdkColormap *)colormap, (GdkColor *)colr, FALSE, TRUE);
+#endif /* _GTK */
+
+#ifdef _MOTIF 
  match = XAllocColor (dsp, colormap, colr);
-#endif /* _GTK */
-   NumCells = 0;
-   if (match == 0)
-     {
+#endif /* #ifdef _MOTIF */
+
+ NumCells = 0;
+
+#ifdef _MOTIF
+  if (match == 0)
+#endif /* #ifdef _MOTIF */
+       
 #ifdef _GTK
-        NumCells = gdk_colormap_get_system_size ();
-#else /* _GTK */
-	NumCells = XDisplayCells (dsp, TtScreen);
+  if (!match)
+#endif /* !_GTK */
+  {
+       
+#ifdef _GTK
+    NumCells = gdk_colormap_get_system_size (); 
 #endif /* _GTK */
-	if (!have_colors)
+    
+#ifdef _MOTIF    
+    NumCells = XDisplayCells (dsp, TtScreen);
+#endif /* #ifdef _MOTIF */
+    
+    if (!have_colors)
 	  {
 	     for (i = 0; i < NumCells; i++)
 		def_colrs[i].pixel = i;
+
 #ifdef _GTK
-	     match =  gdk_colormap_alloc_colors (colormap, &def_colrs[0], NumCells, FALSE, TRUE, NULL); 
-#else /* _GTK */
-	     XQueryColors (dsp, colormap, def_colrs, NumCells);
+	     match = gdk_colormap_alloc_colors ((GdkColormap *)colormap,
+						(GdkColor *)&def_colrs[0],
+						NumCells,
+						FALSE,
+						TRUE, 
+						NULL); 
 #endif /* _GTK */
+
+#ifdef _MOTIF       
+	     XQueryColors (dsp, colormap, def_colrs, NumCells);
+#endif /* _MOTIF */
+       
 	     have_colors = 1;
 	  }
 	mindist = 196608;	/* 256 * 256 * 3 */
@@ -113,7 +154,8 @@ static void FindOutColor (Display *dsp, Colormap colormap,
 	   have_colors = 0;
      }
 }
-#endif /* !_WINDOWS */
+
+#endif /* #if defined(_MOTIF) || defined(_GTK) */
 
 
 /*----------------------------------------------------------------------
@@ -121,9 +163,26 @@ static void FindOutColor (Display *dsp, Colormap colormap,
   ----------------------------------------------------------------------*/
 static void InstallColor (int i)
 {
-#ifdef _WINDOWS
+#ifdef _WINGUI
    Pix_Color[i] = RGB (RGB_Table[i].red, RGB_Table[i].green, RGB_Table[i].blue);
-#else  /* _WINDOWS */
+#endif /* _WINGUI */
+   
+#if defined(_WX)
+   if (Pix_Color[i])
+     {
+       Pix_Color[i]->Set( RGB_Table[i].red,
+			  RGB_Table[i].green,
+			  RGB_Table[i].blue );
+     }
+   else
+     {
+       Pix_Color[i] = new wxColour( RGB_Table[i].red,
+				    RGB_Table[i].green,
+				    RGB_Table[i].blue );
+     }
+#endif /* #if defined(_WX) */
+   
+#if defined(_MOTIF) || defined(_GTK)
    ThotColorStruct     col;
 
    if (Color_Table[i] != NULL)
@@ -134,11 +193,11 @@ static void InstallColor (int i)
 	col.blue  = RGB_Table[i].blue * 256;
 
 	/* Find closest color */
-	FindOutColor (TtDisplay, TtCmap, &col);
+	FindOutColor (TtDisplay, (Colormap)TtCmap, &col);
 	Pix_Color[i] = col.pixel;
 	/* TODO: find the nearest color */
      }
-#endif /* _WINDOWS */
+#endif /* #if defined(_MOTIF) || defined(_GTK)*/
 }
 
 /*----------------------------------------------------------------------
@@ -164,7 +223,8 @@ static void ApproximateColors (void)
 	 * the fourth row should be allocated or the whole line is
 	 * white.
 	 */
-	col = Pix_Color[line * 8 + 4];
+#ifndef _WX
+        col = Pix_Color[line * 8 + 4];
 	for (b = 4; b < 8; b++)
 	   if (Pix_Color[line * 8 + b] != white)
 	      col = Pix_Color[line * 8 + b];
@@ -177,6 +237,31 @@ static void ApproximateColors (void)
 	      col = Pix_Color[line * 8 + b];
 	   else
 	      Pix_Color[line * 8 + b] = col;
+#else /* #ifndef _WX */
+        col = Pix_Color[line * 8 + 4];
+	for (b = 4; b < 8; b++)
+	   if (*Pix_Color[line * 8 + b] != *white)
+	      col = Pix_Color[line * 8 + b];
+	   else
+	   {
+	      if (Pix_Color[line * 8 + b])
+		*Pix_Color[line * 8 + b] = *col;
+	      else
+		Pix_Color[line * 8 + b] = new wxColour(*col);
+	   }	
+
+	col = Pix_Color[line * 8 + 4];
+	for (b = 4; b >= 0; b--)
+	   if (*Pix_Color[line * 8 + b] != *white)
+	      col = Pix_Color[line * 8 + b];
+	   else
+	   {
+	      if (Pix_Color[line * 8 + b])
+		*Pix_Color[line * 8 + b] = *col;
+	      else
+		Pix_Color[line * 8 + b] = new wxColour(*col);
+	   }
+#endif /* #ifndef _WX */
      }
 }
 
@@ -186,15 +271,17 @@ static void ApproximateColors (void)
 void         FreeDocColors ()
 {
 #ifndef _WIN_PRINT
-#ifdef _WINDOWS
+
+#ifdef _WINGUI
   /* free extended colors */
   if (!TtIsTrueColor && TtCmap && !DeleteObject (TtCmap))
     WinErrorBox (WIN_Main_Wd, "FreeDocColors (1)");
   TtCmap = 0;
-#else /* _WINDOWS */
-  int        i;
+#endif /* _WINGUI */
 
 #ifdef _GTK
+  int        i;
+
   /* free standard colors */
   gdk_colors_free (TtCmap, &Pix_Color[0], NColors, (gulong)0 );
 
@@ -202,7 +289,11 @@ void         FreeDocColors ()
   for (i = 0; i < NbExtColors; i++)
     if (ExtColor[i])
       gdk_colors_free (TtCmap, &ExtColor[i], 1, (gulong)0);
-#else /* _GTK */
+#endif /* _GTK */
+
+#ifdef _MOTIF  
+  int        i;
+
   /* free standard colors */
   XFreeColors (TtDisplay, TtCmap, Pix_Color, NColors, (unsigned long) 0);
 
@@ -210,8 +301,28 @@ void         FreeDocColors ()
   for (i = 0; i < NbExtColors; i++)
     if (ExtColor[i])
       XFreeColors (TtDisplay, TtCmap, &ExtColor[i], 1, (unsigned long) 0);
-#endif /* _GTK */
-#endif /* _WINDOWS */
+#endif /* _MOTIF */
+
+#if defined(_WX)
+  int        i;
+  
+  /* free standard colors */
+  for (i = 0; i < NColors; i++)
+    if (Pix_Color && Pix_Color[i])
+      {
+	delete ((wxColour *)Pix_Color[i]);
+	Pix_Color[i] = NULL;
+      }
+  
+  /* free extended colors */
+  for (i = 0; i < NbExtColors; i++)
+    if (ExtColor && ExtColor[i])
+      {
+	delete ((wxColour *)ExtColor[i]);
+	ExtColor[i] = NULL;
+      }
+#endif /* #if defined(_WX) */
+
 #endif /* _WIN_PRINT */
 
   TtaFreeMemory (ExtRGB_Table);
@@ -229,7 +340,7 @@ void         FreeDocColors ()
   ----------------------------------------------------------------------*/
 void InitDocColors (char *name)
 {
-#ifdef _WINDOWS
+#ifdef _WINGUI
   PALETTEENTRY        sysPalEntries[256];
   int                 palSize, max;
   int                 best;
@@ -298,7 +409,9 @@ void InitDocColors (char *name)
 			     sysPalEntries[j].peBlue);
 	}
     }
-#else  /* _WINDOWS */
+#endif /* _WINGUI */
+
+#if defined(_GTK) || defined(_MOTIF) || defined(_WX)
   int                 i, j, k;
   char               *value;
   ThotBool            reducecolor;
@@ -323,17 +436,33 @@ void InitDocColors (char *name)
 
   /* set up black and white Pixels */
 #ifdef _GTK
-  gdk_color_white (TtCmap, &gdkwhite);  
-  gdk_color_black (TtCmap, &gdkblack);
+  gdk_color_white (TtCmap, (GdkColor *)&gdkwhite);  
+  gdk_color_black (TtCmap, (GdkColor *)&gdkblack);
   Pix_Color[0] = gdkwhite.pixel;
   Pix_Color[1] = gdkblack.pixel;
-#else /* _GTK */
+#endif /* _GTK */
+
+#ifdef _MOTIF
   Pix_Color[0] = WhitePixel (TtDisplay, DefaultScreen (TtDisplay));
   Pix_Color[1] = BlackPixel (TtDisplay, DefaultScreen (TtDisplay));
-#endif /* _GTK */
+#endif /* _MOTIF */
+
+#ifdef _WX
+  Pix_Color[0] = new wxColour(_T("WHITE"));
+  Pix_Color[1] = new wxColour(_T("BLACK"));;
+#endif /* _WX */
+
+#ifndef _WX  
   /* clean up everything with white */
   for (i = 2; i < NColors; i++)
     Pix_Color[i] = Pix_Color[0];
+#else /* #ifndef _WX */
+  /* clean up everything with white */
+  for (i = 2; i < NColors; i++)
+  {
+    Pix_Color[i] = new wxColour( *Pix_Color[0] );  
+  }
+#endif /* #ifndef _WX */
   
   /* setup greyscale colors */
   for (i = 2; i < 8; i++)
@@ -350,8 +479,19 @@ void InitDocColors (char *name)
   if (colormap_full)
     {
       for (j = 1; j <= (NColors / 8); j++)
-	for (i = j * 8, k = 0; (i < NColors) && (k < 8); i++, k++)
-	  Pix_Color[i] = Pix_Color[j * 8 + 4];
+	      for (i = j * 8, k = 0; (i < NColors) && (k < 8); i++, k++)
+#ifndef _WX  
+                Pix_Color[i] = Pix_Color[j * 8 + 4];
+#else /* #ifndef _WX */
+              {         
+                if (Pix_Color[i])
+		    *Pix_Color[i] = *Pix_Color[j * 8 + 4];
+		else
+		  Pix_Color[i] = new wxColour( *Pix_Color[j * 8 + 4] );  
+              }
+#endif /* #ifndef _WX */
+  
+	        
       return;
     }
   
@@ -394,10 +534,16 @@ void InitDocColors (char *name)
     Max_Extend_Colors = 256;
   else
     Max_Extend_Colors = 512;
+
   ExtRGB_Table = (RGBstruct *) TtaGetMemory (Max_Extend_Colors * sizeof (RGBstruct));
+  memset( ExtRGB_Table, 0, Max_Extend_Colors * sizeof (RGBstruct) );
+
   ExtColor = (ThotColor *) TtaGetMemory (Max_Extend_Colors * sizeof (ThotColor));
+  memset( ExtColor, 0, Max_Extend_Colors * sizeof (ThotColor) );
+
   ExtCount_Table = (int *) TtaGetMemory (Max_Extend_Colors * sizeof (int));
-#endif /* _WINDOWS */
+  memset( ExtCount_Table, 0, Max_Extend_Colors * sizeof (int) );  
+#endif /* #if defined(_GTK) || defined(_MOTIF) || defined(_WX) */
 }
 
 /*----------------------------------------------------------------------
@@ -425,7 +571,7 @@ char *ColorName (int num)
   ----------------------------------------------------------------------*/
 ThotColor ColorPixel (int num)
 {
-#ifdef _GTK 
+#ifdef _GTK
   unsigned short   red, green, blue;
   ThotColor        color;
 
@@ -438,14 +584,16 @@ ThotColor ColorPixel (int num)
   color <<= 8;
   color |= (blue & 0xFF);
   return (color);
-#else /* _GTK */
+#endif /* _GTK */
+  
+#if defined(_MOTIF) || defined(_WINGUI) || defined(_WX)
    if (num < NColors && num >= 0)
     return (Pix_Color[num]);
   else if (num < NColors + NbExtColors && num >= 0)
     return (ExtColor[num - NColors]);
   else
     return ((ThotColor) 0);
-#endif /* _GTK */
+#endif /* #if defined(_MOTIF) || defined(_WINGUI) || defined(_WX) */
 }
 
 /*----------------------------------------------------------------------
@@ -454,21 +602,30 @@ ThotColor ColorPixel (int num)
 void             TtaFreeThotColor (int num)
 {
   if (num < NColors + NbExtColors && num >= NColors)
-    {
-      num -= NColors;
-      if (ExtCount_Table[num] == 1)
-	{
-#ifndef _WINDOWS
+  {
+    num -= NColors;
+    if (ExtCount_Table[num] == 1)
+	  {
+
+#if defined(_WX)
+	    if (ExtColor[num])
+	      delete ExtColor[num];
+	    ExtColor[num] = (ThotColor) 0;
+#endif /* defined(_WX) */
+
 #ifdef _GTK
-	  gdk_colors_free (TtCmap, &ExtColor[num], 1, (gulong)0);
-#else /* _GTK */
-	  XFreeColors (TtDisplay, TtCmap, &ExtColor[num], 1, (unsigned long) 0);
+	    gdk_colors_free (TtCmap, &ExtColor[num], 1, (gulong)0);
+	    ExtColor[num] = (ThotColor) 0;	    
 #endif /* _GTK */
-#endif /* _WINDOWS */
-	  ExtColor[num] = (ThotColor) 0;
-	}
-      ExtCount_Table[num]--;
-    }
+
+#ifdef _MOTIF    
+	    XFreeColors (TtDisplay, TtCmap, &ExtColor[num], 1, (unsigned long) 0);
+    	    ExtColor[num] = (ThotColor) 0;
+#endif /* _GTK */
+
+	  }
+    ExtCount_Table[num]--;
+  }
 }
 
 /*----------------------------------------------------------------------
@@ -483,11 +640,17 @@ int TtaGetThotColor (unsigned short red, unsigned short green,
    int                 i, prev;
    unsigned int        dsquare;
    unsigned int        best_dsquare = (unsigned int) -1;
-#ifndef _WINDOWS
+
+#if defined(_MOTIF) || defined(_GTK)
    ThotColorStruct     col;
-#endif /* _WINDOWS */
+#endif /* #if defined(_MOTIF) || defined(_GTK) */
+   
    ThotBool            found;
 
+#if defined(_NOGUI)
+  return 0;
+#endif /* #ifdef _NOGUI */  
+   
    /*
     * lookup for the color number among the color set allocated
     * by the application.
@@ -539,33 +702,69 @@ int TtaGetThotColor (unsigned short red, unsigned short green,
        if (best_dsquare != 0 && prev < Max_Extend_Colors)
 	 {
 	   /* try to allocate the right color */
-#ifdef _WINDOWS
+#ifdef _WINGUI
 	   ExtColor[prev] = RGB ((BYTE)red, (BYTE)green, (BYTE)blue);
-#else  /* _WINDOWS */
+#endif  /* _WINGUI */
+
+#if defined(_WX)
+	   if (ExtColor[prev])
+	     {
+	       ExtColor[prev]->Set( red,
+				    green,
+				    blue );
+	     }
+	   else
+	     {
+	       ExtColor[prev] = new wxColour( red,
+					      green,
+					      blue );
+	     }
+#endif /* #if defined(_WX) */
+
+#if defined(_MOTIF) || defined(_GTK)
 	   col.red   = red * 256;
 	   col.green = green * 256;
 	   col.blue  = blue * 256;
-	   FindOutColor (TtDisplay, TtCmap, &col);
+	   FindOutColor (TtDisplay, (Colormap)TtCmap, &col);
 	   ExtColor[prev] = col.pixel;
-#endif /* _WINDOWS */
+#endif /* #if defined(_MOTIF) || defined(_GTK) */
+
+#ifndef _WX	   
 	   /* check if this color is already in the table */
 	   found = FALSE;
 	   for (i = 0; i < NColors && !found; i++)
 	     found = (ExtColor[prev] == Pix_Color[i]);
 	   for (i = 0; i < NbExtColors && !found; i++)
 	     found = (ExtColor[prev] == ExtColor[i]);
-
+#else /* #ifndef _WX */
+	   /* check if this color is already in the table */
+	   found = FALSE;
+	   for (i = 0; i < NColors && !found; i++)
+	     found = (*ExtColor[prev] == *Pix_Color[i]);
+	   for (i = 0; i < NbExtColors && !found; i++)
+	     found = (*ExtColor[prev] == *ExtColor[i]);	   
+#endif /* #ifndef _WX */
+	   
 	   if (!found)
 	     {
-#ifdef _WINDOWS
+#ifdef _WINGUI
 	       ExtRGB_Table[prev].red = red;
 	       ExtRGB_Table[prev].green = green;
 	       ExtRGB_Table[prev].blue = blue;
-#else  /* _WINDOWS */
+#endif  /* _WINGUI */
+
+#ifdef _WX
+	       ExtRGB_Table[prev].red = red;
+	       ExtRGB_Table[prev].green = green;
+	       ExtRGB_Table[prev].blue = blue;
+#endif  /* _WX */
+	       
+#if defined(_MOTIF) || defined(_GTK)
 	       ExtRGB_Table[prev].red = col.red / 256;
 	       ExtRGB_Table[prev].green = col.green / 256;
 	       ExtRGB_Table[prev].blue = col.blue / 256;
-#endif /* _WINDOWS */
+#endif /* #if defined(_MOTIF) || defined(_GTK) */
+         
 	       best = prev + NColors;
 	       ExtCount_Table[prev] = 1;
 	       if (prev == NbExtColors)
@@ -582,7 +781,7 @@ int TtaGetThotColor (unsigned short red, unsigned short green,
    return (best);
 }
 
-#ifdef _WINDOWS
+#ifdef _WINGUI
 /*----------------------------------------------------------------------
   GetSystemColorIndex returns the index in the system palette
   ----------------------------------------------------------------------*/
@@ -610,7 +809,7 @@ unsigned char GetSystemColorIndex (unsigned short red, unsigned short green,
     }
    return (best);
 }
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
 
 /*----------------------------------------------------------------------
    TtaGiveThotRGB returns the Red Green and Blue values corresponding
@@ -720,15 +919,15 @@ int PatternNumber (char *name)
    parameters fg, bg, and motif indicate respectively
    the drawing color, background color and the pattern.
   ----------------------------------------------------------------------*/
-Pixmap CreatePattern (int disp, int fg, int bg, int motif)
+ThotPixmap CreatePattern (int disp, int fg, int bg, int motif)
 {
-   unsigned long       FgPixel;
-   unsigned long       BgPixel;
-   Pixmap              pixmap;
-#ifdef _WINDOWS
+   ThotColor           FgPixel;
+   ThotColor           BgPixel;
+   ThotPixmap          pixmap;
+#ifdef _WINGUI
    BITMAP              bitmap = {0, 0, 0, 1, 1, 0};
    HBITMAP             hBitmap;
-#endif /* _WINDOWS */
+#endif /* _WINGUI */
 
 #ifdef _GTK
    ThotColorStruct     gdkFgPixel;
@@ -742,7 +941,7 @@ Pixmap CreatePattern (int disp, int fg, int bg, int motif)
    gdkBgPixel.pixel = gdk_rgb_xpixel_from_rgb (BgPixel);
 #endif /* _GTK */
 
-#ifdef _WINDOWS
+#ifdef _WINGUI
    if (WIN_LastBitmap != 0)
      {
 	if (!DeleteObject (WIN_LastBitmap))
@@ -1051,131 +1250,134 @@ Pixmap CreatePattern (int disp, int fg, int bg, int motif)
      }
    /* WIN_LastBitmap = hBitmap; */
    pixmap = hBitmap;
-#else /* _WINDOWS */
+#endif /* _WINGUI */
+   
 #ifdef _GTK
    switch (motif)
      {
      case 1:
-       pixmap = gdk_pixmap_create_from_data (DefaultDrawable, (char *) gray8_bits, gray8_width,
-					     gray8_height, TtWDepth, &gdkFgPixel, &gdkBgPixel);
+       pixmap = (ThotPixmap)gdk_pixmap_create_from_data (DefaultDrawable, (char *) gray8_bits, gray8_width,
+					     gray8_height, TtWDepth, (GdkColor *)&gdkFgPixel, (GdkColor *)&gdkBgPixel);
        break;
      case 2:
-       pixmap = gdk_pixmap_create_from_data (DefaultDrawable, (char *) gray0_bits, gray0_width,
-					     gray0_height, TtWDepth, &gdkFgPixel, &gdkBgPixel);
+       pixmap = (ThotPixmap)gdk_pixmap_create_from_data (DefaultDrawable, (char *) gray0_bits, gray0_width,
+					     gray0_height, TtWDepth, (GdkColor *)&gdkFgPixel, (GdkColor *)&gdkBgPixel);
        break;
      case 3:
-       pixmap = gdk_pixmap_create_from_data (DefaultDrawable, (char *) gray1_bits, gray1_width,
-					     gray1_height, TtWDepth, &gdkFgPixel, &gdkBgPixel);
+       pixmap = (ThotPixmap)gdk_pixmap_create_from_data (DefaultDrawable, (char *) gray1_bits, gray1_width,
+					     gray1_height, TtWDepth, (GdkColor *)&gdkFgPixel, (GdkColor *)&gdkBgPixel);
        break;
      case 4:
-       pixmap = gdk_pixmap_create_from_data (DefaultDrawable, (char *) gray2_bits, gray2_width,
-					     gray2_height, TtWDepth, &gdkFgPixel, &gdkBgPixel);
+       pixmap = (ThotPixmap)gdk_pixmap_create_from_data (DefaultDrawable, (char *) gray2_bits, gray2_width,
+					     gray2_height, TtWDepth, (GdkColor *)&gdkFgPixel, (GdkColor *)&gdkBgPixel);
        break;
      case 5:
-       pixmap = gdk_pixmap_create_from_data (DefaultDrawable, (char *) gray3_bits, gray3_width,
-					     gray3_height, TtWDepth, &gdkFgPixel, &gdkBgPixel);
+       pixmap = (ThotPixmap)gdk_pixmap_create_from_data (DefaultDrawable, (char *) gray3_bits, gray3_width,
+					     gray3_height, TtWDepth, (GdkColor *)&gdkFgPixel, (GdkColor *)&gdkBgPixel);
        break;
      case 6:
-       pixmap = gdk_pixmap_create_from_data (DefaultDrawable, (char *) gray4_bits, gray4_width,
-					     gray4_height, TtWDepth, &gdkFgPixel, &gdkBgPixel);
+       pixmap = (ThotPixmap)gdk_pixmap_create_from_data (DefaultDrawable, (char *) gray4_bits, gray4_width,
+					     gray4_height, TtWDepth, (GdkColor *)&gdkFgPixel, (GdkColor *)&gdkBgPixel);
        break;
      case 7:
-       pixmap = gdk_pixmap_create_from_data (DefaultDrawable, (char *) gray5_bits, gray5_width,
-					     gray5_height, TtWDepth, &gdkFgPixel, &gdkBgPixel);
+       pixmap = (ThotPixmap)gdk_pixmap_create_from_data (DefaultDrawable, (char *) gray5_bits, gray5_width,
+					     gray5_height, TtWDepth, (GdkColor *)&gdkFgPixel, (GdkColor *)&gdkBgPixel);
        break;
      case 8:
-       pixmap = gdk_pixmap_create_from_data (DefaultDrawable, (char *) gray6_bits, gray6_width,
-					     gray6_height, TtWDepth, &gdkFgPixel, &gdkBgPixel);
+       pixmap = (ThotPixmap)gdk_pixmap_create_from_data (DefaultDrawable, (char *) gray6_bits, gray6_width,
+					     gray6_height, TtWDepth, (GdkColor *)&gdkFgPixel, (GdkColor *)&gdkBgPixel);
        break;
      case 9:
-       pixmap = gdk_pixmap_create_from_data (DefaultDrawable, (char *) gray7_bits, gray7_width,
-					     gray7_height, TtWDepth, &gdkFgPixel, &gdkBgPixel);
+       pixmap = (ThotPixmap)gdk_pixmap_create_from_data (DefaultDrawable, (char *) gray7_bits, gray7_width,
+					     gray7_height, TtWDepth, (GdkColor *)&gdkFgPixel, (GdkColor *)&gdkBgPixel);
        break;
      case 10:
-       pixmap = gdk_pixmap_create_from_data (DefaultDrawable, (char *) horiz1_bits, horiz1_width,
-					     horiz1_height, TtWDepth, &gdkFgPixel, &gdkBgPixel);
+       pixmap = (ThotPixmap)gdk_pixmap_create_from_data (DefaultDrawable, (char *) horiz1_bits, horiz1_width,
+					     horiz1_height, TtWDepth, (GdkColor *)&gdkFgPixel, (GdkColor *)&gdkBgPixel);
        break;
      case 11:
-       pixmap = gdk_pixmap_create_from_data (DefaultDrawable, (char *) horiz2_bits, horiz2_width,
-					     horiz2_height, TtWDepth, &gdkFgPixel, &gdkBgPixel);
+       pixmap = (ThotPixmap)gdk_pixmap_create_from_data (DefaultDrawable, (char *) horiz2_bits, horiz2_width,
+					     horiz2_height, TtWDepth, (GdkColor *)&gdkFgPixel, (GdkColor *)&gdkBgPixel);
        break;
      case 12:
-       pixmap = gdk_pixmap_create_from_data (DefaultDrawable, (char *) horiz3_bits, horiz3_width,
-					     horiz3_height, TtWDepth, &gdkFgPixel, &gdkBgPixel);
+       pixmap = (ThotPixmap)gdk_pixmap_create_from_data (DefaultDrawable, (char *) horiz3_bits, horiz3_width,
+					     horiz3_height, TtWDepth, (GdkColor *)&gdkFgPixel, (GdkColor *)&gdkBgPixel);
        break;
      case 13:
-       pixmap = gdk_pixmap_create_from_data (DefaultDrawable, (char *) vert1_bits, vert1_width,
-					     vert1_height, TtWDepth, &gdkFgPixel, &gdkBgPixel);
+       pixmap = (ThotPixmap)gdk_pixmap_create_from_data (DefaultDrawable, (char *) vert1_bits, vert1_width,
+					     vert1_height, TtWDepth, (GdkColor *)&gdkFgPixel, (GdkColor *)&gdkBgPixel);
        break;
      case 14:
-       pixmap = gdk_pixmap_create_from_data (DefaultDrawable, (char *) vert2_bits, vert2_width,
-					     vert2_height, TtWDepth, &gdkFgPixel, &gdkBgPixel);
+       pixmap = (ThotPixmap)gdk_pixmap_create_from_data (DefaultDrawable, (char *) vert2_bits, vert2_width,
+					     vert2_height, TtWDepth, (GdkColor *)&gdkFgPixel, (GdkColor *)&gdkBgPixel);
        break;
      case 15:
-       pixmap = gdk_pixmap_create_from_data (DefaultDrawable, (char *) vert3_bits, vert3_width,
-					     vert3_height, TtWDepth, &gdkFgPixel, &gdkBgPixel);
+       pixmap = (ThotPixmap)gdk_pixmap_create_from_data (DefaultDrawable, (char *) vert3_bits, vert3_width,
+					     vert3_height, TtWDepth, (GdkColor *)&gdkFgPixel, (GdkColor *)&gdkBgPixel);
        break;
      case 16:
-       pixmap = gdk_pixmap_create_from_data (DefaultDrawable, (char *) left1_bits, left1_width,
-					     left1_height, TtWDepth, &gdkFgPixel, &gdkBgPixel);
+       pixmap = (ThotPixmap)gdk_pixmap_create_from_data (DefaultDrawable, (char *) left1_bits, left1_width,
+					     left1_height, TtWDepth, (GdkColor *)&gdkFgPixel, (GdkColor *)&gdkBgPixel);
        break;
      case 17:
-       pixmap = gdk_pixmap_create_from_data (DefaultDrawable, (char *) left2_bits, left2_width,
-					     left2_height, TtWDepth, &gdkFgPixel, &gdkBgPixel);
+       pixmap = (ThotPixmap)gdk_pixmap_create_from_data (DefaultDrawable, (char *) left2_bits, left2_width,
+					     left2_height, TtWDepth, (GdkColor *)&gdkFgPixel, (GdkColor *)&gdkBgPixel);
        break;
      case 18:
-       pixmap = gdk_pixmap_create_from_data (DefaultDrawable, (char *) left3_bits, left3_width,
-					     left3_height, TtWDepth, &gdkFgPixel, &gdkBgPixel);
+       pixmap = (ThotPixmap)gdk_pixmap_create_from_data (DefaultDrawable, (char *) left3_bits, left3_width,
+					     left3_height, TtWDepth, (GdkColor *)&gdkFgPixel, (GdkColor *)&gdkBgPixel);
        break;
      case 19:
-       pixmap = gdk_pixmap_create_from_data (DefaultDrawable, (char *) right1_bits, right1_width,
-					     right1_height, TtWDepth, &gdkFgPixel, &gdkBgPixel);
+       pixmap = (ThotPixmap)gdk_pixmap_create_from_data (DefaultDrawable, (char *) right1_bits, right1_width,
+					     right1_height, TtWDepth, (GdkColor *)&gdkFgPixel, (GdkColor *)&gdkBgPixel);
        break;
      case 20:
-       pixmap = gdk_pixmap_create_from_data (DefaultDrawable, (char *) right2_bits, right2_width,
-					     right2_height, TtWDepth, &gdkFgPixel, &gdkBgPixel);
+       pixmap = (ThotPixmap)gdk_pixmap_create_from_data (DefaultDrawable, (char *) right2_bits, right2_width,
+					     right2_height, TtWDepth, (GdkColor *)&gdkFgPixel, (GdkColor *)&gdkBgPixel);
        break;
      case 21:
-       pixmap = gdk_pixmap_create_from_data (DefaultDrawable, (char *) right3_bits, right3_width,
-					     right3_height, TtWDepth, &gdkFgPixel, &gdkBgPixel);
+       pixmap = (ThotPixmap)gdk_pixmap_create_from_data (DefaultDrawable, (char *) right3_bits, right3_width,
+					     right3_height, TtWDepth, (GdkColor *)&gdkFgPixel, (GdkColor *)&gdkBgPixel);
        break;
      case 22:
-       pixmap = gdk_pixmap_create_from_data (DefaultDrawable, (char *) square1_bits, square1_width,
-					     square1_height, TtWDepth, &gdkFgPixel, &gdkBgPixel);
+       pixmap = (ThotPixmap)gdk_pixmap_create_from_data (DefaultDrawable, (char *) square1_bits, square1_width,
+					     square1_height, TtWDepth, (GdkColor *)&gdkFgPixel, (GdkColor *)&gdkBgPixel);
        break;
      case 23:
-       pixmap = gdk_pixmap_create_from_data (DefaultDrawable, (char *) square2_bits, square2_width,
-					     square2_height, TtWDepth, &gdkFgPixel, &gdkBgPixel);
+       pixmap = (ThotPixmap)gdk_pixmap_create_from_data (DefaultDrawable, (char *) square2_bits, square2_width,
+					     square2_height, TtWDepth,(GdkColor *) &gdkFgPixel, (GdkColor *)&gdkBgPixel);
        break;
      case 24:
-       pixmap = gdk_pixmap_create_from_data (DefaultDrawable, (char *) square3_bits, square3_width,
-					     square3_height, TtWDepth, &gdkFgPixel, &gdkBgPixel);
+       pixmap = (ThotPixmap)gdk_pixmap_create_from_data (DefaultDrawable, (char *) square3_bits, square3_width,
+					     square3_height, TtWDepth, (GdkColor *)&gdkFgPixel, (GdkColor *)&gdkBgPixel);
        break;
      case 25:
-       pixmap = gdk_pixmap_create_from_data (DefaultDrawable, (char *) lozenge_bits, lozenge_width,
-					     lozenge_height, TtWDepth, &gdkFgPixel, &gdkBgPixel);
+       pixmap = (ThotPixmap)gdk_pixmap_create_from_data (DefaultDrawable, (char *) lozenge_bits, lozenge_width,
+					     lozenge_height, TtWDepth, (GdkColor *)&gdkFgPixel, (GdkColor *)&gdkBgPixel);
        break;
      case 26:
-       pixmap = gdk_pixmap_create_from_data (DefaultDrawable, (char *) brick_bits, brick_width,
-					     brick_height, TtWDepth, &gdkFgPixel, &gdkBgPixel);
+       pixmap = (ThotPixmap)gdk_pixmap_create_from_data (DefaultDrawable, (char *) brick_bits, brick_width,
+					     brick_height, TtWDepth, (GdkColor *)&gdkFgPixel, (GdkColor *)&gdkBgPixel);
        break;
      case 27:
-       pixmap = gdk_pixmap_create_from_data (DefaultDrawable, (char *) tile_bits, tile_width,
-					     tile_height, TtWDepth, &gdkFgPixel, &gdkBgPixel);
+       pixmap = (ThotPixmap)gdk_pixmap_create_from_data (DefaultDrawable, (char *) tile_bits, tile_width,
+					     tile_height, TtWDepth, (GdkColor *)&gdkFgPixel, (GdkColor *)&gdkBgPixel);
        break;
      case 28:
-       pixmap = gdk_pixmap_create_from_data (DefaultDrawable, (char *) sea_bits, sea_width,
-					     sea_height, TtWDepth, &gdkFgPixel, &gdkBgPixel);
+       pixmap = (ThotPixmap)gdk_pixmap_create_from_data (DefaultDrawable, (char *) sea_bits, sea_width,
+					     sea_height, TtWDepth, (GdkColor *)&gdkFgPixel, (GdkColor *)&gdkBgPixel);
        break;
      case 29:
-       pixmap = gdk_pixmap_create_from_data (DefaultDrawable, (char *) basket_bits, basket_width,
-					     basket_height, TtWDepth, &gdkFgPixel, &gdkBgPixel);
+       pixmap = (ThotPixmap)gdk_pixmap_create_from_data (DefaultDrawable, (char *) basket_bits, basket_width,
+					     basket_height, TtWDepth, (GdkColor *)&gdkFgPixel, (GdkColor *)&gdkBgPixel);
        break;
      default:
        pixmap = None;
        break;
      }
-#else /* _GTK */
+#endif /* _GTK */
+
+#ifdef _MOTIF   
    switch (motif)
      {
      case 1:
@@ -1299,7 +1501,11 @@ Pixmap CreatePattern (int disp, int fg, int bg, int motif)
        break;
      }
    XFlush (TtDisplay);
-#endif /* _GTK */
-#endif /* _WINDOWS */
+#endif /* _MOTIF */
+
+#if defined(_WX)
+   /* TODO */
+#endif /* defined(_WX) */
+   
    return (pixmap);
 }
