@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -13,6 +13,7 @@
  *
  */
 #ifndef _WIN_PRINT
+#ifndef _GL
 
 #include "ustring.h"
 #include "math.h"
@@ -59,10 +60,9 @@ static char*       Patterns_PS[] =
 
 
 /* Handling of loaded fonts */
-extern ptrfont      PoscriptFont;
+extern ThotFont     PostscriptFont;
 extern int          ColorPs;
-extern int          LastPageNumber, LastPageWidth, LastPageHeight;
-static STRING       Scale = NULL;
+static char        *Scale = NULL;
 int                 X, Y;
 static int          SameBox = 0; /* 1 if the text is in the same box */
 static int          NbWhiteSp;
@@ -80,7 +80,7 @@ static int          NbWhiteSp;
    and the one asked, and load it if needed.
    num is an index in the Thot internal color table.
   ----------------------------------------------------------------------*/
-static void CurrentColor (FILE * fout, int num)
+static void CurrentColor (FILE *fout, int num)
 {
   unsigned short      red;
   unsigned short      green;
@@ -96,7 +96,7 @@ static void CurrentColor (FILE * fout, int num)
 	fprintf (fout, "0 setgray\n");
       else
 	{
-	  /* write the Poscript command */
+	  /* write the Postscript command */
 	  fact = 255;
 	  fprintf (fout, "%f %f %f setrgbcolor\n", ((float) red) / fact, ((float) green) / fact, ((float) blue) / fact);
 	}
@@ -110,10 +110,10 @@ static void CurrentColor (FILE * fout, int num)
    and emit the code to load it, if necessary.
    Returns 0 if it is a Latin font and 1 for a Greek one.
   ----------------------------------------------------------------------*/
-static int CurrentFont (FILE * fout, ptrfont font)
+static int CurrentFont (FILE *fout, ThotFont font)
 {
   int                 i, result;
-  CHAR_T              c1, c2;
+  char                c0, c1, c2;
 
   /* browse the table of fonts */
   i = 0;
@@ -123,27 +123,36 @@ static int CurrentFont (FILE * fout, ptrfont font)
   if (i >= MAX_FONT)
     i = 0;
   i = i * 8;
-  if (font != PoscriptFont)
+  if (font != PostscriptFont)
     {
-      PoscriptFont = font;
-      if (TtPsFontName[i] == 'g')  /* Greek alphabet */
+      PostscriptFont = font;
+      if (TtPsFontName[i] == 'g')  /* Greek script */
 	{
+	  c0 = TtPsFontName[i];
 	  c1 = TtPsFontName[i];
 	  c2 = 'r';	     /* Symbol only has one style available */
 	  result = 1;
 	}
       else
 	{
-	  /* Latin Alphabet */
+	  /* Latin Script */
+	  c0 = 'l';
 	  c1 = TtPsFontName[i + 1]; /* font Helvetica Times Courrier */
 	  /* convert lowercase to uppercase */
 	  c2 = TtPsFontName[i + 2]; /* Style normal bold italique */
+#ifndef _GL
+	  if (c1 == 'n')
+	    {
+	      c0 = c1 = 'g';
+	      c2 = 'r';
+	    }
+#endif /* _GL */
 	  result = 0;
 	}
       
       /* update the scaling factor */
       Scale = &TtPsFontName[i + 3];
-      fprintf (fout, "%c%c%c %s sf\n", TtPsFontName[i], c1, c2, Scale);
+      fprintf (fout, "%c%c%c %s sf\n", c0, c1, c2, Scale);
       return result;
     }
   /* returns the indicator for the family of fonts */
@@ -157,7 +166,7 @@ static int CurrentFont (FILE * fout, ptrfont font)
   DrawChar draw a char at location (x, y) in frame and with font.
   The parameter fg indicates the drawing color
   ----------------------------------------------------------------------*/
-void DrawChar (UCHAR_T car, int frame, int x, int y, ptrfont font, int fg)
+void DrawChar (char car, int frame, int x, int y, ThotFont font, int fg)
 {
    FILE               *fout;
    int                 w;
@@ -166,7 +175,6 @@ void DrawChar (UCHAR_T car, int frame, int x, int y, ptrfont font, int fg)
    if (y < 0)
       return;
    y += FrameTable[frame].FrTopMargin;
-   y = y + FontBase (font);
    if (fg >= 0)
      {
        /* Do we need to change the current color ? */
@@ -181,7 +189,7 @@ void DrawChar (UCHAR_T car, int frame, int x, int y, ptrfont font, int fg)
 /*----------------------------------------------------------------------
    Transcode emit the PostScript code for the given char.
   ----------------------------------------------------------------------*/
-static void Transcode (FILE * fout, int encoding, UCHAR_T car)
+static void Transcode (FILE *fout, int encoding, unsigned char car)
 {
   switch (car)
     {
@@ -217,7 +225,7 @@ static void Transcode (FILE * fout, int encoding, UCHAR_T car)
    pattern, or the drawing color, or the background color,
    or keep it transparent, depending on pattern value.
   ----------------------------------------------------------------------*/
-static void FillWithPattern (FILE * fout, int fg, int bg, int pattern)
+static void FillWithPattern (FILE *fout, int fg, int bg, int pattern)
 {
    unsigned short      red;
    unsigned short      green;
@@ -233,7 +241,7 @@ static void FillWithPattern (FILE * fout, int fg, int bg, int pattern)
      {
 	/* Ask for the RedGreenBlue values */
 	TtaGiveThotRGB (fg, &red, &green, &blue);
-	/* Emit the Poscript command */
+	/* Emit the Postscript command */
 	fprintf (fout, "%f %f %f -1\n", ((float) red) / fact,
 		 ((float) green) / fact, ((float) blue) / fact);
      }
@@ -241,7 +249,7 @@ static void FillWithPattern (FILE * fout, int fg, int bg, int pattern)
      {
 	/* Ask for the RedGreenBlue values */
 	TtaGiveThotRGB (bg, &red, &green, &blue);
-	/* Emit the Poscript command */
+	/* Emit the Postscript command */
 	fprintf (fout, "%f %f %f -1\n", ((float) red) / fact,
 		 ((float) green) / fact, ((float) blue) / fact);
      }
@@ -257,42 +265,38 @@ static void FillWithPattern (FILE * fout, int fg, int bg, int pattern)
 }
 
 /*----------------------------------------------------------------------
-   DrawString draw a char string of lg chars beginning at buff[i].
+   DrawString draw a char string of lg chars beginning in buff.
    Drawing starts at (x, y) in frame and using font.
    boxWidth gives the width of the final box or zero,
    this is used only by the thot formmating engine.
-   bl indicate taht there is a space before the string
+   bl indicate that there are one or more spaces before the string
    hyphen indicate whether an hyphen char has to be added.
-   StartABlock is 1 if the text is at a paragraph beginning
+   StartABlock is not null if the text is at a paragraph beginning
    (no justification of first spaces).
    parameter fg indicate the drawing color
    Returns the lenght of the string drawn.
   ----------------------------------------------------------------------*/
-int DrawString (STRING buff, int i, int lg, int frame, int x, int y,
-		ptrfont font, int boxWidth, int bl, int hyphen,
-		int startABlock, int fg, int shadow)
+int DrawString (unsigned char *buff, int lg, int frame, int x, int y,
+		ThotFont font, int boxWidth, int bl, int hyphen,
+		int startABlock, int fg)
 {
   FILE               *fout;
-  STRING              ptcar;
-  int                 j, encoding, width;
+  int                 j, i, encoding, width;
   int                 noJustifiedWhiteSp;
 
   fout = (FILE *) FrRef[frame];
   encoding = 0;
   if (y < 0)
     return 0;
-
   y += FrameTable[frame].FrTopMargin;
-  /* noJustifiedWhiteSp is > 0 if writing a fixed lenght is needed */
-  /* and equal to 0 if a justified space is to be printed */  
-  noJustifiedWhiteSp = startABlock;
+
   /* Is this a new box ? */
   if (SameBox == 0)
     {
-      /* Beginning of a new box */
+      /* store the start position for the justified box */
       SameBox = 1;
       X = x;
-      Y = y + FontBase (font);
+      Y = y;
       NbWhiteSp = 0;
       if (fg >= 0)
 	{
@@ -303,60 +307,66 @@ int DrawString (STRING buff, int i, int lg, int frame, int x, int y,
 	  fprintf (fout, "(");
 	}
     }
-
-  if (shadow)
-    {
-      /* replace each character by a star */
-      j = 0;
-      ptcar = TtaAllocString ((size_t) (lg + 1));
-      while (j < lg)
-	ptcar[j++] = '*';
-      ptcar[lg] = EOS;
-      bl = 0;
-    }
-  else
-    ptcar = &buff[i - 1];
-
+  
+  buff[lg] = EOS;
   /* Add the justified white space */
   if (bl > 0)
     {
-      NbWhiteSp++;
+      NbWhiteSp += bl;
       if (fg >= 0)
-	Transcode (fout, encoding, ' ');
-    }
-  
-   /* Emit the chars */
-  for (j = 0; j < lg; j++)
-    {
-      /* enumerate the white spaces */
-      if (ptcar[j] == ' ')
 	{
-	  if (noJustifiedWhiteSp == 0)
-	    {
-	      /* write a justified white space */
-	      NbWhiteSp++;
-	      if (fg >= 0)
-		Transcode (fout, encoding, ptcar[j]);
-	    }
-	  else if (fg >= 0)
-	    /* write a fixed lenght white space */
-	    fputs ("\\240", fout);
+	  for (i = 1; i <= bl; i++)
+	    fprintf (fout, "%c", ' ');
+	  /* Transcode (fout, encoding, ' '); */
 	}
-      else
+    }
+
+  width = 0;
+  if (lg > 0)
+    {
+      /* noJustifiedWhiteSp is > 0 if writing a fixed lenght is needed */
+      /* and equal to 0 if a justified space is to be printed */  
+      noJustifiedWhiteSp = startABlock;
+      /* Emit the chars */
+      for (j = 0; j < lg; j++)
 	{
-	  noJustifiedWhiteSp = 0;
-	  if (fg >= 0)
-	    Transcode (fout, encoding, ptcar[j]);
+	  /* compute the width of the string */
+	  width += CharacterWidth (buff[j], font);
+	  /* enumerate the white spaces */
+	  if (buff[j] == ' ')
+	    {
+	      if (noJustifiedWhiteSp == 0)
+		{
+		  /* write a justified white space */
+		  NbWhiteSp++;
+		  if (fg >= 0)
+		    fputs (" ", fout);
+		}
+	      else if (fg >= 0)
+		/* write a fixed lenght white space */
+		fputs ("\\240", fout);
+	    }
+	  else
+	    {
+	      noJustifiedWhiteSp = 0;
+	      if (fg >= 0)
+		Transcode (fout, encoding, buff[j]);
+	    }
 	}
     }
    
   /* Is an hyphen needed ? */
   if (hyphen && fg >= 0)
-    Transcode (fout, encoding, '\255');
+    fprintf (fout, "%c", '\255');
   /* is this the end of the box */
-  if (boxWidth != 0)
+  if (boxWidth != 0 && SameBox == 1)
     {
-      boxWidth = boxWidth;
+      /* now let Postscript justify the text with the right width */
+      if (boxWidth < 0)
+	boxWidth = - boxWidth + width;
+      else if (boxWidth < width)
+	/* not enough space to display the last piece of text */
+	boxWidth = width;
       /* Is justification needed ? */
       if (fg >= 0)
 	{
@@ -366,26 +376,132 @@ int DrawString (STRING buff, int i, int lg, int frame, int x, int y,
 	    fprintf (fout, ") %d %d %d %d j\n", NbWhiteSp, boxWidth, X, -Y);
 	}
       SameBox = 0;
+      NbWhiteSp = 0;
     }
+  return (width);
+}
+
+/*----------------------------------------------------------------------
+   WDrawString draw a char string of lg chars beginning in buff.
+   Drawing starts at (x, y) in frame and using font.
+   boxWidth gives the width of the final box or zero,
+   this is used only by the thot formmating engine.
+   bl indicate that there are one or more spaces before the string
+   hyphen indicate whether an hyphen char has to be added.
+   StartABlock is 1 if the text is at a paragraph beginning
+   (no justification of first spaces).
+   parameter fg indicate the drawing color
+   Returns the lenght of the string drawn.
+  ----------------------------------------------------------------------*/
+int WDrawString (wchar_t *buff, int lg, int frame, int x, int y,
+		ThotFont font, int boxWidth, int bl, int hyphen,
+		int startABlock, int fg)
+{
+  FILE               *fout;
+  int                 j, i, encoding, width;
+  int                 noJustifiedWhiteSp;
+
+  fout = (FILE *) FrRef[frame];
+  encoding = 0;
+  if (y < 0)
+    return 0;
+  y += FrameTable[frame].FrTopMargin;
+
+  /* Is this a new box ? */
+  if (SameBox == 0)
+    {
+      /* store the start position for the justified box */
+      SameBox = 1;
+      X = x;
+      Y = y;
+      NbWhiteSp = 0;
+      if (fg >= 0)
+	{
+	  /* Do we need to change the current color ? */
+	  CurrentColor (fout, fg);
+	  /* Do we need to change the current font ? */
+	  encoding = CurrentFont (fout, font);
+	  fprintf (fout, "(");
+	}
+    }
+  
+  buff[lg] = EOS;
+  /* Add the justified white space */
+  if (bl > 0)
+    {
+      NbWhiteSp += bl;
+      if (fg >= 0)
+	{
+	  for (i = 1; i <= bl; i++)
+	    fprintf (fout, "%c", ' ');
+	}
+    }
+
+  width = 0;
   if (lg > 0)
     {
-      /* compute the width of the string */
-      width = 0;
-      j = 0;
-      while (j < lg)
-	width += CharacterWidth (ptcar[j++], font);
-      return (width);
-    } 
-  if (shadow)
-    TtaFreeMemory (ptcar);
-  return (0);
+      /* noJustifiedWhiteSp is > 0 if writing a fixed lenght is needed */
+      /* and equal to 0 if a justified space is to be printed */  
+      noJustifiedWhiteSp = startABlock;
+      /* Emit the chars */
+      for (j = 0; j < lg; j++)
+	{
+	  /* compute the width of the string */
+	  width += CharacterWidth (buff[j], font);
+	  /* enumerate the white spaces */
+	  if (buff[j] == ' ')
+	    {
+	      if (noJustifiedWhiteSp == 0)
+		{
+		  /* write a justified white space */
+		  NbWhiteSp++;
+		  if (fg >= 0)
+		    fputs (" ", fout);
+		}
+	      else if (fg >= 0)
+		/* write a fixed lenght white space */
+		fputs ("\\240", fout);
+	    }
+	  else
+	    {
+	      noJustifiedWhiteSp = 0;
+	      if (fg >= 0)
+		fprintf (fout, "%c", (char)buff[j]);
+	    }
+	}
+    }
+
+  /* Is an hyphen needed ? */
+  if (hyphen && fg >= 0)
+    fprintf (fout, "%c", '\255');
+  /* is this the end of the box */
+  if (boxWidth != 0 && SameBox == 1)
+    {
+      /* now let Postscript justify the text with the right width */
+      if (boxWidth < 0)
+	boxWidth = - boxWidth + width;
+      else if (boxWidth < width)
+	/* not enough space to display the last piece of text */
+	boxWidth = width;
+      /* Is justification needed ? */
+      if (fg >= 0)
+	{
+	  if (NbWhiteSp == 0)
+	    fprintf (fout, ") %d %d %d s\n", boxWidth, X, -Y);
+	  else
+	    fprintf (fout, ") %d %d %d %d j\n", NbWhiteSp, boxWidth, X, -Y);
+	}
+      SameBox = 0;
+      NbWhiteSp = 0;
+    }
+  return (width);
 }
 
 
 /*----------------------------------------------------------------------
    DisplayUnderline draw the underline, overline or cross line
   ----------------------------------------------------------------------*/
-void DisplayUnderline (int frame, int x, int y, ptrfont font, int type,
+void DisplayUnderline (int frame, int x, int y, ThotFont font, int type,
 		       int lg, int fg)
 {
   int                 fheight;	/* font height           */
@@ -449,7 +565,7 @@ void DisplayUnderline (int frame, int x, int y, ptrfont font, int type,
    DrawRadical Draw a radical symbol.
   ----------------------------------------------------------------------*/
 void DrawRadical (int frame, int thick, int x, int y, int l, int h,
-		  ptrfont font, int fg)
+		  ThotFont font, int fg)
 {
    int                 fh;
    int                 ex;
@@ -490,7 +606,7 @@ void DrawRadical (int frame, int thick, int x, int y, int l, int h,
    - double if type = 2.
   ----------------------------------------------------------------------*/
 void DrawIntegral (int frame, int thick, int x, int y, int l, int h,
-		   int type, ptrfont font, int fg)
+		   int type, ThotFont font, int fg)
 {
    int                 yf;
    int                 ey, ym;
@@ -523,9 +639,9 @@ void DrawIntegral (int frame, int thick, int x, int y, int l, int h,
 	 {
 	   /* double integral */
 	   fprintf (fout, "%d %d (\\362) c\n", -ym,
-		    x - CharacterWidth ('\362', font) / 4);
+		    x - CharacterWidth (242, font) / 4);
 	   fprintf (fout, "%d %d (\\362) c\n", -ym,
-		    x + CharacterWidth ('\362', font) / 4);
+		    x + CharacterWidth (242, font) / 4);
 	 }
        else
 	 {
@@ -541,10 +657,10 @@ void DrawIntegral (int frame, int thick, int x, int y, int l, int h,
 	 {
 	   /* double integral */
 	   fprintf (fout, "%d %d %d %s (\\363) (\\364) (\\365) s3\n",
-		    x - CharacterWidth ('\364', font) / 4,
+		    x - CharacterWidth (244, font) / 4,
 		    -yf, -y, Scale);
 	   fprintf (fout, "%d %d %d %s (\\363) (\\364) (\\365) s3\n",
-		    x + CharacterWidth ('\364', font) / 4,
+		    x + CharacterWidth (244, font) / 4,
 		    -yf, -y, Scale);
 	 }
        else
@@ -561,7 +677,7 @@ void DrawIntegral (int frame, int thick, int x, int y, int l, int h,
 /*----------------------------------------------------------------------
    DrawSigma draw a Sigma symbol.
   ----------------------------------------------------------------------*/
-void DrawSigma (int frame, int x, int y, int l, int h, ptrfont font, int fg)
+void DrawSigma (int frame, int x, int y, int l, int h, ThotFont font, int fg)
 {
    FILE               *fout;
 
@@ -574,7 +690,7 @@ void DrawSigma (int frame, int x, int y, int l, int h, ptrfont font, int fg)
    CurrentColor (fout, fg);
 
    /* Change the current font */
-   PoscriptFont = NULL;
+   PostscriptFont = NULL;
    fprintf (fout, "(Symbol) %.0f sf\n", FontHeight (font) * 0.9);
    x = x + (l / 2);
    y = y + h - FontHeight (font + FontBase (font));
@@ -585,7 +701,7 @@ void DrawSigma (int frame, int x, int y, int l, int h, ptrfont font, int fg)
 /*----------------------------------------------------------------------
    DrawPi draw a PI symbol.
   ----------------------------------------------------------------------*/
-void DrawPi (int frame, int x, int y, int l, int h, ptrfont font, int fg)
+void DrawPi (int frame, int x, int y, int l, int h, ThotFont font, int fg)
 {
    FILE             *fout;
 
@@ -598,7 +714,7 @@ void DrawPi (int frame, int x, int y, int l, int h, ptrfont font, int fg)
    CurrentColor (fout, fg);
 
    /* Change the current font */
-   PoscriptFont = NULL;
+   PostscriptFont = NULL;
    fprintf (fout, "(Symbol) %.0f sf\n", FontHeight (font) * 0.9);
    x = x + (l / 2);
    y = y + h - FontHeight (font + FontBase (font));
@@ -609,7 +725,7 @@ void DrawPi (int frame, int x, int y, int l, int h, ptrfont font, int fg)
 /*----------------------------------------------------------------------
    DrawUnion draw an Union symbol.
   ----------------------------------------------------------------------*/
-void DrawUnion (int frame, int x, int y, int l, int h, ptrfont font, int fg)
+void DrawUnion (int frame, int x, int y, int l, int h, ThotFont font, int fg)
 {
    FILE               *fout;
 
@@ -622,7 +738,7 @@ void DrawUnion (int frame, int x, int y, int l, int h, ptrfont font, int fg)
    CurrentColor (fout, fg);
 
    /* Change the current font */
-   PoscriptFont = NULL;
+   PostscriptFont = NULL;
    fprintf (fout, "(Symbol) %.0f sf\n", FontHeight (font) * 0.9);
    x = x + (l / 2);
    y = y + h - FontHeight (font + FontBase (font));
@@ -633,7 +749,7 @@ void DrawUnion (int frame, int x, int y, int l, int h, ptrfont font, int fg)
 /*----------------------------------------------------------------------
    DrawIntersection draw an intersection symbol.
   ----------------------------------------------------------------------*/
-void DrawIntersection (int frame, int x, int y, int l, int h, ptrfont font,
+void DrawIntersection (int frame, int x, int y, int l, int h, ThotFont font,
 		       int fg)
 {
    FILE               *fout;
@@ -646,7 +762,7 @@ void DrawIntersection (int frame, int x, int y, int l, int h, ptrfont font,
    CurrentColor (fout, fg);
 
    /* Change the current font */
-   PoscriptFont = NULL;
+   PostscriptFont = NULL;
    fprintf (fout, "(Symbol) %.0f sf\n", FontHeight (font) * 0.9);
    x = x + (l / 2);
    y = y + h - FontHeight (font + FontBase (font));
@@ -679,8 +795,6 @@ void DrawArrow (int frame, int thick, int style, int x, int y, int l,
    /* Do we need to change the current color ? */
    CurrentColor (fout, fg);
 
-   l--;
-   h--;
    xm = x + l / 2;
    xf = x + l;
    ym = y + h / 2;
@@ -753,22 +867,22 @@ void DrawArrow (int frame, int thick, int style, int x, int y, int l,
    DrawBracket draw an opening or closing bracket (depending on direction)
   ----------------------------------------------------------------------*/
 void DrawBracket (int frame, int thick, int x, int y, int l, int h,
-		  int direction, ptrfont font, int fg)
+		  int direction, ThotFont font, int fg)
 {
    int                 ey, yf;
    FILE               *fout;
 
+   if (y < 0)
+     return;
+   if (thick < 0 || fg < 0)
+      return;
    fout = (FILE *) FrRef[frame];
-  if (y < 0)
-    return;
-
-   y += FrameTable[frame].FrTopMargin;
    /* Do we need to change the current color ? */
    CurrentColor (fout, fg);
-
    /* Do we need to change the current font ? */
    CurrentFont (fout, font);
 
+   y += FrameTable[frame].FrTopMargin;
    l--;
    h--;
    ey = FontHeight (font);
@@ -780,21 +894,22 @@ void DrawBracket (int frame, int thick, int x, int y, int l, int h,
 
    if (h < ey / 4)
      {
-	/* Made of only one glyph */
-	if (direction == 0)
-	   fprintf (fout, "%d %d ([) c\n", -yf, x);
-	else
-	   fprintf (fout, "%d %d (])c\n", -yf, x);
+       yf = yf - (h / 2);
+       /* Made of only one glyph */
+       if (direction == 0)
+	 fprintf (fout, "%d %d ([) c\n", -yf, x);
+       else
+	 fprintf (fout, "%d %d (])c\n", -yf, x);
      }
    else
      {
-	/* Drawn with more than one glyph */
-	if (direction == 0)	/* Trace un crochet ouvrant */
-	   fprintf (fout, "%d %d %d %s (\\351) (\\352) (\\353) s3\n",
-		    x + 1, -yf, -y, Scale);
-	else
-	   fprintf (fout, "%d %d %d %s (\\371) (\\372) (\\373) s3\n",
-		    x, -yf, -y, Scale);
+       /* Drawn with more than one glyph */
+       if (direction == 0)	/* Trace un crochet ouvrant */
+	 fprintf (fout, "%d %d %d %s (\\351) (\\352) (\\353) s3\n",
+		  x + 1, -yf, -y, Scale);
+       else
+	 fprintf (fout, "%d %d %d %s (\\371) (\\372) (\\373) s3\n",
+		  x, -yf, -y, Scale);
      }
 }
 
@@ -804,55 +919,55 @@ void DrawBracket (int frame, int thick, int x, int y, int l, int h,
    on direction)
   ----------------------------------------------------------------------*/
 void DrawPointyBracket (int frame, int thick, int x, int y, int l, int h,
-			int direction, ptrfont font, int fg)
+			int direction, ThotFont font, int fg)
 {
    int                 ey, yf;
    FILE               *fout;
 
+   if (y < 0)
+     return;
+   if (thick < 0 || fg < 0)
+      return;
    fout = (FILE *) FrRef[frame];
-  if (y < 0)
-    return;
-
-   y += FrameTable[frame].FrTopMargin;
    /* Do we need to change the current color ? */
    CurrentColor (fout, fg);
-
    /* Do we need to change the current font ? */
    CurrentFont (fout, font);
 
+   y += FrameTable[frame].FrTopMargin;
    l--;
    h--;
    ey = FontHeight (font);
-   h -= ey;
-   y += FontBase (font);
 
-   if (h < ey / 4)
+   if (h - ey < ey / 4)
      {
-       x = x + (l / 2);
+       /* Made of only one glyph */
+       h -= ey;
+       y += FontBase (font);
        yf = y + h;
        y = y + 1;
-	/* Made of only one glyph */
-	if (direction == 0)
-	   fprintf (fout, "%d %d (\341) c\n", -yf, x);
-	else
-	   fprintf (fout, "%d %d (\361)c\n", -yf, x);
+       x = x + (l / 2);
+       yf = yf - (h / 2);
+       if (direction == 0)
+	 fprintf (fout, "%d %d (\341) c\n", -yf, x);
+       else
+	 fprintf (fout, "%d %d (\361)c\n", -yf, x);
      }
    else
      {
-	/* Drawn with more than one glyph */
-	if (direction == 0)
-	  /* Trace un crochet ouvrant */
-	  fprintf (fout, "%d %d %d %d %d %d %d %d %d Seg\n",
-		   x + l, -(y),
-		   x, -(y + (h / 2)),
-		   x + l, -(y + h),
-		   5, 1, 3);
-	else
-	  fprintf (fout, "%d %d %d %d %d %d %d %d %d Seg\n",
-		   x, -(y),
-		   x + l, -(y + (h / 2)),
-		   x, -(y + h),
-		   5, 1, 3);
+       /* Drawn with more than one glyph */
+       if (direction == 0)
+	 fprintf (fout, "%d %d %d %d %d %d %d %d %d Seg\n",
+		  x + l, -(y),
+		  x, -(y + (h / 2)),
+		  x + l, -(y + h),
+		  5, 1, 3);
+       else
+	 fprintf (fout, "%d %d %d %d %d %d %d %d %d Seg\n",
+		  x, -(y),
+		  x + l, -(y + (h / 2)),
+		  x, -(y + h),
+		  5, 1, 3);
      }
 }
 
@@ -861,25 +976,22 @@ void DrawPointyBracket (int frame, int thick, int x, int y, int l, int h,
    DrawParenthesis draw a closing or opening parenthesis (direction).
   ----------------------------------------------------------------------*/
 void DrawParenthesis (int frame, int thick, int x, int y, int l, int h,
-		      int direction, ptrfont font, int fg)
+		      int direction, ThotFont font, int fg)
 {
    int                 ey, yf;
    FILE               *fout;
 
    if (y < 0)
       return;
-
-   y += FrameTable[frame].FrTopMargin;
    if (thick < 0 || fg < 0)
       return;
-
    fout = (FILE *) FrRef[frame];
    /* Do we need to change the current color ? */
    CurrentColor (fout, fg);
-
    /* Do we need to change the current font ? */
    CurrentFont (fout, font);
 
+   y += FrameTable[frame].FrTopMargin;
    l--;
    h--;
    ey = FontHeight (font);
@@ -891,22 +1003,23 @@ void DrawParenthesis (int frame, int thick, int x, int y, int l, int h,
 
    if (h < ey / 3)
      {
-	/* Made of only one glyph */
-	if (direction == 0)
-	   /* draw an opening parenthesis */
-	   fprintf (fout, "%d %d (\\() c\n", -yf, x);
-	else
-	   fprintf (fout, "%d %d (\\)) c\n", -yf, x);
+       /* Made of only one glyph */
+       yf = yf - (h / 2);
+       if (direction == 0)
+	 /* draw an opening parenthesis */
+	 fprintf (fout, "%d %d (\\() c\n", -yf, x);
+       else
+	 fprintf (fout, "%d %d (\\)) c\n", -yf, x);
      }
    else
      {
-	/* Drawn with more than one glyph */
-	if (direction == 0)
-	   fprintf (fout, "%d %d %d %s (\\346) (\\347) (\\350) s3\n",
-		    x+1, -yf, -y, Scale);
-	else
-	   fprintf (fout, "%d %d %d %s (\\366) (\\367) (\\370) s3\n",
-		    x, -yf, -y, Scale);
+       /* Drawn with more than one glyph */
+       if (direction == 0)
+	 fprintf (fout, "%d %d %d %s (\\346) (\\347) (\\350) s3\n",
+		  x+1, -yf, -y, Scale);
+       else
+	 fprintf (fout, "%d %d %d %s (\\366) (\\367) (\\370) s3\n",
+		  x, -yf, -y, Scale);
      }
 }
 
@@ -915,48 +1028,49 @@ void DrawParenthesis (int frame, int thick, int x, int y, int l, int h,
    DrawBrace draw an opening of closing brace (depending on direction).
   ----------------------------------------------------------------------*/
 void DrawBrace (int frame, int thick, int x, int y, int l, int h,
-		int direction, ptrfont font, int fg)
+		int direction, ThotFont font, int fg)
 {
    int                 ey, yf;
    FILE               *fout;
 
+   if (y < 0)
+     return;
+   if (thick < 0 || fg < 0)
+      return;
    fout = (FILE *) FrRef[frame];
-  if (y < 0)
-    return;
-
-   y += FrameTable[frame].FrTopMargin;
    /* Do we need to change the current color ? */
    CurrentColor (fout, fg);
-
    /* Do we need to change the current font ? */
    CurrentFont (fout, font);
 
+   y += FrameTable[frame].FrTopMargin;
    l--;
    h--;
    ey = FontHeight (font);
    h -= ey;
    y += FontBase (font);
-   x = x + (l / 2);
    yf = y + h;
    y = y + 1;
+   x = x + (l / 2);
 
    if (h < ey - 1)
      {
-	/* Made of only one glyph */
-	if (direction == 0)
-	   fprintf (fout, "%d %d ({) c\n", -yf, x);
-	else
-	   fprintf (fout, "%d %d (}) c\n", -yf, x);
+       /* Made of only one glyph */
+       yf = yf - (h / 2);
+       if (direction == 0)
+	 fprintf (fout, "%d %d ({) c\n", -yf, x);
+       else
+	 fprintf (fout, "%d %d (}) c\n", -yf, x);
      }
    else
      {
-	/* Drawn with more than one glyph */
-	if (direction == 0)
-	   fprintf (fout, "%d %d %d %s (\\354) (\\355) (\\356) (\\357) s4\n",
-		    x, -yf, -y, Scale);
-	else
-	   fprintf (fout, "%d %d %d %s (\\374) (\\375) (\\376) (\\357) s4\n",
-		    x, -yf, -y, Scale);
+       /* Drawn with more than one glyph */
+       if (direction == 0)
+	 fprintf (fout, "%d %d %d %s (\\354) (\\355) (\\356) (\\357) s4\n",
+		  x, -yf, -y, Scale);
+       else
+	 fprintf (fout, "%d %d %d %s (\\374) (\\375) (\\376) (\\357) s4\n",
+		  x, -yf, -y, Scale);
      }
 }
 
@@ -1348,6 +1462,107 @@ void DrawPath (int frame, int thick, int style, int x, int y,
 
 	    case PtEllipticalArc:
 	      /**** to do ****/
+	      /* draws a Bezier if it's a half or quarter of a circle */
+	      if (pPa->XRadius == pPa->YRadius)
+		/* it's an arc of a circle */
+		{
+		  x1 = (float) (x + PixelValue (pPa->XStart, UnPixel, NULL, 0));
+		  y1 = (float) (y + PixelValue (pPa->YStart, UnPixel, NULL, 0));
+		  x2 = (float) (x + PixelValue (pPa->XEnd, UnPixel, NULL, 0));
+		  y2 = (float) (y + PixelValue (pPa->YEnd, UnPixel, NULL, 0));
+		  if (pPa->XStart == pPa->XEnd &&
+		      abs (pPa->YEnd - pPa->YStart) == 2 * pPa->XRadius)
+		    /* half circle (vertical) */
+		    {
+		    if ((pPa->Sweep  && pPa->YEnd > pPa->YStart) ||
+			(!pPa->Sweep && pPa->YEnd < pPa->YStart))
+		      cx1 = (float) (x + PixelValue ((int)(pPa->XStart + 1.36 * pPa->XRadius)
+						     , UnPixel, NULL, 0));
+		    else
+		      cx1 = (float) (x + PixelValue ((int)(pPa->XStart - 1.36 * pPa->XRadius),
+						     UnPixel, NULL, 0));
+		    cy1 = y1;
+		    cx2 = cx1;
+		    cy2 = y2;
+		    fprintf (fout, "%f %f %f %f %f %f curveto ",
+		       cx1, -cy1, cx2, -cy2, x2, -y2);
+		    }
+		  else if (pPa->YStart == pPa->YEnd &&
+		      abs (pPa->XEnd - pPa->XStart) == 2 * pPa->XRadius)
+		    /* half circle (horizontal) */
+		    {
+		    if ((pPa->Sweep  && pPa->XEnd < pPa->XStart) ||
+			(!pPa->Sweep && pPa->XEnd > pPa->XStart))
+		      cy1 = (float) (y + PixelValue ((int)(pPa->YStart + 1.36 * pPa->YRadius)
+						     , UnPixel, NULL, 0));
+		    else
+		      cy1 = (float) (y + PixelValue ((int)(pPa->YStart - 1.36 * pPa->YRadius)
+						     , UnPixel, NULL, 0));
+		    cy2 = cy1;
+		    cx1 = x1;
+		    cx2 = x2;
+		    fprintf (fout, "%f %f %f %f %f %f curveto ",
+		       cx1, -cy1, cx2, -cy2, x2, -y2);
+		    }
+		  else if (abs (pPa->YEnd - pPa->YStart) == pPa->YRadius &&
+			   abs (pPa->XEnd - pPa->XStart) == pPa->XRadius)
+		    /* a quarter or 3/4 of a circle */
+		    {
+		      if (!pPa->LargeArc)
+			/* a quarter of a circle */
+			{
+			  if (pPa->XStart < pPa->XEnd)
+			    if (pPa->YStart < pPa->YEnd)
+			      if (pPa->Sweep)
+			        {
+			          cx1 = x2;
+			          cy1 = y1;
+			        }
+			      else
+			        {
+			          cx1 = x1;
+			          cy1 = y2;
+			        }
+			    else
+			      if (pPa->Sweep)
+			        {
+			          cx1 = x1;
+			          cy1 = y2;
+			        }
+			      else
+			        {
+			          cx1 = x2;
+			          cy1 = y1;
+			        }
+			  else
+			    if (pPa->YStart < pPa->YEnd)
+			      if (pPa->Sweep)
+			        {
+			          cx1 = x1;
+			          cy1 = y2;
+			        }
+			      else
+			        {
+			          cx1 = x2;
+			          cy1 = y1;
+			        }
+			    else
+			      if (pPa->Sweep)
+			        {
+			          cx1 = x2;
+			          cy1 = y1;
+			        }
+			      else
+			        {
+			          cx1 = x1;
+			          cy1 = y2;
+			        }
+			  fprintf (fout, "%f %f %f %f %f %f curveto ",
+				   x1+((2*(cx1-x1))/3), -y1-((2*(cy1-y1))/3),
+				   x2+((2*(cx1-x2))/3), -y2-((2*(cy1-y2))/3), x2, -y2);
+			}
+		    }
+		}
 	      break;
 	    }
 	  pPa = pPa->PaNext;
@@ -1484,7 +1699,7 @@ void DrawEllips (int frame, int thick, int style, int x, int y, int width,
        /* Draw a circle */
        fprintf (fout, "%d %d %d %d %d cer\n", style, thick, xm, -ym, width);
      }
-   else
+   else if (height != 0)
      {
        /* Draw an ellips */
        fprintf (fout, "%d %d %d %d %d %d ellipse\n", style, thick,
@@ -1559,7 +1774,7 @@ void DrawRectangleFrame (int frame, int thick, int style, int x, int y,
    if (thick > 0)
       CurrentColor (fout, fg);
 
-   arc = 3 * 72 / 25.4;
+   arc = (int)(3 * 72 / 25.4);
    xf = x + width - 1;
    yf = y + height - 1;
    x = x;
@@ -1618,12 +1833,12 @@ void DrawEllipsFrame (int frame, int thick, int style, int x, int y,
 	fprintf (fout, "%d %d %d %d %d %d ellipse\n",
 		 style, thick, xm, -ym, width, height);
      }
-   px7mm = 7 * 72 / 25.4 + 0.5;
+   px7mm = (int)(7 * 72 / 25.4 + 0.5);
    if (height > px7mm)
      {
 	y = (ym - height + px7mm);
 	A = ((double) height - px7mm) / height;
-	shiftX = width * sqrt (1 - A * A) + 0.5;
+	shiftX = (int)(width * sqrt (1 - A * A) + 0.5);
 	fprintf (fout, "%d %d  %d %d %d %d %d Seg\n",
 		 xm - shiftX, -y, xm + shiftX, -y, style, thick, 2);
      }
@@ -1828,8 +2043,6 @@ void DrawSlash (int frame, int thick, int style, int x, int y, int l,
    /* Do we need to change the current color ? */
    CurrentColor (fout, fg);
 
-   l--;
-   h--;
    xf = x + l;
    yf = y + h;
    x = x;
@@ -1844,37 +2057,6 @@ void DrawSlash (int frame, int thick, int style, int x, int y, int l,
 }
 
 
-/*----------------------------------------------------------------------
-   StorePageInfo store the page number, width and height of the page,
-   used later by DrawPage.
-  ----------------------------------------------------------------------*/
-void StorePageInfo (int pagenum, int width, int height)
-{
-   LastPageNumber = pagenum;
-   LastPageWidth = width;
-   LastPageHeight = height;
-}
-
-
-/*----------------------------------------------------------------------
-   psBoundingBox output the %%BoundingBox macro for Postscript.
-  ----------------------------------------------------------------------*/
-void psBoundingBox (int frame, int width, int height)
-{
-   FILE               *fout;
-
-   fout = (FILE *) FrRef[frame];
-   /* Since the origin of Postscript coordinates is the lower-left    */
-   /* corner of the page, that an A4 page is 2970 mm (i.e 2970*72/254 */
-   /* = 841 pts) and that Thot adds a 50 pts margin on top and height */
-   /* of the output image, here is the correct values :               */
-
-   fprintf (fout, "%%%%BoundingBox: %d %d %d %d\n",
-	    50, 791 - height,
-	    50 + width, 791);
-}
-
-
 /*----------------------------------------------------------------------
    PaintWithPattern fill the rectangle associated to a window w (or frame if w= 0)
    located on (x , y) and geometry width x height, using the
@@ -1906,3 +2088,4 @@ void PaintWithPattern (int frame, int x, int y, int width, int height,
      }
 }
 #endif /* _WIN_PRINT */
+#endif /* _GL */
