@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2004
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -8,7 +8,7 @@
 /*
  * Authors: V. Quint, I. Vatton (INRIA)
  */
-
+#include "thot_gui.h"
 #include "thot_sys.h"
 #include "constmedia.h"
 #include "typemedia.h"
@@ -65,26 +65,30 @@
 #include "thotmsg_f.h"
 #include "tree_f.h"
 #include "units_f.h"
+#include "unstructlocate_f.h"
 #include "views_f.h"
 #include "viewapi_f.h"
 
 static char           nameBuffer[MAX_NAME_LENGTH];
 
 /*----------------------------------------------------------------------
-   TtaOpenMainView
+  TtaOpenMainView
 
-   Opens the main view of a document. This document must have a PSchema
-   (see TtaSetPSchema).
-   Parameters:
-   document: the document for which a window must be open.
-   x, y: coordinate (in millimeters) of the upper left corner of the
-   window that will display the view.
-   w, h: width and height (in millimeters) of the upper left corner of the
-   window that will display the view.
-   Return value:
-   the view opened or 0 if the view cannot be opened.
+  Opens the main view of a document. This document must have a PSchema
+  (see TtaSetPSchema).
+  Parameters:
+  document: the document for which a window must be open.
+  x, y: coordinate (in millimeters) of the upper left corner of the
+  window that will display the view.
+  w, h: width and height (in millimeters) of the upper left corner of the
+  window that will display the view.
+  withMenu is TRUE when menus are displayed
+  withButton is TRUE when buttons are displayed
+  Return value:
+  the view opened or 0 if the view cannot be opened.
   ----------------------------------------------------------------------*/
-View TtaOpenMainView (Document document, int x, int y, int w, int h)
+View TtaOpenMainView (Document document, int x, int y, int w, int h,
+		      ThotBool withMenu, ThotBool withButton)
 {
   PtrDocument         pDoc;
   PtrPSchema          pPS;
@@ -114,7 +118,7 @@ View TtaOpenMainView (Document document, int x, int y, int w, int h)
 		AddLastPageBreak (pDoc->DocDocElement, 1, pDoc, FALSE);
 	      nView = CreateAbstractImage (pDoc, 1, pDoc->DocSSchema, 1,
 					   TRUE, NULL);
-	      OpenCreatedView (pDoc, nView, x, y, w, h);
+	      OpenCreatedView (pDoc, nView, x, y, w, h, withMenu, withButton);
 	      view = nView;
 	    }
      }
@@ -198,7 +202,7 @@ static View OpenView (Document document, char *viewName, int x, int y,
 	      {
 		if (viewHasBeenOpen)
 		  {
-		    OpenCreatedView (pDoc, nView, x, y, w, h);
+		    OpenCreatedView (pDoc, nView, x, y, w, h, TRUE, TRUE);
 		    view = nView;
 		  }
 	      }
@@ -273,27 +277,39 @@ void TtaCloseView (Document document, View view)
 }
 
 /*----------------------------------------------------------------------
-   TtaChangeViewTitle
+   TtaChangeWindowTitle
 
    Changes the title of a view.
+   if view == 0, changes the title of all windows of document
+   otherwise change the window title of the specified view.
    Parameters:
    document: the document.
    view: the view.
    title: the new title.
+   encoding: the title encoding;
   ----------------------------------------------------------------------*/
-void TtaChangeViewTitle (Document document, View view, char *title)
+void TtaChangeWindowTitle (Document document, View view, char *title,
+			   CHARSET encoding)
 {
-   UserErrorCode = 0;
-   /* Checks the parameter document */
-   if (document < 1 || document > MAX_DOCUMENTS)
-      TtaError (ERR_invalid_document_parameter);
-   else if (LoadedDocument[document - 1] == NULL)
-      TtaError (ERR_invalid_document_parameter);
-   else if (view < 1 || view > MAX_VIEW_DOC)
-      TtaError (ERR_invalid_parameter);
-   else
-      ChangeFrameTitle (LoadedDocument[document - 1]->DocViewFrame[view - 1],
-			title);
+  int          idwindow, v;
+  PtrDocument  pDoc;
+  
+  if (document < 1 || document > MAX_DOCUMENTS)
+    return;
+  if (view > 0)
+    {
+      idwindow = GetWindowNumber (document, view);
+      if (idwindow > 0) 
+	ChangeFrameTitle (idwindow, (unsigned char *)title, encoding);
+    }
+  else
+    {
+      pDoc = LoadedDocument[document - 1];
+      /* traite les vues du document */
+      for (v = 0; v < MAX_VIEW_DOC; v++)
+	if (pDoc->DocView[v].DvPSchemaView > 0)
+	  ChangeFrameTitle (pDoc->DocViewFrame[v], (unsigned char *)title, encoding);
+    }
 }
 
 
@@ -394,27 +410,30 @@ Element TtaGetFirstElementShown (Document document, View view, int *position)
   if (frame != 0)
     {
       pFrame = &ViewFrameTable[frame - 1];
+#ifndef _GL
       x = pFrame->FrXOrg;
       y = pFrame->FrYOrg;
+#else /* _GL */
+      x = y = 0;
+#endif /* _GL */
       pRootAb = pFrame->FrAbstractBox;
       if (pRootAb != NULL && pRootAb->AbBox != NULL)
 	{
 	  charsNumber = 0;
 	  /* recupere la boite selectionnee */
-	  if (ThotLocalActions[T_selecbox] != NULL)
+	  GetClickedBox (&pBox, pRootAb, frame, x, y, 1, &charsNumber);
+	  if (pBox && pBox->BxAbstractBox)
 	    {
-	      (*ThotLocalActions[T_selecbox]) (&pBox, pRootAb, frame, x, y,
-					       &charsNumber);
-	      if (pBox != NULL && pBox->BxAbstractBox != NULL)
-		{
-		  if (pBox->BxType == BoPiece || pBox->BxType == BoSplit)
-		    pBox = pBox->BxAbstractBox->AbBox->BxNexChild;
-		  pEl = pBox->BxAbstractBox->AbElement;
-		  /* width and height of the frame */
-		  GetSizesFrame (frame, &width, &height);
-		  /* position of the box top in the frame in % */
-		  *position = (pBox->BxYOrg - pFrame->FrYOrg) * 100 / height;
-		}
+	      if (pBox->BxType == BoPiece ||
+		  pBox->BxType == BoScript ||
+		  pBox->BxType == BoMulScript ||
+		  pBox->BxType == BoSplit)
+		pBox = pBox->BxAbstractBox->AbBox->BxNexChild;
+	      pEl = pBox->BxAbstractBox->AbElement;
+	      /* width and height of the frame */
+	      GetSizesFrame (frame, &width, &height);
+	      /* position of the box top in the frame in % */
+	      *position = (pBox->BxYOrg - pFrame->FrYOrg) * 100 / height;
 	    }
 	}
 
@@ -465,7 +484,7 @@ void TtaShowElement (Document document, View view, Element element, int position
 	    DestroyAbsBoxesView (pEl, LoadedDocument[document - 1], FALSE,
 				 aView);
 	  /* and CheckAbsBox will rebuild it at the beginning of the element */
-	  CheckAbsBox (pEl, aView, LoadedDocument[document - 1], FALSE, FALSE);
+	  CheckAbsBox (pEl, aView, LoadedDocument[document - 1], FALSE, TRUE);
 	  if (pEl->ElAbstractBox[aView - 1] != NULL)
 	    ShowBox (frame, pEl->ElAbstractBox[aView - 1]->AbBox, 0, position);
 	}
@@ -559,7 +578,7 @@ int TtaIsPSchemaValid (char *structureName, char *presentationName)
    else
      {
 	/* Gets the corresponding structure schema name */
-	TtaReadName (file, gotStructName);
+	TtaReadName (file, (unsigned char *)gotStructName);
 	if (strcmp (structureName, gotStructName) == 0)
 	   result = 1;
 	TtaReadClose (file);
@@ -712,6 +731,132 @@ View TtaGetViewFromName (Document document, char *viewName)
    return view;
 }
 
+/*----------------------------------------------------------------------
+   TtaGetWindowId
+
+   Returns the window id to which that document belongs.
+   Parameter:
+     + document
+   Return values:
+     + int window_id : the document window id
+  ----------------------------------------------------------------------*/
+int TtaGetWindowId( Document document )
+{
+  int window_id = -1;
+#ifdef _WX  
+  PtrDocument pDoc;
+  
+  if (document < 1 || document > MAX_DOCUMENTS)
+      TtaError (ERR_invalid_document_parameter);
+   else if (LoadedDocument[document - 1] == NULL)
+      TtaError (ERR_invalid_document_parameter);
+   else
+     {
+       /* parameter document is ok */
+       pDoc = LoadedDocument[document - 1];
+       
+       /* return the corresponding window id */
+       window_id = pDoc->DocWindowId;
+     }
+#endif /* _WX */
+  return window_id;
+}
+
+/*----------------------------------------------------------------------
+   TtaSetWindowId
+
+   Set the document window id (the window is the document container).
+   Parameter:
+     + document
+     + window_id
+   Return values:
+     + nothing
+  ----------------------------------------------------------------------*/
+void TtaSetWindowId( Document document, int window_id )
+{
+#ifdef _WX  
+  PtrDocument pDoc;
+  
+  if (document < 1 || document > MAX_DOCUMENTS)
+      TtaError (ERR_invalid_document_parameter);
+   else if (LoadedDocument[document - 1] == NULL)
+      TtaError (ERR_invalid_document_parameter);
+   else
+     {
+       /* parameter document is ok */
+       pDoc = LoadedDocument[document - 1];
+       
+       /* set the corresponding window id */
+       pDoc->DocWindowId = window_id;
+     }
+#else
+   return;
+#endif /* _WX */
+}
+
+/*----------------------------------------------------------------------
+   TtaGetPageId
+
+   Returns the page id to which that document belongs.
+   Parameter:
+     + document
+   Return values:
+     + int page_id : the document page id
+  ----------------------------------------------------------------------*/
+int TtaGetPageId( Document document )
+{
+  int page_id = -1;
+#ifdef _WX  
+  PtrDocument pDoc;
+  
+  if (document < 1 || document > MAX_DOCUMENTS)
+      TtaError (ERR_invalid_document_parameter);
+   else if (LoadedDocument[document - 1] == NULL)
+      TtaError (ERR_invalid_document_parameter);
+   else
+     {
+       /* parameter document is ok */
+       pDoc = LoadedDocument[document - 1];
+       
+       /* return the corresponding page id */
+       page_id = pDoc->DocPageId;
+     }
+#endif /* _WX */
+  return page_id;
+}
+
+/*----------------------------------------------------------------------
+   TtaSetPageId
+
+   Set the document page id
+   Parameter:
+     + document
+     + page_id
+   Return values:
+     + nothing
+  ----------------------------------------------------------------------*/
+void TtaSetPageId( Document document, int page_id )
+{
+#ifdef _WX  
+  PtrDocument pDoc;
+  
+  if (document < 1 || document > MAX_DOCUMENTS)
+      TtaError (ERR_invalid_document_parameter);
+   else if (LoadedDocument[document - 1] == NULL)
+      TtaError (ERR_invalid_document_parameter);
+   else
+     {
+       /* parameter document is ok */
+       pDoc = LoadedDocument[document - 1];
+       
+       /* set the corresponding page id */
+       pDoc->DocPageId = page_id;
+     }
+#else
+   return;
+#endif /* _WX */
+}
+
 /*----------------------------------------------------------------------
    TtaGiveActiveView
 
@@ -854,7 +999,7 @@ void RedisplayLeaf (PtrElement element, Document document, int delta)
 	       break;
 	     case LtText:
 	       pAbbox1->AbText = element->ElText;
-	       pAbbox1->AbLanguage = element->ElLanguage;
+	       pAbbox1->AbLang = element->ElLanguage;
 	       break;
 	     case LtPolyLine:
 	       pAbbox1->AbLeafType = LtPolyLine;
@@ -871,7 +1016,7 @@ void RedisplayLeaf (PtrElement element, Document document, int delta)
 	     case LtGraphics:
 	       pAbbox1->AbLeafType = element->ElLeafType;
 	       pAbbox1->AbShape = element->ElGraph;
-	       pAbbox1->AbGraphAlphabet = 'G';
+	       pAbbox1->AbGraphScript = 'G';
 	       if (element->ElLeafType == LtGraphics)
 		 {
 		   if (element->ElGraph == 'a' &&
@@ -908,6 +1053,7 @@ void RedisplayLeaf (PtrElement element, Document document, int delta)
 	     }
 	}
 
+
    RedisplayCommand (document);
    /* si l'element modifie' appartient soit a un element copie' */
    /* dans des paves par une regle Copy, soit a un element inclus */
@@ -1052,10 +1198,14 @@ void UndisplayInheritedAttributes (PtrElement pEl, PtrAttribute pAttr,
    else
       /* l'element porte-t-il deja un attribut du meme type ? */
       pOldAttr = AttributeValue (pEl, pAttr);
+   pPS = PresentationSchema (pAttr->AeAttrSSchema, pDoc); 
+   /* si l'attribut n'a pas de schema de presentation pour ce document,
+      on ne fait rien */
+   if (pPS == NULL)
+     return;
    /* doit-on se preoccuper des heritages et comparaisons d'attributs? */
-   pPS = PresentationSchema (pAttr->AeAttrSSchema, pDoc);
-   inheritance = (pPS->PsNHeirElems[pAttr->AeAttrNum - 1] > 0);
-   comparaison = (pPS->PsNComparAttrs[pAttr->AeAttrNum - 1] > 0);
+   inheritance = (pPS->PsNHeirElems->Num[pAttr->AeAttrNum - 1] > 0);
+   comparaison = (pPS->PsNComparAttrs->Num[pAttr->AeAttrNum - 1] > 0);
    if (inheritance || comparaison)
       /* cherche le premier attribut de meme type pose' sur un ascendant */
       /* de pEl */
@@ -1113,6 +1263,7 @@ void UndisplayElement (PtrElement pEl, Document document)
    ThotBool            stop;
    PtrDocument         pDoc;
    int                 savePageHeight;
+   ThotBool            immediat;
 
    pDoc = LoadedDocument[document - 1];
    if (pDoc == NULL)
@@ -1128,6 +1279,7 @@ void UndisplayElement (PtrElement pEl, Document document)
    /* on retire simplement l'element de l'arbre abstrait */
    /* sauf si c'est la racine car son pave n'avait pas ete detruit */
    /* il faut donc executer la suite de la procedure */
+   immediat = documentDisplayMode[document - 1] == DisplayImmediately;
    if (documentDisplayMode[document - 1] == NoComputedDisplay)
      {
        if (pEl->ElParent == NULL)
@@ -1144,8 +1296,6 @@ void UndisplayElement (PtrElement pEl, Document document)
 	   return;
 	 }
      }
-   if (ThotLocalActions[T_createhairline] != NULL)
-     (*ThotLocalActions[T_checksel]) (pEl, document);
    /* cherche l'element qui precede l'element a detruire : pPrevious */
    pPrevious = pEl->ElPrevious;
    /* saute les marques de page */
@@ -1193,7 +1343,7 @@ void UndisplayElement (PtrElement pEl, Document document)
 	   PageHeight = 1;
 	while (pE != NULL)
 	  {
-	     UpdateNumbers (pNext, pE, pDoc, (ThotBool)(documentDisplayMode[document - 1] == DisplayImmediately));
+	     UpdateNumbers (pNext, pE, pDoc, immediat);
 	     pE = pE->ElNext;
 	  }
 	PageHeight = savePageHeight;
@@ -1231,9 +1381,6 @@ void UndisplayElement (PtrElement pEl, Document document)
 	if (pNeighbour == NULL)
 	   /* l'element qui precede la partie detruite devient dernier */
 	   ChangeFirstLast (pPrevious, pDoc, FALSE, FALSE);
-	/* traitement particulier aux tableaux */
-	if (ThotLocalActions[T_createhairline] != NULL)
-	   (*ThotLocalActions[T_createhairline]) (pPrevious, pEl, pDoc);
      }
    /* reevalue l'image de toutes les vues */
    AbstractImageUpdated (pDoc);
@@ -1241,17 +1388,17 @@ void UndisplayElement (PtrElement pEl, Document document)
    if (pEl != NULL)
      {
 	/* reaffiche les paves qui copient les elements detruits */
-	RedisplayCopies (pEl, pDoc, (ThotBool)(documentDisplayMode[document - 1] == DisplayImmediately));
+	RedisplayCopies (pEl, pDoc, immediat);
 	/* la renumerotation est faite plus haut */
 	/* reaffiche les references aux elements detruits */
 	/* et enregistre les references sortantes coupees */
 	/* ainsi que les elements coupe's qui sont reference's par */
 	/* d'autres documents */
-	RedisplayEmptyReferences (pEl, &pDoc, (ThotBool)(documentDisplayMode[document - 1] == DisplayImmediately));
+	RedisplayEmptyReferences (pEl, &pDoc, immediat);
 	/* Retransmet les valeurs des compteurs et attributs TRANSMIT */
 	/* s'il y a des elements apres */
         if (pEl->ElStructSchema)
-	   if (pEl->ElStructSchema->SsRule[pEl->ElTypeNumber - 1].SrRefImportedDoc)
+	   if (pEl->ElStructSchema->SsRule->SrElem[pEl->ElTypeNumber - 1]->SrRefImportedDoc)
 	      RepApplyTransmitRules (pEl, pSS, pDoc);
      }
 }
